@@ -47,7 +47,7 @@
 class AccGroupInfo
 {
 public:
-  AccGroupInfo(nsAccessible* aItem, PRUint32 aRole);
+  AccGroupInfo(nsAccessible* aItem, mozilla::a11y::role aRole);
   ~AccGroupInfo() { MOZ_COUNT_DTOR(AccGroupInfo); }
 
   PRInt32 PosInSet() const { return mPosInSet; }
@@ -59,17 +59,17 @@ public:
    */
   static AccGroupInfo* CreateGroupInfo(nsAccessible* aAccessible)
   {
-    PRUint32 role = aAccessible->Role();
-    if (role != nsIAccessibleRole::ROLE_ROW &&
-        role != nsIAccessibleRole::ROLE_GRID_CELL &&
-        role != nsIAccessibleRole::ROLE_OUTLINEITEM &&
-        role != nsIAccessibleRole::ROLE_OPTION &&
-        role != nsIAccessibleRole::ROLE_LISTITEM &&
-        role != nsIAccessibleRole::ROLE_MENUITEM &&
-        role != nsIAccessibleRole::ROLE_CHECK_MENU_ITEM &&
-        role != nsIAccessibleRole::ROLE_RADIO_MENU_ITEM &&
-        role != nsIAccessibleRole::ROLE_RADIOBUTTON &&
-        role != nsIAccessibleRole::ROLE_PAGETAB)
+    mozilla::a11y::role role = aAccessible->Role();
+    if (role != mozilla::a11y::roles::ROW &&
+        role != mozilla::a11y::roles::GRID_CELL &&
+        role != mozilla::a11y::roles::OUTLINEITEM &&
+        role != mozilla::a11y::roles::OPTION &&
+        role != mozilla::a11y::roles::LISTITEM &&
+        role != mozilla::a11y::roles::MENUITEM &&
+        role != mozilla::a11y::roles::CHECK_MENU_ITEM &&
+        role != mozilla::a11y::roles::RADIO_MENU_ITEM &&
+        role != mozilla::a11y::roles::RADIOBUTTON &&
+        role != mozilla::a11y::roles::PAGETAB)
       return nsnull;
 
     AccGroupInfo* info = new AccGroupInfo(aAccessible, BaseRole(role));
@@ -80,11 +80,11 @@ private:
   AccGroupInfo(const AccGroupInfo&);
   AccGroupInfo& operator =(const AccGroupInfo&);
 
-  static PRUint32 BaseRole(PRUint32 aRole)
+  static mozilla::a11y::role BaseRole(mozilla::a11y::role aRole)
   {
-    if (aRole == nsIAccessibleRole::ROLE_CHECK_MENU_ITEM ||
-        aRole == nsIAccessibleRole::ROLE_RADIO_MENU_ITEM)
-      return nsIAccessibleRole::ROLE_MENUITEM;
+    if (aRole == mozilla::a11y::roles::CHECK_MENU_ITEM ||
+        aRole == mozilla::a11y::roles::RADIO_MENU_ITEM)
+      return mozilla::a11y::roles::MENUITEM;
     return aRole;
   }
 
@@ -92,7 +92,8 @@ private:
    * Return true if the given parent role is conceptual parent of the given
    * role.
    */
-  static bool IsConceptualParent(PRUint32 aRole, PRUint32 aParentRole);
+  static bool IsConceptualParent(mozilla::a11y::role aRole,
+				 mozilla::a11y::role aParentRole);
 
   PRUint32 mPosInSet;
   PRUint32 mSetSize;
