@@ -40,6 +40,7 @@
 
 #include "AccIterator.h"
 #include "nsAccUtils.h"
+#include "Role.h"
 #include "States.h"
 
 #include "nsIMutableArray.h"
@@ -316,10 +317,10 @@ nsARIAGridAccessible::GetRowDescription(PRInt32 aRow, nsAString& aDescription)
 }
 
 NS_IMETHODIMP
-nsARIAGridAccessible::IsColumnSelected(PRInt32 aColumn, PRBool *aIsSelected)
+nsARIAGridAccessible::IsColumnSelected(PRInt32 aColumn, bool *aIsSelected)
 {
   NS_ENSURE_ARG_POINTER(aIsSelected);
-  *aIsSelected = PR_FALSE;
+  *aIsSelected = false;
 
   if (IsDefunct())
     return NS_ERROR_FAILURE;
@@ -342,15 +343,15 @@ nsARIAGridAccessible::IsColumnSelected(PRInt32 aColumn, PRBool *aIsSelected)
     }
   } while ((row = rowIter.Next()));
 
-  *aIsSelected = PR_TRUE;
+  *aIsSelected = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsARIAGridAccessible::IsRowSelected(PRInt32 aRow, PRBool *aIsSelected)
+nsARIAGridAccessible::IsRowSelected(PRInt32 aRow, bool *aIsSelected)
 {
   NS_ENSURE_ARG_POINTER(aIsSelected);
-  *aIsSelected = PR_FALSE;
+  *aIsSelected = false;
 
   if (IsDefunct())
     return NS_ERROR_FAILURE;
@@ -367,16 +368,16 @@ nsARIAGridAccessible::IsRowSelected(PRInt32 aRow, PRBool *aIsSelected)
     }
   }
 
-  *aIsSelected = PR_TRUE;
+  *aIsSelected = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsARIAGridAccessible::IsCellSelected(PRInt32 aRow, PRInt32 aColumn,
-                                     PRBool *aIsSelected)
+                                     bool *aIsSelected)
 {
   NS_ENSURE_ARG_POINTER(aIsSelected);
-  *aIsSelected = PR_FALSE;
+  *aIsSelected = false;
 
   if (IsDefunct())
     return NS_ERROR_FAILURE;
@@ -392,7 +393,7 @@ nsARIAGridAccessible::IsCellSelected(PRInt32 aRow, PRInt32 aColumn,
       return NS_OK;
   }
 
-  *aIsSelected = PR_TRUE;
+  *aIsSelected = true;
   return NS_OK;
 }
 
@@ -458,10 +459,10 @@ nsARIAGridAccessible::GetSelectedRowCount(PRUint32* aCount)
     if (!cell)
       continue;
 
-    PRBool isRowSelected = PR_TRUE;
+    bool isRowSelected = true;
     do {
       if (!nsAccUtils::IsARIASelected(cell)) {
-        isRowSelected = PR_FALSE;
+        isRowSelected = false;
         break;
       }
     } while ((cell = cellIter.Next()));
@@ -496,14 +497,14 @@ nsARIAGridAccessible::GetSelectedCells(nsIArray **aCells)
 
     if (nsAccUtils::IsARIASelected(row)) {
       while ((cell = cellIter.Next()))
-        selCells->AppendElement(static_cast<nsIAccessible *>(cell), PR_FALSE);
+        selCells->AppendElement(static_cast<nsIAccessible *>(cell), false);
 
       continue;
     }
 
     while ((cell = cellIter.Next())) {
       if (nsAccUtils::IsARIASelected(cell))
-        selCells->AppendElement(static_cast<nsIAccessible *>(cell), PR_FALSE);
+        selCells->AppendElement(static_cast<nsIAccessible *>(cell), false);
     }
   }
 
@@ -605,10 +606,10 @@ nsARIAGridAccessible::GetSelectedRowIndices(PRUint32 *arowCount,
     if (!cell)
       continue;
 
-    PRBool isRowSelected = PR_TRUE;
+    bool isRowSelected = true;
     do {
       if (!nsAccUtils::IsARIASelected(cell)) {
-        isRowSelected = PR_FALSE;
+        isRowSelected = false;
         break;
       }
     } while ((cell = cellIter.Next()));
@@ -661,13 +662,13 @@ nsARIAGridAccessible::SelectColumn(PRInt32 aColumn)
   nsAccessible *row = nsnull;
   while ((row = rowIter.Next())) {
     // Unselect all cells in the row.
-    nsresult rv = SetARIASelected(row, PR_FALSE);
+    nsresult rv = SetARIASelected(row, false);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // Select cell at the column index.
     nsAccessible *cell = GetCellInRowAt(row, aColumn);
     if (cell) {
-      rv = SetARIASelected(cell, PR_TRUE);
+      rv = SetARIASelected(cell, true);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -684,7 +685,7 @@ nsARIAGridAccessible::UnselectRow(PRInt32 aRow)
   nsAccessible *row = GetRowAt(aRow);
   NS_ENSURE_ARG(row);
 
-  return SetARIASelected(row, PR_FALSE);
+  return SetARIASelected(row, false);
 }
 
 NS_IMETHODIMP
@@ -701,7 +702,7 @@ nsARIAGridAccessible::UnselectColumn(PRInt32 aColumn)
   while ((row = rowIter.Next())) {
     nsAccessible *cell = GetCellInRowAt(row, aColumn);
     if (cell) {
-      nsresult rv = SetARIASelected(cell, PR_FALSE);
+      nsresult rv = SetARIASelected(cell, false);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -710,10 +711,10 @@ nsARIAGridAccessible::UnselectColumn(PRInt32 aColumn)
 }
 
 NS_IMETHODIMP
-nsARIAGridAccessible::IsProbablyForLayout(PRBool *aIsProbablyForLayout)
+nsARIAGridAccessible::IsProbablyForLayout(bool *aIsProbablyForLayout)
 {
   NS_ENSURE_ARG_POINTER(aIsProbablyForLayout);
-  *aIsProbablyForLayout = PR_FALSE;
+  *aIsProbablyForLayout = false;
 
   return NS_OK;
 }
@@ -721,38 +722,38 @@ nsARIAGridAccessible::IsProbablyForLayout(PRBool *aIsProbablyForLayout)
 ////////////////////////////////////////////////////////////////////////////////
 // Protected
 
-PRBool
+bool
 nsARIAGridAccessible::IsValidRow(PRInt32 aRow)
 {
   if (aRow < 0)
-    return PR_FALSE;
+    return false;
   
   PRInt32 rowCount = 0;
   GetRowCount(&rowCount);
   return aRow < rowCount;
 }
 
-PRBool
+bool
 nsARIAGridAccessible::IsValidColumn(PRInt32 aColumn)
 {
   if (aColumn < 0)
-    return PR_FALSE;
+    return false;
 
   PRInt32 colCount = 0;
   GetColumnCount(&colCount);
   return aColumn < colCount;
 }
 
-PRBool
+bool
 nsARIAGridAccessible::IsValidRowNColumn(PRInt32 aRow, PRInt32 aColumn)
 {
   if (aRow < 0 || aColumn < 0)
-    return PR_FALSE;
+    return false;
   
   PRInt32 rowCount = 0;
   GetRowCount(&rowCount);
   if (aRow >= rowCount)
-    return PR_FALSE;
+    return false;
 
   PRInt32 colCount = 0;
   GetColumnCount(&colCount);
@@ -788,7 +789,7 @@ nsARIAGridAccessible::GetCellInRowAt(nsAccessible *aRow, PRInt32 aColumn)
 
 nsresult
 nsARIAGridAccessible::SetARIASelected(nsAccessible *aAccessible,
-                                      PRBool aIsSelected, PRBool aNotify)
+                                      bool aIsSelected, bool aNotify)
 {
   nsIContent *content = aAccessible->GetContent();
   NS_ENSURE_STATE(content);
@@ -813,16 +814,16 @@ nsARIAGridAccessible::SetARIASelected(nsAccessible *aAccessible,
   if (aIsSelected)
     return NS_OK;
 
-  PRUint32 role = aAccessible->Role();
+  roles::Role role = aAccessible->Role();
 
   // If the given accessible is row that was unselected then remove
   // aria-selected from cell accessible.
-  if (role == nsIAccessibleRole::ROLE_ROW) {
+  if (role == roles::ROW) {
     AccIterator cellIter(aAccessible, filters::GetCell);
     nsAccessible *cell = nsnull;
 
     while ((cell = cellIter.Next())) {
-      rv = SetARIASelected(cell, PR_FALSE, PR_FALSE);
+      rv = SetARIASelected(cell, false, false);
       NS_ENSURE_SUCCESS(rv, rv);
     }
     return NS_OK;
@@ -831,21 +832,20 @@ nsARIAGridAccessible::SetARIASelected(nsAccessible *aAccessible,
   // If the given accessible is cell that was unselected and its row is selected
   // then remove aria-selected from row and put aria-selected on
   // siblings cells.
-  if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
-      role == nsIAccessibleRole::ROLE_ROWHEADER ||
-      role == nsIAccessibleRole::ROLE_COLUMNHEADER) {
+  if (role == roles::GRID_CELL || role == roles::ROWHEADER ||
+      role == roles::COLUMNHEADER) {
     nsAccessible* row = aAccessible->Parent();
 
-    if (row && row->Role() == nsIAccessibleRole::ROLE_ROW &&
+    if (row && row->Role() == roles::ROW &&
         nsAccUtils::IsARIASelected(row)) {
-      rv = SetARIASelected(row, PR_FALSE, PR_FALSE);
+      rv = SetARIASelected(row, false, false);
       NS_ENSURE_SUCCESS(rv, rv);
 
       AccIterator cellIter(row, filters::GetCell);
       nsAccessible *cell = nsnull;
       while ((cell = cellIter.Next())) {
         if (cell != aAccessible) {
-          rv = SetARIASelected(cell, PR_TRUE, PR_FALSE);
+          rv = SetARIASelected(cell, true, false);
           NS_ENSURE_SUCCESS(rv, rv);
         }
       }
@@ -879,10 +879,10 @@ nsARIAGridAccessible::GetSelectedColumnsArray(PRUint32 *acolumnCount,
 
   PRInt32 selColCount = colCount;
 
-  nsTArray<PRBool> isColSelArray(selColCount);
+  nsTArray<bool> isColSelArray(selColCount);
   isColSelArray.AppendElements(selColCount);
   for (PRInt32 i = 0; i < selColCount; i++)
-    isColSelArray[i] = PR_TRUE;
+    isColSelArray[i] = true;
 
   do {
     if (nsAccUtils::IsARIASelected(row))
@@ -893,9 +893,9 @@ nsARIAGridAccessible::GetSelectedColumnsArray(PRUint32 *acolumnCount,
     AccIterator cellIter(row, filters::GetCell);
     nsAccessible *cell = nsnull;
     for (colIdx = 0; (cell = cellIter.Next()); colIdx++) {
-      if (isColSelArray.SafeElementAt(colIdx, PR_FALSE) &&
+      if (isColSelArray.SafeElementAt(colIdx, false) &&
           !nsAccUtils::IsARIASelected(cell)) {
-        isColSelArray[colIdx] = PR_FALSE;
+        isColSelArray[colIdx] = false;
         selColCount--;
       }
     }
@@ -954,16 +954,15 @@ nsARIAGridCellAccessible::GetTable(nsIAccessibleTable **aTable)
   *aTable = nsnull;
 
   nsAccessible* thisRow = Parent();
-  if (!thisRow || thisRow->Role() != nsIAccessibleRole::ROLE_ROW)
+  if (!thisRow || thisRow->Role() != roles::ROW)
     return NS_OK;
 
   nsAccessible* table = thisRow->Parent();
   if (!table)
     return NS_OK;
 
-  PRUint32 tableRole = table->Role();
-  if (tableRole != nsIAccessibleRole::ROLE_TABLE &&
-      tableRole != nsIAccessibleRole::ROLE_TREE_TABLE)
+  roles::Role tableRole = table->Role();
+  if (tableRole != roles::TABLE && tableRole != roles::TREE_TABLE)
     return NS_OK;
 
   CallQueryInterface(table, aTable);
@@ -988,10 +987,9 @@ nsARIAGridCellAccessible::GetColumnIndex(PRInt32 *aColumnIndex)
   PRInt32 indexInRow = IndexInParent();
   for (PRInt32 idx = 0; idx < indexInRow; idx++) {
     nsAccessible* cell = row->GetChildAt(idx);
-    PRUint32 role = cell->Role();
-    if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
-        role == nsIAccessibleRole::ROLE_ROWHEADER ||
-        role == nsIAccessibleRole::ROLE_COLUMNHEADER)
+    roles::Role role = cell->Role();
+    if (role == roles::GRID_CELL || role == roles::ROWHEADER ||
+        role == roles::COLUMNHEADER)
       (*aColumnIndex)++;
   }
 
@@ -1020,7 +1018,7 @@ nsARIAGridCellAccessible::GetRowIndex(PRInt32 *aRowIndex)
   PRInt32 indexInTable = row->IndexInParent();
   for (PRInt32 idx = 0; idx < indexInTable; idx++) {
     row = table->GetChildAt(idx);
-    if (row->Role() == nsIAccessibleRole::ROLE_ROW)
+    if (row->Role() == roles::ROW)
       (*aRowIndex)++;
   }
 
@@ -1092,22 +1090,22 @@ nsARIAGridCellAccessible::GetRowHeaderCells(nsIArray **aHeaderCells)
 }
 
 NS_IMETHODIMP
-nsARIAGridCellAccessible::IsSelected(PRBool *aIsSelected)
+nsARIAGridCellAccessible::IsSelected(bool *aIsSelected)
 {
   NS_ENSURE_ARG_POINTER(aIsSelected);
-  *aIsSelected = PR_FALSE;
+  *aIsSelected = false;
 
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
   nsAccessible* row = Parent();
-  if (!row || row->Role() != nsIAccessibleRole::ROLE_ROW)
+  if (!row || row->Role() != roles::ROW)
     return NS_OK;
 
   if (!nsAccUtils::IsARIASelected(row) && !nsAccUtils::IsARIASelected(this))
     return NS_OK;
 
-  *aIsSelected = PR_TRUE;
+  *aIsSelected = true;
   return NS_OK;
 }
 
@@ -1125,7 +1123,7 @@ nsARIAGridCellAccessible::ApplyARIAState(PRUint64* aState)
 
   // Check aria-selected="true" on the row.
   nsAccessible* row = Parent();
-  if (!row || row->Role() != nsIAccessibleRole::ROLE_ROW)
+  if (!row || row->Role() != roles::ROW)
     return;
 
   nsIContent *rowContent = row->GetContent();
@@ -1149,7 +1147,7 @@ nsARIAGridCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttrib
   // Expose "table-cell-index" attribute.
 
   nsAccessible* thisRow = Parent();
-  if (!thisRow || thisRow->Role() != nsIAccessibleRole::ROLE_ROW)
+  if (!thisRow || thisRow->Role() != roles::ROW)
     return NS_OK;
 
   PRInt32 colIdx = 0, colCount = 0;
@@ -1159,10 +1157,9 @@ nsARIAGridCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttrib
     if (child == this)
       colIdx = colCount;
 
-    PRUint32 role = child->Role();
-    if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
-        role == nsIAccessibleRole::ROLE_ROWHEADER ||
-        role == nsIAccessibleRole::ROLE_COLUMNHEADER)
+    roles::Role role = child->Role();
+    if (role == roles::GRID_CELL || role == roles::ROWHEADER ||
+        role == roles::COLUMNHEADER)
       colCount++;
   }
 
@@ -1170,9 +1167,8 @@ nsARIAGridCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttrib
   if (!table)
     return NS_OK;
 
-  PRUint32 tableRole = table->Role();
-  if (tableRole != nsIAccessibleRole::ROLE_TABLE &&
-      tableRole != nsIAccessibleRole::ROLE_TREE_TABLE)
+  roles::Role tableRole = table->Role();
+  if (tableRole != roles::TABLE && tableRole != roles::TREE_TABLE)
     return NS_OK;
 
   PRInt32 rowIdx = 0;
@@ -1182,7 +1178,7 @@ nsARIAGridCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttrib
     if (child == thisRow)
       break;
 
-    if (child->Role() == nsIAccessibleRole::ROLE_ROW)
+    if (child->Role() == roles::ROW)
       rowIdx++;
   }
 
