@@ -39,8 +39,9 @@
 
 #include "nsHTMLLinkAccessible.h"
 
-#include "States.h"
 #include "nsCoreUtils.h"
+#include "Role.h"
+#include "States.h"
 
 #include "nsEventStates.h"
 #include "mozilla/dom/Element.h"
@@ -64,10 +65,10 @@ NS_IMPL_ISUPPORTS_INHERITED1(nsHTMLLinkAccessible, nsHyperTextAccessibleWrap,
 ////////////////////////////////////////////////////////////////////////////////
 // nsIAccessible
 
-PRUint32
+role
 nsHTMLLinkAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_LINK;
+  return roles::LINK;
 }
 
 PRUint64
@@ -178,11 +179,11 @@ nsHTMLLinkAccessible::AnchorURIAt(PRUint32 aAnchorIndex)
 ////////////////////////////////////////////////////////////////////////////////
 // Protected members
 
-PRBool
+bool
 nsHTMLLinkAccessible::IsLinked()
 {
   if (IsDefunct())
-    return PR_FALSE;
+    return false;
 
   nsEventStates state = mContent->AsElement()->State();
   return state.HasAtLeastOneOfStates(NS_EVENT_STATE_VISITED |
