@@ -41,49 +41,49 @@
 
 #import "nsRoleMap.h"
 
-#import "mozAccessibleWrapper.h"
+#include "Role.h"
+
 #import "mozAccessible.h"
 #import "mozActionElements.h"
+#import "mozHTMLAccessible.h"
 #import "mozTextAccessible.h"
 
+using namespace mozilla::a11y;
+
 nsAccessibleWrap::
   nsAccessibleWrap(nsIContent *aContent, nsIWeakReference *aShell) :
-  nsAccessible(aContent, aShell), mNativeWrapper(nsnull)
+  nsAccessible(aContent, aShell), mNativeObject(nil),
+  mNativeInited(false)
 {
 }
 
 nsAccessibleWrap::~nsAccessibleWrap()
 {
-  if (mNativeWrapper) {
-    delete mNativeWrapper;
-    mNativeWrapper = nsnull;
-  }
 }
 
-PRBool
-nsAccessibleWrap::Init () 
+mozAccessible* 
+nsAccessibleWrap::GetNativeObject()
 {
-  if (!nsAccessible::Init())
-    return PR_FALSE;
-
-  if (!mNativeWrapper && !AncestorIsFlat()) {
-    // Create our native object using the class type specified in GetNativeType().
-    mNativeWrapper = new AccessibleWrapper (this, GetNativeType());
-    if (!mNativeWrapper)
-      return PR_FALSE;
-  }
-
-  return PR_TRUE;
+  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
+  
+  if (!mNativeInited && !mNativeObject && !IsDefunct() && !AncestorIsFlat())
+    mNativeObject = [[GetNativeType() alloc] initWithAccessible:this];
+  
+  mNativeInited = true;
+  
+  return mNativeObject;
+  
+  NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
 
 NS_IMETHODIMP
 nsAccessibleWrap::GetNativeInterface (void **aOutInterface) 
 {
-  if (mNativeWrapper) {
-    *aOutInterface = (void**)mNativeWrapper->getNativeObject();
-    return NS_OK;
-  }
-  return NS_ERROR_FAILURE;
+  NS_ENSURE_ARG_POINTER(aOutInterface);
+
+  *aOutInterface = static_cast<void*>(GetNativeObject());
+    
+  return *aOutInterface ? NS_OK : NS_ERROR_FAILURE;
 }
 
 // overridden in subclasses to create the right kind of object. by default we create a generic
@@ -93,37 +93,45 @@ nsAccessibleWrap::GetNativeType ()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
-  PRUint32 role = Role();
+  roles::Role role = Role();
   switch (role) {
-    case nsIAccessibleRole::ROLE_PUSHBUTTON:
-    case nsIAccessibleRole::ROLE_SPLITBUTTON:
-    case nsIAccessibleRole::ROLE_TOGGLE_BUTTON:
+    case roles::PUSHBUTTON:
+    case roles::SPLITBUTTON:
+    case roles::TOGGLE_BUTTON:
     {
       // if this button may show a popup, let's make it of the popupbutton type.
-      if (HasPopup())
-        return [mozPopupButtonAccessible class];
-        
-      // regular button
-      return [mozButtonAccessible class];
+      return HasPopup() ? [mozPopupButtonAccessible class] : 
+             [mozButtonAccessible class];
     }
     
-    case nsIAccessibleRole::ROLE_CHECKBUTTON:
+    case roles::PAGETAB:
+      return [mozButtonAccessible class];
+
+    case roles::CHECKBUTTON:
       return [mozCheckboxAccessible class];
       
-    case nsIAccessibleRole::ROLE_AUTOCOMPLETE:
+    case roles::AUTOCOMPLETE:
       return [mozComboboxAccessible class];
+
+    case roles::HEADING:
+      return [mozHeadingAccessible class];
+
+    case roles::PAGETABLIST:
+      return [mozTabsAccessible class];
       
-    case nsIAccessibleRole::ROLE_ENTRY:
-    case nsIAccessibleRole::ROLE_STATICTEXT:
-    case nsIAccessibleRole::ROLE_HEADING:
-    case nsIAccessibleRole::ROLE_LABEL:
-    case nsIAccessibleRole::ROLE_CAPTION:
-    case nsIAccessibleRole::ROLE_ACCEL_LABEL:
-    case nsIAccessibleRole::ROLE_TEXT_LEAF:
+    case roles::ENTRY:
+    case roles::STATICTEXT:
+    case roles::LABEL:
+    case roles::CAPTION:
+    case roles::ACCEL_LABEL:
+    case roles::TEXT_LEAF:
       // normal textfield (static or editable)
       return [mozTextAccessible class]; 
-      
-    case nsIAccessibleRole::ROLE_COMBOBOX:
+
+    case roles::LINK:
+      return [mozLinkAccessible class];
+
+    case roles::COMBOBOX:
       return [mozPopupButtonAccessible class];
       
     default:
@@ -141,11 +149,16 @@ nsAccessibleWrap::GetNativeType ()
 void
 nsAccessibleWrap::Shutdown ()
 {
-  if (mNativeWrapper) {
-    delete mNativeWrapper;
-    mNativeWrapper = nsnull;
+  // this ensure we will not try to re-create the native object.
+  mNativeInited = true;
+
+  // we really intend to access the member directly.
+  if (mNativeObject) {
+    [mNativeObject expire];
+    [mNativeObject release];
+    mNativeObject = nil;
   }
-  
+
   nsAccessible::Shutdown();
 }
 
@@ -201,57 +214,46 @@ nsAccessibleWrap::InvalidateChildren()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
-  if (mNativeWrapper) {
-    mozAccessible *object = mNativeWrapper->getNativeObject();
-    [object invalidateChildren];
-  }
+  [GetNativeObject() invalidateChildren];
+
   nsAccessible::InvalidateChildren();
 
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-PRInt32
-nsAccessibleWrap::GetUnignoredChildCount(PRBool aDeepCount)
+bool
+nsAccessibleWrap::AppendChild(nsAccessible *aAccessible)
 {
-  // if we're flat, we have no children.
-  if (nsAccUtils::MustPrune(this))
-    return 0;
-
-  PRInt32 resultChildCount = 0;
-
-  PRInt32 childCount = GetChildCount();
-  for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
-    nsAccessibleWrap *childAcc =
-      static_cast<nsAccessibleWrap*>(GetChildAt(childIdx));
-
-    // if the current child is not ignored, count it.
-    if (!childAcc->IsIgnored())
-      ++resultChildCount;
-
-    // if it's flat, we don't care to inspect its children.
-    if (nsAccUtils::MustPrune(childAcc))
-      continue;
-
-    if (aDeepCount) {
-      // recursively count the unignored children of our children since it's a deep count.
-      resultChildCount += childAcc->GetUnignoredChildCount(PR_TRUE);
-    } else {
-      // no deep counting, but if the child is ignored, we want to substitute it for its
-      // children.
-      if (childAcc->IsIgnored()) 
-        resultChildCount += childAcc->GetUnignoredChildCount(PR_FALSE);
-    }
-  } 
+  bool appended = nsAccessible::AppendChild(aAccessible);
   
-  return resultChildCount;
+  if (appended && mNativeObject)
+    [mNativeObject appendChild:aAccessible];
+
+  return appended;
+}
+
+bool
+nsAccessibleWrap::RemoveChild(nsAccessible *aAccessible)
+{
+  bool removed = nsAccessible::RemoveChild(aAccessible);
+
+  if (removed && mNativeObject)
+    [mNativeObject invalidateChildren];
+
+  return removed;
 }
 
 // if we for some reason have no native accessible, we should be skipped over (and traversed)
 // when fetching all unignored children, etc.  when counting unignored children, we will not be counted.
-PRBool 
+bool 
 nsAccessibleWrap::IsIgnored() 
 {
-  return (mNativeWrapper == nsnull) || mNativeWrapper->isIgnored();
+  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
+  
+  mozAccessible* nativeObject = GetNativeObject();
+  return (!nativeObject) || [nativeObject accessibilityIsIgnored];
+  
+  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(false);
 }
 
 void
@@ -285,7 +287,7 @@ nsAccessibleWrap::GetUnignoredChildren(nsTArray<nsRefPtr<nsAccessibleWrap> > &aC
 already_AddRefed<nsIAccessible>
 nsAccessibleWrap::GetUnignoredParent()
 {
-  nsAccessibleWrap *parentWrap = static_cast<nsAccessibleWrap*>(GetParent());
+  nsAccessibleWrap* parentWrap = static_cast<nsAccessibleWrap*>(Parent());
   if (!parentWrap)
     return nsnull;
     
@@ -302,7 +304,7 @@ nsAccessibleWrap::GetUnignoredParent()
 ////////////////////////////////////////////////////////////////////////////////
 // nsAccessibleWrap protected
 
-PRBool
+bool
 nsAccessibleWrap::AncestorIsFlat()
 {
   // We don't create a native object if we're child of a "flat" accessible;
@@ -313,13 +315,13 @@ nsAccessibleWrap::AncestorIsFlat()
   // look the same on all platforms, we still let the C++ objects be created
   // though.
 
-  nsAccessible* parent(GetParent());
+  nsAccessible* parent = Parent();
   while (parent) {
     if (nsAccUtils::MustPrune(parent))
-      return PR_TRUE;
+      return true;
 
-    parent = parent->GetParent();
+    parent = parent->Parent();
   }
   // no parent was flat
-  return PR_FALSE;
+  return false;
 }
