@@ -44,6 +44,7 @@
 #include "nsCoreUtils.h"
 #include "nsDocAccessible.h"
 #include "Relation.h"
+#include "Role.h"
 #include "States.h"
 
 // NOTE: alphabetically ordered
@@ -114,10 +115,10 @@ nsXULButtonAccessible::DoAction(PRUint8 aIndex)
 ////////////////////////////////////////////////////////////////////////////////
 // nsXULButtonAccessible: nsAccessible
 
-PRUint32
+role
 nsXULButtonAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_PUSHBUTTON;
+  return roles::PUSHBUTTON;
 }
 
 PRUint64
@@ -235,13 +236,13 @@ nsXULButtonAccessible::CacheChildren()
 
   nsAccessible* child = nsnull;
   while ((child = walker.NextChild())) {
-    PRUint32 role = child->Role();
+    roles::Role role = child->Role();
 
-    if (role == nsIAccessibleRole::ROLE_MENUPOPUP) {
+    if (role == roles::MENUPOPUP) {
       // Get an accessible for menupopup or panel elements.
       menupopup = child;
 
-    } else if (isMenuButton && role == nsIAccessibleRole::ROLE_PUSHBUTTON) {
+    } else if (isMenuButton && role == roles::PUSHBUTTON) {
       // Button type="menu-button" contains a real button. Get an accessible
       // for it. Ignore dropmarker button which is placed as a last child.
       button = child;
@@ -344,10 +345,10 @@ NS_IMETHODIMP nsXULDropmarkerAccessible::DoAction(PRUint8 index)
   return NS_ERROR_INVALID_ARG;
 }
 
-PRUint32
+role
 nsXULDropmarkerAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_PUSHBUTTON;
+  return roles::PUSHBUTTON;
 }
 
 PRUint64
@@ -366,10 +367,10 @@ nsXULCheckboxAccessible::
 {
 }
 
-PRUint32
+role
 nsXULCheckboxAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_CHECKBUTTON;
+  return roles::CHECKBUTTON;
 }
 
 PRUint8
@@ -446,10 +447,10 @@ nsXULGroupboxAccessible::
 {
 }
 
-PRUint32
+role
 nsXULGroupboxAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_GROUPING;
+  return roles::GROUPING;
 }
 
 nsresult
@@ -477,7 +478,7 @@ nsXULGroupboxAccessible::RelationByType(PRUint32 aType)
   PRInt32 childCount = GetChildCount();
   for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
     nsAccessible *childAcc = GetChildAt(childIdx);
-    if (childAcc->Role() == nsIAccessibleRole::ROLE_LABEL) {
+    if (childAcc->Role() == roles::LABEL) {
       // Ensure that it's our label
       Relation reverseRel =
         childAcc->RelationByType(nsIAccessibleRelation::RELATION_LABEL_FOR);
@@ -558,14 +559,14 @@ nsXULRadioButtonAccessible::ContainerWidget() const
 
 nsXULRadioGroupAccessible::
   nsXULRadioGroupAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
-  nsXULSelectableAccessible(aContent, aShell)
+  XULSelectControlAccessible(aContent, aShell)
 { 
 }
 
-PRUint32
+role
 nsXULRadioGroupAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_GROUPING;
+  return roles::GROUPING;
 }
 
 PRUint64
@@ -609,10 +610,10 @@ nsXULStatusBarAccessible::
 {
 }
 
-PRUint32
+role
 nsXULStatusBarAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_STATUSBAR;
+  return roles::STATUSBAR;
 }
 
 
@@ -684,10 +685,10 @@ nsXULToolbarAccessible::
 {
 }
 
-PRUint32
+role
 nsXULToolbarAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_TOOLBAR;
+  return roles::TOOLBAR;
 }
 
 nsresult
@@ -714,10 +715,10 @@ nsXULToolbarSeparatorAccessible::
 {
 }
 
-PRUint32
+role
 nsXULToolbarSeparatorAccessible::NativeRole()
 {
-  return nsIAccessibleRole::ROLE_SEPARATOR;
+  return roles::SEPARATOR;
 }
 
 PRUint64
@@ -801,13 +802,14 @@ nsXULTextFieldAccessible::NativeState()
   return state;
 }
 
-PRUint32
+role
 nsXULTextFieldAccessible::NativeRole()
 {
   if (mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
                             nsGkAtoms::password, eIgnoreCase))
-    return nsIAccessibleRole::ROLE_PASSWORD_TEXT;
-  return nsIAccessibleRole::ROLE_ENTRY;
+    return roles::PASSWORD_TEXT;
+  
+  return roles::ENTRY;
 }
 
 /**
