@@ -82,11 +82,10 @@ public:
   virtual void Shutdown();
 
   // nsAccessible
-  virtual PRUint32 NativeRole();
+  virtual mozilla::a11y::role NativeRole();
   virtual PRUint64 NativeState();
   virtual nsAccessible* ChildAtPoint(PRInt32 aX, PRInt32 aY,
                                      EWhichChildAtPoint aWhichChild);
-  virtual nsAccessible* FocusedChild();
 
   virtual nsAccessible* GetChildAt(PRUint32 aIndex);
   virtual PRInt32 GetChildCount();
@@ -102,6 +101,15 @@ public:
   virtual bool SelectAll();
   virtual bool UnselectAll();
 
+  // Widgets
+  virtual bool IsWidget() const;
+  virtual bool IsActiveWidget() const;
+  virtual bool AreItemsOperable() const;
+  virtual nsAccessible* CurrentItem();
+  virtual void SetCurrentItem(nsAccessible* aItem);
+
+  virtual nsAccessible* ContainerWidget() const;
+
   // nsXULTreeAccessible
 
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_XULTREEACCESSIBLE_IMPL_CID)
@@ -185,7 +193,7 @@ public:
   NS_IMETHOD GetBounds(PRInt32 *aX, PRInt32 *aY,
                        PRInt32 *aWidth, PRInt32 *aHeight);
 
-  NS_IMETHOD SetSelected(PRBool aSelect); 
+  NS_IMETHOD SetSelected(bool aSelect); 
   NS_IMETHOD TakeFocus();
 
   NS_IMETHOD GroupPosition(PRInt32 *aGroupLevel,
@@ -209,6 +217,9 @@ public:
   // ActionAccessible
   virtual PRUint8 ActionCount();
 
+  // Widgets
+  virtual nsAccessible* ContainerWidget() const;
+
   // nsXULTreeItemAccessibleBase
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_XULTREEITEMBASEACCESSIBLE_IMPL_CID)
 
@@ -242,7 +253,7 @@ protected:
   /**
    * Return true if the tree item accessible is expandable (contains subrows).
    */
-  PRBool IsExpandable();
+  bool IsExpandable();
 
   /**
    * Return name for cell at the given column.
@@ -277,11 +288,11 @@ public:
 
   // nsAccessNode
   virtual bool IsDefunct() const;
-  virtual PRBool Init();
+  virtual bool Init();
   virtual void Shutdown();
 
   // nsAccessible
-  virtual PRUint32 NativeRole();
+  virtual mozilla::a11y::role NativeRole();
 
   // nsXULTreeItemAccessibleBase
   virtual void RowInvalidated(PRInt32 aStartColIdx, PRInt32 aEndColIdx);
