@@ -40,6 +40,21 @@
 
 // gSyncUI handles updating the tools menu
 let gSyncUI = {
+  _obs: ["weave:service:sync:start",
+         "weave:service:sync:delayed",
+         "weave:service:quota:remaining",
+         "weave:service:setup-complete",
+         "weave:service:login:start",
+         "weave:service:login:finish",
+         "weave:service:logout:finish",
+         "weave:service:start-over",
+         "weave:ui:login:error",
+         "weave:ui:sync:error",
+         "weave:ui:sync:finish",
+         "weave:ui:clear-error"],
+
+  _unloaded: false,
+
   init: function SUI_init() {
     // Proceed to set up the UI if Sync has already started up.
     // Otherwise we'll do it when Sync is firing up.
@@ -52,48 +67,35 @@ let gSyncUI = {
 
     // Remove the observer if the window is closed before the observer
     // was triggered.
-    window.addEventListener("unload", function() {
-      window.removeEventListener("unload", arguments.callee, false);
+    window.addEventListener("unload", function onUnload() {
+      gSyncUI._unloaded = true;
+      window.removeEventListener("unload", onUnload, false);
       Services.obs.removeObserver(gSyncUI, "weave:service:ready");
+
+      if (Weave.Status.ready) {
+        gSyncUI._obs.forEach(function(topic) {
+          Services.obs.removeObserver(gSyncUI, topic);
+        });
+      }
     }, false);
   },
 
   initUI: function SUI_initUI() {
-    let obs = ["weave:service:sync:start",
-               "weave:service:sync:finish",
-               "weave:service:sync:error",
-               "weave:service:sync:delayed",
-               "weave:service:quota:remaining",
-               "weave:service:setup-complete",
-               "weave:service:login:start",
-               "weave:service:login:finish",
-               "weave:service:login:error",
-               "weave:service:logout:finish",
-               "weave:service:start-over"];
-
     // If this is a browser window?
     if (gBrowser) {
-      obs.push("weave:notification:added");
+      this._obs.push("weave:notification:added");
     }
 
-    obs.forEach(function(topic) {
+    this._obs.forEach(function(topic) {
       Services.obs.addObserver(this, topic, true);
     }, this);
 
-    // Find the alltabs-popup, only if there is a gBrowser
-    if (gBrowser) {
-      let popup = document.getElementById("alltabs-popup");
-      if (popup) {
-        popup.addEventListener(
-          "popupshowing", this.alltabsPopupShowing.bind(this), true);
-      }
-
-      if (Weave.Notifications.notifications.length)
-        this.initNotifications();
+    if (gBrowser && Weave.Notifications.notifications.length) {
+      this.initNotifications();
     }
     this.updateUI();
   },
-  
+
   initNotifications: function SUI_initNotifications() {
     const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
     let notificationbox = document.createElementNS(XULNS, "notificationbox");
@@ -139,34 +141,6 @@ let gSyncUI = {
       button.removeAttribute("tooltiptext");
   },
 
-  alltabsPopupShowing: function(event) {
-    // Should we show the menu item?
-    //XXXphilikon We should remove the check for isLoggedIn here and have
-    //            about:sync-tabs auto-login (bug 583344)
-    if (!Weave.Service.isLoggedIn || !Weave.Engines.get("tabs").enabled)
-      return;
-
-    let label = this._stringBundle.GetStringFromName("tabs.fromOtherComputers.label");
-
-    let popup = document.getElementById("alltabs-popup");
-    if (!popup)
-      return;
-    
-    let menuitem = document.createElement("menuitem");
-    menuitem.setAttribute("id", "sync-tabs-menuitem");
-    menuitem.setAttribute("label", label);
-    menuitem.setAttribute("class", "alltabs-item");
-    menuitem.setAttribute("oncommand", "BrowserOpenSyncTabs();");
-
-    // Fake the tab object on the menu entries, so that we don't have to worry
-    // about removing them ourselves. They will just get cleaned up by popup
-    // binding.
-    menuitem.tab = { "linkedBrowser": { "currentURI": { "spec": label } } };
-
-    let sep = document.getElementById("alltabs-popup-separator");
-    popup.insertBefore(menuitem, sep);
-  },
-
 
   // Functions called by observers
   onActivityStart: function SUI_onActivityStart() {
@@ -180,14 +154,6 @@ let gSyncUI = {
     button.setAttribute("status", "active");
   },
 
-  onSyncFinish: function SUI_onSyncFinish() {
-    this._onSyncEnd(true);
-  },
-
-  onSyncError: function SUI_onSyncError() {
-    this._onSyncEnd(false);
-  },
-
   onSyncDelay: function SUI_onSyncDelay() {
     // basically, we want to just inform users that stuff is going to take a while
     let title = this._stringBundle.GetStringFromName("error.sync.no_node_found.title");
@@ -206,9 +172,25 @@ let gSyncUI = {
   onLoginFinish: function SUI_onLoginFinish() {
     // Clear out any login failure notifications
     let title = this._stringBundle.GetStringFromName("error.login.title");
-    Weave.Notifications.removeAll(title);
+    this.clearError(title);
+  },
 
-    this.updateUI();
+  // Set visibility of "Setup Sync" link
+  showSetupSyncAboutHome: function SUI_showSetupSyncAboutHome(toShow) {
+    let browsers = gBrowser.browsers;
+    for (let i = 0; i < browsers.length; i++) {
+      let b = browsers[i];
+      if ("about:home" == b.currentURI.spec) {
+        b.contentDocument.getElementById("setupSyncLink").hidden = !toShow;
+      }
+    }
+  },
+
+  onSetupComplete: function SUI_onSetupComplete() {
+    // Remove "setup sync" link in about:home if it is open. 
+    this.showSetupSyncAboutHome(false);
+
+    onLoginFinish();
   },
 
   onLoginError: function SUI_onLoginError() {
@@ -216,15 +198,26 @@ let gSyncUI = {
     Weave.Notifications.removeAll();
 
     // if we haven't set up the client, don't show errors
-    if (this._needsSetup() || Weave.Service.shouldIgnoreError()) {
+    if (this._needsSetup()) {
       this.updateUI();
       return;
     }
 
     let title = this._stringBundle.GetStringFromName("error.login.title");
-    let reason = Weave.Utils.getErrorString(Weave.Status.login);
-    let description =
-      this._stringBundle.formatStringFromName("error.login.description", [reason], 1);
+
+    let description;
+    if (Weave.Status.sync == Weave.PROLONGED_SYNC_FAILURE) {
+      // Convert to days
+      let lastSync =
+        Services.prefs.getIntPref("services.sync.errorhandler.networkFailureReportTimeout") / 86400;
+      description =
+        this._stringBundle.formatStringFromName("error.sync.prolonged_failure", [lastSync], 1);
+    } else {
+      let reason = Weave.Utils.getErrorString(Weave.Status.login);
+      description =
+        this._stringBundle.formatStringFromName("error.sync.description", [reason], 1);
+    }
+
     let buttons = [];
     buttons.push(new Weave.NotificationButton(
       this._stringBundle.GetStringFromName("error.login.prefs.label"),
@@ -243,8 +236,9 @@ let gSyncUI = {
   },
 
   onStartOver: function SUI_onStartOver() {
-    Weave.Notifications.removeAll();
-    this.updateUI();
+    this.clearError();
+    // Make "setup sync" link visible in about:home if it is open. 
+    this.showSetupSyncAboutHome(true);
   },
 
   onQuotaNotice: function onQuotaNotice(subject, data) {
@@ -269,7 +263,7 @@ let gSyncUI = {
 
   // Commands
   doSync: function SUI_doSync() {
-    setTimeout(function() Weave.Service.sync(), 0);
+    setTimeout(function() Weave.ErrorHandler.syncAndReportErrors(), 0);
   },
 
   handleToolbarButton: function SUI_handleStatusbarButton() {
@@ -281,21 +275,45 @@ let gSyncUI = {
 
   //XXXzpao should be part of syncCommon.js - which we might want to make a module...
   //        To be fixed in a followup (bug 583366)
-  openSetup: function SUI_openSetup() {
+
+  /**
+   * Invoke the Sync setup wizard.
+   *
+   * @param wizardType
+   *        Indicates type of wizard to launch:
+   *          null    -- regular set up wizard
+   *          "pair"  -- pair a device first
+   *          "reset" -- reset sync
+   */
+
+  openSetup: function SUI_openSetup(wizardType) {
     let win = Services.wm.getMostRecentWindow("Weave:AccountSetup");
     if (win)
       win.focus();
     else {
       window.openDialog("chrome://browser/content/syncSetup.xul",
-                        "weaveSetup", "centerscreen,chrome,resizable=no");
+                        "weaveSetup", "centerscreen,chrome,resizable=no",
+                        wizardType);
     }
   },
 
+  openAddDevice: function () {
+    if (!Weave.Utils.ensureMPUnlocked())
+      return;
+
+    let win = Services.wm.getMostRecentWindow("Sync:AddDevice");
+    if (win)
+      win.focus();
+    else
+      window.openDialog("chrome://browser/content/syncAddDevice.xul",
+                        "syncAddDevice", "centerscreen,chrome,resizable=no");
+  },
+
   openQuotaDialog: function SUI_openQuotaDialog() {
     let win = Services.wm.getMostRecentWindow("Sync:ViewQuota");
     if (win)
       win.focus();
-    else 
+    else
       Services.ww.activeWindow.openDialog(
         "chrome://browser/content/syncQuota.xul", "",
         "centerscreen,chrome,dialog,modal");
@@ -328,84 +346,98 @@ let gSyncUI = {
     // Show the day-of-week and time (HH:MM) of last sync
     let lastSyncDate = new Date(lastSync).toLocaleFormat("%a %H:%M");
     let lastSyncLabel =
-      this._stringBundle.formatStringFromName("lastSync.label", [lastSyncDate], 1);
+      this._stringBundle.formatStringFromName("lastSync2.label", [lastSyncDate], 1);
 
     syncButton.setAttribute("tooltiptext", lastSyncLabel);
   },
 
-  _onSyncEnd: function SUI__onSyncEnd(success) {
+  clearError: function SUI_clearError(errorString) {
+    Weave.Notifications.removeAll(errorString);
+    this.updateUI();
+  },
+
+  onSyncFinish: function SUI_onSyncFinish() {
     let title = this._stringBundle.GetStringFromName("error.sync.title");
-    if (!success) {
-      if (Weave.Status.login != Weave.LOGIN_SUCCEEDED) {
-        this.onLoginError();
-        return;
-      }
 
-      // Ignore network related errors unless we haven't been able to
-      // sync for a while.
-      if (Weave.Service.shouldIgnoreError()) {
-        this.updateUI();
-        return;
-      }
+    // Clear out sync failures on a successful sync
+    this.clearError(title);
 
+    if (this._wasDelayed && Weave.Status.sync != Weave.NO_SYNC_NODE_FOUND) {
+      title = this._stringBundle.GetStringFromName("error.sync.no_node_found.title");
+      this.clearError(title);
+      this._wasDelayed = false;
+    }
+  },
+
+  onSyncError: function SUI_onSyncError() {
+    let title = this._stringBundle.GetStringFromName("error.sync.title");
+
+    if (Weave.Status.login != Weave.LOGIN_SUCCEEDED) {
+      this.onLoginError();
+      return;
+    }
+
+    let description;
+    if (Weave.Status.sync == Weave.PROLONGED_SYNC_FAILURE) {
+      // Convert to days
+      let lastSync =
+        Services.prefs.getIntPref("services.sync.errorhandler.networkFailureReportTimeout") / 86400;
+      description =
+        this._stringBundle.formatStringFromName("error.sync.prolonged_failure", [lastSync], 1);
+    } else {
       let error = Weave.Utils.getErrorString(Weave.Status.sync);
-      let description =
+      description =
         this._stringBundle.formatStringFromName("error.sync.description", [error], 1);
+    }
+    let priority = Weave.Notifications.PRIORITY_WARNING;
+    let buttons = [];
 
-      let priority = Weave.Notifications.PRIORITY_WARNING;
-      let buttons = [];
+    // Check if the client is outdated in some way
+    let outdated = Weave.Status.sync == Weave.VERSION_OUT_OF_DATE;
+    for (let [engine, reason] in Iterator(Weave.Status.engines))
+      outdated = outdated || reason == Weave.VERSION_OUT_OF_DATE;
 
-      // Check if the client is outdated in some way
-      let outdated = Weave.Status.sync == Weave.VERSION_OUT_OF_DATE;
-      for (let [engine, reason] in Iterator(Weave.Status.engines))
-        outdated = outdated || reason == Weave.VERSION_OUT_OF_DATE;
-
-      if (outdated) {
-        description = this._stringBundle.GetStringFromName(
-          "error.sync.needUpdate.description");
-        buttons.push(new Weave.NotificationButton(
-          this._stringBundle.GetStringFromName("error.sync.needUpdate.label"),
-          this._stringBundle.GetStringFromName("error.sync.needUpdate.accesskey"),
-          function() { window.openUILinkIn("https://services.mozilla.com/update/", "tab"); return true; }
-        ));
-      }
-      else if (Weave.Status.sync == Weave.OVER_QUOTA) {
-        description = this._stringBundle.GetStringFromName(
-          "error.sync.quota.description");
-        buttons.push(new Weave.NotificationButton(
-          this._stringBundle.GetStringFromName(
-            "error.sync.viewQuotaButton.label"),
-          this._stringBundle.GetStringFromName(
-            "error.sync.viewQuotaButton.accesskey"),
-          function() { gSyncUI.openQuotaDialog(); return true; } )
-        );
-      }
-      else if (Weave.Status.enforceBackoff) {
-        priority = Weave.Notifications.PRIORITY_INFO;
-        buttons.push(new Weave.NotificationButton(
-          this._stringBundle.GetStringFromName("error.sync.serverStatusButton.label"),
-          this._stringBundle.GetStringFromName("error.sync.serverStatusButton.accesskey"),
-          function() { gSyncUI.openServerStatus(); return true; }
-        ));
-      }
-      else {
-        priority = Weave.Notifications.PRIORITY_INFO;
-        buttons.push(new Weave.NotificationButton(
-          this._stringBundle.GetStringFromName("error.sync.tryAgainButton.label"),
-          this._stringBundle.GetStringFromName("error.sync.tryAgainButton.accesskey"),
-          function() { gSyncUI.doSync(); return true; }
-        ));
-      }
-
-      let notification =
-        new Weave.Notification(title, description, null, priority, buttons);
-      Weave.Notifications.replaceTitle(notification);
+    if (outdated) {
+      description = this._stringBundle.GetStringFromName(
+        "error.sync.needUpdate.description");
+      buttons.push(new Weave.NotificationButton(
+        this._stringBundle.GetStringFromName("error.sync.needUpdate.label"),
+        this._stringBundle.GetStringFromName("error.sync.needUpdate.accesskey"),
+        function() { window.openUILinkIn("https://services.mozilla.com/update/", "tab"); return true; }
+      ));
+    }
+    else if (Weave.Status.sync == Weave.OVER_QUOTA) {
+      description = this._stringBundle.GetStringFromName(
+        "error.sync.quota.description");
+      buttons.push(new Weave.NotificationButton(
+        this._stringBundle.GetStringFromName(
+          "error.sync.viewQuotaButton.label"),
+        this._stringBundle.GetStringFromName(
+          "error.sync.viewQuotaButton.accesskey"),
+        function() { gSyncUI.openQuotaDialog(); return true; } )
+      );
+    }
+    else if (Weave.Status.enforceBackoff) {
+      priority = Weave.Notifications.PRIORITY_INFO;
+      buttons.push(new Weave.NotificationButton(
+        this._stringBundle.GetStringFromName("error.sync.serverStatusButton.label"),
+        this._stringBundle.GetStringFromName("error.sync.serverStatusButton.accesskey"),
+        function() { gSyncUI.openServerStatus(); return true; }
+      ));
     }
     else {
-      // Clear out sync failures on a successful sync
-      Weave.Notifications.removeAll(title);
+      priority = Weave.Notifications.PRIORITY_INFO;
+      buttons.push(new Weave.NotificationButton(
+        this._stringBundle.GetStringFromName("error.sync.tryAgainButton.label"),
+        this._stringBundle.GetStringFromName("error.sync.tryAgainButton.accesskey"),
+        function() { gSyncUI.doSync(); return true; }
+      ));
     }
 
+    let notification =
+      new Weave.Notification(title, description, null, priority, buttons);
+    Weave.Notifications.replaceTitle(notification);
+
     if (this._wasDelayed && Weave.Status.sync != Weave.NO_SYNC_NODE_FOUND) {
       title = this._stringBundle.GetStringFromName("error.sync.no_node_found.title");
       Weave.Notifications.removeAll(title);
@@ -414,16 +446,21 @@ let gSyncUI = {
 
     this.updateUI();
   },
-  
+
   observe: function SUI_observe(subject, topic, data) {
+    if (this._unloaded) {
+      Cu.reportError("SyncUI observer called after unload: " + topic);
+      return;
+    }
+
     switch (topic) {
       case "weave:service:sync:start":
         this.onActivityStart();
         break;
-      case "weave:service:sync:finish":
+      case "weave:ui:sync:finish":
         this.onSyncFinish();
         break;
-      case "weave:service:sync:error":
+      case "weave:ui:sync:error":
         this.onSyncError();
         break;
       case "weave:service:sync:delayed":
@@ -433,7 +470,7 @@ let gSyncUI = {
         this.onQuotaNotice();
         break;
       case "weave:service:setup-complete":
-        this.onLoginFinish();
+        this.onSetupComplete();
         break;
       case "weave:service:login:start":
         this.onActivityStart();
@@ -441,7 +478,7 @@ let gSyncUI = {
       case "weave:service:login:finish":
         this.onLoginFinish();
         break;
-      case "weave:service:login:error":
+      case "weave:ui:login:error":
         this.onLoginError();
         break;
       case "weave:service:logout:finish":
@@ -456,6 +493,9 @@ let gSyncUI = {
       case "weave:notification:added":
         this.initNotifications();
         break;
+      case "weave:ui:clear-error":
+        this.clearError();
+        break;
     }
   },
 
