@@ -130,45 +130,28 @@ const APP_ICON_ATTR_NAME = "appHandlerIcon";
 //****************************************************************************//
 // Utilities
 
-function getDisplayNameForFile(aFile) {
-/*
+function getFileDisplayName(file) {
 #ifdef XP_WIN
-*/
-  if (aFile instanceof Ci.nsILocalFileWin) {
+  if (file instanceof Ci.nsILocalFileWin) {
     try {
-      return aFile.getVersionInfoField("FileDescription"); 
-    }
-    catch(ex) {
-      // fall through to the file name
-    }
+      return file.getVersionInfoField("FileDescription");
+    } catch (e) {}
   }
-/*
 #endif
 #ifdef XP_MACOSX
-*/
-  if (aFile instanceof Ci.nsILocalFileMac) {
+  if (file instanceof Ci.nsILocalFileMac) {
     try {
-      return aFile.bundleDisplayName;
-    }
-    catch(ex) {
-      // fall through to the file name
-    }
+      return file.bundleDisplayName;
+    } catch (e) {}
   }
-/*
 #endif
-*/
-
-  return Cc["@mozilla.org/network/io-service;1"].
-         getService(Ci.nsIIOService).
-         newFileURI(aFile).
-         QueryInterface(Ci.nsIURL).
-         fileName;
+  return file.leafName;
 }
 
 function getLocalHandlerApp(aFile) {
   var localHandlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                         createInstance(Ci.nsILocalHandlerApp);
-  localHandlerApp.name = getDisplayNameForFile(aFile);
+  localHandlerApp.name = getFileDisplayName(aFile);
   localHandlerApp.executable = aFile;
 
   return localHandlerApp;
@@ -496,10 +479,6 @@ HandlerInfoWrapper.prototype = {
     return this._getIcon(16);
   },
 
-  get largeIcon() {
-    return this._getIcon(32);
-  },
-
   _getIcon: function(aSize) {
     if (this.primaryExtension)
       return "moz-icon://goat." + this.primaryExtension + "?size=" + aSize;
@@ -661,7 +640,7 @@ FeedHandlerInfo.prototype = {
     }
 
     // Add the registered web handlers.  There can be any number of these.
-    var webHandlers = this._converterSvc.getContentHandlers(this.type, {});
+    var webHandlers = this._converterSvc.getContentHandlers(this.type);
     for each (let webHandler in webHandlers)
       this._possibleApplicationHandlers.appendElement(webHandler, false);
 
@@ -686,7 +665,7 @@ FeedHandlerInfo.prototype = {
     if (defaultFeedReader) {
       let handlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                        createInstance(Ci.nsIHandlerApp);
-      handlerApp.name = getDisplayNameForFile(defaultFeedReader);
+      handlerApp.name = getFileDisplayName(defaultFeedReader);
       handlerApp.QueryInterface(Ci.nsILocalHandlerApp);
       handlerApp.executable = defaultFeedReader;
 
@@ -838,10 +817,6 @@ FeedHandlerInfo.prototype = {
 
   get smallIcon() {
     return this._smallIcon;
-  },
-
-  get largeIcon() {
-    return this._largeIcon;
   }
 
 };
@@ -853,7 +828,6 @@ var feedHandlerInfo = {
   _prefSelectedAction: PREF_FEED_SELECTED_ACTION, 
   _prefSelectedReader: PREF_FEED_SELECTED_READER,
   _smallIcon: "chrome://browser/skin/feeds/feedIcon16.png",
-  _largeIcon: "chrome://browser/skin/feeds/feedIcon.png",
   _appPrefLabel: "webFeed"
 }
 
@@ -864,7 +838,6 @@ var videoFeedHandlerInfo = {
   _prefSelectedAction: PREF_VIDEO_FEED_SELECTED_ACTION, 
   _prefSelectedReader: PREF_VIDEO_FEED_SELECTED_READER,
   _smallIcon: "chrome://browser/skin/feeds/videoFeedIcon16.png",
-  _largeIcon: "chrome://browser/skin/feeds/videoFeedIcon.png",
   _appPrefLabel: "videoPodcastFeed"
 }
 
@@ -875,7 +848,6 @@ var audioFeedHandlerInfo = {
   _prefSelectedAction: PREF_AUDIO_FEED_SELECTED_ACTION, 
   _prefSelectedReader: PREF_AUDIO_FEED_SELECTED_READER,
   _smallIcon: "chrome://browser/skin/feeds/audioFeedIcon16.png",
-  _largeIcon: "chrome://browser/skin/feeds/audioFeedIcon.png",
   _appPrefLabel: "audioPodcastFeed"
 }
 
@@ -1281,7 +1253,7 @@ var gApplicationsPane = {
         var preferredApp = aHandlerInfo.preferredApplicationHandler;
         var name;
         if (preferredApp instanceof Ci.nsILocalHandlerApp)
-          name = getDisplayNameForFile(preferredApp.executable);
+          name = getFileDisplayName(preferredApp.executable);
         else
           name = preferredApp.name;
         return this._prefsBundle.getFormattedString("useApp", [name]);
@@ -1369,7 +1341,7 @@ var gApplicationsPane = {
 #expand    aExecutable.leafName != "__MOZ_APP_NAME__.exe";
 #else
 #ifdef XP_MACOSX
-#expand    aExecutable.leafName != "__MOZ_APP_DISPLAYNAME__.app";
+#expand    aExecutable.leafName != "__MOZ_MACBUNDLE_NAME__";
 #else
 #expand    aExecutable.leafName != "__MOZ_APP_NAME__-bin";
 #endif
@@ -1464,7 +1436,7 @@ var gApplicationsPane = {
       menuItem.setAttribute("action", Ci.nsIHandlerInfo.useHelperApp);
       let label;
       if (possibleApp instanceof Ci.nsILocalHandlerApp)
-        label = getDisplayNameForFile(possibleApp.executable);
+        label = getFileDisplayName(possibleApp.executable);
       else
         label = possibleApp.name;
       label = this._prefsBundle.getFormattedString("useApp", [label]);
@@ -1734,9 +1706,7 @@ var gApplicationsPane = {
                       "chrome,modal,centerscreen,titlebar,dialog=yes",
                       params);
 
-    if (params.handlerApp && 
-        params.handlerApp.executable && 
-        params.handlerApp.executable.isFile()) {
+    if (this.isValidHandlerApp(params.handlerApp)) {
       handlerApp = params.handlerApp;
 
       // Add the app to the type's list of possible handlers.
@@ -1754,7 +1724,7 @@ var gApplicationsPane = {
         this._isValidHandlerExecutable(fp.file)) {
       handlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                    createInstance(Ci.nsILocalHandlerApp);
-      handlerApp.name = getDisplayNameForFile(fp.file);
+      handlerApp.name = getFileDisplayName(fp.file);
       handlerApp.executable = fp.file;
 
       // Add the app to the type's list of possible handlers.
