@@ -97,6 +97,7 @@
 #include "nsIChannelPolicy.h"
 #include "nsChannelPolicy.h"
 #include "mozilla/dom/Element.h"
+#include "sampler.h"
 
 #ifdef PR_LOGGING
 static PRLogModuleInfo* gObjectLog = PR_NewLogModule("objlc");
@@ -105,6 +106,8 @@ static PRLogModuleInfo* gObjectLog = PR_NewLogModule("objlc");
 #define LOG(args) PR_LOG(gObjectLog, PR_LOG_DEBUG, args)
 #define LOG_ENABLED() PR_LOG_TEST(gObjectLog, PR_LOG_DEBUG)
 
+#include "mozilla/Preferences.h"
+
 class nsAsyncInstantiateEvent : public nsRunnable {
 public:
   // This stores both the content and the frame so that Instantiate calls can be
@@ -166,7 +169,7 @@ nsAsyncInstantiateEvent::Run()
 
     nsresult rv = mContent->Instantiate(frame, mContentType, mURI);
     if (NS_FAILED(rv)) {
-      mContent->Fallback(PR_TRUE);
+      mContent->Fallback(true);
     }
   } else {
     LOG(("OBJLC [%p]: Discarding event, data changed\n", mContent));
@@ -200,6 +203,9 @@ nsPluginErrorEvent::Run()
        mContent.get()));
   nsString type;
   switch (mState) {
+    case ePluginClickToPlay:
+      type = NS_LITERAL_STRING("PluginClickToPlay");
+      break;
     case ePluginUnsupported:
       type = NS_LITERAL_STRING("PluginNotFound");
       break;
@@ -216,7 +222,7 @@ nsPluginErrorEvent::Run()
       return NS_OK;
   }
   nsContentUtils::DispatchTrustedEvent(mContent->GetDocument(), mContent,
-                                       type, PR_TRUE, PR_TRUE);
+                                       type, true, true);
 
   return NS_OK;
 }
@@ -231,14 +237,14 @@ public:
   nsString mBrowserDumpID;
   nsString mPluginName;
   nsString mPluginFilename;
-  PRBool mSubmittedCrashReport;
+  bool mSubmittedCrashReport;
 
   nsPluginCrashedEvent(nsIContent* aContent,
                        const nsAString& aPluginDumpID,
                        const nsAString& aBrowserDumpID,
                        const nsAString& aPluginName,
                        const nsAString& aPluginFilename,
-                       PRBool submittedCrashReport)
+                       bool submittedCrashReport)
     : mContent(aContent),
       mPluginDumpID(aPluginDumpID),
       mBrowserDumpID(aBrowserDumpID),
@@ -275,8 +281,8 @@ nsPluginCrashedEvent::Run()
     return NS_OK;
   }
 
-  event->InitEvent(NS_LITERAL_STRING("PluginCrashed"), PR_TRUE, PR_TRUE);
-  privateEvent->SetTrusted(PR_TRUE);
+  event->InitEvent(NS_LITERAL_STRING("PluginCrashed"), true, true);
+  privateEvent->SetTrusted(true);
   privateEvent->GetInternalNSEvent()->flags |= NS_EVENT_FLAG_ONLY_CHROME_DISPATCH;
   
   nsCOMPtr<nsIWritableVariant> variant;
@@ -332,13 +338,13 @@ nsPluginCrashedEvent::Run()
 
 class AutoNotifier {
   public:
-    AutoNotifier(nsObjectLoadingContent* aContent, PRBool aNotify) :
+    AutoNotifier(nsObjectLoadingContent* aContent, bool aNotify) :
       mContent(aContent), mNotify(aNotify) {
         mOldType = aContent->Type();
         mOldState = aContent->ObjectState();
     }
     ~AutoNotifier() {
-      mContent->NotifyStateChanged(mOldType, mOldState, PR_FALSE, mNotify);
+      mContent->NotifyStateChanged(mOldType, mOldState, false, mNotify);
     }
 
     /**
@@ -349,14 +355,14 @@ class AutoNotifier {
     void Notify() {
       NS_ASSERTION(mNotify, "Should not notify when notify=false");
 
-      mContent->NotifyStateChanged(mOldType, mOldState, PR_TRUE, PR_TRUE);
+      mContent->NotifyStateChanged(mOldType, mOldState, true, true);
       mOldType = mContent->Type();
       mOldState = mContent->ObjectState();
     }
 
   private:
     nsObjectLoadingContent*            mContent;
-    PRBool                             mNotify;
+    bool                               mNotify;
     nsObjectLoadingContent::ObjectType mOldType;
     nsEventStates                      mOldState;
 };
@@ -372,7 +378,7 @@ class AutoFallback {
     ~AutoFallback() {
       if (NS_FAILED(*mResult)) {
         LOG(("OBJLC [%p]: rv=%08x, falling back\n", mContent, *mResult));
-        mContent->Fallback(PR_FALSE);
+        mContent->Fallback(false);
         if (mPluginState != ePluginOtherState) {
           mContent->mFallbackReason = mPluginState;
         }
@@ -400,34 +406,34 @@ class AutoFallback {
 class AutoSetInstantiatingToFalse {
   public:
     AutoSetInstantiatingToFalse(nsObjectLoadingContent* objlc) : mContent(objlc) {}
-    ~AutoSetInstantiatingToFalse() { mContent->mInstantiating = PR_FALSE; }
+    ~AutoSetInstantiatingToFalse() { mContent->mInstantiating = false; }
   private:
     nsObjectLoadingContent* mContent;
 };
 
 // helper functions
-static PRBool
+static bool
 IsSupportedImage(const nsCString& aMimeType)
 {
   imgILoader* loader = nsContentUtils::GetImgLoader();
   if (!loader) {
-    return PR_FALSE;
+    return false;
   }
 
-  PRBool supported;
+  bool supported;
   nsresult rv = loader->SupportImageWithMimeType(aMimeType.get(), &supported);
   return NS_SUCCEEDED(rv) && supported;
 }
 
-static PRBool
-IsSupportedPlugin(const nsCString& aMIMEType)
+static bool
+IsSupportedPlugin(const nsCString& aMIMEType, bool aShouldPlay)
 {
   nsCOMPtr<nsIPluginHost> pluginHostCOM(do_GetService(MOZ_PLUGIN_HOST_CONTRACTID));
   nsPluginHost *pluginHost = static_cast<nsPluginHost*>(pluginHostCOM.get());
   if (!pluginHost) {
-    return PR_FALSE;
+    return false;
   }
-  nsresult rv = pluginHost->IsPluginEnabledForType(aMIMEType.get());
+  nsresult rv = pluginHost->IsPluginEnabledForType(aMIMEType.get(), aShouldPlay);
   return NS_SUCCEEDED(rv);
 }
 
@@ -452,38 +458,41 @@ GetExtensionFromURI(nsIURI* uri, nsCString& ext)
  * Checks whether a plugin exists and is enabled for the extension
  * in the given URI. The MIME type is returned in the mimeType out parameter.
  */
-static PRBool
-IsPluginEnabledByExtension(nsIURI* uri, nsCString& mimeType)
+static bool
+IsPluginEnabledByExtension(nsIURI* uri, nsCString& mimeType, bool aShouldPlay)
 {
   nsCAutoString ext;
   GetExtensionFromURI(uri, ext);
 
   if (ext.IsEmpty()) {
-    return PR_FALSE;
+    return false;
   }
 
   nsCOMPtr<nsIPluginHost> pluginHostCOM(do_GetService(MOZ_PLUGIN_HOST_CONTRACTID));
   nsPluginHost *pluginHost = static_cast<nsPluginHost*>(pluginHostCOM.get());
   if (!pluginHost) {
-    return PR_FALSE;
+    return false;
   }
 
   const char* typeFromExt;
-  if (NS_SUCCEEDED(pluginHost->IsPluginEnabledForExtension(ext.get(), typeFromExt))) {
+  if (NS_SUCCEEDED(pluginHost->IsPluginEnabledForExtension(ext.get(), typeFromExt,
+                                                           aShouldPlay))) {
     mimeType = typeFromExt;
-    return PR_TRUE;
+    return true;
   }
-  return PR_FALSE;
+  return false;
 }
 
 nsObjectLoadingContent::nsObjectLoadingContent()
   : mPendingInstantiateEvent(nsnull)
   , mChannel(nsnull)
   , mType(eType_Loading)
-  , mInstantiating(PR_FALSE)
-  , mUserDisabled(PR_FALSE)
-  , mSuppressed(PR_FALSE)
-  , mNetworkCreated(PR_TRUE)
+  , mInstantiating(false)
+  , mUserDisabled(false)
+  , mSuppressed(false)
+  , mNetworkCreated(true)
+  // If plugins.click_to_play is false, plugins should always play
+  , mShouldPlay(!mozilla::Preferences::GetBool("plugins.click_to_play", false))
   , mFallbackReason(ePluginOtherState)
 {
 }
@@ -501,17 +510,18 @@ NS_IMETHODIMP
 nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
                                        nsISupports *aContext)
 {
-  if (aRequest != mChannel) {
+  SAMPLE_LABEL("nsObjectLoadingContent", "OnStartRequest");
+  if (aRequest != mChannel || !aRequest) {
     // This is a bit of an edge case - happens when a new load starts before the
     // previous one got here
     return NS_BINDING_ABORTED;
   }
 
-  AutoNotifier notifier(this, PR_TRUE);
+  AutoNotifier notifier(this, true);
 
   if (!IsSuccessfulRequest(aRequest)) {
     LOG(("OBJLC [%p]: OnStartRequest: Request failed\n", this));
-    Fallback(PR_FALSE);
+    Fallback(false);
     return NS_BINDING_ABORTED;
   }
 
@@ -545,7 +555,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
       // Need to check IsSupportedPlugin() in addition to GetTypeOfContent()
       // because otherwise the default plug-in's catch-all behavior would
       // confuse things.
-      (IsSupportedPlugin(mContentType) && 
+      (IsSupportedPlugin(mContentType, mShouldPlay) && 
        GetTypeOfContent(mContentType) == eType_Plugin)) {
     // Set the type we'll use for dispatch on the channel.  Otherwise we could
     // end up trying to dispatch to a nsFrameLoader, which will complain that
@@ -565,7 +575,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
 
   if (mContentType.EqualsASCII(APPLICATION_OCTET_STREAM)) {
     nsCAutoString extType;
-    if (IsPluginEnabledByExtension(uri, extType)) {
+    if (IsPluginEnabledByExtension(uri, extType, mShouldPlay)) {
       mContentType = extType;
       chan->SetContentType(extType);
     }
@@ -596,11 +606,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
     do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
   NS_ASSERTION(thisContent, "must be a content");
 
-  nsIDocument* doc = thisContent->GetOwnerDoc();
-  if (!doc) {
-    Fallback(PR_FALSE);
-    return NS_BINDING_ABORTED;    
-  }
+  nsIDocument* doc = thisContent->OwnerDoc();
 
   PRInt16 shouldProcess = nsIContentPolicy::ACCEPT;
   rv =
@@ -638,16 +644,17 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
       break;
     case eType_Document: {
       if (!mFrameLoader) {
-        mFrameLoader = nsFrameLoader::Create(thisContent, mNetworkCreated);
+        mFrameLoader = nsFrameLoader::Create(thisContent->AsElement(),
+                                             mNetworkCreated);
         if (!mFrameLoader) {
-          Fallback(PR_FALSE);
+          Fallback(false);
           return NS_ERROR_UNEXPECTED;
         }
       }
 
       rv = mFrameLoader->CheckForRecursiveLoad(uri);
       if (NS_FAILED(rv)) {
-        Fallback(PR_FALSE);
+        Fallback(false);
         return rv;
       }
 
@@ -687,7 +694,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
       break;
     }
     case eType_Plugin:
-      mInstantiating = PR_TRUE;
+      mInstantiating = true;
       if (mType != newType) {
         // This can go away once plugin loading moves to content (bug 90268)
         mType = newType;
@@ -699,7 +706,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
         // Do nothing in this case: This is probably due to a display:none
         // frame. If we ever get a frame, HasNewFrame will do the right thing.
         // Abort the load though, we have no use for the data.
-        mInstantiating = PR_FALSE;
+        mInstantiating = false;
         return NS_BINDING_ABORTED;
       }
 
@@ -710,7 +717,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
 
         rv = frame->Instantiate(chan, getter_AddRefs(mFinalListener));
 
-        mInstantiating = PR_FALSE;
+        mInstantiating = false;
 
         if (!weakFrame.IsAlive()) {
           // The frame was destroyed while instantiating. Abort the load.
@@ -726,7 +733,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
       // Need to fallback here (instead of using the case below), so that we can
       // set mFallbackReason without it being overwritten. This is also why we
       // return early.
-      Fallback(PR_FALSE);
+      Fallback(false);
 
       PluginSupportState pluginState = GetPluginSupportState(thisContent,
                                                              mContentType);
@@ -753,7 +760,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
         return NS_BINDING_ABORTED;
       }
 #endif
-      Fallback(PR_FALSE);
+      Fallback(false);
     } else if (mType == eType_Plugin) {
       nsIObjectFrame* frame = GetExistingFrame(eFlushContent);
       if (frame) {
@@ -767,7 +774,7 @@ nsObjectLoadingContent::OnStartRequest(nsIRequest *aRequest,
   }
 
   LOG(("OBJLC [%p]: Found no listener, falling back\n", this));
-  Fallback(PR_FALSE);
+  Fallback(false);
   return NS_BINDING_ABORTED;
 }
 
@@ -776,6 +783,8 @@ nsObjectLoadingContent::OnStopRequest(nsIRequest *aRequest,
                                       nsISupports *aContext,
                                       nsresult aStatusCode)
 {
+  NS_ENSURE_TRUE(nsContentUtils::IsCallerChrome(), NS_ERROR_NOT_AVAILABLE);
+
   if (aRequest != mChannel) {
     return NS_BINDING_ABORTED;
   }
@@ -794,8 +803,13 @@ nsObjectLoadingContent::OnStopRequest(nsIRequest *aRequest,
 
 // nsIStreamListener
 NS_IMETHODIMP
-nsObjectLoadingContent::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext, nsIInputStream *aInputStream, PRUint32 aOffset, PRUint32 aCount)
+nsObjectLoadingContent::OnDataAvailable(nsIRequest *aRequest,
+                                        nsISupports *aContext,
+                                        nsIInputStream *aInputStream,
+                                        PRUint32 aOffset, PRUint32 aCount)
 {
+  NS_ENSURE_TRUE(nsContentUtils::IsCallerChrome(), NS_ERROR_NOT_AVAILABLE);
+
   if (aRequest != mChannel) {
     return NS_BINDING_ABORTED;
   }
@@ -874,7 +888,7 @@ nsObjectLoadingContent::EnsureInstantiation(nsNPAPIPluginInstance** aInstance)
     }
 
     // Trigger frame construction
-    mInstantiating = PR_TRUE;
+    mInstantiating = true;
 
     nsCOMPtr<nsIContent> thisContent = 
       do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
@@ -883,13 +897,13 @@ nsObjectLoadingContent::EnsureInstantiation(nsNPAPIPluginInstance** aInstance)
     nsIDocument* doc = thisContent->GetCurrentDoc();
     if (!doc) {
       // Nothing we can do while plugin loading is done in layout...
-      mInstantiating = PR_FALSE;
+      mInstantiating = false;
       return NS_OK;
     }
 
     doc->FlushPendingNotifications(Flush_Frames);
 
-    mInstantiating = PR_FALSE;
+    mInstantiating = false;
 
     frame = GetExistingFrame(eFlushContent);
     if (!frame) {
@@ -922,7 +936,7 @@ nsObjectLoadingContent::EnsureInstantiation(nsNPAPIPluginInstance** aInstance)
     if (NS_SUCCEEDED(rv) && weakFrame.IsAlive()) {
       rv = frame->GetPluginInstance(aInstance);
     } else {
-      Fallback(PR_TRUE);
+      Fallback(true);
     }
   }
   return rv;
@@ -937,8 +951,8 @@ nsObjectLoadingContent::HasNewFrame(nsIObjectFrame* aFrame)
   nsCOMPtr<nsIContent> thisContent = 
     do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
   NS_ASSERTION(thisContent, "must be a content");
-  nsIDocument* doc = thisContent->GetOwnerDoc();
-  if (!doc || doc->IsStaticDocument() || doc->IsBeingUsedAsImage()) {
+  nsIDocument* doc = thisContent->OwnerDoc();
+  if (doc->IsStaticDocument() || doc->IsBeingUsedAsImage()) {
     return NS_OK;
   }
 
@@ -966,7 +980,7 @@ nsObjectLoadingContent::HasNewFrame(nsIObjectFrame* aFrame)
     // instantiate
     nsCOMPtr<nsIPluginDocument> pDoc (do_QueryInterface(GetOurDocument()));
     if (pDoc) {
-      PRBool willHandleInstantiation;
+      bool willHandleInstantiation;
       pDoc->GetWillHandleInstantiation(&willHandleInstantiation);
       if (willHandleInstantiation) {
         return NS_OK;
@@ -1069,6 +1083,8 @@ nsObjectLoadingContent::ObjectState() const
       // Otherwise, broken
       nsEventStates state = NS_EVENT_STATE_BROKEN;
       switch (mFallbackReason) {
+        case ePluginClickToPlay:
+          return NS_EVENT_STATE_TYPE_CLICK_TO_PLAY;
         case ePluginDisabled:
           state |= NS_EVENT_STATE_HANDLER_DISABLED;
           break;
@@ -1096,9 +1112,9 @@ nsObjectLoadingContent::ObjectState() const
 // <protected>
 nsresult
 nsObjectLoadingContent::LoadObject(const nsAString& aURI,
-                                   PRBool aNotify,
+                                   bool aNotify,
                                    const nsCString& aTypeHint,
-                                   PRBool aForceLoad)
+                                   bool aForceLoad)
 {
   LOG(("OBJLC [%p]: Loading object: URI string=<%s> notify=%i type=<%s> forceload=%i\n",
        this, NS_ConvertUTF16toUTF8(aURI).get(), aNotify, aTypeHint.get(), aForceLoad));
@@ -1110,7 +1126,7 @@ nsObjectLoadingContent::LoadObject(const nsAString& aURI,
     do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
   NS_ASSERTION(thisContent, "must be a content");
 
-  nsIDocument* doc = thisContent->GetOwnerDoc();
+  nsIDocument* doc = thisContent->OwnerDoc();
   nsCOMPtr<nsIURI> baseURI;
   GetObjectBaseURI(thisContent, getter_AddRefs(baseURI));
 
@@ -1145,15 +1161,15 @@ nsObjectLoadingContent::UpdateFallbackState(nsIContent* aContent,
 
 nsresult
 nsObjectLoadingContent::LoadObject(nsIURI* aURI,
-                                   PRBool aNotify,
+                                   bool aNotify,
                                    const nsCString& aTypeHint,
-                                   PRBool aForceLoad)
+                                   bool aForceLoad)
 {
   LOG(("OBJLC [%p]: Loading object: URI=<%p> notify=%i type=<%s> forceload=%i\n",
        this, aURI, aNotify, aTypeHint.get(), aForceLoad));
 
   if (mURI && aURI && !aForceLoad) {
-    PRBool equal;
+    bool equal;
     nsresult rv = mURI->Equals(aURI, &equal);
     if (NS_SUCCEEDED(rv) && equal) {
       // URI didn't change, do nothing
@@ -1173,10 +1189,10 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
   // the AutoNotifier triggers frame construction, events can be posted as
   // appropriate.
   NS_ASSERTION(!mInstantiating, "LoadObject was reentered?");
-  mInstantiating = PR_TRUE;
+  mInstantiating = true;
   AutoSetInstantiatingToFalse autoset(this);
 
-  mUserDisabled = mSuppressed = PR_FALSE;
+  mUserDisabled = mSuppressed = false;
 
   mURI = aURI;
   mContentType = aTypeHint;
@@ -1185,8 +1201,8 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
     do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
   NS_ASSERTION(thisContent, "must be a content");
 
-  nsIDocument* doc = thisContent->GetOwnerDoc();
-  if (!doc || doc->IsBeingUsedAsImage()) {
+  nsIDocument* doc = thisContent->OwnerDoc();
+  if (doc->IsBeingUsedAsImage()) {
     return NS_OK;
   }
 
@@ -1213,7 +1229,7 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
   // Security checks
   if (doc->IsLoadedAsData()) {
     if (!doc->IsStaticDocument()) {
-      Fallback(PR_FALSE);
+      Fallback(false);
     }
     return NS_OK;
   }
@@ -1228,7 +1244,7 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
     nsresult rv =
       secMan->CheckLoadURIWithPrincipal(thisContent->NodePrincipal(), aURI, 0);
     if (NS_FAILED(rv)) {
-      Fallback(PR_FALSE);
+      Fallback(false);
       return NS_OK;
     }
 
@@ -1259,8 +1275,8 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
 
   nsCAutoString overrideType;
   if ((caps & eOverrideServerType) &&
-      ((!aTypeHint.IsEmpty() && IsSupportedPlugin(aTypeHint)) ||
-       (aURI && IsPluginEnabledByExtension(aURI, overrideType)))) {
+      ((!aTypeHint.IsEmpty() && IsSupportedPlugin(aTypeHint, mShouldPlay)) ||
+       (aURI && IsPluginEnabledByExtension(aURI, overrideType, mShouldPlay)))) {
     ObjectType newType;
     if (overrideType.IsEmpty()) {
       newType = GetTypeOfContent(aTypeHint);
@@ -1277,7 +1293,8 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
       // Must have a frameloader before creating a frame, or the frame will
       // create its own.
       if (!mFrameLoader && newType == eType_Document) {
-        mFrameLoader = nsFrameLoader::Create(thisContent, mNetworkCreated);
+        mFrameLoader = nsFrameLoader::Create(thisContent->AsElement(),
+                                             mNetworkCreated);
         if (!mFrameLoader) {
           mURI = nsnull;
           return NS_OK;
@@ -1296,7 +1313,7 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
       case eType_Image:
         // Don't notify, because we will take care of that ourselves.
         if (aURI) {
-          rv = LoadImage(aURI, aForceLoad, PR_FALSE);
+          rv = LoadImage(aURI, aForceLoad, false);
         } else {
           rv = NS_ERROR_NOT_AVAILABLE;
         }
@@ -1323,14 +1340,14 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
 
   // If the class ID specifies a supported plugin, or if we have no explicit URI
   // but a type, immediately instantiate the plugin.
-  PRBool isSupportedClassID = PR_FALSE;
-  nsCAutoString typeForID; // Will be set iff isSupportedClassID == PR_TRUE
-  PRBool hasID = PR_FALSE;
+  bool isSupportedClassID = false;
+  nsCAutoString typeForID; // Will be set iff isSupportedClassID == true
+  bool hasID = false;
   if (caps & eSupportClassID) {
     nsAutoString classid;
     thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::classid, classid);
     if (!classid.IsEmpty()) {
-      hasID = PR_TRUE;
+      hasID = true;
       isSupportedClassID = NS_SUCCEEDED(TypeForClassID(classid, typeForID));
     }
   }
@@ -1397,7 +1414,7 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
       return NS_OK;
     }
 
-    if (IsSupportedPlugin(aTypeHint)) {
+    if (IsSupportedPlugin(aTypeHint, mShouldPlay)) {
       mType = eType_Plugin;
 
       rv = TryInstantiate(aTypeHint, aURI);
@@ -1444,7 +1461,7 @@ nsObjectLoadingContent::LoadObject(nsIURI* aURI,
 
   // Set up the channel's principal and such, like nsDocShell::DoURILoad does
   nsContentUtils::SetUpChannelOwner(thisContent->NodePrincipal(),
-                                    chan, aURI, PR_TRUE);
+                                    chan, aURI, true);
 
   nsCOMPtr<nsIScriptChannel> scriptChannel = do_QueryInterface(chan);
   if (scriptChannel) {
@@ -1475,7 +1492,7 @@ nsObjectLoadingContent::GetCapabilities() const
 }
 
 void
-nsObjectLoadingContent::Fallback(PRBool aNotify)
+nsObjectLoadingContent::Fallback(bool aNotify)
 {
   LOG(("OBJLC [%p]: Falling back (Notify=%i)\n", this, aNotify));
 
@@ -1509,45 +1526,45 @@ nsObjectLoadingContent::Traverse(nsObjectLoadingContent *tmp,
 }
 
 // <private>
-/* static */ PRBool
+/* static */ bool
 nsObjectLoadingContent::IsSuccessfulRequest(nsIRequest* aRequest)
 {
   nsresult status;
   nsresult rv = aRequest->GetStatus(&status);
   if (NS_FAILED(rv) || NS_FAILED(status)) {
-    return PR_FALSE;
+    return false;
   }
 
   // This may still be an error page or somesuch
   nsCOMPtr<nsIHttpChannel> httpChan(do_QueryInterface(aRequest));
   if (httpChan) {
-    PRBool success;
+    bool success;
     rv = httpChan->GetRequestSucceeded(&success);
     if (NS_FAILED(rv) || !success) {
-      return PR_FALSE;
+      return false;
     }
   }
 
   // Otherwise, the request is successful
-  return PR_TRUE;
+  return true;
 }
 
-/* static */ PRBool
+/* static */ bool
 nsObjectLoadingContent::CanHandleURI(nsIURI* aURI)
 {
   nsCAutoString scheme;
   if (NS_FAILED(aURI->GetScheme(scheme))) {
-    return PR_FALSE;
+    return false;
   }
 
   nsIIOService* ios = nsContentUtils::GetIOService();
   if (!ios)
-    return PR_FALSE;
+    return false;
   
   nsCOMPtr<nsIProtocolHandler> handler;
   ios->GetProtocolHandler(scheme.get(), getter_AddRefs(handler));
   if (!handler) {
-    return PR_FALSE;
+    return false;
   }
   
   nsCOMPtr<nsIExternalProtocolHandler> extHandler =
@@ -1556,7 +1573,7 @@ nsObjectLoadingContent::CanHandleURI(nsIURI* aURI)
   return extHandler == nsnull;
 }
 
-PRBool
+bool
 nsObjectLoadingContent::IsSupportedDocument(const nsCString& aMimeType)
 {
   nsCOMPtr<nsIContent> thisContent = 
@@ -1584,7 +1601,7 @@ nsObjectLoadingContent::IsSupportedDocument(const nsCString& aMimeType)
       // return an error, and we'll fallback.
       nsCOMPtr<nsIStreamConverterService> convServ =
         do_GetService("@mozilla.org/streamConverters;1");
-      PRBool canConvert = PR_FALSE;
+      bool canConvert = false;
       if (convServ) {
         rv = convServ->CanConvert(aMimeType.get(), "*/*", &canConvert);
       }
@@ -1596,28 +1613,28 @@ nsObjectLoadingContent::IsSupportedDocument(const nsCString& aMimeType)
     return supported != nsIWebNavigationInfo::PLUGIN;
   }
 
-  return PR_FALSE;
+  return false;
 }
 
 void
 nsObjectLoadingContent::UnloadContent()
 {
   // Don't notify in CancelImageRequests. We do it ourselves.
-  CancelImageRequests(PR_FALSE);
+  CancelImageRequests(false);
   if (mFrameLoader) {
     mFrameLoader->Destroy();
     mFrameLoader = nsnull;
   }
   mType = eType_Null;
-  mUserDisabled = mSuppressed = PR_FALSE;
+  mUserDisabled = mSuppressed = false;
   mFallbackReason = ePluginOtherState;
 }
 
 void
 nsObjectLoadingContent::NotifyStateChanged(ObjectType aOldType,
                                            nsEventStates aOldState,
-                                           PRBool aSync,
-                                           PRBool aNotify)
+                                           bool aSync,
+                                           bool aNotify)
 {
   LOG(("OBJLC [%p]: Notifying about state change: (%u, %llx) -> (%u, %llx) (sync=%i)\n",
        this, aOldType, aOldState.GetInternalValue(), mType,
@@ -1692,14 +1709,14 @@ nsObjectLoadingContent::GetTypeOfContent(const nsCString& aMIMEType)
     return eType_Image;
   }
 
-  PRBool isSVG = aMIMEType.LowerCaseEqualsLiteral("image/svg+xml");
-  PRBool supportedSVG = isSVG && (caps & eSupportSVG);
+  bool isSVG = aMIMEType.LowerCaseEqualsLiteral("image/svg+xml");
+  bool supportedSVG = isSVG && (caps & eSupportSVG);
   if (((caps & eSupportDocuments) || supportedSVG) &&
       IsSupportedDocument(aMIMEType)) {
     return eType_Document;
   }
 
-  if ((caps & eSupportPlugins) && IsSupportedPlugin(aMIMEType)) {
+  if ((caps & eSupportPlugins) && IsSupportedPlugin(aMIMEType, mShouldPlay)) {
     return eType_Plugin;
   }
 
@@ -1754,7 +1771,7 @@ nsObjectLoadingContent::GetObjectBaseURI(nsIContent* thisContent, nsIURI** aURI)
                        codebase);
   if (!codebase.IsEmpty()) {
     nsContentUtils::NewURIWithDocumentCharset(aURI, codebase,
-                                              thisContent->GetOwnerDoc(),
+                                              thisContent->OwnerDoc(),
                                               baseURI);
   } else {
     baseURI.swap(*aURI);
@@ -1804,9 +1821,9 @@ nsObjectLoadingContent::HandleBeingBlockedByContentPolicy(nsresult aStatus,
   UnloadContent();
   if (NS_SUCCEEDED(aStatus)) {
     if (aRetval == nsIContentPolicy::REJECT_TYPE) {
-      mUserDisabled = PR_TRUE;
+      mUserDisabled = true;
     } else if (aRetval == nsIContentPolicy::REJECT_SERVER) {
-      mSuppressed = PR_TRUE;
+      mSuppressed = true;
     }
   }
 }
@@ -1855,12 +1872,12 @@ nsObjectLoadingContent::Instantiate(nsIObjectFrame* aFrame,
 
   // Mark that we're instantiating now so that we don't end up
   // re-entering instantiation code.
-  PRBool oldInstantiatingValue = mInstantiating;
-  mInstantiating = PR_TRUE;
+  bool oldInstantiatingValue = mInstantiating;
+  mInstantiating = true;
 
   nsCString typeToUse(aMIMEType);
   if (typeToUse.IsEmpty() && aURI) {
-    IsPluginEnabledByExtension(aURI, typeToUse);
+    IsPluginEnabledByExtension(aURI, typeToUse, mShouldPlay);
   }
 
   nsCOMPtr<nsIContent> thisContent = 
@@ -1923,7 +1940,7 @@ nsObjectLoadingContent::GetPluginSupportState(nsIContent* aContent,
     return GetPluginDisabledState(aContentType);
   }
 
-  PRBool hasAlternateContent = PR_FALSE;
+  bool hasAlternateContent = false;
 
   // Search for a child <param> with a pluginurl name
   for (nsIContent* child = aContent->GetFirstChild();
@@ -1936,7 +1953,7 @@ nsObjectLoadingContent::GetPluginSupportState(nsIContent* aContent,
       }
     } else if (!hasAlternateContent) {
       hasAlternateContent =
-        nsStyleUtil::IsSignificantChild(child, PR_TRUE, PR_FALSE);
+        nsStyleUtil::IsSignificantChild(child, true, false);
     }
   }
 
@@ -1956,6 +1973,8 @@ nsObjectLoadingContent::GetPluginDisabledState(const nsCString& aContentType)
   nsresult rv = pluginHost->IsPluginEnabledForType(aContentType.get());
   if (rv == NS_ERROR_PLUGIN_DISABLED)
     return ePluginDisabled;
+  if (rv == NS_ERROR_PLUGIN_CLICKTOPLAY)
+    return ePluginClickToPlay;
   if (rv == NS_ERROR_PLUGIN_BLOCKLISTED)
     return ePluginBlocklisted;
   return ePluginUnsupported;
@@ -1979,8 +1998,8 @@ nsObjectLoadingContent::CreateStaticClone(nsObjectLoadingContent* aDest) const
 
   if (mFrameLoader) {
     nsCOMPtr<nsIContent> content =
-      do_QueryInterface(static_cast<nsIImageLoadingContent*>((aDest)));
-    nsFrameLoader* fl = nsFrameLoader::Create(content, PR_FALSE);
+      do_QueryInterface(static_cast<nsIImageLoadingContent*>(aDest));
+    nsFrameLoader* fl = nsFrameLoader::Create(content->AsElement(), false);
     if (fl) {
       aDest->mFrameLoader = fl;
       mFrameLoader->CreateStaticClone(fl);
@@ -1999,9 +2018,9 @@ NS_IMETHODIMP
 nsObjectLoadingContent::PluginCrashed(nsIPluginTag* aPluginTag,
                                       const nsAString& pluginDumpID,
                                       const nsAString& browserDumpID,
-                                      PRBool submittedCrashReport)
+                                      bool submittedCrashReport)
 {
-  AutoNotifier notifier(this, PR_TRUE);
+  AutoNotifier notifier(this, true);
   UnloadContent();
   mFallbackReason = ePluginCrashed;
   nsCOMPtr<nsIContent> thisContent = do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
@@ -2025,3 +2044,13 @@ nsObjectLoadingContent::PluginCrashed(nsIPluginTag* aPluginTag,
   }
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsObjectLoadingContent::PlayPlugin()
+{
+  if (!nsContentUtils::IsCallerChrome())
+    return NS_OK;
+
+  mShouldPlay = true;
+  return LoadObject(mURI, true, mContentType, true);
+}
