@@ -54,16 +54,20 @@ using namespace mozilla;
 /*
  * Pull all the data out of the program that will be used by validate later on
  */
-PRBool
+bool
 WebGLProgram::UpdateInfo(gl::GLContext *gl)
 {
-    gl->fGetProgramiv(mName, LOCAL_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, &mAttribMaxNameLength);
-    gl->fGetProgramiv(mName, LOCAL_GL_ACTIVE_UNIFORM_MAX_LENGTH, &mUniformMaxNameLength);
-    gl->fGetProgramiv(mName, LOCAL_GL_ACTIVE_UNIFORMS, &mUniformCount);
-    gl->fGetProgramiv(mName, LOCAL_GL_ACTIVE_ATTRIBUTES, &mAttribCount);
+    gl->fGetProgramiv(mGLName, LOCAL_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, &mAttribMaxNameLength);
+    gl->fGetProgramiv(mGLName, LOCAL_GL_ACTIVE_UNIFORM_MAX_LENGTH, &mUniformMaxNameLength);
+    gl->fGetProgramiv(mGLName, LOCAL_GL_ACTIVE_UNIFORMS, &mUniformCount);
+    gl->fGetProgramiv(mGLName, LOCAL_GL_ACTIVE_ATTRIBUTES, &mAttribCount);
 
     GLint numVertexAttribs;
-    gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_ATTRIBS, &numVertexAttribs);
+    if (mContext->MinCapabilityMode())  {
+        numVertexAttribs = MINVALUE_GL_MAX_VERTEX_ATTRIBS;
+    } else {
+        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_ATTRIBS, &numVertexAttribs);
+    }
     mAttribsInUse.clear();
     mAttribsInUse.resize(numVertexAttribs);
 
@@ -73,14 +77,14 @@ WebGLProgram::UpdateInfo(gl::GLContext *gl)
         GLint attrnamelen;
         GLint attrsize;
         GLenum attrtype;
-        gl->fGetActiveAttrib(mName, i, mAttribMaxNameLength, &attrnamelen, &attrsize, &attrtype, nameBuf);
+        gl->fGetActiveAttrib(mGLName, i, mAttribMaxNameLength, &attrnamelen, &attrsize, &attrtype, nameBuf);
         if (attrnamelen > 0) {
-            GLint loc = gl->fGetAttribLocation(mName, nameBuf);
+            GLint loc = gl->fGetAttribLocation(mGLName, nameBuf);
             mAttribsInUse[loc] = true;
         }
     }
 
-    return PR_TRUE;
+    return true;
 }
 
 /*
@@ -88,7 +92,7 @@ WebGLProgram::UpdateInfo(gl::GLContext *gl)
  * that will be legal to be read from bound VBOs.
  */
 
-PRBool
+bool
 WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
 {
 #ifdef DEBUG
@@ -98,7 +102,7 @@ WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
     NS_ASSERTION(GLuint(currentProgram) == mCurrentProgram->GLName(),
                  "WebGL: current program doesn't agree with GL state");
     if (GLuint(currentProgram) != mCurrentProgram->GLName())
-        return PR_FALSE;
+        return false;
 #endif
 
     *maxAllowedCount = -1;
@@ -114,7 +118,7 @@ WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
 
         if (vd.buf == nsnull) {
             ErrorInvalidOperation("%s: no VBO bound to enabled vertex attrib index %d!", info, i);
-            return PR_FALSE;
+            return false;
         }
 
         // If the attrib is not in use, then we don't have to validate
@@ -132,7 +136,7 @@ WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
             !checked_sizeOfLastElement.valid())
         {
           ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
-          return PR_FALSE;
+          return false;
         }
 
         if (checked_byteLength.value() < checked_sizeOfLastElement.value()) {
@@ -143,7 +147,7 @@ WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
 
           if (!checked_maxAllowedCount.valid()) {
             ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
-            return PR_FALSE;
+            return false;
           }
 
           if (*maxAllowedCount == -1 || *maxAllowedCount > checked_maxAllowedCount.value())
@@ -151,10 +155,10 @@ WebGLContext::ValidateBuffers(PRInt32 *maxAllowedCount, const char *info)
         }
     }
 
-    return PR_TRUE;
+    return true;
 }
 
-PRBool WebGLContext::ValidateCapabilityEnum(WebGLenum cap, const char *info)
+bool WebGLContext::ValidateCapabilityEnum(WebGLenum cap, const char *info)
 {
     switch (cap) {
         case LOCAL_GL_BLEND:
@@ -166,27 +170,27 @@ PRBool WebGLContext::ValidateCapabilityEnum(WebGLenum cap, const char *info)
         case LOCAL_GL_SAMPLE_COVERAGE:
         case LOCAL_GL_SCISSOR_TEST:
         case LOCAL_GL_STENCIL_TEST:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, cap);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateBlendEquationEnum(WebGLenum mode, const char *info)
+bool WebGLContext::ValidateBlendEquationEnum(WebGLenum mode, const char *info)
 {
     switch (mode) {
         case LOCAL_GL_FUNC_ADD:
         case LOCAL_GL_FUNC_SUBTRACT:
         case LOCAL_GL_FUNC_REVERSE_SUBTRACT:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, mode);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateBlendFuncDstEnum(WebGLenum factor, const char *info)
+bool WebGLContext::ValidateBlendFuncDstEnum(WebGLenum factor, const char *info)
 {
     switch (factor) {
         case LOCAL_GL_ZERO:
@@ -203,53 +207,53 @@ PRBool WebGLContext::ValidateBlendFuncDstEnum(WebGLenum factor, const char *info
         case LOCAL_GL_ONE_MINUS_CONSTANT_COLOR:
         case LOCAL_GL_CONSTANT_ALPHA:
         case LOCAL_GL_ONE_MINUS_CONSTANT_ALPHA:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, factor);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateBlendFuncSrcEnum(WebGLenum factor, const char *info)
+bool WebGLContext::ValidateBlendFuncSrcEnum(WebGLenum factor, const char *info)
 {
     if (factor == LOCAL_GL_SRC_ALPHA_SATURATE)
-        return PR_TRUE;
+        return true;
     else
         return ValidateBlendFuncDstEnum(factor, info);
 }
 
-PRBool WebGLContext::ValidateBlendFuncEnumsCompatibility(WebGLenum sfactor, WebGLenum dfactor, const char *info)
+bool WebGLContext::ValidateBlendFuncEnumsCompatibility(WebGLenum sfactor, WebGLenum dfactor, const char *info)
 {
-    PRBool sfactorIsConstantColor = sfactor == LOCAL_GL_CONSTANT_COLOR ||
+    bool sfactorIsConstantColor = sfactor == LOCAL_GL_CONSTANT_COLOR ||
                                     sfactor == LOCAL_GL_ONE_MINUS_CONSTANT_COLOR;
-    PRBool sfactorIsConstantAlpha = sfactor == LOCAL_GL_CONSTANT_ALPHA ||
+    bool sfactorIsConstantAlpha = sfactor == LOCAL_GL_CONSTANT_ALPHA ||
                                     sfactor == LOCAL_GL_ONE_MINUS_CONSTANT_ALPHA;
-    PRBool dfactorIsConstantColor = dfactor == LOCAL_GL_CONSTANT_COLOR ||
+    bool dfactorIsConstantColor = dfactor == LOCAL_GL_CONSTANT_COLOR ||
                                     dfactor == LOCAL_GL_ONE_MINUS_CONSTANT_COLOR;
-    PRBool dfactorIsConstantAlpha = dfactor == LOCAL_GL_CONSTANT_ALPHA ||
+    bool dfactorIsConstantAlpha = dfactor == LOCAL_GL_CONSTANT_ALPHA ||
                                     dfactor == LOCAL_GL_ONE_MINUS_CONSTANT_ALPHA;
     if ( (sfactorIsConstantColor && dfactorIsConstantAlpha) ||
          (dfactorIsConstantColor && sfactorIsConstantAlpha) ) {
         ErrorInvalidOperation("%s are mutually incompatible, see section 6.8 in the WebGL 1.0 spec", info);
-        return PR_FALSE;
+        return false;
     } else {
-        return PR_TRUE;
+        return true;
     }
 }
 
-PRBool WebGLContext::ValidateTextureTargetEnum(WebGLenum target, const char *info)
+bool WebGLContext::ValidateTextureTargetEnum(WebGLenum target, const char *info)
 {
     switch (target) {
         case LOCAL_GL_TEXTURE_2D:
         case LOCAL_GL_TEXTURE_CUBE_MAP:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, target);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateComparisonEnum(WebGLenum target, const char *info)
+bool WebGLContext::ValidateComparisonEnum(WebGLenum target, const char *info)
 {
     switch (target) {
         case LOCAL_GL_NEVER:
@@ -260,14 +264,14 @@ PRBool WebGLContext::ValidateComparisonEnum(WebGLenum target, const char *info)
         case LOCAL_GL_EQUAL:
         case LOCAL_GL_NOTEQUAL:
         case LOCAL_GL_ALWAYS:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, target);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateStencilOpEnum(WebGLenum action, const char *info)
+bool WebGLContext::ValidateStencilOpEnum(WebGLenum action, const char *info)
 {
     switch (action) {
         case LOCAL_GL_KEEP:
@@ -278,40 +282,40 @@ PRBool WebGLContext::ValidateStencilOpEnum(WebGLenum action, const char *info)
         case LOCAL_GL_DECR:
         case LOCAL_GL_DECR_WRAP:
         case LOCAL_GL_INVERT:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, action);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateFaceEnum(WebGLenum face, const char *info)
+bool WebGLContext::ValidateFaceEnum(WebGLenum face, const char *info)
 {
     switch (face) {
         case LOCAL_GL_FRONT:
         case LOCAL_GL_BACK:
         case LOCAL_GL_FRONT_AND_BACK:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, face);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateBufferUsageEnum(WebGLenum target, const char *info)
+bool WebGLContext::ValidateBufferUsageEnum(WebGLenum target, const char *info)
 {
     switch (target) {
         case LOCAL_GL_STREAM_DRAW:
         case LOCAL_GL_STATIC_DRAW:
         case LOCAL_GL_DYNAMIC_DRAW:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, target);
-            return PR_FALSE;
+            return false;
     }
 }
 
-PRBool WebGLContext::ValidateDrawModeEnum(WebGLenum mode, const char *info)
+bool WebGLContext::ValidateDrawModeEnum(WebGLenum mode, const char *info)
 {
     switch (mode) {
         case LOCAL_GL_TRIANGLES:
@@ -321,10 +325,10 @@ PRBool WebGLContext::ValidateDrawModeEnum(WebGLenum mode, const char *info)
         case LOCAL_GL_LINE_STRIP:
         case LOCAL_GL_LINE_LOOP:
         case LOCAL_GL_LINES:
-            return PR_TRUE;
+            return true;
         default:
             ErrorInvalidEnumInfo(info, mode);
-            return PR_FALSE;
+            return false;
     }
 }
 
@@ -384,7 +388,7 @@ PRUint32 WebGLContext::GetTexelSize(WebGLenum format, WebGLenum type)
     return 0;
 }
 
-PRBool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type, int jsArrayType,
+bool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type, int jsArrayType,
                                               PRUint32 *texelSize, const char *info)
 {
     if (type == LOCAL_GL_UNSIGNED_BYTE ||
@@ -395,7 +399,7 @@ PRBool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type,
                 (type == LOCAL_GL_FLOAT && jsArrayType != js::TypedArray::TYPE_FLOAT32))
             {
                 ErrorInvalidOperation("%s: invalid typed array type for given format", info);
-                return PR_FALSE;
+                return false;
             }
         }
 
@@ -404,22 +408,22 @@ PRBool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type,
             case LOCAL_GL_ALPHA:
             case LOCAL_GL_LUMINANCE:
                 *texelSize = 1 * texMultiplier;
-                return PR_TRUE;
+                return true;
             case LOCAL_GL_LUMINANCE_ALPHA:
                 *texelSize = 2 * texMultiplier;
-                return PR_TRUE;
+                return true;
             case LOCAL_GL_RGB:
                 *texelSize = 3 * texMultiplier;
-                return PR_TRUE;
+                return true;
             case LOCAL_GL_RGBA:
                 *texelSize = 4 * texMultiplier;
-                return PR_TRUE;
+                return true;
             default:
                 break;
         }
 
         ErrorInvalidEnum("%s: invalid format 0x%x", info, format);
-        return PR_FALSE;
+        return false;
     }
 
     switch (type) {
@@ -427,38 +431,38 @@ PRBool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type,
         case LOCAL_GL_UNSIGNED_SHORT_5_5_5_1:
             if (jsArrayType != -1 && jsArrayType != js::TypedArray::TYPE_UINT16) {
                 ErrorInvalidOperation("%s: invalid typed array type for given format", info);
-                return PR_FALSE;
+                return false;
             }
 
             if (format == LOCAL_GL_RGBA) {
                 *texelSize = 2;
-                return PR_TRUE;
+                return true;
             }
             ErrorInvalidOperation("%s: mutually incompatible format and type", info);
-            return PR_FALSE;
+            return false;
 
         case LOCAL_GL_UNSIGNED_SHORT_5_6_5:
             if (jsArrayType != -1 && jsArrayType != js::TypedArray::TYPE_UINT16) {
                 ErrorInvalidOperation("%s: invalid typed array type for given format", info);
-                return PR_FALSE;
+                return false;
             }
 
             if (format == LOCAL_GL_RGB) {
                 *texelSize = 2;
-                return PR_TRUE;
+                return true;
             }
             ErrorInvalidOperation("%s: mutually incompatible format and type", info);
-            return PR_FALSE;
+            return false;
 
         default:
             break;
         }
 
     ErrorInvalidEnum("%s: invalid type 0x%x", info, type);
-    return PR_FALSE;
+    return false;
 }
 
-PRBool WebGLContext::ValidateAttribIndex(WebGLuint index, const char *info)
+bool WebGLContext::ValidateAttribIndex(WebGLuint index, const char *info)
 {
     if (index >= mAttribBuffers.Length()) {
         if (index == WebGLuint(-1)) {
@@ -468,47 +472,49 @@ PRBool WebGLContext::ValidateAttribIndex(WebGLuint index, const char *info)
         } else {
              ErrorInvalidValue("%s: index %d is out of range", info, index);
         }
-        return PR_FALSE;
+        return false;
     } else {
-        return PR_TRUE;
+        return true;
     }
 }
 
-PRBool WebGLContext::ValidateStencilParamsForDrawCall()
+bool WebGLContext::ValidateStencilParamsForDrawCall()
 {
   const char *msg = "%s set different front and back stencil %s. Drawing in this configuration is not allowed.";
   if (mStencilRefFront != mStencilRefBack) {
       ErrorInvalidOperation(msg, "stencilFuncSeparate", "reference values");
-      return PR_FALSE;
+      return false;
   }
   if (mStencilValueMaskFront != mStencilValueMaskBack) {
       ErrorInvalidOperation(msg, "stencilFuncSeparate", "value masks");
-      return PR_FALSE;
+      return false;
   }
   if (mStencilWriteMaskFront != mStencilWriteMaskBack) {
       ErrorInvalidOperation(msg, "stencilMaskSeparate", "write masks");
-      return PR_FALSE;
+      return false;
   }
-  return PR_TRUE;
+  return true;
 }
 
-PRBool
+bool
 WebGLContext::InitAndValidateGL()
 {
-    if (!gl) return PR_FALSE;
+    if (!gl) return false;
 
     GLenum error = gl->fGetError();
     if (error != LOCAL_GL_NO_ERROR) {
         LogMessage("GL error 0x%x occurred during OpenGL context initialization, before WebGL initialization!", error);
-        return PR_FALSE;
+        return false;
     }
 
+    mMinCapability = Preferences::GetBool("webgl.min_capability_mode", false);
+    mDisableExtensions = Preferences::GetBool("webgl.disable-extensions", false);
+
     mActiveTexture = 0;
     mWebGLError = LOCAL_GL_NO_ERROR;
 
     mAttribBuffers.Clear();
 
-    mUniformTextures.Clear();
     mBound2DTextures.Clear();
     mBoundCubeMapTextures.Clear();
 
@@ -519,13 +525,6 @@ WebGLContext::InitAndValidateGL()
     mBoundFramebuffer = nsnull;
     mBoundRenderbuffer = nsnull;
 
-    mMapTextures.Clear();
-    mMapBuffers.Clear();
-    mMapPrograms.Clear();
-    mMapShaders.Clear();
-    mMapFramebuffers.Clear();
-    mMapRenderbuffers.Clear();
-
     MakeContextCurrent();
 
     // on desktop OpenGL, we always keep vertex attrib 0 array enabled
@@ -533,10 +532,14 @@ WebGLContext::InitAndValidateGL()
         gl->fEnableVertexAttribArray(0);
     }
 
-    gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_ATTRIBS, &mGLMaxVertexAttribs);
+    if (MinCapabilityMode()) {
+        mGLMaxVertexAttribs = MINVALUE_GL_MAX_VERTEX_ATTRIBS;
+    } else {
+        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_ATTRIBS, &mGLMaxVertexAttribs);
+    }
     if (mGLMaxVertexAttribs < 8) {
         LogMessage("GL_MAX_VERTEX_ATTRIBS: %d is < 8!", mGLMaxVertexAttribs);
-        return PR_FALSE;
+        return false;
     }
 
     mAttribBuffers.SetLength(mGLMaxVertexAttribs);
@@ -544,17 +547,30 @@ WebGLContext::InitAndValidateGL()
     // Note: GL_MAX_TEXTURE_UNITS is fixed at 4 for most desktop hardware,
     // even though the hardware supports much more.  The
     // GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS value is the accurate value.
-    gl->fGetIntegerv(LOCAL_GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &mGLMaxTextureUnits);
+    if (MinCapabilityMode()) {
+        mGLMaxTextureUnits = MINVALUE_GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS;
+    } else {
+        gl->fGetIntegerv(LOCAL_GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &mGLMaxTextureUnits);
+    }
     if (mGLMaxTextureUnits < 8) {
         LogMessage("GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS: %d is < 8!", mGLMaxTextureUnits);
-        return PR_FALSE;
+        return false;
     }
 
     mBound2DTextures.SetLength(mGLMaxTextureUnits);
     mBoundCubeMapTextures.SetLength(mGLMaxTextureUnits);
 
-    gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_SIZE, &mGLMaxTextureSize);
-    gl->fGetIntegerv(LOCAL_GL_MAX_CUBE_MAP_TEXTURE_SIZE, &mGLMaxCubeMapTextureSize);
+    if (MinCapabilityMode()) {
+        mGLMaxTextureSize = MINVALUE_GL_MAX_TEXTURE_SIZE;
+        mGLMaxCubeMapTextureSize = MINVALUE_GL_MAX_CUBE_MAP_TEXTURE_SIZE;
+        mGLMaxTextureImageUnits = MINVALUE_GL_MAX_TEXTURE_IMAGE_UNITS;
+        mGLMaxVertexTextureImageUnits = MINVALUE_GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS;
+    } else {
+        gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_SIZE, &mGLMaxTextureSize);
+        gl->fGetIntegerv(LOCAL_GL_MAX_CUBE_MAP_TEXTURE_SIZE, &mGLMaxCubeMapTextureSize);
+        gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_IMAGE_UNITS, &mGLMaxTextureImageUnits);
+        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS, &mGLMaxVertexTextureImageUnits);
+    }
     
 #ifdef XP_MACOSX
     if (gl->Vendor() == gl::GLContext::VendorIntel) {
@@ -564,48 +580,51 @@ WebGLContext::InitAndValidateGL()
     }
 #endif
 
-    gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_IMAGE_UNITS, &mGLMaxTextureImageUnits);
-    gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS, &mGLMaxVertexTextureImageUnits);
-
-    if (gl->HasES2Compatibility()) {
-        gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_UNIFORM_VECTORS, &mGLMaxFragmentUniformVectors);
-        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_UNIFORM_VECTORS, &mGLMaxVertexUniformVectors);
-        gl->fGetIntegerv(LOCAL_GL_MAX_VARYING_VECTORS, &mGLMaxVaryingVectors);
+    if (MinCapabilityMode()) {
+        mGLMaxFragmentUniformVectors = MINVALUE_GL_MAX_FRAGMENT_UNIFORM_VECTORS;
+        mGLMaxVertexUniformVectors = MINVALUE_GL_MAX_VERTEX_UNIFORM_VECTORS;
+        mGLMaxVaryingVectors = MINVALUE_GL_MAX_VARYING_VECTORS;
     } else {
-        gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_UNIFORM_COMPONENTS, &mGLMaxFragmentUniformVectors);
-        mGLMaxFragmentUniformVectors /= 4;
-        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_UNIFORM_COMPONENTS, &mGLMaxVertexUniformVectors);
-        mGLMaxVertexUniformVectors /= 4;
+        if (gl->HasES2Compatibility()) {
+            gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_UNIFORM_VECTORS, &mGLMaxFragmentUniformVectors);
+            gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_UNIFORM_VECTORS, &mGLMaxVertexUniformVectors);
+            gl->fGetIntegerv(LOCAL_GL_MAX_VARYING_VECTORS, &mGLMaxVaryingVectors);
+        } else {
+            gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_UNIFORM_COMPONENTS, &mGLMaxFragmentUniformVectors);
+            mGLMaxFragmentUniformVectors /= 4;
+            gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_UNIFORM_COMPONENTS, &mGLMaxVertexUniformVectors);
+            mGLMaxVertexUniformVectors /= 4;
 
-        // we are now going to try to read GL_MAX_VERTEX_OUTPUT_COMPONENTS and GL_MAX_FRAGMENT_INPUT_COMPONENTS,
-        // however these constants only entered the OpenGL standard at OpenGL 3.2. So we will try reading,
-        // and check OpenGL error for INVALID_ENUM.
+            // we are now going to try to read GL_MAX_VERTEX_OUTPUT_COMPONENTS and GL_MAX_FRAGMENT_INPUT_COMPONENTS,
+            // however these constants only entered the OpenGL standard at OpenGL 3.2. So we will try reading,
+            // and check OpenGL error for INVALID_ENUM.
 
-        // before we start, we check that no error already occurred, to prevent hiding it in our subsequent error handling
-        error = gl->GetAndClearError();
-        if (error != LOCAL_GL_NO_ERROR) {
-            LogMessage("GL error 0x%x occurred during WebGL context initialization!", error);
-            return PR_FALSE;
-        }
-
-        // On the public_webgl list, "problematic GetParameter pnames" thread, the following formula was given:
-        //   mGLMaxVaryingVectors = min (GL_MAX_VERTEX_OUTPUT_COMPONENTS, GL_MAX_FRAGMENT_INPUT_COMPONENTS) / 4
-        GLint maxVertexOutputComponents,
-              minFragmentInputComponents;
-        gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_OUTPUT_COMPONENTS, &maxVertexOutputComponents);
-        gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_INPUT_COMPONENTS, &minFragmentInputComponents);
-
-        error = gl->fGetError();
-        switch (error) {
-            case LOCAL_GL_NO_ERROR:
-                mGLMaxVaryingVectors = NS_MIN(maxVertexOutputComponents, minFragmentInputComponents) / 4;
-                break;
-            case LOCAL_GL_INVALID_ENUM:
-                mGLMaxVaryingVectors = 16; // = 64/4, 64 is the min value for maxVertexOutputComponents in OpenGL 3.2 spec
-                break;
-            default:
+            // before we start, we check that no error already occurred, to prevent hiding it in our subsequent error handling
+            error = gl->GetAndClearError();
+            if (error != LOCAL_GL_NO_ERROR) {
                 LogMessage("GL error 0x%x occurred during WebGL context initialization!", error);
-                return PR_FALSE;
+                return false;
+            }
+
+            // On the public_webgl list, "problematic GetParameter pnames" thread, the following formula was given:
+            //   mGLMaxVaryingVectors = min (GL_MAX_VERTEX_OUTPUT_COMPONENTS, GL_MAX_FRAGMENT_INPUT_COMPONENTS) / 4
+            GLint maxVertexOutputComponents,
+                  minFragmentInputComponents;
+            gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_OUTPUT_COMPONENTS, &maxVertexOutputComponents);
+            gl->fGetIntegerv(LOCAL_GL_MAX_FRAGMENT_INPUT_COMPONENTS, &minFragmentInputComponents);
+
+            error = gl->GetAndClearError();
+            switch (error) {
+                case LOCAL_GL_NO_ERROR:
+                    mGLMaxVaryingVectors = NS_MIN(maxVertexOutputComponents, minFragmentInputComponents) / 4;
+                    break;
+                case LOCAL_GL_INVALID_ENUM:
+                    mGLMaxVaryingVectors = 16; // = 64/4, 64 is the min value for maxVertexOutputComponents in OpenGL 3.2 spec
+                    break;
+                default:
+                    LogMessage("GL error 0x%x occurred during WebGL context initialization!", error);
+                    return false;
+            }   
         }
     }
 
@@ -635,7 +654,7 @@ WebGLContext::InitAndValidateGL()
     }
 
     // Check the shader validator pref
-    NS_ENSURE_TRUE(Preferences::GetRootBranch(), PR_FALSE);
+    NS_ENSURE_TRUE(Preferences::GetRootBranch(), false);
 
     mShaderValidation =
         Preferences::GetBool("webgl.shader_validator", mShaderValidation);
@@ -645,7 +664,7 @@ WebGLContext::InitAndValidateGL()
     if (mShaderValidation) {
         if (!ShInitialize()) {
             LogMessage("GLSL translator initialization failed!");
-            return PR_FALSE;
+            return false;
         }
     }
 #endif
@@ -655,8 +674,8 @@ WebGLContext::InitAndValidateGL()
     error = gl->GetAndClearError();
     if (error != LOCAL_GL_NO_ERROR) {
         LogMessage("GL error 0x%x occurred during WebGL context initialization!", error);
-        return PR_FALSE;
+        return false;
     }
 
-    return PR_TRUE;
+    return true;
 }
