@@ -36,6 +36,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Util.h"
+
 #include "nsFormSubmission.h"
 
 #include "nsCOMPtr.h"
@@ -69,19 +71,20 @@
 #include "nsIStringBundle.h"
 #include "nsCExternalHandlerService.h"
 #include "nsIFileStreams.h"
+#include "nsContentUtils.h"
+
+using namespace mozilla;
 
 static void
 SendJSWarning(nsIDocument* aDocument,
               const char* aWarningName,
               const PRUnichar** aWarningArgs, PRUint32 aWarningArgsLen)
 {
-  nsContentUtils::ReportToConsole(nsContentUtils::eFORMS_PROPERTIES,
+  nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                  "HTML", aDocument,
+                                  nsContentUtils::eFORMS_PROPERTIES,
                                   aWarningName,
-                                  aWarningArgs, aWarningArgsLen,
-                                  nsnull,
-                                  EmptyString(), 0, 0,
-                                  nsIScriptError::warningFlag,
-                                  "HTML", aDocument);
+                                  aWarningArgs, aWarningArgsLen);
 }
 
 // --------------------------------------------------------------------------
@@ -101,7 +104,7 @@ public:
     : nsEncodingFormSubmission(aCharset, aOriginatingElement),
       mMethod(aMethod),
       mDocument(aDocument),
-      mWarnedFileControl(PR_FALSE)
+      mWarnedFileControl(false)
   {
   }
 
@@ -112,9 +115,9 @@ public:
   virtual nsresult GetEncodedSubmission(nsIURI* aURI,
                                         nsIInputStream** aPostDataStream);
 
-  virtual PRBool SupportsIsindexSubmission()
+  virtual bool SupportsIsindexSubmission()
   {
-    return PR_TRUE;
+    return true;
   }
 
   virtual nsresult AddIsindex(const nsAString& aValue);
@@ -145,7 +148,7 @@ private:
   nsCOMPtr<nsIDocument> mDocument;
 
   /** Whether or not we have warned about a file control not being submitted */
-  PRBool mWarnedFileControl;
+  bool mWarnedFileControl;
 };
 
 nsresult
@@ -198,7 +201,7 @@ nsFSURLEncoded::AddNameFilePair(const nsAString& aName,
 {
   if (!mWarnedFileControl) {
     SendJSWarning(mDocument, "ForgotFileEnctypeWarning", nsnull, 0);
-    mWarnedFileControl = PR_TRUE;
+    mWarnedFileControl = true;
   }
 
   nsAutoString filename;
@@ -214,11 +217,11 @@ static void
 HandleMailtoSubject(nsCString& aPath) {
 
   // Walk through the string and see if we have a subject already.
-  PRBool hasSubject = PR_FALSE;
-  PRBool hasParams = PR_FALSE;
+  bool hasSubject = false;
+  bool hasParams = false;
   PRInt32 paramSep = aPath.FindChar('?');
   while (paramSep != kNotFound && paramSep < (PRInt32)aPath.Length()) {
-    hasParams = PR_TRUE;
+    hasParams = true;
 
     // Get the end of the name at the = op.  If it is *after* the next &,
     // assume that someone made a parameter without an = in it
@@ -237,7 +240,7 @@ HandleMailtoSubject(nsCString& aPath) {
     if (nameEnd != kNotFound) {
       if (Substring(aPath, paramSep+1, nameEnd-(paramSep+1)).
           LowerCaseEqualsLiteral("subject")) {
-        hasSubject = PR_TRUE;
+        hasSubject = true;
         break;
       }
     }
@@ -266,7 +269,7 @@ HandleMailtoSubject(nsCString& aPath) {
                                            nsContentUtils::eFORMS_PROPERTIES,
                                            "DefaultFormSubject",
                                            formatStrings,
-                                           NS_ARRAY_LENGTH(formatStrings),
+                                           ArrayLength(formatStrings),
                                            subjectStr);
     if (NS_FAILED(rv))
       return;
@@ -287,7 +290,7 @@ nsFSURLEncoded::GetEncodedSubmission(nsIURI* aURI,
 
   if (mMethod == NS_FORM_METHOD_POST) {
 
-    PRBool isMailto = PR_FALSE;
+    bool isMailto = false;
     aURI->SchemeIs("mailto", &isMailto);
     if (isMailto) {
 
@@ -328,7 +331,7 @@ nsFSURLEncoded::GetEncodedSubmission(nsIURI* aURI,
       mimeStream->AddHeader("Content-Type",
                             "application/x-www-form-urlencoded");
 #endif
-      mimeStream->SetAddContentLength(PR_TRUE);
+      mimeStream->SetAddContentLength(true);
       mimeStream->SetData(dataStream);
 
       *aPostDataStream = mimeStream;
@@ -337,7 +340,7 @@ nsFSURLEncoded::GetEncodedSubmission(nsIURI* aURI,
 
   } else {
     // Get the full query string
-    PRBool schemeIsJavaScript;
+    bool schemeIsJavaScript;
     rv = aURI->SchemeIs("javascript", &schemeIsJavaScript);
     NS_ENSURE_SUCCESS(rv, rv);
     if (schemeIsJavaScript) {
@@ -485,6 +488,11 @@ nsFSMultipartFormData::AddNameFilePair(const nsAString& aName,
       rv = file->GetName(filename16);
       NS_ENSURE_SUCCESS(rv, rv);
     }
+
+    if (filename16.IsEmpty()) {
+      filename16.AssignLiteral("blob");
+    }
+
     rv = EncodeVal(filename16, filename, true);
     NS_ENSURE_SUCCESS(rv, rv);
   
@@ -561,7 +569,7 @@ nsFSMultipartFormData::GetEncodedSubmission(nsIURI* aURI,
   nsCAutoString contentType;
   GetContentType(contentType);
   mimeStream->AddHeader("Content-Type", contentType.get());
-  mimeStream->SetAddContentLength(PR_TRUE);
+  mimeStream->SetAddContentLength(true);
   mimeStream->SetData(GetSubmissionBody());
 
   *aPostDataStream = mimeStream.forget().get();
@@ -644,7 +652,7 @@ nsFSTextPlain::GetEncodedSubmission(nsIURI* aURI,
   // XXX HACK We are using the standard URL mechanism to give the body to the
   // mailer instead of passing the post data stream to it, since that sounds
   // hard.
-  PRBool isMailto = PR_FALSE;
+  bool isMailto = false;
   aURI->SchemeIs("mailto", &isMailto);
   if (isMailto) {
     nsCAutoString path;
@@ -689,7 +697,7 @@ nsFSTextPlain::GetEncodedSubmission(nsIURI* aURI,
     NS_ENSURE_SUCCESS(rv, rv);
 
     mimeStream->AddHeader("Content-Type", "text/plain");
-    mimeStream->SetAddContentLength(PR_TRUE);
+    mimeStream->SetAddContentLength(true);
     mimeStream->SetData(bodyStream);
     CallQueryInterface(mimeStream, aPostDataStream);
   }
@@ -710,13 +718,23 @@ nsEncodingFormSubmission::nsEncodingFormSubmission(const nsACString& aCharset,
     charset.AssignLiteral("windows-1252");
   }
 
-  // use UTF-8 for UTF-16* and UTF-32* (per WHATWG and existing practice of
+  // use UTF-8 for UTF-16* (per WHATWG and existing practice of
   // MS IE/Opera). 
-  if (StringBeginsWith(charset, NS_LITERAL_CSTRING("UTF-16")) || 
-      StringBeginsWith(charset, NS_LITERAL_CSTRING("UTF-32"))) {
+  if (StringBeginsWith(charset, NS_LITERAL_CSTRING("UTF-16"))) {
     charset.AssignLiteral("UTF-8");
   }
 
+  if (!(charset.EqualsLiteral("UTF-8") || charset.EqualsLiteral("gb18030"))) {
+    nsAutoString charsetUtf16;
+    CopyUTF8toUTF16(charset, charsetUtf16);
+    const PRUnichar* charsetPtr = charsetUtf16.get();
+    SendJSWarning(aOriginatingElement ? aOriginatingElement->GetOwnerDocument()
+                                      : nsnull,
+                  "CannotEncodeAllUnicode",
+                  &charsetPtr,
+                  1);
+  }
+
   mEncoder = do_CreateInstance(NS_SAVEASCHARSET_CONTRACTID);
   if (mEncoder) {
     nsresult rv =
@@ -859,7 +877,7 @@ GetSubmissionFromForm(nsGenericHTMLElement* aForm,
              enctype == NS_FORM_ENCTYPE_TEXTPLAIN) {
     *aFormSubmission = new nsFSTextPlain(charset, aOriginatingElement);
   } else {
-    nsIDocument* doc = aForm->GetOwnerDoc();
+    nsIDocument* doc = aForm->OwnerDoc();
     if (enctype == NS_FORM_ENCTYPE_MULTIPART ||
         enctype == NS_FORM_ENCTYPE_TEXTPLAIN) {
       nsAutoString enctypeStr;
