@@ -41,9 +41,7 @@
 #include "nsDocument.h"
 #include "nsIHTMLDocument.h"
 #include "nsIDOMHTMLDocument.h"
-#include "nsIDOMNSHTMLDocument.h"
 #include "nsIDOMHTMLBodyElement.h"
-#include "nsIDOMHTMLMapElement.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIScriptElement.h"
 #include "jsapi.h"
@@ -65,15 +63,16 @@ class nsIEditorDocShell;
 class nsIParser;
 class nsIURI;
 class nsIMarkupDocumentViewer;
-class nsIDocumentCharsetInfo;
-class nsICacheEntryDescriptor;
+class nsIDocShell;
+class nsICachingChannel;
 
 class nsHTMLDocument : public nsDocument,
                        public nsIHTMLDocument,
-                       public nsIDOMHTMLDocument,
-                       public nsIDOMNSHTMLDocument
+                       public nsIDOMHTMLDocument
 {
 public:
+  using nsDocument::SetDocumentURI;
+
   nsHTMLDocument();
   virtual nsresult Init();
 
@@ -85,7 +84,6 @@ public:
   virtual void Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup);
   virtual void ResetToURI(nsIURI* aURI, nsILoadGroup* aLoadGroup,
                           nsIPrincipal* aPrincipal);
-  virtual nsStyleSet::sheetType GetAttrSheetType();
 
   virtual nsresult CreateShell(nsPresContext* aContext,
                                nsIViewManager* aViewManager,
@@ -97,93 +95,66 @@ public:
                                      nsILoadGroup* aLoadGroup,
                                      nsISupports* aContainer,
                                      nsIStreamListener **aDocListener,
-                                     PRBool aReset = PR_TRUE,
+                                     bool aReset = true,
                                      nsIContentSink* aSink = nsnull);
   virtual void StopDocumentLoad();
 
+  virtual void BeginLoad();
+
   virtual void EndLoad();
 
-  virtual nsresult AddImageMap(nsIDOMHTMLMapElement* aMap);
-
-  virtual void RemoveImageMap(nsIDOMHTMLMapElement* aMap);
-
-  virtual nsIDOMHTMLMapElement *GetImageMap(const nsAString& aMapName);
-
   virtual void SetCompatibilityMode(nsCompatibility aMode);
 
-  virtual PRBool IsWriting()
+  virtual bool IsWriting()
   {
     return mWriteLevel != PRUint32(0);
   }
 
-  virtual PRBool GetIsFrameset() { return mIsFrameset; }
-  virtual void SetIsFrameset(PRBool aFrameset) { mIsFrameset = aFrameset; }
+  virtual bool GetIsFrameset() { return mIsFrameset; }
+  virtual void SetIsFrameset(bool aFrameset) { mIsFrameset = aFrameset; }
 
   virtual NS_HIDDEN_(nsContentList*) GetForms();
  
   virtual NS_HIDDEN_(nsContentList*) GetFormControls();
  
-  virtual PRBool IsCaseSensitive();
-
   // nsIDOMDocument interface
-  NS_DECL_NSIDOMDOCUMENT
-
-  // nsIDOM3Document interface
-  NS_IMETHOD GetXmlEncoding(nsAString& aXmlVersion);
-  NS_IMETHOD GetXmlStandalone(PRBool *aXmlStandalone);
-  NS_IMETHOD SetXmlStandalone(PRBool aXmlStandalone);
-  NS_IMETHOD GetXmlVersion(nsAString& aXmlVersion);
-  NS_IMETHOD SetXmlVersion(const nsAString& aXmlVersion);
+  NS_FORWARD_NSIDOMDOCUMENT(nsDocument::)
 
   // nsIDOMNode interface
   NS_FORWARD_NSIDOMNODE(nsDocument::)
 
-  // nsIDOM3Node interface
-  NS_IMETHOD GetBaseURI(nsAString& aBaseURI);
-
   // nsIDOMHTMLDocument interface
-  NS_IMETHOD GetTitle(nsAString & aTitle);
-  NS_IMETHOD SetTitle(const nsAString & aTitle);
-  NS_IMETHOD GetReferrer(nsAString & aReferrer);
-  NS_IMETHOD GetURL(nsAString & aURL);
-  NS_IMETHOD GetBody(nsIDOMHTMLElement * *aBody);
-  NS_IMETHOD SetBody(nsIDOMHTMLElement * aBody);
-  NS_IMETHOD GetImages(nsIDOMHTMLCollection * *aImages);
-  NS_IMETHOD GetApplets(nsIDOMHTMLCollection * *aApplets);
-  NS_IMETHOD GetLinks(nsIDOMHTMLCollection * *aLinks);
-  NS_IMETHOD GetForms(nsIDOMHTMLCollection * *aForms);
-  NS_IMETHOD GetAnchors(nsIDOMHTMLCollection * *aAnchors);
-  NS_IMETHOD GetCookie(nsAString & aCookie);
-  NS_IMETHOD SetCookie(const nsAString & aCookie);
-  NS_IMETHOD Open(void);
-  NS_IMETHOD Close(void);
-  NS_IMETHOD Write(const nsAString & text);
-  NS_IMETHOD Writeln(const nsAString & text);
-  NS_IMETHOD GetElementsByName(const nsAString & elementName,
-                               nsIDOMNodeList **_retval);
-  virtual nsresult GetDocumentAllResult(const nsAString& aID,
-                                        nsISupports** aResult);
+  NS_DECL_NSIDOMHTMLDOCUMENT
+
+  /**
+   * Returns the result of document.all[aID] which can either be a node
+   * or a nodelist depending on if there are multiple nodes with the same
+   * id.
+   */
+  nsISupports *GetDocumentAllResult(const nsAString& aID,
+                                    nsWrapperCache **aCache,
+                                    nsresult *aResult);
+
+  nsIContent *GetBody();
+  Element *GetHead() { return GetHeadElement(); }
+  already_AddRefed<nsContentList> GetElementsByName(const nsAString & aName)
+  {
+    return NS_GetFuncStringContentList(this, MatchNameAttribute, nsnull,
+                                       UseExistingNameString, aName);
+  }
 
-  // nsIDOMNSHTMLDocument interface
-  NS_DECL_NSIDOMNSHTMLDOCUMENT
 
   virtual nsresult ResolveName(const nsAString& aName,
-                               nsIDOMHTMLFormElement *aForm,
-                               nsISupports **aResult);
-
-  virtual void ScriptLoading(nsIScriptElement *aScript);
-  virtual void ScriptExecuted(nsIScriptElement *aScript);
+                               nsIContent *aForm,
+                               nsISupports **aResult,
+                               nsWrapperCache **aCache);
 
   virtual void AddedForm();
   virtual void RemovedForm();
   virtual PRInt32 GetNumFormsSynchronous();
   virtual void TearingDownEditor(nsIEditor *aEditor);
-  virtual void SetIsXHTML(PRBool aXHTML) { mIsRegularHTML = !aXHTML; }
-  PRBool IsXHTML()
-  {
-    return !mIsRegularHTML;
-  }
-  virtual void SetDocWriteDisabled(PRBool aDisabled)
+  virtual void SetIsXHTML(bool aXHTML) { mIsRegularHTML = !aXHTML; }
+  virtual void SetDocWriteDisabled(bool aDisabled)
   {
     mDisableDocWrite = aDisabled;
   }
@@ -198,11 +169,9 @@ public:
 
   virtual void DisableCookieAccess()
   {
-    mDisableCookieAccess = PR_TRUE;
+    mDisableCookieAccess = true;
   }
 
-  virtual nsIContent* GetBodyContentExternal();
-  
   class nsAutoEditingState {
   public:
     nsAutoEditingState(nsHTMLDocument* aDoc, EditingState aState)
@@ -221,14 +190,7 @@ public:
 
   void EndUpdate(nsUpdateType aUpdateType);
 
-  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED_NO_UNLINK(nsHTMLDocument, nsDocument)
-
-  virtual already_AddRefed<nsIParser> GetFragmentParser() {
-    return mFragmentParser.forget();
-  }
-  virtual void SetFragmentParser(nsIParser* aParser) {
-    mFragmentParser = aParser;
-  }
+  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsHTMLDocument, nsDocument)
 
   virtual nsresult SetEditingState(EditingState aState);
 
@@ -236,29 +198,32 @@ public:
 
   virtual NS_HIDDEN_(void) RemovedFromDocShell();
 
+  virtual mozilla::dom::Element *GetElementById(const nsAString& aElementId)
+  {
+    return nsDocument::GetElementById(aElementId);
+  }
+
+  virtual nsXPCClassInfo* GetClassInfo();
 protected:
   nsresult GetBodySize(PRInt32* aWidth,
                        PRInt32* aHeight);
 
-  nsresult PrePopulateIdentifierMap();
-
   nsIContent *MatchId(nsIContent *aContent, const nsAString& aId);
 
-  static PRBool MatchLinks(nsIContent *aContent, PRInt32 aNamespaceID,
+  static bool MatchLinks(nsIContent *aContent, PRInt32 aNamespaceID,
                            nsIAtom* aAtom, void* aData);
-  static PRBool MatchAnchors(nsIContent *aContent, PRInt32 aNamespaceID,
+  static bool MatchAnchors(nsIContent *aContent, PRInt32 aNamespaceID,
                              nsIAtom* aAtom, void* aData);
-  static PRBool MatchNameAttribute(nsIContent* aContent, PRInt32 aNamespaceID,
+  static bool MatchNameAttribute(nsIContent* aContent, PRInt32 aNamespaceID,
                                    nsIAtom* aAtom, void* aData);
+  static void* UseExistingNameString(nsINode* aRootNode, const nsString* aName);
 
   static void DocumentWriteTerminationFunc(nsISupports *aRef);
 
   void GetDomainURI(nsIURI **uri);
 
-  nsresult WriteCommon(const nsAString& aText,
-                       PRBool aNewlineTerminate);
-  nsresult ScriptWriteCommon(PRBool aNewlineTerminate);
-  nsresult OpenCommon(const nsACString& aContentType, PRBool aReplace);
+  nsresult WriteCommon(JSContext *cx, const nsAString& aText,
+                       bool aNewlineTerminate);
 
   nsresult CreateAndAddWyciwygChannel(void);
   nsresult RemoveWyciwygChannel(void);
@@ -266,22 +231,16 @@ protected:
   /**
    * Like IsEditingOn(), but will flush as needed first.
    */
-  PRBool IsEditingOnAfterFlush();
+  bool IsEditingOnAfterFlush();
 
   void *GenerateParserKey(void);
 
-  virtual PRInt32 GetDefaultNamespaceID() const
-  {
-    return kNameSpaceID_XHTML;
-  }
-
-  nsCOMArray<nsIDOMHTMLMapElement> mImageMaps;
-
   nsCOMPtr<nsIDOMHTMLCollection> mImages;
   nsCOMPtr<nsIDOMHTMLCollection> mApplets;
   nsCOMPtr<nsIDOMHTMLCollection> mEmbeds;
   nsCOMPtr<nsIDOMHTMLCollection> mLinks;
   nsCOMPtr<nsIDOMHTMLCollection> mAnchors;
+  nsCOMPtr<nsIDOMHTMLCollection> mScripts;
   nsRefPtr<nsContentList> mForms;
   nsRefPtr<nsContentList> mFormControls;
 
@@ -290,67 +249,45 @@ protected:
 
   static PRUint32 gWyciwygSessionCnt;
 
-  static PRBool TryHintCharset(nsIMarkupDocumentViewer* aMarkupDV,
+  static bool TryHintCharset(nsIMarkupDocumentViewer* aMarkupDV,
                                PRInt32& aCharsetSource,
                                nsACString& aCharset);
-  static PRBool TryUserForcedCharset(nsIMarkupDocumentViewer* aMarkupDV,
-                                     nsIDocumentCharsetInfo*  aDocInfo,
+  static bool TryUserForcedCharset(nsIMarkupDocumentViewer* aMarkupDV,
+                                     nsIDocShell*  aDocShell,
                                      PRInt32& aCharsetSource,
                                      nsACString& aCharset);
-  static PRBool TryCacheCharset(nsICacheEntryDescriptor* aCacheDescriptor,
+  static bool TryCacheCharset(nsICachingChannel* aCachingChannel,
                                 PRInt32& aCharsetSource,
                                 nsACString& aCharset);
-  static PRBool TryBookmarkCharset(nsIDocShell* aDocShell,
-                                   nsIChannel* aChannel,
-                                   PRInt32& aCharsetSource,
-                                   nsACString& aCharset);
   // aParentDocument could be null.
-  PRBool TryParentCharset(nsIDocumentCharsetInfo*  aDocInfo,
+  bool TryParentCharset(nsIDocShell*  aDocShell,
                           nsIDocument* aParentDocument,
                           PRInt32& charsetSource, nsACString& aCharset);
-  static PRBool UseWeakDocTypeDefault(PRInt32& aCharsetSource,
+  static bool UseWeakDocTypeDefault(PRInt32& aCharsetSource,
                                       nsACString& aCharset);
-  static PRBool TryDefaultCharset(nsIMarkupDocumentViewer* aMarkupDV,
+  static bool TryDefaultCharset(nsIMarkupDocumentViewer* aMarkupDV,
                                   PRInt32& aCharsetSource,
                                   nsACString& aCharset);
 
-  void StartAutodetection(nsIDocShell *aDocShell, nsACString& aCharset,
-                          const char* aCommand);
-
   // Override so we can munge the charset on our wyciwyg channel as needed.
   virtual void SetDocumentCharacterSet(const nsACString& aCharSetID);
 
-  // mWriteState tracks the status of this document if the document is being
-  // entirely created by script. In the normal load case, mWriteState will be
-  // eNotWriting. Once document.open has been called (either implicitly or
-  // explicitly), mWriteState will be eDocumentOpened. When document.close has
-  // been called, mWriteState will become eDocumentClosed if there have been no
-  // external script loads in the meantime. If there have been, then mWriteState
-  // becomes ePendingClose, indicating that we might still be writing, but that
-  // we shouldn't process any further close() calls.
-  enum {
-    eNotWriting,
-    eDocumentOpened,
-    ePendingClose,
-    eDocumentClosed
-  } mWriteState;
-
   // Tracks if we are currently processing any document.write calls (either
   // implicit or explicit). Note that if a write call writes out something which
   // would block the parser, then mWriteLevel will be incorrect until the parser
   // finishes processing that script.
   PRUint32 mWriteLevel;
 
-  nsAutoTArray<nsIScriptElement*, 1> mPendingScripts;
-
   // Load flags of the document's channel
   PRUint32 mLoadFlags;
 
-  PRPackedBool mIsFrameset;
+  bool mIsFrameset;
 
-  PRPackedBool mTooDeepWriteRecursion;
+  bool mTooDeepWriteRecursion;
 
-  PRPackedBool mDisableDocWrite;
+  bool mDisableDocWrite;
+
+  bool mWarnedWidthHeight;
 
   nsCOMPtr<nsIWyciwygChannel> mWyciwygChannel;
 
@@ -366,21 +303,17 @@ protected:
   PRUint32 mContentEditableCount;
   EditingState mEditingState;
 
-  nsresult   DoClipboardSecurityCheck(PRBool aPaste);
-  static jsval       sCutCopyInternal_id;
-  static jsval       sPasteInternal_id;
+  nsresult   DoClipboardSecurityCheck(bool aPaste);
+  static jsid        sCutCopyInternal_id;
+  static jsid        sPasteInternal_id;
 
   // When false, the .cookies property is completely disabled
-  PRBool mDisableCookieAccess;
-
-  // Parser used for constructing document fragments.
-  nsCOMPtr<nsIParser> mFragmentParser;
+  bool mDisableCookieAccess;
 };
 
 #define NS_HTML_DOCUMENT_INTERFACE_TABLE_BEGIN(_class)                        \
     NS_DOCUMENT_INTERFACE_TABLE_BEGIN(_class)                                 \
     NS_INTERFACE_TABLE_ENTRY(_class, nsIHTMLDocument)                         \
-    NS_INTERFACE_TABLE_ENTRY(_class, nsIDOMHTMLDocument)                      \
-    NS_INTERFACE_TABLE_ENTRY(_class, nsIDOMNSHTMLDocument)
+    NS_INTERFACE_TABLE_ENTRY(_class, nsIDOMHTMLDocument)
 
 #endif /* nsHTMLDocument_h___ */
