@@ -74,7 +74,6 @@ nsMediaChannelStream::nsMediaChannelStream(nsMediaDecoder* aDecoder,
     mReopenOnError(false), mIgnoreClose(false),
     mCacheStream(this),
     mLock("nsMediaChannelStream.mLock"),
-    mCacheSuspendCount(0),
     mIgnoreResume(false)
 {
 }
@@ -534,9 +533,15 @@ already_AddRefed<nsIPrincipal> nsMediaChannelStream::GetCurrentPrincipal()
   return principal.forget();
 }
 
+bool nsMediaChannelStream::CanClone()
+{
+  return mCacheStream.IsAvailableForSharing();
+}
+
 nsMediaStream* nsMediaChannelStream::CloneData(nsMediaDecoder* aDecoder)
 {
   NS_ASSERTION(NS_IsMainThread(), "Only call on main thread");
+  NS_ASSERTION(mCacheStream.IsAvailableForSharing(), "Stream can't be cloned");
 
   nsMediaChannelStream* stream = new nsMediaChannelStream(aDecoder, nsnull, mURI);
   if (stream) {
@@ -547,8 +552,9 @@ nsMediaStream* nsMediaChannelStream::CloneData(nsMediaDecoder* aDecoder)
     // is already in the cache we don't create an unneccesary HTTP channel
     // and perform a useless HTTP transaction.
     stream->mSuspendCount = 1;
-    stream->mCacheSuspendCount = 1;
     stream->mCacheStream.InitAsClone(&mCacheStream);
+    stream->mChannelStatistics = mChannelStatistics;
+    stream->mChannelStatistics.Stop(TimeStamp::Now());
   }
   return stream;
 }
@@ -770,11 +776,6 @@ nsMediaChannelStream::CacheClientSeek(PRInt64 aOffset, bool aResume)
     NS_ASSERTION(mSuspendCount > 0, "Too many resumes!");
     // No need to mess with the channel, since we're making a new one
     --mSuspendCount;
-    {
-      MutexAutoLock lock(mLock);
-      NS_ASSERTION(mCacheSuspendCount > 0, "CacheClientSeek(aResume=true) without previous CacheClientSuspend!");
-      --mCacheSuspendCount;
-    }
   }
 
   nsresult rv = RecreateChannel();
@@ -788,10 +789,6 @@ nsMediaChannelStream::CacheClientSeek(PRInt64 aOffset, bool aResume)
 nsresult
 nsMediaChannelStream::CacheClientSuspend()
 {
-  {
-    MutexAutoLock lock(mLock);
-    ++mCacheSuspendCount;
-  }
   Suspend(false);
 
   mDecoder->NotifySuspendedStatusChanged();
@@ -802,11 +799,6 @@ nsresult
 nsMediaChannelStream::CacheClientResume()
 {
   Resume();
-  {
-    MutexAutoLock lock(mLock);
-    NS_ASSERTION(mCacheSuspendCount > 0, "CacheClientResume without previous CacheClientSuspend!");
-    --mCacheSuspendCount;
-  }
 
   mDecoder->NotifySuspendedStatusChanged();
   return NS_OK;
@@ -830,11 +822,16 @@ nsMediaChannelStream::IsDataCachedToEndOfStream(PRInt64 aOffset)
   return mCacheStream.IsDataCachedToEndOfStream(aOffset);
 }
 
-bool
-nsMediaChannelStream::IsSuspendedByCache()
+void
+nsMediaChannelStream::EnsureCacheUpToDate()
 {
-  MutexAutoLock lock(mLock);
-  return mCacheSuspendCount > 0;
+  mCacheStream.EnsureCacheUpdate();
+}
+
+bool
+nsMediaChannelStream::IsSuspendedByCache(nsMediaStream** aActiveStream)
+{
+  return mCacheStream.AreAllStreamsForResourceSuspended(aActiveStream);
 }
 
 bool
@@ -922,6 +919,7 @@ public:
   virtual void     Suspend(bool aCloseImmediately) {}
   virtual void     Resume() {}
   virtual already_AddRefed<nsIPrincipal> GetCurrentPrincipal();
+  virtual bool     CanClone();
   virtual nsMediaStream* CloneData(nsMediaDecoder* aDecoder);
   virtual nsresult ReadFromCache(char* aBuffer, PRInt64 aOffset, PRUint32 aCount);
 
@@ -950,7 +948,13 @@ public:
   }
   virtual PRInt64 GetCachedDataEnd(PRInt64 aOffset) { return NS_MAX(aOffset, mSize); }
   virtual bool    IsDataCachedToEndOfStream(PRInt64 aOffset) { return true; }
-  virtual bool    IsSuspendedByCache() { return false; }
+  virtual bool    IsSuspendedByCache(nsMediaStream** aActiveStream)
+  {
+    if (aActiveStream) {
+      *aActiveStream = nsnull;
+    }
+    return false;
+  }
   virtual bool    IsSuspended() { return false; }
 
   nsresult GetCachedRanges(nsTArray<nsByteRange>& aRanges);
@@ -1093,6 +1097,11 @@ already_AddRefed<nsIPrincipal> nsMediaFileStream::GetCurrentPrincipal()
   return principal.forget();
 }
 
+bool nsMediaFileStream::CanClone()
+{
+  return true;
+}
+
 nsMediaStream* nsMediaFileStream::CloneData(nsMediaDecoder* aDecoder)
 {
   NS_ASSERTION(NS_IsMainThread(), "Only call on main thread");
