@@ -95,13 +95,20 @@ public:
   // Methods for resampling all animations
   // (A resample performs the same operations as a sample but doesn't advance
   // the current time and doesn't check if the container is paused)
-  void Resample() { DoSample(PR_FALSE); }
+  // This will flush pending style changes for the document.
+  void Resample() { DoSample(false); }
+
   void SetResampleNeeded()
   {
     if (!mRunningSample) {
-      mResampleNeeded = PR_TRUE;
+      if (!mResampleNeeded) {
+        FlagDocumentNeedsFlush();
+      }
+      mResampleNeeded = true;
     }
   }
+
+  // This will flush pending style changes for the document.
   void FlushResampleRequests()
   {
     if (!mResampleNeeded)
@@ -123,7 +130,7 @@ public:
   void NotifyRefreshDriverDestroying(nsRefreshDriver* aRefreshDriver);
 
   // Helper to check if we have any animation elements at all
-  PRBool HasRegisteredAnimations()
+  bool HasRegisteredAnimations()
   { return mAnimationElementTable.Count() != 0; }
 
 protected:
@@ -136,7 +143,7 @@ protected:
   struct SampleTimeContainerParams
   {
     TimeContainerHashtable* mActiveContainers;
-    PRBool                  mSkipUnchangedContainers;
+    bool                    mSkipUnchangedContainers;
   };
 
   struct SampleAnimationParams
@@ -167,7 +174,7 @@ protected:
 
   // Sample-related callbacks and implementation helpers
   virtual void DoSample();
-  void DoSample(PRBool aSkipUnchangedContainers);
+  void DoSample(bool aSkipUnchangedContainers);
 
   void RewindElements();
   PR_STATIC_CALLBACK(PLDHashOperator) RewindNeeded(
@@ -191,13 +198,15 @@ protected:
                                  TimeContainerHashtable* aActiveContainers);
   static void AddAnimationToCompositorTable(
     nsISMILAnimationElement* aElement, nsSMILCompositorTable* aCompositorTable);
-  static PRBool GetTargetIdentifierForAnimation(
+  static bool GetTargetIdentifierForAnimation(
       nsISMILAnimationElement* aAnimElem, nsSMILTargetIdentifier& aResult);
 
   // Methods for adding/removing time containers
   virtual nsresult AddChild(nsSMILTimeContainer& aChild);
   virtual void     RemoveChild(nsSMILTimeContainer& aChild);
 
+  void FlagDocumentNeedsFlush();
+
   // Members
   nsAutoRefCnt mRefCnt;
   NS_DECL_OWNINGTHREAD
@@ -225,12 +234,12 @@ protected:
   // differently such as not dispatching events).
   nsSMILTime                 mAvgTimeBetweenSamples;
 
-  PRPackedBool               mResampleNeeded;
+  bool                       mResampleNeeded;
   // If we're told to start sampling but there are no animation elements we just
   // record the time, set the following flag, and then wait until we have an
   // animation element. Then we'll reset this flag and actually start sampling.
-  PRPackedBool               mDeferredStartSampling;
-  PRPackedBool               mRunningSample;
+  bool                       mDeferredStartSampling;
+  bool                       mRunningSample;
 
   // Store raw ptr to mDocument.  It owns the controller, so controller
   // shouldn't outlive it
