@@ -36,10 +36,17 @@
 
 #include "SVGPathSegListSMILType.h"
 #include "nsSMILValue.h"
+#include "SVGPathSegUtils.h"
 #include "SVGPathData.h"
+#include "mozilla/Util.h"
 #include <math.h>
 
-using namespace mozilla;
+// Indices of boolean flags within 'arc' segment chunks in path-data arrays
+// (where '0' would correspond to the index of the encoded segment type):
+#define LARGE_ARC_FLAG_IDX 4
+#define SWEEP_FLAG_IDX     5
+
+namespace mozilla {
 
 /*static*/ SVGPathSegListSMILType SVGPathSegListSMILType::sSingleton;
 
@@ -78,7 +85,7 @@ SVGPathSegListSMILType::Assign(nsSMILValue& aDest,
   return dest->CopyFrom(*src);
 }
 
-PRBool
+bool
 SVGPathSegListSMILType::IsEqual(const nsSMILValue& aLeft,
                                 const nsSMILValue& aRight) const
 {
@@ -89,7 +96,7 @@ SVGPathSegListSMILType::IsEqual(const nsSMILValue& aLeft,
          *static_cast<const SVGPathDataAndOwner*>(aRight.mU.mPtr);
 }
 
-static PRBool
+static bool
 ArcFlagsDiffer(SVGPathDataAndOwner::const_iterator aPathData1,
                SVGPathDataAndOwner::const_iterator aPathData2)
 {
@@ -100,8 +107,8 @@ ArcFlagsDiffer(SVGPathDataAndOwner::const_iterator aPathData1,
     (SVGPathSegUtils::IsArcType(SVGPathSegUtils::DecodeType(aPathData2[0])),
                                 "ArcFlagsDiffer called with non-arc segment");
 
-  return aPathData1[4] != aPathData2[4] ||  // large arc flag
-         aPathData1[5] != aPathData2[5];    // sweep flag
+  return aPathData1[LARGE_ARC_FLAG_IDX] != aPathData2[LARGE_ARC_FLAG_IDX] ||
+         aPathData1[SWEEP_FLAG_IDX]     != aPathData2[SWEEP_FLAG_IDX];
 }
 
 enum PathInterpolationResult {
@@ -114,7 +121,7 @@ static PathInterpolationResult
 CanInterpolate(const SVGPathDataAndOwner& aStart,
                const SVGPathDataAndOwner& aEnd)
 {
-  if (aStart.IsEmpty()) {
+  if (aStart.IsIdentity()) {
     return eCanInterpolate;
   }
 
@@ -161,39 +168,6 @@ CanInterpolate(const SVGPathDataAndOwner& aStart,
   return result;
 }
 
-static void
-InterpolatePathSegmentData(SVGPathDataAndOwner::const_iterator& aStart,
-                           SVGPathDataAndOwner::const_iterator& aEnd,
-                           SVGPathDataAndOwner::iterator& aResult,
-                           float aUnitDistance)
-{
-  PRUint32 startType = SVGPathSegUtils::DecodeType(*aStart);
-  PRUint32 endType = SVGPathSegUtils::DecodeType(*aEnd);
-
-  NS_ABORT_IF_FALSE
-    (startType == endType,
-     "InterpolatePathSegmentData expects segment types to be the same!");
-
-  NS_ABORT_IF_FALSE
-    (!(SVGPathSegUtils::IsArcType(startType) && ArcFlagsDiffer(aStart, aEnd)),
-     "InterpolatePathSegmentData cannot interpolate arc segments with different flag values!");
-
-  PRUint32 argCount = SVGPathSegUtils::ArgCountForType(startType);
-
-  // Copy over segment type.
-  *aResult++ = *aStart++;
-  ++aEnd;
-
-  // Interpolate the arguments.
-  SVGPathDataAndOwner::const_iterator startSegEnd = aStart + argCount;
-  while (aStart != startSegEnd) {
-    *aResult = *aStart + (*aEnd - *aStart) * aUnitDistance;
-    ++aStart;
-    ++aEnd;
-    ++aResult;
-  }
-}
-
 enum RelativenessAdjustmentType {
   eAbsoluteToRelative,
   eRelativeToAbsolute
@@ -213,6 +187,134 @@ AdjustSegmentForRelativeness(RelativenessAdjustmentType aAdjustmentType,
   }
 }
 
+/**
+ * Helper function for AddWeightedPathSegLists, to add multiples of two
+ * path-segments of the same type.
+ *
+ * NOTE: |aSeg1| is allowed to be nsnull, so we use |aSeg2| as the
+ * authoritative source of things like segment-type and boolean arc flags.
+ *
+ * @param aCoeff1    The coefficient to use on the first segment.
+ * @param aSeg1      An iterator pointing to the first segment.  This can be
+ *                   null, which is treated as identity (zero).
+ * @param aCoeff2    The coefficient to use on the second segment.
+ * @param aSeg2      An iterator pointing to the second segment.
+ * @param [out] aResultSeg An iterator pointing to where we should write the
+ *                         result of this operation.
+ */
+static inline void
+AddWeightedPathSegs(double aCoeff1,
+                    SVGPathDataAndOwner::const_iterator& aSeg1,
+                    double aCoeff2,
+                    SVGPathDataAndOwner::const_iterator& aSeg2,
+                    SVGPathDataAndOwner::iterator& aResultSeg)
+{
+  NS_ABORT_IF_FALSE(aSeg2, "2nd segment must be non-null");
+  NS_ABORT_IF_FALSE(aResultSeg, "result segment must be non-null");
+
+  PRUint32 segType = SVGPathSegUtils::DecodeType(aSeg2[0]);
+  NS_ABORT_IF_FALSE(!aSeg1 || SVGPathSegUtils::DecodeType(*aSeg1) == segType,
+                    "unexpected segment type");
+
+  // FIRST: Directly copy the arguments that don't make sense to add.
+  aResultSeg[0] = aSeg2[0];  // encoded segment type
+
+  bool isArcType = SVGPathSegUtils::IsArcType(segType);
+  if (isArcType) {
+    // Copy boolean arc flags.
+    NS_ABORT_IF_FALSE(!aSeg1 || !ArcFlagsDiffer(aSeg1, aSeg2),
+                      "Expecting arc flags to match");
+    aResultSeg[LARGE_ARC_FLAG_IDX] = aSeg2[LARGE_ARC_FLAG_IDX];
+    aResultSeg[SWEEP_FLAG_IDX]     = aSeg2[SWEEP_FLAG_IDX];
+  }
+
+  // SECOND: Add the arguments that are supposed to be added.
+  // (The 1's below are to account for segment type)
+  PRUint32 numArgs = SVGPathSegUtils::ArgCountForType(segType);
+  for (PRUint32 i = 1; i < 1 + numArgs; ++i) {
+     // Need to skip arc flags for arc-type segments. (already handled them)
+    if (!(isArcType && (i == LARGE_ARC_FLAG_IDX || i == SWEEP_FLAG_IDX))) {
+      aResultSeg[i] = (aSeg1 ? aCoeff1 * aSeg1[i] : 0.0) + aCoeff2 * aSeg2[i];
+    }
+  }
+
+  // FINALLY: Shift iterators forward. ("1+" is to include seg-type)
+  if (aSeg1) {
+    aSeg1 += 1 + numArgs;
+  }
+  aSeg2 += 1 + numArgs;
+  aResultSeg += 1 + numArgs;
+}
+
+/**
+ * Helper function for Add & Interpolate, to add multiples of two path-segment
+ * lists.
+ *
+ * NOTE: aList1 and aList2 are assumed to have their segment-types and
+ * segment-count match exactly (unless aList1 is an identity value).
+ *
+ * NOTE: aResult, the output list, is expected to either be an identity value
+ * (in which case we'll grow it) *or* to already have the exactly right length
+ * (e.g. in cases where aList1 and aResult are actually the same list).
+ *
+ * @param aCoeff1    The coefficient to use on the first path segment list.
+ * @param aList1     The first path segment list. Allowed to be identity.
+ * @param aCoeff2    The coefficient to use on the second path segment list.
+ * @param aList2     The second path segment list.
+ * @param [out] aResultSeg The resulting path segment list. Allowed to be
+ *                         identity, in which case we'll grow it to the right
+ *                         size. Also allowed to be the same list as aList1.
+ */
+static void
+AddWeightedPathSegLists(double aCoeff1, const SVGPathDataAndOwner& aList1,
+                        double aCoeff2, const SVGPathDataAndOwner& aList2,
+                        SVGPathDataAndOwner& aResult)
+{
+  NS_ABORT_IF_FALSE(aCoeff1 >= 0.0 && aCoeff2 >= 0.0,
+                    "expecting non-negative coefficients");
+  NS_ABORT_IF_FALSE(!aList2.IsIdentity(),
+                    "expecting 2nd list to be non-identity");
+  NS_ABORT_IF_FALSE(aList1.IsIdentity() || aList1.Length() == aList2.Length(),
+                    "expecting 1st list to be identity or to have same "
+                    "length as 2nd list");
+  NS_ABORT_IF_FALSE(aResult.IsIdentity() || aResult.Length() == aList2.Length(),
+                    "expecting result list to be identity or to have same "
+                    "length as 2nd list");
+
+  SVGPathDataAndOwner::const_iterator iter1, end1;
+  if (aList1.IsIdentity()) {
+    iter1 = end1 = nsnull; // indicate that this is an identity list
+  } else {
+    iter1 = aList1.begin();
+    end1 = aList1.end();
+  }
+  SVGPathDataAndOwner::const_iterator iter2 = aList2.begin();
+  SVGPathDataAndOwner::const_iterator end2 = aList2.end();
+
+  // Grow |aResult| if necessary. (NOTE: It's possible that aResult and aList1
+  // are the same list, so this may implicitly resize aList1. That's fine,
+  // because in that case, we will have already set iter1 to nsnull above, to
+  // record that our first operand is an identity value.)
+  if (aResult.IsIdentity()) {
+    DebugOnly<bool> success = aResult.SetLength(aList2.Length());
+    NS_ABORT_IF_FALSE(success, "infallible nsTArray::SetLength should succeed");
+    aResult.SetElement(aList2.Element()); // propagate target element info!
+  }
+
+  SVGPathDataAndOwner::iterator resultIter = aResult.begin();
+
+  while ((!iter1 || iter1 != end1) &&
+         iter2 != end2) {
+    AddWeightedPathSegs(aCoeff1, iter1,
+                        aCoeff2, iter2,
+                        resultIter);
+  }
+  NS_ABORT_IF_FALSE((!iter1 || iter1 == end1) &&
+                    iter2 == end2 &&
+                    resultIter == aResult.end(),
+                    "Very, very bad - path data corrupt");
+}
+
 static void
 ConvertPathSegmentData(SVGPathDataAndOwner::const_iterator& aStart,
                        SVGPathDataAndOwner::const_iterator& aEnd,
@@ -336,40 +438,32 @@ SVGPathSegListSMILType::Add(nsSMILValue& aDest,
   const SVGPathDataAndOwner& valueToAdd =
     *static_cast<const SVGPathDataAndOwner*>(aValueToAdd.mU.mPtr);
 
-  // Allow addition to empty dest.
-  if (dest.IsEmpty()) {
-    return dest.CopyFrom(valueToAdd);
+  if (valueToAdd.IsIdentity()) { // Adding identity value - no-op
+    return NS_OK;
   }
 
-  PathInterpolationResult check = CanInterpolate(dest, valueToAdd);
+  if (!dest.IsIdentity()) {
+    // Neither value is identity; make sure they're compatible.
+    NS_ABORT_IF_FALSE(dest.Element() == valueToAdd.Element(),
+                      "adding values from different elements...?");
 
-  if (check == eCannotInterpolate) {
-    // nsSVGUtils::ReportToConsole - can't add path segment lists with different
-    // numbers of segments, with arcs with different flag values, or with
-    // incompatible segment types.
-    return NS_ERROR_FAILURE;
-  }
-
-  if (check == eRequiresConversion) {
-    ConvertAllPathSegmentData(dest.begin(), dest.end(),
-                              valueToAdd.begin(), valueToAdd.end(),
-                              dest.begin());
-  }
-
-  PRUint32 i = 0;
-  while (i < dest.Length()) {
-    PRUint32 type = SVGPathSegUtils::DecodeType(dest[i]);
-    i++;
-    PRUint32 segEnd = i + SVGPathSegUtils::ArgCountForType(type);
-    for (; i < segEnd; ++i) {
-      dest[i] += valueToAdd[i] * aCount;
+    PathInterpolationResult check = CanInterpolate(dest, valueToAdd);
+    if (check == eCannotInterpolate) {
+      // nsSVGUtils::ReportToConsole - can't add path segment lists with
+      // different numbers of segments, with arcs that have different flag
+      // values, or with incompatible segment types.
+      return NS_ERROR_FAILURE;
+    }
+    if (check == eRequiresConversion) {
+      // Convert dest, in-place, to match the types in valueToAdd:
+      ConvertAllPathSegmentData(dest.begin(), dest.end(),
+                                valueToAdd.begin(), valueToAdd.end(),
+                                dest.begin());
     }
   }
 
-  NS_ABORT_IF_FALSE(i == dest.Length(), "Very, very bad - path data corrupt");
+  AddWeightedPathSegLists(1.0, dest, aCount, valueToAdd, dest);
 
-  // For now we only support pure 'to' animation.
-  // nsSVGUtils::ReportToConsole
   return NS_OK;
 }
 
@@ -405,6 +499,8 @@ SVGPathSegListSMILType::Interpolate(const nsSMILValue& aStartVal,
     *static_cast<const SVGPathDataAndOwner*>(aEndVal.mU.mPtr);
   SVGPathDataAndOwner& result =
     *static_cast<SVGPathDataAndOwner*>(aResult.mU.mPtr);
+  NS_ABORT_IF_FALSE(result.IsIdentity(),
+                    "expecting outparam to start out as identity");
 
   PathInterpolationResult check = CanInterpolate(start, end); 
 
@@ -415,57 +511,24 @@ SVGPathSegListSMILType::Interpolate(const nsSMILValue& aStartVal,
     return NS_ERROR_FAILURE;
   }
 
-  if (!result.SetLength(end.Length())) {
-    return NS_ERROR_OUT_OF_MEMORY;
+  const SVGPathDataAndOwner* startListToUse = &start;
+  if (check == eRequiresConversion) {
+    // Can't convert |start| in-place, since it's const. Instead, we copy it
+    // into |result|, converting the types as we go, and use that as our start.
+    DebugOnly<bool> success = result.SetLength(end.Length());
+    NS_ABORT_IF_FALSE(success, "infallible nsTArray::SetLength should succeed");
+    result.SetElement(end.Element()); // propagate target element info!
+
+    ConvertAllPathSegmentData(start.begin(), start.end(),
+                              end.begin(), end.end(),
+                              result.begin());
+    startListToUse = &result;
   }
 
-  if (start.IsEmpty()) { // identity path
-    PRUint32 i = 0;
-    while (i < end.Length()) {
-      PRUint32 type = SVGPathSegUtils::DecodeType(end[i]);
-      result[i] = end[i];
-      i++;
-      PRUint32 segEnd = i + SVGPathSegUtils::ArgCountForType(type);
-      if (SVGPathSegUtils::IsArcType(type)) {
-        result[i] = end[i] * aUnitDistance;
-        result[i+1] = end[i+1] * aUnitDistance;
-        result[i+2] = end[i+2] * aUnitDistance;
-        // boolean args largeArcFlag and sweepFlag must be the same
-        result[i+3] = end[i+3];
-        result[i+4] = end[i+4];
-        result[i+5] = end[i+5] * aUnitDistance;
-        result[i+6] = end[i+6] * aUnitDistance;
-        i = segEnd;
-      } else {
-        for (; i < segEnd; ++i) {
-          result[i] = end[i] * aUnitDistance;
-        }
-      }
-    }
-    NS_ABORT_IF_FALSE(i == end.Length() && i == result.Length(),
-                      "Very, very bad - path data corrupt");
-  } else {
-    SVGPathDataAndOwner::const_iterator pStart = start.begin();
-    SVGPathDataAndOwner::const_iterator pStartDataEnd = start.end();
-    SVGPathDataAndOwner::const_iterator pEnd = end.begin();
-    SVGPathDataAndOwner::const_iterator pEndDataEnd = end.end();
-    SVGPathDataAndOwner::iterator pResult = result.begin();
-
-    if (check == eRequiresConversion) {
-      ConvertAllPathSegmentData(pStart, pStartDataEnd, pEnd, pEndDataEnd,
-                                pResult);
-      pStart = pResult;
-      pStartDataEnd = result.end();
-    }
-
-    while (pStart != pStartDataEnd && pEnd != pEndDataEnd) {
-      InterpolatePathSegmentData(pStart, pEnd, pResult, aUnitDistance);
-    }
-
-    NS_ABORT_IF_FALSE(pStart == pStartDataEnd && pEnd == pEndDataEnd &&
-                      pResult == result.end(),
-                      "Very, very bad - path data corrupt");
-  }
+  AddWeightedPathSegLists(1.0 - aUnitDistance, *startListToUse,
+                          aUnitDistance, end, result);
 
   return NS_OK;
 }
+
+} // namespace mozilla
