@@ -39,14 +39,8 @@
 #ifndef __NS_SVGPATHDATAPARSER_H__
 #define __NS_SVGPATHDATAPARSER_H__
 
-#include "nsSVGDataParser.h"
-#include "nsCOMPtr.h"
-#include "nsCOMArray.h"
-#include "nsIDOMSVGPathSeg.h"
-#include "nsTArray.h"
 #include "gfxPoint.h"
-
-class nsSVGPathList;
+#include "nsSVGDataParser.h"
 
 namespace mozilla {
 class SVGPathData;
