@@ -39,14 +39,15 @@
 #ifndef __NS_SVGPATTERNELEMENT_H__
 #define __NS_SVGPATTERNELEMENT_H__
 
-#include "nsSVGStylableElement.h"
-#include "nsIDOMSVGURIReference.h"
+#include "DOMSVGTests.h"
 #include "nsIDOMSVGFitToViewBox.h"
 #include "nsIDOMSVGPatternElement.h"
 #include "nsIDOMSVGUnitTypes.h"
-#include "nsSVGLength2.h"
+#include "nsIDOMSVGURIReference.h"
 #include "nsSVGEnum.h"
+#include "nsSVGLength2.h"
 #include "nsSVGString.h"
+#include "nsSVGStylableElement.h"
 #include "nsSVGViewBox.h"
 #include "SVGAnimatedPreserveAspectRatio.h"
 #include "SVGAnimatedTransformList.h"
@@ -56,9 +57,10 @@
 typedef nsSVGStylableElement nsSVGPatternElementBase;
 
 class nsSVGPatternElement : public nsSVGPatternElementBase,
+                            public nsIDOMSVGPatternElement,
+                            public DOMSVGTests,
                             public nsIDOMSVGURIReference,
                             public nsIDOMSVGFitToViewBox,
-                            public nsIDOMSVGPatternElement,
                             public nsIDOMSVGUnitTypes
 {
   friend class nsSVGPatternFrame;
@@ -88,7 +90,7 @@ public:
   NS_FORWARD_NSIDOMSVGELEMENT(nsSVGElement::)
 
   // nsIContent interface
-  NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* name) const;
+  NS_IMETHOD_(bool) IsAttributeMapped(const nsIAtom* name) const;
 
   virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
