@@ -90,15 +90,12 @@ public:
   // nsScriptElement
   virtual bool HasScriptContent();
 
-  // nsSVGElement specializations:
-  virtual void DidChangeString(PRUint8 aAttrEnum);
-
   // nsIContent specializations:
-  virtual nsresult DoneAddingChildren(bool aHaveNotified);
-  virtual bool IsDoneAddingChildren();
   virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
                               nsIContent* aBindingParent,
                               bool aCompileEventHandlers);
+  virtual nsresult AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+                                const nsAString* aValue, bool aNotify);
 
   virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
@@ -254,16 +251,6 @@ nsSVGScriptElement::HasScriptContent()
 //----------------------------------------------------------------------
 // nsSVGElement methods
 
-void
-nsSVGScriptElement::DidChangeString(PRUint8 aAttrEnum)
-{
-  nsSVGScriptElementBase::DidChangeString(aAttrEnum);
-
-  if (aAttrEnum == HREF) {
-    MaybeProcessScript();
-  }
-}
-
 nsSVGElement::StringAttributesInfo
 nsSVGScriptElement::GetStringInfo()
 {
@@ -274,25 +261,6 @@ nsSVGScriptElement::GetStringInfo()
 //----------------------------------------------------------------------
 // nsIContent methods
 
-nsresult
-nsSVGScriptElement::DoneAddingChildren(bool aHaveNotified)
-{
-  mDoneAddingChildren = true;
-  nsresult rv = MaybeProcessScript();
-  if (!mAlreadyStarted) {
-    // Need to lose parser-insertedness here to allow another script to cause
-    // execution later.
-    LoseParserInsertedness();
-  }
-  return rv;
-}
-
-bool
-nsSVGScriptElement::IsDoneAddingChildren()
-{
-  return mDoneAddingChildren;
-}
-
 nsresult
 nsSVGScriptElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
                                nsIContent* aBindingParent,
@@ -310,3 +278,13 @@ nsSVGScriptElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
   return NS_OK;
 }
 
+nsresult
+nsSVGScriptElement::AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+                                 const nsAString* aValue, bool aNotify)
+{
+  if (aNamespaceID == kNameSpaceID_XLink && aName == nsGkAtoms::href) {
+    MaybeProcessScript();
+  }
+  return nsSVGScriptElementBase::AfterSetAttr(aNamespaceID, aName,
+                                              aValue, aNotify);
+}
