@@ -34,6 +34,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Util.h"
+
 #include "nsSVGUseElement.h"
 #include "nsIDOMSVGGElement.h"
 #include "nsGkAtoms.h"
@@ -45,6 +47,7 @@
 #include "mozilla/dom/Element.h"
 #include "nsContentUtils.h"
 
+using namespace mozilla;
 using namespace mozilla::dom;
 
 ////////////////////////////////////////////////////////////////////////
@@ -60,7 +63,7 @@ nsSVGElement::LengthInfo nsSVGUseElement::sLengthInfo[4] =
 
 nsSVGElement::StringInfo nsSVGUseElement::sStringInfo[1] =
 {
-  { &nsGkAtoms::href, kNameSpaceID_XLink, PR_TRUE }
+  { &nsGkAtoms::href, kNameSpaceID_XLink, true }
 };
 
 NS_IMPL_NS_NEW_SVG_ELEMENT(Use)
@@ -89,8 +92,9 @@ NS_IMPL_RELEASE_INHERITED(nsSVGUseElement,nsSVGUseElementBase)
 DOMCI_NODE_DATA(SVGUseElement, nsSVGUseElement)
 
 NS_INTERFACE_TABLE_HEAD_CYCLE_COLLECTION_INHERITED(nsSVGUseElement)
-  NS_NODE_INTERFACE_TABLE6(nsSVGUseElement, nsIDOMNode, nsIDOMElement,
-                           nsIDOMSVGElement, nsIDOMSVGURIReference,
+  NS_NODE_INTERFACE_TABLE7(nsSVGUseElement, nsIDOMNode, nsIDOMElement,
+                           nsIDOMSVGElement, nsIDOMSVGTests,
+                           nsIDOMSVGURIReference,
                            nsIDOMSVGUseElement, nsIMutationObserver)
   NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGUseElement)
   if (aIID.Equals(NS_GET_IID(nsSVGUseElement)))
@@ -315,9 +319,9 @@ nsSVGUseElement::CreateAnonymousContent()
   nsCOMPtr<nsIDOMNode> newnode;
   nsCOMArray<nsINode> unused;
   nsNodeInfoManager* nodeInfoManager =
-    targetContent->GetOwnerDoc() == GetOwnerDoc() ?
-      nsnull : GetOwnerDoc()->NodeInfoManager();
-  nsNodeUtils::Clone(targetContent, PR_TRUE, nodeInfoManager, unused,
+    targetContent->OwnerDoc() == OwnerDoc() ?
+      nsnull : OwnerDoc()->NodeInfoManager();
+  nsNodeUtils::Clone(targetContent, true, nodeInfoManager, unused,
                      getter_AddRefs(newnode));
 
   nsCOMPtr<nsIContent> newcontent = do_QueryInterface(newnode);
@@ -360,15 +364,15 @@ nsSVGUseElement::CreateAnonymousContent()
       nsIAtom* lname = name->LocalName();
 
       newcontent->GetAttr(nsID, lname, value);
-      svgNode->SetAttr(nsID, lname, name->GetPrefix(), value, PR_FALSE);
+      svgNode->SetAttr(nsID, lname, name->GetPrefix(), value, false);
     }
 
     // move the children over
     PRUint32 num = newcontent->GetChildCount();
     for (i = 0; i < num; i++) {
       nsCOMPtr<nsIContent> child = newcontent->GetFirstChild();
-      newcontent->RemoveChildAt(0, PR_FALSE);
-      svgNode->InsertChildAt(child, i, PR_TRUE);
+      newcontent->RemoveChildAt(0, false);
+      svgNode->InsertChildAt(child, i, true);
     }
 
     newcontent = svgNode;
@@ -390,7 +394,7 @@ nsSVGUseElement::CreateAnonymousContent()
   nsCAutoString spec;
   baseURI->GetSpec(spec);
   newcontent->SetAttr(kNameSpaceID_XML, nsGkAtoms::base,
-                      NS_ConvertUTF8toUTF16(spec), PR_FALSE);
+                      NS_ConvertUTF8toUTF16(spec), false);
 
   targetContent->AddMutationObserver(this);
   mClone = newcontent;
@@ -406,7 +410,7 @@ nsSVGUseElement::DestroyAnonymousContent()
 //----------------------------------------------------------------------
 // implementation helpers
 
-PRBool nsSVGUseElement::HasValidDimensions()
+bool nsSVGUseElement::HasValidDimensions()
 {
   nsSVGSVGElement *ctx = GetCtx();
 
@@ -417,24 +421,25 @@ PRBool nsSVGUseElement::HasValidDimensions()
 }
 
 void
-nsSVGUseElement::SyncWidthHeight(PRUint8 aAttrEnum)
+nsSVGUseElement::SyncWidthHeight(nsIAtom* aName)
 {
+  NS_ASSERTION(aName == nsGkAtoms::width || aName == nsGkAtoms::height,
+               "The clue is in the function name");
+
   if (HasValidDimensions() == !mClone) {
     TriggerReclone();
     return;
   }
 
-  if (mClone && (aAttrEnum == WIDTH || aAttrEnum == HEIGHT)) {
+  if (mClone) {
     nsCOMPtr<nsIDOMSVGSymbolElement> symbol = do_QueryInterface(mClone);
     nsCOMPtr<nsIDOMSVGSVGElement>    svg    = do_QueryInterface(mClone);
 
     if (symbol || svg) {
-      nsIAtom* aAttrName = aAttrEnum == WIDTH ?
-        nsGkAtoms::width : nsGkAtoms::height;
-
-      if (mLengthAttributes[aAttrEnum].IsExplicitlySet()) {
+      PRUint32 index = *sLengthInfo[WIDTH].mName == aName ? WIDTH : HEIGHT;
+      if (mLengthAttributes[index].IsExplicitlySet()) {
         static_cast<nsSVGElement*>(mClone.get())->
-          SetLength(aAttrName, mLengthAttributes[aAttrEnum]);
+          SetLength(aName, mLengthAttributes[index]);
       } else {
         // Our width/height attribute is now no longer explicitly set, so we
         // need to revert the clone's width/height to the width/height of the
@@ -497,67 +502,24 @@ nsSVGUseElement::PrependLocalTransformTo(const gfxMatrix &aMatrix) const
   return matrix.PreMultiply(gfxMatrix().Translate(gfxPoint(x, y)));
 }
 
-void
-nsSVGUseElement::DidChangeLength(PRUint8 aAttrEnum, PRBool aDoSetAttr)
-{
-  nsSVGUseElementBase::DidChangeLength(aAttrEnum, aDoSetAttr);
-
-  SyncWidthHeight(aAttrEnum);
-}
-
-void
-nsSVGUseElement::DidAnimateLength(PRUint8 aAttrEnum)
-{
-  nsSVGUseElementBase::DidAnimateLength(aAttrEnum);
-
-  SyncWidthHeight(aAttrEnum);
-}
-
 nsSVGElement::LengthAttributesInfo
 nsSVGUseElement::GetLengthInfo()
 {
   return LengthAttributesInfo(mLengthAttributes, sLengthInfo,
-                              NS_ARRAY_LENGTH(sLengthInfo));
-}
-
-void
-nsSVGUseElement::DidChangeString(PRUint8 aAttrEnum)
-{
-  nsSVGUseElementBase::DidChangeString(aAttrEnum);
-
-  if (aAttrEnum == HREF) {
-    // we're changing our nature, clear out the clone information
-    mOriginal = nsnull;
-    UnlinkSource();
-    TriggerReclone();
-  }
-}
-
-void
-nsSVGUseElement::DidAnimateString(PRUint8 aAttrEnum)
-{
-  if (aAttrEnum == HREF) {
-    // we're changing our nature, clear out the clone information
-    mOriginal = nsnull;
-    UnlinkSource();
-    TriggerReclone();
-    return;
-  }
-
-  nsSVGUseElementBase::DidAnimateString(aAttrEnum);
+                              ArrayLength(sLengthInfo));
 }
 
 nsSVGElement::StringAttributesInfo
 nsSVGUseElement::GetStringInfo()
 {
   return StringAttributesInfo(mStringAttributes, sStringInfo,
-                              NS_ARRAY_LENGTH(sStringInfo));
+                              ArrayLength(sStringInfo));
 }
 
 //----------------------------------------------------------------------
 // nsIContent methods
 
-NS_IMETHODIMP_(PRBool)
+NS_IMETHODIMP_(bool)
 nsSVGUseElement::IsAttributeMapped(const nsIAtom* name) const
 {
   static const MappedAttributeEntry* const map[] = {
@@ -571,7 +533,7 @@ nsSVGUseElement::IsAttributeMapped(const nsIAtom* name) const
     sViewportsMap
   };
 
-  return FindAttributeDependence(name, map, NS_ARRAY_LENGTH(map)) ||
+  return FindAttributeDependence(name, map) ||
     nsSVGUseElementBase::IsAttributeMapped(name);
 }
 
