@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -81,10 +81,10 @@ public:
     NS_DECL_CYCLE_COLLECTING_ISUPPORTS
 
     // nsIScriptGlobalObject methods
-    virtual void OnFinalize(PRUint32 aLangID, void *aGlobal);
-    virtual void SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
+    virtual void OnFinalize(JSObject* aObject);
+    virtual void SetScriptsEnabled(bool aEnabled, bool aFireTimeouts);
 
-    virtual void *GetScriptGlobal(PRUint32 lang);
+    virtual JSObject* GetGlobalJSObject();
     virtual nsresult EnsureScriptEnvironment(PRUint32 aLangID);
 
     virtual nsIScriptContext *GetScriptContext(PRUint32 lang);
@@ -103,8 +103,8 @@ protected:
 
     nsXULPrototypeDocument* mGlobalObjectOwner; // weak reference
 
-    nsCOMPtr<nsIScriptContext>  mScriptContexts[NS_STID_ARRAY_UBOUND];
-    void *                      mScriptGlobals[NS_STID_ARRAY_UBOUND];
+    nsCOMPtr<nsIScriptContext> mContext;
+    JSObject* mJSObject;
 
     nsCOMPtr<nsIPrincipal> mCachedPrincipal;
 
@@ -124,7 +124,7 @@ nsXULPDGlobalObject_finalize(JSContext *cx, JSObject *obj)
     nsCOMPtr<nsIScriptGlobalObject> sgo(do_QueryInterface(nativeThis));
 
     if (sgo) {
-        sgo->OnFinalize(nsIProgrammingLanguage::JAVASCRIPT, obj);
+        sgo->OnFinalize(obj);
     }
 
     // The addref was part of JSObject construction
@@ -143,10 +143,11 @@ nsXULPDGlobalObject_resolve(JSContext *cx, JSObject *obj, jsid id)
 
 JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
     "nsXULPrototypeScript compilation scope",
-    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS | JSCLASS_GLOBAL_FLAGS,
+    XPCONNECT_GLOBAL_FLAGS,
     JS_PropertyStub,  JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, nsXULPDGlobalObject_resolve,  JS_ConvertStub,
-    nsXULPDGlobalObject_finalize
+    nsXULPDGlobalObject_finalize, NULL, NULL, NULL, NULL, NULL, NULL,
+    TraceXPCGlobal
 };
 
 
@@ -158,7 +159,8 @@ JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
 
 nsXULPrototypeDocument::nsXULPrototypeDocument()
     : mRoot(nsnull),
-      mLoaded(PR_FALSE)
+      mLoaded(false),
+      mCCGeneration(0)
 {
     ++gRefCnt;
 }
@@ -190,13 +192,23 @@ nsXULPrototypeDocument::~nsXULPrototypeDocument()
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPrototypeDocument)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_0(nsXULPrototypeDocument)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULPrototypeDocument)
+    tmp->mPrototypeWaiters.Clear();
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPrototypeDocument)
+    if (nsCCUncollectableMarker::InGeneration(cb, tmp->mCCGeneration)) {
+        return NS_SUCCESS_INTERRUPTED_TRAVERSE;
+    }
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_MEMBER(mRoot,
                                                     nsXULPrototypeElement)
+    NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mGlobalObject");
     cb.NoteXPCOMChild(static_cast<nsIScriptGlobalObject*>(tmp->mGlobalObject));
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_MEMBER(mNodeInfoManager,
                                                     nsNodeInfoManager)
+    for (PRUint32 i = 0; i < tmp->mPrototypeWaiters.Length(); ++i) {
+        NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mPrototypeWaiters[i]");
+        cb.NoteXPCOMChild(static_cast<nsINode*>(tmp->mPrototypeWaiters[i].get()));
+    }
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPrototypeDocument)
@@ -205,10 +217,8 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPrototypeDocument)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIScriptGlobalObjectOwner)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_CYCLE_COLLECTING_ADDREF_AMBIGUOUS(nsXULPrototypeDocument,
-                                          nsIScriptGlobalObjectOwner)
-NS_IMPL_CYCLE_COLLECTING_RELEASE_AMBIGUOUS(nsXULPrototypeDocument,
-                                           nsIScriptGlobalObjectOwner)
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULPrototypeDocument)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsXULPrototypeDocument)
 
 NS_IMETHODIMP
 NS_NewXULPrototypeDocument(nsXULPrototypeDocument** aResult)
@@ -270,7 +280,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     NS_TIME_FUNCTION;
     nsresult rv;
 
-    rv = aStream->ReadObject(PR_TRUE, getter_AddRefs(mURI));
+    rv = aStream->ReadObject(true, getter_AddRefs(mURI));
 
     PRUint32 count, i;
     nsCOMPtr<nsIURI> styleOverlayURI;
@@ -279,14 +289,14 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     if (NS_FAILED(rv)) return rv;
 
     for (i = 0; i < count; ++i) {
-        rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(styleOverlayURI));
+        rv |= aStream->ReadObject(true, getter_AddRefs(styleOverlayURI));
         mStyleSheetReferences.AppendObject(styleOverlayURI);
     }
 
 
     // nsIPrincipal mNodeInfoManager->mPrincipal
     nsCOMPtr<nsIPrincipal> principal;
-    rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(principal));
+    rv |= aStream->ReadObject(true, getter_AddRefs(principal));
     // Better safe than sorry....
     mNodeInfoManager->SetDocumentPrincipal(principal);
 
@@ -305,7 +315,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 
     rv |= aStream->Read32(&count);
     nsAutoString namespaceURI, prefixStr, localName;
-    PRBool prefixIsNull;
+    bool prefixIsNull;
     nsCOMPtr<nsIAtom> prefix;
     for (i = 0; i < count; ++i) {
         rv |= aStream->ReadString(namespaceURI);
@@ -319,7 +329,11 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
         rv |= aStream->ReadString(localName);
 
         nsCOMPtr<nsINodeInfo> nodeInfo;
+        // Using PR_UINT16_MAX here as we don't know which nodeinfos will be
+        // used for attributes and which for elements. And that doesn't really
+        // matter.
         rv |= mNodeInfoManager->GetNodeInfo(localName, prefix, namespaceURI,
+                                            PR_UINT16_MAX,
                                             getter_AddRefs(nodeInfo));
         if (!nodeInfos.AppendObject(nodeInfo))
             rv |= NS_ERROR_OUT_OF_MEMORY;
@@ -371,7 +385,8 @@ GetNodeInfos(nsXULPrototypeElement* aPrototype,
         nsAttrName* name = &aPrototype->mAttributes[i].mName;
         if (name->IsAtom()) {
             ni = aPrototype->mNodeInfo->NodeInfoManager()->
-                GetNodeInfo(name->Atom(), nsnull, kNameSpaceID_None);
+                GetNodeInfo(name->Atom(), nsnull, kNameSpaceID_None,
+                            nsIDOMNode::ATTRIBUTE_NODE);
             NS_ENSURE_TRUE(ni, NS_ERROR_OUT_OF_MEMORY);
         }
         else {
@@ -403,7 +418,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
 {
     nsresult rv;
 
-    rv = aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), PR_TRUE);
+    rv = aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), true);
     
     PRUint32 count;
 
@@ -413,13 +428,20 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     PRUint32 i;
     for (i = 0; i < count; ++i) {
         rv |= aStream->WriteCompoundObject(mStyleSheetReferences[i],
-                                           NS_GET_IID(nsIURI), PR_TRUE);
+                                           NS_GET_IID(nsIURI), true);
     }
 
     // nsIPrincipal mNodeInfoManager->mPrincipal
     rv |= aStream->WriteObject(mNodeInfoManager->DocumentPrincipal(),
-                               PR_TRUE);
+                               true);
     
+#ifdef DEBUG
+    // XXX Worrisome if we're caching things without system principal.
+    if (!nsContentUtils::IsSystemPrincipal(mNodeInfoManager->DocumentPrincipal())) {
+        NS_WARNING("Serializing document without system principal");
+    }
+#endif
+
     // nsINodeInfo table
     nsCOMArray<nsINodeInfo> nodeInfos;
     if (mRoot)
@@ -437,7 +459,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
 
         nsAutoString prefix;
         nodeInfo->GetPrefix(prefix);
-        PRBool nullPrefix = DOMStringIsNull(prefix);
+        bool nullPrefix = DOMStringIsNull(prefix);
         rv |= aStream->WriteBoolean(nullPrefix);
         if (!nullPrefix) {
             rv |= aStream->WriteWStringZ(prefix.get());
@@ -571,7 +593,7 @@ nsXULPrototypeDocument::GetNodeInfoManager()
 
 
 nsresult
-nsXULPrototypeDocument::AwaitLoadDone(nsXULDocument* aDocument, PRBool* aResult)
+nsXULPrototypeDocument::AwaitLoadDone(nsXULDocument* aDocument, bool* aResult)
 {
     nsresult rv = NS_OK;
 
@@ -596,13 +618,13 @@ nsXULPrototypeDocument::NotifyLoadDone()
 
     nsresult rv = NS_OK;
 
-    mLoaded = PR_TRUE;
+    mLoaded = true;
 
     for (PRUint32 i = mPrototypeWaiters.Length(); i > 0; ) {
         --i;
-        // PR_TRUE means that OnPrototypeLoadDone will also
+        // true means that OnPrototypeLoadDone will also
         // call ResumeWalk().
-        rv = mPrototypeWaiters[i]->OnPrototypeLoadDone(PR_TRUE);
+        rv = mPrototypeWaiters[i]->OnPrototypeLoadDone(true);
         if (NS_FAILED(rv)) break;
     }
     mPrototypeWaiters.Clear();
@@ -630,9 +652,9 @@ nsXULPrototypeDocument::GetScriptGlobalObject()
 //
 
 nsXULPDGlobalObject::nsXULPDGlobalObject(nsXULPrototypeDocument* owner)
-    :  mGlobalObjectOwner(owner)
+  : mGlobalObjectOwner(owner)
+  , mJSObject(NULL)
 {
-  memset(mScriptGlobals, 0, sizeof(mScriptGlobals));
 }
 
 
@@ -643,12 +665,7 @@ nsXULPDGlobalObject::~nsXULPDGlobalObject()
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPDGlobalObject)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_0(nsXULPDGlobalObject)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPDGlobalObject)
-  {
-    PRUint32 lang_index;
-    NS_STID_FOR_INDEX(lang_index) {
-      cb.NoteXPCOMChild(tmp->mScriptContexts[lang_index]);
-    }
-  }
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mContext)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPDGlobalObject)
@@ -657,10 +674,8 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPDGlobalObject)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIScriptGlobalObject)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_CYCLE_COLLECTING_ADDREF_AMBIGUOUS(nsXULPDGlobalObject,
-                                          nsIScriptGlobalObject)
-NS_IMPL_CYCLE_COLLECTING_RELEASE_AMBIGUOUS(nsXULPDGlobalObject,
-                                           nsIScriptGlobalObject)
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULPDGlobalObject)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsXULPDGlobalObject)
 
 //----------------------------------------------------------------------
 //
@@ -670,150 +685,128 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE_AMBIGUOUS(nsXULPDGlobalObject,
 nsresult
 nsXULPDGlobalObject::SetScriptContext(PRUint32 lang_id, nsIScriptContext *aScriptContext)
 {
+  NS_ABORT_IF_FALSE(lang_id == nsIProgrammingLanguage::JAVASCRIPT,
+                    "We don't support this language ID");
   // almost a clone of nsGlobalWindow
-  nsresult rv;
-
-  PRBool ok = NS_STID_VALID(lang_id);
-  NS_ASSERTION(ok, "Invalid programming language ID requested");
-  NS_ENSURE_TRUE(ok, NS_ERROR_INVALID_ARG);
-  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
-
-  if (!aScriptContext)
+  if (!aScriptContext) {
     NS_WARNING("Possibly early removal of script object, see bug #41608");
-  else {
+  } else {
     // should probably assert the context is clean???
     aScriptContext->WillInitializeContext();
-    // NOTE: We init this context with a NULL global - this is subtly
-    // different than nsGlobalWindow which passes 'this'
-    rv = aScriptContext->InitContext();
+    nsresult rv = aScriptContext->InitContext();
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  NS_ASSERTION(!aScriptContext || !mScriptContexts[lang_ndx],
-               "Bad call to SetContext()!");
+  NS_ASSERTION(!aScriptContext || !mContext, "Bad call to SetContext()!");
 
-  void *script_glob = nsnull;
+  JSObject* global = NULL;
 
   if (aScriptContext) {
-    aScriptContext->SetGCOnDestruction(PR_FALSE);
+    aScriptContext->SetGCOnDestruction(false);
     aScriptContext->DidInitializeContext();
-    script_glob = aScriptContext->GetNativeGlobal();
-    NS_ASSERTION(script_glob, "GetNativeGlobal returned NULL!");
+    global = aScriptContext->GetNativeGlobal();
+    NS_ASSERTION(global, "GetNativeGlobal returned NULL!");
   }
-  mScriptContexts[lang_ndx] = aScriptContext;
-  mScriptGlobals[lang_ndx] = script_glob;
+  mContext = aScriptContext;
+  mJSObject = global;
   return NS_OK;
 }
 
 nsresult
 nsXULPDGlobalObject::EnsureScriptEnvironment(PRUint32 lang_id)
 {
-  PRBool ok = NS_STID_VALID(lang_id);
-  NS_ASSERTION(ok, "Invalid programming language ID requested");
-  NS_ENSURE_TRUE(ok, NS_ERROR_INVALID_ARG);
-  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
-
-  if (mScriptContexts[lang_ndx] == nsnull) {
-    nsresult rv;
-    NS_ASSERTION(mScriptGlobals[lang_ndx] == nsnull, "Have global without context?");
-
-    nsCOMPtr<nsIScriptRuntime> languageRuntime;
-    rv = NS_GetScriptRuntimeByID(lang_id, getter_AddRefs(languageRuntime));
-    NS_ENSURE_SUCCESS(rv, nsnull);
-
-    nsCOMPtr<nsIScriptContext> ctxNew;
-    rv = languageRuntime->CreateContext(getter_AddRefs(ctxNew));
-    // For JS, we have to setup a special global object.  We do this then
-    // attach it as the global for this context.  Then, ::SetScriptContext
-    // will re-fetch the global and set it up in our language globals array.
-    if (lang_id == nsIProgrammingLanguage::JAVASCRIPT) {
-      // some special JS specific code we should abstract
-      JSContext *cx = (JSContext *)ctxNew->GetNativeContext();
-      JSAutoRequest ar(cx);
-
-      nsIPrincipal *principal = GetPrincipal();
-      JSObject *newGlob;
-      JSCompartment *compartment;
-
-      rv = xpc_CreateGlobalObject(cx, &gSharedGlobalClass, principal, nsnull,
-                                  false, &newGlob, &compartment);
-      NS_ENSURE_SUCCESS(rv, nsnull);
-
-      ::JS_SetGlobalObject(cx, newGlob);
-
-      // Add an owning reference from JS back to us. This'll be
-      // released when the JSObject is finalized.
-      ::JS_SetPrivate(cx, newGlob, this);
-      NS_ADDREF(this);
-    }
-
-    NS_ENSURE_SUCCESS(rv, nsnull);
-    rv = SetScriptContext(lang_id, ctxNew);
-    NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ABORT_IF_FALSE(lang_id == nsIProgrammingLanguage::JAVASCRIPT,
+                    "We don't support this language ID");
+  if (mContext) {
+    return NS_OK;
   }
+  NS_ASSERTION(!mJSObject, "Have global without context?");
+
+  nsCOMPtr<nsIScriptRuntime> languageRuntime;
+  nsresult rv = NS_GetScriptRuntimeByID(nsIProgrammingLanguage::JAVASCRIPT,
+                                        getter_AddRefs(languageRuntime));
+  NS_ENSURE_SUCCESS(rv, NS_OK);
+
+  nsCOMPtr<nsIScriptContext> ctxNew = languageRuntime->CreateContext();
+  // We have to setup a special global object.  We do this then
+  // attach it as the global for this context.  Then, ::SetScriptContext
+  // will re-fetch the global and set it up in our language globals array.
+  {
+    JSContext *cx = ctxNew->GetNativeContext();
+    JSAutoRequest ar(cx);
+
+    nsIPrincipal *principal = GetPrincipal();
+    JSObject *newGlob;
+    JSCompartment *compartment;
+
+    rv = xpc_CreateGlobalObject(cx, &gSharedGlobalClass, principal, nsnull,
+                                false, &newGlob, &compartment);
+    NS_ENSURE_SUCCESS(rv, NS_OK);
+
+    ::JS_SetGlobalObject(cx, newGlob);
+
+    // Add an owning reference from JS back to us. This'll be
+    // released when the JSObject is finalized.
+    ::JS_SetPrivate(cx, newGlob, this);
+    NS_ADDREF(this);
+  }
+
+  NS_ENSURE_SUCCESS(rv, NS_OK);
+  rv = SetScriptContext(lang_id, ctxNew);
+  NS_ENSURE_SUCCESS(rv, NS_OK);
   return NS_OK;
 }
 
-nsIScriptContext *
+nsIScriptContext*
 nsXULPDGlobalObject::GetScriptContext(PRUint32 lang_id)
 {
+  NS_ABORT_IF_FALSE(lang_id == nsIProgrammingLanguage::JAVASCRIPT,
+                    "We don't support this language ID");
   // This global object creates a context on demand - do that now.
-  nsresult rv = EnsureScriptEnvironment(lang_id);
+  nsresult rv = EnsureScriptEnvironment(nsIProgrammingLanguage::JAVASCRIPT);
   if (NS_FAILED(rv)) {
     NS_ERROR("Failed to setup script language");
-    return nsnull;
+    return NULL;
   }
   // Note that EnsureScriptEnvironment has validated lang_id
-  return mScriptContexts[NS_STID_INDEX(lang_id)];
+  return mContext;
 }
 
-void *
-nsXULPDGlobalObject::GetScriptGlobal(PRUint32 lang_id)
+JSObject*
+nsXULPDGlobalObject::GetGlobalJSObject()
 {
-  PRBool ok = NS_STID_VALID(lang_id);
-  NS_ASSERTION(ok, "Invalid programming language ID requested");
-  NS_ENSURE_TRUE(ok, nsnull);
-  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
-
-  NS_ASSERTION(mScriptContexts[lang_ndx] != nsnull, "Querying for global before setting up context?");
-  return mScriptGlobals[lang_ndx];
+  return mJSObject;
 }
 
 
 void
 nsXULPDGlobalObject::ClearGlobalObjectOwner()
 {
-    NS_ASSERTION(!mCachedPrincipal, "This shouldn't ever be set until now!");
+  NS_ASSERTION(!mCachedPrincipal, "This shouldn't ever be set until now!");
 
-    // Cache mGlobalObjectOwner's principal if possible.
-    if (this != nsXULPrototypeDocument::gSystemGlobal)
-        mCachedPrincipal = mGlobalObjectOwner->DocumentPrincipal();
+  // Cache mGlobalObjectOwner's principal if possible.
+  if (this != nsXULPrototypeDocument::gSystemGlobal)
+    mCachedPrincipal = mGlobalObjectOwner->DocumentPrincipal();
 
-    PRUint32 lang_ndx;
-    NS_STID_FOR_INDEX(lang_ndx) {
-        if (mScriptContexts[lang_ndx]) {
-            mScriptContexts[lang_ndx]->FinalizeContext();
-            mScriptContexts[lang_ndx] = nsnull;
-        }
-    }
+  if (mContext) {
+    mContext->FinalizeContext();
+    mContext = NULL;
+  }
 
-    mGlobalObjectOwner = nsnull;
+  mGlobalObjectOwner = NULL;
 }
 
 
 void
-nsXULPDGlobalObject::OnFinalize(PRUint32 aLangID, void *aObject)
+nsXULPDGlobalObject::OnFinalize(JSObject* aObject)
 {
-    NS_ASSERTION(NS_STID_VALID(aLangID), "Invalid language ID");
-    NS_ASSERTION(aObject == mScriptGlobals[NS_STID_INDEX(aLangID)],
-                 "Wrong object finalized!");
-    mScriptGlobals[NS_STID_INDEX(aLangID)] = nsnull;
+  mJSObject = NULL;
 }
 
 void
-nsXULPDGlobalObject::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
+nsXULPDGlobalObject::SetScriptsEnabled(bool aEnabled, bool aFireTimeouts)
 {
-    // We don't care...
+  // We don't care...
 }
 
 //----------------------------------------------------------------------
