@@ -41,6 +41,7 @@
 #include "nsWeakReference.h"
 #include "nsIObserver.h"
 #include "nsIContent.h"
+#include "nsIWidget.h"
 
 #define FOCUSMETHOD_MASK 0xF000
 #define FOCUSMETHODANDRING_MASK 0xF0F000
@@ -67,6 +68,8 @@ class nsFocusManager : public nsIFocusManager,
                        public nsIObserver,
                        public nsSupportsWeakReference
 {
+  typedef mozilla::widget::InputContextAction InputContextAction;
+
 public:
 
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsFocusManager, nsIFocusManager)
@@ -131,11 +134,9 @@ public:
   static nsIContent* GetRedirectedFocus(nsIContent* aContent);
 
   /**
-   * Returns a flag indicating the source and/or reason of the focus change.
-   * This is used to indicate to the IME code if the focus come from a user 
-   * input or a script for example.
+   * Returns an InputContextAction cause for aFlags.
    */
-  static PRUint32 GetFocusMoveReason(PRUint32 aFlags);
+  static InputContextAction::Cause GetFocusMoveActionCause(PRUint32 aFlags);
 
   static bool sMouseFocusesFormControl;
 
@@ -526,6 +527,8 @@ private:
   // can access the document node, the caller should succeed in moving focus.
   nsCOMPtr<nsIDocument> mMouseDownEventHandlingDocument;
 
+  static bool sTestMode;
+
   // the single focus manager
   static nsFocusManager* sInstance;
 };
