@@ -40,10 +40,13 @@
 #ifndef mozilla_dom_indexeddb_indexeddatabasemanager_h__
 #define mozilla_dom_indexeddb_indexeddatabasemanager_h__
 
+#include "mozilla/dom/indexedDB/FileManager.h"
 #include "mozilla/dom/indexedDB/IndexedDatabase.h"
 #include "mozilla/dom/indexedDB/IDBDatabase.h"
 #include "mozilla/dom/indexedDB/IDBRequest.h"
 
+#include "mozilla/Mutex.h"
+
 #include "nsIIndexedDatabaseManager.h"
 #include "nsIObserver.h"
 #include "nsIRunnable.h"
@@ -51,6 +54,7 @@
 #include "nsIURI.h"
 
 #include "nsClassHashtable.h"
+#include "nsRefPtrHashtable.h"
 #include "nsHashKeys.h"
 
 #define INDEXEDDB_MANAGER_CONTRACTID "@mozilla.org/dom/indexeddb/manager;1"
@@ -62,6 +66,8 @@ BEGIN_INDEXEDDB_NAMESPACE
 
 class AsyncConnectionHelper;
 
+class CheckQuotaHelper;
+
 class IndexedDatabaseManager : public nsIIndexedDatabaseManager,
                                public nsIObserver
 {
@@ -98,6 +104,8 @@ public:
   // Returns true if we've begun the shutdown process.
   static bool IsShuttingDown();
 
+  static bool IsClosed();
+
   typedef void (*WaitingOnDatabasesCallback)(nsTArray<nsRefPtr<IDBDatabase> >&, void*);
 
   // Acquire exclusive access to the database given (waits for all others to
@@ -129,13 +137,69 @@ public:
   // Used to check if there are running transactions in a given window.
   bool HasOpenTransactions(nsPIDOMWindow* aWindow);
 
-  static bool
-  SetCurrentDatabase(IDBDatabase* aDatabase);
+  // Set the Window that the current thread is doing operations for.
+  // The caller is responsible for ensuring that aWindow is held alive.
+  static inline void
+  SetCurrentWindow(nsPIDOMWindow* aWindow)
+  {
+    IndexedDatabaseManager* mgr = Get();
+    NS_ASSERTION(mgr, "Must have a manager here!");
+
+    return mgr->SetCurrentWindowInternal(aWindow);
+  }
 
   static PRUint32
   GetIndexedDBQuotaMB();
 
-  nsresult EnsureQuotaManagementForDirectory(nsIFile* aDirectory);
+  nsresult EnsureOriginIsInitialized(const nsACString& aOrigin,
+                                     nsIFile** aDirectory);
+
+  // Determine if the quota is lifted for the Window the current thread is
+  // using.
+  static inline bool
+  QuotaIsLifted()
+  {
+    IndexedDatabaseManager* mgr = Get();
+    NS_ASSERTION(mgr, "Must have a manager here!");
+
+    return mgr->QuotaIsLiftedInternal();
+  }
+
+  static inline void
+  CancelPromptsForWindow(nsPIDOMWindow* aWindow)
+  {
+    IndexedDatabaseManager* mgr = Get();
+    NS_ASSERTION(mgr, "Must have a manager here!");
+
+    mgr->CancelPromptsForWindowInternal(aWindow);
+  }
+
+  static nsresult
+  GetASCIIOriginFromWindow(nsPIDOMWindow* aWindow, nsCString& aASCIIOrigin);
+
+  already_AddRefed<FileManager>
+  GetOrCreateFileManager(const nsACString& aOrigin,
+                         const nsAString& aDatabaseName);
+
+  already_AddRefed<FileManager>
+  GetFileManager(const nsACString& aOrigin,
+                 const nsAString& aDatabaseName);
+
+  void InvalidateFileManagersForOrigin(const nsACString& aOrigin);
+
+  void InvalidateFileManager(const nsACString& aOrigin,
+                             const nsAString& aDatabaseName);
+
+  nsresult AsyncDeleteFile(FileManager* aFileManager,
+                           PRInt64 aFileId);
+
+  static mozilla::Mutex& FileMutex()
+  {
+    IndexedDatabaseManager* mgr = Get();
+    NS_ASSERTION(mgr, "Must have a manager here!");
+
+    return mgr->mFileMutex;
+  }
 
 private:
   IndexedDatabaseManager();
@@ -147,6 +211,10 @@ private:
                                   WaitingOnDatabasesCallback aCallback,
                                   void* aClosure);
 
+  void SetCurrentWindowInternal(nsPIDOMWindow* aWindow);
+  bool QuotaIsLiftedInternal();
+  void CancelPromptsForWindowInternal(nsPIDOMWindow* aWindow);
+
   // Called when a database is created.
   bool RegisterDatabase(IDBDatabase* aDatabase);
 
@@ -210,10 +278,14 @@ private:
     // to the main thread in case of an error.
     inline nsresult RunInternal();
 
+    nsresult GetUsageForDirectory(nsIFile* aDirectory,
+                                  PRUint64* aUsage);
+
     nsCOMPtr<nsIURI> mURI;
     nsCString mOrigin;
     nsCOMPtr<nsIIndexedDatabaseUsageCallback> mCallback;
     PRUint64 mUsage;
+    PRUint64 mFileUsage;
     PRInt32 mCanceled;
   };
 
@@ -262,11 +334,39 @@ private:
     SynchronizedOp* mOp;
   };
 
+  class AsyncDeleteFileRunnable : public nsIRunnable
+  {
+  public:
+    NS_DECL_ISUPPORTS
+    NS_DECL_NSIRUNNABLE
+    AsyncDeleteFileRunnable(const nsAString& aFilePath)
+    : mFilePath(aFilePath)
+    { }
+
+  private:
+    nsString mFilePath;
+  };
+
   static nsresult DispatchHelper(AsyncConnectionHelper* aHelper);
 
   // Maintains a list of live databases per origin.
   nsClassHashtable<nsCStringHashKey, nsTArray<IDBDatabase*> > mLiveDatabases;
 
+  // TLS storage index for the current thread's window
+  PRUintn mCurrentWindowIndex;
+
+  // Lock protecting mQuotaHelperHash
+  mozilla::Mutex mQuotaHelperMutex;
+
+  // A map of Windows to the corresponding quota helper.
+  nsRefPtrHashtable<nsPtrHashKey<nsPIDOMWindow>, CheckQuotaHelper> mQuotaHelperHash;
+
+  // Maintains a list of all file managers per origin. The list is actually also
+  // a list of all origins that were successfully initialized. This list
+  // isn't protected by any mutex but it is only ever touched on the IO thread.
+  nsClassHashtable<nsCStringHashKey,
+                   nsTArray<nsRefPtr<FileManager> > > mFileManagers;
+
   // Maintains a list of origins that we're currently enumerating to gather
   // usage statistics.
   nsAutoTArray<nsRefPtr<AsyncUsageRunnable>, 1> mUsageRunnables;
@@ -284,10 +384,24 @@ private:
   // thread during GetOrCreate().
   nsCOMPtr<mozIStorageQuotaCallback> mQuotaCallbackSingleton;
 
-  // A list of all paths that are under SQLite's quota tracking system. This
-  // list isn't protected by any mutex but it is only ever touched on the IO
-  // thread.
-  nsTArray<nsCString> mTrackedQuotaPaths;
+  // Lock protecting FileManager.mFileInfos and nsDOMFileBase.mFileInfos
+  // It's s also used to atomically update FileInfo.mRefCnt, FileInfo.mDBRefCnt
+  // and FileInfo.mSliceRefCnt
+  mozilla::Mutex mFileMutex;
+};
+
+class AutoEnterWindow
+{
+public:
+  AutoEnterWindow(nsPIDOMWindow* aWindow)
+  {
+    IndexedDatabaseManager::SetCurrentWindow(aWindow);
+  }
+
+  ~AutoEnterWindow()
+  {
+    IndexedDatabaseManager::SetCurrentWindow(nsnull);
+  }
 };
 
 END_INDEXEDDB_NAMESPACE
