@@ -38,8 +38,8 @@
 
 #include "base/basictypes.h"
 
-// FIXME(bug 332648): Give me a real API please!
-#include "jscntxt.h"
+#include "jsapi.h"
+#include "jsfriendapi.h"
 
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsJSNPRuntime.h"
@@ -711,7 +711,7 @@ doInvoke(NPObject *npobj, NPIdentifier method, const NPVariant *args,
   JSBool ok;
 
   {
-    js::AutoArrayRooter tvr(cx, 0, jsargs);
+    JS::AutoArrayRooter tvr(cx, 0, jsargs);
 
     // Convert args
     for (PRUint32 i = 0; i < argCount; ++i) {
@@ -868,7 +868,7 @@ nsJSObjWrapper::NP_SetProperty(NPObject *npobj, NPIdentifier id,
     return false;
 
   jsval v = NPVariantToJSVal(npp, cx, value);
-  js::AutoValueRooter tvr(cx, v);
+  JS::AutoValueRooter tvr(cx, v);
 
   NS_ASSERTION(NPIdentifierIsInt(id) || NPIdentifierIsString(id),
                "id must be either string or int!\n");
@@ -964,25 +964,21 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **idarray,
   if (!ac.enter(cx, npjsobj->mJSObj))
     return false;
 
-  JSIdArray *ida = ::JS_Enumerate(cx, npjsobj->mJSObj);
+  JS::AutoIdArray ida(cx, JS_Enumerate(cx, npjsobj->mJSObj));
   if (!ida) {
     return false;
   }
 
-  *count = ida->length;
+  *count = ida.length();
   *idarray = (NPIdentifier *)PR_Malloc(*count * sizeof(NPIdentifier));
   if (!*idarray) {
     ThrowJSException(cx, "Memory allocation failed for NPIdentifier!");
-
-    ::JS_DestroyIdArray(cx, ida);
-
     return false;
   }
 
   for (PRUint32 i = 0; i < *count; i++) {
     jsval v;
-    if (!::JS_IdToValue(cx, ida->vector[i], &v)) {
-      ::JS_DestroyIdArray(cx, ida);
+    if (!JS_IdToValue(cx, ida[i], &v)) {
       PR_Free(*idarray);
       return false;
     }
@@ -991,10 +987,8 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **idarray,
     if (JSVAL_IS_STRING(v)) {
       JSString *str = JS_InternJSString(cx, JSVAL_TO_STRING(v));
       if (!str) {
-          ::JS_DestroyIdArray(cx, ida);
-          PR_Free(*idarray);
-
-          return false;
+        PR_Free(*idarray);
+        return false;
       }
       id = StringToNPIdentifier(cx, str);
     } else {
@@ -1006,8 +1000,6 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **idarray,
     (*idarray)[i] = id;
   }
 
-  ::JS_DestroyIdArray(cx, ida);
-
   return true;
 }
 
@@ -1483,7 +1475,7 @@ CallNPMethodInternal(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     // the function object.
 
     if (npobj->_class->invoke) {
-      JSFunction *fun = (JSFunction *)::JS_GetPrivate(cx, funobj);
+      JSFunction *fun = ::JS_GetObjectFunction(funobj);
       JSString *name = ::JS_InternJSString(cx, ::JS_GetFunctionId(fun));
       NPIdentifier id = StringToNPIdentifier(cx, name);
 
