@@ -94,6 +94,12 @@ class gfxXlibSurface;
 #include <os2.h>
 #endif
 
+#ifdef MOZ_WIDGET_ANDROID
+namespace mozilla {
+  class AndroidMediaLayer;
+}
+#endif
+
 // X.h defines KeyPress
 #ifdef KeyPress
 #undef KeyPress
@@ -136,7 +142,7 @@ public:
 
   nsresult Destroy();  
   
-  void PrepareToStop(PRBool aDelayedStop);
+  void PrepareToStop(bool aDelayedStop);
   
 #ifdef XP_WIN
   void Paint(const RECT& aDirty, HDC aDC);
@@ -154,7 +160,7 @@ public:
   
 #ifdef MAC_CARBON_PLUGINS
   void CancelTimer();
-  void StartTimer(PRBool isVisible);
+  void StartTimer(bool isVisible);
 #endif
   void SendIdleEvent();
   
@@ -178,7 +184,7 @@ public:
   enum { ePluginPaintEnable, ePluginPaintDisable };
   
   NPDrawingModel GetDrawingModel();
-  PRBool IsRemoteDrawingCoreAnimation();
+  bool IsRemoteDrawingCoreAnimation();
   NPEventModel GetEventModel();
   static void CARefresh(nsITimer *aTimer, void *aClosure);
   static void AddToCARefreshTimer(nsPluginInstanceOwner *aPluginInstance);
@@ -189,7 +195,7 @@ public:
   void HidePluginWindow();
   // Set a flag that (if true) indicates the plugin port info has changed and
   // SetWindow() needs to be called.
-  void SetPluginPortChanged(PRBool aState) { mPluginPortChanged = aState; }
+  void SetPluginPortChanged(bool aState) { mPluginPortChanged = aState; }
   // Return a pointer to the internal nsPluginPort structure that's used to
   // store a copy of plugin port info and to detect when it's been changed.
   void* GetPluginPortCopy();
@@ -206,9 +212,9 @@ public:
   void BeginCGPaint();
   void EndCGPaint();
 #else // XP_MACOSX
-  void UpdateWindowPositionAndClipRect(PRBool aSetWindow);
-  void UpdateWindowVisibility(PRBool aVisible);
-  void UpdateDocumentActiveState(PRBool aIsActive);
+  void UpdateWindowPositionAndClipRect(bool aSetWindow);
+  void UpdateWindowVisibility(bool aVisible);
+  void UpdateDocumentActiveState(bool aIsActive);
 #endif // XP_MACOSX
   void CallSetWindow();
   
@@ -260,7 +266,7 @@ public:
   }
 #endif
   
-  PRBool SendNativeEvents()
+  bool SendNativeEvents()
   {
 #ifdef XP_WIN
     // XXX we should remove the plugin name check
@@ -268,20 +274,20 @@ public:
     (MatchPluginName("Shockwave Flash") ||
      MatchPluginName("Test Plug-in"));
 #elif defined(MOZ_X11) || defined(XP_MACOSX)
-    return PR_TRUE;
+    return true;
 #else
-    return PR_FALSE;
+    return false;
 #endif
   }
   
-  PRBool MatchPluginName(const char *aPluginName)
+  bool MatchPluginName(const char *aPluginName)
   {
     return strncmp(GetPluginName(), aPluginName, strlen(aPluginName)) == 0;
   }
   
   void NotifyPaintWaiter(nsDisplayListBuilder* aBuilder);
   // Return true if we set image with valid surface
-  PRBool SetCurrentImage(ImageContainer* aContainer);
+  bool SetCurrentImage(ImageContainer* aContainer);
   /**
    * Returns the bounds of the current async-rendered surface. This can only
    * change in response to messages received by the event loop (i.e. not during
@@ -296,12 +302,32 @@ public:
   already_AddRefed<gfxContext> BeginUpdateBackground(const nsIntRect& aRect);
   void EndUpdateBackground(gfxContext* aContext, const nsIntRect& aRect);
   
-  PRBool UseAsyncRendering();
+  bool UseAsyncRendering();
+
+#ifdef ANDROID
+  nsIntRect GetVisibleRect() {
+    return nsIntRect(0, 0, mPluginWindow->width, mPluginWindow->height);
+  }
+
+  void SetInverted(bool aInverted) {
+    mInverted = aInverted;
+  }
+
+  bool Inverted() {
+    return mInverted;
+  }
+
+  mozilla::AndroidMediaLayer* Layer() {
+    return mLayer;
+  }
+
+  void Invalidate();
+#endif
   
 private:
   
   // return FALSE if LayerSurface dirty (newly created and don't have valid plugin content yet)
-  PRBool IsUpToDate()
+  bool IsUpToDate()
   {
     nsIntSize size;
     return NS_SUCCEEDED(mInstance->GetImageSize(&size)) &&
@@ -310,7 +336,17 @@ private:
   
   void FixUpURLS(const nsString &name, nsAString &value);
 #ifdef ANDROID
+  void SendSize(int width, int height);
+  void SendOnScreenEvent(bool onScreen);
+
+  bool AddPluginView(const gfxRect& aRect);
   void RemovePluginView();
+
+  bool mOnScreen;
+  bool mInverted;
+
+  // For kOpenGL_ANPDrawingModel
+  mozilla::AndroidMediaLayer *mLayer;
 #endif 
  
   nsPluginNativeWindow       *mPluginWindow;
@@ -333,31 +369,31 @@ private:
   CGColorSpaceRef                           mColorProfile;
   static nsCOMPtr<nsITimer>                *sCATimer;
   static nsTArray<nsPluginInstanceOwner*>  *sCARefreshListeners;
-  PRBool                                    mSentInitialTopLevelWindowEvent;
+  bool                                      mSentInitialTopLevelWindowEvent;
 #endif
   // We need to know if async hide window is required since we
   // can not check UseAsyncRendering when executing StopPlugin
-  PRBool                                    mAsyncHidePluginWindow;
+  bool                                      mAsyncHidePluginWindow;
   
   // Initially, the event loop nesting level we were created on, it's updated
   // if we detect the appshell is on a lower level as long as we're not stopped.
   // We delay DoStopPlugin() until the appshell reaches this level or lower.
   PRUint32                    mLastEventloopNestingLevel;
-  PRPackedBool                mContentFocused;
-  PRPackedBool                mWidgetVisible;    // used on Mac to store our widget's visible state
+  bool                        mContentFocused;
+  bool                        mWidgetVisible;    // used on Mac to store our widget's visible state
 #ifdef XP_MACOSX
-  PRPackedBool                mPluginPortChanged;
+  bool                        mPluginPortChanged;
 #endif
 #ifdef MOZ_X11
   // Used with windowless plugins only, initialized in CreateWidget().
-  PRPackedBool                mFlash10Quirks;
+  bool                        mFlash10Quirks;
 #endif
-  PRPackedBool                mPluginWindowVisible;
-  PRPackedBool                mPluginDocumentActiveState;
+  bool                        mPluginWindowVisible;
+  bool                        mPluginDocumentActiveState;
   
   // If true, destroy the widget on destruction. Used when plugin stop
   // is being delayed to a safer point in time.
-  PRPackedBool                mDestroyWidget;
+  bool                        mDestroyWidget;
   PRUint16          mNumCachedAttrs;
   PRUint16          mNumCachedParams;
   char              **mCachedAttrParamNames;
@@ -400,7 +436,7 @@ private:
   };
 #endif
 
-  PRPackedBool mWaitingForPaint;
+  bool mWaitingForPaint;
 };
 
 #endif // nsPluginInstanceOwner_h_
