@@ -105,6 +105,7 @@ public:
 
 class nsDOMStorageManager : public nsIDOMStorageManager
                           , public nsIObserver
+                          , public nsSupportsWeakReference
 {
 public:
   // nsISupports
@@ -128,6 +129,7 @@ public:
   static nsresult Initialize();
   static nsDOMStorageManager* GetInstance();
   static void Shutdown();
+  static void ShutdownDB();
 
   /**
    * Checks whether there is any data waiting to be flushed from a temp table.
@@ -420,7 +422,7 @@ public:
   nsDOMStorageType mStorageType;
 
   friend class nsIDOMStorage2;
-  nsPIDOMStorage* mSecurityChecker;
+  nsCOMPtr<nsIPrincipal> mPrincipal;
   nsPIDOMStorage* mEventBroadcaster;
 };
 
