@@ -67,7 +67,7 @@ AllKeyEnum(nsSessionStorageEntry* aEntry, void* userArg)
   aEntry->mItem->GetValue(item->mValue);
   nsresult rv = aEntry->mItem->GetSecure(&item->mSecure);
   if (NS_FAILED(rv))
-    item->mSecure = PR_FALSE;
+    item->mSecure = false;
 
   target->Put(aEntry->GetKey(), item);
   return PL_DHASH_NEXT;
@@ -100,9 +100,9 @@ nsDOMStorageMemoryDB::GetItemsTable(DOMStorageImpl* aStorage,
     rv = mPreloadDB->GetAllKeys(aStorage, &keys);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    mPreloading = PR_TRUE;
+    mPreloading = true;
     keys.EnumerateEntries(AllKeyEnum, &storageData->mTable);
-    mPreloading = PR_FALSE;
+    mPreloading = false;
   }
 
   mData.Put(aStorage->GetScopeDBKey(), storageData);
@@ -160,7 +160,7 @@ nsresult
 nsDOMStorageMemoryDB::GetKeyValue(DOMStorageImpl* aStorage,
                                   const nsAString& aKey,
                                   nsAString& aValue,
-                                  PRBool* aSecure)
+                                  bool* aSecure)
 {
   if (mPreloading) {
     NS_PRECONDITION(mPreloadDB, "Must have a preload DB set when preloading");
@@ -186,9 +186,9 @@ nsresult
 nsDOMStorageMemoryDB::SetKey(DOMStorageImpl* aStorage,
                              const nsAString& aKey,
                              const nsAString& aValue,
-                             PRBool aSecure,
+                             bool aSecure,
                              PRInt32 aQuota,
-                             PRBool aExcludeOfflineFromUsage,
+                             bool aExcludeOfflineFromUsage,
                              PRInt32 *aNewUsage)
 {
   nsresult rv;
@@ -243,7 +243,7 @@ nsDOMStorageMemoryDB::SetKey(DOMStorageImpl* aStorage,
 nsresult
 nsDOMStorageMemoryDB::SetSecure(DOMStorageImpl* aStorage,
                                 const nsAString& aKey,
-                                const PRBool aSecure)
+                                const bool aSecure)
 {
   nsresult rv;
 
@@ -265,7 +265,7 @@ nsDOMStorageMemoryDB::SetSecure(DOMStorageImpl* aStorage,
 nsresult
 nsDOMStorageMemoryDB::RemoveKey(DOMStorageImpl* aStorage,
                                 const nsAString& aKey,
-                                PRBool aExcludeOfflineFromUsage,
+                                bool aExcludeOfflineFromUsage,
                                 PRInt32 aKeyUsage)
 {
   nsresult rv;
@@ -325,7 +325,7 @@ nsDOMStorageMemoryDB::DropStorage(DOMStorageImpl* aStorage)
 struct RemoveOwnersStruc
 {
   nsCString* mSubDomain;
-  PRBool mMatch;
+  bool mMatch;
 };
 
 static PLDHashOperator
@@ -343,7 +343,7 @@ RemoveOwnersEnum(const nsACString& key,
 
 nsresult
 nsDOMStorageMemoryDB::RemoveOwner(const nsACString& aOwner,
-                                  PRBool aIncludeSubDomains)
+                                  bool aIncludeSubDomains)
 {
   nsCAutoString subdomainsDBKey;
   nsDOMStorageDBWrapper::CreateDomainScopeDBKey(aOwner, subdomainsDBKey);
@@ -353,7 +353,7 @@ nsDOMStorageMemoryDB::RemoveOwner(const nsACString& aOwner,
 
   RemoveOwnersStruc struc;
   struc.mSubDomain = &subdomainsDBKey;
-  struc.mMatch = PR_TRUE;
+  struc.mMatch = true;
   mData.Enumerate(RemoveOwnersEnum, &struc);
 
   MarkAllScopesDirty();
@@ -364,8 +364,8 @@ nsDOMStorageMemoryDB::RemoveOwner(const nsACString& aOwner,
 
 nsresult
 nsDOMStorageMemoryDB::RemoveOwners(const nsTArray<nsString> &aOwners,
-                                   PRBool aIncludeSubDomains,
-                                   PRBool aMatch)
+                                   bool aIncludeSubDomains,
+                                   bool aMatch)
 {
   if (aOwners.Length() == 0) {
     if (aMatch) {
@@ -407,7 +407,7 @@ nsDOMStorageMemoryDB::RemoveAll()
 
 nsresult
 nsDOMStorageMemoryDB::GetUsage(DOMStorageImpl* aStorage,
-                               PRBool aExcludeOfflineFromUsage, PRInt32 *aUsage)
+                               bool aExcludeOfflineFromUsage, PRInt32 *aUsage)
 {
   return GetUsageInternal(aStorage->GetQuotaDomainDBKey(!aExcludeOfflineFromUsage),
                           aExcludeOfflineFromUsage, aUsage);
@@ -415,7 +415,7 @@ nsDOMStorageMemoryDB::GetUsage(DOMStorageImpl* aStorage,
 
 nsresult
 nsDOMStorageMemoryDB::GetUsage(const nsACString& aDomain,
-                               PRBool aIncludeSubDomains,
+                               bool aIncludeSubDomains,
                                PRInt32 *aUsage)
 {
   nsresult rv;
@@ -423,11 +423,11 @@ nsDOMStorageMemoryDB::GetUsage(const nsACString& aDomain,
   nsCAutoString quotadomainDBKey;
   rv = nsDOMStorageDBWrapper::CreateQuotaDomainDBKey(aDomain,
                                                      aIncludeSubDomains,
-                                                     PR_FALSE,
+                                                     false,
                                                      quotadomainDBKey);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return GetUsageInternal(quotadomainDBKey, PR_FALSE, aUsage);
+  return GetUsageInternal(quotadomainDBKey, false, aUsage);
 }
 
 struct GetUsageEnumStruc
@@ -460,7 +460,7 @@ GetUsageEnum(const nsACString& key,
 
 nsresult
 nsDOMStorageMemoryDB::GetUsageInternal(const nsACString& aQuotaDomainDBKey,
-                                       PRBool aExcludeOfflineFromUsage,
+                                       bool aExcludeOfflineFromUsage,
                                        PRInt32 *aUsage)
 {
   GetUsageEnumStruc struc;
