@@ -46,6 +46,7 @@
 #include "nsTHashtable.h"
 #include "nsDataHashtable.h"
 #include "mozilla/TimeStamp.h"
+#include "mozilla/storage/StatementCache.h"
 
 class DOMStorageImpl;
 class nsSessionStorageEntry;
@@ -55,6 +56,8 @@ using mozilla::TimeDuration;
 
 class nsDOMStoragePersistentDB : public nsDOMStorageBaseDB
 {
+  typedef mozilla::storage::StatementCache<mozIStorageStatement> StatementCache;
+
 public:
   nsDOMStoragePersistentDB();
   ~nsDOMStoragePersistentDB() {}
@@ -62,6 +65,12 @@ public:
   nsresult
   Init(const nsString& aDatabaseName);
 
+  /**
+   * Close the connection, finalizing all the cached statements.
+   */
+  void
+  Close();
+
   /**
    * Retrieve a list of all the keys associated with a particular domain.
    */
@@ -78,7 +87,7 @@ public:
   GetKeyValue(DOMStorageImpl* aStorage,
               const nsAString& aKey,
               nsAString& aValue,
-              PRBool* aSecure);
+              bool* aSecure);
 
   /**
    * Set the value and secure flag for a key in storage.
@@ -87,9 +96,9 @@ public:
   SetKey(DOMStorageImpl* aStorage,
          const nsAString& aKey,
          const nsAString& aValue,
-         PRBool aSecure,
+         bool aSecure,
          PRInt32 aQuota,
-         PRBool aExcludeOfflineFromUsage,
+         bool aExcludeOfflineFromUsage,
          PRInt32* aNewUsage);
 
   /**
@@ -99,7 +108,7 @@ public:
   nsresult
   SetSecure(DOMStorageImpl* aStorage,
             const nsAString& aKey,
-            const PRBool aSecure);
+            const bool aSecure);
 
   /**
    * Removes a key from storage.
@@ -107,7 +116,7 @@ public:
   nsresult
   RemoveKey(DOMStorageImpl* aStorage,
             const nsAString& aKey,
-            PRBool aExcludeOfflineFromUsage,
+            bool aExcludeOfflineFromUsage,
             PRInt32 aKeyUsage);
 
   /**
@@ -119,7 +128,7 @@ public:
    * Removes all keys added by a given domain.
    */
   nsresult
-  RemoveOwner(const nsACString& aOwner, PRBool aIncludeSubDomains);
+  RemoveOwner(const nsACString& aOwner, bool aIncludeSubDomains);
 
   /**
    * Removes keys owned by domains that either match or don't match the
@@ -127,7 +136,7 @@ public:
    */
   nsresult
   RemoveOwners(const nsTArray<nsString>& aOwners,
-               PRBool aIncludeSubDomains, PRBool aMatch);
+               bool aIncludeSubDomains, bool aMatch);
 
   /**
    * Removes all keys from storage. Used when clearing storage.
@@ -139,13 +148,13 @@ public:
     * Returns usage for a storage using its GetQuotaDomainDBKey() as a key.
     */
   nsresult
-  GetUsage(DOMStorageImpl* aStorage, PRBool aExcludeOfflineFromUsage, PRInt32 *aUsage);
+  GetUsage(DOMStorageImpl* aStorage, bool aExcludeOfflineFromUsage, PRInt32 *aUsage);
 
   /**
     * Returns usage of the domain and optionaly by any subdomain.
     */
   nsresult
-  GetUsage(const nsACString& aDomain, PRBool aIncludeSubDomains, PRInt32 *aUsage);
+  GetUsage(const nsACString& aDomain, bool aIncludeSubDomains, PRInt32 *aUsage);
 
   /**
    * Clears all in-memory data from private browsing mode
@@ -185,20 +194,7 @@ protected:
                                              void* aUserArg);       
 
   nsCOMPtr<mozIStorageConnection> mConnection;
-
-  nsCOMPtr<mozIStorageStatement> mCopyToTempTableStatement;
-  nsCOMPtr<mozIStorageStatement> mCopyBackToDiskStatement;
-  nsCOMPtr<mozIStorageStatement> mDeleteTemporaryTableStatement;
-  nsCOMPtr<mozIStorageStatement> mGetAllKeysStatement;
-  nsCOMPtr<mozIStorageStatement> mGetKeyValueStatement;
-  nsCOMPtr<mozIStorageStatement> mInsertKeyStatement;
-  nsCOMPtr<mozIStorageStatement> mSetSecureStatement;
-  nsCOMPtr<mozIStorageStatement> mRemoveKeyStatement;
-  nsCOMPtr<mozIStorageStatement> mRemoveOwnerStatement;
-  nsCOMPtr<mozIStorageStatement> mRemoveStorageStatement;
-  nsCOMPtr<mozIStorageStatement> mRemoveAllStatement;
-  nsCOMPtr<mozIStorageStatement> mGetOfflineExcludedUsageStatement;
-  nsCOMPtr<mozIStorageStatement> mGetFullUsageStatement;
+  StatementCache mStatements;
 
   nsCString mCachedOwner;
   PRInt32 mCachedUsage;
@@ -211,7 +207,12 @@ protected:
   friend class nsDOMStorageDBWrapper;
   friend class nsDOMStorageMemoryDB;
   nsresult
-  GetUsageInternal(const nsACString& aQuotaDomainDBKey, PRBool aExcludeOfflineFromUsage, PRInt32 *aUsage);
+  GetUsageInternal(const nsACString& aQuotaDomainDBKey, bool aExcludeOfflineFromUsage, PRInt32 *aUsage);
+
+  // Compares aDomain with the mCachedOwner and returns false if changes
+  // in aDomain don't affect mCachedUsage.
+  bool DomainMaybeCached(const nsACString& aDomain);
+
 };
 
 #endif /* nsDOMStorageDB_h___ */
