@@ -37,11 +37,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Util.h"
+
 #include "Events.h"
 
 #include "jsapi.h"
-#include "jscntxt.h"
-#include "jsobj.h"
+#include "jsfriendapi.h"
 
 #include "nsTraceRefcnt.h"
 
@@ -57,6 +58,7 @@
 #define CONSTANT_FLAGS \
   JSPROP_ENUMERATE | JSPROP_SHARED | JSPROP_PERMANENT | JSPROP_READONLY
 
+using namespace mozilla;
 USING_WORKERS_NAMESPACE
 
 namespace {
@@ -72,6 +74,7 @@ class Event : public PrivatizableBase
 
 protected:
   bool mStopPropagationCalled;
+  bool mStopImmediatePropagationCalled;
 
 public:
   static bool
@@ -165,9 +168,16 @@ public:
     return JSVAL_TO_BOOLEAN(canceled);
   }
 
+  static bool
+  ImmediatePropagationStopped(JSContext* aCx, JSObject* aEvent)
+  {
+    Event* event = GetPrivate(aCx, aEvent);
+    return event ? event->mStopImmediatePropagationCalled : false;
+  }
+
 protected:
   Event()
-  : mStopPropagationCalled(false)
+  : mStopPropagationCalled(false), mStopImmediatePropagationCalled(false)
   {
     MOZ_COUNT_CTOR(mozilla::dom::workers::Event);
   }
@@ -227,6 +237,7 @@ protected:
                   bool aIsTrusted)
   {
     aEvent->mStopPropagationCalled = false;
+    aEvent->mStopImmediatePropagationCalled = false;
 
     jsval now;
     if (!JS_NewNumberValue(aCx, JS_Now(), &now)) {
@@ -305,6 +316,9 @@ private:
   StopPropagation(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     Event* event = GetInstancePrivate(aCx, obj, sFunctions[0].name);
     if (!event) {
@@ -316,10 +330,31 @@ private:
     return true;
   }
 
+  static JSBool
+  StopImmediatePropagation(JSContext* aCx, uintN aArgc, jsval* aVp)
+  {
+    JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
+
+    Event* event = GetInstancePrivate(aCx, obj, sFunctions[3].name);
+    if (!event) {
+      return false;
+    }
+
+    event->mStopImmediatePropagationCalled = true;
+
+    return true;
+  }
+  
   static JSBool
   PreventDefault(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     Event* event = GetInstancePrivate(aCx, obj, sFunctions[1].name);
     if (!event) {
@@ -340,6 +375,9 @@ private:
   InitEvent(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     Event* event = GetInstancePrivate(aCx, obj, sFunctions[2].name);
     if (!event) {
@@ -396,6 +434,7 @@ JSFunctionSpec Event::sFunctions[] = {
   JS_FN("stopPropagation", StopPropagation, 0, FUNCTION_FLAGS),
   JS_FN("preventDefault", PreventDefault, 0, FUNCTION_FLAGS),
   JS_FN("initEvent", InitEvent, 3, FUNCTION_FLAGS),
+  JS_FN("stopImmediatePropagation", StopImmediatePropagation, 0, FUNCTION_FLAGS),
   JS_FS_END
 };
 
@@ -415,7 +454,7 @@ class MessageEvent : public Event
   static JSFunctionSpec sFunctions[];
 
 protected:
-  uint64* mData;
+  uint64_t* mData;
   size_t mDataByteCount;
   nsTArray<nsCOMPtr<nsISupports> > mClonedObjects;
   bool mMainRuntime;
@@ -599,6 +638,9 @@ private:
   InitMessageEvent(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     MessageEvent* event = GetInstancePrivate(aCx, obj, sFunctions[0].name);
     if (!event) {
@@ -795,6 +837,9 @@ private:
   InitErrorEvent(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     ErrorEvent* event = GetInstancePrivate(aCx, obj, sFunctions[0].name);
     if (!event) {
@@ -803,7 +848,7 @@ private:
 
     JSString* type, *message, *filename;
     JSBool bubbles, cancelable;
-    uint32 lineNumber;
+    uint32_t lineNumber;
     if (!JS_ConvertArguments(aCx, aArgc, JS_ARGV(aCx, aVp), "SbbSSu", &type,
                              &bubbles, &cancelable, &message, &filename,
                              &lineNumber)) {
@@ -982,6 +1027,9 @@ private:
   InitProgressEvent(JSContext* aCx, uintN aArgc, jsval* aVp)
   {
     JSObject* obj = JS_THIS_OBJECT(aCx, aVp);
+    if (!obj) {
+      return false;
+    }
 
     ProgressEvent* event = GetInstancePrivate(aCx, obj, sFunctions[0].name);
     if (!event) {
@@ -1112,6 +1160,12 @@ EventWasCanceled(JSContext* aCx, JSObject* aEvent)
   return Event::WasCanceled(aCx, aEvent);
 }
 
+bool
+EventImmediatePropagationStopped(JSContext* aCx, JSObject* aEvent)
+{
+  return Event::ImmediatePropagationStopped(aCx, aEvent);
+}
+
 bool
 DispatchEventToTarget(JSContext* aCx, JSObject* aTarget, JSObject* aEvent,
                       bool* aPreventDefaultCalled)
@@ -1126,7 +1180,7 @@ DispatchEventToTarget(JSContext* aCx, JSObject* aTarget, JSObject* aEvent,
   if (hasProperty) {
     jsval argv[] = { OBJECT_TO_JSVAL(aEvent) };
     jsval rval = JSVAL_VOID;
-    if (!JS_CallFunctionName(aCx, aTarget, kFunctionName, JS_ARRAY_LENGTH(argv),
+    if (!JS_CallFunctionName(aCx, aTarget, kFunctionName, ArrayLength(argv),
                              argv, &rval) ||
         !JS_ValueToBoolean(aCx, rval, &preventDefaultCalled)) {
       return false;
