@@ -64,7 +64,6 @@
 #include "nsIContent.h"
 #include "nsIContentIterator.h"
 #include "nsIDOMRange.h"
-#include "nsIDOMNSRange.h"
 #include "nsCOMArray.h"
 #include "nsIFile.h"
 #include "nsIURL.h"
@@ -140,7 +139,8 @@ static NS_DEFINE_CID(kCParserCID,     NS_PARSER_CID);
 #define NS_FOUND_TARGET NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_EDITOR, 3)
 
 // some little helpers
-static PRInt32 FindPositiveIntegerAfterString(const char *aLeadingString, nsCString &aCStr);
+static bool FindIntegerAfterString(const char *aLeadingString, 
+                                     nsCString &aCStr, PRInt32 &foundNumber);
 static nsresult RemoveFragComments(nsCString &theStr);
 static void RemoveBodyAndHead(nsIDOMNode *aNode);
 static nsresult FindTargetNode(nsIDOMNode *aStart, nsCOMPtr<nsIDOMNode> &aResult);
@@ -211,13 +211,11 @@ NS_IMETHODIMP nsHTMLEditor::LoadHTML(const nsAString & aInputString)
     res = selection->GetRangeAt(0, getter_AddRefs(range));
     NS_ENSURE_SUCCESS(res, res);
     NS_ENSURE_TRUE(range, NS_ERROR_NULL_POINTER);
-    nsCOMPtr<nsIDOMNSRange> nsrange (do_QueryInterface(range));
-    NS_ENSURE_TRUE(nsrange, NS_ERROR_NO_INTERFACE);
 
     // create fragment for pasted html
     nsCOMPtr<nsIDOMDocumentFragment> docfrag;
     {
-      res = nsrange->CreateContextualFragment(aInputString, getter_AddRefs(docfrag));
+      res = range->CreateContextualFragment(aInputString, getter_AddRefs(docfrag));
       NS_ENSURE_SUCCESS(res, res);
     }
     // put the fragment into the document
@@ -369,10 +367,6 @@ nsHTMLEditor::DoInsertHTMLWithContext(const nsAString & aInputString,
   if (nodeList.Count() == 0)
     return NS_OK;
 
-  // walk list of nodes; perform surgery on nodes (relativize) with _mozattr
-  res = RelativizeURIInFragmentList(nodeList, aFlavor, aSourceDoc, targetNode);
-  // ignore results from this call, try to paste/insert anyways
- 
   // are there any table elements in the list?  
   // node and offset for insertion
   nsCOMPtr<nsIDOMNode> parentNode;
@@ -893,92 +887,6 @@ nsHTMLEditor::GetAttributeToModifyOnNode(nsIDOMNode *aNode, nsAString &aAttr)
   return NS_OK;
 }
 
-nsresult
-nsHTMLEditor::RelativizeURIForNode(nsIDOMNode *aNode, nsIURL *aDestURL)
-{
-  nsAutoString attributeToModify;
-  GetAttributeToModifyOnNode(aNode, attributeToModify);
-  if (attributeToModify.IsEmpty())
-    return NS_OK;
-
-  nsCOMPtr<nsIDOMNamedNodeMap> attrMap;
-  nsresult rv = aNode->GetAttributes(getter_AddRefs(attrMap));
-  NS_ENSURE_SUCCESS(rv, NS_OK);
-  NS_ENSURE_TRUE(attrMap, NS_OK); // assume errors here shouldn't cancel insertion
-
-  nsCOMPtr<nsIDOMNode> attrNode;
-  rv = attrMap->GetNamedItem(attributeToModify, getter_AddRefs(attrNode));
-  NS_ENSURE_SUCCESS(rv, NS_OK); // assume errors here shouldn't cancel insertion
-
-  if (attrNode)
-  {
-    nsAutoString oldValue;
-    attrNode->GetNodeValue(oldValue);
-    if (!oldValue.IsEmpty())
-    {
-      NS_ConvertUTF16toUTF8 oldCValue(oldValue);
-      nsCOMPtr<nsIURI> currentNodeURI;
-      rv = NS_NewURI(getter_AddRefs(currentNodeURI), oldCValue);
-      if (NS_SUCCEEDED(rv))
-      {
-        nsCAutoString newRelativePath;
-        aDestURL->GetRelativeSpec(currentNodeURI, newRelativePath);
-        if (!newRelativePath.IsEmpty())
-        {
-          NS_ConvertUTF8toUTF16 newCValue(newRelativePath);
-          attrNode->SetNodeValue(newCValue);
-        }
-      }
-    }
-  }
-
-  return NS_OK;
-}
-
-nsresult
-nsHTMLEditor::RelativizeURIInFragmentList(const nsCOMArray<nsIDOMNode> &aNodeList,
-                                          const nsAString &aFlavor,
-                                          nsIDOMDocument *aSourceDoc,
-                                          nsIDOMNode *aTargetNode)
-{
-  // determine destination URL
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  GetDocument(getter_AddRefs(domDoc));
-  NS_ENSURE_TRUE(domDoc, NS_ERROR_FAILURE);
-
-  nsCOMPtr<nsIDocument> destDoc = do_QueryInterface(domDoc);
-  NS_ENSURE_TRUE(destDoc, NS_ERROR_FAILURE);
-
-  nsCOMPtr<nsIURL> destURL = do_QueryInterface(destDoc->GetDocBaseURI());
-  NS_ENSURE_TRUE(destURL, NS_ERROR_FAILURE);
-
-  PRInt32 listCount = aNodeList.Count();
-  PRInt32 j;
-  for (j = 0; j < listCount; j++)
-  {
-    nsIDOMNode* somenode = aNodeList[j];
-
-    nsCOMPtr<nsIDOMTreeWalker> walker;
-    nsresult rv = domDoc->CreateTreeWalker(somenode,
-                                           nsIDOMNodeFilter::SHOW_ELEMENT,
-                                           nsnull, true,
-                                           getter_AddRefs(walker));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIDOMNode> currentNode;
-    walker->GetCurrentNode(getter_AddRefs(currentNode));
-    while (currentNode)
-    {
-      rv = RelativizeURIForNode(currentNode, destURL);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      walker->NextNode(getter_AddRefs(currentNode));
-    }
-  }
-
-  return NS_OK;
-}
-
 nsresult
 nsHTMLEditor::AddInsertionListener(nsIContentFilter *aListener)
 {
@@ -1148,22 +1056,24 @@ NS_IMETHODIMP nsHTMLEditor::PrepareHTMLTransferable(nsITransferable **aTransfera
   return NS_OK;
 }
 
-PRInt32
-FindPositiveIntegerAfterString(const char *aLeadingString, nsCString &aCStr)
+bool
+FindIntegerAfterString(const char *aLeadingString, 
+                       nsCString &aCStr, PRInt32 &foundNumber)
 {
   // first obtain offsets from cfhtml str
   PRInt32 numFront = aCStr.Find(aLeadingString);
   if (numFront == -1)
-    return -1;
+    return false;
   numFront += strlen(aLeadingString); 
   
   PRInt32 numBack = aCStr.FindCharInSet(CRLF, numFront);
   if (numBack == -1)
-    return -1;
+    return false;
    
   nsCAutoString numStr(Substring(aCStr, numFront, numBack-numFront));
   PRInt32 errorCode;
-  return numStr.ToInteger(&errorCode);
+  foundNumber = numStr.ToInteger(&errorCode);
+  return true;
 }
 
 nsresult
@@ -1190,14 +1100,36 @@ RemoveFragComments(nsCString & aStr)
 nsresult
 nsHTMLEditor::ParseCFHTML(nsCString & aCfhtml, PRUnichar **aStuffToPaste, PRUnichar **aCfcontext)
 {
-  // first obtain offsets from cfhtml str
-  PRInt32 startHTML     = FindPositiveIntegerAfterString("StartHTML:", aCfhtml);
-  PRInt32 endHTML       = FindPositiveIntegerAfterString("EndHTML:", aCfhtml);
-  PRInt32 startFragment = FindPositiveIntegerAfterString("StartFragment:", aCfhtml);
-  PRInt32 endFragment   = FindPositiveIntegerAfterString("EndFragment:", aCfhtml);
-
-  if ((startHTML<0) || (endHTML<0) || (startFragment<0) || (endFragment<0))
+  // First obtain offsets from cfhtml str.
+  PRInt32 startHTML, endHTML, startFragment, endFragment;
+  if (!FindIntegerAfterString("StartHTML:", aCfhtml, startHTML) || 
+      startHTML < -1)
     return NS_ERROR_FAILURE;
+  if (!FindIntegerAfterString("EndHTML:", aCfhtml, endHTML) || 
+      endHTML < -1) 
+    return NS_ERROR_FAILURE;
+  if (!FindIntegerAfterString("StartFragment:", aCfhtml, startFragment) || 
+      startFragment < 0) 
+    return NS_ERROR_FAILURE;
+  if (!FindIntegerAfterString("EndFragment:", aCfhtml, endFragment) || 
+      startFragment < 0)
+    return NS_ERROR_FAILURE;
+
+  // The StartHTML and EndHTML markers are allowed to be -1 to include everything.
+  //   See Reference: MSDN doc entitled "HTML Clipboard Format"
+  //   http://msdn.microsoft.com/en-us/library/aa767917(VS.85).aspx#unknown_854
+  if (startHTML == -1) {
+    startHTML = aCfhtml.Find("<!--StartFragment-->");
+    if (startHTML == -1)
+      return false;
+  }
+  if (endHTML == -1) {
+    const char endFragmentMarker[] = "<!--EndFragment-->";
+    endHTML = aCfhtml.Find(endFragmentMarker);
+    if (endHTML == -1)
+      return false;
+    endHTML += ArrayLength(endFragmentMarker) - 1;
+  }
 
   // create context string
   nsCAutoString contextUTF8(Substring(aCfhtml, startHTML, startFragment - startHTML) +
@@ -1629,11 +1561,7 @@ NS_IMETHODIMP nsHTMLEditor::InsertFromDrop(nsIDOMEvent* aDropEvent)
           if (NS_FAILED(rv) || !range) 
             continue;//don't bail yet, iterate through them all
 
-          nsCOMPtr<nsIDOMNSRange> nsrange(do_QueryInterface(range));
-          if (NS_FAILED(rv) || !nsrange) 
-            continue;//don't bail yet, iterate through them all
-
-          rv = nsrange->IsPointInRange(newSelectionParent, newSelectionOffset, &cursorIsInSelection);
+          rv = range->IsPointInRange(newSelectionParent, newSelectionOffset, &cursorIsInSelection);
           if(cursorIsInSelection)
             break;
         }
@@ -2728,10 +2656,7 @@ nsresult nsHTMLEditor::CreateListOfNodesToPaste(nsIDOMNode  *aFragmentAsNode,
     aEndOffset = fragLen;
   }
 
-  nsCOMPtr<nsIDOMRange> docFragRange =
-                          do_CreateInstance("@mozilla.org/content/range;1");
-  NS_ENSURE_TRUE(docFragRange, NS_ERROR_OUT_OF_MEMORY);
-
+  nsRefPtr<nsRange> docFragRange = new nsRange();
   res = docFragRange->SetStart(aStartNode, aStartOffset);
   NS_ENSURE_SUCCESS(res, res);
   res = docFragRange->SetEnd(aEndNode, aEndOffset);
