@@ -47,7 +47,6 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsISelection.h"
 #include "nsIDOMRange.h"
-#include "nsIDOMNSRange.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMHTMLTableElement.h"
 #include "nsIDOMHTMLTableCellElement.h"
@@ -129,7 +128,7 @@ nsHTMLEditorEventListener::MouseDown(nsIDOMEvent* aMouseEvent)
   nsresult res = mouseEvent->GetButton(&buttonNumber);
   NS_ENSURE_SUCCESS(res, res);
 
-  PRBool isContextClick = buttonNumber == 2;
+  bool isContextClick = buttonNumber == 2;
 
   PRInt32 clickCount;
   res = mouseEvent->GetDetail(&clickCount);
@@ -159,10 +158,10 @@ nsHTMLEditorEventListener::MouseDown(nsIDOMEvent* aMouseEvent)
     NS_ENSURE_SUCCESS(res, res);
 
     // Detect if mouse point is within current selection for context click
-    PRBool nodeIsInSelection = PR_FALSE;
+    bool nodeIsInSelection = false;
     if (isContextClick)
     {
-      PRBool isCollapsed;
+      bool isCollapsed;
       selection->GetIsCollapsed(&isCollapsed);
       if (!isCollapsed)
       {
@@ -178,11 +177,7 @@ nsHTMLEditorEventListener::MouseDown(nsIDOMEvent* aMouseEvent)
           if (NS_FAILED(res) || !range) 
             continue;//don't bail yet, iterate through them all
 
-          nsCOMPtr<nsIDOMNSRange> nsrange(do_QueryInterface(range));
-          if (NS_FAILED(res) || !nsrange) 
-            continue;//don't bail yet, iterate through them all
-
-          res = nsrange->IsPointInRange(parent, offset, &nodeIsInSelection);
+          res = range->IsPointInRange(parent, offset, &nodeIsInSelection);
 
           // Done when we find a range that we are in
           if (nodeIsInSelection)
