@@ -37,6 +37,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Util.h"
+
 #include "nscore.h"
 #include "nsLayoutCID.h"
 #include "nsIAtom.h"
@@ -47,7 +49,7 @@
 #include "nsIContentIterator.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMRange.h"
-#include "nsIRangeUtils.h"
+#include "nsContentUtils.h"
 #include "nsISelection.h"
 #include "nsIPlaintextEditor.h"
 #include "nsTextServicesDocument.h"
@@ -64,13 +66,14 @@
 #define LOCK_DOC(doc)
 #define UNLOCK_DOC(doc)
 
+using namespace mozilla;
 
 class OffsetEntry
 {
 public:
   OffsetEntry(nsIDOMNode *aNode, PRInt32 aOffset, PRInt32 aLength)
     : mNode(aNode), mNodeOffset(0), mStrOffset(aOffset), mLength(aLength),
-      mIsInsertedText(PR_FALSE), mIsValid(PR_TRUE)
+      mIsInsertedText(false), mIsValid(true)
   {
     if (mStrOffset < 1)
       mStrOffset = 0;
@@ -85,23 +88,21 @@ public:
     mNodeOffset = 0;
     mStrOffset  = 0;
     mLength     = 0;
-    mIsValid    = PR_FALSE;
+    mIsValid    = false;
   }
 
   nsIDOMNode *mNode;
   PRInt32 mNodeOffset;
   PRInt32 mStrOffset;
   PRInt32 mLength;
-  PRBool  mIsInsertedText;
-  PRBool  mIsValid;
+  bool    mIsInsertedText;
+  bool    mIsValid;
 };
 
 #define TS_ATOM(name_, value_) nsIAtom* nsTextServicesDocument::name_ = 0;
 #include "nsTSAtomList.h"
 #undef TS_ATOM
 
-nsIRangeUtils* nsTextServicesDocument::sRangeHelper;
-
 nsTextServicesDocument::nsTextServicesDocument()
 {
   mRefCnt         = 0;
@@ -133,14 +134,7 @@ nsTextServicesDocument::RegisterAtoms()
 #undef TS_ATOM
   };
 
-  NS_RegisterStaticAtoms(ts_atoms, NS_ARRAY_LENGTH(ts_atoms));
-}
-
-/* static */
-void
-nsTextServicesDocument::Shutdown()
-{
-  NS_IF_RELEASE(sRangeHelper);
+  NS_RegisterStaticAtoms(ts_atoms, ArrayLength(ts_atoms));
 }
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsTextServicesDocument)
@@ -347,7 +341,7 @@ nsTextServicesDocument::ExpandRangeToWordBoundaries(nsIDOMRange *aRange)
   {
     // We should never get here because a first text block
     // was found above.
-    NS_ASSERTION(PR_FALSE, "Found a first without a last!");
+    NS_ASSERTION(false, "Found a first without a last!");
     return NS_ERROR_FAILURE;
   }
 
@@ -553,7 +547,7 @@ nsTextServicesDocument::LastSelectedBlock(TSDBlockSelectionStatus *aSelStatus,
   }
 
   nsCOMPtr<nsISelection> selection;
-  PRBool isCollapsed = PR_FALSE;
+  bool isCollapsed = false;
 
   result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
@@ -669,7 +663,7 @@ nsTextServicesDocument::LastSelectedBlock(TSDBlockSelectionStatus *aSelStatus,
       }
 
       if (*aSelStatus == nsITextServicesDocument::eBlockContains)
-        result = SetSelectionInternal(*aSelOffset, *aSelLength, PR_FALSE);
+        result = SetSelectionInternal(*aSelOffset, *aSelLength, false);
     }
     else
     {
@@ -678,7 +672,7 @@ nsTextServicesDocument::LastSelectedBlock(TSDBlockSelectionStatus *aSelStatus,
       // position to the end of the document, then walk forwards
       // till you find a text node, then find the beginning of it's block.
 
-      result = CreateDocumentContentRootToNodeOffsetRange(parent, offset, PR_FALSE, getter_AddRefs(range));
+      result = CreateDocumentContentRootToNodeOffsetRange(parent, offset, false, getter_AddRefs(range));
 
       if (NS_FAILED(result))
       {
@@ -914,7 +908,7 @@ nsTextServicesDocument::LastSelectedBlock(TSDBlockSelectionStatus *aSelStatus,
     return result;
   }
 
-  result = CreateDocumentContentRootToNodeOffsetRange(parent, offset, PR_FALSE, getter_AddRefs(range));
+  result = CreateDocumentContentRootToNodeOffsetRange(parent, offset, false, getter_AddRefs(range));
 
   if (NS_FAILED(result))
   {
@@ -1155,17 +1149,17 @@ nsTextServicesDocument::NextBlock()
 }
 
 NS_IMETHODIMP
-nsTextServicesDocument::IsDone(PRBool *aIsDone)
+nsTextServicesDocument::IsDone(bool *aIsDone)
 {
   NS_ENSURE_TRUE(aIsDone, NS_ERROR_NULL_POINTER);
 
-  *aIsDone = PR_FALSE;
+  *aIsDone = false;
 
   NS_ENSURE_TRUE(mIterator, NS_ERROR_FAILURE);
 
   LOCK_DOC(this);
 
-  *aIsDone = (mIteratorStatus == nsTextServicesDocument::eIsDone) ? PR_TRUE : PR_FALSE;
+  *aIsDone = (mIteratorStatus == nsTextServicesDocument::eIsDone) ? true : false;
 
   UNLOCK_DOC(this);
 
@@ -1181,7 +1175,7 @@ nsTextServicesDocument::SetSelection(PRInt32 aOffset, PRInt32 aLength)
 
   LOCK_DOC(this);
 
-  result = SetSelectionInternal(aOffset, aLength, PR_TRUE);
+  result = SetSelectionInternal(aOffset, aLength, true);
 
   UNLOCK_DOC(this);
 
@@ -1310,7 +1304,7 @@ nsTextServicesDocument::DeleteSelection()
         // The entire entry is contained in the selection. Mark the
         // entry invalid.
 
-        entry->mIsValid = PR_FALSE;
+        entry->mIsValid = false;
       }
     }
 
@@ -1329,7 +1323,7 @@ nsTextServicesDocument::DeleteSelection()
         // of the selection is in an inserted text offset entry,
         // the selection includes the entire entry!
 
-        entry->mIsValid = PR_FALSE;
+        entry->mIsValid = false;
       }
       else
       {
@@ -1364,7 +1358,7 @@ nsTextServicesDocument::DeleteSelection()
           // The entire entry is contained in the selection. Mark the
           // entry invalid.
 
-          entry->mIsValid = PR_FALSE;
+          entry->mIsValid = false;
         }
       }
     }
@@ -1374,7 +1368,7 @@ nsTextServicesDocument::DeleteSelection()
       // The entire entry is contained in the selection. Mark the
       // entry invalid.
 
-      entry->mIsValid = PR_FALSE;
+      entry->mIsValid = false;
     }
   }
 
@@ -1532,7 +1526,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
   // retain as much of the original style of the content
   // being deleted.
 
-  PRBool collapsedSelection = SelectionIsCollapsed();
+  bool collapsedSelection = SelectionIsCollapsed();
   PRInt32 savedSelOffset = mSelStartOffset;
   PRInt32 savedSelLength = mSelEndOffset - mSelStartOffset;
 
@@ -1607,7 +1601,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
         return NS_ERROR_OUT_OF_MEMORY;
       }
 
-      itEntry->mIsInsertedText = PR_TRUE;
+      itEntry->mIsInsertedText = true;
       itEntry->mNodeOffset = entry->mNodeOffset;
 
       if (!mOffsetTable.InsertElementAt(mSelStartIndex, itEntry))
@@ -1661,7 +1655,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
       }
 
       itEntry->mNodeOffset = entry->mNodeOffset + entry->mLength;
-      itEntry->mIsInsertedText = PR_TRUE;
+      itEntry->mIsInsertedText = true;
 
       if (!mOffsetTable.InsertElementAt(i, itEntry))
       {
@@ -1722,7 +1716,7 @@ nsTextServicesDocument::InsertText(const nsString *aText)
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    itEntry->mIsInsertedText = PR_TRUE;
+    itEntry->mIsInsertedText = true;
     itEntry->mNodeOffset     = entry->mNodeOffset + entry->mLength;
 
     if (!mOffsetTable.InsertElementAt(mSelStartIndex + 1, itEntry))
@@ -1810,7 +1804,7 @@ nsTextServicesDocument::DidDeleteNode(nsIDOMNode *aChild, nsresult aResult)
   LOCK_DOC(this);
 
   PRInt32 nodeIndex = 0;
-  PRBool hasEntry = PR_FALSE;
+  bool hasEntry = false;
   OffsetEntry *entry;
 
   nsresult result = NodeHasOffsetEntry(&mOffsetTable, aChild, &hasEntry, &nodeIndex);
@@ -1858,7 +1852,7 @@ nsTextServicesDocument::DidDeleteNode(nsIDOMNode *aChild, nsresult aResult)
 
     if (entry->mNode == aChild)
     {
-      entry->mIsValid = PR_FALSE;
+      entry->mIsValid = false;
     }
 
     nodeIndex++;
@@ -1903,7 +1897,7 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
 
   result = aLeftNode->GetNodeType(&type);
 
-  NS_ENSURE_SUCCESS(result, PR_FALSE);
+  NS_ENSURE_SUCCESS(result, false);
 
   if (nsIDOMNode::TEXT_NODE != type)
   {
@@ -1913,7 +1907,7 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
 
   result = aRightNode->GetNodeType(&type);
 
-  NS_ENSURE_SUCCESS(result, PR_FALSE);
+  NS_ENSURE_SUCCESS(result, false);
 
   if (nsIDOMNode::TEXT_NODE != type)
   {
@@ -1926,8 +1920,8 @@ nsTextServicesDocument::DidJoinNodes(nsIDOMNode  *aLeftNode,
 
   PRInt32 leftIndex = 0;
   PRInt32 rightIndex = 0;
-  PRBool leftHasEntry = PR_FALSE;
-  PRBool rightHasEntry = PR_FALSE;
+  bool leftHasEntry = false;
+  bool rightHasEntry = false;
 
   result = NodeHasOffsetEntry(&mOffsetTable, aLeftNode, &leftHasEntry, &leftIndex);
 
@@ -2105,42 +2099,25 @@ nsTextServicesDocument::GetDocumentContentRootNode(nsIDOMNode **aNode)
 nsresult
 nsTextServicesDocument::CreateDocumentContentRange(nsIDOMRange **aRange)
 {
-  nsresult result;
-
-  NS_ENSURE_TRUE(aRange, NS_ERROR_NULL_POINTER);
-
-  *aRange = 0;
-
-  nsCOMPtr<nsIDOMNode>node;
-
-  result = GetDocumentContentRootNode(getter_AddRefs(node));
-
-  NS_ENSURE_SUCCESS(result, result);
+  *aRange = NULL;
 
+  nsCOMPtr<nsIDOMNode> node;
+  nsresult rv = GetDocumentContentRootNode(getter_AddRefs(node));
+  NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(node, NS_ERROR_NULL_POINTER);
 
-  result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  NS_ENSURE_SUCCESS(result, result);
+  nsRefPtr<nsRange> range = new nsRange();
 
-  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
-
-  result = (*aRange)->SelectNodeContents(node);
-
-  if (NS_FAILED(result))
-  {
-    NS_RELEASE((*aRange));
-    *aRange = 0;
-    return result;
-  }
+  rv = range->SelectNodeContents(node);
+  NS_ENSURE_SUCCESS(rv, rv);
 
+  range.forget(aRange);
   return NS_OK;
 }
 
 nsresult
-nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *aParent, PRInt32 aOffset, PRBool aToStart, nsIDOMRange **aRange)
+nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *aParent, PRInt32 aOffset, bool aToStart, nsIDOMRange **aRange)
 {
-  nsresult result;
-
   NS_ENSURE_TRUE(aParent && aRange, NS_ERROR_NULL_POINTER);
 
   *aRange = 0;
@@ -2151,71 +2128,47 @@ nsTextServicesDocument::CreateDocumentContentRootToNodeOffsetRange(nsIDOMNode *a
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMNode> bodyNode; 
-
-  result = GetDocumentContentRootNode(getter_AddRefs(bodyNode));
-
-  NS_ENSURE_SUCCESS(result, result);
-
+  nsresult rv = GetDocumentContentRootNode(getter_AddRefs(bodyNode));
+  NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(bodyNode, NS_ERROR_NULL_POINTER);
 
   nsCOMPtr<nsIDOMNode> startNode;
   nsCOMPtr<nsIDOMNode> endNode;
   PRInt32 startOffset, endOffset;
 
-  if (aToStart)
-  {
+  if (aToStart) {
     // The range should begin at the start of the document
     // and extend up until (aParent, aOffset).
 
     startNode   = bodyNode;
     startOffset = 0;
-    endNode     = do_QueryInterface(aParent);
+    endNode     = aParent;
     endOffset   = aOffset;
-  }
-  else
-  {
+  } else {
     // The range should begin at (aParent, aOffset) and
     // extend to the end of the document.
 
     nsCOMPtr<nsIDOMNodeList> nodeList;
     PRUint32 nodeListLength;
 
-    startNode   = do_QueryInterface(aParent);
+    startNode   = aParent;
     startOffset = aOffset;
     endNode     = bodyNode;
     endOffset   = 0;
 
-    result = bodyNode->GetChildNodes(getter_AddRefs(nodeList));
+    rv = bodyNode->GetChildNodes(getter_AddRefs(nodeList));
+    NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
 
-    NS_ENSURE_SUCCESS(result, NS_ERROR_FAILURE);
-
-    if (nodeList)
-    {
-      result = nodeList->GetLength(&nodeListLength);
-
-      NS_ENSURE_SUCCESS(result, NS_ERROR_FAILURE);
+    if (nodeList) {
+      rv = nodeList->GetLength(&nodeListLength);
+      NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
 
       endOffset = (PRInt32)nodeListLength;
     }
   }
 
-  result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  NS_ENSURE_SUCCESS(result, result);
-
-  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
-
-  result = (*aRange)->SetStart(startNode, startOffset);
-
-  if (NS_SUCCEEDED(result))
-    result = (*aRange)->SetEnd(endNode, endOffset);
-
-  if (NS_FAILED(result))
-  {
-    NS_RELEASE((*aRange));
-    *aRange = 0;
-  }
-
-  return result;
+  return nsRange::CreateRange(startNode, startOffset, endNode, endOffset,
+                              aRange);
 }
 
 nsresult
@@ -2252,7 +2205,7 @@ nsTextServicesDocument::AdjustContentIterator()
 
   nsIDOMNode *prevValidNode = 0;
   nsIDOMNode *nextValidNode = 0;
-  PRBool foundEntry = PR_FALSE;
+  bool foundEntry = false;
   OffsetEntry *entry;
 
   for (PRInt32 i = 0; i < tcount && !nextValidNode; i++)
@@ -2276,7 +2229,7 @@ nsTextServicesDocument::AdjustContentIterator()
         // the current iterator node. Stop looking for
         // a previous valid node!
 
-        foundEntry = PR_TRUE;
+        foundEntry = true;
       }
     }
 
@@ -2343,21 +2296,21 @@ nsTextServicesDocument::AdjustContentIterator()
   return NS_OK;
 }
 
-PRBool
+bool
 nsTextServicesDocument::DidSkip(nsIContentIterator* aFilteredIter)
 {
   // We can assume here that the Iterator is a nsFilteredContentIterator because
   // all the iterator are created in CreateContentIterator which create a 
   // nsFilteredContentIterator
   // So if the iterator bailed on one of the "filtered" content nodes then we 
-  // consider that to be a block and bail with PR_TRUE
+  // consider that to be a block and bail with true
   if (aFilteredIter) {
     nsFilteredContentIterator* filter = static_cast<nsFilteredContentIterator *>(aFilteredIter);
     if (filter && filter->DidSkip()) {
-      return PR_TRUE;
+      return true;
     }
   }
-  return PR_FALSE;
+  return false;
 }
 
 void
@@ -2370,12 +2323,12 @@ nsTextServicesDocument::ClearDidSkip(nsIContentIterator* aFilteredIter)
   }
 }
 
-PRBool
+bool
 nsTextServicesDocument::IsBlockNode(nsIContent *aContent)
 {
   if (!aContent) {
     NS_ERROR("How did a null pointer get passed to IsBlockNode?");
-    return PR_FALSE;
+    return false;
   }
 
   nsIAtom *atom = aContent->Tag();
@@ -2409,7 +2362,7 @@ nsTextServicesDocument::IsBlockNode(nsIContent *aContent)
           sWbrAtom     != atom);
 }
 
-PRBool
+bool
 nsTextServicesDocument::HasSameBlockNodeParent(nsIContent *aContent1, nsIContent *aContent2)
 {
   nsIContent* p1 = aContent1->GetParent();
@@ -2418,7 +2371,7 @@ nsTextServicesDocument::HasSameBlockNodeParent(nsIContent *aContent1, nsIContent
   // Quick test:
 
   if (p1 == p2)
-    return PR_TRUE;
+    return true;
 
   // Walk up the parent hierarchy looking for closest block boundary node:
 
@@ -2435,32 +2388,24 @@ nsTextServicesDocument::HasSameBlockNodeParent(nsIContent *aContent1, nsIContent
   return p1 == p2;
 }
 
-PRBool
+bool
 nsTextServicesDocument::IsTextNode(nsIContent *aContent)
 {
-  NS_ENSURE_TRUE(aContent, PR_FALSE);
-
-  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aContent);
-
-  return IsTextNode(node);
+  NS_ENSURE_TRUE(aContent, false);
+  return nsIDOMNode::TEXT_NODE == aContent->NodeType();
 }
 
-PRBool
+bool
 nsTextServicesDocument::IsTextNode(nsIDOMNode *aNode)
 {
-  NS_ENSURE_TRUE(aNode, PR_FALSE);
+  NS_ENSURE_TRUE(aNode, false);
 
-  PRUint16 type;
-
-  nsresult result = aNode->GetNodeType(&type);
-
-  NS_ENSURE_SUCCESS(result, PR_FALSE);
-
-  return nsIDOMNode::TEXT_NODE == type;
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
+  return IsTextNode(content);
 }
 
 nsresult
-nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, PRBool aDoUpdate)
+nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, bool aDoUpdate)
 {
   nsresult result = NS_OK;
 
@@ -2491,11 +2436,11 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
       }
       else if (aOffset >= entry->mStrOffset)
       {
-        PRBool foundEntry = PR_FALSE;
+        bool foundEntry = false;
         PRInt32 strEndOffset = entry->mStrOffset + entry->mLength;
 
         if (aOffset < strEndOffset)
-          foundEntry = PR_TRUE;
+          foundEntry = true;
         else if (aOffset == strEndOffset)
         {
           // Peek after this entry to see if we have any
@@ -2511,7 +2456,7 @@ nsTextServicesDocument::SetSelectionInternal(PRInt32 aOffset, PRInt32 aLength, P
             {
               // Next offset entry isn't an exact match, so we'll
               // just use the current entry.
-              foundEntry = PR_TRUE;
+              foundEntry = true;
             }
           }
         }
@@ -2629,7 +2574,7 @@ nsTextServicesDocument::GetSelection(nsITextServicesDocument::TSDBlockSelectionS
     return NS_OK;
 
   nsCOMPtr<nsISelection> selection;
-  PRBool isCollapsed;
+  bool isCollapsed;
 
   result = mSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
 
@@ -2714,13 +2659,10 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
 
   NS_ENSURE_SUCCESS(result, result);
 
-  result = ComparePoints(eStart->mNode, eStartOffset, parent, offset, &e1s1);
-
-  NS_ENSURE_SUCCESS(result, result);
-
-  result = ComparePoints(eEnd->mNode, eEndOffset, parent, offset, &e2s1);
-
-  NS_ENSURE_SUCCESS(result, result);
+  e1s1 = nsContentUtils::ComparePoints(eStart->mNode, eStartOffset,
+                                       parent, offset);
+  e2s1 = nsContentUtils::ComparePoints(eEnd->mNode, eEndOffset,
+                                       parent, offset);
 
   if (e1s1 > 0 || e2s1 < 0)
   {
@@ -2769,7 +2711,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
   nsCOMPtr<nsIDOMNode> node, saveNode;
   nsCOMPtr<nsIDOMNodeList> children;
   nsCOMPtr<nsIContentIterator> iter;
-  PRBool hasChildren;
+  bool hasChildren;
 
   result = CreateRange(eStart->mNode, eStartOffset, eEnd->mNode, eEndOffset, getter_AddRefs(range));
 
@@ -2938,7 +2880,7 @@ nsTextServicesDocument::GetCollapsedSelection(nsITextServicesDocument::TSDBlockS
       // In most cases, the user shouldn't see any movement in the caret
       // on screen.
 
-      result = SetSelectionInternal(*aSelOffset, *aSelLength, PR_TRUE);
+      result = SetSelectionInternal(*aSelOffset, *aSelLength, true);
 
       return result;
     }
@@ -3008,13 +2950,10 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
     NS_ENSURE_SUCCESS(result, result);
 
-    result = ComparePoints(eStart->mNode, eStartOffset, endParent, endOffset, &e1s2);
-
-    NS_ENSURE_SUCCESS(result, result);
-
-    result = ComparePoints(eEnd->mNode, eEndOffset, startParent, startOffset, &e2s1);
-
-    NS_ENSURE_SUCCESS(result, result);
+    e1s2 = nsContentUtils::ComparePoints(eStart->mNode, eStartOffset,
+                                         endParent, endOffset);
+    e2s1 = nsContentUtils::ComparePoints(eEnd->mNode, eEndOffset,
+                                         startParent, startOffset);
 
     // Break out of the loop if the text block intersects the current range.
 
@@ -3033,13 +2972,10 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   // Now that we have an intersecting range, find out more info:
 
-  result = ComparePoints(eStart->mNode, eStartOffset, startParent, startOffset, &e1s1);
-
-  NS_ENSURE_SUCCESS(result, result);
-
-  result = ComparePoints(eEnd->mNode, eEndOffset, endParent, endOffset, &e2s2);
-
-  NS_ENSURE_SUCCESS(result, result);
+  e1s1 = nsContentUtils::ComparePoints(eStart->mNode, eStartOffset,
+                                       startParent, startOffset);
+  e2s2 = nsContentUtils::ComparePoints(eEnd->mNode, eEndOffset,
+                                       endParent, endOffset);
 
   if (rangeCount > 1)
   {
@@ -3119,14 +3055,14 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   // Find the first text node in the range.
   
-  PRBool found;
+  bool found;
   nsCOMPtr<nsIContent> content;
 
   iter->First();
 
   if (!IsTextNode(p1))
   {
-    found = PR_FALSE;
+    found = false;
 
     while (!iter->IsDone())
     {
@@ -3139,7 +3075,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
         NS_ENSURE_TRUE(p1, NS_ERROR_FAILURE);
 
         o1 = 0;
-        found = PR_TRUE;
+        found = true;
 
         break;
       }
@@ -3156,7 +3092,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
 
   if (! IsTextNode(p2))
   {
-    found = PR_FALSE;
+    found = false;
 
     while (!iter->IsDone())
     {
@@ -3175,7 +3111,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
         NS_ENSURE_SUCCESS(result, result);
 
         o2 = str.Length();
-        found = PR_TRUE;
+        found = true;
 
         break;
       }
@@ -3186,7 +3122,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
     NS_ENSURE_TRUE(found, NS_ERROR_FAILURE);
   }
 
-  found    = PR_FALSE;
+  found    = false;
   *aSelLength = 0;
 
   for (i = 0; i < tableCount; i++)
@@ -3219,7 +3155,7 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
           *aSelLength = entry->mLength - (o1 - entry->mNodeOffset);
         }
 
-        found = PR_TRUE;
+        found = true;
       }
     }
     else // found
@@ -3245,36 +3181,18 @@ nsTextServicesDocument::GetUncollapsedSelection(nsITextServicesDocument::TSDBloc
   return result;
 }
 
-PRBool
+bool
 nsTextServicesDocument::SelectionIsCollapsed()
 {
   return(mSelStartIndex == mSelEndIndex && mSelStartOffset == mSelEndOffset);
 }
 
-PRBool
+bool
 nsTextServicesDocument::SelectionIsValid()
 {
   return(mSelStartIndex >= 0);
 }
 
-nsresult
-nsTextServicesDocument::ComparePoints(nsIDOMNode* aParent1, PRInt32 aOffset1,
-                                      nsIDOMNode* aParent2, PRInt32 aOffset2,
-                                      PRInt32 *aResult)
-{
-  nsresult result;
-  
-  if (!sRangeHelper) {
-    result = CallGetService("@mozilla.org/content/range-utils;1",
-                            &sRangeHelper);
-    NS_ENSURE_TRUE(sRangeHelper, result);
-  }
-
-  *aResult = sRangeHelper->ComparePoints(aParent1, aOffset1,
-                                         aParent2, aOffset2);
-  return NS_OK;
-}
-
 nsresult
 nsTextServicesDocument::GetRangeEndPoints(nsIDOMRange *aRange,
                                           nsIDOMNode **aStartParent, PRInt32 *aStartOffset,
@@ -3311,25 +3229,8 @@ nsTextServicesDocument::CreateRange(nsIDOMNode *aStartParent, PRInt32 aStartOffs
                                     nsIDOMNode *aEndParent, PRInt32 aEndOffset,
                                     nsIDOMRange **aRange)
 {
-  nsresult result;
-
-  result = CallCreateInstance("@mozilla.org/content/range;1", aRange);
-  NS_ENSURE_SUCCESS(result, result);
-
-  NS_ENSURE_TRUE(*aRange, NS_ERROR_NULL_POINTER);
-
-  result = (*aRange)->SetStart(aStartParent, aStartOffset);
-
-  if (NS_SUCCEEDED(result))
-    result = (*aRange)->SetEnd(aEndParent, aEndOffset);
-
-  if (NS_FAILED(result))
-  {
-    NS_RELEASE((*aRange));
-    *aRange = 0;
-  }
-
-  return result;
+  return nsRange::CreateRange(aStartParent, aStartOffset, aEndParent,
+                              aEndOffset, aRange);
 }
 
 nsresult
@@ -3341,12 +3242,8 @@ nsTextServicesDocument::FirstTextNode(nsIContentIterator *aIterator,
 
   aIterator->First();
 
-  while (!aIterator->IsDone())
-  {
-    nsCOMPtr<nsIContent> content = do_QueryInterface(aIterator->GetCurrentNode());
-
-    if (IsTextNode(content))
-    {
+  while (!aIterator->IsDone()) {
+    if (aIterator->GetCurrentNode()->NodeType() == nsIDOMNode::TEXT_NODE) {
       if (aIteratorStatus)
         *aIteratorStatus = nsTextServicesDocument::eValid;
       break;
@@ -3367,12 +3264,8 @@ nsTextServicesDocument::LastTextNode(nsIContentIterator *aIterator,
 
   aIterator->Last();
 
-  while (!aIterator->IsDone())
-  {
-    nsCOMPtr<nsIContent> content = do_QueryInterface(aIterator->GetCurrentNode());
-
-    if (IsTextNode(content))
-    {
+  while (!aIterator->IsDone()) {
+    if (aIterator->GetCurrentNode()->NodeType() == nsIDOMNode::TEXT_NODE) {
       if (aIteratorStatus)
         *aIteratorStatus = nsTextServicesDocument::eValid;
       break;
@@ -3461,7 +3354,7 @@ nsresult
 nsTextServicesDocument::FirstTextNodeInNextBlock(nsIContentIterator *aIterator)
 {
   nsCOMPtr<nsIContent> prev;
-  PRBool crossedBlockBoundary = PR_FALSE;
+  bool crossedBlockBoundary = false;
 
   NS_ENSURE_TRUE(aIterator, NS_ERROR_NULL_POINTER);
 
@@ -3479,12 +3372,12 @@ nsTextServicesDocument::FirstTextNodeInNextBlock(nsIContentIterator *aIterator)
         break;
     }
     else if (!crossedBlockBoundary && IsBlockNode(content))
-      crossedBlockBoundary = PR_TRUE;
+      crossedBlockBoundary = true;
 
     aIterator->Next();
 
     if (!crossedBlockBoundary && DidSkip(aIterator))
-      crossedBlockBoundary = PR_TRUE;
+      crossedBlockBoundary = true;
   }
 
   return NS_OK;
@@ -3516,7 +3409,7 @@ nsTextServicesDocument::GetFirstTextNodeInPrevBlock(nsIContent **aContent)
   if (!mIterator->IsDone())
   {
     nsCOMPtr<nsIContent> current = do_QueryInterface(mIterator->GetCurrentNode());
-    current.swap(*aContent);
+    current.forget(aContent);
   }
 
   // Restore the iterator:
@@ -3550,7 +3443,7 @@ nsTextServicesDocument::GetFirstTextNodeInNextBlock(nsIContent **aContent)
   if (!mIterator->IsDone())
   {
     nsCOMPtr<nsIContent> current = do_QueryInterface(mIterator->GetCurrentNode());
-    current.swap(*aContent);
+    current.forget(aContent);
   }
 
   // Restore the iterator:
@@ -3628,9 +3521,6 @@ nsTextServicesDocument::CreateOffsetTable(nsTArray<OffsetEntry*> *aOffsetTable,
           // Add an entry for this text node into the offset table:
 
           OffsetEntry *entry = new OffsetEntry(node, offset, str.Length());
-
-          NS_ENSURE_TRUE(entry, NS_ERROR_OUT_OF_MEMORY);
-
           aOffsetTable->AppendElement(entry);
 
           // If one or both of the endpoints of the iteration range
@@ -3640,18 +3530,18 @@ nsTextServicesDocument::CreateOffsetTable(nsTArray<OffsetEntry*> *aOffsetTable,
 
           PRInt32 startOffset = 0;
           PRInt32 endOffset   = str.Length();
-          PRBool adjustStr    = PR_FALSE;
+          bool adjustStr    = false;
 
           if (entry->mNode == rngStartNode)
           {
             entry->mNodeOffset = startOffset = rngStartOffset;
-            adjustStr = PR_TRUE;
+            adjustStr = true;
           }
 
           if (entry->mNode == rngEndNode)
           {
             endOffset = rngEndOffset;
-            adjustStr = PR_TRUE;
+            adjustStr = true;
           }
 
           if (adjustStr)
@@ -3775,8 +3665,6 @@ nsTextServicesDocument::SplitOffsetEntry(PRInt32 aTableIndex, PRInt32 aNewEntryL
                                           entry->mStrOffset + oldLength,
                                           aNewEntryLength);
 
-  NS_ENSURE_TRUE(newEntry, NS_ERROR_OUT_OF_MEMORY);
-
   if (!mOffsetTable.InsertElementAt(aTableIndex + 1, newEntry))
   {
     delete newEntry;
@@ -3792,7 +3680,7 @@ nsTextServicesDocument::SplitOffsetEntry(PRInt32 aTableIndex, PRInt32 aNewEntryL
 }
 
 nsresult
-nsTextServicesDocument::NodeHasOffsetEntry(nsTArray<OffsetEntry*> *aOffsetTable, nsIDOMNode *aNode, PRBool *aHasEntry, PRInt32 *aEntryIndex)
+nsTextServicesDocument::NodeHasOffsetEntry(nsTArray<OffsetEntry*> *aOffsetTable, nsIDOMNode *aNode, bool *aHasEntry, PRInt32 *aEntryIndex)
 {
   OffsetEntry *entry;
   PRUint32 i;
@@ -3807,14 +3695,14 @@ nsTextServicesDocument::NodeHasOffsetEntry(nsTArray<OffsetEntry*> *aOffsetTable,
 
     if (entry->mNode == aNode)
     {
-      *aHasEntry   = PR_TRUE;
+      *aHasEntry   = true;
       *aEntryIndex = i;
 
       return NS_OK;
     }
   }
 
-  *aHasEntry   = PR_FALSE;
+  *aHasEntry   = false;
   *aEntryIndex = -1;
 
   return NS_OK;
@@ -3845,7 +3733,7 @@ nsTextServicesDocument::FindWordBounds(nsTArray<OffsetEntry*> *aOffsetTable,
     *aWordEndOffset = 0;
 
   PRInt32 entryIndex = 0;
-  PRBool hasEntry = PR_FALSE;
+  bool hasEntry = false;
 
   // It's assumed that aNode is a text node. The first thing
   // we do is get it's index in the offset table so we can
