@@ -44,8 +44,6 @@
 #include "nsIEnumerator.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMNodeList.h"
-#include "nsIDOMDocumentRange.h"
-#include "nsIDOMDocumentTraversal.h"
 #include "nsISelection.h"
 #include "nsISelectionController.h"
 #include "nsIFrame.h"
@@ -63,14 +61,13 @@
 #include "nsIDOMElement.h"
 #include "nsIWordBreaker.h"
 #include "nsCRT.h"
-#include "nsIRange.h"
+#include "nsRange.h"
 
 // Yikes!  Casting a char to unichar can fill with ones!
 #define CHAR_TO_UNICHAR(c) ((PRUnichar)(const unsigned char)c)
 
 static NS_DEFINE_CID(kCContentIteratorCID, NS_CONTENTITERATOR_CID);
 static NS_DEFINE_CID(kCPreContentIteratorCID, NS_PRECONTENTITERATOR_CID);
-static NS_DEFINE_IID(kRangeCID, NS_RANGE_CID);
 
 #define CH_SHY ((PRUnichar) 0xAD)
 
@@ -110,7 +107,7 @@ PR_STATIC_ASSERT(CH_SHY <= 255);
 class nsFindContentIterator : public nsIContentIterator
 {
 public:
-  nsFindContentIterator(PRBool aFindBackward)
+  nsFindContentIterator(bool aFindBackward)
     : mStartOffset(0),
       mEndOffset(0),
       mFindBackward(aFindBackward)
@@ -135,11 +132,6 @@ public:
     NS_NOTREACHED("internal error");
     return NS_ERROR_NOT_IMPLEMENTED;
   }
-  virtual nsresult Init(nsIRange* aRange)
-  {
-    NS_NOTREACHED("internal error");
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
   // Not a range because one of the endpoints may be anonymous.
   nsresult Init(nsIDOMNode* aStartNode, PRInt32 aStartOffset,
                 nsIDOMNode* aEndNode, PRInt32 aEndOffset);
@@ -148,7 +140,7 @@ public:
   virtual void Next();
   virtual void Prev();
   virtual nsINode* GetCurrentNode();
-  virtual PRBool IsDone();
+  virtual bool IsDone();
   virtual nsresult PositionAt(nsINode* aCurNode);
 
 private:
@@ -163,7 +155,7 @@ private:
   
   nsCOMPtr<nsIContent> mStartOuterContent;
   nsCOMPtr<nsIContent> mEndOuterContent;
-  PRBool mFindBackward;
+  bool mFindBackward;
 
   void Reset();
   void MaybeSetupInnerIterator();
@@ -254,10 +246,10 @@ nsFindContentIterator::GetCurrentNode()
   return mOuterIterator->GetCurrentNode();
 }
 
-PRBool
+bool
 nsFindContentIterator::IsDone() {
   if (mInnerIterator && !mInnerIterator->IsDone()) {
-    return PR_FALSE;
+    return false;
   }
   return mOuterIterator->IsDone();
 }
@@ -347,7 +339,7 @@ nsFindContentIterator::MaybeSetupInnerIterator()
     return;
 
   nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(content));
-  if (!formControl->IsTextControl(PR_TRUE)) {
+  if (!formControl->IsTextControl(true)) {
     return;
   }
 
@@ -437,7 +429,7 @@ nsFindContentIterator::SetupInnerIterator(nsIContent* aContent)
     if (NS_FAILED(res)) {
       // we are done with the outer-iterator, the 
       // inner-iterator will traverse what we want
-      outerRange->Collapse(PR_TRUE);
+      outerRange->Collapse(true);
     }
 
     // Note: we just re-init here, using the segment of our search range that
@@ -449,7 +441,7 @@ nsFindContentIterator::SetupInnerIterator(nsIContent* aContent)
 }
 
 nsresult
-NS_NewFindContentIterator(PRBool aFindBackward,
+NS_NewFindContentIterator(bool aFindBackward,
                           nsIContentIterator** aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
@@ -479,8 +471,8 @@ nsIAtom* nsFind::sTdAtom = nsnull;
 NS_IMPL_ISUPPORTS1(nsFind, nsIFind)
 
 nsFind::nsFind()
-  : mFindBackward(PR_FALSE)
-  , mCaseSensitive(PR_FALSE)
+  : mFindBackward(false)
+  , mCaseSensitive(false)
   , mIterOffset(0)
 {
   // Initialize the atoms if they aren't already:
@@ -571,7 +563,7 @@ nsFind::InitIterator(nsIDOMNode* aStartNode, PRInt32 aStartOffset,
 
 /* attribute boolean findBackward; */
 NS_IMETHODIMP
-nsFind::GetFindBackwards(PRBool *aFindBackward)
+nsFind::GetFindBackwards(bool *aFindBackward)
 {
   if (!aFindBackward)
     return NS_ERROR_NULL_POINTER;
@@ -580,7 +572,7 @@ nsFind::GetFindBackwards(PRBool *aFindBackward)
   return NS_OK;
 }
 NS_IMETHODIMP
-nsFind::SetFindBackwards(PRBool aFindBackward)
+nsFind::SetFindBackwards(bool aFindBackward)
 {
   mFindBackward = aFindBackward;
   return NS_OK;
@@ -588,7 +580,7 @@ nsFind::SetFindBackwards(PRBool aFindBackward)
 
 /* attribute boolean caseSensitive; */
 NS_IMETHODIMP
-nsFind::GetCaseSensitive(PRBool *aCaseSensitive)
+nsFind::GetCaseSensitive(bool *aCaseSensitive)
 {
   if (!aCaseSensitive)
     return NS_ERROR_NULL_POINTER;
@@ -597,7 +589,7 @@ nsFind::GetCaseSensitive(PRBool *aCaseSensitive)
   return NS_OK;
 }
 NS_IMETHODIMP
-nsFind::SetCaseSensitive(PRBool aCaseSensitive)
+nsFind::SetCaseSensitive(bool aCaseSensitive)
 {
   mCaseSensitive = aCaseSensitive;
   return NS_OK;
@@ -638,7 +630,7 @@ nsFind::SetWordBreaker(nsIWordBreaker* aWordBreaker)
 nsresult
 nsFind::NextNode(nsIDOMRange* aSearchRange,
                  nsIDOMRange* aStartPoint, nsIDOMRange* aEndPoint,
-                 PRBool aContinueOk)
+                 bool aContinueOk)
 {
   nsresult rv;
 
@@ -780,10 +772,10 @@ nsFind::NextNode(nsIDOMRange* aSearchRange,
   return NS_OK;
 }
 
-PRBool nsFind::IsBlockNode(nsIContent* aContent)
+bool nsFind::IsBlockNode(nsIContent* aContent)
 {
   if (!aContent->IsHTML()) {
-    return PR_FALSE;
+    return false;
   }
 
   nsIAtom *atom = aContent->Tag();
@@ -792,20 +784,20 @@ PRBool nsFind::IsBlockNode(nsIContent* aContent)
       atom == sHRAtom ||
       atom == sThAtom ||
       atom == sTdAtom)
-    return PR_TRUE;
+    return true;
 
   if (!mParserService) {
     mParserService = do_GetService(NS_PARSERSERVICE_CONTRACTID);
     if (!mParserService)
-      return PR_FALSE;
+      return false;
   }
 
-  PRBool isBlock = PR_FALSE;
+  bool isBlock = false;
   mParserService->IsBlock(mParserService->HTMLAtomTagToId(atom), isBlock);
   return isBlock;
 }
 
-PRBool nsFind::IsTextNode(nsIDOMNode* aNode)
+bool nsFind::IsTextNode(nsIDOMNode* aNode)
 {
   PRUint16 nodeType;
   aNode->GetNodeType(&nodeType);
@@ -814,22 +806,22 @@ PRBool nsFind::IsTextNode(nsIDOMNode* aNode)
          nodeType == nsIDOMNode::CDATA_SECTION_NODE;
 }
 
-PRBool nsFind::IsVisibleNode(nsIDOMNode *aDOMNode)
+bool nsFind::IsVisibleNode(nsIDOMNode *aDOMNode)
 {
   nsCOMPtr<nsIContent> content(do_QueryInterface(aDOMNode));
   if (!content)
-    return PR_FALSE;
+    return false;
 
   nsIFrame *frame = content->GetPrimaryFrame();
   if (!frame) {
     // No frame! Not visible then.
-    return PR_FALSE;
+    return false;
   }
 
   return frame->GetStyleVisibility()->IsVisible();
 }
 
-PRBool nsFind::SkipNode(nsIContent* aContent)
+bool nsFind::SkipNode(nsIContent* aContent)
 {
   nsIAtom *atom;
 
@@ -867,17 +859,17 @@ PRBool nsFind::SkipNode(nsIContent* aContent)
       DumpNode(node);
 #endif
 
-      return PR_TRUE;
+      return true;
     }
 
     // Only climb to the nearest block node
     if (IsBlockNode(content))
-      return PR_FALSE;
+      return false;
 
     content = content->GetParent();
   }
 
-  return PR_FALSE;
+  return false;
 #endif /* HAVE_BIDI_ITERATOR */
 }
 
@@ -972,7 +964,7 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
 
   // Keep track of when we're in whitespace:
   // (only matters when we're matching)
-  PRBool inWhitespace = PR_FALSE;
+  bool inWhitespace = false;
 
   // Place to save the range start point in case we find a match:
   nsCOMPtr<nsIDOMNode> matchAnchorNode;
@@ -996,13 +988,13 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
     {
 
       tc = nsnull;
-      NextNode(aSearchRange, aStartPoint, aEndPoint, PR_FALSE);
+      NextNode(aSearchRange, aStartPoint, aEndPoint, false);
       if (!mIterNode)    // Out of nodes
       {
         // Are we in the middle of a match?
         // If so, try again with continuation.
         if (matchAnchorNode)
-          NextNode(aSearchRange, aStartPoint, aEndPoint, PR_TRUE);
+          NextNode(aSearchRange, aStartPoint, aEndPoint, true);
 
         // Reset the iterator, so this nsFind will be usable if
         // the user wants to search again (from beginning/end).
@@ -1030,7 +1022,7 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
         matchAnchorNode = nsnull;
         matchAnchorOffset = 0;
         pindex = (mFindBackward ? patLen : 0);
-        inWhitespace = PR_FALSE;
+        inWhitespace = false;
       }
  
       // Get the text content:
@@ -1145,19 +1137,19 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
     // has already matched at least one space in the document.
     if (inWhitespace && !IsSpace(c))
     {
-      inWhitespace = PR_FALSE;
+      inWhitespace = false;
       pindex += incr;
 #ifdef DEBUG
       // This shouldn't happen -- if we were still matching, and we
       // were at the end of the pat string, then we should have
       // caught it in the last iteration and returned success.
       if (OVERFLOW_PINDEX)
-        NS_ASSERTION(PR_FALSE, "Missed a whitespace match\n");
+        NS_ASSERTION(false, "Missed a whitespace match\n");
 #endif
       patc = patStr[pindex];
     }
     if (!inWhitespace && IsSpace(patc))
-      inWhitespace = PR_TRUE;
+      inWhitespace = true;
 
     // convert to lower case if necessary
     else if (!inWhitespace && !mCaseSensitive && IsUpperCase(c))
@@ -1262,7 +1254,7 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
         if (!inWhitespace || DONE_WITH_PINDEX || IsSpace(patStr[pindex+incr]))
         {
           pindex += incr;
-          inWhitespace = PR_FALSE;
+          inWhitespace = false;
 #ifdef DEBUG_FIND
           printf("Advancing pindex to %d\n", pindex);
 #endif
@@ -1305,7 +1297,7 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
     }
     matchAnchorNode = nsnull;
     matchAnchorOffset = 0;
-    inWhitespace = PR_FALSE;
+    inWhitespace = false;
     pindex = (mFindBackward ? patLen : 0);
 #ifdef DEBUG_FIND
     printf("Setting findex back to %d, pindex to %d\n", findex, pindex);
@@ -1322,14 +1314,7 @@ nsFind::Find(const PRUnichar *aPatText, nsIDOMRange* aSearchRange,
 already_AddRefed<nsIDOMRange>
 nsFind::CreateRange()
 {
-  nsCOMPtr<nsIRange> range = do_CreateInstance(kRangeCID);
-  if (!range) {
-    return nsnull;
-  }
-
-  range->SetMaySpanAnonymousSubtrees(PR_TRUE);
-
-  nsIDOMRange* result;
-  CallQueryInterface(range.get(), &result);
-  return result;
+  nsRefPtr<nsRange> range = new nsRange();
+  range->SetMaySpanAnonymousSubtrees(true);
+  return range.forget();
 }
