@@ -65,7 +65,7 @@ using mozilla::dom::ContentParent;
 using mozilla::dom::ContentChild;
 using mozilla::unused; // ha!
 
-static PRBool
+static bool
 IsChildProcess()
 {
   return XRE_GetProcessType() == GeckoProcessType_Content;
@@ -199,8 +199,8 @@ nsPermissionManager::Init()
 
   mObserverService = do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv)) {
-    mObserverService->AddObserver(this, "profile-before-change", PR_TRUE);
-    mObserverService->AddObserver(this, "profile-do-change", PR_TRUE);
+    mObserverService->AddObserver(this, "profile-before-change", true);
+    mObserverService->AddObserver(this, "profile-do-change", true);
   }
 
   if (IsChildProcess()) {
@@ -221,13 +221,13 @@ nsPermissionManager::Init()
   // ignore failure here, since it's non-fatal (we can run fine without
   // persistent storage - e.g. if there's no profile).
   // XXX should we tell the user about this?
-  InitDB(PR_FALSE);
+  InitDB(false);
 
   return NS_OK;
 }
 
 nsresult
-nsPermissionManager::InitDB(PRBool aRemoveFile)
+nsPermissionManager::InitDB(bool aRemoveFile)
 {
   nsCOMPtr<nsIFile> permissionsFile;
   NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(permissionsFile));
@@ -238,11 +238,11 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (aRemoveFile) {
-    PRBool exists = PR_FALSE;
+    bool exists = false;
     rv = permissionsFile->Exists(&exists);
     NS_ENSURE_SUCCESS(rv, rv);
     if (exists) {
-      rv = permissionsFile->Remove(PR_FALSE);
+      rv = permissionsFile->Remove(false);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -255,11 +255,11 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
   rv = storage->OpenDatabase(permissionsFile, getter_AddRefs(mDBConn));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRBool ready;
+  bool ready;
   mDBConn->GetConnectionReady(&ready);
   if (!ready) {
     // delete and try again
-    rv = permissionsFile->Remove(PR_FALSE);
+    rv = permissionsFile->Remove(false);
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = storage->OpenDatabase(permissionsFile, getter_AddRefs(mDBConn));
@@ -270,7 +270,7 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
       return NS_ERROR_UNEXPECTED;
   }
 
-  PRBool tableExists = PR_FALSE;
+  bool tableExists = false;
   mDBConn->TableExists(NS_LITERAL_CSTRING("moz_hosts"), &tableExists);
   if (!tableExists) {
     rv = CreateTable();
@@ -464,7 +464,7 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
   }
 
   // look up the type index
-  PRInt32 typeIndex = GetTypeIndex(aType.get(), PR_TRUE);
+  PRInt32 typeIndex = GetTypeIndex(aType.get(), true);
   NS_ENSURE_TRUE(typeIndex != -1, NS_ERROR_OUT_OF_MEMORY);
 
   // When an entry already exists, PutEntry will return that, instead
@@ -620,6 +620,20 @@ nsPermissionManager::RemoveAll()
   return rv;
 }
 
+void
+nsPermissionManager::CloseDB()
+{
+  // Null the statements, this will finalize them.
+  mStmtInsert = nsnull;
+  mStmtDelete = nsnull;
+  mStmtUpdate = nsnull;
+  if (mDBConn) {
+    mozilla::DebugOnly<nsresult> rv = mDBConn->Close();
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
+    mDBConn = nsnull;
+  }
+}
+
 nsresult
 nsPermissionManager::RemoveAllInternal()
 {
@@ -629,11 +643,8 @@ nsPermissionManager::RemoveAllInternal()
   if (mDBConn) {
     nsresult rv = mDBConn->ExecuteSimpleSQL(NS_LITERAL_CSTRING("DELETE FROM moz_hosts"));
     if (NS_FAILED(rv)) {
-      mStmtInsert = nsnull;
-      mStmtDelete = nsnull;
-      mStmtUpdate = nsnull;
-      mDBConn = nsnull;
-      rv = InitDB(PR_TRUE);
+      CloseDB();
+      rv = InitDB(true);
       return rv;
     }
   }
@@ -646,7 +657,7 @@ nsPermissionManager::TestExactPermission(nsIURI     *aURI,
                                          const char *aType,
                                          PRUint32   *aPermission)
 {
-  return CommonTestPermission(aURI, aType, aPermission, PR_TRUE);
+  return CommonTestPermission(aURI, aType, aPermission, true);
 }
 
 NS_IMETHODIMP
@@ -654,14 +665,14 @@ nsPermissionManager::TestPermission(nsIURI     *aURI,
                                     const char *aType,
                                     PRUint32   *aPermission)
 {
-  return CommonTestPermission(aURI, aType, aPermission, PR_FALSE);
+  return CommonTestPermission(aURI, aType, aPermission, false);
 }
 
 nsresult
 nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
                                           const char *aType,
                                           PRUint32   *aPermission,
-                                          PRBool      aExactHostMatch)
+                                          bool        aExactHostMatch)
 {
   NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(aType);
@@ -674,7 +685,7 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
   // No host doesn't mean an error. Just return the default. Unless this is
   // a file uri. In that case use a magic host.
   if (NS_FAILED(rv)) {
-    PRBool isFile;
+    bool isFile;
     rv = aURI->SchemeIs("file", &isFile);
     NS_ENSURE_SUCCESS(rv, rv);
     if (isFile) {
@@ -685,7 +696,7 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
     }
   }
   
-  PRInt32 typeIndex = GetTypeIndex(aType, PR_FALSE);
+  PRInt32 typeIndex = GetTypeIndex(aType, false);
   // If type == -1, the type isn't known,
   // so just return NS_OK
   if (typeIndex == -1) return NS_OK;
@@ -705,7 +716,7 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
 nsHostEntry *
 nsPermissionManager::GetHostEntry(const nsAFlatCString &aHost,
                                   PRUint32              aType,
-                                  PRBool                aExactHostMatch)
+                                  bool                  aExactHostMatch)
 {
   PRUint32 offset = 0;
   nsHostEntry *entry;
@@ -792,10 +803,11 @@ NS_IMETHODIMP nsPermissionManager::Observe(nsISupports *aSubject, const char *aT
     } else {
       RemoveAllFromMemory();
     }
+    CloseDB();
   }
   else if (!nsCRT::strcmp(aTopic, "profile-do-change")) {
     // the profile has already changed; init the db from the new location
-    InitDB(PR_FALSE);
+    InitDB(false);
   }
 
   return NS_OK;
@@ -822,7 +834,7 @@ nsPermissionManager::RemoveAllFromMemory()
 // Returns -1 on failure
 PRInt32
 nsPermissionManager::GetTypeIndex(const char *aType,
-                                  PRBool      aAdd)
+                                  bool        aAdd)
 {
   for (PRUint32 i = 0; i < mTypeArray.Length(); ++i)
     if (mTypeArray[i].Equals(aType))
@@ -897,7 +909,7 @@ nsPermissionManager::Read()
     rv = stmtDeleteExpired->BindInt64ByIndex(1, PR_Now() / 1000);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    PRBool hasResult;
+    bool hasResult;
     rv = stmtDeleteExpired->ExecuteStep(&hasResult);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -913,7 +925,7 @@ nsPermissionManager::Read()
   PRUint32 permission;
   PRUint32 expireType;
   PRInt64 expireTime;
-  PRBool hasResult;
+  bool hasResult;
   while (NS_SUCCEEDED(stmt->ExecuteStep(&hasResult)) && hasResult) {
     // explicitly set our entry id counter for use in AddInternal(),
     // and keep track of the largest id so we know where to pick up.
@@ -967,7 +979,7 @@ nsPermissionManager::Import()
 
   // start a transaction on the storage db, to optimize insertions.
   // transaction will automically commit on completion
-  mozStorageTransaction transaction(mDBConn, PR_TRUE);
+  mozStorageTransaction transaction(mDBConn, true);
 
   /* format is:
    * matchtype \t type \t permission \t host
@@ -977,7 +989,7 @@ nsPermissionManager::Import()
    */
 
   nsCAutoString buffer;
-  PRBool isMore = PR_TRUE;
+  bool isMore = true;
   while (isMore && NS_SUCCEEDED(lineInputStream->ReadLine(buffer, &isMore))) {
     if (buffer.IsEmpty() || buffer.First() == '#') {
       continue;
@@ -1010,7 +1022,7 @@ nsPermissionManager::Import()
   }
 
   // we're done importing - delete the old file
-  permissionsFile->Remove(PR_FALSE);
+  permissionsFile->Remove(false);
 
   return NS_OK;
 }
@@ -1112,7 +1124,7 @@ nsPermissionManager::UpdateDB(OperationType         aOp,
   }
 
   if (NS_SUCCEEDED(rv)) {
-    PRBool hasResult;
+    bool hasResult;
     rv = aStmt->ExecuteStep(&hasResult);
     aStmt->Reset();
   }
