@@ -39,14 +39,25 @@
 
 #ifdef USE_CAIRO
 #include "DrawTargetCairo.h"
+#include "ScaledFontBase.h"
 #endif
 
 #ifdef USE_SKIA
 #include "DrawTargetSkia.h"
+#include "ScaledFontBase.h"
+#endif
+
+#ifdef WIN32
+#include "ScaledFontWin.h"
+#endif
+
 #ifdef XP_MACOSX
 #include "ScaledFontMac.h"
 #endif
-#include "ScaledFontSkia.h"
+
+
+#ifdef XP_MACOSX
+#include "DrawTargetCG.h"
 #endif
 
 #ifdef WIN32
@@ -86,7 +97,7 @@ Factory::CreateDrawTarget(BackendType aBackend, const IntSize &aSize, SurfaceFor
       }
       break;
     }
-#endif
+#elif defined XP_MACOSX || defined ANDROID || defined LINUX
 #ifdef USE_SKIA
   case BACKEND_SKIA:
     {
@@ -97,6 +108,18 @@ Factory::CreateDrawTarget(BackendType aBackend, const IntSize &aSize, SurfaceFor
       }
       break;
     }
+#endif
+#ifdef XP_MACOSX
+  case BACKEND_COREGRAPHICS:
+    {
+      RefPtr<DrawTargetCG> newTarget;
+      newTarget = new DrawTargetCG();
+      if (newTarget->Init(aSize, aFormat)) {
+        return newTarget;
+      }
+      break;
+    }
+#endif
 #endif
   default:
     gfxDebug() << "Invalid draw target type specified.";
@@ -118,16 +141,28 @@ Factory::CreateScaledFontForNativeFont(const NativeFont &aNativeFont, Float aSiz
       return new ScaledFontDWrite(static_cast<IDWriteFontFace*>(aNativeFont.mFont), aSize);
     }
 #endif
-#ifdef USE_SKIA
 #ifdef XP_MACOSX
   case NATIVE_FONT_MAC_FONT_FACE:
     {
       return new ScaledFontMac(static_cast<CGFontRef>(aNativeFont.mFont), aSize);
     }
+#endif
+#ifdef USE_SKIA
+#ifdef WIN32
+  case NATIVE_FONT_GDI_FONT_FACE:
+    {
+      return new ScaledFontWin(static_cast<gfxGDIFont*>(aNativeFont.mFont), aSize);
+    }
 #endif
   case NATIVE_FONT_SKIA_FONT_FACE:
     {
-      return new ScaledFontSkia(static_cast<gfxFont*>(aNativeFont.mFont), aSize);
+      return new ScaledFontBase(static_cast<gfxFont*>(aNativeFont.mFont), aSize);
+    }
+#endif
+#ifdef USE_CAIRO
+  case NATIVE_FONT_CAIRO_FONT_FACE:
+    {
+      return new ScaledFontBase(aSize);
     }
 #endif
   default:
@@ -136,6 +171,22 @@ Factory::CreateScaledFontForNativeFont(const NativeFont &aNativeFont, Float aSiz
   }
 }
 
+TemporaryRef<ScaledFont>
+Factory::CreateScaledFontWithCairo(const NativeFont& aNativeFont, Float aSize, cairo_scaled_font_t* aScaledFont)
+{
+#ifdef USE_CAIRO
+  // In theory, we could pull the NativeFont out of the cairo_scaled_font_t*,
+  // but that would require a lot of code that would be otherwise repeated in
+  // various backends.
+  // Therefore, we just reuse CreateScaledFontForNativeFont's implementation.
+  RefPtr<ScaledFont> font = CreateScaledFontForNativeFont(aNativeFont, aSize);
+  static_cast<ScaledFontBase*>(font.get())->SetCairoScaledFont(aScaledFont);
+  return font;
+#else
+  return NULL;
+#endif
+}
+
 #ifdef WIN32
 TemporaryRef<DrawTarget>
 Factory::CreateDrawTargetForD3D10Texture(ID3D10Texture2D *aTexture, SurfaceFormat aFormat)
