@@ -38,22 +38,7 @@
 #ifndef MOZILLA_GFX_TYPES_H_
 #define MOZILLA_GFX_TYPES_H_
 
-#if defined (_SVR4) || defined (SVR4) || defined (__OpenBSD__) || defined (_sgi) || defined (__sun) || defined (sun) || defined (__digital__)
-#  include <inttypes.h>
-#elif defined (_MSC_VER)
-typedef unsigned __int8 uint8_t;
-typedef __int16 int16_t;
-typedef unsigned __int16 uint16_t;
-typedef __int32 int32_t;
-typedef unsigned __int32 uint32_t;
-typedef __int64 int64_t;
-typedef unsigned __int64 uint64_t;
-
-#elif defined (_AIX)
-#  include <sys/inttypes.h>
-#else
-#  include <stdint.h>
-#endif
+#include "mozilla/StdInt.h"
 
 #include <stddef.h>
 
@@ -68,40 +53,57 @@ enum SurfaceType
   SURFACE_D2D1_BITMAP, /* Surface wrapping a ID2D1Bitmap */
   SURFACE_D2D1_DRAWTARGET, /* Surface made from a D2D draw target */
   SURFACE_CAIRO, /* Surface wrapping a cairo surface */
-  SURFACE_COREGRAPHICS_IMAGE /* Surface wrapping a CoreGraphics Image */
+  SURFACE_CAIRO_IMAGE, /* Data surface wrapping a cairo image surface */
+  SURFACE_COREGRAPHICS_IMAGE, /* Surface wrapping a CoreGraphics Image */
+  SURFACE_COREGRAPHICS_CGCONTEXT, /* Surface wrapping a CG context */
+  SURFACE_SKIA /* Surface wrapping a Skia bitmap */
 };
 
 enum SurfaceFormat
 {
   FORMAT_B8G8R8A8,
   FORMAT_B8G8R8X8,
+  FORMAT_R5G6B5,
   FORMAT_A8
 };
 
 enum BackendType
 {
+  BACKEND_NONE,
   BACKEND_DIRECT2D,
   BACKEND_COREGRAPHICS,
-  BACKEND_CAIRO
+  BACKEND_CAIRO,
+  BACKEND_SKIA
 };
 
 enum FontType
 {
-  FONT_DWRITE
+  FONT_DWRITE,
+  FONT_GDI,
+  FONT_MAC,
+  FONT_SKIA,
+  FONT_CAIRO,
+  FONT_COREGRAPHICS
 };
 
 enum NativeSurfaceType
 {
-  NATIVE_SURFACE_D3D10_TEXTURE
+  NATIVE_SURFACE_D3D10_TEXTURE,
+  NATIVE_SURFACE_CAIRO_SURFACE,
+  NATIVE_SURFACE_CGCONTEXT
 };
 
 enum NativeFontType
 {
-  NATIVE_FONT_DWRITE_FONT_FACE
+  NATIVE_FONT_DWRITE_FONT_FACE,
+  NATIVE_FONT_GDI_FONT_FACE,
+  NATIVE_FONT_MAC_FONT_FACE,
+  NATIVE_FONT_SKIA_FONT_FACE,
+  NATIVE_FONT_CAIRO_FONT_FACE
 };
 
 enum CompositionOp { OP_OVER, OP_ADD, OP_ATOP, OP_OUT, OP_IN, OP_SOURCE, OP_DEST_IN, OP_DEST_OUT, OP_DEST_OVER, OP_DEST_ATOP, OP_XOR, OP_COUNT };
-enum ExtendMode { EXTEND_CLAMP, EXTEND_WRAP, EXTEND_MIRROR };
+enum ExtendMode { EXTEND_CLAMP, EXTEND_REPEAT, EXTEND_REFLECT };
 enum FillRule { FILL_WINDING, FILL_EVEN_ODD };
 enum AntialiasMode { AA_NONE, AA_GRAY, AA_SUBPIXEL };
 enum Snapping { SNAP_NONE, SNAP_ALIGNED };
@@ -109,6 +111,7 @@ enum Filter { FILTER_LINEAR, FILTER_POINT };
 enum PatternType { PATTERN_COLOR, PATTERN_SURFACE, PATTERN_LINEAR_GRADIENT, PATTERN_RADIAL_GRADIENT };
 enum JoinStyle { JOIN_BEVEL, JOIN_ROUND, JOIN_MITER, JOIN_MITER_OR_BEVEL };
 enum CapStyle { CAP_BUTT, CAP_ROUND, CAP_SQUARE };
+enum SamplingBounds { SAMPLING_UNBOUNDED, SAMPLING_BOUNDED };
 
 /* Color is stored in non-premultiplied form */
 struct Color
@@ -139,6 +142,10 @@ public:
 
 struct GradientStop
 {
+  bool operator<(const GradientStop& aOther) const {
+    return offset < aOther.offset;
+  }
+
   Float offset;
   Color color;
 };
