@@ -819,6 +819,11 @@ ThebesLayerOGL::IsEmpty()
   return !mBuffer;
 }
 
+void
+ThebesLayerOGL::CleanupResources()
+{
+  mBuffer = nsnull;
+}
 
 class ShadowBufferOGL : public ThebesLayerBufferOGL
 {
@@ -965,5 +970,11 @@ ShadowThebesLayerOGL::RenderLayer(int aPreviousFrameBuffer,
   mBuffer->RenderTo(aOffset, mOGLManager, 0);
 }
 
+void
+ShadowThebesLayerOGL::CleanupResources()
+{
+  DestroyFrontBuffer();
+}
+
 } /* layers */
 } /* mozilla */
