@@ -44,6 +44,7 @@
 #include "nsIView.h"
 #include "nsIWidget.h"
 
+#include "mozilla/Attributes.h"
 #include "mozilla/Services.h"
 #include "mozilla/Preferences.h"
 #include "nsIServiceManager.h"
@@ -76,7 +77,11 @@ static nsSystemFontsMac *gSystemFonts = nsnull;
 #include "nsSystemFontsQt.h"
 #include "gfxPDFSurface.h"
 static nsSystemFontsQt *gSystemFonts = nsnull;
-#elif defined(ANDROID)
+#elif defined(MOZ_WIDGET_ANDROID)
+#include "nsSystemFontsAndroid.h"
+#include "gfxPDFSurface.h"
+static nsSystemFontsAndroid *gSystemFonts = nsnull;
+#elif defined(MOZ_WIDGET_GONK)
 #include "nsSystemFontsAndroid.h"
 #include "gfxPDFSurface.h"
 static nsSystemFontsAndroid *gSystemFonts = nsnull;
@@ -87,7 +92,7 @@ static nsSystemFontsAndroid *gSystemFonts = nsnull;
 using namespace mozilla;
 using mozilla::services::GetObserverService;
 
-class nsFontCache : public nsIObserver
+class nsFontCache MOZ_FINAL : public nsIObserver
 {
 public:
     nsFontCache()   { MOZ_COUNT_CTOR(nsFontCache); }
@@ -126,7 +131,7 @@ nsFontCache::Init(nsDeviceContext* aContext)
     // in low-memory situations.
     nsCOMPtr<nsIObserverService> obs = GetObserverService();
     if (obs)
-        obs->AddObserver(this, "memory-pressure", PR_FALSE);
+        obs->AddObserver(this, "memory-pressure", false);
 
     nsCOMPtr<nsILanguageAtomService> langService;
     langService = do_GetService(NS_LANGUAGEATOMSERVICE_CONTRACTID);
@@ -322,7 +327,7 @@ nsDeviceContext::FontMetricsDeleted(const nsFontMetrics* aFontMetrics)
     return NS_OK;
 }
 
-PRBool
+bool
 nsDeviceContext::IsPrinterSurface()
 {
     return(mPrintingSurface != NULL);
@@ -724,32 +729,32 @@ nsDeviceContext::CalcPrintingSize()
     if (!mPrintingSurface)
         return;
 
-    PRBool inPoints = PR_TRUE;
+    bool inPoints = true;
 
     gfxSize size(0, 0);
     switch (mPrintingSurface->GetType()) {
     case gfxASurface::SurfaceTypeImage:
-        inPoints = PR_FALSE;
+        inPoints = false;
         size = reinterpret_cast<gfxImageSurface*>(mPrintingSurface.get())->GetSize();
         break;
 
 #if defined(MOZ_PDF_PRINTING)
     case gfxASurface::SurfaceTypePDF:
-        inPoints = PR_TRUE;
+        inPoints = true;
         size = reinterpret_cast<gfxPDFSurface*>(mPrintingSurface.get())->GetSize();
         break;
 #endif
 
 #ifdef MOZ_ENABLE_GTK2
     case gfxASurface::SurfaceTypePS:
-        inPoints = PR_TRUE;
+        inPoints = true;
         size = reinterpret_cast<gfxPSSurface*>(mPrintingSurface.get())->GetSize();
         break;
 #endif
 
 #ifdef XP_MACOSX
     case gfxASurface::SurfaceTypeQuartz:
-        inPoints = PR_TRUE; // this is really only true when we're printing
+        inPoints = true; // this is really only true when we're printing
         size = reinterpret_cast<gfxQuartzSurface*>(mPrintingSurface.get())->GetSize();
         break;
 #endif
@@ -758,7 +763,7 @@ nsDeviceContext::CalcPrintingSize()
     case gfxASurface::SurfaceTypeWin32:
     case gfxASurface::SurfaceTypeWin32Printing:
         {
-            inPoints = PR_FALSE;
+            inPoints = false;
             HDC dc = reinterpret_cast<gfxWindowsSurface*>(mPrintingSurface.get())->GetDC();
             if (!dc)
                 dc = GetDC((HWND)mWidget->GetNativeData(NS_NATIVE_WIDGET));
@@ -774,7 +779,7 @@ nsDeviceContext::CalcPrintingSize()
 #ifdef XP_OS2
     case gfxASurface::SurfaceTypeOS2:
         {
-            inPoints = PR_FALSE;
+            inPoints = false;
             // we already set the size in the surface constructor we set for
             // printing, so just get those values here
             size = reinterpret_cast<gfxOS2Surface*>(mPrintingSurface.get())->GetSize();
@@ -806,7 +811,7 @@ nsDeviceContext::CalcPrintingSize()
     }
 }
 
-PRBool nsDeviceContext::CheckDPIChange() {
+bool nsDeviceContext::CheckDPIChange() {
     PRInt32 oldDevPixels = mAppUnitsPerDevNotScaledPixel;
     PRInt32 oldInches = mAppUnitsPerPhysicalInch;
 
@@ -816,12 +821,12 @@ PRBool nsDeviceContext::CheckDPIChange() {
         oldInches != mAppUnitsPerPhysicalInch;
 }
 
-PRBool
+bool
 nsDeviceContext::SetPixelScale(float aScale)
 {
     if (aScale <= 0) {
         NS_NOTREACHED("Invalid pixel scale value");
-        return PR_FALSE;
+        return false;
     }
     PRInt32 oldAppUnitsPerDevPixel = mAppUnitsPerDevPixel;
     mPixelScale = aScale;
