@@ -49,6 +49,9 @@
 #include "gfxPattern.h"
 #include "gfxPath.h"
 #include "nsISupportsImpl.h"
+#include "nsTArray.h"
+
+#include "mozilla/gfx/2D.h"
 
 typedef struct _cairo cairo_t;
 template <typename T> class FallibleTArray;
@@ -75,6 +78,12 @@ public:
      * Initialize this context from a surface.
      */
     gfxContext(gfxASurface *surface);
+
+    /**
+     * Initialize this context from a DrawTarget.
+     */
+    gfxContext(mozilla::gfx::DrawTarget *aTarget);
+
     ~gfxContext();
 
     /**
@@ -97,12 +106,14 @@ public:
      * Return the raw cairo_t object.
      * XXX this should go away at some point.
      */
-    cairo_t *GetCairo() { return mCairo; }
+    cairo_t *GetCairo();
+
+    mozilla::gfx::DrawTarget *GetDrawTarget() { return mDT; }
 
     /**
      * Returns true if the cairo context is in an error state.
      */
-    PRBool HasError();
+    bool HasError();
 
     /**
      ** State
@@ -174,7 +185,7 @@ public:
     /**
      * Returns the current point in the current path.
      */
-    gfxPoint CurrentPoint() const;
+    gfxPoint CurrentPoint();
 
     /**
      * Draws a line from the current point to pt.
@@ -224,7 +235,7 @@ public:
      * Draws the rectangle given by rect.
      * @param snapToPixels ?
      */
-    void Rectangle(const gfxRect& rect, PRBool snapToPixels = PR_FALSE);
+    void Rectangle(const gfxRect& rect, bool snapToPixels = false);
 
     /**
      * Draw an ellipse at the center corner with the given dimensions.
@@ -244,12 +255,12 @@ public:
      * corners.  The corners specify the radii of the two axes of an
      * ellipse (the horizontal and vertical directions given by the
      * width and height, respectively).  By default the ellipse is
-     * drawn in a clockwise direction; if draw_clockwise is PR_FALSE,
+     * drawn in a clockwise direction; if draw_clockwise is false,
      * then it's drawn counterclockwise.
      */
     void RoundedRectangle(const gfxRect& rect,
                           const gfxCornerSizes& corners,
-                          PRBool draw_clockwise = PR_TRUE);
+                          bool draw_clockwise = true);
 
     /**
      ** Transformation Matrix manipulation
@@ -344,29 +355,29 @@ public:
 
     /**
      * Takes the given rect and tries to align it to device pixels.  If
-     * this succeeds, the method will return PR_TRUE, and the rect will
+     * this succeeds, the method will return true, and the rect will
      * be in device coordinates (already transformed by the CTM).  If it 
-     * fails, the method will return PR_FALSE, and the rect will not be
+     * fails, the method will return false, and the rect will not be
      * changed.
      *
-     * If ignoreScale is PR_TRUE, then snapping will take place even if
+     * If ignoreScale is true, then snapping will take place even if
      * the CTM has a scale applied.  Snapping never takes place if
      * there is a rotation in the CTM.
      */
-    PRBool UserToDevicePixelSnapped(gfxRect& rect, PRBool ignoreScale = PR_FALSE) const;
+    bool UserToDevicePixelSnapped(gfxRect& rect, bool ignoreScale = false) const;
 
     /**
      * Takes the given point and tries to align it to device pixels.  If
-     * this succeeds, the method will return PR_TRUE, and the point will
+     * this succeeds, the method will return true, and the point will
      * be in device coordinates (already transformed by the CTM).  If it 
-     * fails, the method will return PR_FALSE, and the point will not be
+     * fails, the method will return false, and the point will not be
      * changed.
      *
-     * If ignoreScale is PR_TRUE, then snapping will take place even if
+     * If ignoreScale is true, then snapping will take place even if
      * the CTM has a scale applied.  Snapping never takes place if
      * there is a rotation in the CTM.
      */
-    PRBool UserToDevicePixelSnapped(gfxPoint& pt, PRBool ignoreScale = PR_FALSE) const;
+    bool UserToDevicePixelSnapped(gfxPoint& pt, bool ignoreScale = false) const;
 
     /**
      * Attempts to pixel snap the rectangle, add it to the current
@@ -389,10 +400,10 @@ public:
 
     /**
      * Gets the current color.  It's returned in the device color space.
-     * returns PR_FALSE if there is something other than a color
+     * returns false if there is something other than a color
      *         set as the current source (pattern, surface, etc)
      */
-    PRBool GetDeviceColor(gfxRGBA& c);
+    bool GetDeviceColor(gfxRGBA& c);
 
     /**
      * Set a solid color in the sRGB color space to use for drawing.
@@ -610,7 +621,7 @@ public:
      * This is conservative; it may return false even when the given rectangle is 
      * fully contained by the current clip.
      */
-    PRBool ClipContainsRect(const gfxRect& aRect);
+    bool ClipContainsRect(const gfxRect& aRect);
 
     /**
      * Groups
@@ -634,8 +645,8 @@ public:
     /**
      ** Hit Testing - check if given point is in the current path
      **/
-    PRBool PointInFill(const gfxPoint& pt);
-    PRBool PointInStroke(const gfxPoint& pt);
+    bool PointInFill(const gfxPoint& pt);
+    bool PointInStroke(const gfxPoint& pt);
 
     /**
      ** Extents - returns user space extent of current path
@@ -681,12 +692,103 @@ public:
     void ClearFlag(PRInt32 aFlag) { mFlags &= ~aFlag; }
     PRInt32 GetFlags() const { return mFlags; }
 
-private:
-    cairo_t *mCairo;
-    nsRefPtr<gfxASurface> mSurface;
-    PRInt32 mFlags;
-};
+    bool IsCairo() const { return !mDT; }
 
+#ifdef MOZ_DUMP_PAINTING
+    /**
+     * Debug functions to encode the current surface as a PNG and export it.
+     */
+
+    /**
+     * Writes a binary PNG file.
+     */
+    void WriteAsPNG(const char* aFile);
+
+    /**
+     * Write as a PNG encoded Data URL to stdout.
+     */
+    void DumpAsDataURL();
+
+    /**
+     * Copy a PNG encoded Data URL to the clipboard.
+     */
+    void CopyAsDataURL();
+#endif
+
+private:
+  friend class GeneralPattern;
+
+  typedef mozilla::gfx::Matrix Matrix;
+  typedef mozilla::gfx::DrawTarget DrawTarget;
+  typedef mozilla::gfx::Color Color;
+  typedef mozilla::gfx::StrokeOptions StrokeOptions;
+  typedef mozilla::gfx::Float Float;
+  typedef mozilla::gfx::Rect Rect;
+  typedef mozilla::gfx::CompositionOp CompositionOp;
+  typedef mozilla::gfx::Path Path;
+  typedef mozilla::gfx::PathBuilder PathBuilder;
+  typedef mozilla::gfx::SourceSurface SourceSurface;
+  
+  struct AzureState {
+    AzureState()
+      : op(mozilla::gfx::OP_OVER)
+      , opIsClear(false)
+      , color(0, 0, 0, 1.0f)
+      , clipWasReset(false)
+      , fillRule(mozilla::gfx::FILL_WINDING)
+      , aaMode(mozilla::gfx::AA_SUBPIXEL)
+    {}
+
+    mozilla::gfx::CompositionOp op;
+    bool opIsClear;
+    Color color;
+    nsRefPtr<gfxPattern> pattern;
+    mozilla::RefPtr<SourceSurface> sourceSurface;
+    Matrix surfTransform;
+    Matrix transform;
+    struct PushedClip {
+      mozilla::RefPtr<Path> path;
+      Rect rect;
+      Matrix transform;
+    };
+    nsTArray<PushedClip> pushedClips;
+    nsTArray<Float> dashPattern;
+    bool clipWasReset;
+    mozilla::gfx::FillRule fillRule;
+    StrokeOptions strokeOptions;
+    mozilla::RefPtr<DrawTarget> drawTarget;
+    mozilla::RefPtr<DrawTarget> parentTarget;
+    mozilla::gfx::AntialiasMode aaMode;
+  };
+
+  // This ensures mPath contains a valid path (in user space!)
+  void EnsurePath();
+  // This ensures mPathBuilder contains a valid PathBuilder (in user space!)
+  void EnsurePathBuilder();
+  void FillAzure(mozilla::gfx::Float aOpacity);
+  void PushClipsToDT(mozilla::gfx::DrawTarget *aDT);
+  CompositionOp GetOp();
+
+  bool mPathIsRect;
+  bool mTransformChanged;
+  Matrix mPathTransform;
+  Rect mRect;
+  mozilla::RefPtr<PathBuilder> mPathBuilder;
+  mozilla::RefPtr<Path> mPath;
+  Matrix mTransform;
+  nsTArray<AzureState> mStateStack;
+
+  AzureState &CurrentState() { return mStateStack[mStateStack.Length() - 1]; }
+  const AzureState &CurrentState() const { return mStateStack[mStateStack.Length() - 1]; }
+
+  cairo_t *mCairo;
+  cairo_t *mRefCairo;
+  nsRefPtr<gfxASurface> mSurface;
+  PRInt32 mFlags;
+
+  mozilla::RefPtr<DrawTarget> mDT;
+  mozilla::RefPtr<DrawTarget> mOriginalDT;
+};
 
 /**
  * Sentry helper class for functions with multiple return points that need to
@@ -740,7 +842,7 @@ class THEBES_API gfxContextPathAutoSaveRestore
 public:
     gfxContextPathAutoSaveRestore() : mContext(nsnull) {}
 
-    gfxContextPathAutoSaveRestore(gfxContext *aContext, PRBool aSave = PR_TRUE) : mContext(aContext)
+    gfxContextPathAutoSaveRestore(gfxContext *aContext, bool aSave = true) : mContext(aContext)
     {
         if (aSave)
             Save();       
@@ -751,7 +853,7 @@ public:
         Restore();
     }
 
-    void SetContext(gfxContext *aContext, PRBool aSave = PR_TRUE)
+    void SetContext(gfxContext *aContext, bool aSave = true)
     {
         mContext = aContext;
         if (aSave)
@@ -819,12 +921,15 @@ private:
 
 class THEBES_API gfxContextAutoDisableSubpixelAntialiasing {
 public:
-    gfxContextAutoDisableSubpixelAntialiasing(gfxContext *aContext, PRBool aDisable)
+    gfxContextAutoDisableSubpixelAntialiasing(gfxContext *aContext, bool aDisable)
     {
         if (aDisable) {
             mSurface = aContext->CurrentSurface();
+            if (!mSurface) {
+              return;
+            }
             mSubpixelAntialiasingEnabled = mSurface->GetSubpixelAntialiasingEnabled();
-            mSurface->SetSubpixelAntialiasingEnabled(PR_FALSE);
+            mSurface->SetSubpixelAntialiasingEnabled(false);
         }
     }
     ~gfxContextAutoDisableSubpixelAntialiasing()
@@ -836,7 +941,7 @@ public:
 
 private:
     nsRefPtr<gfxASurface> mSurface;
-    PRPackedBool mSubpixelAntialiasingEnabled;
+    bool mSubpixelAntialiasingEnabled;
 };
 
 #endif /* GFX_CONTEXT_H */
