@@ -37,6 +37,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "prtypes.h"
+#include "nsAlgorithm.h"
 #include "prmem.h"
 #include "nsString.h"
 #include "nsBidiUtils.h"
@@ -58,6 +59,10 @@
 #include "nsUnicodeRange.h"
 #include "nsCRT.h"
 
+#if defined(XP_WIN)
+#include "gfxWindowsPlatform.h"
+#endif
+
 #define FloatToFixed(f) (65536 * (f))
 #define FixedToFloat(f) ((f) * (1.0 / 65536.0))
 // Right shifts of negative (signed) integers are undefined, as are overflows
@@ -83,7 +88,7 @@ gfxHarfBuzzShaper::gfxHarfBuzzShaper(gfxFont *aFont)
       mSubtableOffset(0),
       mUVSTableOffset(0),
       mUseFontGetGlyph(aFont->ProvidesGetGlyph()),
-      mUseFontGlyphWidths(PR_FALSE)
+      mUseFontGlyphWidths(false)
 {
 }
 
@@ -110,8 +115,14 @@ HBGetTable(hb_tag_t aTag, void *aUserData)
     // bug 589682 - ignore the GDEF table in buggy fonts (applies to
     // Italic and BoldItalic faces of Times New Roman)
     if (aTag == TRUETYPE_TAG('G','D','E','F') &&
-        font->GetFontEntry()->IgnoreGDEF())
-    {
+        font->GetFontEntry()->IgnoreGDEF()) {
+        return nsnull;
+    }
+
+    // bug 721719 - ignore the GSUB table in buggy fonts (applies to Roboto,
+    // at least on some Android ICS devices; set in gfxFT2FontList.cpp)
+    if (aTag == TRUETYPE_TAG('G','S','U','B') &&
+        font->GetFontEntry()->IgnoreGSUB()) {
         return nsnull;
     }
 
@@ -298,8 +309,8 @@ GetKernValueFmt0(const void* aSubtable,
                  PRUint16 aFirstGlyph,
                  PRUint16 aSecondGlyph,
                  PRInt32& aValue,
-                 PRBool   aIsOverride = PR_FALSE,
-                 PRBool   aIsMinimum = PR_FALSE)
+                 bool     aIsOverride = false,
+                 bool     aIsMinimum = false)
 {
     const KernHeaderFmt0* hdr =
         reinterpret_cast<const KernHeaderFmt0*>(aSubtable);
@@ -331,7 +342,7 @@ GetKernValueFmt0(const void* aSubtable,
         if (aIsOverride) {
             aValue = PRInt16(lo->value);
         } else if (aIsMinimum) {
-            aValue = PR_MAX(aValue, PRInt16(lo->value));
+            aValue = NS_MAX(aValue, PRInt32(lo->value));
         } else {
             aValue += PRInt16(lo->value);
         }
@@ -707,13 +718,10 @@ HBGetEastAsianWidth(hb_codepoint_t aCh)
 static hb_font_funcs_t * sHBFontFuncs = nsnull;
 static hb_unicode_funcs_t * sHBUnicodeFuncs = nsnull;
 
-PRBool
-gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
-                               gfxTextRun *aTextRun,
-                               const PRUnichar *aString,
-                               PRUint32 aRunStart,
-                               PRUint32 aRunLength,
-                               PRInt32 aRunScript)
+bool
+gfxHarfBuzzShaper::ShapeWord(gfxContext      *aContext,
+                             gfxShapedWord   *aShapedWord,
+                             const PRUnichar *aText)
 {
     // some font back-ends require this in order to get proper hinted metrics
     mFont->SetupCairoFont(aContext);
@@ -754,10 +762,10 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
             mCmapTable = mFont->GetFontTable(TRUETYPE_TAG('c','m','a','p'));
             if (!mCmapTable) {
                 NS_WARNING("failed to load cmap, glyphs will be missing");
-                return PR_FALSE;
+                return false;
             }
             const PRUint8* data = (const PRUint8*)hb_blob_lock(mCmapTable);
-            PRBool symbol;
+            bool symbol;
             mCmapFormat = gfxFontUtils::
                 FindPreferredSubtable(data, hb_blob_get_length(mCmapTable),
                                       &mSubtableOffset, &mUVSTableOffset,
@@ -803,7 +811,7 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
     if ((!mUseFontGetGlyph && mCmapFormat <= 0) ||
         (!mUseFontGlyphWidths && !mHmtxTable)) {
         // unable to shape with this font
-        return PR_FALSE;
+        return false;
     }
 
     FontCallbackData fcd(this, aContext);
@@ -813,29 +821,23 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
     PRUint32 scale = FloatToFixed(mFont->GetAdjustedSize()); // 16.16 fixed-point
     hb_font_set_scale(font, scale, scale);
 
-    // aRunStart and aRunLength define the section of the textRun and of
-    // aString that is to be drawn with this particular font
-
-    PRBool disableLigatures =
-        (aTextRun->GetFlags() &
-         gfxTextRunFactory::TEXT_DISABLE_OPTIONAL_LIGATURES) != 0;
-
     nsAutoTArray<hb_feature_t,20> features;
 
     // Ligature features are enabled by default in the generic shaper,
     // so we explicitly turn them off if necessary (for letter-spacing)
-    if (disableLigatures) {
-        hb_feature_t ligaOff = { HB_TAG('l','i','g','a'), 0, 0, -1 };
-        hb_feature_t cligOff = { HB_TAG('c','l','i','g'), 0, 0, -1 };
+    if (aShapedWord->DisableLigatures()) {
+        hb_feature_t ligaOff = { HB_TAG('l','i','g','a'), 0, 0, UINT_MAX };
+        hb_feature_t cligOff = { HB_TAG('c','l','i','g'), 0, 0, UINT_MAX };
         features.AppendElement(ligaOff);
         features.AppendElement(cligOff);
     }
 
     // css features need to be merged with the existing ones, if any
-    const gfxFontStyle *style = aTextRun->GetFontGroup()->GetStyle();
+    gfxFontEntry *entry = mFont->GetFontEntry();
+    const gfxFontStyle *style = mFont->GetStyle();
     const nsTArray<gfxFontFeature> *cssFeatures = &style->featureSettings;
     if (cssFeatures->IsEmpty()) {
-        cssFeatures = &mFont->GetFontEntry()->mFeatureSettings;
+        cssFeatures = &entry->mFeatureSettings;
     }
     for (PRUint32 i = 0; i < cssFeatures->Length(); ++i) {
         PRUint32 j;
@@ -847,29 +849,29 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
         }
         if (j == features.Length()) {
             const gfxFontFeature& f = cssFeatures->ElementAt(i);
-            hb_feature_t hbf = { f.mTag, f.mValue, 0, -1 };
+            hb_feature_t hbf = { f.mTag, f.mValue, 0, UINT_MAX };
             features.AppendElement(hbf);
         }
     }
 
-    hb_buffer_t *buffer = hb_buffer_create(aRunLength);
+    bool isRightToLeft = aShapedWord->IsRightToLeft();
+    hb_buffer_t *buffer = hb_buffer_create(aShapedWord->Length());
     hb_buffer_set_unicode_funcs(buffer, sHBUnicodeFuncs);
-    hb_buffer_set_direction(buffer,
-                            aTextRun->IsRightToLeft() ?
-                                HB_DIRECTION_RTL : HB_DIRECTION_LTR);
+    hb_buffer_set_direction(buffer, isRightToLeft ? HB_DIRECTION_RTL :
+                                                    HB_DIRECTION_LTR);
     // For unresolved "common" or "inherited" runs, default to Latin for now.
     // (Should we somehow use the language or locale to try and infer
     // a better default?)
     hb_buffer_set_script(buffer,
-                         aRunScript <= HB_SCRIPT_INHERITED ? HB_SCRIPT_LATIN
-                         : hb_script_t(aRunScript));
+                         aShapedWord->Script() <= HB_SCRIPT_INHERITED ?
+                             HB_SCRIPT_LATIN :
+                             hb_script_t(aShapedWord->Script()));
 
     hb_language_t language;
     if (style->languageOverride) {
         language = hb_ot_tag_to_language(style->languageOverride);
-    } else if (mFont->GetFontEntry()->mLanguageOverride) {
-        language =
-            hb_ot_tag_to_language(mFont->GetFontEntry()->mLanguageOverride);
+    } else if (entry->mLanguageOverride) {
+        language = hb_ot_tag_to_language(entry->mLanguageOverride);
     } else {
         nsCString langString;
         style->language->ToUTF8String(langString);
@@ -877,24 +879,23 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
     }
     hb_buffer_set_language(buffer, language);
 
-    hb_buffer_add_utf16(buffer, reinterpret_cast<const uint16_t*>(aString + aRunStart),
-                        aRunLength, 0, aRunLength);
+    PRUint32 length = aShapedWord->Length();
+    hb_buffer_add_utf16(buffer, reinterpret_cast<const uint16_t*>(aText),
+                        length, 0, length);
 
     hb_shape(font, mHBFace, buffer, features.Elements(), features.Length());
 
-    if (aTextRun->IsRightToLeft()) {
+    if (isRightToLeft) {
         hb_buffer_reverse(buffer);
     }
 
-#ifdef DEBUG
-    nsresult rv =
-#endif
-    SetGlyphsFromRun(aContext, aTextRun, buffer, aRunStart, aRunLength);
-    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "failed to store glyphs into textrun");
+    nsresult rv = SetGlyphsFromRun(aContext, aShapedWord, buffer);
+
+    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "failed to store glyphs into gfxShapedWord");
     hb_buffer_destroy(buffer);
     hb_font_destroy(font);
 
-    return PR_TRUE;
+    return NS_SUCCEEDED(rv);
 }
 
 /**
@@ -908,20 +909,20 @@ gfxHarfBuzzShaper::InitTextRun(gfxContext *aContext,
  */
 static void
 GetRoundOffsetsToPixels(gfxContext *aContext,
-                        PRBool *aRoundX, PRBool *aRoundY)
+                        bool *aRoundX, bool *aRoundY)
 {
-    *aRoundX = PR_FALSE;
+    *aRoundX = false;
     // Could do something fancy here for ScaleFactors of
     // AxisAlignedTransforms, but we leave things simple.
     // Not much point rounding if a matrix will mess things up anyway.
     if (aContext->CurrentMatrix().HasNonTranslation()) {
-        *aRoundY = PR_FALSE;
+        *aRoundY = false;
         return;
     }
 
     // All raster backends snap glyphs to pixels vertically.
     // Print backends set CAIRO_HINT_METRICS_OFF.
-    *aRoundY = PR_TRUE;
+    *aRoundY = true;
 
     cairo_t *cr = aContext->GetCairo();
     cairo_scaled_font_t *scaled_font = cairo_get_scaled_font(cr);
@@ -934,7 +935,7 @@ GetRoundOffsetsToPixels(gfxContext *aContext,
 
     switch (hint_metrics) {
     case CAIRO_HINT_METRICS_OFF:
-        *aRoundY = PR_FALSE;
+        *aRoundY = false;
         return;
     case CAIRO_HINT_METRICS_DEFAULT:
         // Here we mimic what cairo surface/font backends do.  Printing
@@ -946,8 +947,13 @@ GetRoundOffsetsToPixels(gfxContext *aContext,
 #if CAIRO_HAS_DWRITE_FONT // dwrite backend is not in std cairo releases yet
         case CAIRO_FONT_TYPE_DWRITE:
             // show_glyphs is implemented on the font and so is used for
-            // all surface types.
-            return;
+            // all surface types; however, it may pixel-snap depending on
+            // the dwrite rendering mode
+            if (!cairo_dwrite_scaled_font_get_force_GDI_classic(scaled_font) &&
+                gfxWindowsPlatform::GetPlatform()->DWriteMeasuringMode() ==
+                    DWRITE_MEASURING_MODE_NATURAL) {
+                return;
+            }
 #endif
         case CAIRO_FONT_TYPE_QUARTZ:
             // Quartz surfaces implement show_glyphs for Quartz fonts
@@ -962,7 +968,7 @@ GetRoundOffsetsToPixels(gfxContext *aContext,
     case CAIRO_HINT_METRICS_ON:
         break;
     }
-    *aRoundX = PR_TRUE;
+    *aRoundX = true;
     return;
 }
 
@@ -972,32 +978,33 @@ GetRoundOffsetsToPixels(gfxContext *aContext,
 
 nsresult
 gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
-                                    gfxTextRun *aTextRun,
-                                    hb_buffer_t *aBuffer,
-                                    PRUint32 aTextRunOffset,
-                                    PRUint32 aRunLength)
+                                    gfxShapedWord *aShapedWord,
+                                    hb_buffer_t *aBuffer)
 {
     PRInt32 numGlyphs = hb_buffer_get_length(aBuffer);
-    if (numGlyphs == 0)
+    if (numGlyphs == 0) {
         return NS_OK;
+    }
 
     const hb_glyph_info_t *ginfo = hb_buffer_get_glyph_infos(aBuffer);
 
     nsAutoTArray<gfxTextRun::DetailedGlyph,1> detailedGlyphs;
 
+    PRUint32 wordLength = aShapedWord->Length();
     static const PRInt32 NO_GLYPH = -1;
     nsAutoTArray<PRInt32,SMALL_GLYPH_RUN> charToGlyphArray;
-    if (!charToGlyphArray.SetLength(aRunLength))
+    if (!charToGlyphArray.SetLength(wordLength)) {
         return NS_ERROR_OUT_OF_MEMORY;
+    }
 
     PRInt32 *charToGlyph = charToGlyphArray.Elements();
-    for (PRUint32 offset = 0; offset < aRunLength; ++offset) {
+    for (PRUint32 offset = 0; offset < wordLength; ++offset) {
         charToGlyph[offset] = NO_GLYPH;
     }
 
     for (PRInt32 i = 0; i < numGlyphs; ++i) {
         PRUint32 loc = ginfo[i].cluster;
-        if (loc < aRunLength) {
+        if (loc < wordLength) {
             charToGlyph[loc] = i;
         }
     }
@@ -1005,14 +1012,27 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
     PRInt32 glyphStart = 0; // looking for a clump that starts at this glyph
     PRInt32 charStart = 0; // and this char index within the range of the run
 
-    PRBool roundX;
-    PRBool roundY;
+    bool roundX;
+    bool roundY;
     GetRoundOffsetsToPixels(aContext, &roundX, &roundY);
-    // This is signed to avoid promotion to unsigned.
-    PRInt32 dev2appUnits = aTextRun->GetAppUnitsPerDevUnit();
+
+    PRInt32 appUnitsPerDevUnit = aShapedWord->AppUnitsPerDevUnit();
+
     // factor to convert 16.16 fixed-point pixels to app units
     // (only used if not rounding)
-    double hb2appUnits = FixedToFloat(aTextRun->GetAppUnitsPerDevUnit());
+    double hb2appUnits = FixedToFloat(aShapedWord->AppUnitsPerDevUnit());
+
+    // Residual from rounding of previous advance, for use in rounding the
+    // subsequent offset or advance appropriately.  16.16 fixed-point
+    //
+    // When rounding, the goal is to make the distance between glyphs and
+    // their base glyph equal to the integral number of pixels closest to that
+    // suggested by that shaper.
+    // i.e. posInfo[n].x_advance - posInfo[n].x_offset + posInfo[n+1].x_offset
+    //
+    // The value of the residual is the part of the desired distance that has
+    // not been included in integer offsets.
+    hb_position_t x_residual = 0;
 
     // keep track of y-position to set glyph offsets if needed
     nscoord yPos = 0;
@@ -1021,10 +1041,10 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
 
     while (glyphStart < numGlyphs) {
 
-        PRBool inOrder = PR_TRUE;
+        bool inOrder = true;
         PRInt32 charEnd = ginfo[glyphStart].cluster;
         PRInt32 glyphEnd = glyphStart;
-        PRInt32 charLimit = aRunLength;
+        PRInt32 charLimit = wordLength;
         while (charEnd < charLimit) {
             // This is normally executed once for each iteration of the outer loop,
             // but in unusual cases where the character/glyph association is complex,
@@ -1039,7 +1059,7 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
             // find the maximum glyph index covered by the clump so far
             for (PRInt32 i = charStart; i < charEnd; ++i) {
                 if (charToGlyph[i] != NO_GLYPH) {
-                    glyphEnd = PR_MAX(glyphEnd, charToGlyph[i] + 1);
+                    glyphEnd = NS_MAX(glyphEnd, charToGlyph[i] + 1);
                     // update extent of glyph range
                 }
             }
@@ -1058,16 +1078,16 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
             // check whether all glyphs in the range are associated with the characters
             // in our clump; if not, we have a discontinuous range, and should extend it
             // unless we've reached the end of the text
-            PRBool allGlyphsAreWithinCluster = PR_TRUE;
+            bool allGlyphsAreWithinCluster = true;
             PRInt32 prevGlyphCharIndex = charStart - 1;
             for (PRInt32 i = glyphStart; i < glyphEnd; ++i) {
                 PRInt32 glyphCharIndex = ginfo[i].cluster;
                 if (glyphCharIndex < charStart || glyphCharIndex >= charEnd) {
-                    allGlyphsAreWithinCluster = PR_FALSE;
+                    allGlyphsAreWithinCluster = false;
                     break;
                 }
                 if (glyphCharIndex <= prevGlyphCharIndex) {
-                    inOrder = PR_FALSE;
+                    inOrder = false;
                 }
                 prevGlyphCharIndex = glyphCharIndex;
             }
@@ -1086,20 +1106,20 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
         // and endCharIndex to the limit (position beyond the last char),
         // adjusting for the offset of the stringRange relative to the textRun.
         PRInt32 baseCharIndex, endCharIndex;
-        while (charEnd < PRInt32(aRunLength) && charToGlyph[charEnd] == NO_GLYPH)
+        while (charEnd < PRInt32(wordLength) && charToGlyph[charEnd] == NO_GLYPH)
             charEnd++;
         baseCharIndex = charStart;
         endCharIndex = charEnd;
 
         // Then we check if the clump falls outside our actual string range;
         // if so, just go to the next.
-        if (baseCharIndex >= PRInt32(aRunLength)) {
+        if (baseCharIndex >= PRInt32(wordLength)) {
             glyphStart = glyphEnd;
             charStart = charEnd;
             continue;
         }
         // Ensure we won't try to go beyond the valid length of the textRun's text
-        endCharIndex = NS_MIN<PRInt32>(endCharIndex, aRunLength);
+        endCharIndex = NS_MIN<PRInt32>(endCharIndex, wordLength);
 
         // Now we're ready to set the glyph info in the textRun
         PRInt32 glyphsInClump = glyphEnd - glyphStart;
@@ -1108,27 +1128,37 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
         // etc by the shaping process, and remove from the run.
         // (This may be done within harfbuzz eventually.)
         if (glyphsInClump == 1 && baseCharIndex + 1 == endCharIndex &&
-            aTextRun->FilterIfIgnorable(aTextRunOffset + baseCharIndex)) {
+            aShapedWord->FilterIfIgnorable(baseCharIndex)) {
             glyphStart = glyphEnd;
             charStart = charEnd;
             continue;
         }
 
-        // Check if it's a simple one-to-one mapping
+        hb_position_t x_offset = posInfo[glyphStart].x_offset;
         hb_position_t x_advance = posInfo[glyphStart].x_advance;
-        nscoord advance =
-            roundX ? dev2appUnits * FixedToIntRound(x_advance)
-            : NS_floor(hb2appUnits * x_advance + 0.5);
-
+        nscoord xOffset, advance;
+        if (roundX) {
+            xOffset =
+                appUnitsPerDevUnit * FixedToIntRound(x_offset + x_residual);
+            // Desired distance from the base glyph to the next reference point.
+            hb_position_t width = x_advance - x_offset;
+            int intWidth = FixedToIntRound(width);
+            x_residual = width - FloatToFixed(intWidth);
+            advance = appUnitsPerDevUnit * intWidth + xOffset;
+        } else {
+            xOffset = floor(hb2appUnits * x_offset + 0.5);
+            advance = floor(hb2appUnits * x_advance + 0.5);
+        }
+        // Check if it's a simple one-to-one mapping
         if (glyphsInClump == 1 &&
             gfxTextRun::CompressedGlyph::IsSimpleGlyphID(ginfo[glyphStart].codepoint) &&
             gfxTextRun::CompressedGlyph::IsSimpleAdvance(advance) &&
-            aTextRun->IsClusterStart(aTextRunOffset + baseCharIndex) &&
-            posInfo[glyphStart].x_offset == 0 &&
+            aShapedWord->IsClusterStart(baseCharIndex) &&
+            xOffset == 0 &&
             posInfo[glyphStart].y_offset == 0 && yPos == 0)
         {
             gfxTextRun::CompressedGlyph g;
-            aTextRun->SetSimpleGlyph(aTextRunOffset + baseCharIndex,
+            aShapedWord->SetSimpleGlyph(baseCharIndex,
                                      g.SetSimpleGlyph(advance,
                                          ginfo[glyphStart].codepoint));
         } else {
@@ -1141,39 +1171,48 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
                     detailedGlyphs.AppendElement();
                 details->mGlyphID = ginfo[glyphStart].codepoint;
 
-                // Rounding offsets independently of advances on the assumption
-                // that clusters use offsets and rounding of offsets should
-                // not accumulate, and that advances are typically between
-                // clusters.
-                hb_position_t x_offset = posInfo[glyphStart].x_offset;
-                details->mXOffset =
-                    roundX ? dev2appUnits * FixedToIntRound(x_offset)
-                    : NS_floor(hb2appUnits * x_offset + 0.5);
+                details->mXOffset = xOffset;
+                details->mAdvance = advance;
+
                 hb_position_t y_offset = posInfo[glyphStart].y_offset;
                 details->mYOffset = yPos -
-                    (roundY ? dev2appUnits * FixedToIntRound(y_offset)
-                     : NS_floor(hb2appUnits * y_offset + 0.5));
+                    (roundY ? appUnitsPerDevUnit * FixedToIntRound(y_offset)
+                     : floor(hb2appUnits * y_offset + 0.5));
 
-                details->mAdvance = advance;
                 hb_position_t y_advance = posInfo[glyphStart].y_advance;
                 if (y_advance != 0) {
                     yPos -=
-                        roundY ? dev2appUnits * FixedToIntRound(y_advance)
-                        : NS_floor(hb2appUnits * y_advance + 0.5);
+                        roundY ? appUnitsPerDevUnit * FixedToIntRound(y_advance)
+                        : floor(hb2appUnits * y_advance + 0.5);
                 }
                 if (++glyphStart >= glyphEnd) {
                     break;
                 }
+
+                x_offset = posInfo[glyphStart].x_offset;
                 x_advance = posInfo[glyphStart].x_advance;
-                advance =
-                    roundX ? dev2appUnits * FixedToIntRound(x_advance)
-                    : NS_floor(hb2appUnits * x_advance + 0.5);
+                if (roundX) {
+                    xOffset = appUnitsPerDevUnit *
+                        FixedToIntRound(x_offset + x_residual);
+                    // Desired distance to the next reference point.  The
+                    // residual is considered here, and includes the residual
+                    // from the base glyph offset and subsequent advances, so
+                    // that the distance from the base glyph is optimized
+                    // rather than the distance from combining marks.
+                    x_advance += x_residual;
+                    int intAdvance = FixedToIntRound(x_advance);
+                    x_residual = x_advance - FloatToFixed(intAdvance);
+                    advance = appUnitsPerDevUnit * intAdvance;
+                } else {
+                    xOffset = floor(hb2appUnits * x_offset + 0.5);
+                    advance = floor(hb2appUnits * x_advance + 0.5);
+                }
             }
 
             gfxTextRun::CompressedGlyph g;
-            g.SetComplex(aTextRun->IsClusterStart(aTextRunOffset + baseCharIndex),
-                         PR_TRUE, detailedGlyphs.Length());
-            aTextRun->SetGlyphs(aTextRunOffset + baseCharIndex,
+            g.SetComplex(aShapedWord->IsClusterStart(baseCharIndex),
+                         true, detailedGlyphs.Length());
+            aShapedWord->SetGlyphs(baseCharIndex,
                                 g, detailedGlyphs.Elements());
 
             detailedGlyphs.Clear();
@@ -1182,12 +1221,12 @@ gfxHarfBuzzShaper::SetGlyphsFromRun(gfxContext *aContext,
         // the rest of the chars in the group are ligature continuations,
         // no associated glyphs
         while (++baseCharIndex != endCharIndex &&
-               baseCharIndex < PRInt32(aRunLength)) {
+               baseCharIndex < PRInt32(wordLength)) {
             gfxTextRun::CompressedGlyph g;
             g.SetComplex(inOrder &&
-                         aTextRun->IsClusterStart(aTextRunOffset + baseCharIndex),
-                         PR_FALSE, 0);
-            aTextRun->SetGlyphs(aTextRunOffset + baseCharIndex, g, nsnull);
+                         aShapedWord->IsClusterStart(baseCharIndex),
+                         false, 0);
+            aShapedWord->SetGlyphs(baseCharIndex, g, nsnull);
         }
 
         glyphStart = glyphEnd;
