@@ -55,11 +55,11 @@
 using namespace mozilla;
 
 #ifdef PR_LOGGING
-static PRLogModuleInfo *gUserFontsLog = PR_NewLogModule("userfonts");
+PRLogModuleInfo *gfxUserFontSet::sUserFontsLog = PR_NewLogModule("userfonts");
 #endif /* PR_LOGGING */
 
-#define LOG(args) PR_LOG(gUserFontsLog, PR_LOG_DEBUG, args)
-#define LOG_ENABLED() PR_LOG_TEST(gUserFontsLog, PR_LOG_DEBUG)
+#define LOG(args) PR_LOG(sUserFontsLog, PR_LOG_DEBUG, args)
+#define LOG_ENABLED() PR_LOG_TEST(sUserFontsLog, PR_LOG_DEBUG)
 
 static PRUint64 sFontSetGeneration = LL_INIT(0, 0);
 
@@ -74,9 +74,10 @@ gfxProxyFontEntry::gfxProxyFontEntry(const nsTArray<gfxFontFaceSrc>& aFontFaceSr
              PRUint32 aLanguageOverride,
              gfxSparseBitSet *aUnicodeRanges)
     : gfxFontEntry(NS_LITERAL_STRING("Proxy"), aFamily),
-      mLoadingState(NOT_LOADING)
+      mLoadingState(NOT_LOADING),
+      mUnsupportedFormat(false)
 {
-    mIsProxy = PR_TRUE;
+    mIsProxy = true;
     mSrcList = aFontFaceSrcList;
     mSrcIndex = 0;
     mWeight = aWeight;
@@ -84,7 +85,7 @@ gfxProxyFontEntry::gfxProxyFontEntry(const nsTArray<gfxFontFaceSrc>& aFontFaceSr
     mItalic = (aItalicStyle & (FONT_STYLE_ITALIC | FONT_STYLE_OBLIQUE)) != 0;
     mFeatureSettings.AppendElements(aFeatureSettings);
     mLanguageOverride = aLanguageOverride;
-    mIsUserFont = PR_TRUE;
+    mIsUserFont = true;
 }
 
 gfxProxyFontEntry::~gfxProxyFontEntry()
@@ -92,7 +93,7 @@ gfxProxyFontEntry::~gfxProxyFontEntry()
 }
 
 gfxFont*
-gfxProxyFontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, PRBool aNeedsBold)
+gfxProxyFontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, bool aNeedsBold)
 {
     // cannot create an actual font for a proxy entry
     return nsnull;
@@ -108,7 +109,7 @@ gfxUserFontSet::~gfxUserFontSet()
 {
 }
 
-void
+gfxFontEntry*
 gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
                             const nsTArray<gfxFontFaceSrc>& aFontFaceSrcList,
                             PRUint32 aWeight,
@@ -118,10 +119,12 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
                             const nsString& aLanguageOverride,
                             gfxSparseBitSet *aUnicodeRanges)
 {
+    gfxProxyFontEntry *proxyEntry = nsnull;
+
     nsAutoString key(aFamilyName);
     ToLowerCase(key);
 
-    PRBool found;
+    bool found;
 
     if (aWeight == 0)
         aWeight = FONT_WEIGHT_NORMAL;
@@ -135,48 +138,66 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     }
 
     // construct a new face and add it into the family
-    if (family) {
-        nsTArray<gfxFontFeature> featureSettings;
-        gfxFontStyle::ParseFontFeatureSettings(aFeatureSettings,
-                                               featureSettings);
-        PRUint32 languageOverride =
-            gfxFontStyle::ParseFontLanguageOverride(aLanguageOverride);
-        gfxProxyFontEntry *proxyEntry = 
-            new gfxProxyFontEntry(aFontFaceSrcList, family, aWeight, aStretch, 
-                                  aItalicStyle,
-                                  featureSettings,
-                                  languageOverride,
-                                  aUnicodeRanges);
-        family->AddFontEntry(proxyEntry);
+    nsTArray<gfxFontFeature> featureSettings;
+    gfxFontStyle::ParseFontFeatureSettings(aFeatureSettings,
+                                           featureSettings);
+    PRUint32 languageOverride =
+        gfxFontStyle::ParseFontLanguageOverride(aLanguageOverride);
+    proxyEntry =
+        new gfxProxyFontEntry(aFontFaceSrcList, family, aWeight, aStretch,
+                              aItalicStyle,
+                              featureSettings,
+                              languageOverride,
+                              aUnicodeRanges);
+    family->AddFontEntry(proxyEntry);
 #ifdef PR_LOGGING
-        if (LOG_ENABLED()) {
-            LOG(("userfonts (%p) added (%s) with style: %s weight: %d stretch: %d", 
-                 this, NS_ConvertUTF16toUTF8(aFamilyName).get(), 
-                 (aItalicStyle & FONT_STYLE_ITALIC ? "italic" : 
-                     (aItalicStyle & FONT_STYLE_OBLIQUE ? "oblique" : "normal")), 
-                 aWeight, aStretch));
-        }
-#endif
+    if (LOG_ENABLED()) {
+        LOG(("userfonts (%p) added (%s) with style: %s weight: %d stretch: %d",
+             this, NS_ConvertUTF16toUTF8(aFamilyName).get(),
+             (aItalicStyle & FONT_STYLE_ITALIC ? "italic" :
+                 (aItalicStyle & FONT_STYLE_OBLIQUE ? "oblique" : "normal")),
+             aWeight, aStretch));
     }
+#endif
+
+    return proxyEntry;
+}
+
+void
+gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
+                            gfxFontEntry     *aFontEntry)
+{
+    nsAutoString key(aFamilyName);
+    ToLowerCase(key);
+
+    bool found;
+
+    gfxMixedFontFamily *family = mFontFamilies.GetWeak(key, &found);
+    if (!family) {
+        family = new gfxMixedFontFamily(aFamilyName);
+        mFontFamilies.Put(key, family);
+    }
+
+    family->AddFontEntry(aFontEntry);
 }
 
 gfxFontEntry*
 gfxUserFontSet::FindFontEntry(const nsAString& aName, 
                               const gfxFontStyle& aFontStyle, 
-                              PRBool& aFoundFamily,
-                              PRBool& aNeedsBold,
-                              PRBool& aWaitForUserFont)
+                              bool& aFoundFamily,
+                              bool& aNeedsBold,
+                              bool& aWaitForUserFont)
 {
-    aWaitForUserFont = PR_FALSE;
+    aWaitForUserFont = false;
     gfxMixedFontFamily *family = GetFamily(aName);
 
     // no user font defined for this name
     if (!family) {
-        aFoundFamily = PR_FALSE;
+        aFoundFamily = false;
         return nsnull;
     }
 
-    aFoundFamily = PR_TRUE;
+    aFoundFamily = true;
     gfxFontEntry* fe = family->FindFontForStyle(aFontStyle, aNeedsBold);
 
     // if not a proxy, font has already been loaded
@@ -343,8 +364,12 @@ SanitizeOpenTypeData(const PRUint8* aData, PRUint32 aLength,
     // limit output/expansion to 256MB
     ExpandingMemoryStream output(aIsCompressed ? aLength * 2 : aLength,
                                  1024 * 1024 * 256);
-    if (ots::Process(&output, aData, aLength,
-        gfxPlatform::GetPlatform()->PreserveOTLTablesWhenSanitizing())) {
+#ifdef MOZ_GRAPHITE
+#define PRESERVE_GRAPHITE true
+#else
+#define PRESERVE_GRAPHITE false
+#endif
+    if (ots::Process(&output, aData, aLength, PRESERVE_GRAPHITE)) {
         aSaneLength = output.Tell();
         return static_cast<PRUint8*>(output.forget());
     } else {
@@ -353,159 +378,133 @@ SanitizeOpenTypeData(const PRUint8* aData, PRUint32 aLength,
     }
 }
 
-// Find the GDEF, GSUB, GPOS tables in aFontData (if present)
-// and cache copies in the given font entry.
-// The sfnt table directory has already been accepted by the OTS
-// sanitizer before this is called, so we can assume entries are valid.
-//
-// This is a temporary workaround until OTS has full support for the
-// G*** tables, so that they can safely be left in the main font.
-// When http://code.google.com/p/chromium/issues/detail?id=27131 gets fixed,
-// we should remove this hack.
 static void
-CacheLayoutTablesFromSFNT(const PRUint8* aFontData, PRUint32 aLength,
-                          gfxFontEntry* aFontEntry)
+StoreUserFontData(gfxFontEntry* aFontEntry, gfxProxyFontEntry* aProxy,
+                  const nsAString& aOriginalName,
+                  nsTArray<PRUint8>* aMetadata, PRUint32 aMetaOrigLen)
 {
-    const SFNTHeader *sfntHeader = reinterpret_cast<const SFNTHeader*>(aFontData);
-    PRUint16 numTables = sfntHeader->numTables;
-    
-    // table directory entries begin immediately following SFNT header
-    const TableDirEntry *dirEntry = 
-        reinterpret_cast<const TableDirEntry*>(aFontData + sizeof(SFNTHeader));
-
-    while (numTables-- > 0) {
-        switch (dirEntry->tag) {
-        case TRUETYPE_TAG('G','D','E','F'):
-        case TRUETYPE_TAG('G','P','O','S'):
-        case TRUETYPE_TAG('G','S','U','B'): {
-                FallibleTArray<PRUint8> buffer;
-                if (!buffer.AppendElements(aFontData + dirEntry->offset,
-                                           dirEntry->length)) {
-                    NS_WARNING("failed to cache font table - out of memory?");
-                    break;
-                }
-                aFontEntry->PreloadFontTable(dirEntry->tag, buffer);
-            }
-            break;
-
-        default:
-            if (dirEntry->tag > TRUETYPE_TAG('G','S','U','B')) {
-                // directory entries are required to be sorted,
-                // so we can terminate as soon as we find a tag > 'GSUB'
-                numTables = 0;
-            }
-            break;
-        }
-        ++dirEntry;
+    if (!aFontEntry->mUserFontData) {
+        aFontEntry->mUserFontData = new gfxUserFontData;
     }
-}
-
-// OTS drops the OT Layout tables when decoding a WOFF file, so retrieve them
-// separately and cache them (unchecked) in the font entry; harfbuzz will
-// sanitize them when it needs to use them.
-static void
-PreloadTableFromWOFF(const PRUint8* aFontData, PRUint32 aLength,
-                     PRUint32 aTableTag, gfxFontEntry* aFontEntry)
-{
-    PRUint32 status = eWOFF_ok;
-    PRUint32 len = woffGetTableSize(aFontData, aLength, aTableTag, &status);
-    if (WOFF_SUCCESS(status) && len > 0) {
-        FallibleTArray<PRUint8> buffer;
-        if (!buffer.AppendElements(len)) {
-            NS_WARNING("failed to cache font table - out of memory?");
-            return;
-        }
-        woffGetTableToBuffer(aFontData, aLength, aTableTag,
-                             buffer.Elements(), buffer.Length(),
-                             &len, &status);
-        if (WOFF_FAILURE(status)) {
-            NS_WARNING("failed to cache font table - WOFF decoding error?");
-            return;
-        }
-        aFontEntry->PreloadFontTable(aTableTag, buffer);
+    gfxUserFontData* userFontData = aFontEntry->mUserFontData;
+    userFontData->mSrcIndex = aProxy->mSrcIndex;
+    const gfxFontFaceSrc& src = aProxy->mSrcList[aProxy->mSrcIndex];
+    if (src.mIsLocal) {
+        userFontData->mLocalName = src.mLocalName;
+    } else {
+        userFontData->mURI = src.mURI;
+    }
+    userFontData->mFormat = src.mFormatFlags;
+    userFontData->mRealName = aOriginalName;
+    if (aMetadata) {
+        userFontData->mMetadata.SwapElements(*aMetadata);
+        userFontData->mMetaOrigLen = aMetaOrigLen;
     }
 }
 
 static void
-CacheLayoutTablesFromWOFF(const PRUint8* aFontData, PRUint32 aLength,
-                          gfxFontEntry* aFontEntry)
+CopyWOFFMetadata(const PRUint8* aFontData, PRUint32 aLength,
+                 nsTArray<PRUint8>* aMetadata, PRUint32* aMetaOrigLen)
 {
-    PreloadTableFromWOFF(aFontData, aLength, TRUETYPE_TAG('G','D','E','F'),
-                         aFontEntry);
-    PreloadTableFromWOFF(aFontData, aLength, TRUETYPE_TAG('G','P','O','S'),
-                         aFontEntry);
-    PreloadTableFromWOFF(aFontData, aLength, TRUETYPE_TAG('G','S','U','B'),
-                         aFontEntry);
+    // This function may be called with arbitrary, unvalidated "font" data
+    // from @font-face, so it needs to be careful to bounds-check, etc.,
+    // before trying to read anything.
+    // This just saves a copy of the compressed data block; it does NOT check
+    // that the block can be successfully decompressed, or that it contains
+    // well-formed/valid XML metadata.
+    struct WOFFHeader {
+        AutoSwap_PRUint32 signature;
+        AutoSwap_PRUint32 flavor;
+        AutoSwap_PRUint32 length;
+        AutoSwap_PRUint16 numTables;
+        AutoSwap_PRUint16 reserved;
+        AutoSwap_PRUint32 totalSfntSize;
+        AutoSwap_PRUint16 majorVersion;
+        AutoSwap_PRUint16 minorVersion;
+        AutoSwap_PRUint32 metaOffset;
+        AutoSwap_PRUint32 metaCompLen;
+        AutoSwap_PRUint32 metaOrigLen;
+        AutoSwap_PRUint32 privOffset;
+        AutoSwap_PRUint32 privLen;
+    };
+    if (aLength < sizeof(WOFFHeader)) {
+        return;
+    }
+    const WOFFHeader* woff = reinterpret_cast<const WOFFHeader*>(aFontData);
+    PRUint32 metaOffset = woff->metaOffset;
+    PRUint32 metaCompLen = woff->metaCompLen;
+    if (!metaOffset || !metaCompLen || !woff->metaOrigLen) {
+        return;
+    }
+    if (metaOffset >= aLength || metaCompLen > aLength - metaOffset) {
+        return;
+    }
+    if (!aMetadata->SetLength(woff->metaCompLen)) {
+        return;
+    }
+    memcpy(aMetadata->Elements(), aFontData + metaOffset, metaCompLen);
+    *aMetaOrigLen = woff->metaOrigLen;
 }
 
 // This is called when a font download finishes.
 // Ownership of aFontData passes in here, and the font set must
 // ensure that it is eventually deleted via NS_Free().
-PRBool 
-gfxUserFontSet::OnLoadComplete(gfxFontEntry *aFontToLoad,
-                               const PRUint8 *aFontData, PRUint32 aLength, 
+bool 
+gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
+                               const PRUint8 *aFontData, PRUint32 aLength,
                                nsresult aDownloadStatus)
 {
-    NS_ASSERTION(aFontToLoad->mIsProxy,
-                 "trying to load font data for wrong font entry type");
-
-    if (!aFontToLoad->mIsProxy) {
-        NS_Free((void*)aFontData);
-        return PR_FALSE;
-    }
-
-    gfxProxyFontEntry *pe = static_cast<gfxProxyFontEntry*> (aFontToLoad);
-
     // download successful, make platform font using font data
     if (NS_SUCCEEDED(aDownloadStatus)) {
         gfxFontEntry *fe = nsnull;
 
+        gfxUserFontType fontType =
+            gfxFontUtils::DetermineFontDataType(aFontData, aLength);
+
+        // Save a copy of the metadata block (if present) for nsIDOMFontFace
+        // to use if required. Ownership of the metadata block will be passed
+        // to the gfxUserFontData record below.
+        // NOTE: after the non-OTS codepath using PrepareOpenTypeData is
+        // removed, we should defer this until after we've created the new
+        // fontEntry.
+        nsTArray<PRUint8> metadata;
+        PRUint32 metaOrigLen = 0;
+        if (fontType == GFX_USERFONT_WOFF) {
+            CopyWOFFMetadata(aFontData, aLength, &metadata, &metaOrigLen);
+        }
+
         // Unwrap/decompress/sanitize or otherwise munge the downloaded data
         // to make a usable sfnt structure.
 
-        if (gfxPlatform::GetPlatform()->SanitizeDownloadedFonts()) {
-            gfxUserFontType fontType =
-                gfxFontUtils::DetermineFontDataType(aFontData, aLength);
+        // Because platform font activation code may replace the name table
+        // in the font with a synthetic one, we save the original name so that
+        // it can be reported via the nsIDOMFontFace API.
+        nsAutoString originalFullName;
 
-            // Call the OTS sanitizer; this will also decode WOFF to sfnt
+        if (gfxPlatform::GetPlatform()->SanitizeDownloadedFonts()) {
+           // Call the OTS sanitizer; this will also decode WOFF to sfnt
             // if necessary. The original data in aFontData is left unchanged.
             PRUint32 saneLen;
             const PRUint8* saneData =
                 SanitizeOpenTypeData(aFontData, aLength, saneLen,
                                      fontType == GFX_USERFONT_WOFF);
-#ifdef DEBUG
             if (!saneData) {
-                char buf[1000];
-                sprintf(buf, "downloaded font rejected for \"%s\"",
-                        NS_ConvertUTF16toUTF8(pe->FamilyName()).get());
-                NS_WARNING(buf);
+                LogMessage(aProxy, "rejected by sanitizer");
             }
-#endif
             if (saneData) {
+                // The sanitizer ensures that we have a valid sfnt and a usable
+                // name table, so this should never fail unless we're out of
+                // memory, and GetFullNameFromSFNT is not directly exposed to
+                // arbitrary/malicious data from the web.
+                gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
+                                                  originalFullName);
                 // Here ownership of saneData is passed to the platform,
                 // which will delete it when no longer required
-                fe = gfxPlatform::GetPlatform()->MakePlatformFont(pe,
+                fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
                                                                   saneData,
                                                                   saneLen);
-                if (fe) {
-                    // if aFontData includes OpenType layout tables, we need to
-                    // cache them in the font entry for harfbuzz to use,
-                    // as they will have been dropped from the sanitized sfnt
-                    // (temporary hack, see CacheLayoutTablesFromSFNT)
-                    switch (fontType) {
-                    case GFX_USERFONT_OPENTYPE:
-                        CacheLayoutTablesFromSFNT(aFontData, aLength, fe);
-                        break;
-
-                    case GFX_USERFONT_WOFF:
-                        CacheLayoutTablesFromWOFF(aFontData, aLength, fe);
-                        break;
-
-                    default:
-                        break;
-                    }
-                } else {
-                    NS_WARNING("failed to make platform font from download");
+                if (!fe) {
+                    LogMessage(aProxy, "not usable by platform");
                 }
             }
         } else {
@@ -516,16 +515,23 @@ gfxUserFontSet::OnLoadComplete(gfxFontEntry *aFontToLoad,
 
             if (aFontData) {
                 if (gfxFontUtils::ValidateSFNTHeaders(aFontData, aLength)) {
+                    // ValidateSFNTHeaders has checked that we have a valid
+                    // sfnt structure and a usable 'name' table
+                    gfxFontUtils::GetFullNameFromSFNT(aFontData, aLength,
+                                                      originalFullName);
                     // Here ownership of aFontData is passed to the platform,
                     // which will delete it when no longer required
-                    fe = gfxPlatform::GetPlatform()->MakePlatformFont(pe,
+                    fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
                                                                       aFontData,
                                                                       aLength);
+                    if (!fe) {
+                        LogMessage(aProxy, "not usable by platform");
+                    }
                     aFontData = nsnull; // we must NOT free this below!
                 } else {
                     // the data was unusable, so just discard it
                     // (error will be reported below, if logging is enabled)
-                    NS_WARNING("failed to make platform font from download");
+                    LogMessage(aProxy, "SFNT header or tables invalid");
                 }
             }
         }
@@ -538,47 +544,40 @@ gfxUserFontSet::OnLoadComplete(gfxFontEntry *aFontToLoad,
         if (fe) {
             // copy OpenType feature/language settings from the proxy to the
             // newly-created font entry
-            fe->mFeatureSettings.AppendElements(pe->mFeatureSettings);
-            fe->mLanguageOverride = pe->mLanguageOverride;
-
+            fe->mFeatureSettings.AppendElements(aProxy->mFeatureSettings);
+            fe->mLanguageOverride = aProxy->mLanguageOverride;
+            StoreUserFontData(fe, aProxy, originalFullName,
+                              &metadata, metaOrigLen);
 #ifdef PR_LOGGING
             // must do this before ReplaceFontEntry() because that will
-            // destroy the proxy!
+            // clear the proxy's mFamily pointer!
             if (LOG_ENABLED()) {
                 nsCAutoString fontURI;
-                pe->mSrcList[pe->mSrcIndex].mURI->GetSpec(fontURI);
+                aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
                 LOG(("userfonts (%p) [src %d] loaded uri: (%s) for (%s) gen: %8.8x\n",
-                     this, pe->mSrcIndex, fontURI.get(),
-                     NS_ConvertUTF16toUTF8(pe->mFamily->Name()).get(),
+                     this, aProxy->mSrcIndex, fontURI.get(),
+                     NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get(),
                      PRUint32(mGeneration)));
             }
 #endif
-            static_cast<gfxMixedFontFamily*>(pe->mFamily)->ReplaceFontEntry(pe, fe);
+            ReplaceFontEntry(aProxy, fe);
             IncrementGeneration();
-            return PR_TRUE;
+            return true;
         } else {
 #ifdef PR_LOGGING
             if (LOG_ENABLED()) {
                 nsCAutoString fontURI;
-                pe->mSrcList[pe->mSrcIndex].mURI->GetSpec(fontURI);
+                aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
                 LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s) error making platform font\n",
-                     this, pe->mSrcIndex, fontURI.get(),
-                     NS_ConvertUTF16toUTF8(pe->mFamily->Name()).get()));
+                     this, aProxy->mSrcIndex, fontURI.get(),
+                     NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get()));
             }
 #endif
         }
     } else {
         // download failed
-#ifdef PR_LOGGING
-        if (LOG_ENABLED()) {
-            nsCAutoString fontURI;
-            pe->mSrcList[pe->mSrcIndex].mURI->GetSpec(fontURI);
-            LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s) error %8.8x downloading font data\n",
-                 this, pe->mSrcIndex, fontURI.get(),
-                 NS_ConvertUTF16toUTF8(pe->mFamily->Name()).get(),
-                 aDownloadStatus));
-        }
-#endif
+        LogMessage(aProxy, "download failed", nsIScriptError::errorFlag,
+                   aDownloadStatus);
     }
 
     if (aFontData) {
@@ -586,15 +585,14 @@ gfxUserFontSet::OnLoadComplete(gfxFontEntry *aFontToLoad,
     }
 
     // error occurred, load next src
-    LoadStatus status;
+    (void)LoadNext(aProxy);
 
-    status = LoadNext(pe);
-
-    // Even if loading failed, we need to bump the font-set generation
+    // We ignore the status returned by LoadNext();
+    // even if loading failed, we need to bump the font-set generation
     // and return true in order to trigger reflow, so that fallback
     // will be used where the text was "masked" by the pending download
     IncrementGeneration();
-    return PR_TRUE;
+    return true;
 }
 
 
@@ -603,10 +601,12 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
 {
     PRUint32 numSrc = aProxyEntry->mSrcList.Length();
 
-    NS_ASSERTION(aProxyEntry->mSrcIndex < numSrc, "already at the end of the src list for user font");
+    NS_ASSERTION(aProxyEntry->mSrcIndex < numSrc,
+                 "already at the end of the src list for user font");
 
     if (aProxyEntry->mLoadingState == gfxProxyFontEntry::NOT_LOADING) {
         aProxyEntry->mLoadingState = gfxProxyFontEntry::LOADING_STARTED;
+        aProxyEntry->mUnsupportedFormat = false;
     } else {
         // we were already loading; move to the next source,
         // but don't reset state - if we've already timed out,
@@ -614,82 +614,75 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
         aProxyEntry->mSrcIndex++;
     }
 
-    // load each src entry in turn, until a local face is found or a download begins successfully
+    // load each src entry in turn, until a local face is found
+    // or a download begins successfully
     while (aProxyEntry->mSrcIndex < numSrc) {
         const gfxFontFaceSrc& currSrc = aProxyEntry->mSrcList[aProxyEntry->mSrcIndex];
 
-        // src local ==> lookup and load   
+        // src local ==> lookup and load
 
         if (currSrc.mIsLocal) {
             gfxFontEntry *fe =
                 gfxPlatform::GetPlatform()->LookupLocalFont(aProxyEntry,
                                                             currSrc.mLocalName);
             if (fe) {
-                LOG(("userfonts (%p) [src %d] loaded local: (%s) for (%s) gen: %8.8x\n", 
-                     this, aProxyEntry->mSrcIndex, 
-                     NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(), 
-                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get(), 
+                LOG(("userfonts (%p) [src %d] loaded local: (%s) for (%s) gen: %8.8x\n",
+                     this, aProxyEntry->mSrcIndex,
+                     NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(),
+                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get(),
                      PRUint32(mGeneration)));
                 fe->mFeatureSettings.AppendElements(aProxyEntry->mFeatureSettings);
                 fe->mLanguageOverride = aProxyEntry->mLanguageOverride;
-                static_cast<gfxMixedFontFamily*>(aProxyEntry->mFamily)->ReplaceFontEntry(aProxyEntry, fe);
+                StoreUserFontData(fe, aProxyEntry, nsString(), nsnull, 0);
+                ReplaceFontEntry(aProxyEntry, fe);
                 return STATUS_LOADED;
             } else {
-                LOG(("userfonts (%p) [src %d] failed local: (%s) for (%s)\n", 
-                     this, aProxyEntry->mSrcIndex, 
-                     NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(), 
-                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));            
+                LOG(("userfonts (%p) [src %d] failed local: (%s) for (%s)\n",
+                     this, aProxyEntry->mSrcIndex,
+                     NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(),
+                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
             }
-        } 
+        }
 
         // src url ==> start the load process
         else {
-            if (gfxPlatform::GetPlatform()->IsFontFormatSupported(currSrc.mURI, 
+            if (gfxPlatform::GetPlatform()->IsFontFormatSupported(currSrc.mURI,
                     currSrc.mFormatFlags)) {
                 nsresult rv = StartLoad(aProxyEntry, &currSrc);
-                PRBool loadOK = NS_SUCCEEDED(rv);
-                
+                bool loadOK = NS_SUCCEEDED(rv);
                 if (loadOK) {
 #ifdef PR_LOGGING
                     if (LOG_ENABLED()) {
                         nsCAutoString fontURI;
                         currSrc.mURI->GetSpec(fontURI);
-                        LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n", 
-                             this, aProxyEntry->mSrcIndex, fontURI.get(), 
+                        LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n",
+                             this, aProxyEntry->mSrcIndex, fontURI.get(),
                              NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
                     }
 #endif
-                    return STATUS_LOADING;                  
+                    return STATUS_LOADING;
                 } else {
-#ifdef PR_LOGGING
-                    if (LOG_ENABLED()) {
-                        nsCAutoString fontURI;
-                        currSrc.mURI->GetSpec(fontURI);
-                        LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s) download failed\n", 
-                             this, aProxyEntry->mSrcIndex, fontURI.get(), 
-                             NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
-                    }
-#endif
+                    LogMessage(aProxyEntry, "download failed",
+                               nsIScriptError::errorFlag, rv);
                 }
             } else {
-#ifdef PR_LOGGING
-                if (LOG_ENABLED()) {
-                    nsCAutoString fontURI;
-                    currSrc.mURI->GetSpec(fontURI);
-                    LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s) format not supported\n", 
-                         this, aProxyEntry->mSrcIndex, fontURI.get(), 
-                         NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
-                }
-#endif
+                // We don't log a warning to the web console yet,
+                // as another source may load successfully
+                aProxyEntry->mUnsupportedFormat = true;
             }
         }
 
         aProxyEntry->mSrcIndex++;
     }
 
+    if (aProxyEntry->mUnsupportedFormat) {
+        LogMessage(aProxyEntry, "no supported format found",
+                   nsIScriptError::warningFlag);
+    }
+
     // all src's failed; mark this entry as unusable (so fallback will occur)
-    LOG(("userfonts (%p) failed all src for (%s)\n", 
-        this, NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));            
+    LOG(("userfonts (%p) failed all src for (%s)\n",
+        this, NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
     aProxyEntry->mLoadingState = gfxProxyFontEntry::LOADING_FAILED;
 
     return STATUS_END_OF_LIST;
@@ -714,13 +707,3 @@ gfxUserFontSet::GetFamily(const nsAString& aFamilyName) const
 
     return mFontFamilies.GetWeak(key);
 }
-
-
-void 
-gfxUserFontSet::RemoveFamily(const nsAString& aFamilyName)
-{
-    nsAutoString key(aFamilyName);
-    ToLowerCase(key);
-
-    mFontFamilies.Remove(key);
-}
