@@ -38,11 +38,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Util.h"
+
 #include "gfxWindowsPlatform.h"
 
 #include "gfxImageSurface.h"
 #include "gfxWindowsSurface.h"
-#include "gfxTextRunWordCache.h"
 
 #include "nsUnicharUtils.h"
 
@@ -73,12 +74,14 @@
 
 #include <string>
 
+using namespace mozilla;
 using namespace mozilla::gfx;
 
 #ifdef CAIRO_HAS_D2D_SURFACE
 #include "gfxD2DSurface.h"
 
 #include <d3d10_1.h>
+#include <dxgi.h>
 
 #include "mozilla/gfx/2D.h"
 
@@ -136,9 +139,9 @@ NS_MEMORY_REPORTER_IMPLEMENT(
 // DirectWrite is not available on all platforms, we need to use the function
 // pointer.
 typedef HRESULT (WINAPI*DWriteCreateFactoryFunc)(
-  __in   DWRITE_FACTORY_TYPE factoryType,
-  __in   REFIID iid,
-  __out  IUnknown **factory
+  DWRITE_FACTORY_TYPE factoryType,
+  REFIID iid,
+  IUnknown **factory
 );
 #endif
 
@@ -152,6 +155,11 @@ typedef HRESULT (WINAPI*D3D10CreateDevice1Func)(
   UINT SDKVersion,
   ID3D10Device1 **ppDevice
 );
+
+typedef HRESULT(WINAPI*CreateDXGIFactory1Func)(
+  REFIID riid,
+  void **ppFactory
+);
 #endif
 
 static __inline void
@@ -169,7 +177,7 @@ gfxWindowsPlatform::gfxWindowsPlatform()
     mUseClearTypeForDownloadableFonts = UNINITIALIZED_VALUE;
     mUseClearTypeAlways = UNINITIALIZED_VALUE;
 
-    mUsingGDIFonts = PR_FALSE;
+    mUsingGDIFonts = false;
 
     /* 
      * Initialize COM 
@@ -217,35 +225,30 @@ gfxWindowsPlatform::UpdateRenderMode()
     ::GetVersionExA(&versionInfo);
     bool isVistaOrHigher = versionInfo.dwMajorVersion >= 6;
 
-    PRBool safeMode = PR_FALSE;
+    bool safeMode = false;
     nsCOMPtr<nsIXULRuntime> xr = do_GetService("@mozilla.org/xre/runtime;1");
     if (xr)
       xr->GetInSafeMode(&safeMode);
 
-    mUseDirectWrite = Preferences::GetBool("gfx.font_rendering.directwrite.enabled", PR_FALSE);
+    mUseDirectWrite = Preferences::GetBool("gfx.font_rendering.directwrite.enabled", false);
 
 #ifdef CAIRO_HAS_D2D_SURFACE
-    PRBool d2dDisabled = PR_FALSE;
-    PRBool d2dForceEnabled = PR_FALSE;
-    PRBool d2dBlocked = PR_FALSE;
+    bool d2dDisabled = false;
+    bool d2dForceEnabled = false;
+    bool d2dBlocked = false;
 
     nsCOMPtr<nsIGfxInfo> gfxInfo = do_GetService("@mozilla.org/gfx/info;1");
     if (gfxInfo) {
         PRInt32 status;
         if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_DIRECT2D, &status))) {
             if (status != nsIGfxInfo::FEATURE_NO_INFO) {
-                d2dDisabled = PR_TRUE;
-                if (status == nsIGfxInfo::FEATURE_BLOCKED_DRIVER_VERSION ||
-                    status == nsIGfxInfo::FEATURE_BLOCKED_DEVICE)
-                {
-                    d2dBlocked = PR_TRUE;
-                }
+                d2dBlocked = true;
             }
         }
     }
 
-    d2dDisabled = Preferences::GetBool("gfx.direct2d.disabled", PR_FALSE);
-    d2dForceEnabled = Preferences::GetBool("gfx.direct2d.force-enabled", PR_FALSE);
+    d2dDisabled = Preferences::GetBool("gfx.direct2d.disabled", false);
+    d2dForceEnabled = Preferences::GetBool("gfx.direct2d.force-enabled", false);
 
     bool tryD2D = !d2dBlocked || d2dForceEnabled;
     
@@ -259,7 +262,7 @@ gfxWindowsPlatform::UpdateRenderMode()
         VerifyD2DDevice(d2dForceEnabled);
         if (mD2DDevice) {
             mRenderMode = RENDER_DIRECT2D;
-            mUseDirectWrite = PR_TRUE;
+            mUseDirectWrite = true;
         }
     } else {
         mD2DDevice = nsnull;
@@ -288,6 +291,11 @@ gfxWindowsPlatform::UpdateRenderMode()
             mDWriteFactory = factory;
             factory->Release();
 
+            if (SUCCEEDED(hr)) {
+                hr = mDWriteFactory->CreateTextAnalyzer(
+                    getter_AddRefs(mDWriteAnalyzer));
+            }
+
             SetupClearTypeParams();
 
             if (hr == S_OK)
@@ -298,7 +306,7 @@ gfxWindowsPlatform::UpdateRenderMode()
 }
 
 void
-gfxWindowsPlatform::VerifyD2DDevice(PRBool aAttemptForce)
+gfxWindowsPlatform::VerifyD2DDevice(bool aAttemptForce)
 {
 #ifdef CAIRO_HAS_D2D_SURFACE
     if (mD2DDevice) {
@@ -310,7 +318,7 @@ gfxWindowsPlatform::VerifyD2DDevice(PRBool aAttemptForce)
         mD2DDevice = nsnull;
     }
 
-    mozilla::ScopedGfxFeatureReporter reporter("D2D");
+    mozilla::ScopedGfxFeatureReporter reporter("D2D", aAttemptForce);
 
     HMODULE d3d10module = LoadLibraryA("d3d10_1.dll");
     D3D10CreateDevice1Func createD3DDevice = (D3D10CreateDevice1Func)
@@ -318,10 +326,40 @@ gfxWindowsPlatform::VerifyD2DDevice(PRBool aAttemptForce)
     nsRefPtr<ID3D10Device1> device;
 
     if (createD3DDevice) {
+        HMODULE dxgiModule = LoadLibraryA("dxgi.dll");
+        CreateDXGIFactory1Func createDXGIFactory1 = (CreateDXGIFactory1Func)
+            GetProcAddress(dxgiModule, "CreateDXGIFactory1");
+
+        // Try to use a DXGI 1.1 adapter in order to share resources
+        // across processes.
+        nsRefPtr<IDXGIAdapter1> adapter1;
+        if (createDXGIFactory1) {
+            nsRefPtr<IDXGIFactory1> factory1;
+            HRESULT hr = createDXGIFactory1(__uuidof(IDXGIFactory1),
+                                            getter_AddRefs(factory1));
+
+            if (FAILED(hr) || !factory1) {
+              // This seems to happen with some people running the iZ3D driver.
+              // They won't get acceleration.
+              return;
+            }
+    
+            nsRefPtr<IDXGIAdapter1> adapter1; 
+            hr = factory1->EnumAdapters1(0, getter_AddRefs(adapter1));
+
+            if (SUCCEEDED(hr) && adapter1) {
+                hr = adapter1->CheckInterfaceSupport(__uuidof(ID3D10Device1),
+                                                     nsnull);
+                if (FAILED(hr)) {
+                    adapter1 = nsnull;
+                }
+            }
+        }
+
         // We try 10.0 first even though we prefer 10.1, since we want to
         // fail as fast as possible if 10.x isn't supported.
         HRESULT hr = createD3DDevice(
-            NULL, 
+            adapter1, 
             D3D10_DRIVER_TYPE_HARDWARE,
             NULL,
             D3D10_CREATE_DEVICE_BGRA_SUPPORT |
@@ -337,7 +375,7 @@ gfxWindowsPlatform::VerifyD2DDevice(PRBool aAttemptForce)
             // clever.
             nsRefPtr<ID3D10Device1> device1;
             hr = createD3DDevice(
-                NULL, 
+                adapter1, 
                 D3D10_DRIVER_TYPE_HARDWARE,
                 NULL,
                 D3D10_CREATE_DEVICE_BGRA_SUPPORT |
@@ -371,7 +409,7 @@ gfxWindowsPlatform::VerifyD2DDevice(PRBool aAttemptForce)
 #ifdef CAIRO_HAS_DWRITE_FONT
 #define WINDOWS7_RTM_BUILD 7600
 
-static PRBool
+static bool
 AllowDirectWrite()
 {
     PRInt32 winVers, buildNum;
@@ -381,17 +419,17 @@ AllowDirectWrite()
         buildNum < WINDOWS7_RTM_BUILD)
     {
         // don't use Direct2D/DirectWrite on older versions of Windows 7
-        return PR_FALSE;
+        return false;
     }
 
-    return PR_TRUE;
+    return true;
 }
 #endif
 
 gfxPlatformFontList*
 gfxWindowsPlatform::CreatePlatformFontList()
 {
-    mUsingGDIFonts = PR_FALSE;
+    mUsingGDIFonts = false;
     gfxPlatformFontList *pfl;
 #ifdef CAIRO_HAS_DWRITE_FONT
     if (AllowDirectWrite() && GetDWriteFactory()) {
@@ -407,7 +445,7 @@ gfxWindowsPlatform::CreatePlatformFontList()
     }
 #endif
     pfl = new gfxGDIFontList();
-    mUsingGDIFonts = PR_TRUE;
+    mUsingGDIFonts = true;
 
     if (NS_SUCCEEDED(pfl->InitFontList())) {
         return pfl;
@@ -456,7 +494,13 @@ gfxWindowsPlatform::GetScaledFontForFont(gfxFont *aFont)
     return scaledFont;
   }
 
-  return NULL;
+  NativeFont nativeFont;
+  nativeFont.mType = NATIVE_FONT_GDI_FONT_FACE;
+  nativeFont.mFont = aFont;
+  RefPtr<ScaledFont> scaledFont =
+    Factory::CreateScaledFontForNativeFont(nativeFont, aFont->GetAdjustedSize());
+
+  return scaledFont;
 }
 
 already_AddRefed<gfxASurface>
@@ -464,27 +508,59 @@ gfxWindowsPlatform::GetThebesSurfaceForDrawTarget(DrawTarget *aTarget)
 {
 #ifdef XP_WIN
   if (aTarget->GetType() == BACKEND_DIRECT2D) {
-    RefPtr<ID3D10Texture2D> texture =
-      static_cast<ID3D10Texture2D*>(aTarget->GetNativeSurface(NATIVE_SURFACE_D3D10_TEXTURE));
+    void *surface = aTarget->GetUserData(&kThebesSurfaceKey);
+    if (surface) {
+      nsRefPtr<gfxASurface> surf = static_cast<gfxASurface*>(surface);
+      return surf.forget();
+    } else {
+      RefPtr<ID3D10Texture2D> texture =
+        static_cast<ID3D10Texture2D*>(aTarget->GetNativeSurface(NATIVE_SURFACE_D3D10_TEXTURE));
 
-    if (!texture) {
-      return gfxPlatform::GetThebesSurfaceForDrawTarget(aTarget);
+      if (!texture) {
+        return gfxPlatform::GetThebesSurfaceForDrawTarget(aTarget);
+      }
+
+      aTarget->Flush();
+
+      nsRefPtr<gfxASurface> surf =
+        new gfxD2DSurface(texture, ContentForFormat(aTarget->GetFormat()));
+
+      // add a reference to be held by the drawTarget
+      surf->AddRef();
+      aTarget->AddUserData(&kThebesSurfaceKey, surf.get(), DestroyThebesSurface);
+      /* "It might be worth it to clear cairo surfaces associated with a drawtarget.
+	  The strong reference means for example for D2D that cairo's scratch surface
+	  will be kept alive (well after a user being done) and consume extra VRAM.
+	  We can deal with this in a follow-up though." */
+
+      // shouldn't this hold a reference?
+      surf->SetData(&kDrawTarget, aTarget, NULL);
+      return surf.forget();
     }
-
-    aTarget->Flush();
-
-    nsRefPtr<gfxASurface> surf =
-      new gfxD2DSurface(texture, ContentForFormat(aTarget->GetFormat()));
-
-    surf->SetData(&kDrawTarget, aTarget, NULL);
-
-    return surf.forget();
   }
 #endif
 
   return gfxPlatform::GetThebesSurfaceForDrawTarget(aTarget);
 }
 
+bool
+gfxWindowsPlatform::SupportsAzure(BackendType& aBackend)
+{
+#ifdef CAIRO_HAS_D2D_SURFACE
+  if (mRenderMode == RENDER_DIRECT2D) {
+      aBackend = BACKEND_DIRECT2D;
+      return true;
+  }
+#endif
+  
+  if (mPreferredDrawTargetBackend != BACKEND_NONE) {
+    aBackend = mPreferredDrawTargetBackend;
+    return true;
+  }
+
+  return false;
+}
+
 nsresult
 gfxWindowsPlatform::GetFontList(nsIAtom *aLangGroup,
                                 const nsACString& aGenericFamily,
@@ -528,12 +604,12 @@ nsresult
 gfxWindowsPlatform::ResolveFontName(const nsAString& aFontName,
                                     FontResolverCallback aCallback,
                                     void *aClosure,
-                                    PRBool& aAborted)
+                                    bool& aAborted)
 {
     nsAutoString resolvedName;
     if (!gfxPlatformFontList::PlatformFontList()->
              ResolveFontName(aFontName, resolvedName)) {
-        aAborted = PR_FALSE;
+        aAborted = false;
         return NS_OK;
     }
     aAborted = !(*aCallback)(resolvedName, aClosure);
@@ -572,7 +648,7 @@ gfxWindowsPlatform::MakePlatformFont(const gfxProxyFontEntry *aProxyEntry,
                                                                      aLength);
 }
 
-PRBool
+bool
 gfxWindowsPlatform::IsFontFormatSupported(nsIURI *aFontURI, PRUint32 aFormatFlags)
 {
     // check for strange format flags
@@ -583,16 +659,16 @@ gfxWindowsPlatform::IsFontFormatSupported(nsIURI *aFontURI, PRUint32 aFormatFlag
     if (aFormatFlags & (gfxUserFontSet::FLAG_FORMAT_WOFF     |
                         gfxUserFontSet::FLAG_FORMAT_OPENTYPE | 
                         gfxUserFontSet::FLAG_FORMAT_TRUETYPE)) {
-        return PR_TRUE;
+        return true;
     }
 
     // reject all other formats, known and unknown
     if (aFormatFlags != 0) {
-        return PR_FALSE;
+        return false;
     }
 
     // no format hint set, need to look at data
-    return PR_TRUE;
+    return true;
 }
 
 gfxFontFamily *
@@ -608,7 +684,7 @@ gfxWindowsPlatform::FindFontEntry(const nsAString& aName, const gfxFontStyle& aF
     if (!ff)
         return nsnull;
 
-    PRBool aNeedsBold;
+    bool aNeedsBold;
     return ff->FindFontForStyle(aFontStyle, aNeedsBold);
 }
 
@@ -647,7 +723,7 @@ gfxWindowsPlatform::GetPlatformCMSOutputProfile()
     return profile;
 }
 
-PRBool
+bool
 gfxWindowsPlatform::GetPrefFontEntries(const nsCString& aKey, nsTArray<nsRefPtr<gfxFontEntry> > *array)
 {
     return mPrefFonts.Get(aKey, array);
@@ -659,21 +735,21 @@ gfxWindowsPlatform::SetPrefFontEntries(const nsCString& aKey, nsTArray<nsRefPtr<
     mPrefFonts.Put(aKey, array);
 }
 
-PRBool
+bool
 gfxWindowsPlatform::UseClearTypeForDownloadableFonts()
 {
     if (mUseClearTypeForDownloadableFonts == UNINITIALIZED_VALUE) {
-        mUseClearTypeForDownloadableFonts = Preferences::GetBool(GFX_DOWNLOADABLE_FONTS_USE_CLEARTYPE, PR_TRUE);
+        mUseClearTypeForDownloadableFonts = Preferences::GetBool(GFX_DOWNLOADABLE_FONTS_USE_CLEARTYPE, true);
     }
 
     return mUseClearTypeForDownloadableFonts;
 }
 
-PRBool
+bool
 gfxWindowsPlatform::UseClearTypeAlways()
 {
     if (mUseClearTypeAlways == UNINITIALIZED_VALUE) {
-        mUseClearTypeAlways = Preferences::GetBool(GFX_USE_CLEARTYPE_ALWAYS, PR_FALSE);
+        mUseClearTypeAlways = Preferences::GetBool(GFX_USE_CLEARTYPE_ALWAYS, false);
     }
 
     return mUseClearTypeAlways;
@@ -769,7 +845,7 @@ gfxWindowsPlatform::GetCleartypeParams(nsTArray<ClearTypeParameterInfo>& aParams
 
     // enumerate over subkeys
     for (i = 0, rv = ERROR_SUCCESS; rv != ERROR_NO_MORE_ITEMS; i++) {
-        size = NS_ARRAY_LENGTH(displayName);
+        size = ArrayLength(displayName);
         rv = RegEnumKeyExW(hKey, i, displayName, &size, NULL, NULL, NULL, NULL);
         if (rv != ERROR_SUCCESS) {
             continue;
@@ -781,7 +857,7 @@ gfxWindowsPlatform::GetCleartypeParams(nsTArray<ClearTypeParameterInfo>& aParams
         DWORD subrv, value;
         bool foundData = false;
 
-        swprintf_s(subkeyName, NS_ARRAY_LENGTH(subkeyName),
+        swprintf_s(subkeyName, ArrayLength(subkeyName),
                    L"Software\\Microsoft\\Avalon.Graphics\\%s", displayName);
 
         // subkey for gamma, pixel structure
@@ -843,7 +919,7 @@ gfxWindowsPlatform::GetCleartypeParams(nsTArray<ClearTypeParameterInfo>& aParams
 void
 gfxWindowsPlatform::FontsPrefsChanged(const char *aPref)
 {
-    PRBool clearTextFontCaches = PR_TRUE;
+    bool clearTextFontCaches = true;
 
     gfxPlatform::FontsPrefsChanged(aPref);
 
@@ -857,7 +933,7 @@ gfxWindowsPlatform::FontsPrefsChanged(const char *aPref)
     } else if (!strncmp(GFX_CLEARTYPE_PARAMS, aPref, strlen(GFX_CLEARTYPE_PARAMS))) {
         SetupClearTypeParams();
     } else {
-        clearTextFontCaches = PR_FALSE;
+        clearTextFontCaches = false;
     }
 
     if (clearTextFontCaches) {    
@@ -865,7 +941,6 @@ gfxWindowsPlatform::FontsPrefsChanged(const char *aPref)
         if (fc) {
             fc->Flush();
         }
-        gfxTextRunWordCache::Flush();
     }
 }
 
