@@ -402,15 +402,15 @@ void nsCARenderer::Destroy() {
     caRenderer.layer = nsnull;
     [caRenderer release];
   }
-  if (mPixelBuffer) {
-    ::CGLDestroyPBuffer((CGLPBufferObj)mPixelBuffer);
-  }
   if (mOpenGLContext) {
-    if (mFBO || mIOTexture) {
+    if (mFBO || mIOTexture || mFBOTexture) {
       // Release these resources with the context that allocated them
       CGLContextObj oldContext = ::CGLGetCurrentContext();
       ::CGLSetCurrentContext(mOpenGLContext);
 
+      if (mFBOTexture) {
+        ::glDeleteTextures(1, &mFBOTexture);
+      }
       if (mIOTexture) {
         ::glDeleteTextures(1, &mIOTexture);
       }
@@ -430,7 +430,7 @@ void nsCARenderer::Destroy() {
   // mCGData is deallocated by cgdata_release_callback
 
   mCARenderer = nil;
-  mPixelBuffer = nsnull;
+  mFBOTexture = 0;
   mOpenGLContext = nsnull;
   mCGImage = nsnull;
   mIOSurface = nsnull;
@@ -438,11 +438,14 @@ void nsCARenderer::Destroy() {
   mIOTexture = nsnull;
 }
 
-nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
+nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight,
+                                     AllowOfflineRendererEnum aAllowOfflineRenderer) {
+  mAllowOfflineRenderer = aAllowOfflineRenderer;
+
   if (aWidth == 0 || aHeight == 0)
     return NS_ERROR_FAILURE;
 
-  if (aWidth == mUnsupportedWidth && 
+  if (aWidth == mUnsupportedWidth &&
       aHeight == mUnsupportedHeight) {
     return NS_ERROR_FAILURE;
   }
@@ -452,20 +455,13 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
 
   CGLPixelFormatAttribute attributes[] = {
     kCGLPFAAccelerated,
-    kCGLPFAAllowOfflineRenderers,
     kCGLPFADepthSize, (CGLPixelFormatAttribute)24,
+    kCGLPFAAllowOfflineRenderers,
     (CGLPixelFormatAttribute)0
   };
 
-  if (!mIOSurface) {
-    CGLError result = ::CGLCreatePBuffer(aWidth, aHeight,
-                         GL_TEXTURE_2D, GL_RGBA, 0, &mPixelBuffer);
-    if (result != kCGLNoError) {
-      mUnsupportedWidth = aWidth;
-      mUnsupportedHeight = aHeight;
-      Destroy();
-      return NS_ERROR_FAILURE;
-    }
+  if (mAllowOfflineRenderer == DISALLOW_OFFLINE_RENDERER) {
+    attributes[3] = (CGLPixelFormatAttribute)0;
   }
 
   GLint screen;
@@ -495,28 +491,10 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
     return NS_ERROR_FAILURE;
   }
 
-  // Create a transaction and disable animations
-  // to make the position update instant.
-  [CATransaction begin];
-  NSMutableDictionary *newActions = [[NSMutableDictionary alloc] initWithObjectsAndKeys:[NSNull null], @"onOrderIn",
-                                   [NSNull null], @"onOrderOut",
-                                   [NSNull null], @"sublayers",
-                                   [NSNull null], @"contents",
-                                   [NSNull null], @"position",
-                                   [NSNull null], @"bounds",
-                                   nil];
-  layer.actions = newActions;
-  [newActions release];
-
-  [CATransaction setValue: [NSNumber numberWithFloat:0.0f] forKey: kCATransactionAnimationDuration];
-  [CATransaction setValue: (id) kCFBooleanTrue forKey: kCATransactionDisableActions];
-  [layer setBounds:CGRectMake(0, 0, aWidth, aHeight)];
-  [layer setPosition:CGPointMake(aWidth/2.0, aHeight/2.0)];
   caRenderer.layer = layer;
-  caRenderer.bounds = CGRectMake(0, 0, aWidth, aHeight);
-  [CATransaction commit];
+  SetBounds(aWidth, aHeight);
 
-  // We either target rendering to a CGImage or IOSurface.
+  // We target rendering to a CGImage if no shared IOSurface are given.
   if (!mIOSurface) {
     mCGData = malloc(aWidth*aHeight*4);
     if (!mCGData) {
@@ -528,7 +506,7 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
 
     CGDataProviderRef dataProvider = nsnull;
     dataProvider = ::CGDataProviderCreateWithData(mCGData,
-                                        mCGData, aHeight*aWidth*4, 
+                                        mCGData, aHeight*aWidth*4,
                                         cgdata_release_callback);
     if (!dataProvider) {
       cgdata_release_callback(mCGData, mCGData, aHeight*aWidth*4);
@@ -540,7 +518,7 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
 
     CGColorSpaceRef colorSpace = CreateSystemColorSpace();
 
-    mCGImage = ::CGImageCreate(aWidth, aHeight, 8, 32, aWidth * 4, colorSpace, 
+    mCGImage = ::CGImageCreate(aWidth, aHeight, 8, 32, aWidth * 4, colorSpace,
                 kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Host,
                 dataProvider, NULL, true, kCGRenderingIntentDefault);
 
@@ -554,10 +532,12 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
       Destroy();
       return NS_ERROR_FAILURE;
     }
-  } else {
-    CGLContextObj oldContext = ::CGLGetCurrentContext();
-    ::CGLSetCurrentContext(mOpenGLContext);
+  }
 
+  CGLContextObj oldContext = ::CGLGetCurrentContext();
+  ::CGLSetCurrentContext(mOpenGLContext);
+
+  if (mIOSurface) {
     // Create the IOSurface mapped texture.
     ::glGenTextures(1, &mIOTexture);
     ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, mIOTexture);
@@ -565,44 +545,43 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
     ::glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
     nsIOSurfaceLib::CGLTexImageIOSurface2D(mOpenGLContext, GL_TEXTURE_RECTANGLE_ARB,
                                            GL_RGBA, aWidth, aHeight,
-                                           GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, 
+                                           GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
                                            mIOSurface->mIOSurfacePtr, 0);
     ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, 0);
-
-    // Create the fbo
-    ::glGenFramebuffersEXT(1, &mFBO);
-    ::glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFBO);
-    ::glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, 
-                                GL_TEXTURE_RECTANGLE_ARB, mIOTexture, 0);
-
-    // Make sure that the Framebuffer configuration is supported on the client machine
-    GLenum fboStatus;
-    fboStatus = ::glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
-    if (fboStatus != GL_FRAMEBUFFER_COMPLETE_EXT) {
-      NS_ERROR("FBO not supported");
-      if (oldContext)
-        ::CGLSetCurrentContext(oldContext);
-      mUnsupportedWidth = aWidth;
-      mUnsupportedHeight = aHeight;
-      Destroy();
-      return NS_ERROR_FAILURE; 
-    }
-
-    if (oldContext)
-      ::CGLSetCurrentContext(oldContext);
+  } else {
+    ::glGenTextures(1, &mFBOTexture);
+    ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, mFBOTexture);
+    ::glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    ::glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, 0);
   }
 
-  CGLContextObj oldContext = ::CGLGetCurrentContext();
-  ::CGLSetCurrentContext(mOpenGLContext);
+  // Create the fbo
+  ::glGenFramebuffersEXT(1, &mFBO);
+  ::glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFBO);
+  if (mIOSurface) {
+   ::glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
+                                GL_TEXTURE_RECTANGLE_ARB, mIOTexture, 0);
+  } else {
+    ::glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
+                                GL_TEXTURE_RECTANGLE_ARB, mFBOTexture, 0);
+  }
 
-  ::glViewport(0.0, 0.0, aWidth, aHeight);
-  ::glMatrixMode(GL_PROJECTION);
-  ::glLoadIdentity();
-  ::glOrtho (0.0, aWidth, 0.0, aHeight, -1, 1);
 
-  // Render upside down to speed up CGContextDrawImage
-  ::glTranslatef(0.0f, aHeight, 0.0);
-  ::glScalef(1.0, -1.0, 1.0);
+  // Make sure that the Framebuffer configuration is supported on the client machine
+  GLenum fboStatus;
+  fboStatus = ::glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
+  if (fboStatus != GL_FRAMEBUFFER_COMPLETE_EXT) {
+    NS_ERROR("FBO not supported");
+    if (oldContext)
+      ::CGLSetCurrentContext(oldContext);
+    mUnsupportedWidth = aWidth;
+    mUnsupportedHeight = aHeight;
+    Destroy();
+    return NS_ERROR_FAILURE;
+  }
+
+  SetViewport(aWidth, aHeight);
 
   GLenum result = ::glGetError();
   if (result != GL_NO_ERROR) {
@@ -621,21 +600,83 @@ nsresult nsCARenderer::SetupRenderer(void *aCALayer, int aWidth, int aHeight) {
   return NS_OK;
 }
 
+void nsCARenderer::SetBounds(int aWidth, int aHeight) {
+  CARenderer* caRenderer = (CARenderer*)mCARenderer;
+  CALayer* layer = [mCARenderer layer];
+
+  // Create a transaction and disable animations
+  // to make the position update instant.
+  [CATransaction begin];
+  NSMutableDictionary *newActions = [[NSMutableDictionary alloc] initWithObjectsAndKeys:[NSNull null], @"onOrderIn",
+                                   [NSNull null], @"onOrderOut",
+                                   [NSNull null], @"sublayers",
+                                   [NSNull null], @"contents",
+                                   [NSNull null], @"position",
+                                   [NSNull null], @"bounds",
+                                   nil];
+  layer.actions = newActions;
+  [newActions release];
+
+  [CATransaction setValue: [NSNumber numberWithFloat:0.0f] forKey: kCATransactionAnimationDuration];
+  [CATransaction setValue: (id) kCFBooleanTrue forKey: kCATransactionDisableActions];
+  [layer setBounds:CGRectMake(0, 0, aWidth, aHeight)];
+  [layer setPosition:CGPointMake(aWidth/2.0, aHeight/2.0)];
+  caRenderer.bounds = CGRectMake(0, 0, aWidth, aHeight);
+  [CATransaction commit];
+
+}
+
+void nsCARenderer::SetViewport(int aWidth, int aHeight) {
+  ::glViewport(0.0, 0.0, aWidth, aHeight);
+  ::glMatrixMode(GL_PROJECTION);
+  ::glLoadIdentity();
+  ::glOrtho (0.0, aWidth, 0.0, aHeight, -1, 1);
+
+  // Render upside down to speed up CGContextDrawImage
+  ::glTranslatef(0.0f, aHeight, 0.0);
+  ::glScalef(1.0, -1.0, 1.0);
+}
+
 void nsCARenderer::AttachIOSurface(nsRefPtr<nsIOSurface> aSurface) {
-  if (mIOSurface && 
+  if (mIOSurface &&
       aSurface->GetIOSurfaceID() == mIOSurface->GetIOSurfaceID()) {
     // This object isn't needed since we already have a
     // handle to the same io surface.
     aSurface = nsnull;
     return;
   }
-  if (mCARenderer) {
-    // We are attaching a larger IOSurface, we need to
-    // resize our elements.
-    Destroy(); 
-  }
 
   mIOSurface = aSurface;
+
+  // Update the framebuffer and viewport
+  if (mCARenderer) {
+    CARenderer* caRenderer = (CARenderer*)mCARenderer;
+    int width = caRenderer.bounds.size.width;
+    int height = caRenderer.bounds.size.height;
+
+    CGLContextObj oldContext = ::CGLGetCurrentContext();
+    ::CGLSetCurrentContext(mOpenGLContext);
+    ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, mIOTexture);
+    nsIOSurfaceLib::CGLTexImageIOSurface2D(mOpenGLContext, GL_TEXTURE_RECTANGLE_ARB,
+                                           GL_RGBA, mIOSurface->GetWidth(), mIOSurface->GetHeight(),
+                                           GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
+                                           mIOSurface->mIOSurfacePtr, 0);
+    ::glBindTexture(GL_TEXTURE_RECTANGLE_ARB, 0);
+
+    // Rebind the FBO to make it live
+    ::glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFBO);
+
+    if (mIOSurface->GetWidth() != width || mIOSurface->GetHeight() != height) {
+      width = mIOSurface->GetWidth();
+      height = mIOSurface->GetHeight();
+      SetBounds(width, height);
+      SetViewport(width, height);
+    }
+
+    if (oldContext) {
+      ::CGLSetCurrentContext(oldContext);
+    }
+  }
 }
 
 IOSurfaceID nsCARenderer::GetIOSurfaceID() {
@@ -672,7 +713,8 @@ nsresult nsCARenderer::Render(int aWidth, int aHeight,
     //      if we are resizing down.
     CALayer* caLayer = [caRenderer layer];
     Destroy();
-    if (SetupRenderer(caLayer, aWidth, aHeight) != NS_OK) {
+    if (SetupRenderer(caLayer, aWidth, aHeight,
+                      mAllowOfflineRenderer) != NS_OK) {
       return NS_ERROR_FAILURE;
     }
 
@@ -682,7 +724,9 @@ nsresult nsCARenderer::Render(int aWidth, int aHeight,
   CGLContextObj oldContext = ::CGLGetCurrentContext();
   ::CGLSetCurrentContext(mOpenGLContext);
   if (!mIOSurface) {
-    ::CGLSetPBuffer(mOpenGLContext, mPixelBuffer, 0, 0, 0);
+    // If no shared IOSurface is given render to our own
+    // texture for readback.
+    ::glGenTextures(1, &mFBOTexture);
   }
 
   GLenum result = ::glGetError();
@@ -736,14 +780,6 @@ nsresult nsCARenderer::DrawSurfaceToCGContext(CGContextRef aContext,
   size_t bytesPerRow = surf->GetBytesPerRow();
   size_t ioWidth = surf->GetWidth();
   size_t ioHeight = surf->GetHeight();
-  void* ioData = surf->GetBaseAddress();
-  CGDataProviderRef dataProvider = ::CGDataProviderCreateWithData(ioData,
-                                      ioData, ioHeight*(bytesPerRow)*4, 
-                                      NULL); //No release callback 
-  if (!dataProvider) {
-    surf->Unlock();
-    return NS_ERROR_FAILURE;
-  }
 
   // We get rendering glitches if we use a width/height that falls
   // outside of the IOSurface.
@@ -754,6 +790,16 @@ nsresult nsCARenderer::DrawSurfaceToCGContext(CGContextRef aContext,
 
   if (aX < 0 || aX >= ioWidth ||
       aY < 0 || aY >= ioHeight) {
+    surf->Unlock();
+    return NS_ERROR_FAILURE;
+  }
+
+  void* ioData = surf->GetBaseAddress();
+  CGDataProviderRef dataProvider = ::CGDataProviderCreateWithData(ioData,
+                                      ioData, ioHeight*(bytesPerRow)*4, 
+                                      NULL); //No release callback 
+  if (!dataProvider) {
+    surf->Unlock();
     return NS_ERROR_FAILURE;
   }
 
@@ -815,10 +861,12 @@ void nsCARenderer::SaveToDisk(nsIOSurface *surf) {
     return;
   }
 
+  CGColorSpaceRef colorSpace = CreateSystemColorSpace();
   CGImageRef cgImage = ::CGImageCreate(ioWidth, ioHeight, 8, 32, bytesPerRow,
-              CreateSystemColorSpace(), kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Host,
+              colorSpace, kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Host,
               dataProvider, NULL, true, kCGRenderingIntentDefault);
   ::CGDataProviderRelease(dataProvider);
+  ::CGColorSpaceRelease(colorSpace);
   if (!cgImage) {
     surf->Unlock();
     return;
@@ -832,16 +880,19 @@ void nsCARenderer::SaveToDisk(nsIOSurface *surf) {
 
   printf("Exporting: %s\n", cstr);
   CFURLRef url = ::CFURLCreateWithString( NULL, cfStr, NULL);
+  ::CFRelease(cfStr);
 
   CFStringRef type = kUTTypePNG;
-  size_t count = 1; 
+  size_t count = 1;
   CFDictionaryRef options = NULL;
   CGImageDestinationRef dest = ::CGImageDestinationCreateWithURL(url, type, count, options);
+  ::CFRelease(url);
 
   ::CGImageDestinationAddImage(dest, cgImage, NULL);
 
   ::CGImageDestinationFinalize(dest);
   ::CFRelease(dest);
+  ::CGImageRelease(cgImage);
 
   surf->Unlock();
 
