@@ -437,20 +437,26 @@ void
 nsBMPEncoder::ConvertHostARGBRow(const PRUint8* aSrc, PRUint8* aDest,
                                  PRUint32 aPixelWidth)
 {
-  for (PRUint32 x = 0; x < aPixelWidth; x ++) {
-    const PRUint32& pixelIn = ((const PRUint32*)(aSrc))[x];
-    PRUint8 *pixelOut = &aDest[x * BytesPerPixel(mBMPInfoHeader.bpp)];
+  int bytes = BytesPerPixel(mBMPInfoHeader.bpp);
 
-    PRUint8 alpha = (pixelIn & 0xff000000) >> 24;
-    if (alpha == 0) {
-      pixelOut[0] = pixelOut[1] = pixelOut[2] = 0;
-    } else {
-      pixelOut[0] = (((pixelIn & 0xff0000) >> 16) * 255 + alpha / 2) / alpha;
-      pixelOut[1] = (((pixelIn & 0x00ff00) >>  8) * 255 + alpha / 2) / alpha;
-      pixelOut[2] = (((pixelIn & 0x0000ff) >>  0) * 255 + alpha / 2) / alpha;
-      if(mBMPInfoHeader.bpp == 32) {
-        pixelOut[3] = alpha;
-      }
+  if (mBMPInfoHeader.bpp == 32) {
+    for (PRUint32 x = 0; x < aPixelWidth; x++) {
+      const PRUint32& pixelIn = ((const PRUint32*)(aSrc))[x];
+      PRUint8 *pixelOut = &aDest[x * bytes];
+
+      pixelOut[0] = (pixelIn & 0x00ff0000) >> 16;
+      pixelOut[1] = (pixelIn & 0x0000ff00) >>  8;
+      pixelOut[2] = (pixelIn & 0x000000ff) >>  0;
+      pixelOut[3] = (pixelIn & 0xff000000) >> 24;
+    }
+  } else {
+    for (PRUint32 x = 0; x < aPixelWidth; x++) {
+      const PRUint32& pixelIn = ((const PRUint32*)(aSrc))[x];
+      PRUint8 *pixelOut = &aDest[x * bytes];
+
+      pixelOut[0] = (pixelIn & 0xff0000) >> 16;
+      pixelOut[1] = (pixelIn & 0x00ff00) >>  8;
+      pixelOut[2] = (pixelIn & 0x0000ff) >>  0;
     }
   }
 }
@@ -548,7 +554,7 @@ nsBMPEncoder::InitInfoHeader(PRUint32 aBPP, PRUint32 aWidth, PRUint32 aHeight)
 void 
 nsBMPEncoder::EncodeFileHeader() 
 {  
-  mozilla::imagelib::BMPFILEHEADER littleEndianBFH = mBMPFileHeader;
+  mozilla::image::BMPFILEHEADER littleEndianBFH = mBMPFileHeader;
   littleEndianBFH.filesize = NATIVE32_TO_LITTLE(littleEndianBFH.filesize);
   littleEndianBFH.reserved = NATIVE32_TO_LITTLE(littleEndianBFH.reserved);
   littleEndianBFH.dataoffset= NATIVE32_TO_LITTLE(littleEndianBFH.dataoffset);
@@ -575,7 +581,7 @@ nsBMPEncoder::EncodeFileHeader()
 void 
 nsBMPEncoder::EncodeInfoHeader()
 {
-  mozilla::imagelib::BMPINFOHEADER littleEndianmBIH = mBMPInfoHeader;
+  mozilla::image::BMPINFOHEADER littleEndianmBIH = mBMPInfoHeader;
   littleEndianmBIH.width =  NATIVE32_TO_LITTLE(littleEndianmBIH.width);
   littleEndianmBIH.height = NATIVE32_TO_LITTLE(littleEndianmBIH.height); 
   littleEndianmBIH.planes = NATIVE16_TO_LITTLE(littleEndianmBIH.planes);
