@@ -48,8 +48,10 @@
 
 namespace js {
 
+class TempAllocPolicy;
+
 /* Integral types for all hash functions. */
-typedef uint32 HashNumber;
+typedef uint32_t HashNumber;
 
 /*****************************************************************************/
 
@@ -157,9 +159,9 @@ class HashTable : private AllocPolicy
         friend class HashTable;
         HashNumber keyHash;
 #ifdef DEBUG
-        uint64 mutationCount;
+        uint64_t mutationCount;
 
-        AddPtr(Entry &entry, HashNumber hn, uint64 mutationCount)
+        AddPtr(Entry &entry, HashNumber hn, uint64_t mutationCount)
             : Ptr(entry), keyHash(hn), mutationCount(mutationCount) {}
 #else
         AddPtr(Entry &entry, HashNumber hn) : Ptr(entry), keyHash(hn) {}
@@ -181,7 +183,7 @@ class HashTable : private AllocPolicy
         friend class HashTable;
 
         Range(Entry *c, Entry *e) : cur(c), end(e) {
-            while (cur != end && !cur->isLive())
+            while (cur < end && !cur->isLive())
                 ++cur;
         }
 
@@ -201,7 +203,8 @@ class HashTable : private AllocPolicy
 
         void popFront() {
             JS_ASSERT(!empty());
-            while (++cur != end && !cur->isLive());
+            while (++cur < end && !cur->isLive())
+                continue;
         }
     };
 
@@ -259,30 +262,28 @@ class HashTable : private AllocPolicy
     };
 
   private:
-    uint32      hashShift;      /* multiplicative hash shift */
-    uint32      tableCapacity;  /* = JS_BIT(sHashBits - hashShift) */
-    uint32      entryCount;     /* number of entries in table */
-    uint32      gen;            /* entry storage generation number */
-    uint32      removedCount;   /* removed entry sentinels in table */
+    uint32_t    hashShift;      /* multiplicative hash shift */
+    uint32_t    entryCount;     /* number of entries in table */
+    uint32_t    gen;            /* entry storage generation number */
+    uint32_t    removedCount;   /* removed entry sentinels in table */
     Entry       *table;         /* entry storage */
 
     void setTableSizeLog2(unsigned sizeLog2) {
         hashShift = sHashBits - sizeLog2;
-        tableCapacity = JS_BIT(sizeLog2);
     }
 
 #ifdef DEBUG
     mutable struct Stats {
-        uint32          searches;       /* total number of table searches */
-        uint32          steps;          /* hash chain links traversed */
-        uint32          hits;           /* searches that found key */
-        uint32          misses;         /* searches that didn't find key */
-        uint32          addOverRemoved; /* adds that recycled a removed entry */
-        uint32          removes;        /* calls to remove */
-        uint32          removeFrees;    /* calls to remove that freed the entry */
-        uint32          grows;          /* table expansions */
-        uint32          shrinks;        /* table contractions */
-        uint32          compresses;     /* table compressions */
+        uint32_t        searches;       /* total number of table searches */
+        uint32_t        steps;          /* hash chain links traversed */
+        uint32_t        hits;           /* searches that found key */
+        uint32_t        misses;         /* searches that didn't find key */
+        uint32_t        addOverRemoved; /* adds that recycled a removed entry */
+        uint32_t        removes;        /* calls to remove */
+        uint32_t        removeFrees;    /* calls to remove that freed the entry */
+        uint32_t        grows;          /* table expansions */
+        uint32_t        shrinks;        /* table contractions */
+        uint32_t        compresses;     /* table compressions */
     } stats;
 #   define METER(x) x
 #else
@@ -292,7 +293,7 @@ class HashTable : private AllocPolicy
 #ifdef DEBUG
     friend class js::ReentrancyGuard;
     mutable bool entered;
-    uint64       mutationCount;
+    uint64_t     mutationCount;
 #endif
 
     /* The default initial capacity is 16, but you can ask for as small as 4. */
@@ -305,9 +306,9 @@ class HashTable : private AllocPolicy
     static const unsigned sMaxInit      = JS_BIT(23);
     static const unsigned sMaxCapacity  = JS_BIT(24);
     static const unsigned sHashBits     = tl::BitSize<HashNumber>::result;
-    static const uint8    sMinAlphaFrac = 64;  /* (0x100 * .25) taken from jsdhash.h */
-    static const uint8    sMaxAlphaFrac = 192; /* (0x100 * .75) taken from jsdhash.h */
-    static const uint8    sInvMaxAlpha  = 171; /* (ceil(0x100 / .75) >> 1) */
+    static const uint8_t  sMinAlphaFrac = 64;  /* (0x100 * .25) taken from jsdhash.h */
+    static const uint8_t  sMaxAlphaFrac = 192; /* (0x100 * .75) taken from jsdhash.h */
+    static const uint8_t  sInvMaxAlpha  = 171; /* (ceil(0x100 / .75) >> 1) */
     static const HashNumber sGoldenRatio  = 0x9E3779B9U;       /* taken from jsdhash.h */
     static const HashNumber sFreeKey = Entry::sFreeKey;
     static const HashNumber sRemovedKey = Entry::sRemovedKey;
@@ -339,19 +340,19 @@ class HashTable : private AllocPolicy
         return keyHash & ~sCollisionBit;
     }
 
-    static Entry *createTable(AllocPolicy &alloc, uint32 capacity)
+    static Entry *createTable(AllocPolicy &alloc, uint32_t capacity)
     {
         Entry *newTable = (Entry *)alloc.malloc_(capacity * sizeof(Entry));
         if (!newTable)
             return NULL;
-        for (Entry *e = newTable, *end = e + capacity; e != end; ++e)
+        for (Entry *e = newTable, *end = e + capacity; e < end; ++e)
             new(e) Entry();
         return newTable;
     }
 
-    static void destroyTable(AllocPolicy &alloc, Entry *oldTable, uint32 capacity)
+    static void destroyTable(AllocPolicy &alloc, Entry *oldTable, uint32_t capacity)
     {
-        for (Entry *e = oldTable, *end = e + capacity; e != end; ++e)
+        for (Entry *e = oldTable, *end = e + capacity; e < end; ++e)
             e->~Entry();
         alloc.free_(oldTable);
     }
@@ -359,6 +360,7 @@ class HashTable : private AllocPolicy
   public:
     HashTable(AllocPolicy ap)
       : AllocPolicy(ap),
+        hashShift(sHashBits),
         entryCount(0),
         gen(0),
         removedCount(0),
@@ -369,7 +371,7 @@ class HashTable : private AllocPolicy
 #endif
     {}
 
-    bool init(uint32 length)
+    bool init(uint32_t length)
     {
         /* Make sure that init isn't called twice. */
         JS_ASSERT(table == NULL);
@@ -382,13 +384,13 @@ class HashTable : private AllocPolicy
             this->reportAllocOverflow();
             return false;
         }
-        uint32 capacity = (length * sInvMaxAlpha) >> 7;
+        uint32_t capacity = (length * sInvMaxAlpha) >> 7;
 
         if (capacity < sMinSize)
             capacity = sMinSize;
 
         /* FIXME: use JS_CEILING_LOG2 when PGO stops crashing (bug 543034). */
-        uint32 roundUp = sMinSize, roundUpLog2 = sMinSizeLog2;
+        uint32_t roundUp = sMinSize, roundUpLog2 = sMinSizeLog2;
         while (roundUp < capacity) {
             roundUp <<= 1;
             ++roundUpLog2;
@@ -414,23 +416,24 @@ class HashTable : private AllocPolicy
     ~HashTable()
     {
         if (table)
-            destroyTable(*this, table, tableCapacity);
+            destroyTable(*this, table, capacity());
     }
 
   private:
-    static HashNumber hash1(HashNumber hash0, uint32 shift) {
+    static HashNumber hash1(HashNumber hash0, uint32_t shift) {
         return hash0 >> shift;
     }
 
-    static HashNumber hash2(HashNumber hash0, uint32 log2, uint32 shift) {
+    static HashNumber hash2(HashNumber hash0, uint32_t log2, uint32_t shift) {
         return ((hash0 << log2) >> shift) | 1;
     }
 
     bool overloaded() {
-        return entryCount + removedCount >= ((sMaxAlphaFrac * tableCapacity) >> 8);
+        return entryCount + removedCount >= ((sMaxAlphaFrac * capacity()) >> 8);
     }
 
     bool underloaded() {
+        uint32_t tableCapacity = capacity();
         return tableCapacity > sMinSize &&
                entryCount <= ((sMinAlphaFrac * tableCapacity) >> 8);
     }
@@ -546,9 +549,9 @@ class HashTable : private AllocPolicy
     {
         /* Look, but don't touch, until we succeed in getting new entry store. */
         Entry *oldTable = table;
-        uint32 oldCap = tableCapacity;
-        uint32 newLog2 = sHashBits - hashShift + deltaLog2;
-        uint32 newCapacity = JS_BIT(newLog2);
+        uint32_t oldCap = capacity();
+        uint32_t newLog2 = sHashBits - hashShift + deltaLog2;
+        uint32_t newCapacity = JS_BIT(newLog2);
         if (newCapacity > sMaxCapacity) {
             this->reportAllocOverflow();
             return false;
@@ -565,7 +568,7 @@ class HashTable : private AllocPolicy
         table = newTable;
 
         /* Copy only live entries, leaving removed ones behind. */
-        for (Entry *src = oldTable, *end = src + oldCap; src != end; ++src) {
+        for (Entry *src = oldTable, *end = src + oldCap; src < end; ++src) {
             if (src->isLive()) {
                 src->unsetCollision();
                 findFreeEntry(src->getKeyHash()) = Move(*src);
@@ -604,9 +607,10 @@ class HashTable : private AllocPolicy
     void clear()
     {
         if (tl::IsPodType<Entry>::result) {
-            memset(table, 0, sizeof(*table) * tableCapacity);
+            memset(table, 0, sizeof(*table) * capacity());
         } else {
-            for (Entry *e = table, *end = table + tableCapacity; e != end; ++e)
+            uint32_t tableCapacity = capacity();
+            for (Entry *e = table, *end = table + tableCapacity; e < end; ++e)
                 *e = Move(Entry());
         }
         removedCount = 0;
@@ -623,7 +627,7 @@ class HashTable : private AllocPolicy
         if (!table)
             return;
         
-        destroyTable(*this, table, tableCapacity);
+        destroyTable(*this, table, capacity());
         table = NULL;
         gen++;
         entryCount = 0;
@@ -634,33 +638,31 @@ class HashTable : private AllocPolicy
     }
 
     Range all() const {
-        return Range(table, table + tableCapacity);
+        return Range(table, table + capacity());
     }
 
     bool empty() const {
         return !entryCount;
     }
 
-    uint32 count() const {
+    uint32_t count() const {
         return entryCount;
     }
 
-    uint32 capacity() const {
-        return tableCapacity;
+    uint32_t capacity() const {
+        return JS_BIT(sHashBits - hashShift);
     }
 
-    uint32 generation() const {
+    uint32_t generation() const {
         return gen;
     }
 
-    /*
-     * This counts the HashTable's |table| array.  If |countMe| is true it also
-     * counts the HashTable object itself.
-     */
-    size_t sizeOf(JSUsableSizeFun usf, bool countMe) const {
-        size_t usable = usf(table) + (countMe ? usf((void*)this) : 0);
-        return usable ? usable
-                      : (tableCapacity * sizeof(Entry)) + (countMe ? sizeof(HashTable) : 0);
+    size_t sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        return mallocSizeOf(table);
+    }
+
+    size_t sizeOfIncludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
     }
 
     Ptr lookup(const Lookup &l) const {
@@ -701,7 +703,7 @@ class HashTable : private AllocPolicy
             if (overloaded()) {
                 /* Compress if a quarter or more of all entries are removed. */
                 int deltaLog2;
-                if (removedCount >= (tableCapacity >> 2)) {
+                if (removedCount >= (capacity() >> 2)) {
                     METER(stats.compresses++);
                     deltaLog2 = 0;
                 } else {
@@ -772,44 +774,6 @@ class HashTable : private AllocPolicy
 
 /*****************************************************************************/
 
-template <typename T>
-class TaggedPointerEntry
-{
-    uintptr_t bits;
-
-    typedef TaggedPointerEntry<T> ThisT;
-
-    static const uintptr_t NO_TAG_MASK = uintptr_t(-1) - 1;
-
-  public:
-    TaggedPointerEntry() : bits(0) {}
-    TaggedPointerEntry(const TaggedPointerEntry &other) : bits(other.bits) {}
-    TaggedPointerEntry(T *ptr, bool tagged)
-      : bits(uintptr_t(ptr) | uintptr_t(tagged))
-    {
-        JS_ASSERT((uintptr_t(ptr) & 0x1) == 0);
-    }
-
-    bool isTagged() const {
-        return bits & 0x1;
-    }
-
-    /*
-     * Non-branching code sequence. Note that the const_cast is safe because
-     * the hash function doesn't consider the tag to be a portion of the key.
-     */
-    void setTagged(bool enabled) const {
-        const_cast<ThisT *>(this)->bits |= uintptr_t(enabled);
-    }
-
-    T *asPtr() const {
-        JS_ASSERT(bits != 0);
-        return reinterpret_cast<T *>(bits & NO_TAG_MASK);
-    }
-};
-
-/*****************************************************************************/
-
 /*
  * Hash policy
  *
@@ -912,7 +876,10 @@ class HashMapEntry
 
   public:
     HashMapEntry() : key(), value() {}
-    HashMapEntry(const Key &k, const Value &v) : key(k), value(v) {}
+
+    template<typename KeyInput, typename ValueInput>
+    HashMapEntry(const KeyInput &k, const ValueInput &v) : key(k), value(v) {}
+
     HashMapEntry(MoveRef<HashMapEntry> rhs) 
       : key(Move(rhs->key)), value(Move(rhs->value)) { }
     void operator=(MoveRef<HashMapEntry> rhs) {
@@ -955,7 +922,10 @@ struct IsPodType<HashMapEntry<K, V> >
  *      called by HashMap must not call back into the same HashMap object.
  * N.B: Due to the lack of exception handling, the user must call |init()|.
  */
-template <class Key, class Value, class HashPolicy, class AllocPolicy>
+template <class Key,
+          class Value,
+          class HashPolicy = DefaultHasher<Key>,
+          class AllocPolicy = TempAllocPolicy>
 class HashMap
 {
   public:
@@ -986,7 +956,7 @@ class HashMap
      * init after constructing a HashMap and check the return value.
      */
     HashMap(AllocPolicy a = AllocPolicy()) : impl(a) {}
-    bool init(uint32 len = Impl::sDefaultInitSize)    { return impl.init(len); }
+    bool init(uint32_t len = Impl::sDefaultInitSize)  { return impl.init(len); }
     bool initialized() const                          { return impl.initialized(); }
 
     /*
@@ -1048,7 +1018,8 @@ class HashMap
         return impl.lookupForAdd(l);
     }
 
-    bool add(AddPtr &p, const Key &k, const Value &v) {
+    template<typename KeyInput, typename ValueInput>
+    bool add(AddPtr &p, const KeyInput &k, const ValueInput &v) {
         Entry *pentry;
         if (!impl.add(p, &pentry))
             return false;
@@ -1074,7 +1045,8 @@ class HashMap
         return true;
     }
 
-    bool relookupOrAdd(AddPtr &p, const Key &k, const Value &v) {
+    template<typename KeyInput, typename ValueInput>
+    bool relookupOrAdd(AddPtr &p, const KeyInput &k, const ValueInput &v) {
         return impl.relookupOrAdd(p, k, Entry(k, v));
     }
 
@@ -1092,7 +1064,16 @@ class HashMap
     Range all() const                                 { return impl.all(); }
     size_t count() const                              { return impl.count(); }
     size_t capacity() const                           { return impl.capacity(); }
-    size_t sizeOf(JSUsableSizeFun usf, bool cm) const { return impl.sizeOf(usf, cm); }
+    size_t sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        return impl.sizeOfExcludingThis(mallocSizeOf);
+    }
+    size_t sizeOfIncludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        /* 
+         * Don't just call |impl.sizeOfExcludingThis()| because there's no
+         * guarantee that |impl| is the first field in HashMap.
+         */
+        return mallocSizeOf(this) + impl.sizeOfExcludingThis(mallocSizeOf);
+    }
 
     /*
      * Typedef for the enumeration class. An Enum may be used to examine and
@@ -1137,7 +1118,8 @@ class HashMap
     }
 
     /* Overwrite existing value with v. Return NULL on oom. */
-    Entry *put(const Key &k, const Value &v) {
+    template<typename KeyInput, typename ValueInput>
+    Entry *put(const KeyInput &k, const ValueInput &v) {
         AddPtr p = lookupForAdd(k);
         if (p) {
             p->value = v;
@@ -1185,7 +1167,7 @@ class HashMap
  *      HashSet must not call back into the same HashSet object.
  * N.B: Due to the lack of exception handling, the user must call |init()|.
  */
-template <class T, class HashPolicy, class AllocPolicy>
+template <class T, class HashPolicy = DefaultHasher<T>, class AllocPolicy = TempAllocPolicy>
 class HashSet
 {
     typedef typename HashPolicy::Lookup Lookup;
@@ -1211,7 +1193,7 @@ class HashSet
      * init after constructing a HashSet and check the return value.
      */
     HashSet(AllocPolicy a = AllocPolicy()) : impl(a) {}
-    bool init(uint32 len = Impl::sDefaultInitSize)    { return impl.init(len); }
+    bool init(uint32_t len = Impl::sDefaultInitSize)  { return impl.init(len); }
     bool initialized() const                          { return impl.initialized(); }
 
     /*
@@ -1292,7 +1274,16 @@ class HashSet
     Range all() const                                 { return impl.all(); }
     size_t count() const                              { return impl.count(); }
     size_t capacity() const                           { return impl.capacity(); }
-    size_t sizeOf(JSUsableSizeFun usf, bool cm) const { return impl.sizeOf(usf, cm); }
+    size_t sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        return impl.sizeOfExcludingThis(mallocSizeOf);
+    }
+    size_t sizeOfIncludingThis(JSMallocSizeOfFun mallocSizeOf) const {
+        /* 
+         * Don't just call |impl.sizeOfExcludingThis()| because there's no
+         * guarantee that |impl| is the first field in HashSet.
+         */
+        return mallocSizeOf(this) + impl.sizeOfExcludingThis(mallocSizeOf);
+    }
 
     /*
      * Typedef for the enumeration class. An Enum may be used to examine and
@@ -1349,6 +1340,12 @@ class HashSet
         return add(p, t);
     }
 
+    bool putNew(const Lookup &l, const T &t) {
+        AddPtr p = lookupForAdd(l);
+        JS_ASSERT(!p);
+        return add(p, t);
+    }
+
     void remove(const Lookup &l) {
         if (Ptr p = lookup(l))
             remove(p);
