@@ -48,56 +48,162 @@
 using namespace js;
 using namespace js::types;
 
-/*
- * Return:
- * - The original if no escaping need be performed.
- * - A new string if escaping need be performed.
- * - NULL on error.
- */
-static JSString *
-EscapeNakedForwardSlashes(JSContext *cx, JSString *unescaped)
+class RegExpMatchBuilder
+{
+    JSContext   * const cx;
+    JSObject    * const array;
+
+    bool setProperty(JSAtom *name, Value v) {
+        return !!js_DefineProperty(cx, array, ATOM_TO_JSID(name), &v,
+                                   JS_PropertyStub, JS_StrictPropertyStub, JSPROP_ENUMERATE);
+    }
+
+  public:
+    RegExpMatchBuilder(JSContext *cx, JSObject *array) : cx(cx), array(array) {}
+
+    bool append(uint32_t index, Value v) {
+        JS_ASSERT(!array->getOps()->getElement);
+        return !!js_DefineElement(cx, array, index, &v, JS_PropertyStub, JS_StrictPropertyStub,
+                                  JSPROP_ENUMERATE);
+    }
+
+    bool setIndex(int index) {
+        return setProperty(cx->runtime->atomState.indexAtom, Int32Value(index));
+    }
+
+    bool setInput(JSString *str) {
+        JS_ASSERT(str);
+        return setProperty(cx->runtime->atomState.inputAtom, StringValue(str));
+    }
+};
+
+static bool
+CreateRegExpMatchResult(JSContext *cx, JSString *input, const jschar *chars, size_t length,
+                        MatchPairs *matchPairs, Value *rval)
+{
+    /*
+     * Create the (slow) result array for a match.
+     *
+     * Array contents:
+     *  0:              matched string
+     *  1..pairCount-1: paren matches
+     *  input:          input string
+     *  index:          start index for the match
+     */
+    JSObject *array = NewSlowEmptyArray(cx);
+    if (!array)
+        return false;
+
+    if (!input) {
+        input = js_NewStringCopyN(cx, chars, length);
+        if (!input)
+            return false;
+    }
+
+    RegExpMatchBuilder builder(cx, array);
+
+    for (size_t i = 0; i < matchPairs->pairCount(); ++i) {
+        MatchPair pair = matchPairs->pair(i);
+
+        JSString *captured;
+        if (pair.isUndefined()) {
+            JS_ASSERT(i != 0); /* Since we had a match, first pair must be present. */
+            if (!builder.append(i, UndefinedValue()))
+                return false;
+        } else {
+            captured = js_NewDependentString(cx, input, pair.start, pair.length());
+            if (!captured || !builder.append(i, StringValue(captured)))
+                return false;
+        }
+    }
+
+    if (!builder.setIndex(matchPairs->pair(0).start) || !builder.setInput(input))
+        return false;
+
+    *rval = ObjectValue(*array);
+    return true;
+}
+
+template <class T>
+bool
+ExecuteRegExpImpl(JSContext *cx, RegExpStatics *res, T *re, JSLinearString *input,
+                  const jschar *chars, size_t length,
+                  size_t *lastIndex, RegExpExecType type, Value *rval)
+{
+    LifoAllocScope allocScope(&cx->tempLifoAlloc());
+    MatchPairs *matchPairs = NULL;
+    RegExpRunStatus status = re->execute(cx, chars, length, lastIndex, allocScope, &matchPairs);
+
+    switch (status) {
+      case RegExpRunStatus_Error:
+        return false;
+      case RegExpRunStatus_Success_NotFound:
+        *rval = NullValue();
+        return true;
+      default:
+        JS_ASSERT(status == RegExpRunStatus_Success);
+        JS_ASSERT(matchPairs);
+    }
+
+    if (res)
+        res->updateFromMatchPairs(cx, input, matchPairs);
+
+    *lastIndex = matchPairs->pair(0).limit;
+
+    if (type == RegExpTest) {
+        *rval = BooleanValue(true);
+        return true;
+    }
+
+    return CreateRegExpMatchResult(cx, input, chars, length, matchPairs, rval);
+}
+
+bool
+js::ExecuteRegExp(JSContext *cx, RegExpStatics *res, RegExpMatcher &matcher, JSLinearString *input,
+                  const jschar *chars, size_t length,
+                  size_t *lastIndex, RegExpExecType type, Value *rval)
+{
+    return ExecuteRegExpImpl(cx, res, &matcher, input, chars, length, lastIndex, type, rval);
+}
+
+bool
+js::ExecuteRegExp(JSContext *cx, RegExpStatics *res, RegExpObject *reobj, JSLinearString *input,
+                  const jschar *chars, size_t length,
+                  size_t *lastIndex, RegExpExecType type, Value *rval)
+{
+    return ExecuteRegExpImpl(cx, res, reobj, input, chars, length, lastIndex, type, rval);
+}
+
+/* Note: returns the original if no escaping need be performed. */
+static JSLinearString *
+EscapeNakedForwardSlashes(JSContext *cx, JSLinearString *unescaped)
 {
     size_t oldLen = unescaped->length();
-    const jschar *oldChars = unescaped->getChars(cx);
-    if (!oldChars)
-        return NULL;
+    const jschar *oldChars = unescaped->chars();
+
     JS::Anchor<JSString *> anchor(unescaped);
 
-    js::Vector<jschar, 128> newChars(cx);
+    /* We may never need to use |sb|. Start using it lazily. */
+    StringBuffer sb(cx);
+
     for (const jschar *it = oldChars; it < oldChars + oldLen; ++it) {
         if (*it == '/' && (it == oldChars || it[-1] != '\\')) {
-            if (!newChars.length()) {
-                if (!newChars.reserve(oldLen + 1))
+            /* There's a forward slash that needs escaping. */
+            if (sb.empty()) {
+                /* This is the first one we've seen, copy everything up to this point. */
+                if (!sb.reserve(oldLen + 1))
                     return NULL;
-                newChars.infallibleAppend(oldChars, size_t(it - oldChars));
+                sb.infallibleAppend(oldChars, size_t(it - oldChars));
             }
-            if (!newChars.append('\\'))
+            if (!sb.append('\\'))
                 return NULL;
         }
 
-        if (!newChars.empty() && !newChars.append(*it))
+        if (!sb.empty() && !sb.append(*it))
             return NULL;
     }
 
-    if (newChars.empty())
-        return unescaped;
-
-    size_t len = newChars.length();
-    if (!newChars.append('\0'))
-        return NULL;
-    jschar *chars = newChars.extractRawBuffer();
-    JSString *escaped = js_NewString(cx, chars, len);
-    if (!escaped)
-        cx->free_(chars);
-    return escaped;
-}
-
-static bool
-ResetRegExpObjectWithStatics(JSContext *cx, RegExpObject *reobj,
-                             JSString *str, RegExpFlag flags = RegExpFlag(0))
-{
-    flags = RegExpFlag(flags | cx->regExpStatics()->getFlags());
-    return ResetRegExpObject(cx, reobj, str, flags);
+    return sb.empty() ? unescaped : sb.finishString();
 }
 
 /*
@@ -112,12 +218,15 @@ ResetRegExpObjectWithStatics(JSContext *cx, RegExpObject *reobj,
  *       flags := ToString(flags) if defined(flags) else ''
  */
 static bool
-CompileRegExpObject(JSContext *cx, RegExpObject *obj, uintN argc, Value *argv, Value *rval)
+CompileRegExpObject(JSContext *cx, RegExpObjectBuilder &builder,
+                    uintN argc, Value *argv, Value *rval)
 {
     if (argc == 0) {
-        if (!ResetRegExpObjectWithStatics(cx, obj, cx->runtime->emptyString))
+        RegExpStatics *res = cx->regExpStatics();
+        RegExpObject *reobj = builder.build(cx->runtime->emptyString, res->getFlags());
+        if (!reobj)
             return false;
-        *rval = ObjectValue(*obj);
+        *rval = ObjectValue(*reobj);
         return true;
     }
 
@@ -125,7 +234,7 @@ CompileRegExpObject(JSContext *cx, RegExpObject *obj, uintN argc, Value *argv, V
     if (ValueIsRegExp(sourceValue)) {
         /*
          * If we get passed in a |RegExpObject| source we return a new
-         * object with the same |RegExpPrivate|.
+         * object with the same source/flags.
          *
          * Note: the regexp static flags are not taken into consideration here.
          */
@@ -135,30 +244,29 @@ CompileRegExpObject(JSContext *cx, RegExpObject *obj, uintN argc, Value *argv, V
             return false;
         }
 
-        RegExpPrivate *rep = sourceObj.asRegExp()->getPrivate();
-        if (!rep)
+        RegExpObject *reobj = builder.build(&sourceObj.asRegExp());
+        if (!reobj)
             return false;
-
-        rep->incref(cx);
-        if (!ResetRegExpObject(cx, obj, AlreadyIncRefed<RegExpPrivate>(rep)))
-            return false;
-        *rval = ObjectValue(*obj);
+        *rval = ObjectValue(*reobj);
         return true;
     }
 
-    JSString *sourceStr;
+    JSLinearString *sourceStr;
     if (sourceValue.isUndefined()) {
         sourceStr = cx->runtime->emptyString;
     } else {
         /* Coerce to string and compile. */
-        sourceStr = js_ValueToString(cx, sourceValue);
+        JSString *str = ToString(cx, sourceValue);
+        if (!str)
+            return false;
+        sourceStr = str->ensureLinear(cx);
         if (!sourceStr)
             return false;
     }
 
     RegExpFlag flags = RegExpFlag(0);
     if (argc > 1 && !argv[1].isUndefined()) {
-        JSString *flagStr = js_ValueToString(cx, argv[1]);
+        JSString *flagStr = ToString(cx, argv[1]);
         if (!flagStr)
             return false;
         argv[1].setString(flagStr);
@@ -166,13 +274,19 @@ CompileRegExpObject(JSContext *cx, RegExpObject *obj, uintN argc, Value *argv, V
             return false;
     }
 
-    JSString *escapedSourceStr = EscapeNakedForwardSlashes(cx, sourceStr);
+    JSLinearString *escapedSourceStr = EscapeNakedForwardSlashes(cx, sourceStr);
     if (!escapedSourceStr)
         return false;
 
-    if (!ResetRegExpObjectWithStatics(cx, obj, escapedSourceStr, flags))
+    if (!CheckRegExpSyntax(cx, escapedSourceStr))
         return false;
-    *rval = ObjectValue(*obj);
+
+    RegExpStatics *res = cx->regExpStatics();
+    RegExpObject *reobj = builder.build(escapedSourceStr, RegExpFlag(flags | res->getFlags()));
+    if (!reobj)
+        return NULL;
+
+    *rval = ObjectValue(*reobj);
     return true;
 }
 
@@ -186,10 +300,8 @@ regexp_compile(JSContext *cx, uintN argc, Value *vp)
     if (!obj)
         return ok;
 
-    RegExpObject *reobj = obj->asRegExp();
-    ok = CompileRegExpObject(cx, reobj, args.length(), args.array(), &args.rval());
-    JS_ASSERT_IF(ok, reobj->getPrivate());
-    return ok;
+    RegExpObjectBuilder builder(cx, &obj->asRegExp());
+    return CompileRegExpObject(cx, builder, args.length(), args.array(), &args.rval());
 }
 
 static JSBool
@@ -209,20 +321,11 @@ regexp_construct(JSContext *cx, uintN argc, Value *vp)
         }
     }
 
-    JSObject *obj = NewBuiltinClassInstance(cx, &RegExpClass);
-    if (!obj)
+    RegExpObjectBuilder builder(cx);
+    if (!CompileRegExpObject(cx, builder, argc, argv, &JS_RVAL(cx, vp)))
         return false;
 
-    PreInitRegExpObject pireo(obj);
-    RegExpObject *reobj = pireo.get();
-
-    if (!CompileRegExpObject(cx, reobj, argc, argv, &JS_RVAL(cx, vp))) {
-        pireo.fail();
-        return false;
-    }
-
-    pireo.succeed();
-    *vp = ObjectValue(*reobj);
+    *vp = ObjectValue(*builder.reobj());
     return true;
 }
 
@@ -236,7 +339,7 @@ regexp_toString(JSContext *cx, uintN argc, Value *vp)
     if (!obj)
         return ok;
 
-    JSString *str = obj->asRegExp()->toString(cx);
+    JSString *str = obj->asRegExp().toString(cx);
     if (!str)
         return false;
 
@@ -312,11 +415,11 @@ DEFINE_STATIC_SETTER(static_multiline_setter,
                          return false;
                      res->setMultiline(cx, !!JSVAL_TO_BOOLEAN(*vp)))
 
-const uint8 REGEXP_STATIC_PROP_ATTRS    = JSPROP_PERMANENT | JSPROP_SHARED | JSPROP_ENUMERATE;
-const uint8 RO_REGEXP_STATIC_PROP_ATTRS = REGEXP_STATIC_PROP_ATTRS | JSPROP_READONLY;
+const uint8_t REGEXP_STATIC_PROP_ATTRS    = JSPROP_PERMANENT | JSPROP_SHARED | JSPROP_ENUMERATE;
+const uint8_t RO_REGEXP_STATIC_PROP_ATTRS = REGEXP_STATIC_PROP_ATTRS | JSPROP_READONLY;
 
-const uint8 HIDDEN_PROP_ATTRS = JSPROP_PERMANENT | JSPROP_SHARED;
-const uint8 RO_HIDDEN_PROP_ATTRS = HIDDEN_PROP_ATTRS | JSPROP_READONLY;
+const uint8_t HIDDEN_PROP_ATTRS = JSPROP_PERMANENT | JSPROP_SHARED;
+const uint8_t RO_HIDDEN_PROP_ATTRS = HIDDEN_PROP_ATTRS | JSPROP_READONLY;
 
 static JSPropertySpec regexp_static_props[] = {
     {"input",        0, REGEXP_STATIC_PROP_ATTRS,    static_input_getter, static_input_setter},
@@ -350,36 +453,17 @@ js_InitRegExpClass(JSContext *cx, JSObject *obj)
 {
     JS_ASSERT(obj->isNative());
 
-    GlobalObject *global = obj->asGlobal();
+    GlobalObject *global = &obj->asGlobal();
 
     JSObject *proto = global->createBlankPrototype(cx, &RegExpClass);
     if (!proto)
         return NULL;
+    proto->setPrivate(NULL);
 
-    {
-        AlreadyIncRefed<RegExpPrivate> rep =
-          RegExpPrivate::create(cx, cx->runtime->emptyString, RegExpFlag(0), NULL);
-        if (!rep)
-            return NULL;
-
-        /*
-         * Associate the empty regular expression with |RegExp.prototype|, and define
-         * the initial non-method properties of any regular expression instance.
-         * These must be added before methods to preserve slot layout.
-         */
-#ifdef DEBUG
-        assertSameCompartment(cx, proto, rep->compartment);
-#endif
-
-        PreInitRegExpObject pireo(proto);
-        RegExpObject *reproto = pireo.get();
-        if (!ResetRegExpObject(cx, reproto, rep)) {
-            pireo.fail();
-            return NULL;
-        }
-
-        pireo.succeed();
-    }
+    RegExpObject *reproto = &proto->asRegExp();
+    RegExpObjectBuilder builder(cx, reproto);
+    if (!builder.build(cx->runtime->emptyString, RegExpFlag(0)))
+        return NULL;
 
     if (!DefinePropertiesAndBrand(cx, proto, NULL, regexp_methods))
         return NULL;
@@ -419,7 +503,7 @@ js_InitRegExpClass(JSContext *cx, JSObject *obj)
  * RegExp.prototype.test doesn't need to create a results array, and we use
  * |execType| to perform this optimization.
  */
-static JSBool
+static bool
 ExecuteRegExp(JSContext *cx, Native native, uintN argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -430,24 +514,29 @@ ExecuteRegExp(JSContext *cx, Native native, uintN argc, Value *vp)
     if (!obj)
         return ok;
 
-    RegExpObject *reobj = obj->asRegExp();
-    RegExpPrivate *re = reobj->getPrivate();
-    if (!re)
-        return true;
+    RegExpObject *reobj = &obj->asRegExp();
+
+    RegExpMatcher matcher(cx);
+    if (reobj->startsWithAtomizedGreedyStar()) {
+        if (!matcher.resetWithTestOptimized(reobj))
+            return false;
+    } else {
+        if (!matcher.reset(reobj))
+            return false;
+    }
 
-    /*
-     * Code execution under this call could swap out the guts of |reobj|, so we
-     * have to take a defensive refcount here.
-     */
-    AutoRefCount<RegExpPrivate> arc(cx, NeedsIncRef<RegExpPrivate>(re));
     RegExpStatics *res = cx->regExpStatics();
 
     /* Step 2. */
-    JSString *input = js_ValueToString(cx, (args.length() > 0) ? args[0] : UndefinedValue());
+    JSString *input = ToString(cx, (args.length() > 0) ? args[0] : UndefinedValue());
     if (!input)
         return false;
 
     /* Step 3. */
+    JSLinearString *linearInput = input->ensureLinear(cx);
+    if (!linearInput)
+        return false;
+    const jschar *chars = linearInput->chars();
     size_t length = input->length();
 
     /* Step 4. */
@@ -459,7 +548,7 @@ ExecuteRegExp(JSContext *cx, Native native, uintN argc, Value *vp)
         return false;
 
     /* Steps 6-7 (with sticky extension). */
-    if (!re->global() && !re->sticky())
+    if (!matcher.global() && !matcher.sticky())
         i = 0;
 
     /* Step 9a. */
@@ -470,12 +559,15 @@ ExecuteRegExp(JSContext *cx, Native native, uintN argc, Value *vp)
     }
 
     /* Steps 8-21. */
+    RegExpExecType execType = (native == regexp_test) ? RegExpTest : RegExpExec;
     size_t lastIndexInt(i);
-    if (!re->execute(cx, res, input, &lastIndexInt, native == regexp_test, &args.rval()))
+    if (!ExecuteRegExp(cx, res, matcher, linearInput, chars, length, &lastIndexInt, execType,
+                       &args.rval())) {
         return false;
+    }
 
     /* Step 11 (with sticky extension). */
-    if (re->global() || (!args.rval().isNull() && re->sticky())) {
+    if (matcher.global() || (!args.rval().isNull() && matcher.sticky())) {
         if (args.rval().isNull())
             reobj->zeroLastIndex();
         else
