@@ -50,8 +50,8 @@ class Test:
         self.slow = False      # True means the test is slow-running
         self.allow_oom = False # True means that OOM is not considered a failure
         self.valgrind = False  # True means run under valgrind
-        self.tmflags = ''      # Value of TMFLAGS env var to pass
-        self.error = ''        # Errors to expect and consider passing
+        self.expect_error = '' # Errors to expect and consider passing
+        self.expect_status = 0 # Exit status to expect from shell
 
     def copy(self):
         t = Test(self.path)
@@ -59,8 +59,8 @@ class Test:
         t.slow = self.slow
         t.allow_oom = self.allow_oom
         t.valgrind = self.valgrind
-        t.tmflags = self.tmflags
-        t.error = self.error
+        t.expect_error = self.expect_error
+        t.expect_status = self.expect_status
         return t
 
     COOKIE = '|jit-test|'
@@ -81,10 +81,13 @@ class Test:
                 name, _, value = part.partition(':')
                 if value:
                     value = value.strip()
-                    if name == 'TMFLAGS':
-                        test.tmflags = value
-                    elif name == 'error':
-                        test.error = value
+                    if name == 'error':
+                        test.expect_error = value
+                    elif name == 'exitstatus':
+                        try:
+                            test.expect_status = int(value, 0);
+                        except ValueError:
+                            print("warning: couldn't parse exit status %s"%value)
                     else:
                         print('warning: unrecognized |jit-test| attribute %s'%part)
                 else:
@@ -201,9 +204,6 @@ def run_cmd_avoid_stdio(cmdline, env, timeout):
     return read_and_unlink(stdoutPath), read_and_unlink(stderrPath), code
 
 def run_test(test, lib_dir, shell_args):
-    env = os.environ.copy()
-    if test.tmflags:
-        env['TMFLAGS'] = test.tmflags
     cmd = get_test_cmd(test.path, test.jitflags, lib_dir, shell_args)
 
     if (test.valgrind and
@@ -225,7 +225,7 @@ def run_test(test, lib_dir, shell_args):
         run = run_cmd_avoid_stdio
     else:
         run = run_cmd
-    out, err, code, timed_out = run(cmd, env, OPTIONS.timeout)
+    out, err, code, timed_out = run(cmd, os.environ, OPTIONS.timeout)
 
     if OPTIONS.show_output:
         sys.stdout.write(out)
@@ -233,12 +233,12 @@ def run_test(test, lib_dir, shell_args):
         sys.stdout.write('Exit code: %s\n' % code)
     if test.valgrind:
         sys.stdout.write(err)
-    return (check_output(out, err, code, test.allow_oom, test.error), 
+    return (check_output(out, err, code, test),
             out, err, code, timed_out)
 
-def check_output(out, err, rc, allow_oom, expectedError):
-    if expectedError:
-        return expectedError in err
+def check_output(out, err, rc, test):
+    if test.expect_error:
+        return test.expect_error in err
 
     for line in out.split('\n'):
         if line.startswith('Trace stats check failed'):
@@ -248,10 +248,10 @@ def check_output(out, err, rc, allow_oom, expectedError):
         if 'Assertion failed:' in line:
             return False
 
-    if rc != 0:
+    if rc != test.expect_status:
         # Allow a non-zero exit code if we want to allow OOM, but only if we
         # actually got OOM.
-        return allow_oom and ': out of memory' in err
+        return test.allow_oom and ': out of memory' in err
 
     return True
 
@@ -267,11 +267,12 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
     if not OPTIONS.hide_progress and not OPTIONS.show_cmd:
         try:
             from progressbar import ProgressBar
-            pb = ProgressBar('', len(tests), 16)
+            pb = ProgressBar('', len(tests), 24)
         except ImportError:
             pass
 
     failures = []
+    timeouts = 0
     complete = False
     doing = 'before starting'
     try:
@@ -282,6 +283,8 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
 
             if not ok:
                 failures.append([ test, out, err, code, timed_out ])
+            if timed_out:
+                timeouts += 1
 
             if OPTIONS.tinderbox:
                 if ok:
@@ -297,7 +300,7 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
 
             n = i + 1
             if pb:
-                pb.label = '[%4d|%4d|%4d]'%(n - len(failures), len(failures), n)
+                pb.label = '[%4d|%4d|%4d|%4d]'%(n - len(failures), len(failures), timeouts, n)
                 pb.update(n)
         complete = True
     except KeyboardInterrupt:
@@ -353,7 +356,7 @@ def parse_jitflags():
                  for flags in OPTIONS.jitflags.split(',') ]
     for flags in jitflags:
         for flag in flags:
-            if flag not in ('-j', '-m', '-a', '-p', '-d', '-n'):
+            if flag not in ('-m', '-a', '-p', '-d', '-n'):
                 print('Invalid jit flag: "%s"'%flag)
                 sys.exit(1)
     return jitflags
@@ -417,8 +420,8 @@ def main(argv):
                   help='Enable the |valgrind| flag, if valgrind is in $PATH.')
     op.add_option('--valgrind-all', dest='valgrind_all', action='store_true',
                   help='Run all tests with valgrind, if valgrind is in $PATH.')
-    op.add_option('--jitflags', dest='jitflags', default='mjp',
-                  help='Example: --jitflags=j,mj,mjp to run each test with -j, -m -j, -m -j -p [default=%default]')
+    op.add_option('--jitflags', dest='jitflags', default='m,mn',
+                  help='Example: --jitflags=m,mn to run each test with -m, -m -n [default=%default]')
     op.add_option('--avoid-stdio', dest='avoid_stdio', action='store_true',
                   help='Use js-shell file indirection instead of piping stdio.')
     op.add_option('--write-failure-output', dest='write_failure_output', action='store_true',
