@@ -47,11 +47,20 @@
 #include "jsobj.h"
 #include "jsstr.h"
 
+inline JSAtom *
+js::AtomStateEntry::asPtr() const
+{
+    JS_ASSERT(bits != 0);
+    JSAtom *atom = reinterpret_cast<JSAtom *>(bits & NO_TAG_MASK);
+    JSString::readBarrier(atom);
+    return atom;
+}
+
 inline bool
 js_ValueToAtom(JSContext *cx, const js::Value &v, JSAtom **atomp)
 {
     if (!v.isString()) {
-        JSString *str = js_ValueToString(cx, v);
+        JSString *str = js::ToStringSlow(cx, v);
         if (!str)
             return false;
         JS::Anchor<JSString *> anchor(str);
@@ -120,7 +129,7 @@ js_InternNonIntElementId(JSContext *cx, JSObject *obj, const js::Value &idval,
 }
 
 inline bool
-js_Int32ToId(JSContext* cx, int32 index, jsid* id)
+js_Int32ToId(JSContext* cx, int32_t index, jsid* id)
 {
     if (INT_FITS_IN_JSID(index)) {
         *id = INT_TO_JSID(index);
@@ -145,7 +154,7 @@ namespace js {
  */
 template <typename T>
 inline mozilla::RangedPtr<T>
-BackfillIndexInCharBuffer(uint32 index, mozilla::RangedPtr<T> end)
+BackfillIndexInCharBuffer(uint32_t index, mozilla::RangedPtr<T> end)
 {
 #ifdef DEBUG
     /*
@@ -157,7 +166,7 @@ BackfillIndexInCharBuffer(uint32 index, mozilla::RangedPtr<T> end)
 #endif
 
     do {
-        uint32 next = index / 10, digit = index % 10;
+        uint32_t next = index / 10, digit = index % 10;
         *--end = '0' + digit;
         index = next;
     } while (index > 0);
@@ -166,25 +175,25 @@ BackfillIndexInCharBuffer(uint32 index, mozilla::RangedPtr<T> end)
 }
 
 inline bool
-IndexToId(JSContext *cx, uint32 index, jsid *idp)
+IndexToId(JSContext *cx, uint32_t index, jsid *idp)
 {
     if (index <= JSID_INT_MAX) {
         *idp = INT_TO_JSID(index);
         return true;
     }
 
-    extern bool IndexToIdSlow(JSContext *cx, uint32 index, jsid *idp);
+    extern bool IndexToIdSlow(JSContext *cx, uint32_t index, jsid *idp);
     return IndexToIdSlow(cx, index, idp);
 }
 
-static JS_ALWAYS_INLINE JSString *
+static JS_ALWAYS_INLINE JSFlatString *
 IdToString(JSContext *cx, jsid id)
 {
     if (JSID_IS_STRING(id))
-        return JSID_TO_STRING(id);
+        return JSID_TO_ATOM(id);
     if (JS_LIKELY(JSID_IS_INT(id)))
-        return js_IntToString(cx, JSID_TO_INT(id));
-    return js_ValueToString(cx, IdToValue(id));
+        return js_IntToString(cx, JSID_TO_INT(id))->ensureFlat(cx);
+    return ToStringSlow(cx, IdToValue(id))->ensureFlat(cx);
 }
 
 } // namespace js
