@@ -48,6 +48,4 @@
 
 typedef JSIntn intN;
 typedef JSUintn uintN;
-typedef JSUword jsuword;
-typedef JSWord jsword;
 #endif /* jscompat_h___ */
