@@ -43,9 +43,10 @@
  */
 #include <stdlib.h>
 #include <string.h>
+
+#include "mozilla/Util.h"
+
 #include "jstypes.h"
-#include "jsstdint.h"
-#include "jsbit.h"
 #include "jsutil.h"
 #include "jsprf.h"
 #include "jsapi.h"
@@ -61,16 +62,18 @@
 #include "jsopcode.h"
 #include "jsscope.h"
 #include "jsscript.h"
-#include "jsstaticcheck.h"
 #include "jswrapper.h"
 
 #include "vm/GlobalObject.h"
 
 #include "jsinferinlines.h"
 #include "jsobjinlines.h"
+#include "jsstrinlines.h"
 
 #include "vm/Stack-inl.h"
+#include "vm/String-inl.h"
 
+using namespace mozilla;
 using namespace js;
 using namespace js::gc;
 using namespace js::types;
@@ -111,7 +114,7 @@ Class js::ErrorClass = {
 };
 
 typedef struct JSStackTraceElem {
-    JSString            *funName;
+    js::HeapPtrString   funName;
     size_t              argc;
     const char          *filename;
     uintN               ulineno;
@@ -120,8 +123,8 @@ typedef struct JSStackTraceElem {
 typedef struct JSExnPrivate {
     /* A copy of the JSErrorReport originally generated. */
     JSErrorReport       *errorReport;
-    JSString            *message;
-    JSString            *filename;
+    js::HeapPtrString   message;
+    js::HeapPtrString   filename;
     uintN               lineno;
     size_t              stackDepth;
     intN                exnType;
@@ -157,7 +160,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
     size_t i, argsArraySize, argsCopySize, argSize;
     size_t mallocSize;
     JSErrorReport *copy;
-    uint8 *cursor;
+    uint8_t *cursor;
 
 #define JS_CHARS_SIZE(jschars) ((js_strlen(jschars) + 1) * sizeof(jschar))
 
@@ -185,7 +188,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
      */
     mallocSize = sizeof(JSErrorReport) + argsArraySize + argsCopySize +
                  ucmessageSize + uclinebufSize + linebufSize + filenameSize;
-    cursor = (uint8 *)cx->malloc_(mallocSize);
+    cursor = (uint8_t *)cx->malloc_(mallocSize);
     if (!cursor)
         return NULL;
 
@@ -199,22 +202,22 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
         for (i = 0; report->messageArgs[i]; ++i) {
             copy->messageArgs[i] = (const jschar *)cursor;
             argSize = JS_CHARS_SIZE(report->messageArgs[i]);
-            memcpy(cursor, report->messageArgs[i], argSize);
+            js_memcpy(cursor, report->messageArgs[i], argSize);
             cursor += argSize;
         }
         copy->messageArgs[i] = NULL;
-        JS_ASSERT(cursor == (uint8 *)copy->messageArgs[0] + argsCopySize);
+        JS_ASSERT(cursor == (uint8_t *)copy->messageArgs[0] + argsCopySize);
     }
 
     if (report->ucmessage) {
         copy->ucmessage = (const jschar *)cursor;
-        memcpy(cursor, report->ucmessage, ucmessageSize);
+        js_memcpy(cursor, report->ucmessage, ucmessageSize);
         cursor += ucmessageSize;
     }
 
     if (report->uclinebuf) {
         copy->uclinebuf = (const jschar *)cursor;
-        memcpy(cursor, report->uclinebuf, uclinebufSize);
+        js_memcpy(cursor, report->uclinebuf, uclinebufSize);
         cursor += uclinebufSize;
         if (report->uctokenptr) {
             copy->uctokenptr = copy->uclinebuf + (report->uctokenptr -
@@ -224,7 +227,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
 
     if (report->linebuf) {
         copy->linebuf = (const char *)cursor;
-        memcpy(cursor, report->linebuf, linebufSize);
+        js_memcpy(cursor, report->linebuf, linebufSize);
         cursor += linebufSize;
         if (report->tokenptr) {
             copy->tokenptr = copy->linebuf + (report->tokenptr -
@@ -234,9 +237,12 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
 
     if (report->filename) {
         copy->filename = (const char *)cursor;
-        memcpy(cursor, report->filename, filenameSize);
+        js_memcpy(cursor, report->filename, filenameSize);
     }
-    JS_ASSERT(cursor + filenameSize == (uint8 *)copy + mallocSize);
+    JS_ASSERT(cursor + filenameSize == (uint8_t *)copy + mallocSize);
+
+    /* HOLD called by the destination error object. */
+    copy->originPrincipals = report->originPrincipals;
 
     /* Copy non-pointer members. */
     copy->lineno = report->lineno;
@@ -249,7 +255,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
     return copy;
 }
 
-static jsval *
+static HeapValue *
 GetStackTraceValueBuffer(JSExnPrivate *priv)
 {
     /*
@@ -260,7 +266,7 @@ GetStackTraceValueBuffer(JSExnPrivate *priv)
      */
     JS_STATIC_ASSERT(sizeof(JSStackTraceElem) % sizeof(jsval) == 0);
 
-    return (jsval *)(priv->stackElems + priv->stackDepth);
+    return reinterpret_cast<HeapValue *>(priv->stackElems + priv->stackDepth);
 }
 
 struct SuppressErrorsGuard
@@ -290,6 +296,9 @@ struct AppendArg {
     }
 };
 
+static void
+SetExnPrivate(JSContext *cx, JSObject *exnObject, JSExnPrivate *priv);
+
 static bool
 InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
                JSString *filename, uintN lineno, JSErrorReport *report, intN exnType)
@@ -326,17 +335,17 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
                 return false;
             JSStackTraceElem &frame = frames.back();
             if (fp->isNonEvalFunctionFrame()) {
-                frame.funName = fp->fun()->atom ? fp->fun()->atom : cx->runtime->emptyString;
+                frame.funName.init(fp->fun()->atom ? fp->fun()->atom : cx->runtime->emptyString);
                 frame.argc = fp->numActualArgs();
                 if (!fp->forEachCanonicalActualArg(AppendArg(values)))
                     return false;
             } else {
-                frame.funName = NULL;
+                frame.funName.init(NULL);
                 frame.argc = 0;
             }
             if (fp->isScriptFrame()) {
                 frame.filename = fp->script()->filename;
-                frame.ulineno = js_FramePCToLineNumber(cx, fp, i.pc());
+                frame.ulineno = js_PCToLineNumber(cx, fp->script(), i.pc());
             } else {
                 frame.ulineno = 0;
                 frame.filename = NULL;
@@ -349,12 +358,15 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
 
     size_t nbytes = offsetof(JSExnPrivate, stackElems) +
                     frames.length() * sizeof(JSStackTraceElem) +
-                    values.length() * sizeof(Value);
+                    values.length() * sizeof(HeapValue);
 
     JSExnPrivate *priv = (JSExnPrivate *)cx->malloc_(nbytes);
     if (!priv)
         return false;
 
+    /* Initialize to zero so that write barriers don't witness undefined values. */
+    memset(priv, 0, nbytes);
+
     if (report) {
         /*
          * Construct a new copy of the error report struct. We can't use the
@@ -371,20 +383,21 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
         priv->errorReport = NULL;
     }
 
-    priv->message = message;
-    priv->filename = filename;
+    priv->message.init(message);
+    priv->filename.init(filename);
     priv->lineno = lineno;
     priv->stackDepth = frames.length();
     priv->exnType = exnType;
 
     JSStackTraceElem *framesDest = priv->stackElems;
-    Value *valuesDest = reinterpret_cast<Value *>(framesDest + frames.length());
+    HeapValue *valuesDest = reinterpret_cast<HeapValue *>(framesDest + frames.length());
     JS_ASSERT(valuesDest == GetStackTraceValueBuffer(priv));
 
     PodCopy(framesDest, frames.begin(), frames.length());
-    PodCopy(valuesDest, values.begin(), values.length());
+    for (size_t i = 0; i < values.length(); ++i)
+        valuesDest[i].init(cx->compartment, values[i]);
 
-    exnObject->setPrivate(priv);
+    SetExnPrivate(cx, exnObject, priv);
     return true;
 }
 
@@ -401,7 +414,7 @@ exn_trace(JSTracer *trc, JSObject *obj)
     JSExnPrivate *priv;
     JSStackTraceElem *elem;
     size_t vcount, i;
-    jsval *vp, v;
+    HeapValue *vp;
 
     priv = GetExnPrivate(obj);
     if (priv) {
@@ -420,18 +433,34 @@ exn_trace(JSTracer *trc, JSObject *obj)
         }
         vp = GetStackTraceValueBuffer(priv);
         for (i = 0; i != vcount; ++i, ++vp) {
-            v = *vp;
-            JS_CALL_VALUE_TRACER(trc, v, "stack trace argument");
+            MarkValue(trc, *vp, "stack trace argument");
         }
     }
 }
 
+/* NB: An error object's private must be set through this function. */
+static void
+SetExnPrivate(JSContext *cx, JSObject *exnObject, JSExnPrivate *priv)
+{
+    JS_ASSERT(!exnObject->getPrivate());
+    JS_ASSERT(exnObject->isError());
+    if (JSErrorReport *report = priv->errorReport) {
+        if (JSPrincipals *prin = report->originPrincipals)
+            JSPRINCIPALS_HOLD(cx, prin);
+    }
+    exnObject->setPrivate(priv);
+}
+
 static void
 exn_finalize(JSContext *cx, JSObject *obj)
 {
     if (JSExnPrivate *priv = GetExnPrivate(obj)) {
-        if (JSErrorReport *report = priv->errorReport)
+        if (JSErrorReport *report = priv->errorReport) {
+            /* HOLD called by SetExnPrivate. */
+            if (JSPrincipals *prin = report->originPrincipals)
+                JSPRINCIPALS_DROP(cx, prin);
             cx->free_(report);
+        }
         cx->free_(priv);
     }
 }
@@ -492,8 +521,6 @@ exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             if (!stack)
                 return false;
 
-            /* Allow to GC all things that were used to build stack trace. */
-            priv->stackDepth = 0;
             prop = js_stack_str;
             v = STRING_TO_JSVAL(stack);
             attrs = JSPROP_ENUMERATE;
@@ -544,7 +571,7 @@ ValueToShortSource(JSContext *cx, const Value &v)
         /*
          * XXX Avoid function decompilation bloat for now.
          */
-        str = JS_GetFunctionId(obj->getFunctionPrivate());
+        str = JS_GetFunctionId(obj->toFunction());
         if (!str && !(str = js_ValueToSource(cx, v))) {
             /*
              * Continue to soldier on if the function couldn't be
@@ -576,7 +603,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
     jschar *stackbuf;
     size_t stacklen, stackmax;
     JSStackTraceElem *elem, *endElem;
-    jsval *values;
+    HeapValue *values;
     size_t i;
     JSString *str;
     const char *cp;
@@ -618,8 +645,8 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
                 length_ >= STACK_LENGTH_LIMIT - stacklen) {                   \
                 goto done;                                                    \
             }                                                                 \
-            stackmax = JS_BIT(JS_CeilingLog2(stacklen + length_));            \
-            ptr_ = cx->realloc_(stackbuf, (stackmax+1) * sizeof(jschar));      \
+            stackmax = RoundUpPow2(stacklen + length_);                       \
+            ptr_ = cx->realloc_(stackbuf, (stackmax+1) * sizeof(jschar));     \
             if (!ptr_)                                                        \
                 goto bad;                                                     \
             stackbuf = (jschar *) ptr_;                                       \
@@ -694,10 +721,6 @@ FilenameToString(JSContext *cx, const char *filename)
     return JS_NewStringCopyZ(cx, filename);
 }
 
-enum {
-    JSSLOT_ERROR_EXNTYPE = 0
-};
-
 static JSBool
 Exception(JSContext *cx, uintN argc, Value *vp)
 {
@@ -720,14 +743,14 @@ Exception(JSContext *cx, uintN argc, Value *vp)
     }
 
     JSObject *errProto = &protov.toObject();
-    JSObject *obj = NewNativeClassInstance(cx, &ErrorClass, errProto, errProto->getParent());
+    JSObject *obj = NewObjectWithGivenProto(cx, &ErrorClass, errProto, NULL);
     if (!obj)
         return false;
 
     /* Set the 'message' property. */
     JSString *message;
-    if (args.argc() != 0 && !args[0].isUndefined()) {
-        message = js_ValueToString(cx, args[0]);
+    if (args.length() != 0 && !args[0].isUndefined()) {
+        message = ToString(cx, args[0]);
         if (!message)
             return false;
         args[0].setString(message);
@@ -742,8 +765,8 @@ Exception(JSContext *cx, uintN argc, Value *vp)
 
     /* Set the 'fileName' property. */
     JSString *filename;
-    if (args.argc() > 1) {
-        filename = js_ValueToString(cx, args[1]);
+    if (args.length() > 1) {
+        filename = ToString(cx, args[1]);
         if (!filename)
             return false;
         args[1].setString(filename);
@@ -759,14 +782,14 @@ Exception(JSContext *cx, uintN argc, Value *vp)
 
     /* Set the 'lineNumber' property. */
     uint32_t lineno;
-    if (args.argc() > 2) {
-        if (!ValueToECMAUint32(cx, args[2], &lineno))
+    if (args.length() > 2) {
+        if (!ToUint32(cx, args[2], &lineno))
             return false;
     } else {
-        lineno = iter.done() ? 0 : js_FramePCToLineNumber(cx, iter.fp(), iter.pc());
+        lineno = iter.done() ? 0 : js_PCToLineNumber(cx, iter.fp()->script(), iter.pc());
     }
 
-    intN exnType = args.callee().getReservedSlot(JSSLOT_ERROR_EXNTYPE).toInt32();
+    intN exnType = args.callee().toFunction()->getExtendedSlot(0).toInt32();
     if (!InitExnPrivate(cx, obj, message, filename, lineno, NULL, exnType))
         return false;
 
@@ -774,68 +797,80 @@ Exception(JSContext *cx, uintN argc, Value *vp)
     return true;
 }
 
-/*
- * Convert to string.
- *
- * This method only uses JavaScript-modifiable properties name, message.  It
- * is left to the host to check for private data and report filename and line
- * number information along with this message.
- */
+/* ES5 15.11.4.4 (NB: with subsequent errata). */
 static JSBool
 exn_toString(JSContext *cx, uintN argc, Value *vp)
 {
-    jsval v;
-    JSString *name, *message, *result;
-    jschar *chars, *cp;
-    size_t name_length, message_length, length;
+    JS_CHECK_RECURSION(cx, return false);
+    CallArgs args = CallArgsFromVp(argc, vp);
 
-    JSObject *obj = ToObject(cx, &vp[1]);
-    if (!obj)
-        return JS_FALSE;
-    if (!obj->getProperty(cx, cx->runtime->atomState.nameAtom, &v))
-        return JS_FALSE;
-    name = JSVAL_IS_STRING(v) ? JSVAL_TO_STRING(v) : cx->runtime->emptyString;
-    vp->setString(name);
-
-    if (!JS_GetProperty(cx, obj, js_message_str, &v))
-        return JS_FALSE;
-    message = JSVAL_IS_STRING(v) ? JSVAL_TO_STRING(v)
-                                 : cx->runtime->emptyString;
-
-    if (message->length() != 0) {
-        name_length = name->length();
-        message_length = message->length();
-        length = (name_length ? name_length + 2 : 0) + message_length;
-        cp = chars = (jschar *) cx->malloc_((length + 1) * sizeof(jschar));
-        if (!chars)
-            return JS_FALSE;
-
-        if (name_length) {
-            const jschar *name_chars = name->getChars(cx);
-            if (!name_chars)
-                return JS_FALSE;
-            js_strncpy(cp, name_chars, name_length);
-            cp += name_length;
-            *cp++ = ':'; *cp++ = ' ';
-        }
-        const jschar *message_chars = message->getChars(cx);
-        if (!message_chars)
-            return JS_FALSE;
-        js_strncpy(cp, message_chars, message_length);
-        cp += message_length;
-        *cp = 0;
-
-        result = js_NewString(cx, chars, length);
-        if (!result) {
-            cx->free_(chars);
-            return JS_FALSE;
-        }
-    } else {
-        result = name;
+    /* Step 2. */
+    if (!args.thisv().isObject()) {
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_BAD_PROTOTYPE, "Error");
+        return false;
     }
 
-    vp->setString(result);
-    return JS_TRUE;
+    /* Step 1. */
+    JSObject &obj = args.thisv().toObject();
+
+    /* Step 3. */
+    Value nameVal;
+    if (!obj.getProperty(cx, cx->runtime->atomState.nameAtom, &nameVal))
+        return false;
+
+    /* Step 4. */
+    JSString *name;
+    if (nameVal.isUndefined()) {
+        name = CLASS_ATOM(cx, Error);
+    } else {
+        name = ToString(cx, nameVal);
+        if (!name)
+            return false;
+    }
+
+    /* Step 5. */
+    Value msgVal;
+    if (!obj.getProperty(cx, cx->runtime->atomState.messageAtom, &msgVal))
+        return false;
+
+    /* Step 6. */
+    JSString *message;
+    if (msgVal.isUndefined()) {
+        message = cx->runtime->emptyString;
+    } else {
+        message = ToString(cx, msgVal);
+        if (!message)
+            return false;
+    }
+
+    /* Step 7. */
+    if (name->empty() && message->empty()) {
+        args.rval().setString(CLASS_ATOM(cx, Error));
+        return true;
+    }
+
+    /* Step 8. */
+    if (name->empty()) {
+        args.rval().setString(message);
+        return true;
+    }
+
+    /* Step 9. */
+    if (message->empty()) {
+        args.rval().setString(name);
+        return true;
+    }
+
+    /* Step 10. */
+    StringBuffer sb(cx);
+    if (!sb.append(name) || !sb.append(": ") || !sb.append(message))
+        return false;
+
+    JSString *str = sb.finishString();
+    if (!str)
+        return false;
+    args.rval().setString(str);
+    return true;
 }
 
 #if JS_HAS_TOSOURCE
@@ -845,134 +880,76 @@ exn_toString(JSContext *cx, uintN argc, Value *vp)
 static JSBool
 exn_toSource(JSContext *cx, uintN argc, Value *vp)
 {
-    JSString *name, *message, *filename, *lineno_as_str, *result;
-    jsval localroots[3] = {JSVAL_NULL, JSVAL_NULL, JSVAL_NULL};
-    size_t lineno_length, name_length, message_length, filename_length, length;
-    jschar *chars, *cp;
+    JS_CHECK_RECURSION(cx, return false);
+    CallArgs args = CallArgsFromVp(argc, vp);
 
-    JSObject *obj = ToObject(cx, &vp[1]);
+    JSObject *obj = ToObject(cx, &args.thisv());
     if (!obj)
         return false;
-    if (!obj->getProperty(cx, cx->runtime->atomState.nameAtom, vp))
-        return false;
-    name = js_ValueToString(cx, *vp);
-    if (!name)
-        return false;
-    vp->setString(name);
 
+    Value nameVal;
+    JSString *name;
+    if (!obj->getProperty(cx, cx->runtime->atomState.nameAtom, &nameVal) ||
+        !(name = ToString(cx, nameVal)))
     {
-        AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(localroots), localroots);
-
-#ifdef __GNUC__
-        message = filename = NULL;
-#endif
-        if (!JS_GetProperty(cx, obj, js_message_str, &localroots[0]) ||
-            !(message = js_ValueToSource(cx, localroots[0]))) {
-            return false;
-        }
-        localroots[0] = STRING_TO_JSVAL(message);
-
-        if (!JS_GetProperty(cx, obj, js_fileName_str, &localroots[1]) ||
-            !(filename = js_ValueToSource(cx, localroots[1]))) {
-            return false;
-        }
-        localroots[1] = STRING_TO_JSVAL(filename);
-
-        if (!JS_GetProperty(cx, obj, js_lineNumber_str, &localroots[2]))
-            return false;
-        uint32_t lineno;
-        if (!ValueToECMAUint32(cx, localroots[2], &lineno))
-            return false;
-
-        if (lineno != 0) {
-            lineno_as_str = js_ValueToString(cx, localroots[2]);
-            if (!lineno_as_str)
-                return false;
-            lineno_length = lineno_as_str->length();
-        } else {
-            lineno_as_str = NULL;
-            lineno_length = 0;
-        }
-
-        /* Magic 8, for the characters in ``(new ())''. */
-        name_length = name->length();
-        message_length = message->length();
-        length = 8 + name_length + message_length;
-
-        filename_length = filename->length();
-        if (filename_length != 0) {
-            /* append filename as ``, {filename}'' */
-            length += 2 + filename_length;
-            if (lineno_as_str) {
-                /* append lineno as ``, {lineno_as_str}'' */
-                length += 2 + lineno_length;
-            }
-        } else {
-            if (lineno_as_str) {
-                /*
-                 * no filename, but have line number,
-                 * need to append ``, "", {lineno_as_str}''
-                 */
-                length += 6 + lineno_length;
-            }
-        }
-
-        cp = chars = (jschar *) cx->malloc_((length + 1) * sizeof(jschar));
-        if (!chars)
-            return false;
-
-        *cp++ = '('; *cp++ = 'n'; *cp++ = 'e'; *cp++ = 'w'; *cp++ = ' ';
-        const jschar *name_chars = name->getChars(cx);
-        if (!name_chars)
-            return false;
-        js_strncpy(cp, name_chars, name_length);
-        cp += name_length;
-        *cp++ = '(';
-        const jschar *message_chars = message->getChars(cx);
-        if (!message_chars)
-            return false;
-        if (message_length != 0) {
-            js_strncpy(cp, message_chars, message_length);
-            cp += message_length;
-        }
-
-        if (filename_length != 0) {
-            /* append filename as ``, {filename}'' */
-            *cp++ = ','; *cp++ = ' ';
-            const jschar *filename_chars = filename->getChars(cx);
-            if (!filename_chars)
-                return false;
-            js_strncpy(cp, filename_chars, filename_length);
-            cp += filename_length;
-        } else {
-            if (lineno_as_str) {
-                /*
-                 * no filename, but have line number,
-                 * need to append ``, "", {lineno_as_str}''
-                 */
-                *cp++ = ','; *cp++ = ' '; *cp++ = '"'; *cp++ = '"';
-            }
-        }
-        if (lineno_as_str) {
-            /* append lineno as ``, {lineno_as_str}'' */
-            *cp++ = ','; *cp++ = ' ';
-            const jschar *lineno_chars = lineno_as_str->getChars(cx);
-            if (!lineno_chars)
-                return false;
-            js_strncpy(cp, lineno_chars, lineno_length);
-            cp += lineno_length;
-        }
-
-        *cp++ = ')'; *cp++ = ')'; *cp = 0;
-
-        result = js_NewString(cx, chars, length);
-        if (!result) {
-            cx->free_(chars);
-            return false;
-        }
-        vp->setString(result);
-        return true;
+        return false;
     }
+
+    Value messageVal;
+    JSString *message;
+    if (!obj->getProperty(cx, cx->runtime->atomState.messageAtom, &messageVal) ||
+        !(message = js_ValueToSource(cx, messageVal)))
+    {
+        return false;
+    }
+
+    Value filenameVal;
+    JSString *filename;
+    if (!obj->getProperty(cx, cx->runtime->atomState.fileNameAtom, &filenameVal) ||
+        !(filename = js_ValueToSource(cx, filenameVal)))
+    {
+        return false;
+    }
+
+    Value linenoVal;
+    uint32_t lineno;
+    if (!obj->getProperty(cx, cx->runtime->atomState.lineNumberAtom, &linenoVal) ||
+        !ToUint32(cx, linenoVal, &lineno))
+    {
+        return false;
+    }
+
+    StringBuffer sb(cx);
+    if (!sb.append("(new ") || !sb.append(name) || !sb.append("("))
+        return false;
+
+    if (!sb.append(message))
+        return false;
+
+    if (!filename->empty()) {
+        if (!sb.append(", ") || !sb.append(filename))
+            return false;
+    }
+    if (lineno != 0) {
+        /* We have a line, but no filename, add empty string */
+        if (filename->empty() && !sb.append(", \"\""))
+                return false;
+
+        JSString *linenumber = ToString(cx, linenoVal);
+        if (!linenumber)
+            return false;
+        if (!sb.append(", ") || !sb.append(linenumber))
+            return false;
+    }
+
+    if (!sb.append("))"))
+        return false;
+
+    JSString *str = sb.finishString();
+    if (!str)
+        return false;
+    args.rval().setString(str);
+    return true;
 }
 #endif
 
@@ -1029,10 +1006,11 @@ InitErrorClass(JSContext *cx, GlobalObject *global, intN type, JSObject &proto)
     }
 
     /* Create the corresponding constructor. */
-    JSFunction *ctor = global->createConstructor(cx, Exception, &ErrorClass, name, 1);
+    JSFunction *ctor = global->createConstructor(cx, Exception, &ErrorClass, name, 1,
+                                                 JSFunction::ExtendedFinalizeKind);
     if (!ctor)
         return NULL;
-    ctor->setReservedSlot(JSSLOT_ERROR_EXNTYPE, Int32Value(int32(type)));
+    ctor->setExtendedSlot(0, Int32Value(int32_t(type)));
 
     if (!LinkConstructorAndPrototype(cx, ctor, errorProto))
         return NULL;
@@ -1051,7 +1029,7 @@ js_InitExceptionClasses(JSContext *cx, JSObject *obj)
     JS_ASSERT(obj->isGlobal());
     JS_ASSERT(obj->isNative());
 
-    GlobalObject *global = obj->asGlobal();
+    GlobalObject *global = &obj->asGlobal();
 
     JSObject *objectProto;
     if (!js_GetClassPrototype(cx, global, JSProto_Object, &objectProto))
@@ -1156,7 +1134,7 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
 
     /* Protect the newly-created strings below from nesting GCs. */
     PodArrayZero(tv);
-    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(tv), tv);
+    AutoArrayRooter tvr(cx, ArrayLength(tv), tv);
 
     /*
      * Try to get an appropriate prototype by looking up the corresponding
@@ -1168,7 +1146,7 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
         goto out;
     tv[0] = OBJECT_TO_JSVAL(errProto);
 
-    errObject = NewNativeClassInstance(cx, &ErrorClass, errProto, errProto->getParent());
+    errObject = NewObjectWithGivenProto(cx, &ErrorClass, errProto, NULL);
     if (!errObject) {
         ok = JS_FALSE;
         goto out;
@@ -1221,13 +1199,13 @@ js_ReportUncaughtException(JSContext *cx)
         return false;
 
     PodArrayZero(roots);
-    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), roots);
+    AutoArrayRooter tvr(cx, ArrayLength(roots), roots);
 
     /*
-     * Because js_ValueToString below could error and an exception object
-     * could become unrooted, we must root exnObject.  Later, if exnObject is
-     * non-null, we need to root other intermediates, so allocate an operand
-     * stack segment to protect all of these values.
+     * Because ToString below could error and an exception object could become
+     * unrooted, we must root exnObject.  Later, if exnObject is non-null, we
+     * need to root other intermediates, so allocate an operand stack segment
+     * to protect all of these values.
      */
     if (JSVAL_IS_PRIMITIVE(exn)) {
         exnObject = NULL;
@@ -1239,13 +1217,13 @@ js_ReportUncaughtException(JSContext *cx)
     JS_ClearPendingException(cx);
     reportp = js_ErrorFromException(cx, exn);
 
-    /* XXX L10N angels cry once again (see also jsemit.c, /L10N gaffes/) */
-    str = js_ValueToString(cx, exn);
+    /* XXX L10N angels cry once again. see also everywhere else */
+    str = ToString(cx, exn);
     JSAutoByteString bytesStorage;
     if (!str) {
         bytes = "unknown (can't convert to string)";
     } else {
-        roots[1] = STRING_TO_JSVAL(str);
+        roots[1] = StringValue(str);
         if (!bytesStorage.encode(cx, str))
             return false;
         bytes = bytesStorage.ptr();
@@ -1264,14 +1242,14 @@ js_ReportUncaughtException(JSContext *cx)
 
         if (!JS_GetProperty(cx, exnObject, js_fileName_str, &roots[3]))
             return false;
-        str = js_ValueToString(cx, roots[3]);
+        str = ToString(cx, roots[3]);
         if (!str || !filename.encode(cx, str))
             return false;
 
         if (!JS_GetProperty(cx, exnObject, js_lineNumber_str, &roots[4]))
             return false;
         uint32_t lineno;
-        if (!ValueToECMAUint32(cx, roots[4], &lineno))
+        if (!ToUint32(cx, roots[4], &lineno))
             return false;
 
         reportp = &report;
@@ -1308,9 +1286,9 @@ js_CopyErrorObject(JSContext *cx, JSObject *errobj, JSObject *scope)
     assertSameCompartment(cx, scope);
     JSExnPrivate *priv = GetExnPrivate(errobj);
 
-    uint32 stackDepth = priv->stackDepth;
+    uint32_t stackDepth = priv->stackDepth;
     size_t valueCount = 0;
-    for (uint32 i = 0; i < stackDepth; i++)
+    for (uint32_t i = 0; i < stackDepth; i++)
         valueCount += priv->stackElems[i].argc;
 
     size_t size = offsetof(JSExnPrivate, stackElems) +
@@ -1340,11 +1318,11 @@ js_CopyErrorObject(JSContext *cx, JSObject *errobj, JSObject *scope)
     } else {
         copy->errorReport = NULL;
     }
-    copy->message = priv->message;
+    copy->message.init(priv->message);
     if (!cx->compartment->wrap(cx, &copy->message))
         return NULL;
     JS::Anchor<JSString *> messageAnchor(copy->message);
-    copy->filename = priv->filename;
+    copy->filename.init(priv->filename);
     if (!cx->compartment->wrap(cx, &copy->filename))
         return NULL;
     JS::Anchor<JSString *> filenameAnchor(copy->filename);
@@ -1354,10 +1332,10 @@ js_CopyErrorObject(JSContext *cx, JSObject *errobj, JSObject *scope)
 
     // Create the Error object.
     JSObject *proto;
-    if (!js_GetClassPrototype(cx, scope->getGlobal(), GetExceptionProtoKey(copy->exnType), &proto))
+    if (!js_GetClassPrototype(cx, &scope->global(), GetExceptionProtoKey(copy->exnType), &proto))
         return NULL;
-    JSObject *copyobj = NewNativeClassInstance(cx, &ErrorClass, proto, proto->getParent());
-    copyobj->setPrivate(copy);
+    JSObject *copyobj = NewObjectWithGivenProto(cx, &ErrorClass, proto, NULL);
+    SetExnPrivate(cx, copyobj, copy);
     autoFree.p = NULL;
     return copyobj;
 }
