@@ -43,6 +43,8 @@
 #include "jsapi.h"
 #include "jsclass.h"
 
+#include "gc/Barrier.h"
+
 typedef struct JSProperty JSProperty;
 
 namespace js {
@@ -58,12 +60,18 @@ namespace js {
 struct JS_FRIEND_API(ArrayBuffer) {
     static Class slowClass;
     static JSPropertySpec jsprops[];
+    static JSFunctionSpec jsfuncs[];
 
     static JSBool prop_getByteLength(JSContext *cx, JSObject *obj, jsid id, Value *vp);
 
+    static JSBool fun_slice(JSContext *cx, uintN argc, Value *vp);
+
     static JSBool class_constructor(JSContext *cx, uintN argc, Value *vp);
 
-    static JSObject *create(JSContext *cx, int32 nbytes);
+    static JSObject *create(JSContext *cx, int32_t nbytes, uint8_t *contents = NULL);
+
+    static JSObject *createSlice(JSContext *cx, JSObject *arrayBuffer,
+                                 uint32_t begin, uint32_t end);
 
     ArrayBuffer()
     {
@@ -75,25 +83,27 @@ struct JS_FRIEND_API(ArrayBuffer) {
     obj_trace(JSTracer *trc, JSObject *obj);
 
     static JSBool
-    obj_lookupProperty(JSContext *cx, JSObject *obj, jsid id,
+    obj_lookupGeneric(JSContext *cx, JSObject *obj, jsid id,
+                      JSObject **objp, JSProperty **propp);
+    static JSBool
+    obj_lookupProperty(JSContext *cx, JSObject *obj, PropertyName *name,
                        JSObject **objp, JSProperty **propp);
-
     static JSBool
-    obj_lookupElement(JSContext *cx, JSObject *obj, uint32 index,
+    obj_lookupElement(JSContext *cx, JSObject *obj, uint32_t index,
                       JSObject **objp, JSProperty **propp);
-
     static JSBool
     obj_lookupSpecial(JSContext *cx, JSObject *obj, SpecialId sid, JSObject **objp,
                       JSProperty **propp);
 
     static JSBool
-    obj_defineProperty(JSContext *cx, JSObject *obj, jsid id, const Value *v,
+    obj_defineGeneric(JSContext *cx, JSObject *obj, jsid id, const Value *v,
+                      PropertyOp getter, StrictPropertyOp setter, uintN attrs);
+    static JSBool
+    obj_defineProperty(JSContext *cx, JSObject *obj, PropertyName *name, const Value *v,
                        PropertyOp getter, StrictPropertyOp setter, uintN attrs);
-
     static JSBool
-    obj_defineElement(JSContext *cx, JSObject *obj, uint32 index, const Value *v,
+    obj_defineElement(JSContext *cx, JSObject *obj, uint32_t index, const Value *v,
                       PropertyOp getter, StrictPropertyOp setter, uintN attrs);
-
     static JSBool
     obj_defineSpecial(JSContext *cx, JSObject *obj, SpecialId sid, const Value *v,
                       PropertyOp getter, StrictPropertyOp setter, uintN attrs);
@@ -106,44 +116,45 @@ struct JS_FRIEND_API(ArrayBuffer) {
                     Value *vp);
 
     static JSBool
-    obj_getElement(JSContext *cx, JSObject *obj, JSObject *receiver, uint32 index, Value *vp);
+    obj_getElement(JSContext *cx, JSObject *obj, JSObject *receiver, uint32_t index, Value *vp);
+    static JSBool
+    obj_getElementIfPresent(JSContext *cx, JSObject *obj, JSObject *receiver, uint32_t index,
+                            Value *vp, bool *present);
 
     static JSBool
     obj_getSpecial(JSContext *cx, JSObject *obj, JSObject *receiver, SpecialId sid, Value *vp);
 
     static JSBool
-    obj_setProperty(JSContext *cx, JSObject *obj, jsid id, Value *vp, JSBool strict);
-
+    obj_setGeneric(JSContext *cx, JSObject *obj, jsid id, Value *vp, JSBool strict);
     static JSBool
-    obj_setElement(JSContext *cx, JSObject *obj, uint32 index, Value *vp, JSBool strict);
-
+    obj_setProperty(JSContext *cx, JSObject *obj, PropertyName *name, Value *vp, JSBool strict);
+    static JSBool
+    obj_setElement(JSContext *cx, JSObject *obj, uint32_t index, Value *vp, JSBool strict);
     static JSBool
     obj_setSpecial(JSContext *cx, JSObject *obj, SpecialId sid, Value *vp, JSBool strict);
 
     static JSBool
-    obj_getAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
-
+    obj_getGenericAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
     static JSBool
-    obj_getElementAttributes(JSContext *cx, JSObject *obj, uint32 index, uintN *attrsp);
-
+    obj_getPropertyAttributes(JSContext *cx, JSObject *obj, PropertyName *name, uintN *attrsp);
+    static JSBool
+    obj_getElementAttributes(JSContext *cx, JSObject *obj, uint32_t index, uintN *attrsp);
     static JSBool
     obj_getSpecialAttributes(JSContext *cx, JSObject *obj, SpecialId sid, uintN *attrsp);
 
     static JSBool
-    obj_setAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
-
+    obj_setGenericAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
     static JSBool
-    obj_setElementAttributes(JSContext *cx, JSObject *obj, uint32 index, uintN *attrsp);
-
+    obj_setPropertyAttributes(JSContext *cx, JSObject *obj, PropertyName *name, uintN *attrsp);
+    static JSBool
+    obj_setElementAttributes(JSContext *cx, JSObject *obj, uint32_t index, uintN *attrsp);
     static JSBool
     obj_setSpecialAttributes(JSContext *cx, JSObject *obj, SpecialId sid, uintN *attrsp);
 
     static JSBool
-    obj_deleteProperty(JSContext *cx, JSObject *obj, jsid id, Value *rval, JSBool strict);
-
+    obj_deleteProperty(JSContext *cx, JSObject *obj, PropertyName *name, Value *rval, JSBool strict);
     static JSBool
-    obj_deleteElement(JSContext *cx, JSObject *obj, uint32 index, Value *rval, JSBool strict);
-
+    obj_deleteElement(JSContext *cx, JSObject *obj, uint32_t index, Value *rval, JSBool strict);
     static JSBool
     obj_deleteSpecial(JSContext *cx, JSObject *obj, SpecialId sid, Value *rval, JSBool strict);
 
@@ -193,7 +204,8 @@ struct JS_FRIEND_API(TypedArray) {
         FIELD_BYTELENGTH,
         FIELD_TYPE,
         FIELD_BUFFER,
-        FIELD_MAX
+        FIELD_MAX,
+        NUM_FIXED_SLOTS = 7
     };
 
     // and MUST NOT be used to construct new objects.
@@ -212,25 +224,29 @@ struct JS_FRIEND_API(TypedArray) {
     static JSBool prop_getByteLength(JSContext *cx, JSObject *obj, jsid id, Value *vp);
     static JSBool prop_getLength(JSContext *cx, JSObject *obj, jsid id, Value *vp);
 
-    static JSBool obj_lookupProperty(JSContext *cx, JSObject *obj, jsid id,
+    static JSBool obj_lookupGeneric(JSContext *cx, JSObject *obj, jsid id,
+                                    JSObject **objp, JSProperty **propp);
+    static JSBool obj_lookupProperty(JSContext *cx, JSObject *obj, PropertyName *name,
                                      JSObject **objp, JSProperty **propp);
-    static JSBool obj_lookupElement(JSContext *cx, JSObject *obj, uint32 index,
+    static JSBool obj_lookupElement(JSContext *cx, JSObject *obj, uint32_t index,
                                     JSObject **objp, JSProperty **propp);
     static JSBool obj_lookupSpecial(JSContext *cx, JSObject *obj, SpecialId sid,
                                     JSObject **objp, JSProperty **propp);
 
-    static JSBool obj_getAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
-    static JSBool obj_getElementAttributes(JSContext *cx, JSObject *obj, uint32 index, uintN *attrsp);
+    static JSBool obj_getGenericAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
+    static JSBool obj_getPropertyAttributes(JSContext *cx, JSObject *obj, PropertyName *name, uintN *attrsp);
+    static JSBool obj_getElementAttributes(JSContext *cx, JSObject *obj, uint32_t index, uintN *attrsp);
     static JSBool obj_getSpecialAttributes(JSContext *cx, JSObject *obj, SpecialId sid, uintN *attrsp);
 
-    static JSBool obj_setAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
-    static JSBool obj_setElementAttributes(JSContext *cx, JSObject *obj, uint32 index, uintN *attrsp);
+    static JSBool obj_setGenericAttributes(JSContext *cx, JSObject *obj, jsid id, uintN *attrsp);
+    static JSBool obj_setPropertyAttributes(JSContext *cx, JSObject *obj, PropertyName *name, uintN *attrsp);
+    static JSBool obj_setElementAttributes(JSContext *cx, JSObject *obj, uint32_t index, uintN *attrsp);
     static JSBool obj_setSpecialAttributes(JSContext *cx, JSObject *obj, SpecialId sid, uintN *attrsp);
 
-    static JSUint32 getLength(JSObject *obj);
-    static JSUint32 getByteOffset(JSObject *obj);
-    static JSUint32 getByteLength(JSObject *obj);
-    static JSUint32 getType(JSObject *obj);
+    static uint32_t getLength(JSObject *obj);
+    static uint32_t getByteOffset(JSObject *obj);
+    static uint32_t getByteLength(JSObject *obj);
+    static uint32_t getType(JSObject *obj);
     static JSObject * getBuffer(JSObject *obj);
     static void * getDataOffset(JSObject *obj);
 
@@ -238,7 +254,7 @@ struct JS_FRIEND_API(TypedArray) {
     static bool
     isArrayIndex(JSContext *cx, JSObject *obj, jsid id, jsuint *ip = NULL);
 
-    static inline uint32 slotWidth(int atype) {
+    static inline uint32_t slotWidth(int atype) {
         switch (atype) {
           case js::TypedArray::TYPE_INT8:
           case js::TypedArray::TYPE_UINT8:
@@ -284,14 +300,14 @@ JS_FRIEND_API(JSBool)
 js_IsArrayBuffer(JSObject *obj);
 
 JS_FRIEND_API(JSObject *)
-js_CreateArrayBuffer(JSContext *cx, jsuint nbytes);
+js_CreateArrayBuffer(JSContext *cx, uint32_t nbytes);
 
 /*
  * Create a new typed array of type atype (one of the TypedArray
  * enumerant values above), with nelements elements.
  */
 JS_FRIEND_API(JSObject *)
-js_CreateTypedArray(JSContext *cx, jsint atype, jsuint nelements);
+js_CreateTypedArray(JSContext *cx, jsint atype, uint32_t nelements);
 
 /*
  * Create a new typed array of type atype (one of the TypedArray
@@ -312,25 +328,31 @@ JS_FRIEND_API(JSObject *)
 js_CreateTypedArrayWithBuffer(JSContext *cx, jsint atype, JSObject *bufArg,
                               jsint byteoffset, jsint length);
 
-extern int32 JS_FASTCALL
+extern int32_t JS_FASTCALL
 js_TypedArray_uint8_clamp_double(const double x);
 
-JS_FRIEND_API(JSUint32)
+JS_FRIEND_API(JSBool)
+JS_IsArrayBufferObject(JSObject *obj);
+
+JS_FRIEND_API(JSObject *)
+JS_NewArrayBuffer(JSContext *cx, jsuint nbytes);
+
+JS_FRIEND_API(uint32_t)
 JS_GetArrayBufferByteLength(JSObject *obj);
 
-JS_FRIEND_API(uint8 *)
+JS_FRIEND_API(uint8_t *)
 JS_GetArrayBufferData(JSObject *obj);
 
-JS_FRIEND_API(JSUint32)
+JS_FRIEND_API(uint32_t)
 JS_GetTypedArrayLength(JSObject *obj);
 
-JS_FRIEND_API(JSUint32)
+JS_FRIEND_API(uint32_t)
 JS_GetTypedArrayByteOffset(JSObject *obj);
 
-JS_FRIEND_API(JSUint32)
+JS_FRIEND_API(uint32_t)
 JS_GetTypedArrayByteLength(JSObject *obj);
 
-JS_FRIEND_API(JSUint32)
+JS_FRIEND_API(uint32_t)
 JS_GetTypedArrayType(JSObject *obj);
 
 JS_FRIEND_API(JSObject *)
