@@ -44,11 +44,24 @@
 #ifndef jsutil_h___
 #define jsutil_h___
 
+#include "mozilla/Attributes.h"
+
 #include "js/Utility.h"
 
 /* Forward declarations. */
 struct JSContext;
 
+static JS_ALWAYS_INLINE void *
+js_memcpy(void *dst_, const void *src_, size_t len)
+{
+    char *dst = (char *) dst_;
+    const char *src = (const char *) src_;
+    JS_ASSERT_IF(dst >= src, (size_t) (dst - src) >= len);
+    JS_ASSERT_IF(src >= dst, (size_t) (src - dst) >= len);
+
+    return memcpy(dst, src, len);
+}
+
 #ifdef __cplusplus
 namespace js {
 
@@ -187,7 +200,7 @@ class AlreadyIncRefed
     RefCountable *obj;
 
   public:
-    explicit AlreadyIncRefed(RefCountable *obj) : obj(obj) {}
+    explicit AlreadyIncRefed(RefCountable *obj = NULL) : obj(obj) {}
 
     bool null() const { return obj == NULL; }
     operator ConvertibleToBool() const { return (ConvertibleToBool)obj; }
@@ -205,7 +218,7 @@ class NeedsIncRef
     RefCountable *obj;
 
   public:
-    explicit NeedsIncRef(RefCountable *obj) : obj(obj) {}
+    explicit NeedsIncRef(RefCountable *obj = NULL) : obj(obj) {}
 
     bool null() const { return obj == NULL; }
     operator ConvertibleToBool() const { return (ConvertibleToBool)obj; }
@@ -223,8 +236,8 @@ class AutoRefCount
     JSContext *const cx;
     RefCountable *obj;
 
-    AutoRefCount(const AutoRefCount &);
-    void operator=(const AutoRefCount &);
+    AutoRefCount(const AutoRefCount &other) MOZ_DELETE;
+    void operator=(const AutoRefCount &other) MOZ_DELETE;
 
   public:
     explicit AutoRefCount(JSContext *cx)
@@ -280,7 +293,14 @@ template <class T>
 JS_ALWAYS_INLINE static void
 PodZero(T *t, size_t nelem)
 {
-    memset(t, 0, nelem * sizeof(T));
+    /*
+     * This function is often called with 'nelem' small; we use an
+     * inline loop instead of calling 'memset' with a non-constant
+     * length.  The compiler should inline the memset call with constant
+     * size, though.
+     */
+    for (T *end = t + nelem; t != end; ++t)
+        memset(t, 0, sizeof(T));
 }
 
 /*
@@ -300,6 +320,13 @@ PodArrayZero(T (&t)[N])
     memset(t, 0, N * sizeof(T));
 }
 
+template <class T>
+JS_ALWAYS_INLINE static void
+PodAssign(T *dst, const T *src)
+{
+    js_memcpy((char *) dst, (const char *) src, sizeof(T));
+}
+
 template <class T>
 JS_ALWAYS_INLINE static void
 PodCopy(T *dst, const T *src, size_t nelem)
@@ -309,8 +336,12 @@ PodCopy(T *dst, const T *src, size_t nelem)
     JS_ASSERT_IF(src >= dst, size_t(src - dst) >= nelem);
 
     if (nelem < 128) {
+        /*
+         * Avoid using operator= in this loop, as it may have been
+         * intentionally deleted by the POD type.
+         */
         for (const T *srcend = src + nelem; src != srcend; ++src, ++dst)
-            *dst = *src;
+            PodAssign(dst, src);
     } else {
         memcpy(dst, src, nelem * sizeof(T));
     }
@@ -339,7 +370,7 @@ UnsignedPtrDiff(const void *bigger, const void *smaller)
 }
 
 /*
- * Ordinarily, a function taking a JSContext* 'cx' paremter reports errors on
+ * Ordinarily, a function taking a JSContext* 'cx' parameter reports errors on
  * the context. In some cases, functions optionally report and indicate this by
  * taking a nullable 'maybecx' parameter. In some cases, though, a function
  * always needs a 'cx', but optionally reports. This option is presented by the
@@ -378,18 +409,9 @@ inline __attribute__ ((unused)) void MUST_FLOW_THROUGH(const char *label) {}
 /* Avoid unused goto-label warnings. */
 # define MUST_FLOW_LABEL(label) goto label; label:
 
-inline JS_FORCES_STACK void VOUCH_DOES_NOT_REQUIRE_STACK() {}
-
-inline JS_FORCES_STACK void
-JS_ASSERT_NOT_ON_TRACE(JSContext *cx)
-{
-    JS_ASSERT(!JS_ON_TRACE(cx));
-}
 #else
 # define MUST_FLOW_THROUGH(label)            ((void) 0)
 # define MUST_FLOW_LABEL(label)
-# define VOUCH_DOES_NOT_REQUIRE_STACK()      ((void) 0)
-# define JS_ASSERT_NOT_ON_TRACE(cx)          JS_ASSERT(!JS_ON_TRACE(cx))
 #endif
 
 /* Crash diagnostics */
@@ -415,12 +437,12 @@ JS_ASSERT_NOT_ON_TRACE(JSContext *cx)
 #ifdef JS_BASIC_STATS
 # include <stdio.h>
 typedef struct JSBasicStats {
-    uint32      num;
-    uint32      max;
+    uint32_t    num;
+    uint32_t    max;
     double      sum;
     double      sqsum;
-    uint32      logscale;           /* logarithmic scale: 0 (linear), 2, 10 */
-    uint32      hist[11];
+    uint32_t    logscale;           /* logarithmic scale: 0 (linear), 2, 10 */
+    uint32_t    hist[11];
 } JSBasicStats;
 # define JS_INIT_STATIC_BASIC_STATS  {0,0,0,0,0,{0,0,0,0,0,0,0,0,0,0,0}}
 # define JS_BASIC_STATS_INIT(bs)     memset((bs), 0, sizeof(JSBasicStats))
@@ -429,9 +451,9 @@ typedef struct JSBasicStats {
 # define JS_MeanAndStdDevBS(bs,sigma)                                         \
     JS_MeanAndStdDev((bs)->num, (bs)->sum, (bs)->sqsum, sigma)
 extern void
-JS_BasicStatsAccum(JSBasicStats *bs, uint32 val);
+JS_BasicStatsAccum(JSBasicStats *bs, uint32_t val);
 extern double
-JS_MeanAndStdDev(uint32 num, double sum, double sqsum, double *sigma);
+JS_MeanAndStdDev(uint32_t num, double sum, double sqsum, double *sigma);
 extern void
 JS_DumpBasicStats(JSBasicStats *bs, const char *title, FILE *fp);
 extern void
@@ -449,6 +471,28 @@ typedef size_t jsbitmap;
 #define JS_CLEAR_BIT(_map,_bit) ((_map)[(_bit)>>JS_BITS_PER_WORD_LOG2] &=     \
                                  ~((jsbitmap)1<<((_bit)&(JS_BITS_PER_WORD-1))))
 
-#define VOUCH_HAVE_STACK                    VOUCH_DOES_NOT_REQUIRE_STACK
+/* Wrapper for various macros to stop warnings coming from their expansions. */
+#if defined(__clang__)
+# define JS_SILENCE_UNUSED_VALUE_IN_EXPR(expr)                                \
+    JS_BEGIN_MACRO                                                            \
+        _Pragma("clang diagnostic push")                                      \
+        _Pragma("clang diagnostic ignored \"-Wunused-value\"")                \
+        expr;                                                                 \
+        _Pragma("clang diagnostic pop")                                       \
+    JS_END_MACRO
+#elif (__GNUC__ >= 5) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
+# define JS_SILENCE_UNUSED_VALUE_IN_EXPR(expr)                                \
+    JS_BEGIN_MACRO                                                            \
+        _Pragma("GCC diagnostic push")                                        \
+        _Pragma("GCC diagnostic ignored \"-Wunused-but-set-variable\"")       \
+        expr;                                                                 \
+        _Pragma("GCC diagnostic pop")                                         \
+    JS_END_MACRO
+#else
+# define JS_SILENCE_UNUSED_VALUE_IN_EXPR(expr)                                \
+    JS_BEGIN_MACRO                                                            \
+        expr;                                                                 \
+    JS_END_MACRO
+#endif
 
 #endif /* jsutil_h___ */
