@@ -86,7 +86,7 @@ FrameState::pruneDeadEntries()
 }
 
 bool
-FrameState::pushActiveFrame(JSScript *script, uint32 argc)
+FrameState::pushActiveFrame(JSScript *script, uint32_t argc)
 {
     if (!a) {
         this->nentries = analyze::TotalSlots(script) + (script->nslots - script->nfixed) +
@@ -94,7 +94,7 @@ FrameState::pushActiveFrame(JSScript *script, uint32 argc)
         size_t totalBytes = sizeof(FrameEntry) * nentries +       // entries[]
                             sizeof(FrameEntry *) * nentries +     // tracker.entries
                             sizeof(StackEntryExtra) * nentries;   // extraArray
-        uint8 *cursor = (uint8 *)cx->calloc_(totalBytes);
+        uint8_t *cursor = (uint8_t *)cx->calloc_(totalBytes);
         if (!cursor)
             return false;
 
@@ -107,7 +107,7 @@ FrameState::pushActiveFrame(JSScript *script, uint32 argc)
         this->extraArray = (StackEntryExtra *)cursor;
         cursor += sizeof(StackEntryExtra) * nentries;
 
-        JS_ASSERT(reinterpret_cast<uint8 *>(this->entries) + totalBytes == cursor);
+        JS_ASSERT(reinterpret_cast<uint8_t *>(this->entries) + totalBytes == cursor);
 
 #if defined JS_NUNBOX32
         if (!reifier.init(cx, *this, nentries))
@@ -261,12 +261,12 @@ FrameState::variableLive(FrameEntry *fe, jsbytecode *pc) const
     JS_ASSERT(cx->typeInferenceEnabled());
     JS_ASSERT(fe > a->callee_ && fe < a->spBase);
 
-    uint32 offset = pc - a->script->code;
+    uint32_t offset = pc - a->script->code;
     return a->analysis->liveness(entrySlot(fe)).live(offset);
 }
 
 AnyRegisterID
-FrameState::bestEvictReg(uint32 mask, bool includePinned) const
+FrameState::bestEvictReg(uint32_t mask, bool includePinned) const
 {
     JS_ASSERT(cx->typeInferenceEnabled());
 
@@ -274,11 +274,11 @@ FrameState::bestEvictReg(uint32 mask, bool includePinned) const
     JS_ASSERT((mask & Registers::AvailRegs) != (mask & Registers::AvailFPRegs));
 
     AnyRegisterID fallback;
-    uint32 fallbackOffset = uint32(-1);
+    uint32_t fallbackOffset = UINT32_MAX;
 
     JaegerSpew(JSpew_Regalloc, "picking best register to evict:\n");
 
-    for (uint32 i = 0; i < Registers::TotalAnyRegisters; i++) {
+    for (uint32_t i = 0; i < Registers::TotalAnyRegisters; i++) {
         AnyRegisterID reg = AnyRegisterID::fromRaw(i);
 
         /* Register is not allocatable, don't bother.  */
@@ -328,7 +328,7 @@ FrameState::bestEvictReg(uint32 mask, bool includePinned) const
              * Additionally, if this is an inlined frame then any entries
              * belonging to parents are treated as live everywhere in the call.
              */
-            uint32 offset = a->parent ? a->script->length : loop->backedgeOffset();
+            uint32_t offset = a->parent ? a->script->length : loop->backedgeOffset();
             if (!fallback.isSet() || offset > fallbackOffset) {
                 fallback = reg;
                 fallbackOffset = offset;
@@ -355,8 +355,7 @@ FrameState::bestEvictReg(uint32 mask, bool includePinned) const
          * Evict variables which are only live in future loop iterations, and are
          * not carried around the loop in a register.
          */
-        JS_ASSERT_IF(lifetime->loopTail, loop);
-        if (lifetime->loopTail && !loop->carriesLoopReg(fe)) {
+        if (lifetime->loopTail && (!loop || !loop->carriesLoopReg(fe))) {
             JaegerSpew(JSpew_Regalloc, "result: %s (%s) only live in later iterations\n",
                        entryName(fe), reg.name());
             return reg;
@@ -388,7 +387,7 @@ FrameState::bestEvictReg(uint32 mask, bool includePinned) const
 void
 FrameState::evictDeadEntries(bool includePinned)
 {
-    for (uint32 i = 0; i < Registers::TotalAnyRegisters; i++) {
+    for (uint32_t i = 0; i < Registers::TotalAnyRegisters; i++) {
         AnyRegisterID reg = AnyRegisterID::fromRaw(i);
 
         /* Follow along with the same filters as bestEvictReg. */
@@ -430,7 +429,7 @@ FrameState::evictDeadEntries(bool includePinned)
 }
 
 AnyRegisterID
-FrameState::evictSomeReg(uint32 mask)
+FrameState::evictSomeReg(uint32_t mask)
 {
     JS_ASSERT(!freeRegs.hasRegInMask(mask));
 
@@ -454,7 +453,7 @@ FrameState::evictSomeReg(uint32 mask)
 
     MaybeRegisterID fallback;
 
-    for (uint32 i = 0; i < JSC::MacroAssembler::TotalRegisters; i++) {
+    for (uint32_t i = 0; i < JSC::MacroAssembler::TotalRegisters; i++) {
         RegisterID reg = RegisterID(i);
 
         /* Register is not allocatable, don't bother.  */
@@ -488,7 +487,7 @@ FrameState::evictSomeReg(uint32 mask)
 void
 FrameState::resetInternalState()
 {
-    for (uint32 i = 0; i < tracker.nentries; i++)
+    for (uint32_t i = 0; i < tracker.nentries; i++)
         tracker[i]->untrack();
 
     tracker.reset();
@@ -548,7 +547,7 @@ FrameState::forgetEverything()
     resetInternalState();
 
 #ifdef DEBUG
-    for (uint32 i = 0; i < Registers::TotalAnyRegisters; i++) {
+    for (uint32_t i = 0; i < Registers::TotalAnyRegisters; i++) {
         AnyRegisterID reg = AnyRegisterID::fromRaw(i);
         JS_ASSERT(!regstate(reg).usedBy());
     }
@@ -575,13 +574,16 @@ RegisterAllocation *
 FrameState::computeAllocation(jsbytecode *target)
 {
     JS_ASSERT(cx->typeInferenceEnabled());
-    RegisterAllocation *alloc = ArenaNew<RegisterAllocation>(cx->compartment->pool, false);
+    RegisterAllocation *alloc = cx->typeLifoAlloc().new_<RegisterAllocation>(false);
     if (!alloc)
         return NULL;
 
-    if (a->analysis->getCode(target).exceptionEntry || a->analysis->getCode(target).switchTarget ||
-        JSOp(*target) == JSOP_TRAP) {
-        /* State must be synced at exception and switch targets, and at traps. */
+    /*
+     * State must be synced at exception and switch targets, at traps and when
+     * crossing between compilation chunks.
+     */
+    if (a->analysis->getCode(target).safePoint ||
+        (!a->parent && !cc.bytecodeInChunk(target))) {
 #ifdef DEBUG
         if (IsJaegerSpewChannelActive(JSpew_Regalloc)) {
             JaegerSpew(JSpew_Regalloc, "allocation at %u:", unsigned(target - a->script->code));
@@ -604,7 +606,7 @@ FrameState::computeAllocation(jsbytecode *target)
         if (fe < a->callee_ ||
             isConstructorThis(fe) ||
             (fe > a->callee_ && fe < a->spBase && variableLive(fe, target)) ||
-            (isTemporary(fe) && (a->parent || uint32(target - a->script->code) <= loop->backedgeOffset()))) {
+            (isTemporary(fe) && (a->parent || uint32_t(target - a->script->code) <= loop->backedgeOffset()))) {
             /*
              * For entries currently in floating point registers, check they
              * are known to be doubles at the target. We don't need to do this
@@ -618,7 +620,7 @@ FrameState::computeAllocation(jsbytecode *target)
                 const SlotValue *newv = a->analysis->newValues(target);
                 while (newv && newv->slot) {
                     if (newv->value.kind() == SSAValue::PHI &&
-                        newv->value.phiOffset() == uint32(target - a->script->code) &&
+                        newv->value.phiOffset() == uint32_t(target - a->script->code) &&
                         newv->slot == entrySlot(fe)) {
                         types::TypeSet *types = a->analysis->getValueTypes(newv->value);
                         if (types->getKnownTypeTag(cx) != JSVAL_TYPE_DOUBLE)
@@ -725,7 +727,7 @@ FrameState::syncForAllocation(RegisterAllocation *alloc, bool inlineReturn, Uses
     if (inlineReturn)
         topEntry = a->parent->sp - (GET_ARGC(a->parent->PC) + 2);
 
-    for (uint32 i = tracker.nentries - 1; i < tracker.nentries; i--) {
+    for (uint32_t i = tracker.nentries - 1; i < tracker.nentries; i--) {
         FrameEntry *fe = tracker[i];
 
         if (deadEntry(fe, uses.nuses))
@@ -839,7 +841,7 @@ FrameState::syncForAllocation(RegisterAllocation *alloc, bool inlineReturn, Uses
 }
 
 bool
-FrameState::discardForJoin(RegisterAllocation *&alloc, uint32 stackDepth)
+FrameState::discardForJoin(RegisterAllocation *&alloc, uint32_t stackDepth)
 {
     if (!cx->typeInferenceEnabled()) {
         resetInternalState();
@@ -853,7 +855,7 @@ FrameState::discardForJoin(RegisterAllocation *&alloc, uint32 stackDepth)
          * This shows up for loop entries which are not reachable from the
          * loop head, and for exception, switch target and trap safe points.
          */
-        alloc = ArenaNew<RegisterAllocation>(cx->compartment->pool, false);
+        alloc = cx->typeLifoAlloc().new_<RegisterAllocation>(false);
         if (!alloc)
             return false;
     }
@@ -882,8 +884,11 @@ FrameState::discardForJoin(RegisterAllocation *&alloc, uint32 stackDepth)
         }
 
         regstate(reg).associate(fe, RematInfo::DATA);
-        if (!alloc->synced(reg))
+        if (!alloc->synced(reg)) {
             fe->data.unsync();
+            if (!reg.isReg())
+                fe->type.unsync();
+        }
     }
 
     a->sp = a->spBase + stackDepth;
@@ -1205,9 +1210,9 @@ FrameState::assertValidRegisterState() const
     Registers checkedFreeRegs(Registers::AvailAnyRegs);
 
     /* Check that copied and copy info balance out. */
-    int32 copyCount = 0;
+    int32_t copyCount = 0;
 
-    for (uint32 i = 0; i < tracker.nentries; i++) {
+    for (uint32_t i = 0; i < tracker.nentries; i++) {
         FrameEntry *fe = tracker[i];
         if (deadEntry(fe))
             continue;
@@ -1246,14 +1251,14 @@ FrameState::assertValidRegisterState() const
     JS_ASSERT(copyCount == 0);
     JS_ASSERT(checkedFreeRegs == freeRegs);
 
-    for (uint32 i = 0; i < Registers::TotalRegisters; i++) {
+    for (uint32_t i = 0; i < Registers::TotalRegisters; i++) {
         AnyRegisterID reg = (RegisterID) i;
         JS_ASSERT(!regstate(reg).isPinned());
         JS_ASSERT_IF(regstate(reg).fe(), !freeRegs.hasReg(reg));
         JS_ASSERT_IF(regstate(reg).fe(), regstate(reg).fe()->isTracked());
     }
 
-    for (uint32 i = 0; i < Registers::TotalFPRegisters; i++) {
+    for (uint32_t i = 0; i < Registers::TotalFPRegisters; i++) {
         AnyRegisterID reg = (FPRegisterID) i;
         JS_ASSERT(!regstate(reg).isPinned());
         JS_ASSERT_IF(regstate(reg).fe(), !freeRegs.hasReg(reg));
@@ -1265,13 +1270,13 @@ FrameState::assertValidRegisterState() const
 
 #if defined JS_NUNBOX32
 void
-FrameState::syncFancy(Assembler &masm, Registers avail, FrameEntry *resumeAt,
-                      FrameEntry *bottom) const
+FrameState::syncFancy(Assembler &masm, Registers avail, int trackerIndex) const
 {
-    reifier.reset(&masm, avail, resumeAt, bottom);
+    reifier.reset(&masm, avail, a->sp, entries);
 
-    for (FrameEntry *fe = resumeAt; fe >= bottom; fe--) {
-        if (!fe->isTracked())
+    for (; trackerIndex >= 0; trackerIndex--) {
+        FrameEntry *fe = tracker[trackerIndex];
+        if (fe >= a->sp)
             continue;
 
         reifier.sync(fe);
@@ -1325,12 +1330,11 @@ FrameState::sync(Assembler &masm, Uses uses) const
     Registers avail(freeRegs.freeMask & Registers::AvailRegs);
     Registers temp(Registers::TempAnyRegs);
 
-    FrameEntry *bottom = (cx->typeInferenceEnabled() || cx->compartment->debugMode())
-        ? entries
-        : a->sp - uses.nuses;
-
-    for (FrameEntry *fe = a->sp - 1; fe >= bottom; fe--) {
-        if (!fe->isTracked())
+    unsigned nentries = tracker.nentries;
+    for (int trackerIndex = nentries - 1; trackerIndex >= 0; trackerIndex--) {
+        JS_ASSERT(tracker.nentries == nentries);
+        FrameEntry *fe = tracker[trackerIndex];
+        if (fe >= a->sp)
             continue;
 
         if (fe->isType(JSVAL_TYPE_DOUBLE)) {
@@ -1340,15 +1344,13 @@ FrameState::sync(Assembler &masm, Uses uses) const
             continue;
         }
 
-        FrameEntry *backing = fe;
-
         if (!fe->isCopy()) {
-            if (fe->data.inRegister())
+            if (fe->data.inRegister() && !regstate(fe->data.reg()).isPinned())
                 avail.putReg(fe->data.reg());
-            if (fe->type.inRegister())
+            if (fe->type.inRegister() && !regstate(fe->type.reg()).isPinned())
                 avail.putReg(fe->type.reg());
         } else {
-            backing = fe->copyOf();
+            FrameEntry *backing = fe->copyOf();
             JS_ASSERT(!backing->isConstant() && !fe->isConstant());
 
 #if defined JS_PUNBOX64
@@ -1379,7 +1381,7 @@ FrameState::sync(Assembler &masm, Uses uses) const
             /* Fall back to a slower sync algorithm if load required. */
             if ((!fe->type.synced() && backing->type.inMemory()) ||
                 (!fe->data.synced() && backing->data.inMemory())) {
-                syncFancy(masm, avail, fe, bottom);
+                syncFancy(masm, avail, trackerIndex);
                 return;
             }
 #endif
@@ -1458,19 +1460,13 @@ FrameState::syncAndKill(Registers kill, Uses uses, Uses ignore)
 #endif
     }
 
-    uint32 maxvisits = tracker.nentries;
 
-    FrameEntry *bottom = (cx->typeInferenceEnabled() || cx->compartment->debugMode())
-        ? entries
-        : a->sp - uses.nuses;
+    unsigned nentries = tracker.nentries;
+    for (int trackerIndex = nentries - 1; trackerIndex >= 0; trackerIndex--) {
+        JS_ASSERT(tracker.nentries == nentries);
+        FrameEntry *fe = tracker[trackerIndex];
 
-    for (FrameEntry *fe = a->sp - 1; fe >= bottom && maxvisits; fe--) {
-        if (!fe->isTracked())
-            continue;
-
-        maxvisits--;
-
-        if (deadEntry(fe, ignore.nuses))
+        if (fe >= a->sp || deadEntry(fe, ignore.nuses))
             continue;
 
         syncFe(fe);
@@ -1504,10 +1500,11 @@ FrameState::syncAndKill(Registers kill, Uses uses, Uses ignore)
         if (!fe || deadEntry(fe, ignore.nuses))
             continue;
 
-        JS_ASSERT(fe->isTracked() && !fe->isType(JSVAL_TYPE_DOUBLE));
+        JS_ASSERT(fe->isTracked());
 
         if (regstate(reg).type() == RematInfo::DATA) {
-            JS_ASSERT(fe->data.reg() == reg.reg());
+            JS_ASSERT_IF(reg.isFPReg(), fe->data.fpreg() == reg.fpreg());
+            JS_ASSERT_IF(!reg.isFPReg(), fe->data.reg() == reg.reg());
             JS_ASSERT(fe->data.synced());
             fe->data.setMemory();
         } else {
@@ -1535,13 +1532,15 @@ FrameState::merge(Assembler &masm, Changes changes) const
      * do not require stub paths to always generate a double when needed.
      * :FIXME: we check this on OOL stub calls, but not inline stub calls.
      */
-    for (unsigned i = 0; i < changes.nchanges; i++) {
-        FrameEntry *fe = a->sp - 1 - i;
-        if (fe->isTracked() && fe->isType(JSVAL_TYPE_DOUBLE))
-            masm.ensureInMemoryDouble(addressOf(fe));
+    if (cx->typeInferenceEnabled()) {
+        for (unsigned i = 0; i < changes.nchanges; i++) {
+            FrameEntry *fe = a->sp - 1 - i;
+            if (fe->isTracked() && fe->isType(JSVAL_TYPE_DOUBLE))
+                masm.ensureInMemoryDouble(addressOf(fe));
+        }
     }
 
-    uint32 mask = Registers::AvailAnyRegs & ~freeRegs.freeMask;
+    uint32_t mask = Registers::AvailAnyRegs & ~freeRegs.freeMask;
     Registers search(mask);
 
     while (!search.empty(mask)) {
@@ -1839,7 +1838,7 @@ FrameState::ensureDouble(FrameEntry *fe)
         fe->clear();
     } else if (fe->isCopied()) {
         /* Sync and forget any copies of this entry. */
-        for (uint32 i = fe->trackerIndex() + 1; i < tracker.nentries; i++) {
+        for (uint32_t i = fe->trackerIndex() + 1; i < tracker.nentries; i++) {
             FrameEntry *nfe = tracker[i];
             if (!deadEntry(nfe) && nfe->isCopy() && nfe->copyOf() == fe) {
                 syncFe(nfe);
@@ -1878,7 +1877,7 @@ FrameState::ensureInteger(FrameEntry *fe)
      */
 
     if (fe->isConstant()) {
-        Value newValue = Int32Value(int32(fe->getValue().toDouble()));
+        Value newValue = Int32Value(int32_t(fe->getValue().toDouble()));
         fe->setConstant(newValue);
         return;
     }
@@ -1918,7 +1917,7 @@ void
 FrameState::ensureInMemoryDoubles(Assembler &masm)
 {
     JS_ASSERT(!a->parent);
-    for (uint32 i = 0; i < tracker.nentries; i++) {
+    for (uint32_t i = 0; i < tracker.nentries; i++) {
         FrameEntry *fe = tracker[i];
         if (!deadEntry(fe) && fe->isType(JSVAL_TYPE_DOUBLE) &&
             !fe->isCopy() && !fe->isConstant()) {
@@ -1950,10 +1949,10 @@ FrameState::pushCopyOf(FrameEntry *backing)
 FrameEntry *
 FrameState::walkTrackerForUncopy(FrameEntry *original)
 {
-    uint32 firstCopy = InvalidIndex;
+    uint32_t firstCopy = InvalidIndex;
     FrameEntry *bestFe = NULL;
-    uint32 ncopies = 0;
-    for (uint32 i = original->trackerIndex() + 1; i < tracker.nentries; i++) {
+    uint32_t ncopies = 0;
+    for (uint32_t i = original->trackerIndex() + 1; i < tracker.nentries; i++) {
         FrameEntry *fe = tracker[i];
         if (deadEntry(fe))
             continue;
@@ -1981,7 +1980,7 @@ FrameState::walkTrackerForUncopy(FrameEntry *original)
     /* Mark all extra copies as copies of the new backing index. */
     bestFe->setCopyOf(NULL);
     if (ncopies > 1) {
-        for (uint32 i = firstCopy; i < tracker.nentries; i++) {
+        for (uint32_t i = firstCopy; i < tracker.nentries; i++) {
             FrameEntry *other = tracker[i];
             if (deadEntry(other) || other == bestFe)
                 continue;
@@ -2013,10 +2012,10 @@ FrameEntry *
 FrameState::walkFrameForUncopy(FrameEntry *original)
 {
     FrameEntry *bestFe = NULL;
-    uint32 ncopies = 0;
+    uint32_t ncopies = 0;
 
     /* It's only necessary to visit as many FEs are being tracked. */
-    uint32 maxvisits = tracker.nentries;
+    uint32_t maxvisits = tracker.nentries;
 
     for (FrameEntry *fe = original + 1; fe < a->sp && maxvisits; fe++) {
         if (!fe->isTracked())
@@ -2069,7 +2068,7 @@ FrameState::uncopy(FrameEntry *original)
      * the tracker is walked twice, so we multiply by 2 for pessimism.
      */
     FrameEntry *fe;
-    if ((tracker.nentries - original->trackerIndex()) * 2 > uint32(a->sp - original))
+    if ((tracker.nentries - original->trackerIndex()) * 2 > uint32_t(a->sp - original))
         fe = walkFrameForUncopy(original);
     else
         fe = walkTrackerForUncopy(original);
@@ -2120,7 +2119,7 @@ FrameState::hasOnlyCopy(FrameEntry *backing, FrameEntry *fe)
 {
     JS_ASSERT(backing->isCopied() && fe->copyOf() == backing);
 
-    for (uint32 i = backing->trackerIndex() + 1; i < tracker.nentries; i++) {
+    for (uint32_t i = backing->trackerIndex() + 1; i < tracker.nentries; i++) {
         FrameEntry *nfe = tracker[i];
         if (nfe != fe && !deadEntry(nfe) && nfe->isCopy() && nfe->copyOf() == backing)
             return false;
@@ -2141,7 +2140,7 @@ FrameState::separateBinaryEntries(FrameEntry *lhs, FrameEntry *rhs)
 }
 
 void
-FrameState::storeLocal(uint32 n, bool popGuaranteed)
+FrameState::storeLocal(uint32_t n, bool popGuaranteed)
 {
     FrameEntry *local = getLocal(n);
 
@@ -2161,7 +2160,7 @@ FrameState::storeLocal(uint32 n, bool popGuaranteed)
 }
 
 void
-FrameState::storeArg(uint32 n, bool popGuaranteed)
+FrameState::storeArg(uint32_t n, bool popGuaranteed)
 {
     // Note that args are always immediately synced, because they can be
     // aliased (but not written to) via f.arguments.
@@ -2274,7 +2273,7 @@ FrameState::storeTop(FrameEntry *target)
          * but even so there's a quick workaround. We take all copies of the
          * backing fe, and redirect them to be copies of the destination.
          */
-        for (uint32 i = backing->trackerIndex() + 1; i < tracker.nentries; i++) {
+        for (uint32_t i = backing->trackerIndex() + 1; i < tracker.nentries; i++) {
             FrameEntry *fe = tracker[i];
             if (deadEntry(fe))
                 continue;
@@ -2330,16 +2329,16 @@ FrameState::storeTop(FrameEntry *target)
 }
 
 void
-FrameState::shimmy(uint32 n)
+FrameState::shimmy(uint32_t n)
 {
     JS_ASSERT(a->sp - n >= a->spBase);
-    int32 depth = 0 - int32(n);
+    int32_t depth = 0 - int32_t(n);
     storeTop(peek(depth - 1));
     popn(n);
 }
 
 void
-FrameState::shift(int32 n)
+FrameState::shift(int32_t n)
 {
     JS_ASSERT(n < 0);
     JS_ASSERT(a->sp + n - 1 >= a->spBase);
@@ -2726,7 +2725,7 @@ FrameState::allocForBinary(FrameEntry *lhs, FrameEntry *rhs, JSOp op, BinaryAllo
         bool rightInReg = backingRight->data.inRegister();
 
         /* If the LHS/RHS types are in registers, don't use them for the result. */
-        uint32 mask = Registers::AvailRegs;
+        uint32_t mask = Registers::AvailRegs;
         if (backingLeft->type.inRegister())
             mask &= ~Registers::maskReg(backingLeft->type.reg());
         if (backingRight->type.inRegister())
@@ -2844,11 +2843,11 @@ FrameState::maybeUnpinReg(MaybeRegisterID reg)
         unpinReg(reg.reg());
 }
 
-uint32
+uint32_t
 FrameState::allocTemporary()
 {
     if (temporariesTop == temporaries + TEMPORARY_LIMIT)
-        return uint32(-1);
+        return UINT32_MAX;
     FrameEntry *fe = temporariesTop++;
     fe->lastLoop = 0;
     fe->temporary = true;
@@ -2882,7 +2881,7 @@ FrameState::getTemporaryCopies(Uses uses)
         if (!fe->isTracked())
             continue;
         if (fe->isCopied()) {
-            for (uint32 i = fe->trackerIndex() + 1; i < tracker.nentries; i++) {
+            for (uint32_t i = fe->trackerIndex() + 1; i < tracker.nentries; i++) {
                 FrameEntry *nfe = tracker[i];
                 if (!deadEntry(nfe, uses.nuses) && nfe->isCopy() && nfe->copyOf() == fe) {
                     if (!res)
