@@ -51,11 +51,10 @@ enum CompileRequest
 };
 
 /*
- * Number of times a script must be called or have back edges taken before we
- * run it in the methodjit. We wait longer if type inference is enabled, to
- * allow more gathering of type information and less recompilation.
+ * We wait before compiling a function with Type Inference to allow more gathering 
+ * of type information and less recompilation.
  */
-static const size_t USES_BEFORE_COMPILE       = 16;
+static const size_t USES_BEFORE_COMPILE = 16;
 static const size_t INFER_USES_BEFORE_COMPILE = 40;
 
 static inline CompileStatus
@@ -66,8 +65,7 @@ CanMethodJIT(JSContext *cx, JSScript *script, bool construct, CompileRequest req
     JITScriptStatus status = script->getJITStatus(construct);
     if (status == JITScript_Invalid)
         return Compile_Abort;
-    if (request == CompileRequest_Interpreter &&
-        status == JITScript_None &&
+    if (request == CompileRequest_Interpreter && status == JITScript_None &&
         !cx->hasRunOption(JSOPTION_METHODJIT_ALWAYS) &&
         (cx->typeInferenceEnabled()
          ? script->incUseCount() <= INFER_USES_BEFORE_COMPILE
@@ -93,20 +91,10 @@ CanMethodJITAtBranch(JSContext *cx, JSScript *script, StackFrame *fp, jsbytecode
     if (status == JITScript_Invalid)
         return Compile_Abort;
     if (status == JITScript_None && !cx->hasRunOption(JSOPTION_METHODJIT_ALWAYS)) {
-        /*
-         * Backedges are counted differently with type inference vs. with the
-         * tracer. For inference, we use the script's use count, so that we can
-         * easily reset the script's uses if we end up recompiling it. For the
-         * tracer, we use the compartment's backedge table so that when
-         * compiling trace ICs we will retain counts for each loop and respect
-         * the HOTLOOP value when deciding to start recording traces.
-         */
-        if (cx->typeInferenceEnabled()) {
-            if (script->incUseCount() <= INFER_USES_BEFORE_COMPILE)
-                return Compile_Skipped;
-        } else {
-            if (cx->compartment->incBackEdgeCount(pc) <= USES_BEFORE_COMPILE)
-                return Compile_Skipped;
+        if ((cx->typeInferenceEnabled())
+             ? script->incUseCount() <= INFER_USES_BEFORE_COMPILE
+             : script->incUseCount() <= USES_BEFORE_COMPILE) {
+            return Compile_Skipped;
         }
     }
     if (status == JITScript_None)
