@@ -38,7 +38,7 @@
 
 
 /**
-   File Name:          7.4.2-3-n.js
+   File Name:          7.4.2-6.js
    ECMA Section:       7.4.2
 
    Description:
@@ -61,16 +61,16 @@
    Date:               12 november 1997
 
 */
-var SECTION = "7.4.2-3-n";
+var SECTION = "7.4.2-6-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Keywords";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var new = true";
+DESCRIPTION = "var function = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var new = true",     "error",    eval("var new = true") );
+new TestCase( SECTION,  "var function = true",     "error",    eval("var function = true") );
 
 test();
