@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Peter Seliger
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,40 +36,25 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 480096;
-var summary = 'Array.lastIndexOf';
+var BUGNUMBER = 312138;
+var summary = 'Array.sort should not eat exceptions';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+expect = "e=1 N=1";
 
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
+var N = 0;
+var array = [4,3,2,1];
 
-  expect = '-12';
-  actual = 0;
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6]);
-  actual += [2, 3,, 4, 5, 6].lastIndexOf();
-  actual += Array.prototype.lastIndexOf.call([2, 3,, 4, 5, 6]);
-  actual += Array.prototype.lastIndexOf.apply([2, 3,, 4, 5, 6], [, -4]);
-  actual += Array.prototype.lastIndexOf.apply([2, 3,, 4, 5, 6], [undefined, -4]);
-  actual += Array.prototype.lastIndexOf.apply([2, 3,, 4, 5, 6], [undefined, -5]);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined, 1);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined, 2);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined, 1);
-  actual += Array.lastIndexOf([2, 3,, 4, 5, 6], undefined, 2);
-
-  actual = String(actual);
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+try {
+  array.sort(function() {
+	       throw ++N;
+	     });
+} catch (e) {
+  actual = ("e="+e+" N="+N);
 }
+
+reportCompare(expect, actual, summary);
