@@ -36,11 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 456477;
-var summary = 'Do not assert with JIT: (m != JSVAL_INT) || isInt32(*vp)" with (0/0)%(-1)';
-var actual = 'No Crash';
-var expect = 'No Crash';
-
+var BUGNUMBER = 470187;
+var summary = 'Do not assert: ATOM_IS_STRING(atom)';
+var actual = '';
+var expect = '';
 
 //-----------------------------------------------------------------------------
 test();
@@ -49,14 +48,11 @@ test();
 function test()
 {
   enterFunc ('test');
+
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  jit(true);
-
-  for (var j = 0; j < 5; ++j) { var t = (0 / 0) % (-1); }
-
-  jit(false);
+  for (var j=0;j<3;++j) ({valueOf: function(){return 2}}) - [];
 
   reportCompare(expect, actual, summary);
 
