@@ -41,6 +41,7 @@ var summary = 'Do not assert: OBJ_SCOPE(pobj)->object == pobj';
 var actual = '';
 var expect = '';
 
+
 //-----------------------------------------------------------------------------
 test();
 //-----------------------------------------------------------------------------
@@ -51,11 +52,16 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  var c = {__proto__: []};
-  var a = {__proto__: {__proto__: {}}};
-  c.hasOwnProperty;
-  c.__proto__ = a;
-  c.hasOwnProperty;
+  function boom()
+  {
+    var c = {__proto__: []};
+    var a = {__proto__: {__proto__: {}}};
+    c.hasOwnProperty("t");
+    c.__proto__ = a;
+    c.hasOwnProperty("v");
+  }
+
+  boom();
 
   reportCompare(expect, actual, summary);
 
