@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 355049;
-var summary = 'decompilation of destructing into two hole';
+var BUGNUMBER = 381963;
+var summary = 'Decompilation of genexp in |while|';
 var actual = '';
 var expect = '';
 
@@ -51,10 +51,11 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var f = function () { [3 for ([, , ] in null)] }
-  expect = 'function () { [3 for ([, , ] in null)]; }';
+
+  expect = 'function() { while(1, (2 for (x in []))) { break; } }';
+  var f = function() { while(1, (2 for (x in []))) break; };
   actual = f + '';
+
   compareSource(expect, actual, summary);
 
   exitFunc ('test');
