@@ -44,7 +44,7 @@
 #ifndef nsBlockFrame_h___
 #define nsBlockFrame_h___
 
-#include "nsHTMLContainerFrame.h"
+#include "nsContainerFrame.h"
 #include "nsHTMLParts.h"
 #include "nsAbsoluteContainingBlock.h"
 #include "nsLineBox.h"
@@ -123,7 +123,7 @@ class nsIntervalSet;
 // (including <BR CLEAR="..."> frames)
 #define NS_BLOCK_HAS_CLEAR_CHILDREN         NS_FRAME_STATE_BIT(27)
 
-#define nsBlockFrameSuper nsHTMLContainerFrame
+#define nsBlockFrameSuper nsContainerFrame
 
 /*
  * Base class for block and inline frames.
@@ -181,8 +181,7 @@ public:
   virtual nscoord GetCaretBaseline() const;
   virtual void DestroyFrom(nsIFrame* aDestructRoot);
   virtual nsSplittableType GetSplittableType() const;
-  virtual PRBool IsContainingBlock() const;
-  virtual PRBool IsFloatContainingBlock() const;
+  virtual bool IsFloatContainingBlock() const;
   NS_IMETHOD BuildDisplayList(nsDisplayListBuilder*   aBuilder,
                               const nsRect&           aDirtyRect,
                               const nsDisplayListSet& aLists);
@@ -190,7 +189,7 @@ public:
                                   nscoord aX, nscoord aY, nsIFrame* aForChild,
                                   PRUint32 aFlags);
   virtual nsIAtom* GetType() const;
-  virtual PRBool IsFrameOfType(PRUint32 aFlags) const
+  virtual bool IsFrameOfType(PRUint32 aFlags) const
   {
     return nsContainerFrame::IsFrameOfType(aFlags &
              ~(nsIFrame::eCanContainOverflowContainers |
@@ -233,16 +232,16 @@ public:
   void SetupLineCursor();
 
   virtual void ChildIsDirty(nsIFrame* aChild);
-  virtual PRBool IsVisibleInSelection(nsISelection* aSelection);
+  virtual bool IsVisibleInSelection(nsISelection* aSelection);
 
-  virtual PRBool IsEmpty();
-  virtual PRBool CachedIsEmpty();
-  virtual PRBool IsSelfEmpty();
+  virtual bool IsEmpty();
+  virtual bool CachedIsEmpty();
+  virtual bool IsSelfEmpty();
 
   // Given that we have a bullet, does it actually draw something, i.e.,
   // do we have either a 'list-style-type' or 'list-style-image' that is
   // not 'none'?
-  PRBool BulletIsEmpty() const;
+  bool BulletIsEmpty() const;
 
   /**
    * Return the bullet text equivalent.
@@ -271,30 +270,30 @@ public:
 
   virtual nsresult StealFrame(nsPresContext* aPresContext,
                               nsIFrame*      aChild,
-                              PRBool         aForceNormal = PR_FALSE);
+                              bool           aForceNormal = false);
 
   virtual void DeleteNextInFlowChild(nsPresContext* aPresContext,
                                      nsIFrame*      aNextInFlow,
-                                     PRBool         aDeletingEmptyFrames);
+                                     bool           aDeletingEmptyFrames);
 
   /**
    * Determines whether the collapsed margin carried out of the last
    * line includes the margin-top of a line with clearance (in which
    * case we must avoid collapsing that margin with our bottom margin)
    */
-  PRBool CheckForCollapsedBottomMarginFromClearanceLine();
+  bool CheckForCollapsedBottomMarginFromClearanceLine();
 
   static nsresult GetCurrentLine(nsBlockReflowState *aState, nsLineBox **aOutCurrentLine);
 
-  static PRBool BlockIsMarginRoot(nsIFrame* aBlock);
-  static PRBool BlockNeedsFloatManager(nsIFrame* aBlock);
+  static bool BlockIsMarginRoot(nsIFrame* aBlock);
+  static bool BlockNeedsFloatManager(nsIFrame* aBlock);
 
   /**
    * Returns whether aFrame is a block frame that will wrap its contents
    * around floats intruding on it from the outside.  (aFrame need not
    * be a block frame, but if it's not, the result will be false.)
    */
-  static PRBool BlockCanIntersectFloats(nsIFrame* aFrame);
+  static bool BlockCanIntersectFloats(nsIFrame* aFrame);
 
   /**
    * Returns the width that needs to be cleared past floats for blocks
@@ -331,10 +330,9 @@ public:
   
 protected:
   nsBlockFrame(nsStyleContext* aContext)
-    : nsHTMLContainerFrame(aContext)
+    : nsContainerFrame(aContext)
     , mMinWidth(NS_INTRINSIC_WIDTH_UNKNOWN)
     , mPrefWidth(NS_INTRINSIC_WIDTH_UNKNOWN)
-    , mAbsoluteContainer(kAbsoluteList)
   {
 #ifdef DEBUG
   InitDebugFlags();
@@ -357,14 +355,14 @@ protected:
   void TryAllLines(nsLineList::iterator* aIterator,
                    nsLineList::iterator* aStartIterator,
                    nsLineList::iterator* aEndIterator,
-                   PRBool* aInOverflowLines);
+                   bool* aInOverflowLines);
 
   void SetFlags(nsFrameState aFlags) {
     mState &= ~NS_BLOCK_FLAGS_MASK;
     mState |= aFlags;
   }
 
-  PRBool HaveOutsideBullet() const {
+  bool HaveOutsideBullet() const {
 #if defined(DEBUG) && !defined(DEBUG_rods)
     if(mState & NS_BLOCK_FRAME_HAS_OUTSIDE_BULLET) {
       NS_ASSERTION(mBullet,"NS_BLOCK_FRAME_HAS_OUTSIDE_BULLET flag set and no mBullet");
@@ -386,9 +384,10 @@ protected:
                                 nsHTMLReflowMetrics&     aMetrics,
                                 nscoord*                 aBottomEdgeOfChildren);
 
-  void ComputeOverflowAreas(const nsHTMLReflowState& aReflowState,
-                            nsHTMLReflowMetrics&     aMetrics,
-                            nscoord                  aBottomEdgeOfChildren);
+  void ComputeOverflowAreas(const nsRect&         aBounds,
+                            const nsStyleDisplay* aDisplay,
+                            nscoord               aBottomEdgeOfChildren,
+                            nsOverflowAreas&      aOverflowAreas);
 
   /** add the frames in aFrameList to this block after aPrevSibling
     * this block thinks in terms of lines, but the frame construction code
@@ -413,7 +412,7 @@ protected:
    * @param aPresContext the pres context
    * @return whether the frame is a BIDI form control
    */
-  PRBool IsVisualFormControl(nsPresContext* aPresContext);
+  bool IsVisualFormControl(nsPresContext* aPresContext);
 #endif
 
 public:
@@ -434,8 +433,10 @@ public:
   nsresult DoRemoveFrame(nsIFrame* aDeletedFrame, PRUint32 aFlags);
 
   void ReparentFloats(nsIFrame* aFirstFrame,
-                      nsBlockFrame* aOldParent, PRBool aFromOverflow,
-                      PRBool aReparentSiblings);
+                      nsBlockFrame* aOldParent, bool aFromOverflow,
+                      bool aReparentSiblings);
+
+  virtual bool UpdateOverflow();
 
   /** Load all of aFrame's floats into the float manager iff aFrame is not a
    *  block formatting context. Handles all necessary float manager translations;
@@ -449,9 +450,9 @@ protected:
 
   /** grab overflow lines from this block's prevInFlow, and make them
     * part of this block's mLines list.
-    * @return PR_TRUE if any lines were drained.
+    * @return true if any lines were drained.
     */
-  PRBool DrainOverflowLines(nsBlockReflowState& aState);
+  bool DrainOverflowLines();
 
   /** grab pushed floats from this block's prevInFlow, and splice
     * them into this block's mFloats list.
@@ -480,7 +481,7 @@ protected:
   line_iterator RemoveFloat(nsIFrame* aFloat);
 
   void CollectFloats(nsIFrame* aFrame, nsFrameList& aList,
-                     PRBool aFromOverflow, PRBool aCollectFromSiblings);
+                     bool aFromOverflow, bool aCollectFromSiblings);
   // Remove a float, abs, rel positioned frame from the appropriate block's list
   static void DoRemoveOutOfFlowFrame(nsIFrame* aFrame);
 
@@ -506,18 +507,18 @@ protected:
    */
   nsresult ReflowLine(nsBlockReflowState& aState,
                       line_iterator aLine,
-                      PRBool* aKeepReflowGoing);
+                      bool* aKeepReflowGoing);
 
   // Return false if it needs another reflow because of reduced space
   // between floats that are next to it (but not next to its top), and
   // return true otherwise.
-  PRBool PlaceLine(nsBlockReflowState& aState,
+  bool PlaceLine(nsBlockReflowState& aState,
                    nsLineLayout&       aLineLayout,
                    line_iterator       aLine,
                    nsFloatManager::SavedState* aFloatStateBeforeLine,
                    nsRect&             aFloatAvailableSpace, /* in-out */
                    nscoord&            aAvailableSpaceHeight, /* in-out */
-                   PRBool*             aKeepReflowGoing);
+                   bool*             aKeepReflowGoing);
 
   /**
    * Mark |aLine| dirty, and, if necessary because of possible
@@ -532,8 +533,8 @@ protected:
                          const nsLineList* aLineList = nsnull);
 
   // XXX where to go
-  PRBool ShouldJustifyLine(nsBlockReflowState& aState,
-                           line_iterator aLine);
+  bool IsLastLine(nsBlockReflowState& aState,
+                  line_iterator aLine);
 
   void DeleteLine(nsBlockReflowState& aState,
                   nsLineList::iterator aLine,
@@ -542,16 +543,16 @@ protected:
   //----------------------------------------
   // Methods for individual frame reflow
 
-  PRBool ShouldApplyTopMargin(nsBlockReflowState& aState,
+  bool ShouldApplyTopMargin(nsBlockReflowState& aState,
                               nsLineBox* aLine);
 
   nsresult ReflowBlockFrame(nsBlockReflowState& aState,
                             line_iterator aLine,
-                            PRBool* aKeepGoing);
+                            bool* aKeepGoing);
 
   nsresult ReflowInlineFrames(nsBlockReflowState& aState,
                               line_iterator aLine,
-                              PRBool* aKeepLineGoing);
+                              bool* aKeepLineGoing);
 
   nsresult DoReflowInlineFrames(nsBlockReflowState& aState,
                                 nsLineLayout& aLineLayout,
@@ -560,9 +561,9 @@ protected:
                                 nscoord& aAvailableSpaceHeight,
                                 nsFloatManager::SavedState*
                                   aFloatStateBeforeLine,
-                                PRBool* aKeepReflowGoing,
+                                bool* aKeepReflowGoing,
                                 LineReflowStatus* aLineReflowStatus,
-                                PRBool aAllowPullUp);
+                                bool aAllowPullUp);
 
   nsresult ReflowInlineFrame(nsBlockReflowState& aState,
                              nsLineLayout& aLineLayout,
@@ -589,7 +590,7 @@ protected:
                        // Whether the float's position
                        // (aAdjustedAvailableSpace) has been pushed down
                        // due to the presence of other floats.
-                       PRBool              aFloatPushedDown,
+                       bool                aFloatPushedDown,
                        nsReflowStatus&     aReflowStatus);
 
   //----------------------------------------
@@ -601,19 +602,19 @@ protected:
    * @param aState the block reflow state
    * @param aLine where to put a new frame
    * @param aFrame the frame
-   * @param aMadeNewFrame PR_TRUE if a new frame was created, PR_FALSE if not
+   * @param aMadeNewFrame true if a new frame was created, false if not
    * @return NS_OK if a next-in-flow already exists or is successfully created
    */
   virtual nsresult CreateContinuationFor(nsBlockReflowState& aState,
                                          nsLineBox*          aLine,
                                          nsIFrame*           aFrame,
-                                         PRBool&             aMadeNewFrame);
+                                         bool&             aMadeNewFrame);
 
   // Push aLine, which cannot be placed on this page/column but should
   // fit on a future one.  Set aKeepReflowGoing to false.
   void PushTruncatedLine(nsBlockReflowState& aState,
                          line_iterator       aLine,
-                         PRBool&             aKeepReflowGoing);
+                         bool&             aKeepReflowGoing);
 
   nsresult SplitLine(nsBlockReflowState& aState,
                      nsLineLayout& aLineLayout,
@@ -644,7 +645,7 @@ protected:
   nsIFrame* PullFrameFrom(nsBlockReflowState&  aState,
                           nsLineBox*           aLine,
                           nsBlockFrame*        aFromContainer,
-                          PRBool               aFromOverflowLine,
+                          bool                 aFromOverflowLine,
                           nsLineList::iterator aFromLine);
 
   void PushLines(nsBlockReflowState& aState,
@@ -659,20 +660,20 @@ protected:
   //----------------------------------------
   // List handling kludge
 
-  // If this returns PR_TRUE, the block it's called on should get the
+  // If this returns true, the block it's called on should get the
   // NS_FRAME_HAS_DIRTY_CHILDREN bit set on it by the caller; either directly
   // if it's already in reflow, or via calling FrameNeedsReflow() to schedule a
   // reflow.
-  PRBool RenumberLists(nsPresContext* aPresContext);
+  bool RenumberLists(nsPresContext* aPresContext);
 
-  static PRBool RenumberListsInBlock(nsPresContext* aPresContext,
+  static bool RenumberListsInBlock(nsPresContext* aPresContext,
                                      nsBlockFrame* aBlockFrame,
                                      PRInt32* aOrdinal,
                                      PRInt32 aDepth);
 
-  static PRBool RenumberListsFor(nsPresContext* aPresContext, nsIFrame* aKid, PRInt32* aOrdinal, PRInt32 aDepth);
+  static bool RenumberListsFor(nsPresContext* aPresContext, nsIFrame* aKid, PRInt32* aOrdinal, PRInt32 aDepth);
 
-  static PRBool FrameStartsCounterScope(nsIFrame* aFrame);
+  static bool FrameStartsCounterScope(nsIFrame* aFrame);
 
   void ReflowBullet(nsBlockReflowState& aState,
                     nsHTMLReflowMetrics& aMetrics,
@@ -732,7 +733,7 @@ protected:
   nsFrameList* RemovePushedFloats();
 
 #ifdef NS_DEBUG
-  void VerifyLines(PRBool aFinalCheckOK);
+  void VerifyLines(bool aFinalCheckOK);
   void VerifyOverflowSituation();
   PRInt32 GetDepth() const;
 #endif
@@ -751,22 +752,18 @@ protected:
   friend class nsBlockReflowState;
   friend class nsBlockInFlowLineIterator;
 
-private:
-  nsAbsoluteContainingBlock mAbsoluteContainer;
-
-
 #ifdef DEBUG
 public:
-  static PRBool gLamePaintMetrics;
-  static PRBool gLameReflowMetrics;
-  static PRBool gNoisy;
-  static PRBool gNoisyDamageRepair;
-  static PRBool gNoisyIntrinsic;
-  static PRBool gNoisyReflow;
-  static PRBool gReallyNoisyReflow;
-  static PRBool gNoisyFloatManager;
-  static PRBool gVerifyLines;
-  static PRBool gDisableResizeOpt;
+  static bool gLamePaintMetrics;
+  static bool gLameReflowMetrics;
+  static bool gNoisy;
+  static bool gNoisyDamageRepair;
+  static bool gNoisyIntrinsic;
+  static bool gNoisyReflow;
+  static bool gReallyNoisyReflow;
+  static bool gNoisyFloatManager;
+  static bool gVerifyLines;
+  static bool gDisableResizeOpt;
 
   static PRInt32 gNoiseIndent;
 
@@ -780,7 +777,7 @@ protected:
 #ifdef DEBUG
 class AutoNoisyIndenter {
 public:
-  AutoNoisyIndenter(PRBool aDoIndent) : mIndented(aDoIndent) {
+  AutoNoisyIndenter(bool aDoIndent) : mIndented(aDoIndent) {
     if (mIndented) {
       nsBlockFrame::gNoiseIndent++;
     }
@@ -791,7 +788,7 @@ public:
     }
   }
 private:
-  PRBool mIndented;
+  bool mIndented;
 };
 #endif
 
@@ -801,14 +798,14 @@ private:
 class nsBlockInFlowLineIterator {
 public:
   typedef nsBlockFrame::line_iterator line_iterator;
-  nsBlockInFlowLineIterator(nsBlockFrame* aFrame, line_iterator aLine, PRBool aInOverflow);
+  nsBlockInFlowLineIterator(nsBlockFrame* aFrame, line_iterator aLine, bool aInOverflow);
   /**
    * Set up the iterator to point to the first line found starting from
    * aFrame. Sets aFoundValidLine to false if there is no such line.
    * After aFoundValidLine has returned false, don't call any methods on this
    * object again.
    */
-  nsBlockInFlowLineIterator(nsBlockFrame* aFrame, PRBool* aFoundValidLine);
+  nsBlockInFlowLineIterator(nsBlockFrame* aFrame, bool* aFoundValidLine);
   /**
    * Set up the iterator to point to the line that contains aFindFrame (either
    * directly or indirectly).  If aFrame is out of flow, or contained in an
@@ -818,12 +815,12 @@ public:
    * object again.
    */
   nsBlockInFlowLineIterator(nsBlockFrame* aFrame, nsIFrame* aFindFrame,
-                            PRBool* aFoundValidLine);
+                            bool* aFoundValidLine);
 
   line_iterator GetLine() { return mLine; }
-  PRBool IsLastLineInList();
+  bool IsLastLineInList();
   nsBlockFrame* GetContainer() { return mFrame; }
-  PRBool GetInOverflow() { return mInOverflowLines != nsnull; }
+  bool GetInOverflow() { return mInOverflowLines != nsnull; }
 
   /**
    * Returns the current line list we're iterating, null means
@@ -840,12 +837,12 @@ public:
    * Returns false if there are no more lines. After this has returned false,
    * don't call any methods on this object again.
    */
-  PRBool Next();
+  bool Next();
   /**
    * Returns false if there are no more lines. After this has returned false,
    * don't call any methods on this object again.
    */
-  PRBool Prev();
+  bool Prev();
 
 private:
   nsBlockFrame* mFrame;
@@ -856,7 +853,7 @@ private:
    * Moves iterator to next valid line reachable from the current block.
    * Returns false if there are no valid lines.
    */
-  PRBool FindValidLine();
+  bool FindValidLine();
 };
 
 #endif /* nsBlockFrame_h___ */
