@@ -40,6 +40,8 @@
 #ifndef nsCSSValue_h___
 #define nsCSSValue_h___
 
+#include "mozilla/Attributes.h"
+
 #include "nsCOMPtr.h"
 #include "nsCRTGlue.h"
 #include "nsCSSKeywords.h"
@@ -495,9 +497,8 @@ public:
   private:
     mutable bool mURIResolved;
 
-    // not to be implemented
-    URL(const URL& aOther);
-    URL& operator=(const URL& aOther);
+    URL(const URL& aOther) MOZ_DELETE;
+    URL& operator=(const URL& aOther) MOZ_DELETE;
   };
 
   struct Image : public URL {
@@ -644,9 +645,8 @@ private:
 #undef CSSVALUE_LIST_FOR_EXTRA_VALUES
 
 private:
-  // not to be implemented
-  Array(const Array& aOther);
-  Array& operator=(const Array& aOther);
+  Array(const Array& aOther) MOZ_DELETE;
+  Array& operator=(const Array& aOther) MOZ_DELETE;
 };
 
 // Prefer nsCSSValue::Array for lists of fixed size.
@@ -1073,9 +1073,8 @@ struct nsCSSValueGradient {
   NS_INLINE_DECL_REFCOUNTING(nsCSSValueGradient)
 
 private:
-  // not to be implemented
-  nsCSSValueGradient(const nsCSSValueGradient& aOther);
-  nsCSSValueGradient& operator=(const nsCSSValueGradient& aOther);
+  nsCSSValueGradient(const nsCSSValueGradient& aOther) MOZ_DELETE;
+  nsCSSValueGradient& operator=(const nsCSSValueGradient& aOther) MOZ_DELETE;
 };
 
 struct nsCSSCornerSizes {
