@@ -50,31 +50,38 @@
 
 class nsStyleContext;
 struct PLDHashTable;
-class nsILanguageAtomService;
 struct nsRuleData;
 class nsIStyleRule;
-struct nsCSSStruct;
 struct nsCSSValueList;
-// Copy of typedef that's in nsCSSStruct.h, for compilation speed.
-typedef nsCSSStruct nsRuleDataStruct;
 
-struct nsRuleDataFont;
 class nsCSSValue;
 struct nsCSSRect;
 
 class nsStyleCoord;
 
+template <nsStyleStructID MinIndex, nsStyleStructID Count>
+class FixedStyleStructArray
+{
+private:
+  void* mArray[Count];
+public:
+  void*& operator[](nsStyleStructID aIndex) {
+    NS_ABORT_IF_FALSE(MinIndex <= aIndex && aIndex < (MinIndex + Count),
+                      "out of range");
+    return mArray[aIndex - MinIndex];
+  }
+
+  const void* operator[](nsStyleStructID aIndex) const {
+    NS_ABORT_IF_FALSE(MinIndex <= aIndex && aIndex < (MinIndex + Count),
+                      "out of range");
+    return mArray[aIndex - MinIndex];
+  }
+};
+
 struct nsInheritedStyleData
 {
-
-#define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args) \
-  nsStyle##name * m##name##Data;
-#define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args)
-
-#include "nsStyleStructList.h"
-
-#undef STYLE_STRUCT_INHERITED
-#undef STYLE_STRUCT_RESET
+  FixedStyleStructArray<nsStyleStructID_Inherited_Start,
+                        nsStyleStructID_Inherited_Count> mStyleStructs;
 
   void* operator new(size_t sz, nsPresContext* aContext) CPP_THROW_NEW {
     return aContext->AllocateFromShell(sz);
@@ -82,8 +89,9 @@ struct nsInheritedStyleData
 
   void DestroyStructs(PRUint32 aBits, nsPresContext* aContext) {
 #define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args) \
-    if (m##name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
-      m##name##Data->Destroy(aContext);
+    void *name##Data = mStyleStructs[eStyleStruct_##name]; \
+    if (name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
+      static_cast<nsStyle##name*>(name##Data)->Destroy(aContext);
 #define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args)
 
 #include "nsStyleStructList.h"
@@ -98,30 +106,26 @@ struct nsInheritedStyleData
   }
 
   nsInheritedStyleData() {
-#define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args) \
-    m##name##Data = nsnull;
-#define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args)
-
-#include "nsStyleStructList.h"
-
-#undef STYLE_STRUCT_INHERITED
-#undef STYLE_STRUCT_RESET
-
+    for (nsStyleStructID i = nsStyleStructID_Inherited_Start;
+         i < nsStyleStructID_Inherited_Start + nsStyleStructID_Inherited_Count;
+         i = nsStyleStructID(i + 1)) {
+      mStyleStructs[i] = nsnull;
+    }
   }
 };
 
 struct nsResetStyleData
 {
+  FixedStyleStructArray<nsStyleStructID_Reset_Start,
+                        nsStyleStructID_Reset_Count> mStyleStructs;
+
   nsResetStyleData()
   {
-#define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args) \
-    m##name##Data = nsnull;
-#define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args)
-
-#include "nsStyleStructList.h"
-
-#undef STYLE_STRUCT_RESET
-#undef STYLE_STRUCT_INHERITED
+    for (nsStyleStructID i = nsStyleStructID_Reset_Start;
+         i < nsStyleStructID_Reset_Start + nsStyleStructID_Reset_Count;
+         i = nsStyleStructID(i + 1)) {
+      mStyleStructs[i] = nsnull;
+    }
   }
 
   void* operator new(size_t sz, nsPresContext* aContext) CPP_THROW_NEW {
@@ -130,8 +134,9 @@ struct nsResetStyleData
 
   void Destroy(PRUint32 aBits, nsPresContext* aContext) {
 #define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args) \
-    if (m##name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
-      m##name##Data->Destroy(aContext);
+    void *name##Data = mStyleStructs[eStyleStruct_##name]; \
+    if (name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
+      static_cast<nsStyle##name*>(name##Data)->Destroy(aContext);
 #define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args)
 
 #include "nsStyleStructList.h"
@@ -141,78 +146,46 @@ struct nsResetStyleData
 
     aContext->FreeToShell(sizeof(nsResetStyleData), this);
   }
-
-#define STYLE_STRUCT_RESET(name, checkdata_cb, ctor_args) \
-  nsStyle##name * m##name##Data;
-#define STYLE_STRUCT_INHERITED(name, checkdata_cb, ctor_args)
-
-#include "nsStyleStructList.h"
-
-#undef STYLE_STRUCT_RESET
-#undef STYLE_STRUCT_INHERITED
-
 };
 
 struct nsCachedStyleData
 {
-  struct StyleStructInfo {
-    ptrdiff_t mCachedStyleDataOffset;
-    ptrdiff_t mInheritResetOffset;
-    PRBool    mIsReset;
-  };
-
-  static StyleStructInfo gInfo[];
-
   nsInheritedStyleData* mInheritedData;
   nsResetStyleData* mResetData;
 
-  static PRBool IsReset(const nsStyleStructID& aSID) {
-    return gInfo[aSID].mIsReset;
+  static bool IsReset(const nsStyleStructID aSID) {
+    NS_ABORT_IF_FALSE(0 <= aSID && aSID < nsStyleStructID_Length,
+                      "must be an inherited or reset SID");
+    return nsStyleStructID_Reset_Start <= aSID;
   }
 
-  static PRUint32 GetBitForSID(const nsStyleStructID& aSID) {
+  static PRUint32 GetBitForSID(const nsStyleStructID aSID) {
     return 1 << aSID;
   }
 
-  void* NS_FASTCALL GetStyleData(const nsStyleStructID& aSID) {
-    // Each struct is stored at this.m##type##Data->m##name##Data where
-    // |type| is either Inherit or Reset, and |name| is the name of the
-    // style struct.  The |gInfo| stores the offset of the appropriate
-    // m##type##Data for the struct within nsCachedStyleData (|this|)
-    // and the offset of the appropriate m##name##Data within the
-    // m##type##Data.  Note that if we don't have any reset structs,
-    // then mResetData is null, and likewise for mInheritedData.  This
-    // saves us from having to go through the long if-else cascade into
-    // which most compilers will turn a case statement.
-
-    // NOTE:  nsStyleContext::SetStyle works roughly the same way.
-
-    const StyleStructInfo& info = gInfo[aSID];
-
-    // Get either &mInheritedData or &mResetData.
-    char* resetOrInheritSlot = reinterpret_cast<char*>(this) + info.mCachedStyleDataOffset;
-
-    // Get either mInheritedData or mResetData.
-    char* resetOrInherit = reinterpret_cast<char*>(*reinterpret_cast<void**>(resetOrInheritSlot));
-
-    void* data = nsnull;
-    if (resetOrInherit) {
-      // If we have the mInheritedData or mResetData, then we might have
-      // the struct, so get it.
-      char* dataSlot = resetOrInherit + info.mInheritResetOffset;
-      data = *reinterpret_cast<void**>(dataSlot);
+  void* NS_FASTCALL GetStyleData(const nsStyleStructID aSID) {
+    if (IsReset(aSID)) {
+      if (mResetData) {
+        return mResetData->mStyleStructs[aSID];
+      }
+    } else {
+      if (mInheritedData) {
+        return mInheritedData->mStyleStructs[aSID];
+      }
     }
-    return data;
+    return nsnull;
   }
 
   // Typesafe and faster versions of the above
   #define STYLE_STRUCT_INHERITED(name_, checkdata_cb_, ctor_args_)       \
-    nsStyle##name_ * NS_FASTCALL GetStyle##name_ () {        \
-      return mInheritedData ? mInheritedData->m##name_##Data : nsnull;   \
+    nsStyle##name_ * NS_FASTCALL GetStyle##name_ () {                    \
+      return mInheritedData ? static_cast<nsStyle##name_*>(              \
+        mInheritedData->mStyleStructs[eStyleStruct_##name_]) : nsnull;   \
     }
   #define STYLE_STRUCT_RESET(name_, checkdata_cb_, ctor_args_)           \
-    nsStyle##name_ * NS_FASTCALL GetStyle##name_ () {        \
-      return mResetData ? mResetData->m##name_##Data : nsnull;           \
+    nsStyle##name_ * NS_FASTCALL GetStyle##name_ () {                    \
+      return mResetData ? static_cast<nsStyle##name_*>(                  \
+        mResetData->mStyleStructs[eStyleStruct_##name_]) : nsnull;       \
     }
   #include "nsStyleStructList.h"
   #undef STYLE_STRUCT_RESET
@@ -316,20 +289,20 @@ private:
   struct Key {
     nsIStyleRule* mRule;
     PRUint8 mLevel;
-    PRPackedBool mIsImportantRule;
+    bool mIsImportantRule;
 
-    Key(nsIStyleRule* aRule, PRUint8 aLevel, PRPackedBool aIsImportantRule)
+    Key(nsIStyleRule* aRule, PRUint8 aLevel, bool aIsImportantRule)
       : mRule(aRule), mLevel(aLevel), mIsImportantRule(aIsImportantRule)
     {}
 
-    PRBool operator==(const Key& aOther) const
+    bool operator==(const Key& aOther) const
     {
       return mRule == aOther.mRule &&
              mLevel == aOther.mLevel &&
              mIsImportantRule == aOther.mIsImportantRule;
     }
 
-    PRBool operator!=(const Key& aOther) const
+    bool operator!=(const Key& aOther) const
     {
       return !(*this == aOther);
     }
@@ -338,7 +311,7 @@ private:
   static PLDHashNumber
   ChildrenHashHashKey(PLDHashTable *aTable, const void *aKey);
 
-  static PRBool
+  static bool
   ChildrenHashMatchEntry(PLDHashTable *aTable,
                          const PLDHashEntryHdr *aHdr,
                          const void *aKey);
@@ -375,10 +348,10 @@ private:
     kMaxChildrenInList = 32
   };
 
-  PRBool HaveChildren() const {
+  bool HaveChildren() const {
     return mChildren.asVoid != nsnull;
   }
-  PRBool ChildrenAreHashed() {
+  bool ChildrenAreHashed() {
     return (PRWord(mChildren.asVoid) & kTypeMask) == kHashType;
   }
   nsRuleNode* ChildrenList() {
@@ -442,7 +415,6 @@ public:
   // (which comes from the presShell) to perform the allocation.
   void* operator new(size_t sz, nsPresContext* aContext) CPP_THROW_NEW;
   void Destroy() { DestroyInternal(nsnull); }
-  static nsILanguageAtomService* gLangService;
 
   // Implemented in nsStyleSet.h, since it needs to know about nsStyleSet.
   inline void AddRef();
@@ -459,193 +431,192 @@ protected:
                                nsStyleContext* aContext);
 
   const void*
-    WalkRuleTree(const nsStyleStructID aSID, nsStyleContext* aContext,
-                 nsRuleData* aRuleData, nsRuleDataStruct* aSpecificData);
+    WalkRuleTree(const nsStyleStructID aSID, nsStyleContext* aContext);
 
   const void*
     ComputeDisplayData(void* aStartStruct,
-                       const nsRuleDataStruct& aData,
+                       const nsRuleData* aRuleData,
                        nsStyleContext* aContext, nsRuleNode* aHighestNode,
                        RuleDetail aRuleDetail,
-                       const PRBool aCanStoreInRuleTree);
+                       const bool aCanStoreInRuleTree);
 
   const void*
     ComputeVisibilityData(void* aStartStruct,
-                          const nsRuleDataStruct& aData,
+                          const nsRuleData* aRuleData,
                           nsStyleContext* aContext, nsRuleNode* aHighestNode,
                           RuleDetail aRuleDetail,
-                          const PRBool aCanStoreInRuleTree);
+                          const bool aCanStoreInRuleTree);
 
   const void*
     ComputeFontData(void* aStartStruct,
-                    const nsRuleDataStruct& aData,
+                    const nsRuleData* aRuleData,
                     nsStyleContext* aContext, nsRuleNode* aHighestNode,
                     RuleDetail aRuleDetail,
-                    const PRBool aCanStoreInRuleTree);
+                    const bool aCanStoreInRuleTree);
 
   const void*
     ComputeColorData(void* aStartStruct,
-                     const nsRuleDataStruct& aData,
+                     const nsRuleData* aRuleData,
                      nsStyleContext* aContext, nsRuleNode* aHighestNode,
                      RuleDetail aRuleDetail,
-                     const PRBool aCanStoreInRuleTree);
+                     const bool aCanStoreInRuleTree);
 
   const void*
     ComputeBackgroundData(void* aStartStruct,
-                          const nsRuleDataStruct& aData,
+                          const nsRuleData* aRuleData,
                           nsStyleContext* aContext, nsRuleNode* aHighestNode,
                           RuleDetail aRuleDetail,
-                          const PRBool aCanStoreInRuleTree);
+                          const bool aCanStoreInRuleTree);
 
   const void*
     ComputeMarginData(void* aStartStruct,
-                      const nsRuleDataStruct& aData,
+                      const nsRuleData* aRuleData,
                       nsStyleContext* aContext, nsRuleNode* aHighestNode,
                       RuleDetail aRuleDetail,
-                      const PRBool aCanStoreInRuleTree);
+                      const bool aCanStoreInRuleTree);
 
   const void*
     ComputeBorderData(void* aStartStruct,
-                      const nsRuleDataStruct& aData,
+                      const nsRuleData* aRuleData,
                       nsStyleContext* aContext, nsRuleNode* aHighestNode,
                       RuleDetail aRuleDetail,
-                      const PRBool aCanStoreInRuleTree);
+                      const bool aCanStoreInRuleTree);
 
   const void*
     ComputePaddingData(void* aStartStruct,
-                       const nsRuleDataStruct& aData,
+                       const nsRuleData* aRuleData,
                        nsStyleContext* aContext, nsRuleNode* aHighestNode,
                        RuleDetail aRuleDetail,
-                       const PRBool aCanStoreInRuleTree);
+                       const bool aCanStoreInRuleTree);
 
   const void*
     ComputeOutlineData(void* aStartStruct,
-                       const nsRuleDataStruct& aData,
+                       const nsRuleData* aRuleData,
                        nsStyleContext* aContext, nsRuleNode* aHighestNode,
                        RuleDetail aRuleDetail,
-                       const PRBool aCanStoreInRuleTree);
+                       const bool aCanStoreInRuleTree);
 
   const void*
     ComputeListData(void* aStartStruct,
-                    const nsRuleDataStruct& aData,
+                    const nsRuleData* aRuleData,
                     nsStyleContext* aContext, nsRuleNode* aHighestNode,
                     RuleDetail aRuleDetail,
-                    const PRBool aCanStoreInRuleTree);
+                    const bool aCanStoreInRuleTree);
 
   const void*
     ComputePositionData(void* aStartStruct,
-                        const nsRuleDataStruct& aData,
+                        const nsRuleData* aRuleData,
                         nsStyleContext* aContext, nsRuleNode* aHighestNode,
                         RuleDetail aRuleDetail,
-                        const PRBool aCanStoreInRuleTree);
+                        const bool aCanStoreInRuleTree);
 
   const void*
     ComputeTableData(void* aStartStruct,
-                     const nsRuleDataStruct& aData,
+                     const nsRuleData* aRuleData,
                      nsStyleContext* aContext, nsRuleNode* aHighestNode,
                      RuleDetail aRuleDetail,
-                     const PRBool aCanStoreInRuleTree);
+                     const bool aCanStoreInRuleTree);
 
   const void*
     ComputeTableBorderData(void* aStartStruct,
-                           const nsRuleDataStruct& aData,
+                           const nsRuleData* aRuleData,
                            nsStyleContext* aContext, nsRuleNode* aHighestNode,
                            RuleDetail aRuleDetail,
-                           const PRBool aCanStoreInRuleTree);
+                           const bool aCanStoreInRuleTree);
 
   const void*
     ComputeContentData(void* aStartStruct,
-                       const nsRuleDataStruct& aData,
+                       const nsRuleData* aRuleData,
                        nsStyleContext* aContext, nsRuleNode* aHighestNode,
                        RuleDetail aRuleDetail,
-                       const PRBool aCanStoreInRuleTree);
+                       const bool aCanStoreInRuleTree);
 
   const void*
     ComputeQuotesData(void* aStartStruct,
-                      const nsRuleDataStruct& aData,
+                      const nsRuleData* aRuleData,
                       nsStyleContext* aContext, nsRuleNode* aHighestNode,
                       RuleDetail aRuleDetail,
-                      const PRBool aCanStoreInRuleTree);
+                      const bool aCanStoreInRuleTree);
 
   const void*
     ComputeTextData(void* aStartStruct,
-                    const nsRuleDataStruct& aData,
+                    const nsRuleData* aRuleData,
                     nsStyleContext* aContext, nsRuleNode* aHighestNode,
                     RuleDetail aRuleDetail,
-                    const PRBool aCanStoreInRuleTree);
+                    const bool aCanStoreInRuleTree);
 
   const void*
     ComputeTextResetData(void* aStartStruct,
-                         const nsRuleDataStruct& aData,
+                         const nsRuleData* aRuleData,
                          nsStyleContext* aContext, nsRuleNode* aHighestNode,
                          RuleDetail aRuleDetail,
-                         const PRBool aCanStoreInRuleTree);
+                         const bool aCanStoreInRuleTree);
 
   const void*
     ComputeUserInterfaceData(void* aStartStruct,
-                             const nsRuleDataStruct& aData,
+                             const nsRuleData* aRuleData,
                              nsStyleContext* aContext,
                              nsRuleNode* aHighestNode,
                              RuleDetail aRuleDetail,
-                             const PRBool aCanStoreInRuleTree);
+                             const bool aCanStoreInRuleTree);
 
   const void*
     ComputeUIResetData(void* aStartStruct,
-                       const nsRuleDataStruct& aData,
+                       const nsRuleData* aRuleData,
                        nsStyleContext* aContext, nsRuleNode* aHighestNode,
                        RuleDetail aRuleDetail,
-                       const PRBool aCanStoreInRuleTree);
+                       const bool aCanStoreInRuleTree);
 
   const void*
     ComputeXULData(void* aStartStruct,
-                   const nsRuleDataStruct& aData,
+                   const nsRuleData* aRuleData,
                    nsStyleContext* aContext, nsRuleNode* aHighestNode,
                    RuleDetail aRuleDetail,
-                   const PRBool aCanStoreInRuleTree);
+                   const bool aCanStoreInRuleTree);
 
   const void*
     ComputeColumnData(void* aStartStruct,
-                      const nsRuleDataStruct& aData,
+                      const nsRuleData* aRuleData,
                       nsStyleContext* aContext, nsRuleNode* aHighestNode,
                       RuleDetail aRuleDetail,
-                      const PRBool aCanStoreInRuleTree);
+                      const bool aCanStoreInRuleTree);
 
   const void*
     ComputeSVGData(void* aStartStruct,
-                   const nsRuleDataStruct& aData,
+                   const nsRuleData* aRuleData,
                    nsStyleContext* aContext, nsRuleNode* aHighestNode,
                    RuleDetail aRuleDetail,
-                   const PRBool aCanStoreInRuleTree);
+                   const bool aCanStoreInRuleTree);
 
   const void*
     ComputeSVGResetData(void* aStartStruct,
-                        const nsRuleDataStruct& aData,
+                        const nsRuleData* aRuleData,
                         nsStyleContext* aContext, nsRuleNode* aHighestNode,
                         RuleDetail aRuleDetail,
-                        const PRBool aCanStoreInRuleTree);
+                        const bool aCanStoreInRuleTree);
 
   // helpers for |ComputeFontData| that need access to |mNoneBits|:
   static void SetFontSize(nsPresContext* aPresContext,
-                          const nsRuleDataFont& aFontData,
+                          const nsRuleData* aRuleData,
                           const nsStyleFont* aFont,
                           const nsStyleFont* aParentFont,
                           nscoord* aSize,
                           const nsFont& aSystemFont,
                           nscoord aParentSize,
                           nscoord aScriptLevelAdjustedParentSize,
-                          PRBool aUsedStartStruct,
-                          PRBool aAtRoot,
-                          PRBool& aCanStoreInRuleTree);
+                          bool aUsedStartStruct,
+                          bool aAtRoot,
+                          bool& aCanStoreInRuleTree);
 
   static void SetFont(nsPresContext* aPresContext,
                       nsStyleContext* aContext,
                       nscoord aMinFontSize,
                       PRUint8 aGenericFontID,
-                      const nsRuleDataFont& aFontData,
+                      const nsRuleData* aRuleData,
                       const nsStyleFont* aParentFont,
                       nsStyleFont* aFont,
-                      PRBool aStartStruct,
-                      PRBool& aCanStoreInRuleTree);
+                      bool aStartStruct,
+                      bool& aCanStoreInRuleTree);
 
   static void SetGenericFont(nsPresContext* aPresContext,
                              nsStyleContext* aContext,
@@ -660,9 +631,10 @@ protected:
                             const nsCSSValue& aRTLLogicalValue,
                             mozilla::css::Side aSide,
                             nsCSSRect& aValueRect,
-                            PRBool& aCanStoreInRuleTree);
+                            bool& aCanStoreInRuleTree);
 
-  inline RuleDetail CheckSpecifiedProperties(const nsStyleStructID aSID, const nsRuleDataStruct& aRuleDataStruct);
+  inline RuleDetail CheckSpecifiedProperties(const nsStyleStructID aSID,
+                                             const nsRuleData* aRuleData);
 
   const void* GetParentData(const nsStyleStructID aSID);
   #define STYLE_STRUCT(name_, checkdata_cb_, ctor_args_)  \
@@ -670,41 +642,15 @@ protected:
   #include "nsStyleStructList.h"
   #undef STYLE_STRUCT
 
-  const void* GetDisplayData(nsStyleContext* aContext);
-  const void* GetVisibilityData(nsStyleContext* aContext);
-  const void* GetFontData(nsStyleContext* aContext);
-  const void* GetColorData(nsStyleContext* aContext);
-  const void* GetBackgroundData(nsStyleContext* aContext);
-  const void* GetMarginData(nsStyleContext* aContext);
-  const void* GetBorderData(nsStyleContext* aContext);
-  const void* GetPaddingData(nsStyleContext* aContext);
-  const void* GetOutlineData(nsStyleContext* aContext);
-  const void* GetListData(nsStyleContext* aContext);
-  const void* GetPositionData(nsStyleContext* aContext);
-  const void* GetTableData(nsStyleContext* aContext);
-  const void* GetTableBorderData(nsStyleContext* aContext);
-
-  const void* GetContentData(nsStyleContext* aContext);
-  const void* GetQuotesData(nsStyleContext* aContext);
-  const void* GetTextData(nsStyleContext* aContext);
-  const void* GetTextResetData(nsStyleContext* aContext);
-  const void* GetUserInterfaceData(nsStyleContext* aContext);
-
-  const void* GetUIResetData(nsStyleContext* aContext);
-  const void* GetXULData(nsStyleContext* aContext);
-  const void* GetColumnData(nsStyleContext* aContext);
-  const void* GetSVGData(nsStyleContext* aContext);
-  const void* GetSVGResetData(nsStyleContext* aContext);
-
   already_AddRefed<nsCSSShadowArray>
               GetShadowData(const nsCSSValueList* aList,
                             nsStyleContext* aContext,
-                            PRBool aIsBoxShadow,
-                            PRBool& inherited);
+                            bool aIsBoxShadow,
+                            bool& inherited);
 
 private:
   nsRuleNode(nsPresContext* aPresContext, nsRuleNode* aParent,
-             nsIStyleRule* aRule, PRUint8 aLevel, PRBool aIsImportant);
+             nsIStyleRule* aRule, PRUint8 aLevel, bool aIsImportant);
   ~nsRuleNode();
 
 public:
@@ -712,9 +658,9 @@ public:
 
   // Transition never returns null; on out of memory it'll just return |this|.
   nsRuleNode* Transition(nsIStyleRule* aRule, PRUint8 aLevel,
-                         PRPackedBool aIsImportantRule);
+                         bool aIsImportantRule);
   nsRuleNode* GetParent() const { return mParent; }
-  PRBool IsRoot() const { return mParent == nsnull; }
+  bool IsRoot() const { return mParent == nsnull; }
 
   // These PRUint8s are really nsStyleSet::sheetType values.
   PRUint8 GetLevel() const {
@@ -722,7 +668,7 @@ public:
     return (mDependentBits & NS_RULE_NODE_LEVEL_MASK) >>
              NS_RULE_NODE_LEVEL_SHIFT;
   }
-  PRBool IsImportantRule() const {
+  bool IsImportantRule() const {
     NS_ASSERTION(!IsRoot(), "can't call on root");
     return (mDependentBits & NS_RULE_NODE_IS_IMPORTANT) != 0;
   }
@@ -734,11 +680,11 @@ public:
 
   const void* GetStyleData(nsStyleStructID aSID,
                            nsStyleContext* aContext,
-                           PRBool aComputeData);
+                           bool aComputeData);
 
   #define STYLE_STRUCT(name_, checkdata_cb_, ctor_args_)                      \
     const nsStyle##name_* GetStyle##name_(nsStyleContext* aContext,           \
-                                          PRBool aComputeData);
+                                          bool aComputeData);
   #include "nsStyleStructList.h"
   #undef STYLE_STRUCT
 
@@ -749,12 +695,12 @@ public:
    * returning true if the node on which it was called was destroyed.
    */
   void Mark();
-  PRBool Sweep();
+  bool Sweep();
 
-  static PRBool
+  static bool
     HasAuthorSpecifiedRules(nsStyleContext* aStyleContext,
                             PRUint32 ruleTypeMask,
-                            PRBool aAuthorColorsAllowed);
+                            bool aAuthorColorsAllowed);
 
   // Expose this so media queries can use it
   static nscoord CalcLengthWithInitialFont(nsPresContext* aPresContext,
@@ -763,7 +709,7 @@ public:
   static nscoord CalcLength(const nsCSSValue& aValue,
                             nsStyleContext* aStyleContext,
                             nsPresContext* aPresContext,
-                            PRBool& aCanStoreInRuleTree);
+                            bool& aCanStoreInRuleTree);
 
   struct ComputedCalc {
     nscoord mLength;
@@ -776,7 +722,7 @@ public:
   SpecifiedCalcToComputedCalc(const nsCSSValue& aValue,
                               nsStyleContext* aStyleContext,
                               nsPresContext* aPresContext,
-                              PRBool& aCanStoreInRuleTree);
+                              bool& aCanStoreInRuleTree);
 
   // Compute the value of an nsStyleCoord that IsCalcUnit().
   // (Values that don't require aPercentageBasis should be handled
@@ -793,10 +739,14 @@ public:
   // cached data such that we need to do dynamic change handling for
   // changes that change the results of media queries or require
   // rebuilding all style data.
-  PRBool TreeHasCachedData() const {
+  bool TreeHasCachedData() const {
     NS_ASSERTION(IsRoot(), "should only be called on root of rule tree");
     return HaveChildren() || mStyleData.mInheritedData || mStyleData.mResetData;
   }
+
+  bool NodeHasCachedData(const nsStyleStructID aSID) {
+    return !!mStyleData.GetStyleData(aSID);
+  }
 };
 
 #endif
