@@ -40,7 +40,7 @@
 #include "nscore.h"
 #include "celldata.h"
 #include "nsTArray.h"
-#include "nsTPtrArray.h"
+#include "nsTArray.h"
 #include "nsRect.h"
 #include "nsCOMPtr.h"
 #include "nsAlgorithm.h"
@@ -86,7 +86,7 @@ class nsTableCellMap
 {
 public:
   nsTableCellMap(nsTableFrame&   aTableFrame,
-                 PRBool          aBorderCollapse);
+                 bool            aBorderCollapse);
 
   /** destructor
     * NOT VIRTUAL BECAUSE THIS CLASS SHOULD **NEVER** BE SUBCLASSED
@@ -115,7 +115,7 @@ public:
   nsTableCellFrame* GetCellFrame(PRInt32   aRowIndex,
                                  PRInt32   aColIndex,
                                  CellData& aData,
-                                 PRBool    aUseRowIfOverlap) const;
+                                 bool      aUseRowIfOverlap) const;
 
   /** return the CellData for the cell at (aRowIndex, aColIndex) */
   CellData* GetDataAt(PRInt32 aRowIndex,
@@ -128,29 +128,29 @@ public:
     */
   CellData* AppendCell(nsTableCellFrame&     aCellFrame,
                        PRInt32               aRowIndex,
-                       PRBool                aRebuildIfNecessary,
-                       nsRect&               aDamageArea);
+                       bool                  aRebuildIfNecessary,
+                       nsIntRect&            aDamageArea);
 
   void InsertCells(nsTArray<nsTableCellFrame*>& aCellFrames,
                    PRInt32                      aRowIndex,
                    PRInt32                      aColIndexBefore,
-                   nsRect&                      aDamageArea);
+                   nsIntRect&                   aDamageArea);
 
   void RemoveCell(nsTableCellFrame* aCellFrame,
                   PRInt32           aRowIndex,
-                  nsRect&           aDamageArea);
+                  nsIntRect&        aDamageArea);
   /** Remove the previously gathered column information */
   void ClearCols();
   void InsertRows(nsTableRowGroupFrame*       aRowGroup,
                   nsTArray<nsTableRowFrame*>& aRows,
                   PRInt32                     aFirstRowIndex,
-                  PRBool                      aConsiderSpans,
-                  nsRect&                     aDamageArea);
+                  bool                        aConsiderSpans,
+                  nsIntRect&                  aDamageArea);
 
   void RemoveRows(PRInt32         aFirstRowIndex,
                   PRInt32         aNumRowsToRemove,
-                  PRBool          aConsiderSpans,
-                  nsRect&               aDamageArea);
+                  bool            aConsiderSpans,
+                  nsIntRect&      aDamageArea);
 
   PRInt32 GetNumCellsOriginatingInRow(PRInt32 aRowIndex) const;
   PRInt32 GetNumCellsOriginatingInCol(PRInt32 aColIndex) const;
@@ -158,7 +158,7 @@ public:
   /** indicate whether the row has more than one cell that either originates
     * or is spanned from the rows above
     */
-  PRBool HasMoreThanOneCell(PRInt32 aRowIndex) const;
+  bool HasMoreThanOneCell(PRInt32 aRowIndex) const;
 
   PRInt32 GetEffectiveRowSpan(PRInt32 aRowIndex,
                               PRInt32 aColIndex) const;
@@ -173,7 +173,7 @@ public:
 
   nsTableCellFrame* GetCellInfoAt(PRInt32  aRowX,
                                   PRInt32  aColX,
-                                  PRBool*  aOriginates = nsnull,
+                                  bool*  aOriginates = nsnull,
                                   PRInt32* aColSpan = nsnull) const;
 
   /**
@@ -202,14 +202,14 @@ public:
   void AddColsAtEnd(PRUint32 aNumCols);
   void RemoveColsAtEnd();
 
-  PRBool RowIsSpannedInto(PRInt32 aRowIndex, PRInt32 aNumEffCols) const;
-  PRBool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols) const;
+  bool RowIsSpannedInto(PRInt32 aRowIndex, PRInt32 aNumEffCols) const;
+  bool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols) const;
   void RebuildConsideringCells(nsCellMap*                   aCellMap,
                                nsTArray<nsTableCellFrame*>* aCellFrames,
                                PRInt32                      aRowIndex,
                                PRInt32                      aColIndex,
-                               PRBool                       aInsert,
-                               nsRect&                      aDamageArea);
+                               bool                         aInsert,
+                               nsIntRect&                   aDamageArea);
 
 protected:
   /**
@@ -224,7 +224,7 @@ protected:
                               PRInt32                     aStartRowIndex,
                               nsTArray<nsTableRowFrame*>* aRowsToInsert,
                               PRInt32                     aNumRowsToRemove,
-                              nsRect&                     aDamageArea);
+                              nsIntRect&                  aDamageArea);
 
 public:
   void ExpandZeroColSpans();
@@ -233,7 +233,7 @@ public:
                      nsCellMap& aCellMap,
                      PRUint32   aYPos,
                      PRUint32   aXPos,
-                     PRBool     aIsLowerRight = PR_FALSE);
+                     bool       aIsLowerRight = false);
 
   void SetBCBorderEdge(mozilla::css::Side aEdge,
                        nsCellMap&    aCellMap,
@@ -243,7 +243,7 @@ public:
                        PRUint32      aLength,
                        BCBorderOwner aOwner,
                        nscoord       aSize,
-                       PRBool        aChanged);
+                       bool          aChanged);
 
   void SetBCBorderCorner(Corner      aCorner,
                          nsCellMap&  aCellMap,
@@ -252,8 +252,8 @@ public:
                          PRUint32    aXPos,
                          mozilla::css::Side aOwner,
                          nscoord     aSubSize,
-                         PRBool      aBevel,
-                         PRBool      aIsBottomRight = PR_FALSE);
+                         bool        aBevel,
+                         bool        aIsBottomRight = false);
 
   /** dump a representation of the cell map to stdout for debugging */
 #ifdef NS_DEBUG
@@ -304,14 +304,14 @@ public:
     * @param aRowGroupFrame the row group frame this is a cellmap for
     * @param aIsBC whether the table is doing border-collapse
     */
-  nsCellMap(nsTableRowGroupFrame* aRowGroupFrame, PRBool aIsBC);
+  nsCellMap(nsTableRowGroupFrame* aRowGroupFrame, bool aIsBC);
 
   /** destructor
     * NOT VIRTUAL BECAUSE THIS CLASS SHOULD **NEVER** BE SUBCLASSED
     */
   ~nsCellMap();
 
-  static nsresult Init();
+  static void Init();
   static void Shutdown();
 
   nsCellMap* GetNextSibling() const;
@@ -322,7 +322,7 @@ public:
   nsTableCellFrame* GetCellFrame(PRInt32   aRowIndex,
                                  PRInt32   aColIndex,
                                  CellData& aData,
-                                 PRBool    aUseRowSpanIfOverlap) const;
+                                 bool      aUseRowSpanIfOverlap) const;
 
   /**
    * Returns highest cell index within the cell map.
@@ -377,8 +377,9 @@ public:
   CellData* AppendCell(nsTableCellMap&   aMap,
                        nsTableCellFrame* aCellFrame,
                        PRInt32           aRowIndex,
-                       PRBool            aRebuildIfNecessary,
-                       nsRect&           aDamageArea,
+                       bool              aRebuildIfNecessary,
+                       PRInt32           aRgFirstRowIndex,
+                       nsIntRect&        aDamageArea,
                        PRInt32*          aBeginSearchAtCol = nsnull);
 
   /** Function to be called when a cell is added at a location which is spanned
@@ -401,41 +402,45 @@ public:
                    nsTArray<nsTableCellFrame*>& aCellFrames,
                    PRInt32                      aRowIndex,
                    PRInt32                      aColIndexBefore,
-                   nsRect&                      aDamageArea);
+                   PRInt32                      aRgFirstRowIndex,
+                   nsIntRect&                   aDamageArea);
 
   void RemoveCell(nsTableCellMap&   aMap,
                   nsTableCellFrame* aCellFrame,
                   PRInt32           aRowIndex,
-                  nsRect&           aDamageArea);
+                  PRInt32           aRgFirstRowIndex,
+                  nsIntRect&        aDamageArea);
 
   void InsertRows(nsTableCellMap&             aMap,
                   nsTArray<nsTableRowFrame*>& aRows,
                   PRInt32                     aFirstRowIndex,
-                  PRBool                      aConsiderSpans,
-                  nsRect&                     aDamageArea);
+                  bool                        aConsiderSpans,
+                  PRInt32                     aRgFirstRowIndex,
+                  nsIntRect&                  aDamageArea);
 
   void RemoveRows(nsTableCellMap& aMap,
                   PRInt32         aFirstRowIndex,
                   PRInt32         aNumRowsToRemove,
-                  PRBool          aConsiderSpans,
-                  nsRect&         aDamageArea);
+                  bool            aConsiderSpans,
+                  PRInt32         aRgFirstRowIndex,
+                  nsIntRect&      aDamageArea);
 
   PRInt32 GetNumCellsOriginatingInRow(PRInt32 aRowIndex) const;
   PRInt32 GetNumCellsOriginatingInCol(PRInt32 aColIndex) const;
 
   /** return the number of rows in the table represented by this CellMap */
-  PRInt32 GetRowCount(PRBool aConsiderDeadRowSpanRows = PR_FALSE) const;
+  PRInt32 GetRowCount(bool aConsiderDeadRowSpanRows = false) const;
 
   nsTableCellFrame* GetCellInfoAt(const nsTableCellMap& aMap,
                                   PRInt32          aRowX,
                                   PRInt32          aColX,
-                                  PRBool*          aOriginates = nsnull,
+                                  bool*          aOriginates = nsnull,
                                   PRInt32*         aColSpan = nsnull) const;
 
-  PRBool RowIsSpannedInto(PRInt32 aRowIndex,
+  bool RowIsSpannedInto(PRInt32 aRowIndex,
                           PRInt32 aNumEffCols) const;
 
-  PRBool RowHasSpanningCells(PRInt32 aRowIndex,
+  bool RowHasSpanningCells(PRInt32 aRowIndex,
                              PRInt32 aNumEffCols) const;
 
   void ExpandZeroColSpans(nsTableCellMap& aMap);
@@ -443,7 +448,7 @@ public:
   /** indicate whether the row has more than one cell that either originates
    * or is spanned from the rows above
    */
-  PRBool HasMoreThanOneCell(PRInt32 aRowIndex) const;
+  bool HasMoreThanOneCell(PRInt32 aRowIndex) const;
 
   /* Get the rowspan for a cell starting at aRowIndex and aColIndex.
    * If aGetEffective is true the size will not exceed the last content based
@@ -453,18 +458,18 @@ public:
    */
   PRInt32 GetRowSpan(PRInt32 aRowIndex,
                      PRInt32 aColIndex,
-                     PRBool  aGetEffective) const;
+                     bool    aGetEffective) const;
 
   PRInt32 GetEffectiveColSpan(const nsTableCellMap& aMap,
                               PRInt32     aRowIndex,
                               PRInt32     aColIndex,
-                              PRBool&     aIsZeroColSpan) const;
+                              bool&     aIsZeroColSpan) const;
 
-  typedef nsTPtrArray<CellData> CellDataArray;
+  typedef nsTArray<CellData*> CellDataArray;
 
   /** dump a representation of the cell map to stdout for debugging */
 #ifdef NS_DEBUG
-  void Dump(PRBool aIsBorderCollapse) const;
+  void Dump(bool aIsBorderCollapse) const;
 #endif
 
 protected:
@@ -478,7 +483,7 @@ protected:
    * Increase the number of rows in this cellmap by aNumRows.  Put the
    * new rows at aRowIndex.  If aRowIndex is -1, put them at the end.
    */
-  PRBool Grow(nsTableCellMap& aMap,
+  bool Grow(nsTableCellMap& aMap,
               PRInt32         aNumRows,
               PRInt32         aRowIndex = -1);
 
@@ -499,26 +504,30 @@ protected:
   void ExpandWithRows(nsTableCellMap&             aMap,
                       nsTArray<nsTableRowFrame*>& aRowFrames,
                       PRInt32                     aStartRowIndex,
-                      nsRect&                     aDamageArea);
+                      PRInt32                     aRgFirstRowIndex,
+                      nsIntRect&                  aDamageArea);
 
   void ExpandWithCells(nsTableCellMap&              aMap,
                        nsTArray<nsTableCellFrame*>& aCellFrames,
                        PRInt32                      aRowIndex,
                        PRInt32                      aColIndex,
                        PRInt32                      aRowSpan,
-                       PRBool                       aRowSpanIsZero,
-                       nsRect&                      aDamageArea);
+                       bool                         aRowSpanIsZero,
+                       PRInt32                      aRgFirstRowIndex,
+                       nsIntRect&                   aDamageArea);
 
   void ShrinkWithoutRows(nsTableCellMap& aMap,
                          PRInt32         aFirstRowIndex,
                          PRInt32         aNumRowsToRemove,
-                         nsRect&         aDamageArea);
+                         PRInt32         aRgFirstRowIndex,
+                         nsIntRect&      aDamageArea);
 
   void ShrinkWithoutCell(nsTableCellMap&   aMap,
                          nsTableCellFrame& aCellFrame,
                          PRInt32           aRowIndex,
                          PRInt32           aColIndex,
-                         nsRect&           aDamageArea);
+                         PRInt32           aRgFirstRowIndex,
+                         nsIntRect&        aDamageArea);
 
   /**
    * Rebuild due to rows being inserted or deleted with cells spanning
@@ -531,18 +540,16 @@ protected:
   void RebuildConsideringRows(nsTableCellMap&             aMap,
                               PRInt32                     aStartRowIndex,
                               nsTArray<nsTableRowFrame*>* aRowsToInsert,
-                              PRInt32                     aNumRowsToRemove,
-                              nsRect&                     aDamageArea);
+                              PRInt32                     aNumRowsToRemove);
 
   void RebuildConsideringCells(nsTableCellMap&              aMap,
                                PRInt32                      aNumOrigCols,
                                nsTArray<nsTableCellFrame*>* aCellFrames,
                                PRInt32                      aRowIndex,
                                PRInt32                      aColIndex,
-                               PRBool                       aInsert,
-                               nsRect&                      aDamageArea);
+                               bool                         aInsert);
 
-  PRBool CellsSpanOut(nsTArray<nsTableRowFrame*>& aNewRows) const;
+  bool CellsSpanOut(nsTArray<nsTableRowFrame*>& aNewRows) const;
 
   /** If a cell spans out of the area defined by aStartRowIndex, aEndRowIndex
     * and aStartColIndex, aEndColIndex the cellmap changes are more severe so
@@ -555,7 +562,7 @@ protected:
     * @return               - true if a cell span crosses the border of the
                               region
     */
-  PRBool CellsSpanInOrOut(PRInt32 aStartRowIndex,
+  bool CellsSpanInOrOut(PRInt32 aStartRowIndex,
                           PRInt32 aEndRowIndex,
                           PRInt32 aStartColIndex,
                           PRInt32 aEndColIndex) const;
@@ -563,15 +570,15 @@ protected:
   void ExpandForZeroSpan(nsTableCellFrame* aCellFrame,
                          PRInt32           aNumColsInTable);
 
-  PRBool CreateEmptyRow(PRInt32 aRowIndex,
+  bool CreateEmptyRow(PRInt32 aRowIndex,
                         PRInt32 aNumCols);
 
   PRInt32 GetRowSpanForNewCell(nsTableCellFrame* aCellFrameToAdd,
                                PRInt32           aRowIndex,
-                               PRBool&           aIsZeroRowSpan) const;
+                               bool&           aIsZeroRowSpan) const;
 
   PRInt32 GetColSpanForNewCell(nsTableCellFrame& aCellFrameToAdd,
-                               PRBool&           aIsZeroColSpan) const;
+                               bool&           aIsZeroColSpan) const;
 
   // Destroy a CellData struct.  This will handle the case of aData
   // actually being a BCCellData properly.
@@ -600,7 +607,7 @@ protected:
   nsCellMap* mNextSibling;
 
   // Whether this is a BC cellmap or not
-  PRBool mIsBC;
+  bool mIsBC;
 
   // Prescontext to deallocate and allocate celldata
   nsRefPtr<nsPresContext> mPresContext;
@@ -695,7 +702,7 @@ inline nsTableRowGroupFrame* nsCellMap::GetRowGroup() const
   return mRowGroupFrame;
 }
 
-inline PRInt32 nsCellMap::GetRowCount(PRBool aConsiderDeadRowSpanRows) const
+inline PRInt32 nsCellMap::GetRowCount(bool aConsiderDeadRowSpanRows) const
 {
   PRInt32 rowCount = (aConsiderDeadRowSpanRows) ? mRows.Length() : mContentRowCount;
   return rowCount;
