@@ -38,7 +38,7 @@
 #define nsTableFrame_h__
 
 #include "nscore.h"
-#include "nsHTMLContainerFrame.h"
+#include "nsContainerFrame.h"
 #include "nsStyleCoord.h"
 #include "nsStyleConsts.h"
 #include "nsITableLayout.h"
@@ -58,8 +58,9 @@ class nsStyleContext;
 
 struct nsTableReflowState;
 struct nsStylePosition;
+struct BCPropertyData;
 
-static inline PRBool IS_TABLE_CELL(nsIAtom* frameType) {
+static inline bool IS_TABLE_CELL(nsIAtom* frameType) {
   return nsGkAtoms::tableCellFrame == frameType ||
     nsGkAtoms::bcTableCellFrame == frameType;
 }
@@ -69,9 +70,9 @@ class nsDisplayTableItem : public nsDisplayItem
 public:
   nsDisplayTableItem(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame) : 
       nsDisplayItem(aBuilder, aFrame),
-      mPartHasFixedBackground(PR_FALSE) {}
+      mPartHasFixedBackground(false) {}
 
-  virtual PRBool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder,
+  virtual bool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder,
                                                 nsIFrame* aFrame);
   // With collapsed borders, parts of the collapsed border can extend outside
   // the table part frames, so allow this display element to blow out to our
@@ -82,7 +83,7 @@ public:
   void UpdateForFrameBackground(nsIFrame* aFrame);
 
 private:
-  PRPackedBool mPartHasFixedBackground;
+  bool mPartHasFixedBackground;
 };
 
 class nsAutoPushCurrentTableItem
@@ -127,7 +128,7 @@ private:
   * The principal child list contains row group frames. There is also an
   * additional child list, kColGroupList, which contains the col group frames.
   */
-class nsTableFrame : public nsHTMLContainerFrame, public nsITableLayout
+class nsTableFrame : public nsContainerFrame, public nsITableLayout
 {
 public:
   NS_DECL_QUERYFRAME
@@ -154,7 +155,7 @@ public:
 
   // Return true if aParentReflowState.frame or any of its ancestors within
   // the containing table have non-auto height. (e.g. pct or fixed height)
-  static PRBool AncestorsHaveStyleHeight(const nsHTMLReflowState& aParentReflowState);
+  static bool AncestorsHaveStyleHeight(const nsHTMLReflowState& aParentReflowState);
 
   // See if a special height reflow will occur due to having a pct height when
   // the pct height basis may not yet be valid.
@@ -164,11 +165,9 @@ public:
   // height reflow will occur. 
   static void RequestSpecialHeightReflow(const nsHTMLReflowState& aReflowState);
 
-  virtual PRBool IsContainingBlock() const;
-
   static void RePositionViews(nsIFrame* aFrame);
 
-  static PRBool PageBreakAfter(nsIFrame* aSourceFrame,
+  static bool PageBreakAfter(nsIFrame* aSourceFrame,
                                nsIFrame* aNextFrame);
 
   nsPoint GetFirstSectionOrigin(const nsHTMLReflowState& aReflowState) const;
@@ -195,6 +194,7 @@ public:
 
   virtual nsMargin GetUsedBorder() const;
   virtual nsMargin GetUsedPadding() const;
+  virtual nsMargin GetUsedMargin() const;
 
   // Get the offset from the border box to the area where the row groups fit
   nsMargin GetChildAreaOffset(const nsHTMLReflowState* aReflowState) const;
@@ -231,12 +231,12 @@ public:
   static nsIFrame* GetFrameAtOrBefore(nsIFrame*       aParentFrame,
                                       nsIFrame*       aPriorChildFrame,
                                       nsIAtom*        aChildType);
-  PRBool IsAutoHeight();
+  bool IsAutoHeight();
   
-  /** @return PR_TRUE if aDisplayType represents a rowgroup of any sort
+  /** @return true if aDisplayType represents a rowgroup of any sort
     * (header, footer, or body)
     */
-  PRBool IsRowGroup(PRInt32 aDisplayType) const;
+  bool IsRowGroup(PRInt32 aDisplayType) const;
 
   /** Initialize the table frame with a set of children.
     * @see nsIFrame::SetInitialChildList 
@@ -292,8 +292,8 @@ public:
 
   friend class nsDelayedCalcBCBorders;
   
-  void SetBCDamageArea(const nsRect& aValue);
-  PRBool BCRecalcNeeded(nsStyleContext* aOldStyleContext,
+  void AddBCDamageArea(const nsIntRect& aValue);
+  bool BCRecalcNeeded(nsStyleContext* aOldStyleContext,
                         nsStyleContext* aNewStyleContext);
   void PaintBCBorders(nsRenderingContext& aRenderingContext,
                       const nsRect&        aDirtyRect);
@@ -309,11 +309,11 @@ public:
   virtual nsSize ComputeSize(nsRenderingContext *aRenderingContext,
                              nsSize aCBSize, nscoord aAvailableWidth,
                              nsSize aMargin, nsSize aBorder, nsSize aPadding,
-                             PRBool aShrinkWrap);
+                             bool aShrinkWrap);
   virtual nsSize ComputeAutoSize(nsRenderingContext *aRenderingContext,
                                  nsSize aCBSize, nscoord aAvailableWidth,
                                  nsSize aMargin, nsSize aBorder,
-                                 nsSize aPadding, PRBool aShrinkWrap);
+                                 nsSize aPadding, bool aShrinkWrap);
   /**
    * A copy of nsFrame::ShrinkWidthToFit that calls a different
    * GetPrefWidth, since tables have two different ones.
@@ -403,7 +403,7 @@ public:
   /** indicate whether the row has more than one cell that either originates
     * or is spanned from the rows above
     */
-  PRBool HasMoreThanOneCell(PRInt32 aRowIndex) const;
+  bool HasMoreThanOneCell(PRInt32 aRowIndex) const;
 
   /** return the column frame associated with aColIndex
     * returns nsnull if the col frame has not yet been allocated, or if
@@ -434,7 +434,7 @@ public:
   void AppendAnonymousColFrames(nsTableColGroupFrame* aColGroupFrame,
                                 PRInt32               aNumColsToAdd,
                                 nsTableColType        aColType,
-                                PRBool                aAddToTable);
+                                bool                  aAddToTable);
 
   void MatchCellMapToColCache(nsTableCellMap* aCellMap);
   /** empty the column frame cache */
@@ -459,11 +459,11 @@ public:
   PRInt32 InsertRows(nsTableRowGroupFrame*       aRowGroupFrame,
                      nsTArray<nsTableRowFrame*>& aFrames,
                      PRInt32                     aRowIndex,
-                     PRBool                      aConsiderSpans);
+                     bool                        aConsiderSpans);
 
   virtual void RemoveRows(nsTableRowFrame& aFirstRowFrame,
                           PRInt32          aNumRowsToRemove,
-                          PRBool           aConsiderSpans);
+                          bool             aConsiderSpans);
 
   /** Insert multiple rowgroups into the table cellmap handling
     * @param aRowGroups - iterator that iterates over the rowgroups to insert
@@ -475,19 +475,19 @@ public:
 
   virtual void RemoveCol(nsTableColGroupFrame* aColGroupFrame,
                          PRInt32               aColIndex,
-                         PRBool                aRemoveFromCache,
-                         PRBool                aRemoveFromCellMap);
+                         bool                  aRemoveFromCache,
+                         bool                  aRemoveFromCellMap);
 
   NS_IMETHOD GetIndexByRowAndColumn(PRInt32 aRow, PRInt32 aColumn, PRInt32 *aIndex);
   NS_IMETHOD GetRowAndColumnByIndex(PRInt32 aIndex, PRInt32 *aRow, PRInt32 *aColumn);
 
-  PRBool ColumnHasCellSpacingBefore(PRInt32 aColIndex) const;
+  bool ColumnHasCellSpacingBefore(PRInt32 aColIndex) const;
 
-  PRBool HasPctCol() const;
-  void SetHasPctCol(PRBool aValue);
+  bool HasPctCol() const;
+  void SetHasPctCol(bool aValue);
 
-  PRBool HasCellSpanningPctCol() const;
-  void SetHasCellSpanningPctCol(PRBool aValue);
+  bool HasCellSpanningPctCol() const;
+  void SetHasCellSpanningPctCol(bool aValue);
 
   /**
    * To be called on a frame by its parent after setting its size/position and
@@ -504,7 +504,9 @@ public:
   static void InvalidateFrame(nsIFrame* aFrame,
                               const nsRect& aOrigRect,
                               const nsRect& aOrigVisualOverflow,
-                              PRBool aIsFirstReflow);
+                              bool aIsFirstReflow);
+
+  virtual bool UpdateOverflow();
 
 protected:
 
@@ -518,12 +520,12 @@ protected:
 
   void InitChildReflowState(nsHTMLReflowState& aReflowState);
 
-  /** implement abstract method on nsHTMLContainerFrame */
+  /** implement abstract method on nsContainerFrame */
   virtual PRIntn GetSkipSides() const;
 
 public:
-  PRBool IsRowInserted() const;
-  void   SetRowInserted(PRBool aValue);
+  bool IsRowInserted() const;
+  void   SetRowInserted(bool aValue);
 
 protected:
     
@@ -626,35 +628,35 @@ public:
   // Return the tfoot, if any
   nsTableRowGroupFrame* GetTFoot() const;
 
-  // Returns PR_TRUE if there are any cells above the row at
+  // Returns true if there are any cells above the row at
   // aRowIndex and spanning into the row at aRowIndex, the number of
   // effective columns limits the search up to that column
-  PRBool RowIsSpannedInto(PRInt32 aRowIndex, PRInt32 aNumEffCols);
+  bool RowIsSpannedInto(PRInt32 aRowIndex, PRInt32 aNumEffCols);
 
-  // Returns PR_TRUE if there is a cell originating in aRowIndex
+  // Returns true if there is a cell originating in aRowIndex
   // which spans into the next row,  the number of effective
   // columns limits the search up to that column
-  PRBool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols);
+  bool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols);
 
 protected:
 
-  PRBool HaveReflowedColGroups() const;
-  void   SetHaveReflowedColGroups(PRBool aValue);
+  bool HaveReflowedColGroups() const;
+  void   SetHaveReflowedColGroups(bool aValue);
 
 public:
-  PRBool IsBorderCollapse() const;
+  bool IsBorderCollapse() const;
 
-  PRBool NeedToCalcBCBorders() const;
-  void SetNeedToCalcBCBorders(PRBool aValue);
+  bool NeedToCalcBCBorders() const;
+  void SetNeedToCalcBCBorders(bool aValue);
 
-  PRBool NeedToCollapse() const;
-  void SetNeedToCollapse(PRBool aValue);
+  bool NeedToCollapse() const;
+  void SetNeedToCollapse(bool aValue);
 
-  PRBool HasZeroColSpans() const;
-  void SetHasZeroColSpans(PRBool aValue);
+  bool HasZeroColSpans() const;
+  void SetHasZeroColSpans(bool aValue);
 
-  PRBool NeedColSpanExpansion() const;
-  void SetNeedColSpanExpansion(PRBool aValue);
+  bool NeedColSpanExpansion() const;
+  void SetNeedColSpanExpansion(bool aValue);
 
   /** The GeometryDirty bit is similar to the NS_FRAME_IS_DIRTY frame
     * state bit, which implies that all descendants are dirty.  The
@@ -662,9 +664,9 @@ public:
     * dirty, but resizing optimizations should still apply to the
     * contents of the individual cells.
     */
-  void SetGeometryDirty() { mBits.mGeometryDirty = PR_TRUE; }
-  void ClearGeometryDirty() { mBits.mGeometryDirty = PR_FALSE; }
-  PRBool IsGeometryDirty() const { return mBits.mGeometryDirty; }
+  void SetGeometryDirty() { mBits.mGeometryDirty = true; }
+  void ClearGeometryDirty() { mBits.mGeometryDirty = false; }
+  bool IsGeometryDirty() const { return mBits.mGeometryDirty; }
 
   /** Get the cell map for this table frame.  It is not always mCellMap.
     * Only the firstInFlow has a legit cell map
@@ -689,13 +691,16 @@ public:
 
   nsTArray<nsTableColFrame*>& GetColCache();
 
+
 protected:
 
-  void SetBorderCollapse(PRBool aValue);
+  void SetBorderCollapse(bool aValue);
 
+  BCPropertyData* GetBCProperty(bool aCreateIfNecessary = false) const;
+  void SetFullBCDamageArea();
   void CalcBCBorders();
 
-  void ExpandBCDamageArea(nsRect& aRect) const;
+  void ExpandBCDamageArea(nsIntRect& aRect) const;
 
   void SetColumnDimensions(nscoord         aHeight,
                            const nsMargin& aReflowState);
@@ -727,8 +732,8 @@ public: /* ----- Cell Map public methods ----- */
   // return the last col index which isn't of type eColAnonymousCell
   PRInt32 GetIndexOfLastRealCol();
 
-  /** returns PR_TRUE if table-layout:auto  */
-  virtual PRBool IsAutoLayout();
+  /** returns true if table-layout:auto  */
+  virtual bool IsAutoLayout();
 
   /*---------------- nsITableLayout methods ------------------------*/
   
@@ -738,7 +743,7 @@ public: /* ----- Cell Map public methods ----- */
                            PRInt32& aStartRowIndex, PRInt32& aStartColIndex, 
                            PRInt32& aRowSpan, PRInt32& aColSpan,
                            PRInt32& aActualRowSpan, PRInt32& aActualColSpan,
-                           PRBool& aIsSelected);
+                           bool& aIsSelected);
 
   /** Get the number of rows and column for a table from the frame's cellmap 
     *  Some rows may not have enough cells (the number returned is the maximum possible),
@@ -751,9 +756,9 @@ public: /* ----- Cell Map public methods ----- */
 public:
  
 #ifdef DEBUG
-  void Dump(PRBool          aDumpRows,
-            PRBool          aDumpCols, 
-            PRBool          aDumpCellMap);
+  void Dump(bool            aDumpRows,
+            bool            aDumpCols, 
+            bool            aDumpCellMap);
 #endif
 
 protected:
@@ -784,81 +789,81 @@ protected:
 };
 
 
-inline PRBool nsTableFrame::IsRowGroup(PRInt32 aDisplayType) const
+inline bool nsTableFrame::IsRowGroup(PRInt32 aDisplayType) const
 {
-  return PRBool((NS_STYLE_DISPLAY_TABLE_HEADER_GROUP == aDisplayType) ||
+  return bool((NS_STYLE_DISPLAY_TABLE_HEADER_GROUP == aDisplayType) ||
                 (NS_STYLE_DISPLAY_TABLE_FOOTER_GROUP == aDisplayType) ||
                 (NS_STYLE_DISPLAY_TABLE_ROW_GROUP    == aDisplayType));
 }
 
-inline void nsTableFrame::SetHaveReflowedColGroups(PRBool aValue)
+inline void nsTableFrame::SetHaveReflowedColGroups(bool aValue)
 {
   mBits.mHaveReflowedColGroups = aValue;
 }
 
-inline PRBool nsTableFrame::HaveReflowedColGroups() const
+inline bool nsTableFrame::HaveReflowedColGroups() const
 {
-  return (PRBool)mBits.mHaveReflowedColGroups;
+  return (bool)mBits.mHaveReflowedColGroups;
 }
 
-inline PRBool nsTableFrame::HasPctCol() const
+inline bool nsTableFrame::HasPctCol() const
 {
-  return (PRBool)mBits.mHasPctCol;
+  return (bool)mBits.mHasPctCol;
 }
 
-inline void nsTableFrame::SetHasPctCol(PRBool aValue)
+inline void nsTableFrame::SetHasPctCol(bool aValue)
 {
   mBits.mHasPctCol = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::HasCellSpanningPctCol() const
+inline bool nsTableFrame::HasCellSpanningPctCol() const
 {
-  return (PRBool)mBits.mCellSpansPctCol;
+  return (bool)mBits.mCellSpansPctCol;
 }
 
-inline void nsTableFrame::SetHasCellSpanningPctCol(PRBool aValue)
+inline void nsTableFrame::SetHasCellSpanningPctCol(bool aValue)
 {
   mBits.mCellSpansPctCol = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::IsRowInserted() const
+inline bool nsTableFrame::IsRowInserted() const
 {
-  return (PRBool)mBits.mRowInserted;
+  return (bool)mBits.mRowInserted;
 }
 
-inline void nsTableFrame::SetRowInserted(PRBool aValue)
+inline void nsTableFrame::SetRowInserted(bool aValue)
 {
   mBits.mRowInserted = (unsigned)aValue;
 }
 
-inline void nsTableFrame::SetNeedToCollapse(PRBool aValue)
+inline void nsTableFrame::SetNeedToCollapse(bool aValue)
 {
   static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::NeedToCollapse() const
+inline bool nsTableFrame::NeedToCollapse() const
 {
-  return (PRBool) static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse;
+  return (bool) static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse;
 }
 
-inline void nsTableFrame::SetHasZeroColSpans(PRBool aValue)
+inline void nsTableFrame::SetHasZeroColSpans(bool aValue)
 {
   mBits.mHasZeroColSpans = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::HasZeroColSpans() const
+inline bool nsTableFrame::HasZeroColSpans() const
 {
-  return (PRBool)mBits.mHasZeroColSpans;
+  return (bool)mBits.mHasZeroColSpans;
 }
 
-inline void nsTableFrame::SetNeedColSpanExpansion(PRBool aValue)
+inline void nsTableFrame::SetNeedColSpanExpansion(bool aValue)
 {
   mBits.mNeedColSpanExpansion = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::NeedColSpanExpansion() const
+inline bool nsTableFrame::NeedColSpanExpansion() const
 {
-  return (PRBool)mBits.mNeedColSpanExpansion;
+  return (bool)mBits.mNeedColSpanExpansion;
 }
 
 
@@ -872,22 +877,22 @@ inline nsTArray<nsTableColFrame*>& nsTableFrame::GetColCache()
   return mColFrames;
 }
 
-inline PRBool nsTableFrame::IsBorderCollapse() const
+inline bool nsTableFrame::IsBorderCollapse() const
 {
-  return (PRBool)mBits.mIsBorderCollapse;
+  return (bool)mBits.mIsBorderCollapse;
 }
 
-inline void nsTableFrame::SetBorderCollapse(PRBool aValue) 
+inline void nsTableFrame::SetBorderCollapse(bool aValue) 
 {
   mBits.mIsBorderCollapse = aValue;
 }
 
-inline PRBool nsTableFrame::NeedToCalcBCBorders() const
+inline bool nsTableFrame::NeedToCalcBCBorders() const
 {
-  return (PRBool)mBits.mNeedToCalcBCBorders;
+  return (bool)mBits.mNeedToCalcBCBorders;
 }
 
-inline void nsTableFrame::SetNeedToCalcBCBorders(PRBool aValue)
+inline void nsTableFrame::SetNeedToCalcBCBorders(bool aValue)
 {
   mBits.mNeedToCalcBCBorders = (unsigned)aValue;
 }
@@ -911,12 +916,12 @@ public:
   nsTableIterator(nsFrameList& aSource);
   nsIFrame* First();
   nsIFrame* Next();
-  PRBool    IsLeftToRight();
+  bool      IsLeftToRight();
   PRInt32   Count();
 
 protected:
   void Init(nsIFrame* aFirstChild);
-  PRBool    mLeftToRight;
+  bool      mLeftToRight;
   nsIFrame* mFirstListChild;
   nsIFrame* mFirstChild;
   nsIFrame* mCurrentChild;
@@ -924,11 +929,11 @@ protected:
 };
 
 #define ABORT0() \
-{NS_ASSERTION(PR_FALSE, "CellIterator program error"); \
+{NS_ASSERTION(false, "CellIterator program error"); \
 return;}
 
 #define ABORT1(aReturn) \
-{NS_ASSERTION(PR_FALSE, "CellIterator program error"); \
+{NS_ASSERTION(false, "CellIterator program error"); \
 return aReturn;} 
 
 #endif
