@@ -52,7 +52,6 @@
 #define READTYPE  PRInt32
 #include "zlib.h"
 #include "nsISupportsUtils.h"
-#include "nsRecyclingAllocator.h"
 #include "prio.h"
 #include "plstr.h"
 #include "prlog.h"
@@ -66,12 +65,6 @@
 #include <windows.h>
 #endif
 
-/**
- * Global allocator used with zlib. Destroyed in module shutdown.
- */
-#define NBUCKETS 6
-nsRecyclingAllocator *gZlibAllocator = NULL;
-
 // For placement new used for arena allocations of zip file list
 #include NEW_H
 #define ZIP_ARENABLOCKSIZE (1*1024)
@@ -118,46 +111,14 @@ static nsresult ResolveSymlink(const char *path);
 #endif
 
 //***********************************************************
-// Allocators for use with zlib
-//
-// Use a recycling allocator, for re-use of of the zlib buffers.
 // For every inflation the following allocations are done:
-// zlibAlloc(1, 9520)
-// zlibAlloc(32768, 1)
+// malloc(1 * 9520)
+// malloc(32768 * 1)
 //***********************************************************
 
-static void *
-zlibAlloc(void *opaque, uInt items, uInt size)
-{
-  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
-  if (zallocator) {
-    return gZlibAllocator->Malloc(items * size);
-  }
-  return malloc(items * size);
-}
-
-static void
-zlibFree(void *opaque, void *ptr)
-{
-  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
-  if (zallocator)
-    zallocator->Free(ptr);
-  else
-    free(ptr);
-}
-
 nsresult gZlibInit(z_stream *zs)
 {
   memset(zs, 0, sizeof(z_stream));
-  //-- ensure we have our zlib allocator for better performance
-  if (!gZlibAllocator) {
-    gZlibAllocator = new nsRecyclingAllocator(NBUCKETS, NS_DEFAULT_RECYCLE_TIMEOUT, "libjar");
-  }
-  if (gZlibAllocator) {
-    zs->zalloc = zlibAlloc;
-    zs->zfree = zlibFree;
-    zs->opaque = gZlibAllocator;
-  }
   int zerr = inflateInit2(zs, -MAX_WBITS);
   if (zerr != Z_OK) return NS_ERROR_OUT_OF_MEMORY;
 
@@ -206,7 +167,7 @@ nsresult nsZipHandle::Init(nsILocalFile *file, nsZipHandle **ret)
   }
 
   handle->mMap = map;
-  handle->mFile = file;
+  handle->mFile.Init(file);
   handle->mLen = (PRUint32) size;
   handle->mFileData = buf;
   *ret = handle.forget().get();
@@ -228,6 +189,7 @@ nsresult nsZipHandle::Init(nsZipArchive *zip, const char *entry,
     return NS_ERROR_UNEXPECTED;
 
   handle->mMap = nsnull;
+  handle->mFile.Init(zip, entry);
   handle->mLen = handle->mBuf->Length();
   handle->mFileData = handle->mBuf->Buffer();
   *ret = handle.forget().get();
@@ -279,7 +241,8 @@ nsresult nsZipArchive::OpenArchive(nsZipHandle *aZipHandle)
     logFile->Create(nsIFile::DIRECTORY_TYPE, 0700);
 
     nsAutoString name;
-    aZipHandle->mFile->GetLeafName(name);
+    nsCOMPtr<nsILocalFile> file = aZipHandle->mFile.GetBaseFile();
+    file->GetLeafName(name);
     name.Append(NS_LITERAL_STRING(".log"));
     logFile->Append(name);
 
@@ -792,8 +755,9 @@ PRInt64 nsZipArchive::SizeOfMapping()
 // nsZipArchive constructor and destructor
 //------------------------------------------
 
-nsZipArchive::nsZipArchive() :
-  mBuiltSynthetics(false)
+nsZipArchive::nsZipArchive()
+  : mRefCnt(0)
+  , mBuiltSynthetics(false)
 {
   MOZ_COUNT_CTOR(nsZipArchive);
 
@@ -801,6 +765,9 @@ nsZipArchive::nsZipArchive() :
   memset(mFiles, 0, sizeof(mFiles));
 }
 
+NS_IMPL_THREADSAFE_ADDREF(nsZipArchive)
+NS_IMPL_THREADSAFE_RELEASE(nsZipArchive)
+
 nsZipArchive::~nsZipArchive()
 {
   CloseArchive();
@@ -1034,7 +1001,7 @@ nsZipCursor::~nsZipCursor()
   }
 }
 
-PRUint8* nsZipCursor::Read(PRUint32 *aBytesRead) {
+PRUint8* nsZipCursor::ReadOrCopy(PRUint32 *aBytesRead, bool aCopy) {
   int zerr;
   PRUint8 *buf = nsnull;
   bool verifyCRC = true;
@@ -1044,10 +1011,17 @@ PRUint8* nsZipCursor::Read(PRUint32 *aBytesRead) {
 MOZ_WIN_MEM_TRY_BEGIN
   switch (mItem->Compression()) {
   case STORED:
-    *aBytesRead = mZs.avail_in;
-    buf = mZs.next_in;
-    mZs.next_in += mZs.avail_in;
-    mZs.avail_in = 0;
+    if (!aCopy) {
+      *aBytesRead = mZs.avail_in;
+      buf = mZs.next_in;
+      mZs.next_in += mZs.avail_in;
+      mZs.avail_in = 0;
+    } else {
+      *aBytesRead = mZs.avail_in > mBufSize ? mBufSize : mZs.avail_in;
+      memcpy(mBuf, mZs.next_in, *aBytesRead);
+      mZs.avail_in -= *aBytesRead;
+      mZs.next_in += *aBytesRead;
+    }
     break;
   case DEFLATED:
     buf = mBuf;
