@@ -45,12 +45,8 @@
 #define PL_ARENA_CONST_ALIGN_MASK 3
 #include "plarena.h"
 
-#if defined(XP_MAC)
-  #include <stat.h>
-#else
-  #ifdef XP_OS2
-    #include <sys/types.h>
-  #endif
+#ifdef XP_OS2
+  #include <sys/types.h>
 #endif
 #ifdef _WIN32
   #include "windows.h"
@@ -72,13 +68,6 @@
 #include <os2.h>
 #endif
 
-#ifdef XP_BEOS
-#include "Alert.h"
-#endif
-
-#define BOGUS_DEFAULT_INT_PREF_VALUE (-5632)
-#define BOGUS_DEFAULT_BOOL_PREF_VALUE (-2)
-
 static void
 clearPrefEntry(PLDHashTable *table, PLDHashEntryHdr *entry)
 {
@@ -96,16 +85,16 @@ clearPrefEntry(PLDHashTable *table, PLDHashEntryHdr *entry)
     memset(entry, 0, table->entrySize);
 }
 
-static PRBool
+static bool
 matchPrefEntry(PLDHashTable*, const PLDHashEntryHdr* entry,
                const void* key)
 {
     const PrefHashEntry *prefEntry =
         static_cast<const PrefHashEntry*>(entry);
 
-    if (prefEntry->key == key) return PR_TRUE;
+    if (prefEntry->key == key) return true;
 
-    if (!prefEntry->key || !key) return PR_FALSE;
+    if (!prefEntry->key || !key) return false;
 
     const char *otherKey = reinterpret_cast<const char*>(key);
     return (strcmp(prefEntry->key, otherKey) == 0);
@@ -113,13 +102,13 @@ matchPrefEntry(PLDHashTable*, const PLDHashEntryHdr* entry,
 
 PLDHashTable        gHashTable = { nsnull };
 static PLArenaPool  gPrefNameArena;
-PRBool              gDirty = PR_FALSE;
+bool                gDirty = false;
 
 static struct CallbackNode* gCallbacks = NULL;
-static PRBool       gIsAnyPrefLocked = PR_FALSE;
+static bool         gIsAnyPrefLocked = false;
 // These are only used during the call to pref_DoCallback
-static PRBool       gCallbacksInProgress = PR_FALSE;
-static PRBool       gShouldCleanupDeadNodes = PR_FALSE;
+static bool         gCallbacksInProgress = false;
+static bool         gShouldCleanupDeadNodes = false;
 
 
 static PLDHashTableOps     pref_HashTableOps = {
@@ -167,7 +156,7 @@ static char *ArenaStrDup(const char* str, PLArenaPool* aArena)
 #define PREF_HAS_USER_VALUE(pref)       ((pref)->flags & PREF_USERSET)
 #define PREF_TYPE(pref)                 (PrefType)((pref)->flags & PREF_VALUETYPE_MASK)
 
-static PRBool pref_ValueChanged(PrefValue oldValue, PrefValue newValue, PrefType type);
+static bool pref_ValueChanged(PrefValue oldValue, PrefValue newValue, PrefType type);
 
 /* -- Privates */
 struct CallbackNode {
@@ -184,7 +173,7 @@ struct CallbackNode {
 static nsresult pref_DoCallback(const char* changed_pref);
 
 
-static nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool defaultPref);
+static nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, bool defaultPref);
 
 #define PREF_HASHTABLE_INITIAL_SIZE	2048
 
@@ -208,7 +197,7 @@ nsresult PREF_Init()
 void PREF_Cleanup()
 {
     NS_ASSERTION(!gCallbacksInProgress,
-        "PREF_Cleanup was called while gCallbacksInProgress is PR_TRUE!");
+        "PREF_Cleanup was called while gCallbacksInProgress is true!");
     struct CallbackNode* node = gCallbacks;
     struct CallbackNode* next_node;
 
@@ -285,7 +274,7 @@ static void str_escape(const char * original, nsAFlatCString& aResult)
 ** External calls
 */
 nsresult
-PREF_SetCharPref(const char *pref_name, const char *value, PRBool set_default)
+PREF_SetCharPref(const char *pref_name, const char *value, bool set_default)
 {
     PrefValue pref;
     pref.stringVal = (char*) value;
@@ -294,7 +283,7 @@ PREF_SetCharPref(const char *pref_name, const char *value, PRBool set_default)
 }
 
 nsresult
-PREF_SetIntPref(const char *pref_name, PRInt32 value, PRBool set_default)
+PREF_SetIntPref(const char *pref_name, PRInt32 value, bool set_default)
 {
     PrefValue pref;
     pref.intVal = value;
@@ -303,16 +292,16 @@ PREF_SetIntPref(const char *pref_name, PRInt32 value, PRBool set_default)
 }
 
 nsresult
-PREF_SetBoolPref(const char *pref_name, PRBool value, PRBool set_default)
+PREF_SetBoolPref(const char *pref_name, bool value, bool set_default)
 {
     PrefValue pref;
-    pref.boolVal = value ? PR_TRUE : PR_FALSE;
+    pref.boolVal = value;
 
     return pref_HashPref(pref_name, pref, PREF_BOOL, set_default);
 }
 
 nsresult
-pref_SetPrefTuple(const PrefTuple &aPref, PRBool set_default)
+pref_SetPrefTuple(const PrefTuple &aPref, bool set_default)
 {
     switch (aPref.type) {
         case PrefTuple::PREF_STRING:
@@ -347,9 +336,10 @@ pref_savePref(PLDHashTable *table, PLDHashEntryHdr *heh, PRUint32 i, void *arg)
     PrefValue* sourcePref;
 
     if (PREF_HAS_USER_VALUE(pref) &&
-        pref_ValueChanged(pref->defaultPref,
-                          pref->userPref,
-                          (PrefType) PREF_TYPE(pref))) {
+        (pref_ValueChanged(pref->defaultPref,
+                           pref->userPref,
+                           (PrefType) PREF_TYPE(pref)) ||
+         !(pref->flags & PREF_HAS_DEFAULT))) {
         sourcePref = &pref->userPref;
     } else {
         if (argData->saveTypes == SAVE_ALL_AND_DEFAULTS) {
@@ -448,53 +438,21 @@ pref_CompareStrings(const void *v1, const void *v2, void *unused)
         return strcmp(s1, s2);
 }
 
-PRBool PREF_HasUserPref(const char *pref_name)
+bool PREF_HasUserPref(const char *pref_name)
 {
     if (!gHashTable.ops)
-        return PR_FALSE;
+        return false;
 
     PrefHashEntry *pref = pref_HashTableLookup(pref_name);
-    if (!pref) return PR_FALSE;
+    if (!pref) return false;
 
     /* convert PREF_HAS_USER_VALUE to bool */
     return (PREF_HAS_USER_VALUE(pref) != 0);
 
 }
-nsresult PREF_GetCharPref(const char *pref_name, char * return_buffer, int * length, PRBool get_default)
-{
-    if (!gHashTable.ops)
-        return NS_ERROR_NOT_INITIALIZED;
-
-    nsresult rv = NS_ERROR_UNEXPECTED;
-    char* stringVal;
-
-    PrefHashEntry* pref = pref_HashTableLookup(pref_name);
-
-    if (pref)
-    {
-        if (get_default || PREF_IS_LOCKED(pref) || !PREF_HAS_USER_VALUE(pref))
-            stringVal = pref->defaultPref.stringVal;
-        else
-            stringVal = pref->userPref.stringVal;
-
-        if (stringVal)
-        {
-            if (*length <= 0)
-                *length = PL_strlen(stringVal) + 1;
-            else
-            {
-                PL_strncpy(return_buffer, stringVal, PR_MIN((size_t)*length - 1, PL_strlen(stringVal) + 1));
-                return_buffer[*length - 1] = '\0';
-            }
-            rv = NS_OK;
-        }
-    }
-
-    return rv;
-}
 
 nsresult
-PREF_CopyCharPref(const char *pref_name, char ** return_buffer, PRBool get_default)
+PREF_CopyCharPref(const char *pref_name, char ** return_buffer, bool get_default)
 {
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
@@ -518,7 +476,7 @@ PREF_CopyCharPref(const char *pref_name, char ** return_buffer, PRBool get_defau
     return rv;
 }
 
-nsresult PREF_GetIntPref(const char *pref_name,PRInt32 * return_int, PRBool get_default)
+nsresult PREF_GetIntPref(const char *pref_name,PRInt32 * return_int, bool get_default)
 {
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
@@ -531,7 +489,7 @@ nsresult PREF_GetIntPref(const char *pref_name,PRInt32 * return_int, PRBool get_
         {
             PRInt32 tempInt = pref->defaultPref.intVal;
             /* check to see if we even had a default */
-            if (tempInt == ((PRInt32) BOGUS_DEFAULT_INT_PREF_VALUE))
+            if (!(pref->flags & PREF_HAS_DEFAULT))
                 return NS_ERROR_UNEXPECTED;
             *return_int = tempInt;
         }
@@ -542,7 +500,7 @@ nsresult PREF_GetIntPref(const char *pref_name,PRInt32 * return_int, PRBool get_
     return rv;
 }
 
-nsresult PREF_GetBoolPref(const char *pref_name, PRBool * return_value, PRBool get_default)
+nsresult PREF_GetBoolPref(const char *pref_name, bool * return_value, bool get_default)
 {
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
@@ -554,9 +512,9 @@ nsresult PREF_GetBoolPref(const char *pref_name, PRBool * return_value, PRBool g
     {
         if (get_default || PREF_IS_LOCKED(pref) || !PREF_HAS_USER_VALUE(pref))
         {
-            PRBool tempBool = pref->defaultPref.boolVal;
+            bool tempBool = pref->defaultPref.boolVal;
             /* check to see if we even had a default */
-            if (tempBool != ((PRBool) BOGUS_DEFAULT_BOOL_PREF_VALUE)) {
+            if (pref->flags & PREF_HAS_DEFAULT) {
                 *return_value = tempBool;
                 rv = NS_OK;
             }
@@ -606,7 +564,7 @@ PREF_DeleteBranch(const char *branch_name)
 
     PL_DHashTableEnumerate(&gHashTable, pref_DeleteItem,
                            (void*) branch_dot.get());
-    gDirty = PR_TRUE;
+    gDirty = true;
     return NS_OK;
 }
 
@@ -617,25 +575,19 @@ PREF_ClearUserPref(const char *pref_name)
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
 
-    nsresult rv = NS_ERROR_UNEXPECTED;
     PrefHashEntry* pref = pref_HashTableLookup(pref_name);
     if (pref && PREF_HAS_USER_VALUE(pref))
     {
         pref->flags &= ~PREF_USERSET;
 
-        if ((pref->flags & PREF_INT && 
-             pref->defaultPref.intVal == ((PRInt32) BOGUS_DEFAULT_INT_PREF_VALUE)) ||
-            (pref->flags & PREF_BOOL && 
-             pref->defaultPref.boolVal == ((PRBool) BOGUS_DEFAULT_BOOL_PREF_VALUE)) ||
-            (pref->flags & PREF_STRING && !pref->defaultPref.stringVal)) {
+        if (!(pref->flags & PREF_HAS_DEFAULT)) {
             PL_DHashTableOperate(&gHashTable, pref_name, PL_DHASH_REMOVE);
         }
 
         pref_DoCallback(pref_name);
-        gDirty = PR_TRUE;
-        rv = NS_OK;
+        gDirty = true;
     }
-    return rv;
+    return NS_OK;
 }
 
 static PLDHashOperator
@@ -650,11 +602,7 @@ pref_ClearUserPref(PLDHashTable *table, PLDHashEntryHdr *he, PRUint32,
     {
         pref->flags &= ~PREF_USERSET;
 
-        if ((pref->flags & PREF_INT && 
-             pref->defaultPref.intVal == ((PRInt32) BOGUS_DEFAULT_INT_PREF_VALUE)) ||
-            (pref->flags & PREF_BOOL && 
-             pref->defaultPref.boolVal == ((PRBool) BOGUS_DEFAULT_BOOL_PREF_VALUE)) ||
-            (pref->flags & PREF_STRING && !pref->defaultPref.stringVal)) {
+        if (!(pref->flags & PREF_HAS_DEFAULT)) {
             nextOp = PL_DHASH_REMOVE;
         }
 
@@ -671,11 +619,11 @@ PREF_ClearAllUserPrefs()
 
     PL_DHashTableEnumerate(&gHashTable, pref_ClearUserPref, nsnull);
 
-    gDirty = PR_TRUE;
+    gDirty = true;
     return NS_OK;
 }
 
-nsresult PREF_LockPref(const char *key, PRBool lockit)
+nsresult PREF_LockPref(const char *key, bool lockit)
 {
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
@@ -688,7 +636,7 @@ nsresult PREF_LockPref(const char *key, PRBool lockit)
         if (!PREF_IS_LOCKED(pref))
         {
             pref->flags |= PREF_LOCKED;
-            gIsAnyPrefLocked = PR_TRUE;
+            gIsAnyPrefLocked = true;
             pref_DoCallback(key);
         }
     }
@@ -706,9 +654,9 @@ nsresult PREF_LockPref(const char *key, PRBool lockit)
 /*
  * Hash table functions
  */
-static PRBool pref_ValueChanged(PrefValue oldValue, PrefValue newValue, PrefType type)
+static bool pref_ValueChanged(PrefValue oldValue, PrefValue newValue, PrefType type)
 {
-    PRBool changed = PR_TRUE;
+    bool changed = true;
     if (type & PREF_STRING)
     {
         if (oldValue.stringVal && newValue.stringVal)
@@ -735,7 +683,7 @@ static void pref_SetValue(PrefValue* oldValue, PrefValue newValue, PrefType type
         default:
             *oldValue = newValue;
     }
-    gDirty = PR_TRUE;
+    gDirty = true;
 }
 
 PrefHashEntry* pref_HashTableLookup(const void *key)
@@ -749,7 +697,7 @@ PrefHashEntry* pref_HashTableLookup(const void *key)
     return result;
 }
 
-nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool set_default)
+nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, bool set_default)
 {
     if (!gHashTable.ops)
         return NS_ERROR_OUT_OF_MEMORY;
@@ -767,15 +715,6 @@ nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool s
         pref->key = ArenaStrDup(key, &gPrefNameArena);
         memset(&pref->defaultPref, 0, sizeof(pref->defaultPref));
         memset(&pref->userPref, 0, sizeof(pref->userPref));
-
-        /* ugly hack -- define it to a default that no pref will ever
-           default to this should really get fixed right by some out
-           of band data
-        */
-        if (pref->flags & PREF_BOOL)
-            pref->defaultPref.boolVal = (PRBool) BOGUS_DEFAULT_BOOL_PREF_VALUE;
-        if (pref->flags & PREF_INT)
-            pref->defaultPref.intVal = (PRInt32) BOGUS_DEFAULT_INT_PREF_VALUE;
     }
     else if ((((PrefType)(pref->flags)) & PREF_VALUETYPE_MASK) !=
                  (type & PREF_VALUETYPE_MASK))
@@ -784,16 +723,18 @@ nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool s
         return NS_ERROR_UNEXPECTED;
     }
 
-    PRBool valueChanged = PR_FALSE;
+    bool valueChanged = false;
     if (set_default)
     {
         if (!PREF_IS_LOCKED(pref))
         {       /* ?? change of semantics? */
-            if (pref_ValueChanged(pref->defaultPref, value, type))
+            if (pref_ValueChanged(pref->defaultPref, value, type) ||
+                !(pref->flags & PREF_HAS_DEFAULT))
             {
                 pref_SetValue(&pref->defaultPref, value, type);
+                pref->flags |= PREF_HAS_DEFAULT;
                 if (!PREF_HAS_USER_VALUE(pref))
-                    valueChanged = PR_TRUE;
+                    valueChanged = true;
             }
         }
     }
@@ -801,13 +742,14 @@ nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool s
     {
         /* If new value is same as the default value, then un-set the user value.
            Otherwise, set the user value only if it has changed */
-        if ( !pref_ValueChanged(pref->defaultPref, value, type) )
+        if (!pref_ValueChanged(pref->defaultPref, value, type) &&
+            pref->flags & PREF_HAS_DEFAULT)
         {
             if (PREF_HAS_USER_VALUE(pref))
             {
                 pref->flags &= ~PREF_USERSET;
                 if (!PREF_IS_LOCKED(pref))
-                    valueChanged = PR_TRUE;
+                    valueChanged = true;
             }
         }
         else if ( !PREF_HAS_USER_VALUE(pref) ||
@@ -816,13 +758,13 @@ nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, PRBool s
             pref_SetValue(&pref->userPref, value, type);
             pref->flags |= PREF_USERSET;
             if (!PREF_IS_LOCKED(pref))
-                valueChanged = PR_TRUE;
+                valueChanged = true;
         }
     }
 
     nsresult rv = NS_OK;
     if (valueChanged) {
-        gDirty = PR_TRUE;
+        gDirty = true;
 
         nsresult rv2 = pref_DoCallback(key);
         if (NS_FAILED(rv2))
@@ -852,14 +794,14 @@ PREF_GetPrefType(const char *pref_name)
 
 /* -- */
 
-PRBool
+bool
 PREF_PrefIsLocked(const char *pref_name)
 {
-    PRBool result = PR_FALSE;
+    bool result = false;
     if (gIsAnyPrefLocked && gHashTable.ops) {
         PrefHashEntry* pref = pref_HashTableLookup(pref_name);
         if (pref && PREF_IS_LOCKED(pref))
-            result = PR_TRUE;
+            result = true;
     }
 
     return result;
@@ -896,7 +838,7 @@ pref_RemoveCallbackNode(struct CallbackNode* node,
     NS_PRECONDITION(prev_node || gCallbacks == node, "invalid params");
 
     NS_ASSERTION(!gCallbacksInProgress,
-        "modifying the callback list while gCallbacksInProgress is PR_TRUE");
+        "modifying the callback list while gCallbacksInProgress is true");
 
     struct CallbackNode* next_node = node->next;
     if (prev_node)
@@ -929,7 +871,7 @@ PREF_UnregisterCallback(const char *pref_node,
                 // postpone the node removal until after
                 // gCallbacks enumeration is finished.
                 node->func = nsnull;
-                gShouldCleanupDeadNodes = PR_TRUE;
+                gShouldCleanupDeadNodes = true;
                 prev_node = node;
                 node = node->next;
             }
@@ -953,9 +895,9 @@ static nsresult pref_DoCallback(const char* changed_pref)
     nsresult rv = NS_OK;
     struct CallbackNode* node;
 
-    PRBool reentered = gCallbacksInProgress;
-    gCallbacksInProgress = PR_TRUE;
-    // Nodes must not be deleted while gCallbacksInProgress is PR_TRUE.
+    bool reentered = gCallbacksInProgress;
+    gCallbacksInProgress = true;
+    // Nodes must not be deleted while gCallbacksInProgress is true.
     // Nodes that need to be deleted are marked for deletion by nulling
     // out the |func| pointer. We release them at the end of this function
     // if we haven't reentered.
@@ -992,7 +934,7 @@ static nsresult pref_DoCallback(const char* changed_pref)
                 node = node->next;
             }
         }
-        gShouldCleanupDeadNodes = PR_FALSE;
+        gShouldCleanupDeadNodes = false;
     }
 
     return rv;
@@ -1002,7 +944,7 @@ void PREF_ReaderCallback(void       *closure,
                          const char *pref,
                          PrefValue   value,
                          PrefType    type,
-                         PRBool      isDefault)
+                         bool        isDefault)
 {
     pref_HashPref(pref, value, type, isDefault);
 }
