@@ -64,10 +64,9 @@ static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);
 
 nsIIDNService *nsStandardURL::gIDN = nsnull;
 nsICharsetConverterManager *nsStandardURL::gCharsetMgr = nsnull;
-PRBool nsStandardURL::gInitialized = PR_FALSE;
-PRBool nsStandardURL::gEscapeUTF8 = PR_TRUE;
-PRBool nsStandardURL::gAlwaysEncodeInUTF8 = PR_TRUE;
-PRBool nsStandardURL::gEncodeQueryInUTF8 = PR_TRUE;
+bool nsStandardURL::gInitialized = false;
+bool nsStandardURL::gEscapeUTF8 = true;
+bool nsStandardURL::gAlwaysEncodeInUTF8 = true;
 
 #if defined(PR_LOGGING)
 //
@@ -144,7 +143,6 @@ end:
 #define NS_NET_PREF_ESCAPEUTF8         "network.standard-url.escape-utf8"
 #define NS_NET_PREF_ENABLEIDN          "network.enableIDN"
 #define NS_NET_PREF_ALWAYSENCODEINUTF8 "network.standard-url.encode-utf8"
-#define NS_NET_PREF_ENCODEQUERYINUTF8  "network.standard-url.encode-query-utf8"
 
 NS_IMPL_ISUPPORTS1(nsStandardURL::nsPrefObserver, nsIObserver)
 
@@ -177,13 +175,13 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
                                      const URLSegment &seg,
                                      PRInt16 mask,
                                      nsAFlatCString &result,
-                                     PRBool &appended,
+                                     bool &appended,
                                      PRUint32 extraLen)
 {
     // extraLen is characters outside the segment that will be 
     // added when the segment is not empty (like the @ following
     // a username).
-    appended = PR_FALSE;
+    appended = false;
     if (!str)
         return 0;
     PRInt32 len = 0;
@@ -217,12 +215,12 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
         // now perform any required escaping
         if (NS_EscapeURL(str + pos, len, mask | escapeFlags, result)) {
             len = result.Length() - initLen;
-            appended = PR_TRUE;
+            appended = true;
         }
         else if (str == encBuf.get()) {
             result += encBuf; // append only!!
             len = encBuf.Length();
-            appended = PR_TRUE;
+            appended = true;
         }
         len += extraLen;
     }
@@ -235,14 +233,14 @@ nsSegmentEncoder::EncodeSegment(const nsASingleFragmentCString &str,
                                 nsAFlatCString &result)
 {
     const char *text;
-    PRBool encoded;
+    bool encoded;
     EncodeSegmentCount(str.BeginReading(text), URLSegment(0, str.Length()), mask, result, encoded);
     if (encoded)
         return result;
     return str;
 }
 
-PRBool nsStandardURL::
+bool nsStandardURL::
 nsSegmentEncoder::InitUnicodeEncoder()
 {
     NS_ASSERTION(!mEncoder, "Don't call this if we have an encoder already!");
@@ -252,7 +250,7 @@ nsSegmentEncoder::InitUnicodeEncoder()
                             &gCharsetMgr);
         if (NS_FAILED(rv)) {
             NS_ERROR("failed to get charset-converter-manager");
-            return PR_FALSE;
+            return false;
         }
     }
 
@@ -260,10 +258,10 @@ nsSegmentEncoder::InitUnicodeEncoder()
     if (NS_FAILED(rv)) {
         NS_ERROR("failed to get unicode encoder");
         mEncoder = 0; // just in case
-        return PR_FALSE;
+        return false;
     }
 
-    return PR_TRUE;
+    return true;
 }
 
 #define GET_SEGMENT_ENCODER_INTERNAL(name, useUTF8) \
@@ -273,8 +271,7 @@ nsSegmentEncoder::InitUnicodeEncoder()
     GET_SEGMENT_ENCODER_INTERNAL(name, gAlwaysEncodeInUTF8)
 
 #define GET_QUERY_ENCODER(name) \
-    GET_SEGMENT_ENCODER_INTERNAL(name, gAlwaysEncodeInUTF8 && \
-                                 gEncodeQueryInUTF8)
+    GET_SEGMENT_ENCODER_INTERNAL(name, false)
 
 //----------------------------------------------------------------------------
 // nsStandardURL <public>
@@ -284,14 +281,14 @@ nsSegmentEncoder::InitUnicodeEncoder()
 static PRCList gAllURLs;
 #endif
 
-nsStandardURL::nsStandardURL(PRBool aSupportsFileURL)
+nsStandardURL::nsStandardURL(bool aSupportsFileURL)
     : mDefaultPort(-1)
     , mPort(-1)
     , mHostA(nsnull)
     , mHostEncoding(eEncoding_ASCII)
     , mSpecEncoding(eEncoding_Unknown)
     , mURLType(URLTYPE_STANDARD)
-    , mMutable(PR_TRUE)
+    , mMutable(true)
     , mSupportsFileURL(aSupportsFileURL)
 {
 #if defined(PR_LOGGING)
@@ -302,7 +299,7 @@ nsStandardURL::nsStandardURL(PRBool aSupportsFileURL)
     LOG(("Creating nsStandardURL @%p\n", this));
 
     if (!gInitialized) {
-        gInitialized = PR_TRUE;
+        gInitialized = true;
         InitGlobalObjects();
     }
 
@@ -348,10 +345,9 @@ nsStandardURL::InitGlobalObjects()
     nsCOMPtr<nsIPrefBranch2> prefBranch( do_GetService(NS_PREFSERVICE_CONTRACTID) );
     if (prefBranch) {
         nsCOMPtr<nsIObserver> obs( new nsPrefObserver() );
-        prefBranch->AddObserver(NS_NET_PREF_ESCAPEUTF8, obs.get(), PR_FALSE);
-        prefBranch->AddObserver(NS_NET_PREF_ALWAYSENCODEINUTF8, obs.get(), PR_FALSE);
-        prefBranch->AddObserver(NS_NET_PREF_ENCODEQUERYINUTF8, obs.get(), PR_FALSE);
-        prefBranch->AddObserver(NS_NET_PREF_ENABLEIDN, obs.get(), PR_FALSE);
+        prefBranch->AddObserver(NS_NET_PREF_ESCAPEUTF8, obs.get(), false);
+        prefBranch->AddObserver(NS_NET_PREF_ALWAYSENCODEINUTF8, obs.get(), false);
+        prefBranch->AddObserver(NS_NET_PREF_ENABLEIDN, obs.get(), false);
 
         PrefsChanged(prefBranch, nsnull);
     }
@@ -407,7 +403,7 @@ nsStandardURL::Clear()
 }
 
 void
-nsStandardURL::InvalidateCache(PRBool invalidateCachedFile)
+nsStandardURL::InvalidateCache(bool invalidateCachedFile)
 {
     if (invalidateCachedFile)
         mFile = 0;
@@ -415,7 +411,7 @@ nsStandardURL::InvalidateCache(PRBool invalidateCachedFile)
     mSpecEncoding = eEncoding_Unknown;
 }
 
-PRBool
+bool
 nsStandardURL::EscapeIPv6(const char *host, nsCString &result)
 {
     // Escape IPv6 address literal by surrounding it with []'s
@@ -423,18 +419,18 @@ nsStandardURL::EscapeIPv6(const char *host, nsCString &result)
         result.Assign('[');
         result.Append(host);
         result.Append(']');
-        return PR_TRUE;
+        return true;
     }
-    return PR_FALSE;
+    return false;
 }
 
-PRBool
+bool
 nsStandardURL::NormalizeIDN(const nsCSubstring &host, nsCString &result)
 {
     // If host is ACE, then convert to UTF-8.  Else, if host is already UTF-8,
     // then make sure it is normalized per IDN.
 
-    // this function returns PR_TRUE if normalization succeeds.
+    // this function returns true if normalization succeeds.
 
     // NOTE: As a side-effect this function sets mHostEncoding.  While it would
     // be nice to avoid side-effects in this function, the implementation of
@@ -444,17 +440,17 @@ nsStandardURL::NormalizeIDN(const nsCSubstring &host, nsCString &result)
 
     NS_ASSERTION(mHostEncoding == eEncoding_ASCII, "unexpected default encoding");
 
-    PRBool isASCII;
+    bool isASCII;
     if (gIDN &&
         NS_SUCCEEDED(gIDN->ConvertToDisplayIDN(host, &isASCII, result))) {
         if (!isASCII)
           mHostEncoding = eEncoding_UTF8;
 
-        return PR_TRUE;
+        return true;
     }
 
     result.Truncate();
-    return PR_FALSE;
+    return false;
 }
 
 void
@@ -472,7 +468,7 @@ nsStandardURL::CoalescePath(netCoalesceFlags coalesceFlag, char *path)
 }
 
 PRUint32
-nsStandardURL::AppendSegmentToBuf(char *buf, PRUint32 i, const char *str, URLSegment &seg, const nsCString *escapedStr, PRBool useEscaped)
+nsStandardURL::AppendSegmentToBuf(char *buf, PRUint32 i, const char *str, URLSegment &seg, const nsCString *escapedStr, bool useEscaped)
 {
     if (seg.mLen > 0) {
         if (useEscaped) {
@@ -511,7 +507,7 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     // escaping is required).
     nsCAutoString encUsername, encPassword, encHost, encDirectory,
       encBasename, encExtension, encQuery, encRef;
-    PRBool useEncUsername, useEncPassword, useEncHost, useEncDirectory,
+    bool useEncUsername, useEncPassword, useEncHost, useEncDirectory,
       useEncBasename, useEncExtension, useEncQuery, useEncRef;
     nsCAutoString portbuf;
 
@@ -704,14 +700,14 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     return NS_OK;
 }
 
-PRBool
-nsStandardURL::SegmentIs(const URLSegment &seg, const char *val, PRBool ignoreCase)
+bool
+nsStandardURL::SegmentIs(const URLSegment &seg, const char *val, bool ignoreCase)
 {
     // one or both may be null
     if (!val || mSpec.IsEmpty())
         return (!val && (mSpec.IsEmpty() || seg.mLen < 0));
     if (seg.mLen < 0)
-        return PR_FALSE;
+        return false;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
     if (ignoreCase)
@@ -722,14 +718,14 @@ nsStandardURL::SegmentIs(const URLSegment &seg, const char *val, PRBool ignoreCa
             && (val[seg.mLen] == '\0');
 }
 
-PRBool
-nsStandardURL::SegmentIs(const char* spec, const URLSegment &seg, const char *val, PRBool ignoreCase)
+bool
+nsStandardURL::SegmentIs(const char* spec, const URLSegment &seg, const char *val, bool ignoreCase)
 {
     // one or both may be null
     if (!val || !spec)
         return (!val && (!spec || seg.mLen < 0));
     if (seg.mLen < 0)
-        return PR_FALSE;
+        return false;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
     if (ignoreCase)
@@ -740,15 +736,15 @@ nsStandardURL::SegmentIs(const char* spec, const URLSegment &seg, const char *va
             && (val[seg.mLen] == '\0');
 }
 
-PRBool
-nsStandardURL::SegmentIs(const URLSegment &seg1, const char *val, const URLSegment &seg2, PRBool ignoreCase)
+bool
+nsStandardURL::SegmentIs(const URLSegment &seg1, const char *val, const URLSegment &seg2, bool ignoreCase)
 {
     if (seg1.mLen != seg2.mLen)
-        return PR_FALSE;
+        return false;
     if (seg1.mLen == -1 || (!val && mSpec.IsEmpty()))
-        return PR_TRUE; // both are empty
+        return true; // both are empty
     if (!val)
-        return PR_FALSE;
+        return false;
     if (ignoreCase)
         return !PL_strncasecmp(mSpec.get() + seg1.mPos, val + seg2.mPos, seg1.mLen); 
     else
@@ -929,7 +925,7 @@ nsStandardURL::WriteSegment(IPC::Message *aMsg, const URLSegment &seg)
 /* static */ void
 nsStandardURL::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
 {
-    PRBool val;
+    bool val;
 
     LOG(("nsStandardURL::PrefsChanged [pref=%s]\n", pref));
 
@@ -958,12 +954,6 @@ nsStandardURL::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
             gAlwaysEncodeInUTF8 = val;
         LOG(("encode in UTF-8 %s\n", gAlwaysEncodeInUTF8 ? "enabled" : "disabled"));
     }
-
-    if (PREF_CHANGED(NS_NET_PREF_ENCODEQUERYINUTF8)) {
-        if (GOT_PREF(NS_NET_PREF_ENCODEQUERYINUTF8, val))
-            gEncodeQueryInUTF8 = val;
-        LOG(("encode query in UTF-8 %s\n", gEncodeQueryInUTF8 ? "enabled" : "disabled"));
-    }
 #undef PREF_CHANGED
 #undef GOT_PREF
 }
@@ -1108,7 +1098,7 @@ nsStandardURL::GetAsciiSpec(nsACString &result)
 
     result = Substring(mSpec, 0, mScheme.mLen + 3);
 
-    NS_EscapeURL(Userpass(PR_TRUE), esc_OnlyNonASCII | esc_AlwaysCopy, result);
+    NS_EscapeURL(Userpass(true), esc_OnlyNonASCII | esc_AlwaysCopy, result);
 
     // get escaped host
     nsCAutoString escHostport;
@@ -1311,7 +1301,7 @@ nsStandardURL::SetUserPass(const nsACString &input)
     nsCAutoString buf;
     if (usernameLen > 0) {
         GET_SEGMENT_ENCODER(encoder);
-        PRBool ignoredOut;
+        bool ignoredOut;
         usernameLen = encoder.EncodeSegmentCount(userpass.get(),
                                                  URLSegment(usernamePos,
                                                             usernameLen),
@@ -1643,13 +1633,13 @@ nsStandardURL::SetPath(const nsACString &input)
 }
 
 NS_IMETHODIMP
-nsStandardURL::Equals(nsIURI *unknownOther, PRBool *result)
+nsStandardURL::Equals(nsIURI *unknownOther, bool *result)
 {
     return EqualsInternal(unknownOther, eHonorRef, result);
 }
 
 NS_IMETHODIMP
-nsStandardURL::EqualsExceptRef(nsIURI *unknownOther, PRBool *result)
+nsStandardURL::EqualsExceptRef(nsIURI *unknownOther, bool *result)
 {
     return EqualsInternal(unknownOther, eIgnoreRef, result);
 }
@@ -1657,7 +1647,7 @@ nsStandardURL::EqualsExceptRef(nsIURI *unknownOther, PRBool *result)
 nsresult
 nsStandardURL::EqualsInternal(nsIURI *unknownOther,
                               nsStandardURL::RefHandlingEnum refHandlingMode,
-                              PRBool *result)
+                              bool *result)
 {
     NS_ENSURE_ARG_POINTER(unknownOther);
     NS_PRECONDITION(result, "null pointer");
@@ -1666,14 +1656,14 @@ nsStandardURL::EqualsInternal(nsIURI *unknownOther,
     nsresult rv = unknownOther->QueryInterface(kThisImplCID,
                                                getter_AddRefs(other));
     if (NS_FAILED(rv)) {
-        *result = PR_FALSE;
+        *result = false;
         return NS_OK;
     }
 
     // First, check whether one URIs is an nsIFileURL while the other
     // is not.  If that's the case, they're different.
     if (mSupportsFileURL != other->mSupportsFileURL) {
-        *result = PR_FALSE;
+        *result = false;
         return NS_OK;
     }
 
@@ -1689,13 +1679,13 @@ nsStandardURL::EqualsInternal(nsIURI *unknownOther,
         Port() != other->Port()) {
         // No need to compare files or other URI parts -- these are different
         // beasties
-        *result = PR_FALSE;
+        *result = false;
         return NS_OK;
     }
 
     if (refHandlingMode == eHonorRef &&
         !SegmentIs(mRef, other->mSpec.get(), other->mRef)) {
-        *result = PR_FALSE;
+        *result = false;
         return NS_OK;
     }
     
@@ -1703,7 +1693,7 @@ nsStandardURL::EqualsInternal(nsIURI *unknownOther,
     if (SegmentIs(mDirectory, other->mSpec.get(), other->mDirectory) &&
         SegmentIs(mBasename, other->mSpec.get(), other->mBasename) &&
         SegmentIs(mExtension, other->mSpec.get(), other->mExtension)) {
-        *result = PR_TRUE;
+        *result = true;
         return NS_OK;
     }
 
@@ -1714,7 +1704,7 @@ nsStandardURL::EqualsInternal(nsIURI *unknownOther,
     if (mSupportsFileURL) {
         // Assume not equal for failure cases... but failures in GetFile are
         // really failures, more or less, so propagate them to caller.
-        *result = PR_FALSE;
+        *result = false;
 
         rv = EnsureFile();
         nsresult rv2 = other->EnsureFile();
@@ -1740,13 +1730,13 @@ nsStandardURL::EqualsInternal(nsIURI *unknownOther,
 
     // The URLs are not identical, and they do not correspond to the
     // same file, so they are different.
-    *result = PR_FALSE;
+    *result = false;
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsStandardURL::SchemeIs(const char *scheme, PRBool *result)
+nsStandardURL::SchemeIs(const char *scheme, bool *result)
 {
     NS_PRECONDITION(result, "null pointer");
 
@@ -1803,7 +1793,7 @@ nsStandardURL::CloneInternal(nsStandardURL::RefHandlingEnum refHandlingMode,
     clone->mParser = mParser;
     clone->mFile = mFile;
     clone->mHostA = mHostA ? nsCRT::strdup(mHostA) : nsnull;
-    clone->mMutable = PR_TRUE;
+    clone->mMutable = true;
     clone->mSupportsFileURL = mSupportsFileURL;
     clone->mHostEncoding = mHostEncoding;
     clone->mSpecEncoding = mSpecEncoding;
@@ -1850,7 +1840,7 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
     nsresult rv;
     URLSegment scheme;
     char *resultPath = nsnull;
-    PRBool relative = PR_FALSE;
+    bool relative = false;
     PRUint32 offset = 0;
     netCoalesceFlags coalesceFlag = NET_COALESCE_NORMAL;
 
@@ -1870,7 +1860,7 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
     if (scheme.mLen >= 0) {
         // add some flags to coalesceFlag if it is an ftp-url
         // need this later on when coalescing the resulting URL
-        if (SegmentIs(relpath, scheme, "ftp", PR_TRUE)) {
+        if (SegmentIs(relpath, scheme, "ftp", true)) {
             coalesceFlag = (netCoalesceFlags) (coalesceFlag 
                                         | NET_COALESCE_ALLOW_RELATIVE_ROOT
                                         | NET_COALESCE_DOUBLE_SLASH_IS_ROOT);
@@ -1878,7 +1868,7 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
         }
         // this URL appears to be absolute
         // but try to find out more
-        if (SegmentIs(mScheme, relpath, scheme, PR_TRUE)) {
+        if (SegmentIs(mScheme, relpath, scheme, true)) {
             // mScheme and Scheme are the same 
             // but this can still be relative
             if (nsCRT::strncmp(relpath + scheme.mPos + scheme.mLen,
@@ -1890,7 +1880,7 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
                 // This is a deprecated form of relative urls like
                 // http:file or http:/path/file
                 // we will support it for now ...
-                relative = PR_TRUE;
+                relative = true;
                 offset = scheme.mLen + 1;
             }
         } else {
@@ -1911,7 +1901,7 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
             result = AppendToSubstring(mScheme.mPos, mScheme.mLen + 1, relpath);
         } else {
             // then it must be relative 
-            relative = PR_TRUE;
+            relative = true;
         }
     }
     if (relative) {
@@ -1985,7 +1975,7 @@ nsStandardURL::GetCommonBaseSpec(nsIURI *uri2, nsACString &aResult)
     NS_ENSURE_ARG_POINTER(uri2);
 
     // if uri's are equal, then return uri as is
-    PRBool isEquals = PR_FALSE;
+    bool isEquals = false;
     if (NS_SUCCEEDED(Equals(uri2, &isEquals)) && isEquals)
         return GetSpec(aResult);
 
@@ -2039,7 +2029,7 @@ nsStandardURL::GetRelativeSpec(nsIURI *uri2, nsACString &aResult)
     aResult.Truncate();
 
     // if uri's are equal, then return empty string
-    PRBool isEquals = PR_FALSE;
+    bool isEquals = false;
     if (NS_SUCCEEDED(Equals(uri2, &isEquals)) && isEquals)
         return NS_OK;
 
@@ -2066,7 +2056,7 @@ nsStandardURL::GetRelativeSpec(nsIURI *uri2, nsACString &aResult)
     thatIndex = stdurl2->mSpec.get() + mDirectory.mPos;
 
 #ifdef XP_WIN
-    PRBool isFileScheme = SegmentIs(mScheme, "file");
+    bool isFileScheme = SegmentIs(mScheme, "file");
     if (isFileScheme)
     {
         // on windows, we need to match the first segment of the path
@@ -2152,7 +2142,7 @@ nsStandardURL::GetRef(nsACString &result)
 }
 
 NS_IMETHODIMP
-nsStandardURL::GetHasRef(PRBool *result)
+nsStandardURL::GetHasRef(bool *result)
 {
     *result = (mRef.mLen >= 0);
     return NS_OK;
@@ -2317,7 +2307,7 @@ nsStandardURL::SetQuery(const nsACString &input)
 
     // encode query if necessary
     nsCAutoString buf;
-    PRBool encoded;
+    bool encoded;
     GET_QUERY_ENCODER(encoder);
     encoder.EncodeSegmentCount(query, URLSegment(0, queryLen), esc_Query,
                                buf, encoded);
@@ -2378,7 +2368,7 @@ nsStandardURL::SetRef(const nsACString &input)
 
     // encode ref if necessary
     nsCAutoString buf;
-    PRBool encoded;
+    bool encoded;
     GET_SEGMENT_ENCODER(encoder);
     encoder.EncodeSegmentCount(ref, URLSegment(0, refLen), esc_Ref,
                                buf, encoded);
@@ -2450,7 +2440,7 @@ nsStandardURL::SetFileName(const nsACString &input)
         }
         else {
             nsCAutoString newFilename;
-            PRBool ignoredOut;
+            bool ignoredOut;
             GET_SEGMENT_ENCODER(encoder);
             basename.mLen = encoder.EncodeSegmentCount(filename, basename,
                                                        esc_FileBaseName |
@@ -2615,7 +2605,7 @@ nsStandardURL::SetFile(nsIFile *file)
 // nsStandardURL::nsIStandardURL
 //----------------------------------------------------------------------------
 
-inline PRBool
+inline bool
 IsUTFCharset(const char *aCharset)
 {
     return ((aCharset[0] == 'U' || aCharset[0] == 'u') &&
@@ -2699,14 +2689,14 @@ nsStandardURL::Init(PRUint32 urlType,
 }
 
 NS_IMETHODIMP
-nsStandardURL::GetMutable(PRBool *value)
+nsStandardURL::GetMutable(bool *value)
 {
     *value = mMutable;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsStandardURL::SetMutable(PRBool value)
+nsStandardURL::SetMutable(bool value)
 {
     NS_ENSURE_ARG(mMutable || !value);
 
@@ -2799,19 +2789,19 @@ nsStandardURL::Read(nsIObjectInputStream *stream)
     rv = NS_ReadOptionalCString(stream, mOriginCharset);
     if (NS_FAILED(rv)) return rv;
 
-    PRBool isMutable;
+    bool isMutable;
     rv = stream->ReadBoolean(&isMutable);
     if (NS_FAILED(rv)) return rv;
-    if (isMutable != PR_TRUE && isMutable != PR_FALSE) {
+    if (isMutable != true && isMutable != false) {
         NS_WARNING("Unexpected boolean value");
         return NS_ERROR_UNEXPECTED;
     }
     mMutable = isMutable;
 
-    PRBool supportsFileURL;
+    bool supportsFileURL;
     rv = stream->ReadBoolean(&supportsFileURL);
     if (NS_FAILED(rv)) return rv;
-    if (supportsFileURL != PR_TRUE && supportsFileURL != PR_FALSE) {
+    if (supportsFileURL != true && supportsFileURL != false) {
         NS_WARNING("Unexpected boolean value");
         return NS_ERROR_UNEXPECTED;
     }
@@ -2923,7 +2913,7 @@ nsStandardURL::Write(nsIObjectOutputStream *stream)
 // nsStandardURL::nsIIPCSerializable
 //---------------------------------------------------------------------------
 
-PRBool
+bool
 nsStandardURL::Read(const IPC::Message *aMsg, void **aIter)
 {
     using IPC::ReadParam;
@@ -2935,7 +2925,7 @@ nsStandardURL::Read(const IPC::Message *aMsg, void **aIter)
     
     PRUint32 urlType;
     if (!ReadParam(aMsg, aIter, &urlType))
-        return PR_FALSE;
+        return false;
     
     mURLType = urlType;
     switch (mURLType) {
@@ -2950,7 +2940,7 @@ nsStandardURL::Read(const IPC::Message *aMsg, void **aIter)
             break;
         default:
             NS_NOTREACHED("bad urlType");
-            return PR_FALSE;
+            return false;
     }
 
     PRUint32 hostEncoding;
@@ -2974,11 +2964,11 @@ nsStandardURL::Read(const IPC::Message *aMsg, void **aIter)
         !ReadParam(aMsg, aIter, &isMutable) ||
         !ReadParam(aMsg, aIter, &supportsFileURL) ||
         !ReadParam(aMsg, aIter, &hostEncoding))
-        return PR_FALSE;
+        return false;
 
     if (hostEncoding != eEncoding_ASCII && hostEncoding != eEncoding_UTF8) {
         NS_WARNING("Unexpected host encoding");
-        return PR_FALSE;
+        return false;
     }
     mHostEncoding = hostEncoding;
     mMutable = isMutable;
@@ -2986,7 +2976,7 @@ nsStandardURL::Read(const IPC::Message *aMsg, void **aIter)
 
     // mSpecEncoding and mHostA are just caches that can be recovered as needed.
 
-    return PR_TRUE;
+    return true;
 }
 
 void
