@@ -308,7 +308,10 @@ nsMIMEHeaderParamImpl::DoParameterInternal(const char *aHeaderValue,
         else if (*valueEnd == '"')
           break;
       }
-      str = valueEnd + 1;
+      str = valueEnd;
+      // *valueEnd != null means that *valueEnd is quote character.
+      if (*valueEnd)
+        str++;
     }
 
     // See if this is the simplest case (case A above),
