@@ -97,7 +97,7 @@ public:
 
   // nsIRequest
   NS_IMETHOD GetName(nsACString& aName);
-  NS_IMETHOD IsPending(PRBool *aIsPending);
+  NS_IMETHOD IsPending(bool *aIsPending);
   NS_IMETHOD GetStatus(nsresult *aStatus);
   NS_IMETHOD GetLoadGroup(nsILoadGroup **aLoadGroup);
   NS_IMETHOD SetLoadGroup(nsILoadGroup *aLoadGroup);
@@ -116,13 +116,16 @@ public:
   NS_IMETHOD SetContentType(const nsACString& aContentType);
   NS_IMETHOD GetContentCharset(nsACString& aContentCharset);
   NS_IMETHOD SetContentCharset(const nsACString& aContentCharset);
+  NS_IMETHOD GetContentDisposition(PRUint32 *aContentDisposition);
+  NS_IMETHOD GetContentDispositionFilename(nsAString& aContentDispositionFilename);
+  NS_IMETHOD GetContentDispositionHeader(nsACString& aContentDispositionHeader);
   NS_IMETHOD GetContentLength(PRInt32 *aContentLength);
   NS_IMETHOD SetContentLength(PRInt32 aContentLength);
   NS_IMETHOD Open(nsIInputStream **aResult);
 
   // nsIEncodedChannel
-  NS_IMETHOD GetApplyConversion(PRBool *value);
-  NS_IMETHOD SetApplyConversion(PRBool value);
+  NS_IMETHOD GetApplyConversion(bool *value);
+  NS_IMETHOD SetApplyConversion(bool value);
   NS_IMETHOD GetContentEncodings(nsIUTF8StringEnumerator** aEncodings);
 
   // HttpBaseChannel::nsIHttpChannel
@@ -132,21 +135,21 @@ public:
   NS_IMETHOD SetReferrer(nsIURI *referrer);
   NS_IMETHOD GetRequestHeader(const nsACString& aHeader, nsACString& aValue);
   NS_IMETHOD SetRequestHeader(const nsACString& aHeader, 
-                              const nsACString& aValue, PRBool aMerge);
+                              const nsACString& aValue, bool aMerge);
   NS_IMETHOD VisitRequestHeaders(nsIHttpHeaderVisitor *visitor);
   NS_IMETHOD GetResponseHeader(const nsACString &header, nsACString &value);
   NS_IMETHOD SetResponseHeader(const nsACString& header, 
-                               const nsACString& value, PRBool merge);
+                               const nsACString& value, bool merge);
   NS_IMETHOD VisitResponseHeaders(nsIHttpHeaderVisitor *visitor);
-  NS_IMETHOD GetAllowPipelining(PRBool *value);
-  NS_IMETHOD SetAllowPipelining(PRBool value);
+  NS_IMETHOD GetAllowPipelining(bool *value);
+  NS_IMETHOD SetAllowPipelining(bool value);
   NS_IMETHOD GetRedirectionLimit(PRUint32 *value);
   NS_IMETHOD SetRedirectionLimit(PRUint32 value);
-  NS_IMETHOD IsNoStoreResponse(PRBool *value);
-  NS_IMETHOD IsNoCacheResponse(PRBool *value);
+  NS_IMETHOD IsNoStoreResponse(bool *value);
+  NS_IMETHOD IsNoCacheResponse(bool *value);
   NS_IMETHOD GetResponseStatus(PRUint32 *aValue);
   NS_IMETHOD GetResponseStatusText(nsACString& aValue);
-  NS_IMETHOD GetRequestSucceeded(PRBool *aValue);
+  NS_IMETHOD GetRequestSucceeded(bool *aValue);
 
   // nsIHttpChannelInternal
   NS_IMETHOD GetDocumentURI(nsIURI **aDocumentURI);
@@ -154,16 +157,19 @@ public:
   NS_IMETHOD GetRequestVersion(PRUint32 *major, PRUint32 *minor);
   NS_IMETHOD GetResponseVersion(PRUint32 *major, PRUint32 *minor);
   NS_IMETHOD SetCookie(const char *aCookieHeader);
-  NS_IMETHOD GetForceAllowThirdPartyCookie(PRBool *aForce);
-  NS_IMETHOD SetForceAllowThirdPartyCookie(PRBool aForce);
-  NS_IMETHOD GetCanceled(PRBool *aCanceled);
-  NS_IMETHOD GetChannelIsForDownload(PRBool *aChannelIsForDownload);
-  NS_IMETHOD SetChannelIsForDownload(PRBool aChannelIsForDownload);
+  NS_IMETHOD GetForceAllowThirdPartyCookie(bool *aForce);
+  NS_IMETHOD SetForceAllowThirdPartyCookie(bool aForce);
+  NS_IMETHOD GetCanceled(bool *aCanceled);
+  NS_IMETHOD GetChannelIsForDownload(bool *aChannelIsForDownload);
+  NS_IMETHOD SetChannelIsForDownload(bool aChannelIsForDownload);
   NS_IMETHOD SetCacheKeysRedirectChain(nsTArray<nsCString> *cacheKeys);
   NS_IMETHOD GetLocalAddress(nsACString& addr);
   NS_IMETHOD GetLocalPort(PRInt32* port);
   NS_IMETHOD GetRemoteAddress(nsACString& addr);
   NS_IMETHOD GetRemotePort(PRInt32* port);
+  NS_IMETHOD GetAllowSpdy(bool *aAllowSpdy);
+  NS_IMETHOD SetAllowSpdy(bool aAllowSpdy);
+  
   inline void CleanRedirectCacheChainIfNecessary()
   {
       if (mRedirectedCachekeys) {
@@ -202,7 +208,7 @@ public:
         // header with it.
         nsCOMPtr<nsIHttpChannel> mChannel;
         
-        PRPackedBool mReady;
+        bool mReady;
     };
 
     nsHttpResponseHead * GetResponseHead() const { return mResponseHead; }
@@ -213,6 +219,9 @@ public:
 
 public: /* Necko internal use only... */
 
+  bool ShouldRewriteRedirectToGET(PRUint32 httpStatus, nsHttpAtom method);
+  bool IsSafeMethod(nsHttpAtom method);
+  
 protected:
 
   // Handle notifying listener, removing from loadgroup if request failed.
@@ -224,7 +233,7 @@ protected:
   void AddCookiesToRequest();
   virtual nsresult SetupReplacementChannel(nsIURI *,
                                            nsIChannel *,
-                                           PRBool preserveMethod);
+                                           bool preserveMethod);
 
   // Helper function to simplify getting notification callbacks.
   template <class T>
@@ -289,6 +298,7 @@ protected:
   PRUint32                          mTracingEnabled             : 1;
   // True if timing collection is enabled
   PRUint32                          mTimingEnabled              : 1;
+  PRUint32                          mAllowSpdy                  : 1;
 
   // Current suspension depth for this channel object
   PRUint32                          mSuspendCount;
@@ -335,7 +345,7 @@ nsresult HttpAsyncAborter<T>::AsyncAbort(nsresult status)
   LOG(("HttpAsyncAborter::AsyncAbort [this=%p status=%x]\n", mThis, status));
 
   mThis->mStatus = status;
-  mThis->mIsPending = PR_FALSE;
+  mThis->mIsPending = false;
 
   // if this fails?  Callers ignore our return value anyway....
   return AsyncCall(&T::HandleAsyncAbort);
