@@ -53,6 +53,9 @@
 #include "nsProxyRelease.h"
 #include "prmem.h"
 #include "nsPreloadedStream.h"
+#include "SpdySession.h"
+#include "mozilla/Telemetry.h"
+#include "nsISupportsPriority.h"
 
 #ifdef DEBUG
 // defined by the socket transport service while active
@@ -75,12 +78,21 @@ nsHttpConnection::nsHttpConnection()
     , mConsiderReusedAfterEpoch(0)
     , mCurrentBytesRead(0)
     , mMaxBytesRead(0)
-    , mKeepAlive(PR_TRUE) // assume to keep-alive by default
-    , mKeepAliveMask(PR_TRUE)
-    , mSupportsPipelining(PR_FALSE) // assume low-grade server
-    , mIsReused(PR_FALSE)
-    , mCompletedProxyConnect(PR_FALSE)
-    , mLastTransactionExpectedNoContent(PR_FALSE)
+    , mTotalBytesRead(0)
+    , mKeepAlive(true) // assume to keep-alive by default
+    , mKeepAliveMask(true)
+    , mSupportsPipelining(false) // assume low-grade server
+    , mIsReused(false)
+    , mCompletedProxyConnect(false)
+    , mLastTransactionExpectedNoContent(false)
+    , mIdleMonitoring(false)
+    , mHttp1xTransactionCount(0)
+    , mNPNComplete(false)
+    , mSetupNPNCalled(false)
+    , mUsingSpdy(false)
+    , mPriority(nsISupportsPriority::PRIORITY_NORMAL)
+    , mReportedSpdy(false)
+    , mEverUsedSpdy(false)
 {
     LOG(("Creating nsHttpConnection @%x\n", this));
 
@@ -102,6 +114,24 @@ nsHttpConnection::~nsHttpConnection()
     // release our reference to the handler
     nsHttpHandler *handler = gHttpHandler;
     NS_RELEASE(handler);
+
+    if (!mEverUsedSpdy) {
+        LOG(("nsHttpConnection %p performed %d HTTP/1.x transactions\n",
+             this, mHttp1xTransactionCount));
+        mozilla::Telemetry::Accumulate(
+            mozilla::Telemetry::HTTP_REQUEST_PER_CONN, mHttp1xTransactionCount);
+    }
+
+    if (mTotalBytesRead) {
+        PRUint32 totalKBRead = static_cast<PRUint32>(mTotalBytesRead >> 10);
+        LOG(("nsHttpConnection %p read %dkb on connection spdy=%d\n",
+             this, totalKBRead, mEverUsedSpdy));
+        mozilla::Telemetry::Accumulate(
+            mEverUsedSpdy ?
+              mozilla::Telemetry::SPDY_KBREAD_PER_CONN :
+              mozilla::Telemetry::HTTP_KBREAD_PER_CONN,
+            totalKBRead);
+    }
 }
 
 nsresult
@@ -140,9 +170,89 @@ nsHttpConnection::Init(nsHttpConnectionInfo *info,
     return NS_OK;
 }
 
+bool
+nsHttpConnection::EnsureNPNComplete()
+{
+    // NPN is only used by SPDY right now.
+    //
+    // If for some reason the components to check on NPN aren't available,
+    // this function will just return true to continue on and disable SPDY
+
+    if (!mSocketTransport) {
+        // this cannot happen
+        NS_ABORT_IF_FALSE(false,
+                          "EnsureNPNComplete socket transport precondition");
+        mNPNComplete = true;
+        return true;
+    }
+
+    if (mNPNComplete)
+        return true;
+    
+    nsresult rv;
+
+    nsCOMPtr<nsISupports> securityInfo;
+    nsCOMPtr<nsISSLSocketControl> ssl;
+    nsCAutoString negotiatedNPN;
+    
+    rv = mSocketTransport->GetSecurityInfo(getter_AddRefs(securityInfo));
+    if (NS_FAILED(rv))
+        goto npnComplete;
+
+    ssl = do_QueryInterface(securityInfo, &rv);
+    if (NS_FAILED(rv))
+        goto npnComplete;
+
+    rv = ssl->GetNegotiatedNPN(negotiatedNPN);
+    if (rv == NS_ERROR_NOT_CONNECTED) {
+    
+        // By writing 0 bytes to the socket the SSL handshake machine is
+        // pushed forward.
+        PRUint32 count = 0;
+        rv = mSocketOut->Write("", 0, &count);
+
+        if (NS_FAILED(rv) && rv != NS_BASE_STREAM_WOULD_BLOCK)
+            goto npnComplete;
+        return false;
+    }
+    
+    if (NS_FAILED(rv))
+        goto npnComplete;
+
+    LOG(("nsHttpConnection::EnsureNPNComplete %p negotiated to '%s'",
+         this, negotiatedNPN.get()));
+    
+    if (negotiatedNPN.Equals(NS_LITERAL_CSTRING("spdy/2"))) {
+        mUsingSpdy = true;
+        mEverUsedSpdy = true;
+
+        // Setting the connection as reused allows some transactions that fail
+        // with NS_ERROR_NET_RESET to be restarted and SPDY uses that code
+        // to handle clean rejections (such as those that arrived after
+        // a server goaway was generated).
+        mIsReused = true;
+
+        // Wrap the old http transaction into the new spdy session
+        // as the first stream
+        mSpdySession = new SpdySession(mTransaction,
+                                       mSocketTransport,
+                                       mPriority);
+        mTransaction = mSpdySession;
+        mIdleTimeout = gHttpHandler->SpdyTimeout();
+    }
+
+    mozilla::Telemetry::Accumulate(mozilla::Telemetry::SPDY_NPN_CONNECT,
+                                   mUsingSpdy);
+
+npnComplete:
+    LOG(("nsHttpConnection::EnsureNPNComplete setting complete to true"));
+    mNPNComplete = true;
+    return true;
+}
+
 // called on the socket thread
 nsresult
-nsHttpConnection::Activate(nsAHttpTransaction *trans, PRUint8 caps)
+nsHttpConnection::Activate(nsAHttpTransaction *trans, PRUint8 caps, PRInt32 pri)
 {
     nsresult rv;
 
@@ -150,6 +260,10 @@ nsHttpConnection::Activate(nsAHttpTransaction *trans, PRUint8 caps)
     LOG(("nsHttpConnection::Activate [this=%x trans=%x caps=%x]\n",
          this, trans, caps));
 
+    mPriority = pri;
+    if (mTransaction && mUsingSpdy)
+        return AddTransaction(trans, pri);
+
     NS_ENSURE_ARG_POINTER(trans);
     NS_ENSURE_TRUE(!mTransaction, NS_ERROR_IN_PROGRESS);
 
@@ -165,9 +279,15 @@ nsHttpConnection::Activate(nsAHttpTransaction *trans, PRUint8 caps)
         mCallbackTarget = callbackTarget;
     }
 
+    SetupNPN(caps); // only for spdy
+
     // take ownership of the transaction
     mTransaction = trans;
 
+    NS_ABORT_IF_FALSE(!mIdleMonitoring,
+                      "Activating a connection with an Idle Monitor");
+    mIdleMonitoring = false;
+
     // set mKeepAlive according to what will be requested
     mKeepAliveMask = mKeepAlive = (caps & NS_HTTP_ALLOW_KEEPALIVE);
 
@@ -183,6 +303,9 @@ nsHttpConnection::Activate(nsAHttpTransaction *trans, PRUint8 caps)
     // Clear the per activation counter
     mCurrentBytesRead = 0;
 
+    // The overflow state is not needed between activations
+    mInputOverflow = nsnull;
+
     rv = OnOutputStreamReady(mSocketOut);
     
 failed_activation:
@@ -193,6 +316,95 @@ failed_activation:
     return rv;
 }
 
+void
+nsHttpConnection::SetupNPN(PRUint8 caps)
+{
+    if (mSetupNPNCalled)                                /* do only once */
+        return;
+    mSetupNPNCalled = true;
+
+    // Setup NPN Negotiation if necessary (only for SPDY)
+    if (!mNPNComplete) {
+
+        mNPNComplete = true;
+
+        if (mConnInfo->UsingSSL() &&
+            !(caps & NS_HTTP_DISALLOW_SPDY) &&
+            !mConnInfo->UsingHttpProxy() &&
+            gHttpHandler->IsSpdyEnabled()) {
+            LOG(("nsHttpConnection::Init Setting up SPDY Negotiation"));
+            nsCOMPtr<nsISupports> securityInfo;
+            nsresult rv =
+                mSocketTransport->GetSecurityInfo(getter_AddRefs(securityInfo));
+            if (NS_FAILED(rv))
+                return;
+
+            nsCOMPtr<nsISSLSocketControl> ssl =
+                do_QueryInterface(securityInfo, &rv);
+            if (NS_FAILED(rv))
+                return;
+
+            nsTArray<nsCString> protocolArray;
+            protocolArray.AppendElement(NS_LITERAL_CSTRING("spdy/2"));
+            protocolArray.AppendElement(NS_LITERAL_CSTRING("http/1.1"));
+            if (NS_SUCCEEDED(ssl->SetNPNList(protocolArray))) {
+                LOG(("nsHttpConnection::Init Setting up SPDY Negotiation OK"));
+                mNPNComplete = false;
+            }
+        }
+    }
+}
+
+void
+nsHttpConnection::HandleAlternateProtocol(nsHttpResponseHead *responseHead)
+{
+    // Look for the Alternate-Protocol header. Alternate-Protocol is
+    // essentially a way to rediect future transactions from http to
+    // spdy.
+    //
+
+    if (!gHttpHandler->IsSpdyEnabled() || mUsingSpdy)
+        return;
+
+    const char *val = responseHead->PeekHeader(nsHttp::Alternate_Protocol);
+    if (!val)
+        return;
+
+    // The spec allows redirections to any port, but due to concerns over
+    // silently redirecting to stealth ports we only allow port 443
+    //
+    // Alternate-Protocol: 5678:somethingelse, 443:npn-spdy/2
+
+    if (nsHttp::FindToken(val, "443:npn-spdy/2", HTTP_HEADER_VALUE_SEPS)) {
+        LOG(("Connection %p Transaction %p found Alternate-Protocol "
+             "header %s", this, mTransaction.get(), val));
+        gHttpHandler->ConnMgr()->ReportSpdyAlternateProtocol(this);
+    }
+}
+
+nsresult
+nsHttpConnection::AddTransaction(nsAHttpTransaction *httpTransaction,
+                                 PRInt32 priority)
+{
+    LOG(("nsHttpConnection::AddTransaction for SPDY"));
+
+    NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
+    NS_ABORT_IF_FALSE(mSpdySession && mUsingSpdy,
+                      "AddTransaction to live http connection without spdy");
+    NS_ABORT_IF_FALSE(mTransaction,
+                      "AddTransaction to idle http connection");
+    
+    if (!mSpdySession->AddStream(httpTransaction, priority)) {
+        NS_ABORT_IF_FALSE(0, "AddStream should never fail due to"
+                          "RoomForMore() admission check");
+        return NS_ERROR_FAILURE;
+    }
+
+    ResumeSend();
+
+    return NS_OK;
+}
+
 void
 nsHttpConnection::Close(nsresult reason)
 {
@@ -201,12 +413,15 @@ nsHttpConnection::Close(nsresult reason)
     NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
     if (NS_FAILED(reason)) {
+        if (mIdleMonitoring)
+            EndIdleMonitoring();
+
         if (mSocketTransport) {
             mSocketTransport->SetSecurityCallbacks(nsnull);
             mSocketTransport->SetEventSink(nsnull, nsnull);
             mSocketTransport->Close(reason);
         }
-        mKeepAlive = PR_FALSE;
+        mKeepAlive = false;
     }
 }
 
@@ -229,30 +444,56 @@ nsHttpConnection::ProxyStartSSL()
     return ssl->ProxyStartSSL();
 }
 
-PRBool
+void
+nsHttpConnection::DontReuse()
+{
+    mKeepAliveMask = false;
+    mKeepAlive = false;
+    mIdleTimeout = 0;
+    if (mSpdySession)
+        mSpdySession->DontReuse();
+}
+
+bool
 nsHttpConnection::CanReuse()
 {
-    PRBool canReuse = IsKeepAlive() &&
+    bool canReuse;
+    
+    if (mUsingSpdy)
+        canReuse = mSpdySession->CanReuse();
+    else
+        canReuse = IsKeepAlive();
+    
+    canReuse = canReuse &&
         (NowInSeconds() - mLastReadTime < mIdleTimeout) &&
         IsAlive();
-    
+
     // An idle persistent connection should not have data waiting to be read
     // before a request is sent. Data here is likely a 408 timeout response
     // which we would deal with later on through the restart logic, but that
-    // path is more expensive than just closing the socket now. SSL check can
-    // be removed with fixing of 631801
+    // path is more expensive than just closing the socket now.
 
     PRUint32 dataSize;
-    if (canReuse && mSocketIn && !mConnInfo->UsingSSL() &&
+    if (canReuse && mSocketIn && !mUsingSpdy &&
         NS_SUCCEEDED(mSocketIn->Available(&dataSize)) && dataSize) {
         LOG(("nsHttpConnection::CanReuse %p %s"
              "Socket not reusable because read data pending (%d) on it.\n",
              this, mConnInfo->Host(), dataSize));
-        canReuse = PR_FALSE;
+        canReuse = false;
     }
     return canReuse;
 }
 
+bool
+nsHttpConnection::CanDirectlyActivate()
+{
+    // return true if a new transaction can be addded to ths connection at any
+    // time through Activate(). In practice this means this is a healthy SPDY
+    // connection with room for more concurrent streams.
+    
+    return UsingSpdy() && CanReuse() && mSpdySession->RoomForMoreStreams();
+}
+
 PRUint32 nsHttpConnection::TimeToLive()
 {
     PRInt32 tmp = mIdleTimeout - (NowInSeconds() - mLastReadTime);
@@ -262,38 +503,46 @@ PRUint32 nsHttpConnection::TimeToLive()
     return tmp;
 }
 
-PRBool
+bool
 nsHttpConnection::IsAlive()
 {
     if (!mSocketTransport)
-        return PR_FALSE;
+        return false;
 
-    PRBool alive;
+    // SocketTransport::IsAlive can run the SSL state machine, so make sure
+    // the NPN options are set before that happens.
+    SetupNPN(0);
+
+    bool alive;
     nsresult rv = mSocketTransport->IsAlive(&alive);
     if (NS_FAILED(rv))
-        alive = PR_FALSE;
+        alive = false;
 
 //#define TEST_RESTART_LOGIC
 #ifdef TEST_RESTART_LOGIC
     if (!alive) {
         LOG(("pretending socket is still alive to test restart logic\n"));
-        alive = PR_TRUE;
+        alive = true;
     }
 #endif
 
     return alive;
 }
 
-PRBool
+bool
 nsHttpConnection::SupportsPipelining(nsHttpResponseHead *responseHead)
 {
+    // SPDY supports infinite parallelism, so no need to pipeline.
+    if (mUsingSpdy)
+        return false;
+
     // XXX there should be a strict mode available that disables this
     // blacklisting.
 
     // assuming connection is HTTP/1.1 with keep-alive enabled
     if (mConnInfo->UsingHttpProxy() && !mConnInfo->UsingSSL()) {
         // XXX check for bad proxy servers...
-        return PR_TRUE;
+        return true;
     }
 
     // XXX what about checking for a Via header? (transparent proxies)
@@ -301,13 +550,13 @@ nsHttpConnection::SupportsPipelining(nsHttpResponseHead *responseHead)
     // check for bad origin servers
     const char *val = responseHead->PeekHeader(nsHttp::Server);
     if (!val)
-        return PR_FALSE; // no header, no love
+        return false; // no header, no love
 
     // The blacklist is indexed by the first character. All of these servers are
     // known to return their identifier as the first thing in the server string,
     // so we can do a leading match. 
 
-    static const char *bad_servers[26][5] = {
+    static const char *bad_servers[26][6] = {
         { nsnull }, { nsnull }, { nsnull }, { nsnull },                 // a - d
         { "EFAServer/", nsnull },                                       // e
         { nsnull }, { nsnull }, { nsnull }, { nsnull },                 // f - i
@@ -317,7 +566,8 @@ nsHttpConnection::SupportsPipelining(nsHttpResponseHead *responseHead)
           "Netscape-Enterprise/5.", "Netscape-Enterprise/6.", nsnull }, // n
         { nsnull }, { nsnull }, { nsnull }, { nsnull },                 // o - r
         { nsnull }, { nsnull }, { nsnull }, { nsnull },                 // s - v
-        { "WebLogic 3.", "WebLogic 4.","WebLogic 5.", "WebLogic 6.", nsnull }, // w 
+        { "WebLogic 3.", "WebLogic 4.","WebLogic 5.", "WebLogic 6.",
+          "Winstone Servlet Engine v0.", nsnull },                      // w 
         { nsnull }, { nsnull }, { nsnull }                              // x - z
     };  
 
@@ -327,13 +577,13 @@ nsHttpConnection::SupportsPipelining(nsHttpResponseHead *responseHead)
         for (int i = 0; bad_servers[index][i] != nsnull; i++) {
             if (!PL_strncmp (val, bad_servers[index][i], strlen (bad_servers[index][i]))) {
                 LOG(("looks like this server does not support pipelining"));
-                return PR_FALSE;
+                return false;
             }
         }
     }
 
     // ok, let's allow pipelining to this server
-    return PR_TRUE;
+    return true;
 }
 
 //----------------------------------------------------------------------------
@@ -344,7 +594,7 @@ nsresult
 nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
                                      nsHttpRequestHead *requestHead,
                                      nsHttpResponseHead *responseHead,
-                                     PRBool *reset)
+                                     bool *reset)
 {
     LOG(("nsHttpConnection::OnHeadersAvailable [this=%p trans=%p response-head=%p]\n",
         this, trans, responseHead));
@@ -359,7 +609,7 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
     // response headers so that it will be ready to receive the new response.
     if (responseHead->Status() == 408) {
         Close(NS_ERROR_NET_RESET);
-        *reset = PR_TRUE;
+        *reset = true;
         return NS_OK;
     }
 
@@ -373,22 +623,22 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
         val = responseHead->PeekHeader(nsHttp::Proxy_Connection);
 
     // reset to default (the server may have changed since we last checked)
-    mSupportsPipelining = PR_FALSE;
+    mSupportsPipelining = false;
 
     if ((responseHead->Version() < NS_HTTP_VERSION_1_1) ||
         (requestHead->Version() < NS_HTTP_VERSION_1_1)) {
         // HTTP/1.0 connections are by default NOT persistent
         if (val && !PL_strcasecmp(val, "keep-alive"))
-            mKeepAlive = PR_TRUE;
+            mKeepAlive = true;
         else
-            mKeepAlive = PR_FALSE;
+            mKeepAlive = false;
     }
     else {
         // HTTP/1.1 connections are by default persistent
         if (val && !PL_strcasecmp(val, "close")) 
-            mKeepAlive = PR_FALSE;
+            mKeepAlive = false;
         else {
-            mKeepAlive = PR_TRUE;
+            mKeepAlive = true;
 
             // Do not support pipelining when we are establishing
             // an SSL tunnel though an HTTP proxy. Pipelining support
@@ -411,32 +661,42 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
     if (mKeepAlive) {
         val = responseHead->PeekHeader(nsHttp::Keep_Alive);
 
-        const char *cp = PL_strcasestr(val, "timeout=");
-        if (cp)
-            mIdleTimeout = (PRUint32) atoi(cp + 8);
-        else
-            mIdleTimeout = gHttpHandler->IdleTimeout();
+        if (!mUsingSpdy) {
+            const char *cp = PL_strcasestr(val, "timeout=");
+            if (cp)
+                mIdleTimeout = (PRUint32) atoi(cp + 8);
+            else
+                mIdleTimeout = gHttpHandler->IdleTimeout();
+        }
+        else {
+            mIdleTimeout = gHttpHandler->SpdyTimeout();
+        }
         
         LOG(("Connection can be reused [this=%x idle-timeout=%u]\n", this, mIdleTimeout));
     }
 
+    if (!mProxyConnectStream)
+        HandleAlternateProtocol(responseHead);
+
     // if we're doing an SSL proxy connect, then we need to check whether or not
     // the connect was successful.  if so, then we have to reset the transaction
     // and step-up the socket connection to SSL. finally, we have to wake up the
     // socket write request.
     if (mProxyConnectStream) {
+        NS_ABORT_IF_FALSE(!mUsingSpdy,
+                          "SPDY NPN Complete while using proxy connect stream");
         mProxyConnectStream = 0;
         if (responseHead->Status() == 200) {
             LOG(("proxy CONNECT succeeded! ssl=%s\n",
                  mConnInfo->UsingSSL() ? "true" :"false"));
-            *reset = PR_TRUE;
+            *reset = true;
             nsresult rv;
             if (mConnInfo->UsingSSL()) {
                 rv = ProxyStartSSL();
                 if (NS_FAILED(rv)) // XXX need to handle this for real
                     LOG(("ProxyStartSSL failed [rv=%x]\n", rv));
             }
-            mCompletedProxyConnect = PR_TRUE;
+            mCompletedProxyConnect = true;
             rv = mSocketOut->AsyncWait(this, 0, 0, nsnull);
             // XXX what if this fails -- need to handle this error
             NS_ASSERTION(NS_SUCCEEDED(rv), "mSocketOut->AsyncWait failed");
@@ -471,13 +731,13 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
     return NS_OK;
 }
 
-PRBool
+bool
 nsHttpConnection::IsReused()
 {
     if (mIsReused)
-        return PR_TRUE;
+        return true;
     if (!mConsiderReusedAfterInterval)
-        return PR_FALSE;
+        return false;
     
     // ReusedAfter allows a socket to be consider reused only after a certain
     // interval of time has passed
@@ -497,6 +757,8 @@ nsHttpConnection::TakeTransport(nsISocketTransport  **aTransport,
                                 nsIAsyncInputStream **aInputStream,
                                 nsIAsyncOutputStream **aOutputStream)
 {
+    if (mUsingSpdy)
+        return NS_ERROR_FAILURE;
     if (mTransaction && !mTransaction->IsDone())
         return NS_ERROR_IN_PROGRESS;
     if (!(mSocketTransport && mSocketIn && mSocketOut))
@@ -571,6 +833,35 @@ nsHttpConnection::ResumeRecv()
     return NS_ERROR_UNEXPECTED;
 }
 
+void
+nsHttpConnection::BeginIdleMonitoring()
+{
+    LOG(("nsHttpConnection::BeginIdleMonitoring [this=%p]\n", this));
+    NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
+    NS_ABORT_IF_FALSE(!mTransaction, "BeginIdleMonitoring() while active");
+    NS_ABORT_IF_FALSE(!mUsingSpdy, "Idle monitoring of spdy not allowed");
+
+    LOG(("Entering Idle Monitoring Mode [this=%p]", this));
+    mIdleMonitoring = true;
+    if (mSocketIn)
+        mSocketIn->AsyncWait(this, 0, 0, nsnull);
+}
+
+void
+nsHttpConnection::EndIdleMonitoring()
+{
+    LOG(("nsHttpConnection::EndIdleMonitoring [this=%p]\n", this));
+    NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
+    NS_ABORT_IF_FALSE(!mTransaction, "EndIdleMonitoring() while active");
+
+    if (mIdleMonitoring) {
+        LOG(("Leaving Idle Monitoring Mode [this=%p]", this));
+        mIdleMonitoring = false;
+        if (mSocketIn)
+            mSocketIn->AsyncWait(nsnull, 0, 0, nsnull);
+    }
+}
+
 //-----------------------------------------------------------------------------
 // nsHttpConnection <private>
 //-----------------------------------------------------------------------------
@@ -591,15 +882,30 @@ nsHttpConnection::CloseTransaction(nsAHttpTransaction *trans, nsresult reason)
     if (reason == NS_BASE_STREAM_CLOSED)
         reason = NS_OK;
 
+    if (mUsingSpdy) {
+        DontReuse();
+        // if !mSpdySession then mUsingSpdy must be false for canreuse()
+        mUsingSpdy = false;
+        mSpdySession = nsnull;
+    }
+
+    mHttp1xTransactionCount += mTransaction->Http1xTransactionCount();
+
     mTransaction->Close(reason);
     mTransaction = nsnull;
 
+    if (mCallbacks) {
+        nsIInterfaceRequestor *cbs = nsnull;
+        mCallbacks.swap(cbs);
+        NS_ProxyRelease(mCallbackTarget, cbs);
+    }
+
     if (NS_FAILED(reason))
         Close(reason);
 
     // flag the connection as reused here for convenience sake.  certainly
     // it might be going away instead ;-)
-    mIsReused = PR_TRUE;
+    mIsReused = true;
 }
 
 NS_METHOD
@@ -646,9 +952,11 @@ nsHttpConnection::OnSocketWritable()
 
     nsresult rv;
     PRUint32 n;
-    PRBool again = PR_TRUE;
+    bool again = true;
 
     do {
+        mSocketOutCondition = NS_OK;
+
         // if we're doing an SSL proxy connect, then we need to bypass calling
         // into the transaction.
         //
@@ -662,7 +970,25 @@ nsHttpConnection::OnSocketWritable()
                                                       nsIOService::gDefaultSegmentSize,
                                                       &n);
         }
+        else if (!EnsureNPNComplete()) {
+            // When SPDY is disabled this branch is not executed because Activate()
+            // sets mNPNComplete to true in that case.
+
+            // We are ready to proceed with SSL but the handshake is not done.
+            // When using NPN to negotiate between HTTPS and SPDY, we need to
+            // see the results of the handshake to know what bytes to send, so
+            // we cannot proceed with the request headers.
+
+            rv = NS_OK;
+            mSocketOutCondition = NS_BASE_STREAM_WOULD_BLOCK;
+            n = 0;
+        }
         else {
+            if (!mReportedSpdy) {
+                mReportedSpdy = true;
+                gHttpHandler->ConnMgr()->ReportSpdyConnection(this, mUsingSpdy);
+            }
+
             LOG(("  writing transaction request stream\n"));
             rv = mTransaction->ReadSegments(this, nsIOService::gDefaultSegmentSize, &n);
         }
@@ -681,14 +1007,14 @@ nsHttpConnection::OnSocketWritable()
             // wait for the transaction to call ResumeSend.
             if (rv == NS_BASE_STREAM_WOULD_BLOCK)
                 rv = NS_OK;
-            again = PR_FALSE;
+            again = false;
         }
         else if (NS_FAILED(mSocketOutCondition)) {
             if (mSocketOutCondition == NS_BASE_STREAM_WOULD_BLOCK)
                 rv = mSocketOut->AsyncWait(this, 0, 0, nsnull); // continue writing
             else
                 rv = mSocketOutCondition;
-            again = PR_FALSE;
+            again = false;
         }
         else if (n == 0) {
             // 
@@ -702,7 +1028,7 @@ nsHttpConnection::OnSocketWritable()
                                             LL_ZERO);
 
             rv = mSocketIn->AsyncWait(this, 0, 0, nsnull); // start reading
-            again = PR_FALSE;
+            again = false;
         }
         // write more to the socket until error or end-of-request...
     } while (again);
@@ -745,14 +1071,14 @@ nsHttpConnection::OnSocketReadable()
         LOG(("max hang time exceeded!\n"));
         // give the handler a chance to create a new persistent connection to
         // this host if we've been busy for too long.
-        mKeepAliveMask = PR_FALSE;
+        mKeepAliveMask = false;
         gHttpHandler->ProcessPendingQ(mConnInfo);
     }
     mLastReadTime = now;
 
     nsresult rv;
     PRUint32 n;
-    PRBool again = PR_TRUE;
+    bool again = true;
 
     do {
         rv = mTransaction->WriteSegments(this, nsIOService::gDefaultSegmentSize, &n);
@@ -761,17 +1087,18 @@ nsHttpConnection::OnSocketReadable()
             // wait for the transaction to call ResumeRecv.
             if (rv == NS_BASE_STREAM_WOULD_BLOCK)
                 rv = NS_OK;
-            again = PR_FALSE;
+            again = false;
         }
         else {
             mCurrentBytesRead += n;
+            mTotalBytesRead += n;
             if (NS_FAILED(mSocketInCondition)) {
                 // continue waiting for the socket if necessary...
                 if (mSocketInCondition == NS_BASE_STREAM_WOULD_BLOCK)
                     rv = mSocketIn->AsyncWait(this, 0, 0, nsnull);
                 else
                     rv = mSocketInCondition;
-                again = PR_FALSE;
+                again = false;
             }
         }
         // read more from the socket until error...
@@ -788,6 +1115,8 @@ nsHttpConnection::SetupProxyConnect()
     LOG(("nsHttpConnection::SetupProxyConnect [this=%x]\n", this));
 
     NS_ENSURE_TRUE(!mProxyConnectStream, NS_ERROR_ALREADY_INITIALIZED);
+    NS_ABORT_IF_FALSE(!mUsingSpdy,
+                      "SPDY NPN Complete while using proxy connect stream");
 
     nsCAutoString buf;
     nsresult rv = nsHttpHandler::GenerateHostPort(
@@ -820,7 +1149,7 @@ nsHttpConnection::SetupProxyConnect()
     }
 
     buf.Truncate();
-    request.Flatten(buf, PR_FALSE);
+    request.Flatten(buf, false);
     buf.AppendLiteral("\r\n");
 
     return NS_NewCStringInputStream(getter_AddRefs(mProxyConnectStream), buf);
@@ -847,6 +1176,25 @@ nsHttpConnection::OnInputStreamReady(nsIAsyncInputStream *in)
     NS_ASSERTION(in == mSocketIn, "unexpected stream");
     NS_ASSERTION(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
+    if (mIdleMonitoring) {
+        NS_ABORT_IF_FALSE(!mTransaction, "Idle Input Event While Active");
+
+        // The only read event that is protocol compliant for an idle connection
+        // is an EOF, which we check for with CanReuse(). If the data is
+        // something else then just ignore it and suspend checking for EOF -
+        // our normal timers or protocol stack are the place to deal with
+        // any exception logic.
+
+        if (!CanReuse()) {
+            LOG(("Server initiated close of idle conn %p\n", this));
+            gHttpHandler->ConnMgr()->CloseIdleConnection(this);
+            return NS_OK;
+        }
+
+        LOG(("Input data on idle conn %p, but not closing yet\n", this));
+        return NS_OK;
+    }
+
     // if the transaction was dropped...
     if (!mTransaction) {
         LOG(("  no transaction; ignoring event\n"));
