@@ -101,6 +101,7 @@ public:
     bool           SendSecureXSiteReferrer() { return mSendSecureXSiteReferrer; }
     PRUint8        RedirectionLimit()        { return mRedirectionLimit; }
     PRUint16       IdleTimeout()             { return mIdleTimeout; }
+    PRUint16       SpdyTimeout()             { return mSpdyTimeout; }
     PRUint16       MaxRequestAttempts()      { return mMaxRequestAttempts; }
     const char    *DefaultSocketType()       { return mDefaultSocketType.get(); /* ok to return null */ }
     nsIIDNService *IDNConverter()            { return mIDNConverter; }
@@ -112,6 +113,11 @@ public:
 
     bool           IsPersistentHttpsCachingEnabled() { return mEnablePersistentHttpsCaching; }
 
+    bool           IsSpdyEnabled() { return mEnableSpdy; }
+    bool           CoalesceSpdy() { return mCoalesceSpdy; }
+    bool           UseAlternateProtocol() { return mUseAlternateProtocol; }
+    PRUint32       SpdySendingChunkSize() { return mSpdySendingChunkSize; }
+
     bool           PromptTempRedirect()      { return mPromptTempRedirect; }
 
     nsHttpAuthCache     *AuthCache() { return &mAuthCache; }
@@ -264,6 +270,7 @@ private:
     bool mFastFallbackToIPv4;
 
     PRUint16 mIdleTimeout;
+    PRUint16 mSpdyTimeout;
     PRUint16 mMaxRequestAttempts;
     PRUint16 mMaxRequestDelay;
     PRUint16 mIdleSynTimeout;
@@ -314,6 +321,7 @@ private:
     nsXPIDLCString mAppName;
     nsXPIDLCString mAppVersion;
     nsCString      mCompatFirefox;
+    nsXPIDLCString mCompatDevice;
 
     nsCString      mUserAgent;
     nsXPIDLCString mUserAgentOverride;
@@ -331,6 +339,12 @@ private:
 
     // For broadcasting the preference to not be tracked
     bool           mDoNotTrackEnabled;
+    
+    // Try to use SPDY features instead of HTTP/1.1 over SSL
+    bool           mEnableSpdy;
+    bool           mCoalesceSpdy;
+    bool           mUseAlternateProtocol;
+    PRUint32       mSpdySendingChunkSize;
 };
 
 //-----------------------------------------------------------------------------
