@@ -1026,127 +1026,3 @@ nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,
 void DebugDumpContainmentRules(nsIDTD& theDTD,const char* aFilename,const char* aTitle) {
 }
 #endif
-
-/**************************************************************
-  This defines the topic object used by the observer service.
-  The observerService uses a list of these, 1 per topic when
-  registering tags.
- **************************************************************/
-NS_IMPL_ISUPPORTS1(nsObserverEntry, nsIObserverEntry)
-
-nsObserverEntry::nsObserverEntry(const nsAString& aTopic) : mTopic(aTopic) 
-{
-  memset(mObservers, 0, sizeof(mObservers));
-}
-
-nsObserverEntry::~nsObserverEntry() {
-  for (PRInt32 i = 0; i <= NS_HTML_TAG_MAX; ++i){
-    delete mObservers[i];
-  }
-}
-
-NS_IMETHODIMP
-nsObserverEntry::Notify(nsIParserNode* aNode,
-                        nsIParser* aParser,
-                        nsISupports* aDocShell,
-                        const PRUint32 aFlags) 
-{
-  NS_ENSURE_ARG_POINTER(aNode);
-  NS_ENSURE_ARG_POINTER(aParser);
-
-  nsresult result = NS_OK;
-  eHTMLTags theTag = (eHTMLTags)aNode->GetNodeType();
- 
-  if (theTag <= NS_HTML_TAG_MAX) {
-    nsCOMArray<nsIElementObserver>* theObservers = mObservers[theTag];
-    if (theObservers) {
-      PRInt32   theCharsetSource;
-      nsCAutoString      charset;
-      aParser->GetDocumentCharset(charset,theCharsetSource);
-      NS_ConvertASCIItoUTF16 theCharsetValue(charset);
-
-      PRInt32 theAttrCount = aNode->GetAttributeCount(); 
-      PRInt32 theObserversCount = theObservers->Count();
-      if (0 < theObserversCount){
-        nsTArray<nsString> keys(theAttrCount + 4), values(theAttrCount + 4);
-
-        // XXX this and the following code may be a performance issue.
-        // Every key and value is copied and added to an voidarray (causing at
-        // least 2 allocations for mImpl, usually more, plus at least 1 per
-        // string (total = 2*(keys+3) + 2(or more) array allocations )).
-        PRInt32 index;
-        for (index = 0; index < theAttrCount; ++index) {
-          keys.AppendElement(aNode->GetKeyAt(index));
-          values.AppendElement(aNode->GetValueAt(index));
-        } 
-
-        nsAutoString intValue;
-
-        keys.AppendElement(NS_LITERAL_STRING("charset")); 
-        values.AppendElement(theCharsetValue);       
-      
-        keys.AppendElement(NS_LITERAL_STRING("charsetSource")); 
-        intValue.AppendInt(PRInt32(theCharsetSource),10);
-        values.AppendElement(intValue); 
-
-        keys.AppendElement(NS_LITERAL_STRING("X_COMMAND"));
-        values.AppendElement(NS_LITERAL_STRING("text/html")); 
-
-        nsCOMPtr<nsIChannel> channel;
-        aParser->GetChannel(getter_AddRefs(channel));
-
-        for (index=0;index<theObserversCount;++index) {
-          nsIElementObserver* observer = theObservers->ObjectAt(index);
-          if (observer) {
-            result = observer->Notify(aDocShell, channel,
-                                      nsHTMLTags::GetStringValue(theTag),
-                                      &keys, &values, aFlags);
-            if (NS_FAILED(result)) {
-              break;
-            }
-
-            if (result == NS_HTMLPARSER_VALID_META_CHARSET) {
-              // Inform the parser that this meta tag contained a valid
-              // charset. See bug 272815
-              aParser->SetDocumentCharset(charset, kCharsetFromMetaTag);
-              result = NS_OK;
-            }
-          }
-        } 
-      } 
-    }
-  }
-  return result;
-}
-
-bool 
-nsObserverEntry::Matches(const nsAString& aString) {
-  bool result = aString.Equals(mTopic);
-  return result;
-}
-
-nsresult
-nsObserverEntry::AddObserver(nsIElementObserver *aObserver,
-                             eHTMLTags aTag) 
-{
-  if (aObserver) {
-    if (!mObservers[aTag]) {
-      mObservers[aTag] = new nsCOMArray<nsIElementObserver>();
-      if (!mObservers[aTag]) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
-    mObservers[aTag]->AppendObject(aObserver);
-  }
-  return NS_OK;
-}
-
-void 
-nsObserverEntry::RemoveObserver(nsIElementObserver *aObserver)
-{
-  for (PRInt32 i=0; i <= NS_HTML_TAG_MAX; ++i){
-    if (mObservers[i]) {
-      mObservers[i]->RemoveObject(aObserver);
-    }
-  }
-}
