@@ -126,7 +126,8 @@ RESTRequest.prototype = {
 
   QueryInterface: XPCOMUtils.generateQI([
     Ci.nsIBadCertListener2,
-    Ci.nsIInterfaceRequestor
+    Ci.nsIInterfaceRequestor,
+    Ci.nsIChannelEventSink
   ]),
 
   /*** Public API: ***/
@@ -364,6 +365,11 @@ RESTRequest.prototype = {
     this.abort();
     let error = Components.Exception("Aborting due to channel inactivity.",
                                      Cr.NS_ERROR_NET_TIMEOUT);
+    if (!this.onComplete) {
+      this._log.error("Unexpected error: onComplete not defined in " +
+                      "abortTimeout.")
+      return;
+    }
     this.onComplete(error);
   },
 
@@ -374,9 +380,18 @@ RESTRequest.prototype = {
       this._log.trace("Not proceeding with onStartRequest, request was aborted.");
       return;
     }
+
+    try {
+      channel.QueryInterface(Ci.nsIHttpChannel);
+    } catch (ex) {
+      this._log.error("Unexpected error: channel is not a nsIHttpChannel!");
+      this.status = this.ABORTED;
+      channel.cancel(Cr.NS_BINDING_ABORTED);
+      return;
+    }
+
     this.status = this.IN_PROGRESS;
 
-    channel.QueryInterface(Ci.nsIHttpChannel);
     this._log.trace("onStartRequest: " + channel.requestMethod + " " +
                     channel.URI.spec);
 
@@ -404,16 +419,32 @@ RESTRequest.prototype = {
       this._log.trace("Not proceeding with onStopRequest, request was aborted.");
       return;
     }
+
+    try {
+      channel.QueryInterface(Ci.nsIHttpChannel);
+    } catch (ex) {
+      this._log.error("Unexpected error: channel not nsIHttpChannel!");
+      this.status = this.ABORTED;
+      return;
+    }
     this.status = this.COMPLETED;
 
+    let statusSuccess = Components.isSuccessCode(statusCode);
     let uri = channel && channel.URI && channel.URI.spec || "<unknown>";
     this._log.trace("Channel for " + channel.requestMethod + " " + uri +
                     " returned status code " + statusCode);
 
+    if (!this.onComplete) {
+      this._log.error("Unexpected error: onComplete not defined in " +
+                      "abortRequest.");
+      this.onProgress = null;
+      return;
+    }
+
     // Throw the failure code and stop execution.  Use Components.Exception()
     // instead of Error() so the exception is QI-able and can be passed across
     // XPCOM borders while preserving the status code.
-    if (!Components.isSuccessCode(statusCode)) {
+    if (!statusSuccess) {
       let message = Components.Exception("", statusCode).name;
       let error = Components.Exception(message, statusCode);
       this.onComplete(error);
@@ -452,6 +483,14 @@ RESTRequest.prototype = {
                      this.method + " " + req.URI.spec);
       this._log.debug("Exception: " + Utils.exceptionStr(ex));
       this.abort();
+
+      if (!this.onComplete) {
+        this._log.error("Unexpected error: onComplete not defined in " +
+                        "onDataAvailable.");
+        this.onProgress = null;
+        return;
+      }
+
       this.onComplete(ex);
       this.onComplete = this.onProgress = null;
       return;
@@ -473,6 +512,24 @@ RESTRequest.prototype = {
     // Suppress invalid HTTPS certificate warnings in the UI.
     // (The request will still fail.)
     return true;
+  },
+
+  /*** nsIChannelEventSink ***/
+  asyncOnChannelRedirect:
+    function asyncOnChannelRedirect(oldChannel, newChannel, flags, callback) {
+
+    try {
+      newChannel.QueryInterface(Ci.nsIHttpChannel);
+    } catch (ex) {
+      this._log.error("Unexpected error: channel not nsIHttpChannel!");
+      callback.onRedirectVerifyCallback(Cr.NS_ERROR_NO_INTERFACE);
+      return;
+    }
+
+    this.channel = newChannel;
+
+    // We let all redirects proceed.
+    callback.onRedirectVerifyCallback(Cr.NS_OK);
   }
 };
 
