@@ -6,6 +6,7 @@
  * here must be in strict JSON format, as it will get parsed by the Python
  * testrunner (no single quotes, extra comma's, etc).
  */
+EnableEngines(["tabs"]);
 
 var phases = { "phase1": "profile1",
                "phase2": "profile2",
@@ -42,7 +43,7 @@ var tabs2 = [
 
 Phase('phase1', [
   [Tabs.add, tabs1],
-  [Sync, SYNC_WIPE_SERVER],
+  [Sync]
 ]);
 
 Phase('phase2', [
