@@ -47,17 +47,21 @@
 #include "nsICollation.h"
 #include "nsIFile.h"
 #include "nsIObserver.h"
+#include "nsTArray.h"
 #include "mozilla/Mutex.h"
 
 #include "mozIStorageService.h"
 #include "mozIStorageServiceQuotaManagement.h"
 
+class nsIMemoryReporter;
+class nsIMemoryMultiReporter;
 class nsIXPConnect;
 struct sqlite3_vfs;
 
 namespace mozilla {
 namespace storage {
 
+class Connection;
 class Service : public mozIStorageService
               , public nsIObserver
               , public mozIStorageServiceQuotaManagement
@@ -103,6 +107,41 @@ public:
    */
   static PRInt32 getSynchronousPref();
 
+  /**
+   * Registers the connection with the storage service.  Connections are
+   * registered so they can be iterated over.
+   *
+   * @pre mRegistrationMutex is not held
+   *
+   * @param  aConnection
+   *         The connection to register.
+   */
+  void registerConnection(Connection *aConnection);
+
+  /**
+   * Unregisters the connection with the storage service.
+   *
+   * @pre mRegistrationMutex is not held
+   *
+   * @param  aConnection
+   *         The connection to unregister.
+   */
+  void unregisterConnection(Connection *aConnection);
+
+  /**
+   * Gets the list of open connections.  Note that you must test each
+   * connection with mozIStorageConnection::connectionReady before doing
+   * anything with it, and skip it if it's not ready.
+   *
+   * @pre mRegistrationMutex is not held
+   *
+   * @param  aConnections
+   *         An inout param;  it is cleared and the connections are appended to
+   *         it.
+   * @return The open connections.
+   */
+  void getConnections(nsTArray<nsRefPtr<Connection> >& aConnections);
+
 private:
   Service();
   virtual ~Service();
@@ -116,6 +155,17 @@ private:
   
   sqlite3_vfs *mSqliteVFS;
 
+  /**
+   * Protects mConnections.
+   */
+  Mutex mRegistrationMutex;
+
+  /**
+   * The list of connections we have created.  Modifications to it are
+   * protected by |mRegistrationMutex|.
+   */
+  nsTArray<nsRefPtr<Connection> > mConnections;
+
   /**
    * Shuts down the storage service, freeing all of the acquired resources.
    */
@@ -141,11 +191,16 @@ private:
 
   nsCOMPtr<nsIFile> mProfileStorageFile;
 
+  nsCOMPtr<nsIMemoryReporter> mStorageSQLiteReporter;
+  nsCOMPtr<nsIMemoryMultiReporter> mStorageSQLiteMultiReporter;
+
   static Service *gService;
 
   static nsIXPConnect *sXPConnect;
 
   static PRInt32 sSynchronousPref;
+
+  friend class ServiceMainThreadInitializer;
 };
 
 } // namespace storage
