@@ -24,6 +24,7 @@
  *   Dean Tessman <dean_tessman@hotmail.com>
  *   Johnny Stenback <jst@mozilla.jstenback.com>
  *   Masayuki Nakano <masayuki@d-toybox.com>
+ *   Michael Ventnor <m.ventnor@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -81,7 +82,6 @@ nsAutoCompleteController::nsAutoCompleteController() :
   mPopupClosedByCompositionStart(false),
   mIsIMEComposing(false),
   mIgnoreHandleText(false),
-  mIsOpen(false),
   mSearchStatus(nsAutoCompleteController::STATUS_NONE),
   mRowCount(0),
   mSearchesOngoing(0),
@@ -131,8 +131,7 @@ nsAutoCompleteController::SetInput(nsIAutoCompleteInput *aInput)
     // Stop all searches in case they are async.
     StopSearch();
     ClearResults();
-    if (mIsOpen)
-      ClosePopup();
+    ClosePopup();
     mSearches.Clear();
   }
 
@@ -192,6 +191,11 @@ nsAutoCompleteController::StartSearch(const nsAString &aSearchString)
 NS_IMETHODIMP
 nsAutoCompleteController::HandleText()
 {
+  // We should do nothing during composition.
+  if (mIsIMEComposing) {
+    return NS_OK;
+  }
+
   if (!mInput) {
     // Stop all searches in case they are async.
     StopSearch();
@@ -520,7 +524,7 @@ nsAutoCompleteController::HandleKeyNavigation(PRUint32 aKey, bool *_retval)
         nsAutoString value;
         nsAutoString inputValue;
         input->GetTextValue(inputValue);
-        if (NS_SUCCEEDED(GetDefaultCompleteValue(selectedIndex, false, value)) &&
+        if (NS_SUCCEEDED(GetDefaultCompleteValue(-1, false, value)) &&
             value.Equals(inputValue, nsCaseInsensitiveStringComparator())) {
           input->SetTextValue(value);
           input->SelectTextRange(value.Length(), value.Length());
@@ -972,7 +976,6 @@ nsAutoCompleteController::OpenPopup()
   mInput->GetMinResultsForPopup(&minResults);
 
   if (mRowCount >= minResults) {
-    mIsOpen = true;
     return mInput->SetPopupOpen(true);
   }
 
@@ -995,7 +998,6 @@ nsAutoCompleteController::ClosePopup()
   mInput->GetPopup(getter_AddRefs(popup));
   NS_ENSURE_TRUE(popup != nsnull, NS_ERROR_FAILURE);
   popup->SetSelectedIndex(-1);
-  mIsOpen = false;
   return mInput->SetPopupOpen(false);
 }
 
@@ -1154,7 +1156,7 @@ nsAutoCompleteController::EnterMatch(bool aIsPopupSelection)
       nsAutoString defaultIndexValue;
       nsAutoString inputValue;
       input->GetTextValue(inputValue);
-      if (NS_SUCCEEDED(GetDefaultCompleteValue(selectedIndex, false, defaultIndexValue)) &&
+      if (NS_SUCCEEDED(GetDefaultCompleteValue(-1, false, defaultIndexValue)) &&
           defaultIndexValue.Equals(inputValue, nsCaseInsensitiveStringComparator()))
         value = defaultIndexValue;
     }
@@ -1261,58 +1263,70 @@ nsAutoCompleteController::ProcessResult(PRInt32 aSearchIndex, nsIAutoCompleteRes
   if (aResult)
     aResult->GetMatchCount(&matchCount);
 
-  PRInt32 oldIndex = mResults.IndexOf(aResult);
-  if (oldIndex == -1) {
+  PRInt32 resultIndex = mResults.IndexOf(aResult);
+  if (resultIndex == -1) {
     // cache the result
     mResults.AppendObject(aResult);
     mMatchCounts.AppendElement(matchCount);
+    resultIndex = mResults.Count() - 1;
   }
   else {
-    // replace the cached result
-    mResults.ReplaceObjectAt(aResult, oldIndex);
     oldMatchCount = mMatchCounts[aSearchIndex];
-    mMatchCounts[oldIndex] = matchCount;
+    mMatchCounts[resultIndex] = matchCount;
   }
 
-  PRUint32 oldRowCount = mRowCount;
-  // If the search failed, increase the match count
-  // to include the error description
-  if (result == nsIAutoCompleteResult::RESULT_FAILURE) {
-    nsAutoString error;
-    aResult->GetErrorDescription(error);
-    if (!error.IsEmpty()) {
-      ++mRowCount;
-      if (mTree)
-        mTree->RowCountChanged(oldRowCount, 1);
+  bool isTypeAheadResult = false;
+  if (aResult) {
+    aResult->GetTypeAheadResult(&isTypeAheadResult);
+  }
+
+  if (!isTypeAheadResult) {
+    PRUint32 oldRowCount = mRowCount;
+    // If the search failed, increase the match count to include the error
+    // description.
+    if (result == nsIAutoCompleteResult::RESULT_FAILURE) {
+      nsAutoString error;
+      aResult->GetErrorDescription(error);
+      if (!error.IsEmpty()) {
+        ++mRowCount;
+        if (mTree) {
+          mTree->RowCountChanged(oldRowCount, 1);
+        }
+      }
+    } else if (result == nsIAutoCompleteResult::RESULT_SUCCESS ||
+               result == nsIAutoCompleteResult::RESULT_SUCCESS_ONGOING) {
+      // Increase the match count for all matches in this result.
+      mRowCount += matchCount - oldMatchCount;
+
+      if (mTree) {
+        mTree->RowCountChanged(oldRowCount, matchCount - oldMatchCount);
+      }
     }
-  } else if (result == nsIAutoCompleteResult::RESULT_SUCCESS ||
-             result == nsIAutoCompleteResult::RESULT_SUCCESS_ONGOING) {
-    // Increase the match count for all matches in this result
-    mRowCount += matchCount - oldMatchCount;
 
-    if (mTree)
-      mTree->RowCountChanged(oldRowCount, matchCount - oldMatchCount);
+    // Refresh the popup view to display the new search results
+    nsCOMPtr<nsIAutoCompletePopup> popup;
+    input->GetPopup(getter_AddRefs(popup));
+    NS_ENSURE_TRUE(popup != nsnull, NS_ERROR_FAILURE);
+    popup->Invalidate();
 
-    // Try to autocomplete the default index for this search
-    CompleteDefaultIndex(aSearchIndex);
+    // Make sure the popup is open, if necessary, since we now have at least one
+    // search result ready to display. Don't force the popup closed if we might
+    // get results in the future to avoid unnecessarily canceling searches.
+    if (mRowCount) {
+      OpenPopup();
+    } else if (result != nsIAutoCompleteResult::RESULT_NOMATCH_ONGOING) {
+      ClosePopup();
+    }
   }
 
-  // Refresh the popup view to display the new search results
-  nsCOMPtr<nsIAutoCompletePopup> popup;
-  input->GetPopup(getter_AddRefs(popup));
-  NS_ENSURE_TRUE(popup != nsnull, NS_ERROR_FAILURE);
-  popup->Invalidate();
-
-  // Make sure the popup is open, if necessary, since we now have at least one
-  // search result ready to display. Don't force the popup closed if we might
-  // get results in the future to avoid unnecessarily canceling searches.
-  if (mRowCount)
-    OpenPopup();
-  else if (result != nsIAutoCompleteResult::RESULT_NOMATCH_ONGOING)
-    ClosePopup();
+  if (result == nsIAutoCompleteResult::RESULT_SUCCESS ||
+      result == nsIAutoCompleteResult::RESULT_SUCCESS_ONGOING) {
+    // Try to autocomplete the default index for this search.
+    CompleteDefaultIndex(resultIndex);
+  }
 
   if (mSearchesOngoing == 0) {
-    // If this is the last search to return, cleanup
+    // If this is the last search to return, cleanup.
     PostSearchCleanup();
   }
 
@@ -1369,9 +1383,9 @@ nsAutoCompleteController::ClearResults()
 }
 
 nsresult
-nsAutoCompleteController::CompleteDefaultIndex(PRInt32 aSearchIndex)
+nsAutoCompleteController::CompleteDefaultIndex(PRInt32 aResultIndex)
 {
-  if (mDefaultIndexCompleted || mBackspaced || mRowCount == 0 || mSearchString.Length() == 0)
+  if (mDefaultIndexCompleted || mBackspaced || mSearchString.Length() == 0 || !mInput)
     return NS_OK;
 
   PRInt32 selectionStart;
@@ -1391,7 +1405,7 @@ nsAutoCompleteController::CompleteDefaultIndex(PRInt32 aSearchIndex)
     return NS_OK;
 
   nsAutoString resultValue;
-  if (NS_SUCCEEDED(GetDefaultCompleteValue(aSearchIndex, true, resultValue)))
+  if (NS_SUCCEEDED(GetDefaultCompleteValue(aResultIndex, true, resultValue)))
     CompleteValue(resultValue);
 
   mDefaultIndexCompleted = true;
@@ -1400,12 +1414,12 @@ nsAutoCompleteController::CompleteDefaultIndex(PRInt32 aSearchIndex)
 }
 
 nsresult
-nsAutoCompleteController::GetDefaultCompleteValue(PRInt32 aSearchIndex,
+nsAutoCompleteController::GetDefaultCompleteValue(PRInt32 aResultIndex,
                                                   bool aPreserveCasing,
                                                   nsAString &_retval)
 {
   PRInt32 defaultIndex = -1;
-  PRInt32 index = aSearchIndex;
+  PRInt32 index = aResultIndex;
   if (index < 0) {
     PRUint32 count = mResults.Count();
     for (PRUint32 i = 0; i < count; ++i) {
@@ -1427,7 +1441,20 @@ nsAutoCompleteController::GetDefaultCompleteValue(PRInt32 aSearchIndex,
     // for us to be able to complete.
     result->GetDefaultIndex(&defaultIndex);
   }
-  NS_ENSURE_TRUE(defaultIndex >= 0, NS_ERROR_FAILURE);
+  if (defaultIndex < 0) {
+    // We were given a result index, but that result doesn't want to
+    // be autocompleted.
+    return NS_ERROR_FAILURE;
+  }
+
+  // If the result wrongly notifies a RESULT_SUCCESS with no matches, or
+  // provides a defaultIndex greater than its matchCount, avoid trying to
+  // complete to an empty value.
+  PRUint32 matchCount = 0;
+  result->GetMatchCount(&matchCount);
+  // Here defaultIndex is surely non-negative, so can be cast to unsigned.
+  if ((PRUint32)defaultIndex >= matchCount)
+    return NS_ERROR_FAILURE;
 
   nsAutoString resultValue;
   result->GetValueAt(defaultIndex, resultValue);
@@ -1524,7 +1551,7 @@ nsAutoCompleteController::GetResultValueAt(PRInt32 aIndex, bool aValueOnly, nsAS
 
 nsresult
 nsAutoCompleteController::GetResultValueLabelAt(PRInt32 aIndex, bool aValueOnly,
-                                               bool aGetValue, nsAString & _retval)
+                                                bool aGetValue, nsAString & _retval)
 {
   NS_ENSURE_TRUE(aIndex >= 0 && (PRUint32) aIndex < mRowCount, NS_ERROR_ILLEGAL_VALUE);
 
@@ -1572,15 +1599,22 @@ nsAutoCompleteController::RowIndexToSearch(PRInt32 aRowIndex, PRInt32 *aSearchIn
     if (!result)
       continue;
 
-    PRUint16 searchResult;
-    result->GetSearchResult(&searchResult);
-
-    // Find out how many results were provided by the
-    // current nsIAutoCompleteSearch
     PRUint32 rowCount = 0;
-    if (searchResult == nsIAutoCompleteResult::RESULT_SUCCESS ||
-        searchResult == nsIAutoCompleteResult::RESULT_SUCCESS_ONGOING) {
-      result->GetMatchCount(&rowCount);
+
+    // Skip past the result completely if it is marked as hidden
+    bool isTypeAheadResult = false;
+    result->GetTypeAheadResult(&isTypeAheadResult);
+
+    if (!isTypeAheadResult) {
+      PRUint16 searchResult;
+      result->GetSearchResult(&searchResult);
+
+      // Find out how many results were provided by the
+      // current nsIAutoCompleteSearch.
+      if (searchResult == nsIAutoCompleteResult::RESULT_SUCCESS ||
+          searchResult == nsIAutoCompleteResult::RESULT_SUCCESS_ONGOING) {
+        result->GetMatchCount(&rowCount);
+      }
     }
 
     // If the given row index is within the results range
