@@ -223,7 +223,7 @@ LoginManagerStorage_mozStorage.prototype = {
      * necessary. Most of the work is done in _deferredInit.
      */
     init : function () {
-        this._dbStmts = [];
+        this._dbStmts = {};
 
         // Connect to the correct preferences branch.
         this._prefBranch = Services.prefs.getBranch("signon.");
@@ -601,7 +601,7 @@ LoginManagerStorage_mozStorage.prototype = {
     _searchLogins : function (matchData) {
         let conditions = [], params = {};
 
-        for (field in matchData) {
+        for (let field in matchData) {
             let value = matchData[field];
             switch (field) {
                 // Historical compatibility requires this special case
@@ -780,7 +780,7 @@ LoginManagerStorage_mozStorage.prototype = {
             httpRealm: httpRealm
         };
         let matchData = { };
-        for each (field in ["hostname", "formSubmitURL", "httpRealm"])
+        for each (let field in ["hostname", "formSubmitURL", "httpRealm"])
           if (loginData[field] != '')
               matchData[field] = loginData[field];
         let [logins, ids] = this._searchLogins(matchData);
@@ -866,7 +866,7 @@ LoginManagerStorage_mozStorage.prototype = {
      */
     _getIdForLogin : function (login) {
         let matchData = { };
-        for each (field in ["hostname", "formSubmitURL", "httpRealm"])
+        for each (let field in ["hostname", "formSubmitURL", "httpRealm"])
             if (login[field] != '')
                 matchData[field] = login[field];
         let [logins, ids] = this._searchLogins(matchData);
@@ -1299,9 +1299,19 @@ LoginManagerStorage_mozStorage.prototype = {
             this._dbCleanup(true);
             throw e;
         }
+
+        Services.obs.addObserver(this, "profile-before-change", false);
         return isFirstRun;
     },
 
+    observe: function (subject, topic, data) {
+        switch (topic) {
+            case "profile-before-change":
+                Services.obs.removeObserver(this, "profile-before-change");
+                this._dbClose();
+            break;
+        }
+    },
 
     _dbCreate: function () {
         this.log("Creating Database");
@@ -1478,7 +1488,7 @@ LoginManagerStorage_mozStorage.prototype = {
 
         // Determine encryption type for each login and update the DB.
         query = "UPDATE moz_logins SET encType = :encType WHERE id = :id";
-        for each (params in logins) {
+        for each (let params in logins) {
             try {
                 stmt = this._dbCreateStatement(query, params);
                 stmt.execute();
@@ -1617,6 +1627,23 @@ LoginManagerStorage_mozStorage.prototype = {
         }
     },
 
+    _dbClose : function () {
+        this.log("Closing the DB connection.");
+        // Finalize all statements to free memory, avoid errors later
+        for each (let stmt in this._dbStmts) {
+            stmt.finalize();
+        }
+        this._dbStmts = {};
+
+        if (this._dbConnection !== null) {
+            try {
+                this._dbConnection.close();
+            } catch (e) {
+                Components.utils.reportError(e);
+            }
+        }
+        this._dbConnection = null;
+    },
 
     /*
      * _dbCleanup
@@ -1633,14 +1660,7 @@ LoginManagerStorage_mozStorage.prototype = {
             this._storageService.backupDatabaseFile(this._signonsFile, backupFile);
         }
 
-        // Finalize all statements to free memory, avoid errors later
-        for each (let stmt in this._dbStmts)
-            stmt.finalize();
-        this._dbStmts = [];
-
-        // Close the connection, ignore 'already closed' error
-        try { this._dbConnection.close() } catch(e) {}
-        this._dbConnection = null;
+        this._dbClose();
         this._signonsFile.remove(false);
     }
 
