@@ -39,6 +39,8 @@
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "PlacesUtils",
+                                  "resource://gre/modules/PlacesUtils.jsm");
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Constants
@@ -100,7 +102,14 @@ const kQueryTypeFiltered = 1;
 const kTitleTagsSeparator = " \u2013 ";
 
 const kBrowserUrlbarBranch = "browser.urlbar.";
+const kBrowserUrlbarAutofillPref = "browser.urlbar.autoFill";
 
+////////////////////////////////////////////////////////////////////////////////
+//// Globals
+
+XPCOMUtils.defineLazyServiceGetter(this, "gTextURIService",
+                                   "@mozilla.org/intl/texttosuburi;1",
+                                   "nsITextToSubURI");
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Helpers
@@ -139,6 +148,67 @@ function initTempTable(aDatabase)
   stmt.finalize();
 }
 
+/**
+ * Used to unescape encoded URI strings, and drop information that we do not
+ * care about for searching.
+ *
+ * @param aURIString
+ *        The text to unescape and modify.
+ * @return the modified uri.
+ */
+function fixupSearchText(aURIString)
+{
+  let uri = aURIString;
+
+  if (uri.indexOf("http://") == 0) {
+    uri = uri.slice(7);
+  }
+  else if (uri.indexOf("https://") == 0) {
+    uri = uri.slice(8);
+  }
+  else if (uri.indexOf("ftp://") == 0) {
+    uri = uri.slice(6);
+  }
+
+  if (uri.indexOf("www.") == 0) {
+    uri = uri.slice(4);
+  }
+
+  return gTextURIService.unEscapeURIForUI("UTF-8", uri);
+}
+
+/**
+ * safePrefGetter get the pref with typo safety.
+ * This will return the default value provided if no pref is set.
+ *
+ * @param aPrefBranch
+ *        The nsIPrefBranch2 containing the required preference
+ * @param aName
+ *        A preference name
+ * @param aDefault
+ *        The preference's default value
+ * @return the preference value or provided default
+ */
+
+function safePrefGetter(aPrefBranch, aName, aDefault) {
+  let types = {
+    boolean: "Bool",
+    number: "Int",
+    string: "Char"
+  };
+  let type = types[typeof(aDefault)];
+  if (!type) {
+    throw "Unknown type!";
+  }
+  // If the pref isn't set, we want to use the default.
+  try {
+    return aPrefBranch["get" + type + "Pref"](aName);
+  }
+  catch (e) {
+    return aDefault;
+  }
+}
+
 
 ////////////////////////////////////////////////////////////////////////////////
 //// AutoCompleteStatementCallbackWrapper class
@@ -178,8 +248,9 @@ AutoCompleteStatementCallbackWrapper.prototype = {
     // Only dispatch handleCompletion if we are not done searching and are a
     // pending search.
     let callback = this._callback;
-    if (!callback.isSearchComplete() && callback.isPendingSearch(this._handle))
+    if (!callback.isSearchComplete() && callback.isPendingSearch(this._handle)) {
       callback.handleCompletion.apply(callback, arguments);
+    }
   },
 
   //////////////////////////////////////////////////////////////////////////////
@@ -264,6 +335,7 @@ function nsPlacesAutoComplete()
       }
       stmt.bindParameters(params);
       stmt.executeAsync();
+      stmt.finalize();
       delete this._openPagesCache;
     }
 
@@ -274,10 +346,6 @@ function nsPlacesAutoComplete()
                                      "@mozilla.org/browser/global-history;2",
                                      "nsIBrowserHistory");
 
-  XPCOMUtils.defineLazyServiceGetter(this, "_textURIService",
-                                     "@mozilla.org/intl/texttosuburi;1",
-                                     "nsITextToSubURI");
-
   XPCOMUtils.defineLazyServiceGetter(this, "_bs",
                                      "@mozilla.org/browser/nav-bookmarks-service;1",
                                      "nsINavBookmarksService");
@@ -439,10 +507,8 @@ nsPlacesAutoComplete.prototype = {
   startSearch: function PAC_startSearch(aSearchString, aSearchParam,
                                         aPreviousResult, aListener)
   {
-    // If a previous query is running and the controller has not taken care
-    // of stopping it, kill it.
-    if ("_pendingQuery" in this)
-      this.stopSearch();
+    // Stop the search in case the controller has not taken care of it.
+    this.stopSearch();
 
     // Note: We don't use aPreviousResult to make sure ordering of results are
     //       consistent.  See bug 412730 for more details.
@@ -452,9 +518,9 @@ nsPlacesAutoComplete.prototype = {
     this._originalSearchString = aSearchString.trim();
 
     this._currentSearchString =
-      this._fixupSearchText(this._originalSearchString.toLowerCase());
+      fixupSearchText(this._originalSearchString.toLowerCase());
 
-    var searchParamParts = aSearchParam.split(" ");
+    let searchParamParts = aSearchParam.split(" ");
     this._enableActions = searchParamParts.indexOf("enable-actions") != -1;
 
     this._listener = aListener;
@@ -471,11 +537,12 @@ nsPlacesAutoComplete.prototype = {
     }
 
     // Reset our search behavior to the default.
-    if (this._currentSearchString)
+    if (this._currentSearchString) {
       this._behavior = this._defaultBehavior;
-    else
+    }
+    else {
       this._behavior = this._emptySearchDefaultBehavior;
-
+    }
     // For any given search, we run up to four queries:
     // 1) keywords (this._keywordQuery)
     // 2) adaptive learning (this._adaptiveQuery)
@@ -504,8 +571,9 @@ nsPlacesAutoComplete.prototype = {
     // We need to cancel our searches so we do not get any [more] results.
     // However, it's possible we haven't actually started any searches, so this
     // method may throw because this._pendingQuery may be undefined.
-    if (this._pendingQuery)
+    if (this._pendingQuery) {
       this._stopActiveQuery();
+    }
 
     this._finishSearch(false);
   },
@@ -515,8 +583,9 @@ nsPlacesAutoComplete.prototype = {
 
   onValueRemoved: function PAC_onValueRemoved(aResult, aURISpec, aRemoveFromDB)
   {
-    if (aRemoveFromDB)
+    if (aRemoveFromDB) {
       this._bh.removePage(this._ioService.newURI(aURISpec, null, null));
+    }
   },
 
   //////////////////////////////////////////////////////////////////////////////
@@ -574,8 +643,9 @@ nsPlacesAutoComplete.prototype = {
     }
 
     // Notify about results if we've gotten them.
-    if (haveMatches)
+    if (haveMatches) {
       this._notifyResults(true);
+    }
   },
 
   handleError: function PAC_handleError(aError)
@@ -587,8 +657,9 @@ nsPlacesAutoComplete.prototype = {
   handleCompletion: function PAC_handleCompletion(aReason)
   {
     // If we have already finished our search, we should bail out early.
-    if (this.isSearchComplete())
+    if (this.isSearchComplete()) {
       return;
+    }
 
     // If we do not have enough results, and our match type is
     // MATCH_BOUNDARY_ANYWHERE, search again with MATCH_ANYWHERE to get more
@@ -655,31 +726,6 @@ nsPlacesAutoComplete.prototype = {
   get _databaseInitialized()
     Object.getOwnPropertyDescriptor(this, "_db").value !== undefined,
 
-  /**
-   * Used to unescape encoded URI strings, and drop information that we do not
-   * care about for searching.
-   *
-   * @param aURIString
-   *        The text to unescape and modify.
-   * @return the modified uri.
-   */
-  _fixupSearchText: function PAC_fixupSearchText(aURIString)
-  {
-    let uri = aURIString;
-
-    if (uri.indexOf("http://") == 0)
-      uri = uri.slice(7);
-    else if (uri.indexOf("https://") == 0)
-      uri = uri.slice(8);
-    else if (uri.indexOf("ftp://") == 0)
-      uri = uri.slice(6);
-
-    if (uri.indexOf("www.") == 0)
-      uri = uri.slice(4);
-
-    return this._textURIService.unEscapeURIForUI("UTF-8", uri);
-  },
-
   /**
    * Generates the tokens used in searching from a given string.
    *
@@ -705,12 +751,14 @@ nsPlacesAutoComplete.prototype = {
   _finishSearch: function PAC_finishSearch(aNotify)
   {
     // Notify about results if we are supposed to.
-    if (aNotify)
+    if (aNotify) {
       this._notifyResults(false);
+    }
 
     // Clear our state
     delete this._originalSearchString;
     delete this._currentSearchString;
+    delete this._strippedPrefix;
     delete this._searchTokens;
     delete this._listener;
     delete this._result;
@@ -756,10 +804,10 @@ nsPlacesAutoComplete.prototype = {
   {
     let result = this._result;
     let resultCode = result.matchCount ? "RESULT_SUCCESS" : "RESULT_NOMATCH";
-    if (aSearchOngoing)
+    if (aSearchOngoing) {
       resultCode += "_ONGOING";
+    }
     result.setSearchResult(Ci.nsIAutoCompleteResult[resultCode]);
-    result.setDefaultIndex(result.matchCount ? 0 : -1);
     this._listener.onSearchResult(this, result);
     if (this._telemetryStartTime) {
       let elapsed = Date.now() - this._telemetryStartTime;
@@ -786,50 +834,37 @@ nsPlacesAutoComplete.prototype = {
   _loadPrefs: function PAC_loadPrefs(aRegisterObserver)
   {
     let self = this;
-    function safeGetter(aName, aDefault) {
-      let types = {
-        boolean: "Bool",
-        number: "Int",
-        string: "Char"
-      };
-      let type = types[typeof(aDefault)];
-      if (!type)
-        throw "Unknown type!";
 
-      // If the pref isn't set, we want to use the default.
-      try {
-        return self._prefs["get" + type + "Pref"](aName);
-      }
-      catch (e) {
-        return aDefault;
-      }
-    }
-
-    this._enabled = safeGetter("autocomplete.enabled", true);
-    this._matchBehavior = safeGetter("matchBehavior", MATCH_BOUNDARY_ANYWHERE);
-    this._filterJavaScript = safeGetter("filter.javascript", true);
-    this._maxRichResults = safeGetter("maxRichResults", 25);
-    this._restrictHistoryToken = safeGetter("restrict.history", "^");
-    this._restrictBookmarkToken = safeGetter("restrict.bookmark", "*");
-    this._restrictTypedToken = safeGetter("restrict.typed", "~");
-    this._restrictTagToken = safeGetter("restrict.tag", "+");
-    this._restrictOpenPageToken = safeGetter("restrict.openpage", "%");
-    this._matchTitleToken = safeGetter("match.title", "#");
-    this._matchURLToken = safeGetter("match.url", "@");
-    this._defaultBehavior = safeGetter("default.behavior", 0);
+    this._enabled = safePrefGetter(this._prefs, "autocomplete.enabled", true);
+    this._matchBehavior = safePrefGetter(this._prefs,
+                                         "matchBehavior",
+                                         MATCH_BOUNDARY_ANYWHERE);
+    this._filterJavaScript = safePrefGetter(this._prefs, "filter.javascript", true);
+    this._maxRichResults = safePrefGetter(this._prefs, "maxRichResults", 25);
+    this._restrictHistoryToken = safePrefGetter(this._prefs,
+                                                "restrict.history", "^");
+    this._restrictBookmarkToken = safePrefGetter(this._prefs,
+                                                 "restrict.bookmark", "*");
+    this._restrictTypedToken = safePrefGetter(this._prefs, "restrict.typed", "~");
+    this._restrictTagToken = safePrefGetter(this._prefs, "restrict.tag", "+");
+    this._restrictOpenPageToken = safePrefGetter(this._prefs,
+                                                 "restrict.openpage", "%");
+    this._matchTitleToken = safePrefGetter(this._prefs, "match.title", "#");
+    this._matchURLToken = safePrefGetter(this._prefs, "match.url", "@");
+    this._defaultBehavior = safePrefGetter(this._prefs, "default.behavior", 0);
     // Further restrictions to apply for "empty searches" (i.e. searches for "").
     this._emptySearchDefaultBehavior =
       this._defaultBehavior |
-      safeGetter("default.behavior.emptyRestriction",
-                 Ci.mozIPlacesAutoComplete.BEHAVIOR_HISTORY |
-                 Ci.mozIPlacesAutoComplete.BEHAVIOR_TYPED);
+      safePrefGetter(this._prefs, "default.behavior.emptyRestriction",
+                     Ci.mozIPlacesAutoComplete.BEHAVIOR_HISTORY |
+                     Ci.mozIPlacesAutoComplete.BEHAVIOR_TYPED);
 
     // Validate matchBehavior; default to MATCH_BOUNDARY_ANYWHERE.
     if (this._matchBehavior != MATCH_ANYWHERE &&
         this._matchBehavior != MATCH_BOUNDARY &&
-        this._matchBehavior != MATCH_BEGINNING)
+        this._matchBehavior != MATCH_BEGINNING) {
       this._matchBehavior = MATCH_BOUNDARY_ANYWHERE;
-
+    }
     // register observer
     if (aRegisterObserver) {
       let pb = this._prefs.QueryInterface(Ci.nsIPrefBranch2);
@@ -864,8 +899,9 @@ nsPlacesAutoComplete.prototype = {
           this._setBehavior("tag");
           break;
         case this._restrictOpenPageToken:
-          if (!this._enableActions)
+          if (!this._enableActions) {
             continue;
+          }
           this._setBehavior("openpage");
           break;
         case this._matchTitleToken:
@@ -888,8 +924,9 @@ nsPlacesAutoComplete.prototype = {
     // Set the right JavaScript behavior based on our preference.  Note that the
     // preference is whether or not we should filter JavaScript, and the
     // behavior is if we should search it or not.
-    if (!this._filterJavaScript)
+    if (!this._filterJavaScript) {
       this._setBehavior("javascript");
+    }
 
     return {
       query: this._getBoundSearchQuery(this._matchBehavior, aTokens),
@@ -977,9 +1014,9 @@ nsPlacesAutoComplete.prototype = {
     let searchString = this._originalSearchString;
     let queryString = "";
     let queryIndex = searchString.indexOf(" ");
-    if (queryIndex != -1)
+    if (queryIndex != -1) {
       queryString = searchString.substring(queryIndex + 1);
-
+    }
     // We need to escape the parameters as if they were the query in a URL
     queryString = encodeURIComponent(queryString).replace("%20", "+", "g");
 
@@ -1004,8 +1041,9 @@ nsPlacesAutoComplete.prototype = {
   _getBoundAdaptiveQuery: function PAC_getBoundAdaptiveQuery(aMatchBehavior)
   {
     // If we were not given a match behavior, use the stored match behavior.
-    if (arguments.length == 0)
+    if (arguments.length == 0) {
       aMatchBehavior = this._matchBehavior;
+    }
 
     let query = this._adaptiveQuery;
     let (params = query.params) {
@@ -1041,7 +1079,7 @@ nsPlacesAutoComplete.prototype = {
                         ["moz-action:switchtab," + escapedEntryURL, "action "] :
                         [escapedEntryURL, ""];
 
-    if (this._inResults(entryId || url)) {
+    if (this._inResults(entryId, url)) {
       return false;
     }
 
@@ -1061,10 +1099,12 @@ nsPlacesAutoComplete.prototype = {
       // know it is a keyword.  Otherwise, we found an exact page match, so just
       // show the page like a regular result.  Because the page title is likely
       // going to be more specific than the bookmark title (keyword title).
-      if (!entryTitle)
+      if (!entryTitle) {
         style = "keyword";
-      else
+      }
+      else {
         title = entryTitle;
+      }
     }
 
     // We will always prefer to show tags if we have them.
@@ -1079,9 +1119,9 @@ nsPlacesAutoComplete.prototype = {
     }
 
     // If we have tags and should show them, we need to add them to the title.
-    if (showTags)
+    if (showTags) {
       title += kTitleTagsSeparator + entryTags;
-
+    }
     // We have to determine the right style to display.  Tags show the tag icon,
     // bookmarks get the bookmark icon, and keywords get the keyword icon.  If
     // the result does not fall into any of those, it just gets the favicon.
@@ -1089,12 +1129,15 @@ nsPlacesAutoComplete.prototype = {
       // It is possible that we already have a style set (from a keyword
       // search or because of the user's preferences), so only set it if we
       // haven't already done so.
-      if (showTags)
+      if (showTags) {
         style = "tag";
-      else if (entryBookmarked)
+      }
+      else if (entryBookmarked) {
         style = "bookmark";
-      else
+      }
+      else {
         style = "favicon";
+      }
     }
 
     this._addToResults(entryId, url, title, entryFavicon, action + style);
@@ -1104,13 +1147,25 @@ nsPlacesAutoComplete.prototype = {
   /**
    * Checks to see if the given place has already been added to the results.
    *
-   * @param aPlaceIdOrUrl
-   *        The place id or url (if the entry does not have a id) to check for.
+   * @param aPlaceId
+   *        The place id to check for, may be null.
+   * @param aUrl
+   *        The url to check for.
    * @return true if the place has been added, false otherwise.
+   *
+   * @note Must check both the id and the url for a negative match, since
+   *       autocomplete may run in the middle of a new page addition.  In such
+   *       a case the switch-to-tab query would hash the page by url, then a
+   *       next query, running after the page addition, would hash it by id.
+   *       It's not possible to just rely on url though, since keywords
+   *       dynamically modify the url to include their search string.
    */
-  _inResults: function PAC_inResults(aPlaceIdOrUrl)
+  _inResults: function PAC_inResults(aPlaceId, aUrl)
   {
-    return aPlaceIdOrUrl in this._usedPlaces;
+    if (aPlaceId && aPlaceId in this._usedPlaces) {
+      return true;
+    }
+    return aUrl in this._usedPlaces;
   },
 
   /**
@@ -1207,14 +1262,333 @@ nsPlacesAutoComplete.prototype = {
 
   classID: Components.ID("d0272978-beab-4adc-a3d4-04b76acfa4e7"),
 
+  _xpcom_factory: XPCOMUtils.generateSingletonFactory(nsPlacesAutoComplete),
+
   QueryInterface: XPCOMUtils.generateQI([
     Ci.nsIAutoCompleteSearch,
     Ci.nsIAutoCompleteSimpleResultListener,
     Ci.mozIPlacesAutoComplete,
     Ci.mozIStorageStatementCallback,
     Ci.nsIObserver,
+    Ci.nsISupportsWeakReference,
   ])
 };
 
-let components = [nsPlacesAutoComplete];
+////////////////////////////////////////////////////////////////////////////////
+//// urlInlineComplete class
+
+function urlInlineComplete()
+{
+  this._loadPrefs(true);
+  // register observers
+  Services.obs.addObserver(this, kTopicShutdown, false);
+}
+
+urlInlineComplete.prototype = {
+
+/////////////////////////////////////////////////////////////////////////////////
+//// Database and query getters
+
+  __db: null,
+
+  get _db()
+  {
+    if (!this.__db && this._autofill) {
+      this.__db = Cc["@mozilla.org/browser/nav-history-service;1"].
+        getService(Ci.nsPIPlacesDatabase).
+        DBConnection.
+        clone(true);
+    }
+    return this.__db;
+  },
+
+  __syncQuery: null,
+
+  get _syncQuery()
+  {
+    if (!this.__syncQuery) {
+      // Add a trailing slash at the end of the hostname, since we always
+      // want to complete up to and including a URL separator.
+      this.__syncQuery = this._db.createStatement(
+        "SELECT host || '/' "
+        + "FROM moz_hosts "
+        + "WHERE host BETWEEN :search_string AND :search_string || X'FFFF' "
+        + "ORDER BY frecency DESC "
+        + "LIMIT 1"
+      );
+    }
+    return this.__syncQuery;
+  },
+
+  __asyncQuery: null,
+
+  get _asyncQuery()
+  {
+    if (!this.__asyncQuery) {
+      this.__asyncQuery = this._db.createAsyncStatement(
+        "SELECT h.url "
+        + "FROM moz_places h "
+        + "WHERE h.frecency <> 0 "
+        +   "AND AUTOCOMPLETE_MATCH(:searchString, h.url, "
+        +                          "h.title, '', "
+        +                          "h.visit_count, h.typed, 0, 0, "
+        +                          ":matchBehavior, :searchBehavior) "
+        + "ORDER BY h.frecency DESC, h.id DESC "
+        + "LIMIT 1"
+      );
+    }
+    return this.__asyncQuery;
+  },
+
+  //////////////////////////////////////////////////////////////////////////////
+  //// nsIAutoCompleteSearch
+
+  startSearch: function UIC_startSearch(aSearchString, aSearchParam,
+                                        aPreviousResult, aListener)
+  {
+    // Stop the search in case the controller has not taken care of it.
+    if (this._pendingQuery) {
+      this.stopSearch();
+    }
+
+    // We want to store the original string with no leading or trailing
+    // whitespace for case sensitive searches.
+    this._originalSearchString = aSearchString;
+    this._currentSearchString =
+      fixupSearchText(this._originalSearchString.toLowerCase());
+    // The protocol and the domain are lowercased by nsIURI, so it's fine to
+    // lowercase the typed prefix to add it back to the results later.
+    this._strippedPrefix = this._originalSearchString.slice(
+      0, this._originalSearchString.length - this._currentSearchString.length
+    ).toLowerCase();
+
+    let result = Cc["@mozilla.org/autocomplete/simple-result;1"].
+                 createInstance(Ci.nsIAutoCompleteSimpleResult);
+    result.setSearchString(aSearchString);
+    result.setTypeAheadResult(true);
+
+    this._result = result;
+    this._listener = aListener;
+
+    // Don't autoFill if the search term is recognized as a keyword, otherwise
+    // it will override default keywords behavior.  Note that keywords are
+    // hashed on first use, so while the first query may delay a little bit,
+    // next ones will just hit the memory hash.
+    if (this._currentSearchString.length == 0 || !this._db ||
+        PlacesUtils.bookmarks.getURIForKeyword(this._currentSearchString)) {
+      this._finishSearch();
+      return;
+    }
+
+    // Do a synchronous search on the table of domains.
+    let query = this._syncQuery;
+    query.params.search_string = this._currentSearchString.toLowerCase();
+
+    // Domains have no "/" in them.
+    let lastSlashIndex = this._currentSearchString.lastIndexOf("/");
+    if (lastSlashIndex == -1) {
+      var hasDomainResult = false;
+      var domain;
+      try {
+        hasDomainResult = query.executeStep();
+        if (hasDomainResult) {
+          domain = query.getString(0);
+        }
+      } finally {
+        query.reset();
+      }
+
+      if (hasDomainResult) {
+        // We got a match for a domain, we can add it immediately.
+        result.appendMatch(this._strippedPrefix + domain, "");
+
+        this._finishSearch();
+        return;
+      }
+    }
+
+    // We did not get a result from the synchronous domain search.
+    // We now do an asynchronous search through places, and complete
+    // up to the next URL separator.
+
+    // First, check if this is necessary.
+    // We don't need to search if we have no "/" separator, or if it's at
+    // the end of the search text.
+    if (lastSlashIndex == -1 ||
+        lastSlashIndex == this._currentSearchString.length - 1) {
+      this._finishSearch();
+      return;
+    }
+
+    // Within the standard autocomplete query, we only search the beginning
+    // of URLs for 1 result.
+    let query = this._asyncQuery;
+    let (params = query.params) {
+      params.matchBehavior = MATCH_BEGINNING;
+      params.searchBehavior = Ci.mozIPlacesAutoComplete["BEHAVIOR_URL"];
+      params.searchString = this._currentSearchString;
+    }
+
+    // Execute the async query
+    let wrapper = new AutoCompleteStatementCallbackWrapper(this, this._db);
+    this._pendingQuery = wrapper.executeAsync([query]);
+  },
+
+  stopSearch: function UIC_stopSearch()
+  {
+    delete this._originalSearchString;
+    delete this._currentSearchString;
+    delete this._result;
+    delete this._listener;
+
+    if (this._pendingQuery) {
+      this._pendingQuery.cancel();
+      delete this._pendingQuery;
+    }
+  },
+
+  /**
+   * Loads the preferences that we care about.
+   *
+   * @param [optional] aRegisterObserver
+   *        Indicates if the preference observer should be added or not.  The
+   *        default value is false.
+   */
+  _loadPrefs: function UIC_loadPrefs(aRegisterObserver)
+  {
+    this._autofill = safePrefGetter(Services.prefs,
+                                    kBrowserUrlbarAutofillPref,
+                                    true);
+    if (aRegisterObserver) {
+      Services.prefs.addObserver(kBrowserUrlbarAutofillPref, this, true);
+    }
+  },
+
+  //////////////////////////////////////////////////////////////////////////////
+  //// mozIStorageStatementCallback
+
+  handleResult: function UIC_handleResult(aResultSet)
+  {
+    let row = aResultSet.getNextRow();
+    let url = fixupSearchText(row.getResultByIndex(0));
+
+    // We must complete the URL up to the next separator (which is /, ? or #).
+    let separatorIndex = url.slice(this._currentSearchString.length)
+                            .search(/[\/\?\#]/);
+    if (separatorIndex != -1) {
+      separatorIndex += this._currentSearchString.length;
+      if (url[separatorIndex] == "/") {
+        separatorIndex++; // Include the "/" separator
+      }
+      url = url.slice(0, separatorIndex);
+    }
+
+    // Add the result
+    this._result.appendMatch(this._strippedPrefix + url, "");
+
+    // handleCompletion() will cause the result listener to be called, and
+    // will display the result in the UI.
+  },
+
+  handleError: function UIC_handleError(aError)
+  {
+    Components.utils.reportError("URL Inline Complete: An async statement encountered an " +
+                                 "error: " + aError.result + ", '" + aError.message + "'");
+  },
+
+  handleCompletion: function UIC_handleCompletion(aReason)
+  {
+    this._finishSearch();
+  },
+
+  //////////////////////////////////////////////////////////////////////////////
+  //// nsIObserver
+
+  observe: function PAC_observe(aSubject, aTopic, aData)
+  {
+    if (aTopic == kTopicShutdown) {
+      Services.obs.removeObserver(this, kTopicShutdown);
+      this._closeDatabase();
+    }
+    else if (aTopic == kPrefChanged) {
+      this._loadPrefs();
+      if (!this._autofill) {
+        this.stopSearch();
+        this._closeDatabase();
+      }
+    }
+  },
+
+  /**
+   *
+   * Finalize and close the database safely
+   *
+   **/
+  _closeDatabase: function UIC_closeDatabase()
+  {
+    // Finalize the statements that we have used.
+    let stmts = [
+      "__syncQuery",
+      "__asyncQuery",
+    ];
+    for (let i = 0; i < stmts.length; i++) {
+      // We do not want to create any query we haven't already created, so
+      // see if it is a getter first.
+      if (this[stmts[i]]) {
+        this[stmts[i]].finalize();
+        this[stmts[i]] = null;
+      }
+    }
+    if (this.__db) {
+      this._db.asyncClose();
+      this.__db = null;
+    }
+  },
+
+  //////////////////////////////////////////////////////////////////////////////
+  //// urlInlineComplete
+
+  _finishSearch: function UIC_finishSearch()
+  {
+    // Notify the result object
+    let result = this._result;
+
+    if (result.matchCount) {
+      result.setDefaultIndex(0);
+      result.setSearchResult(Ci.nsIAutoCompleteResult["RESULT_SUCCESS"]);
+    } else {
+      result.setDefaultIndex(-1);
+      result.setSearchResult(Ci.nsIAutoCompleteResult["RESULT_NOMATCH"]);
+    }
+
+    this._listener.onSearchResult(this, result);
+    this.stopSearch();
+  },
+
+  isSearchComplete: function UIC_isSearchComplete()
+  {
+    return this._pendingQuery == null;
+  },
+
+  isPendingSearch: function UIC_isPendingSearch(aHandle)
+  {
+    return this._pendingQuery == aHandle;
+  },
+
+  //////////////////////////////////////////////////////////////////////////////
+  //// nsISupports
+
+  classID: Components.ID("c88fae2d-25cf-4338-a1f4-64a320ea7440"),
+
+  _xpcom_factory: XPCOMUtils.generateSingletonFactory(urlInlineComplete),
+
+  QueryInterface: XPCOMUtils.generateQI([
+    Ci.nsIAutoCompleteSearch,
+    Ci.mozIStorageStatementCallback,
+    Ci.nsIObserver,
+    Ci.nsISupportsWeakReference,
+  ])
+};
+
+let components = [nsPlacesAutoComplete, urlInlineComplete];
 const NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
