@@ -74,4 +74,6 @@ function checkServices() {
   checkService("cookies", Ci.nsICookieManager2);
   checkService("logins", Ci.nsILoginManager);
   checkService("telemetry", Ci.nsITelemetry);
+  checkService("sysinfo", Ci.nsIPropertyBag2);
+  checkService("clipboard", Ci.nsIClipboard);
 }
