@@ -40,6 +40,7 @@ const Cc = Components.classes;
 const Ci = Components.interfaces;
 
 var reportsDir, pendingDir;
+var reportURL;
 
 Components.utils.import("resource://gre/modules/CrashSubmit.jsm");
 
@@ -76,8 +77,11 @@ function submitError(dumpid) {
 function submitPendingReport(event) {
   var link = event.target;
   var id = link.firstChild.textContent;
-  if (CrashSubmit.submit(id, document.body, submitSuccess, submitError, true))
+  if (CrashSubmit.submit(id, { submitSuccess: submitSuccess,
+                               submitError: submitError,
+                               noThrottle: true })) {
     link.className = "submitting";
+  }
   event.preventDefault();
   return false;
 }
