@@ -11,13 +11,22 @@ var gProvider;
 const SETTINGS_ROWS = 8;
 
 var MockFilePicker = SpecialPowers.MockFilePicker;
-MockFilePicker.reset();
+MockFilePicker.init();
 
 var observer = {
   lastData: null,
   observe: function(aSubject, aTopic, aData) {
-    if (aTopic == "addon-options-displayed")
+    if (aTopic == "addon-options-displayed") {
       this.lastData = aData;
+      // Test if the binding has applied before the observers are notified. We test the second setting here,
+      // because the code operates on the first setting and we want to check it applies to all.
+      var setting = aSubject.querySelector("rows > setting[first-row] ~ setting");
+      var input = gManagerWindow.document.getAnonymousElementByAttribute(setting, "class", "setting-label");
+      isnot(input, null, "XBL binding should be applied");
+
+      // Add some extra height to the scrolling pane to ensure that it needs to scroll when appropriate.
+      gManagerWindow.document.getElementById("detail-controls").style.marginBottom = "1000px";
+    }
   }
 };
 
@@ -33,6 +42,17 @@ function installAddon(aCallback) {
   }, "application/x-xpinstall");
 }
 
+function checkScrolling(aShouldHaveScrolled) {
+  var detailView = gManagerWindow.document.getElementById("detail-view");
+  var boxObject = detailView.boxObject;
+  boxObject.QueryInterface(Ci.nsIScrollBoxObject);
+  ok(detailView.scrollHeight > boxObject.height, "Page should require scrolling");
+  if (aShouldHaveScrolled)
+    isnot(detailView.scrollTop, 0, "Page should have scrolled");
+  else
+    is(detailView.scrollTop, 0, "Page should not have scrolled");
+}
+
 function test() {
   waitForExplicitFinish();
 
@@ -85,7 +105,7 @@ function end_test() {
   Services.prefs.clearUserPref("extensions.inlinesettings3.radioString");
   Services.prefs.clearUserPref("extensions.inlinesettings3.menulist");
 
-  MockFilePicker.reset();
+  MockFilePicker.cleanup();
 
   close_manager(gManagerWindow, function() {
     AddonManager.getAddonByID("inlinesettings1@tests.mozilla.org", function(aAddon) {
@@ -141,14 +161,15 @@ add_test(function() {
 
   wait_for_view_load(gManagerWindow, function() {
     is(observer.lastData, "inlinesettings1@tests.mozilla.org", "Observer notification should have fired");
+    is(gManagerWindow.gViewController.currentViewId,
+       "addons://detail/inlinesettings1%40tests.mozilla.org/preferences",
+       "Current view should scroll to preferences");
+    checkScrolling(true);
 
     var grid = gManagerWindow.document.getElementById("detail-grid");
     var settings = grid.querySelectorAll("rows > setting");
     is(settings.length, SETTINGS_ROWS, "Grid should have settings children");
 
-    // Force bindings to apply
-    settings[0].clientTop;
-
     ok(settings[0].hasAttribute("first-row"), "First visible row should have first-row attribute");
     Services.prefs.setBoolPref("extensions.inlinesettings1.bool", false);
     var input = gManagerWindow.document.getAnonymousElementByAttribute(settings[0], "anonid", "input");
@@ -221,6 +242,7 @@ add_test(function() {
       var button = gManagerWindow.document.getAnonymousElementByAttribute(settings[6], "anonid", "button");
       input = gManagerWindow.document.getAnonymousElementByAttribute(settings[6], "anonid", "input");
       is(input.value, "", "Label value should be empty");
+      is(input.tooltipText, "", "Label tooltip should be empty");
 
       var profD = Services.dirsvc.get("ProfD", Ci.nsIFile);
       var curProcD = Services.dirsvc.get("CurProcD", Ci.nsIFile);
@@ -230,6 +252,7 @@ add_test(function() {
       EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
       is(MockFilePicker.mode, Ci.nsIFilePicker.modeOpen, "File picker mode should be open file");
       is(input.value, profD.path, "Label value should match file chosen");
+      is(input.tooltipText, profD.path, "Label tooltip should match file chosen");
       is(Services.prefs.getCharPref("extensions.inlinesettings1.file"), profD.path, "File pref should match file chosen");
 
       MockFilePicker.returnFiles = [curProcD];
@@ -237,18 +260,21 @@ add_test(function() {
       EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
       is(MockFilePicker.mode, Ci.nsIFilePicker.modeOpen, "File picker mode should be open file");
       is(input.value, profD.path, "Label value should not have changed");
+      is(input.tooltipText, profD.path, "Label tooltip should not have changed");
       is(Services.prefs.getCharPref("extensions.inlinesettings1.file"), profD.path, "File pref should not have changed");
 
       ok(!settings[7].hasAttribute("first-row"), "Not the first row");
       button = gManagerWindow.document.getAnonymousElementByAttribute(settings[7], "anonid", "button");
       input = gManagerWindow.document.getAnonymousElementByAttribute(settings[7], "anonid", "input");
       is(input.value, "", "Label value should be empty");
+      is(input.tooltipText, "", "Label tooltip should be empty");
 
       MockFilePicker.returnFiles = [profD];
       MockFilePicker.returnValue = Ci.nsIFilePicker.returnOK;
       EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
       is(MockFilePicker.mode, Ci.nsIFilePicker.modeGetFolder, "File picker mode should be directory");
       is(input.value, profD.path, "Label value should match file chosen");
+      is(input.tooltipText, profD.path, "Label tooltip should match file chosen");
       is(Services.prefs.getCharPref("extensions.inlinesettings1.directory"), profD.path, "Directory pref should match file chosen");
 
       MockFilePicker.returnFiles = [curProcD];
@@ -256,6 +282,7 @@ add_test(function() {
       EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
       is(MockFilePicker.mode, Ci.nsIFilePicker.modeGetFolder, "File picker mode should be directory");
       is(input.value, profD.path, "Label value should not have changed");
+      is(input.tooltipText, profD.path, "Label tooltip should not have changed");
       is(Services.prefs.getCharPref("extensions.inlinesettings1.directory"), profD.path, "Directory pref should not have changed");
 
     } finally {
@@ -282,9 +309,6 @@ add_test(function() {
     var settings = grid.querySelectorAll("rows > setting");
     is(settings.length, 4, "Grid should have settings children");
 
-    // Force bindings to apply
-    settings[0].clientTop;
-
     ok(settings[0].hasAttribute("first-row"), "First visible row should have first-row attribute");
     Services.prefs.setBoolPref("extensions.inlinesettings3.radioBool", false);
     var radios = settings[0].getElementsByTagName("radio");
@@ -348,9 +372,6 @@ add_test(function() {
     var settings = grid.querySelectorAll("rows > setting");
     is(settings.length, 5, "Grid should have settings children");
 
-    // Force bindings to apply
-    settings[0].clientTop;
-
     var node = settings[0];
     node = settings[0];
     is_element_hidden(node, "Unsupported settings should not be visible");
@@ -433,10 +454,15 @@ add_test(function() {
   var addon = get_addon_element(gManagerWindow, "inlinesettings1@tests.mozilla.org");
   addon.parentNode.ensureElementIsVisible(addon);
 
-  var button = gManagerWindow.document.getAnonymousElementByAttribute(addon, "anonid", "preferences-btn");
+  var button = gManagerWindow.document.getAnonymousElementByAttribute(addon, "anonid", "details-btn");
   EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
 
   wait_for_view_load(gManagerWindow, function() {
+    is(gManagerWindow.gViewController.currentViewId,
+       "addons://detail/inlinesettings1%40tests.mozilla.org",
+       "Current view should not scroll to preferences");
+    checkScrolling(false);
+
     var grid = gManagerWindow.document.getElementById("detail-grid");
     var settings = grid.querySelectorAll("rows > setting");
     is(settings.length, SETTINGS_ROWS, "Grid should have settings children");
@@ -463,11 +489,15 @@ add_test(function() {
         var grid = gManagerWindow.document.getElementById("detail-grid");
         var settings = grid.querySelectorAll("rows > setting");
         is(settings.length, 0, "Grid should not have settings children");
+        
+        observer.lastData = null;
 
         // enable
         var button = gManagerWindow.document.getElementById("detail-enable-btn");
         EventUtils.synthesizeMouseAtCenter(button, { clickCount: 1 }, gManagerWindow);
 
+        is(observer.lastData, "inlinesettings1@tests.mozilla.org", "Observer notification should have fired");
+
         settings = grid.querySelectorAll("rows > setting");
         is(settings.length, SETTINGS_ROWS, "Grid should have settings children");
 
