@@ -4,7 +4,9 @@
 
 // Tests the list view
 
-Components.utils.import("resource://gre/modules/LightweightThemeManager.jsm");
+let tempScope = {};
+Components.utils.import("resource://gre/modules/LightweightThemeManager.jsm", tempScope);
+let LightweightThemeManager = tempScope.LightweightThemeManager;
 
 
 var gProvider;
@@ -62,22 +64,26 @@ function test() {
                  AddonManager.PERM_CAN_UPGRADE
   }, {
     id: "addon4@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon4@tests.mozilla.org",
     name: "Test add-on 4",
     _userDisabled: true,
     isActive: false,
     blocklistState: Ci.nsIBlocklistService.STATE_SOFTBLOCKED
   }, {
     id: "addon5@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon5@tests.mozilla.org",
     name: "Test add-on 5",
     isActive: false,
     blocklistState: Ci.nsIBlocklistService.STATE_BLOCKED,
     appDisabled: true
   }, {
     id: "addon6@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon6@tests.mozilla.org",
     name: "Test add-on 6",
     operationsRequiringRestart: AddonManager.OP_NEEDS_RESTART_NONE
   }, {
     id: "addon7@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon7@tests.mozilla.org",
     name: "Test add-on 7",
     blocklistState: Ci.nsIBlocklistService.STATE_OUTDATED,
   }]);
@@ -230,7 +236,7 @@ add_test(function() {
     is(get_node(addon, "warning").textContent, "Test add-on 4 is known to cause security or stability issues.", "Warning message should be correct");
     is_element_visible(get_node(addon, "warning-link"), "Warning link should be visible");
     is(get_node(addon, "warning-link").value, "More Information", "Warning link text should be correct");
-    is(get_node(addon, "warning-link").href, gBlocklistURL, "Warning link should be correct");
+    is(get_node(addon, "warning-link").href, "http://example.com/addon4@tests.mozilla.org", "Warning link should be correct");
     is_element_hidden(get_node(addon, "error"), "Error message should be hidden");
     is_element_hidden(get_node(addon, "error-link"), "Error link should be hidden");
     is_element_hidden(get_node(addon, "pending"), "Pending message should be hidden");
@@ -265,7 +271,7 @@ add_test(function() {
     is(get_node(addon, "error").textContent, "Test add-on 5 has been disabled due to security or stability issues.", "Error message should be correct");
     is_element_visible(get_node(addon, "error-link"), "Error link should be visible");
     is(get_node(addon, "error-link").value, "More Information", "Error link text should be correct");
-    is(get_node(addon, "error-link").href, gBlocklistURL, "Error link should be correct");
+    is(get_node(addon, "error-link").href, "http://example.com/addon5@tests.mozilla.org", "Error link should be correct");
     is_element_hidden(get_node(addon, "pending"), "Pending message should be hidden");
 
     info("Addon 6");
@@ -461,7 +467,7 @@ add_test(function() {
       is(get_node(addon, "warning").textContent, "Test add-on 4 is known to cause security or stability issues.", "Warning message should be correct");
       is_element_visible(get_node(addon, "warning-link"), "Warning link should be visible");
       is(get_node(addon, "warning-link").value, "More Information", "Warning link text should be correct");
-      is(get_node(addon, "warning-link").href, gBlocklistURL, "Warning link should be correct");
+      is(get_node(addon, "warning-link").href, "http://example.com/addon4@tests.mozilla.org", "Warning link should be correct");
       is_element_hidden(get_node(addon, "error"), "Error message should be hidden");
       is_element_hidden(get_node(addon, "error-link"), "Error link should be hidden");
       is_element_hidden(get_node(addon, "pending"), "Pending message should be hidden");
