@@ -111,7 +111,11 @@ x_error_handler(Display *, XErrorEvent *ev)
 static void glxtest()
 {
   ///// Open libGL and load needed symbols /////
+#ifdef __OpenBSD__
+  void *libgl = dlopen("libGL.so", RTLD_LAZY);
+#else
   void *libgl = dlopen("libGL.so.1", RTLD_LAZY);
+#endif
   if (!libgl)
     fatal_error("Unable to load libGL.so.1");
   
@@ -198,7 +202,7 @@ static void glxtest()
     fatal_error("No visual found for first FBConfig");
 
   ///// Get a Pixmap and a GLXPixmap /////
-  Pixmap pixmap = XCreatePixmap(dpy, RootWindow(dpy, vInfo->screen), 4, 4, 32);
+  Pixmap pixmap = XCreatePixmap(dpy, RootWindow(dpy, vInfo->screen), 4, 4, vInfo->depth);
   GLXPixmap glxpixmap = glXCreatePixmap(dpy, fbConfigs[0], pixmap, NULL);
 
   ///// Get a GL context and make it current //////
@@ -227,15 +231,6 @@ static void glxtest()
   if (length >= bufsize)
     fatal_error("GL strings length too large for buffer size");
 
-  ///// Check that no X error happened /////
-  // In case of X errors, our X error handler will exit() now.
-  // We really want to make sure that the system is able to create a GL context without generating X errors,
-  // as these would crash the application.
-  XSync(dpy, False);
-  
-  ///// Finally write data to the pipe /////
-  write(write_end_of_the_pipe, buf, length);
-
   ///// Clean up. Indeed, the parent process might fail to kill us (e.g. if it doesn't need to check GL info)
   ///// so we might be staying alive for longer than expected, so it's important to consume as little memory as
   ///// possible. Also we want to check that we're able to do that too without generating X errors.
@@ -245,6 +240,9 @@ static void glxtest()
   XFreePixmap(dpy, pixmap);
   XCloseDisplay(dpy);
   dlclose(libgl);
+
+  ///// Finally write data to the pipe
+  write(write_end_of_the_pipe, buf, length);
 }
 
 /** \returns true in the child glxtest process, false in the parent process */
