@@ -46,11 +46,10 @@ class nsIWidget;
 struct nsRect;
 class nsRegion;
 class nsDeviceContext;
-class nsIViewObserver;
 
 #define NS_IVIEWMANAGER_IID \
-{ 0x144ef328, 0xbece, 0x43d6, \
-  { 0xac, 0xac, 0x1a, 0x90, 0x4b, 0x5c, 0xc1, 0x11 } }
+{ 0x1262a33f, 0xc19f, 0x4e5b, \
+  { 0x85, 0x00, 0xab, 0xf3, 0x7d, 0xcf, 0x30, 0x1d } }
 
 class nsIViewManager : public nsISupports
 {
@@ -59,7 +58,7 @@ public:
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_IVIEWMANAGER_IID)
   /**
    * Initialize the ViewManager
-   * Note: this instance does not hold a reference to the viewobserver
+   * Note: this instance does not hold a reference to the presshell
    * because it holds a reference to this instance.
    * @result The result of the initialization, NS_OK if no errors
    */
@@ -258,15 +257,15 @@ public:
   NS_IMETHOD  SetViewFloating(nsIView *aView, bool aFloatingView) = 0;
 
   /**
-   * Set the view observer associated with this manager
-   * @param aObserver - new observer
+   * Set the presshell associated with this manager
+   * @param aPresShell - new presshell
    */
-  virtual void SetViewObserver(nsIViewObserver *aObserver) = 0;
+  virtual void SetPresShell(nsIPresShell *aPresShell) = 0;
 
   /**
-   * Get the view observer associated with this manager
+   * Get the pres shell associated with this manager
    */
-  virtual nsIViewObserver* GetViewObserver() = 0;
+  virtual nsIPresShell* GetPresShell() = 0;
 
   /**
    * Get the device context associated with this manager
