@@ -59,7 +59,6 @@
 #include "nsCRT.h"
 #include "nsQuickSort.h"
 #include "nsEnumeratorUtils.h"
-#include "nsIProxyObjectManager.h"
 #include "nsThreadUtils.h"
 #include "mozilla/Services.h"
 
@@ -129,7 +128,7 @@ protected:
 NS_IMPL_ISUPPORTS2(BaseStringEnumerator, nsISimpleEnumerator, nsIUTF8StringEnumerator)
 
 NS_IMETHODIMP
-BaseStringEnumerator::HasMoreElements(PRBool *_retval)
+BaseStringEnumerator::HasMoreElements(bool *_retval)
 {
   *_retval = (mSimpleCurItem < mCount);
 
@@ -153,7 +152,7 @@ BaseStringEnumerator::GetNext(nsISupports **_retval)
 }
 
 NS_IMETHODIMP
-BaseStringEnumerator::HasMore(PRBool *_retval)
+BaseStringEnumerator::HasMore(bool *_retval)
 {
   *_retval = (mStringCurItem < mCount);
 
@@ -352,7 +351,7 @@ CategoryNode::Enumerate(nsISimpleEnumerator **_retval)
 struct persistent_userstruct {
   PRFileDesc* fd;
   const char* categoryName;
-  PRBool      success;
+  bool        success;
 };
 
 PLDHashOperator
@@ -369,7 +368,7 @@ enumfunc_pentries(CategoryLeaf* aLeaf, void* userArg)
                    args->categoryName,
                    aLeaf->GetKey(),
                    aLeaf->value) == (PRUint32) -1) {
-      args->success = PR_FALSE;
+      args->success = false;
       status = PL_DHASH_STOP;
     }
   }
@@ -470,7 +469,7 @@ nsCategoryManager::Create(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 
 nsCategoryManager::nsCategoryManager()
   : mLock("nsCategoryManager")
-  , mSuppressNotifications(PR_FALSE)
+  , mSuppressNotifications(false)
 {
   PL_INIT_ARENA_POOL(&mArena, "CategoryManagerArena",
                      NS_CATEGORYMANAGER_ARENA_SIZE);
@@ -497,6 +496,41 @@ nsCategoryManager::get_category(const char* aName) {
   return node;
 }
 
+namespace {
+
+class CategoryNotificationRunnable : public nsRunnable
+{
+public:
+  CategoryNotificationRunnable(nsISupports* aSubject,
+                               const char* aTopic,
+                               const char* aData)
+    : mSubject(aSubject)
+    , mTopic(aTopic)
+    , mData(aData)
+  { }
+
+  NS_DECL_NSIRUNNABLE
+
+private:
+  nsCOMPtr<nsISupports> mSubject;
+  const char* mTopic;
+  NS_ConvertUTF8toUTF16 mData;
+};
+
+NS_IMETHODIMP
+CategoryNotificationRunnable::Run()
+{
+  nsCOMPtr<nsIObserverService> observerService =
+    mozilla::services::GetObserverService();
+  if (observerService)
+    observerService->NotifyObservers(mSubject, mTopic, mData.get());
+
+  return NS_OK;
+}
+  
+} // anonymous namespace
+
+
 void
 nsCategoryManager::NotifyObservers( const char *aTopic,
                                     const char *aCategoryName,
@@ -505,19 +539,7 @@ nsCategoryManager::NotifyObservers( const char *aTopic,
   if (mSuppressNotifications)
     return;
 
-  nsCOMPtr<nsIObserverService> observerService =
-    mozilla::services::GetObserverService();
-  if (!observerService)
-    return;
-
-  nsCOMPtr<nsIObserverService> obsProxy;
-  NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
-                       NS_GET_IID(nsIObserverService),
-                       observerService,
-                       NS_PROXY_ASYNC,
-                       getter_AddRefs(obsProxy));
-  if (!obsProxy)
-    return;
+  nsRefPtr<CategoryNotificationRunnable> r;
 
   if (aEntryName) {
     nsCOMPtr<nsISupportsCString> entry
@@ -529,12 +551,12 @@ nsCategoryManager::NotifyObservers( const char *aTopic,
     if (NS_FAILED(rv))
       return;
 
-    obsProxy->NotifyObservers(entry, aTopic,
-                              NS_ConvertUTF8toUTF16(aCategoryName).get());
+    r = new CategoryNotificationRunnable(entry, aTopic, aCategoryName);
   } else {
-    obsProxy->NotifyObservers(this, aTopic,
-                              NS_ConvertUTF8toUTF16(aCategoryName).get());
+    r = new CategoryNotificationRunnable(this, aTopic, aCategoryName);
   }
+
+  NS_DispatchToMainThread(r);
 }
 
 NS_IMETHODIMP
@@ -565,8 +587,8 @@ NS_IMETHODIMP
 nsCategoryManager::AddCategoryEntry( const char *aCategoryName,
                                      const char *aEntryName,
                                      const char *aValue,
-                                     PRBool aPersist,
-                                     PRBool aReplace,
+                                     bool aPersist,
+                                     bool aReplace,
                                      char **_retval )
 {
   if (aPersist) {
@@ -634,7 +656,7 @@ nsCategoryManager::AddCategoryEntry(const char *aCategoryName,
 NS_IMETHODIMP
 nsCategoryManager::DeleteCategoryEntry( const char *aCategoryName,
                                         const char *aEntryName,
-                                        PRBool aDontPersist)
+                                        bool aDontPersist)
 {
   NS_ENSURE_ARG_POINTER(aCategoryName);
   NS_ENSURE_ARG_POINTER(aEntryName);
@@ -723,11 +745,11 @@ nsCategoryManager::EnumerateCategories(nsISimpleEnumerator **_retval)
 
 struct writecat_struct {
   PRFileDesc* fd;
-  PRBool      success;
+  bool        success;
 };
 
 NS_METHOD
-nsCategoryManager::SuppressNotifications(PRBool aSuppress)
+nsCategoryManager::SuppressNotifications(bool aSuppress)
 {
   mSuppressNotifications = aSuppress;
   return NS_OK;
@@ -770,7 +792,7 @@ NS_CreateServicesFromCategory(const char *category,
     return;
   }
 
-  PRBool hasMore;
+  bool hasMore;
   while (NS_SUCCEEDED(senumerator->HasMore(&hasMore)) && hasMore) {
     // From here on just skip any error we get.
     nsCAutoString entryString;
