@@ -100,7 +100,7 @@ nsBinaryOutputStream::WriteSegments(nsReadSegmentFun reader, void * closure, PRU
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::IsNonBlocking(PRBool *aNonBlocking)
+nsBinaryOutputStream::IsNonBlocking(bool *aNonBlocking)
 {
     NS_ENSURE_STATE(mOutputStream);
     return mOutputStream->IsNonBlocking(aNonBlocking);
@@ -131,7 +131,7 @@ nsBinaryOutputStream::SetOutputStream(nsIOutputStream *aOutputStream)
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteBoolean(PRBool aBoolean)
+nsBinaryOutputStream::WriteBoolean(bool aBoolean)
 {
     return Write8(aBoolean);
 }
@@ -261,7 +261,7 @@ nsBinaryOutputStream::WriteByteArray(PRUint8 *aBytes, PRUint32 aLength)
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteObject(nsISupports* aObject, PRBool aIsStrongRef)
+nsBinaryOutputStream::WriteObject(nsISupports* aObject, bool aIsStrongRef)
 {
     return WriteCompoundObject(aObject, NS_GET_IID(nsISupports),
                                aIsStrongRef);
@@ -271,13 +271,13 @@ NS_IMETHODIMP
 nsBinaryOutputStream::WriteSingleRefObject(nsISupports* aObject)
 {
     return WriteCompoundObject(aObject, NS_GET_IID(nsISupports),
-                               PR_TRUE);
+                               true);
 }
 
 NS_IMETHODIMP
 nsBinaryOutputStream::WriteCompoundObject(nsISupports* aObject,
                                           const nsIID& aIID,
-                                          PRBool aIsStrongRef)
+                                          bool aIsStrongRef)
 {
     // Can't deal with weak refs
     NS_ENSURE_TRUE(aIsStrongRef, NS_ERROR_UNEXPECTED);
@@ -448,7 +448,7 @@ nsBinaryInputStream::ReadSegments(nsWriteSegmentFun writer, void * closure, PRUi
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::IsNonBlocking(PRBool *aNonBlocking)
+nsBinaryInputStream::IsNonBlocking(bool *aNonBlocking)
 {
     NS_ENSURE_STATE(mInputStream);
     return mInputStream->IsNonBlocking(aNonBlocking);
@@ -471,7 +471,7 @@ nsBinaryInputStream::SetInputStream(nsIInputStream *aInputStream)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::ReadBoolean(PRBool* aBoolean)
+nsBinaryInputStream::ReadBoolean(bool* aBoolean)
 {
     PRUint8 byteResult;
     nsresult rv = Read8(&byteResult);
@@ -592,7 +592,7 @@ nsBinaryInputStream::ReadCString(nsACString& aString)
 // bytes, which means we only have half of the last PRUnichar
 struct WriteStringClosure {
     PRUnichar *mWriteCursor;
-    PRPackedBool mHasCarryoverByte;
+    bool mHasCarryoverByte;
     char mCarryoverByte;
 };
 
@@ -648,7 +648,7 @@ WriteSegmentToString(nsIInputStream* aStream,
         ++aFromSegment;
         --aCount;
 
-        closure->mHasCarryoverByte = PR_FALSE;
+        closure->mHasCarryoverByte = false;
     }
     
     // this array is possibly unaligned... be careful how we access it!
@@ -674,7 +674,7 @@ WriteSegmentToString(nsIInputStream* aStream,
         // we must have had a carryover byte, that we'll need the next
         // time around
         closure->mCarryoverByte = aFromSegment[aCount - 1];
-        closure->mHasCarryoverByte = PR_TRUE;
+        closure->mHasCarryoverByte = true;
     }
     
     return NS_OK;
@@ -704,7 +704,7 @@ nsBinaryInputStream::ReadString(nsAString& aString)
     
     WriteStringClosure closure;
     closure.mWriteCursor = start.get();
-    closure.mHasCarryoverByte = PR_FALSE;
+    closure.mHasCarryoverByte = false;
     
     rv = ReadSegments(WriteSegmentToString, &closure,
                       length*sizeof(PRUnichar), &bytesRead);
@@ -750,7 +750,7 @@ nsBinaryInputStream::ReadByteArray(PRUint32 aLength, PRUint8* *_rval)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::ReadObject(PRBool aIsStrongRef, nsISupports* *aObject)
+nsBinaryInputStream::ReadObject(bool aIsStrongRef, nsISupports* *aObject)
 {
     nsCID cid;
     nsIID iid;
@@ -768,7 +768,19 @@ nsBinaryInputStream::ReadObject(PRBool aIsStrongRef, nsISupports* *aObject)
         { 0x7a22cc0, 0xce5, 0x11d3,
           { 0x93, 0x31, 0x0, 0x10, 0x4b, 0xa0, 0xfd, 0x40 }};
 
-    if (iid.Equals(oldURIiid)) {
+    // hackaround for bug 670542
+    static const nsIID oldURIiid2 =
+        { 0xd6d04c36, 0x0fa4, 0x4db3,
+          { 0xbe, 0x05, 0x4a, 0x18, 0x39, 0x71, 0x03, 0xe2 }};
+
+    // hackaround for bug 682031
+    static const nsIID oldURIiid3 =
+        { 0x12120b20, 0x0929, 0x40e9,
+          { 0x88, 0xcf, 0x6e, 0x08, 0x76, 0x6e, 0x8b, 0x23 }};
+
+    if (iid.Equals(oldURIiid) ||
+        iid.Equals(oldURIiid2) ||
+        iid.Equals(oldURIiid3)) {
         const nsIID newURIiid = NS_IURI_IID;
         iid = newURIiid;
     }
