@@ -59,7 +59,12 @@ class nsTDependentSubstring_CharT : public nsTSubstring_CharT
 
       void Rebind( const substring_type&, PRUint32 startPos, PRUint32 length = size_type(-1) );
 
-      void Rebind( const char_type* start, const char_type* end );
+      void Rebind( const char_type* data, size_type length );
+
+      void Rebind( const char_type* start, const char_type* end )
+        {
+          Rebind(start, size_type(end - start));
+        }
 
       nsTDependentSubstring_CharT( const substring_type& str, PRUint32 startPos, PRUint32 length = size_type(-1) )
         : substring_type()
@@ -67,6 +72,9 @@ class nsTDependentSubstring_CharT : public nsTSubstring_CharT
           Rebind(str, startPos, length);
         }
 
+      nsTDependentSubstring_CharT( const char_type* data, size_type length )
+        : substring_type(const_cast<char_type*>(data), length, F_NONE) {}
+
       nsTDependentSubstring_CharT( const char_type* start, const char_type* end )
         : substring_type(const_cast<char_type*>(start), PRUint32(end - start), F_NONE) {}
 
@@ -98,6 +106,13 @@ Substring( const nsReadingIterator<CharT>& start, const nsReadingIterator<CharT>
     return nsTDependentSubstring_CharT(start.get(), end.get());
   }
 
+inline
+const nsTDependentSubstring_CharT
+Substring( const CharT* data, PRUint32 length )
+  {
+    return nsTDependentSubstring_CharT(data, length);
+  }
+
 inline
 const nsTDependentSubstring_CharT
 Substring( const CharT* start, const CharT* end )
