@@ -44,7 +44,9 @@
 #include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "prlog.h"
-#include "nsThreadUtilsInternal.h"
+#include "nsIObserverService.h"
+#include "mozilla/HangMonitor.h"
+#include "mozilla/Services.h"
 
 #define HAVE_UALARM _BSD_SOURCE || (_XOPEN_SOURCE >= 500 ||                 \
                       _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED) &&           \
@@ -78,7 +80,22 @@ static PRLogModuleInfo *sLog = PR_NewLogModule("nsThread");
 
 NS_DECL_CI_INTERFACE_GETTER(nsThread)
 
-nsIThreadObserver* nsThread::sGlobalObserver;
+namespace mozilla {
+
+// Fun fact: Android's GCC won't convert bool* to PRInt32*, so we can't
+// PR_ATOMIC_SET a bool.
+static PRInt32 sMemoryPressurePending = 0;
+
+/*
+ * It's important that this function not acquire any locks, nor do anything
+ * which might cause malloc to run.
+ */
+void ScheduleMemoryPressureEvent()
+{
+  PR_ATOMIC_SET(&sMemoryPressurePending, 1);
+}
+
+} // namespace mozilla
 
 //-----------------------------------------------------------------------------
 // Because we do not have our own nsIFactory, we have to implement nsIClassInfo
@@ -306,20 +323,7 @@ nsThread::ThreadFunc(void *arg)
 
 //-----------------------------------------------------------------------------
 
-nsThread::nsThread()
-  : mLock("nsThread.mLock")
-  , mEvents(&mEventsRoot)
-  , mPriority(PRIORITY_NORMAL)
-  , mThread(nsnull)
-  , mRunningEvent(0)
-  , mStackSize(0)
-  , mShutdownContext(nsnull)
-  , mShutdownRequired(false)
-  , mEventsAreDoomed(false)
-{
-}
-
-nsThread::nsThread(PRUint32 aStackSize)
+nsThread::nsThread(MainThreadFlag aMainThread, PRUint32 aStackSize)
   : mLock("nsThread.mLock")
   , mEvents(&mEventsRoot)
   , mPriority(PRIORITY_NORMAL)
@@ -329,6 +333,7 @@ nsThread::nsThread(PRUint32 aStackSize)
   , mShutdownContext(nsnull)
   , mShutdownRequired(false)
   , mEventsAreDoomed(false)
+  , mIsMainThread(aMainThread)
 {
 }
 
@@ -498,6 +503,11 @@ nsThread::Shutdown()
   PR_JoinThread(mThread);
   mThread = nsnull;
 
+  // We hold strong references to our event observers, and once the thread is
+  // shut down the observers can't easily unregister themselves. Do it here
+  // to avoid leaking.
+  ClearObservers();
+
 #ifdef DEBUG
   {
     MutexAutoLock lock(mLock);
@@ -585,10 +595,24 @@ nsThread::ProcessNextEvent(bool mayWait, bool *result)
 
   NS_ENSURE_STATE(PR_GetCurrentThread() == mThread);
 
-  bool notifyGlobalObserver = (sGlobalObserver != nsnull);
-  if (notifyGlobalObserver) 
-    sGlobalObserver->OnProcessNextEvent(this, mayWait && !ShuttingDown(),
-                                        mRunningEvent);
+  if (MAIN_THREAD == mIsMainThread && mayWait && !ShuttingDown())
+    HangMonitor::Suspend();
+
+  // Fire a memory pressure notification, if we're the main thread and one is
+  // pending.
+  if (MAIN_THREAD == mIsMainThread && !ShuttingDown()) {
+    bool mpPending = PR_ATOMIC_SET(&sMemoryPressurePending, 0);
+    if (mpPending) {
+      nsCOMPtr<nsIObserverService> os = services::GetObserverService();
+      if (os) {
+        os->NotifyObservers(nsnull, "memory-pressure",
+                            NS_LITERAL_STRING("low-memory").get());
+      }
+      else {
+        NS_WARNING("Can't get observer service!");
+      }
+    }
+  }
 
   nsCOMPtr<nsIThreadObserver> obs = mObserver;
   if (obs)
@@ -615,7 +639,7 @@ nsThread::ProcessNextEvent(bool mayWait, bool *result)
 
 #ifdef NS_FUNCTION_TIMER
     char message[1024] = {'\0'};
-    if (NS_IsMainThread()) {
+    if (MAIN_THREAD == mIsMainThread) {
         mozilla::FunctionTimer::ft_snprintf(message, sizeof(message), 
                                             "@ Main Thread Event %p", (void*)event.get());
     }
@@ -628,6 +652,8 @@ nsThread::ProcessNextEvent(bool mayWait, bool *result)
 
     if (event) {
       LOG(("THRD(%p) running [%p]\n", this, event.get()));
+      if (MAIN_THREAD == mIsMainThread)
+        HangMonitor::NotifyActivity();
       event->Run();
     } else if (mayWait) {
       NS_ASSERTION(ShuttingDown(),
@@ -643,9 +669,6 @@ nsThread::ProcessNextEvent(bool mayWait, bool *result)
   if (obs)
     obs->AfterProcessNextEvent(this, mRunningEvent);
 
-  if (notifyGlobalObserver && sGlobalObserver)
-    sGlobalObserver->AfterProcessNextEvent(this, mRunningEvent);
-
   return rv;
 }
 
@@ -810,18 +833,3 @@ nsThreadSyncDispatch::Run()
   }
   return NS_OK;
 }
-
-nsresult
-NS_SetGlobalThreadObserver(nsIThreadObserver* aObserver)
-{
-  if (aObserver && nsThread::sGlobalObserver) {
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
-  if (!NS_IsMainThread()) {
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  nsThread::sGlobalObserver = aObserver;
-  return NS_OK;
-}
