@@ -44,6 +44,7 @@
 #include "nsThreadManager.h"
 #include "nsThreadUtils.h"
 #include "prmem.h"
+#include "sampler.h"
 
 using mozilla::TimeDuration;
 using mozilla::TimeStamp;
@@ -376,6 +377,8 @@ void nsTimerImpl::Fire()
   if (mCanceled)
     return;
 
+  SAMPLE_LABEL("Timer", "Fire");
+
   TimeStamp now = TimeStamp::Now();
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
